/**
* DevExtreme (cjs/renovation/ui/scheduler/workspaces/base/work_space.js)
* Version: 21.2.3
* Build date: Thu Nov 11 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
"use strict";

exports.viewFunction = exports.prepareGenerationOptions = exports.WorkSpace = void 0;

var _inferno = require("inferno");

var _inferno2 = require("@devextreme/runtime/inferno");

var _combine_classes = require("../../../../utils/combine_classes");

var _ordinary_layout = require("./ordinary_layout");

var _view_data_provider = _interopRequireDefault(require("../../../../../ui/scheduler/workspaces/view_model/view_data_provider"));

var _utils = require("./utils");

var _props = require("../props");

var _work_space_config = require("./work_space_config");

var _utils2 = require("../utils");

var _cross_scrolling_layout = require("./cross_scrolling_layout");

var _utils3 = require("../../../../../ui/scheduler/workspaces/view_model/utils");

var _base = require("../../view_model/to_test/views/utils/base");

var _date_header_data_generator = require("../../../../../ui/scheduler/workspaces/view_model/date_header_data_generator");

var _time_panel_data_generator = require("../../../../../ui/scheduler/workspaces/view_model/time_panel_data_generator");

var _utils4 = require("../../view_model/group_panel/utils");

var _excluded = ["accessKey", "activeStateEnabled", "allDayAppointments", "allDayPanelExpanded", "allowMultipleCellSelection", "appointments", "cellDuration", "className", "crossScrollingEnabled", "currentDate", "dataCellTemplate", "dateCellTemplate", "disabled", "endDayHour", "firstDayOfWeek", "focusStateEnabled", "groupByDate", "groupOrientation", "groups", "height", "hint", "hoursInterval", "hoverStateEnabled", "indicatorTime", "indicatorUpdateInterval", "intervalCount", "onClick", "onKeyDown", "onViewRendered", "resourceCellTemplate", "rtlEnabled", "schedulerHeight", "schedulerWidth", "scrolling", "selectedCellData", "shadeUntilCurrentTime", "showAllDayPanel", "showCurrentTimeIndicator", "startDate", "startDayHour", "tabIndex", "timeCellTemplate", "type", "visible", "width"];

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var prepareGenerationOptions = function prepareGenerationOptions(workSpaceProps, renderConfig, isAllDayPanelVisible) {
  var cellDuration = workSpaceProps.cellDuration,
      currentDate = workSpaceProps.currentDate,
      endDayHour = workSpaceProps.endDayHour,
      firstDayOfWeek = workSpaceProps.firstDayOfWeek,
      groupByDate = workSpaceProps.groupByDate,
      groupOrientation = workSpaceProps.groupOrientation,
      groups = workSpaceProps.groups,
      hoursInterval = workSpaceProps.hoursInterval,
      intervalCount = workSpaceProps.intervalCount,
      startDate = workSpaceProps.startDate,
      startDayHour = workSpaceProps.startDayHour,
      type = workSpaceProps.type;
  var getDateForHeaderText = renderConfig.getDateForHeaderText,
      headerCellTextFormat = renderConfig.headerCellTextFormat,
      isGenerateWeekDaysHeaderData = renderConfig.isGenerateWeekDaysHeaderData,
      isProvideVirtualCellsWidth = renderConfig.isProvideVirtualCellsWidth,
      isRenderTimePanel = renderConfig.isRenderTimePanel;
  return {
    startRowIndex: 0,
    startCellIndex: 0,
    groupOrientation: groupOrientation,
    groupByDate: groupByDate,
    groups: groups,
    isProvideVirtualCellsWidth: isProvideVirtualCellsWidth,
    isAllDayPanelVisible: isAllDayPanelVisible,
    selectedCells: undefined,
    focusedCell: undefined,
    headerCellTextFormat: headerCellTextFormat,
    getDateForHeaderText: getDateForHeaderText,
    startDayHour: startDayHour,
    endDayHour: endDayHour,
    cellDuration: cellDuration,
    viewType: type,
    intervalCount: intervalCount,
    hoursInterval: hoursInterval,
    currentDate: currentDate,
    startDate: startDate,
    firstDayOfWeek: firstDayOfWeek,
    isGenerateTimePanelData: isRenderTimePanel,
    isGenerateWeekDaysHeaderData: isGenerateWeekDaysHeaderData
  };
};

exports.prepareGenerationOptions = prepareGenerationOptions;

var viewFunction = function viewFunction(_ref) {
  var allDayPanelRef = _ref.allDayPanelRef,
      classes = _ref.classes,
      dateHeaderData = _ref.dateHeaderData,
      dateTableRef = _ref.dateTableRef,
      dateTableTemplate = _ref.dateTableTemplate,
      groupOrientation = _ref.groupOrientation,
      groupPanelData = _ref.groupPanelData,
      groupPanelHeight = _ref.groupPanelHeight,
      groupPanelRef = _ref.groupPanelRef,
      headerEmptyCellWidth = _ref.headerEmptyCellWidth,
      headerPanelTemplate = _ref.headerPanelTemplate,
      isAllDayPanelVisible = _ref.isAllDayPanelVisible,
      isGroupedByDate = _ref.isGroupedByDate,
      isRenderHeaderEmptyCell = _ref.isRenderHeaderEmptyCell,
      isStandaloneAllDayPanel = _ref.isStandaloneAllDayPanel,
      isVerticalGrouping = _ref.isVerticalGrouping,
      Layout = _ref.layout,
      layoutRef = _ref.layoutRef,
      _ref$props = _ref.props,
      allDayAppointments = _ref$props.allDayAppointments,
      allDayPanelExpanded = _ref$props.allDayPanelExpanded,
      appointments = _ref$props.appointments,
      dataCellTemplate = _ref$props.dataCellTemplate,
      dateCellTemplate = _ref$props.dateCellTemplate,
      groups = _ref$props.groups,
      intervalCount = _ref$props.intervalCount,
      resourceCellTemplate = _ref$props.resourceCellTemplate,
      timeCellTemplate = _ref$props.timeCellTemplate,
      _ref$renderConfig = _ref.renderConfig,
      groupPanelClassName = _ref$renderConfig.groupPanelClassName,
      isRenderDateHeader = _ref$renderConfig.isRenderDateHeader,
      scrollingDirection = _ref$renderConfig.scrollingDirection,
      tablesWidth = _ref.tablesWidth,
      timePanelData = _ref.timePanelData,
      timePanelRef = _ref.timePanelRef,
      timePanelTemplate = _ref.timePanelTemplate,
      viewData = _ref.viewData;
  return (0, _inferno.createComponentVNode)(2, Layout, {
    "viewData": viewData,
    "dateHeaderData": dateHeaderData,
    "timePanelData": timePanelData,
    "groupPanelData": groupPanelData,
    "dataCellTemplate": dataCellTemplate,
    "dateCellTemplate": dateCellTemplate,
    "timeCellTemplate": timeCellTemplate,
    "resourceCellTemplate": resourceCellTemplate,
    "groups": groups,
    "groupByDate": isGroupedByDate,
    "groupOrientation": groupOrientation,
    "groupPanelClassName": groupPanelClassName,
    "intervalCount": intervalCount,
    "headerPanelTemplate": headerPanelTemplate,
    "dateTableTemplate": dateTableTemplate,
    "timePanelTemplate": timePanelTemplate,
    "isAllDayPanelCollapsed": !allDayPanelExpanded,
    "isAllDayPanelVisible": isAllDayPanelVisible,
    "isRenderDateHeader": isRenderDateHeader,
    "isRenderHeaderEmptyCell": isRenderHeaderEmptyCell,
    "isRenderGroupPanel": isVerticalGrouping,
    "isStandaloneAllDayPanel": isStandaloneAllDayPanel,
    "scrollingDirection": scrollingDirection,
    "groupPanelHeight": groupPanelHeight,
    "headerEmptyCellWidth": headerEmptyCellWidth,
    "tablesWidth": tablesWidth,
    "className": classes,
    "dateTableRef": dateTableRef,
    "allDayPanelRef": allDayPanelRef,
    "timePanelRef": timePanelRef,
    "groupPanelRef": groupPanelRef,
    "appointments": appointments,
    "allDayAppointments": allDayAppointments
  }, null, layoutRef);
};

exports.viewFunction = viewFunction;

var getTemplate = function getTemplate(TemplateProp) {
  return TemplateProp && (TemplateProp.defaultProps ? function (props) {
    return (0, _inferno.normalizeProps)((0, _inferno.createComponentVNode)(2, TemplateProp, _extends({}, props)));
  } : TemplateProp);
};

var WorkSpace = /*#__PURE__*/function (_InfernoComponent) {
  _inheritsLoose(WorkSpace, _InfernoComponent);

  function WorkSpace(props) {
    var _this;

    _this = _InfernoComponent.call(this, props) || this;
    _this.dateTableRef = (0, _inferno.createRef)();
    _this.allDayPanelRef = (0, _inferno.createRef)();
    _this.timePanelRef = (0, _inferno.createRef)();
    _this.groupPanelRef = (0, _inferno.createRef)();
    _this.layoutRef = (0, _inferno.createRef)();
    _this.__getterCache = {};
    _this.state = {
      groupPanelHeight: undefined,
      headerEmptyCellWidth: undefined,
      tablesWidth: undefined
    };
    _this.groupPanelHeightEffect = _this.groupPanelHeightEffect.bind(_assertThisInitialized(_this));
    _this.headerEmptyCellWidthEffect = _this.headerEmptyCellWidthEffect.bind(_assertThisInitialized(_this));
    _this.tablesWidthEffect = _this.tablesWidthEffect.bind(_assertThisInitialized(_this));
    _this.onViewRendered = _this.onViewRendered.bind(_assertThisInitialized(_this));
    _this.createDateTableElementsMeta = _this.createDateTableElementsMeta.bind(_assertThisInitialized(_this));
    _this.createAllDayPanelElementsMeta = _this.createAllDayPanelElementsMeta.bind(_assertThisInitialized(_this));
    return _this;
  }

  var _proto = WorkSpace.prototype;

  _proto.createEffects = function createEffects() {
    return [new _inferno2.InfernoEffect(this.groupPanelHeightEffect, [this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]), new _inferno2.InfernoEffect(this.headerEmptyCellWidthEffect, [this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]), new _inferno2.InfernoEffect(this.tablesWidthEffect, [this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]), new _inferno2.InfernoEffect(this.onViewRendered, [this.props.allDayPanelExpanded, this.props.cellDuration, this.props.crossScrollingEnabled, this.props.currentDate, this.props.endDayHour, this.props.firstDayOfWeek, this.props.groupByDate, this.props.groupOrientation, this.props.type, this.props.intervalCount, this.props.groups, this.props.hoursInterval, this.props.onViewRendered, this.props.scrolling, this.props.showAllDayPanel, this.props.startDate, this.props.startDayHour, this.state.tablesWidth])];
  };

  _proto.updateEffects = function updateEffects() {
    var _this$_effects$, _this$_effects$2, _this$_effects$3, _this$_effects$4;

    (_this$_effects$ = this._effects[0]) === null || _this$_effects$ === void 0 ? void 0 : _this$_effects$.update([this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]);
    (_this$_effects$2 = this._effects[1]) === null || _this$_effects$2 === void 0 ? void 0 : _this$_effects$2.update([this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]);
    (_this$_effects$3 = this._effects[2]) === null || _this$_effects$3 === void 0 ? void 0 : _this$_effects$3.update([this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]);
    (_this$_effects$4 = this._effects[3]) === null || _this$_effects$4 === void 0 ? void 0 : _this$_effects$4.update([this.props.allDayPanelExpanded, this.props.cellDuration, this.props.crossScrollingEnabled, this.props.currentDate, this.props.endDayHour, this.props.firstDayOfWeek, this.props.groupByDate, this.props.groupOrientation, this.props.type, this.props.intervalCount, this.props.groups, this.props.hoursInterval, this.props.onViewRendered, this.props.scrolling, this.props.showAllDayPanel, this.props.startDate, this.props.startDayHour, this.state.tablesWidth]);
  };

  _proto.groupPanelHeightEffect = function groupPanelHeightEffect() {
    var _this2 = this;

    this.setState(function (__state_argument) {
      var _this2$dateTableRef$c;

      return {
        groupPanelHeight: (_this2$dateTableRef$c = _this2.dateTableRef.current) === null || _this2$dateTableRef$c === void 0 ? void 0 : _this2$dateTableRef$c.getBoundingClientRect().height
      };
    });
  };

  _proto.headerEmptyCellWidthEffect = function headerEmptyCellWidthEffect() {
    var _this$timePanelRef$cu, _this$timePanelRef$cu2, _this$groupPanelRef$c, _this$groupPanelRef$c2;

    var timePanelWidth = (_this$timePanelRef$cu = (_this$timePanelRef$cu2 = this.timePanelRef.current) === null || _this$timePanelRef$cu2 === void 0 ? void 0 : _this$timePanelRef$cu2.getBoundingClientRect().width) !== null && _this$timePanelRef$cu !== void 0 ? _this$timePanelRef$cu : 0;
    var groupPanelWidth = (_this$groupPanelRef$c = (_this$groupPanelRef$c2 = this.groupPanelRef.current) === null || _this$groupPanelRef$c2 === void 0 ? void 0 : _this$groupPanelRef$c2.getBoundingClientRect().width) !== null && _this$groupPanelRef$c !== void 0 ? _this$groupPanelRef$c : 0;
    this.setState(function (__state_argument) {
      return {
        headerEmptyCellWidth: timePanelWidth + groupPanelWidth
      };
    });
  };

  _proto.tablesWidthEffect = function tablesWidthEffect() {
    var _this3 = this;

    if (this.isCalculateTablesWidth) {
      var _this$props = this.props,
          currentDate = _this$props.currentDate,
          endDayHour = _this$props.endDayHour,
          groups = _this$props.groups,
          hoursInterval = _this$props.hoursInterval,
          intervalCount = _this$props.intervalCount,
          startDayHour = _this$props.startDayHour,
          viewType = _this$props.type;
      this.setState(function (__state_argument) {
        return {
          tablesWidth: (0, _utils.getDateTableWidth)(_this3.layoutRef.current.getScrollableWidth(), _this3.dateTableRef.current, _this3.viewDataProvider, {
            intervalCount: intervalCount,
            currentDate: currentDate,
            viewType: viewType,
            hoursInterval: hoursInterval,
            startDayHour: startDayHour,
            endDayHour: endDayHour,
            groups: groups,
            groupOrientation: _this3.groupOrientation
          })
        };
      });
    }
  };

  _proto.onViewRendered = function onViewRendered() {
    var _this$props2 = this.props,
        allDayPanelExpanded = _this$props2.allDayPanelExpanded,
        cellDuration = _this$props2.cellDuration,
        crossScrollingEnabled = _this$props2.crossScrollingEnabled,
        currentDate = _this$props2.currentDate,
        endDayHour = _this$props2.endDayHour,
        firstDayOfWeek = _this$props2.firstDayOfWeek,
        groupByDate = _this$props2.groupByDate,
        groupOrientation = _this$props2.groupOrientation,
        groups = _this$props2.groups,
        hoursInterval = _this$props2.hoursInterval,
        intervalCount = _this$props2.intervalCount,
        onViewRendered = _this$props2.onViewRendered,
        scrolling = _this$props2.scrolling,
        showAllDayPanel = _this$props2.showAllDayPanel,
        startDate = _this$props2.startDate,
        startDayHour = _this$props2.startDayHour,
        viewType = _this$props2.type;
    var tableWidths = (0, _utils.getDateTableWidth)(this.layoutRef.current.getScrollableWidth(), this.dateTableRef.current, this.viewDataProvider, {
      intervalCount: intervalCount,
      currentDate: currentDate,
      viewType: viewType,
      hoursInterval: hoursInterval,
      startDayHour: startDayHour,
      endDayHour: endDayHour,
      groups: groups,
      groupOrientation: this.groupOrientation
    });

    if (!this.isCalculateTablesWidth || tableWidths === this.state.tablesWidth) {
      var cellCount = this.viewDataProvider.getCellCount({
        intervalCount: intervalCount,
        currentDate: currentDate,
        viewType: viewType,
        hoursInterval: hoursInterval,
        startDayHour: startDayHour,
        endDayHour: endDayHour
      });
      var totalCellCount = (0, _utils.getTotalCellCount)(cellCount, this.groupOrientation, groups);
      var dateTableCellsMeta = this.createDateTableElementsMeta(totalCellCount);
      var allDayPanelCellsMeta = this.createAllDayPanelElementsMeta();
      onViewRendered({
        viewDataProvider: this.viewDataProvider,
        cellsMetaData: {
          dateTableCellsMeta: dateTableCellsMeta,
          allDayPanelCellsMeta: allDayPanelCellsMeta
        },
        viewDataProviderValidationOptions: {
          intervalCount: intervalCount,
          currentDate: currentDate,
          type: viewType,
          hoursInterval: hoursInterval,
          startDayHour: startDayHour,
          endDayHour: endDayHour,
          groups: groups,
          groupOrientation: groupOrientation,
          groupByDate: groupByDate,
          crossScrollingEnabled: crossScrollingEnabled,
          firstDayOfWeek: firstDayOfWeek,
          startDate: startDate,
          showAllDayPanel: showAllDayPanel,
          allDayPanelExpanded: allDayPanelExpanded,
          scrolling: scrolling,
          cellDuration: cellDuration
        }
      });
    }
  };

  _proto.createDateTableElementsMeta = function createDateTableElementsMeta(totalCellCount) {
    var dateTableCells = this.dateTableRef.current.querySelectorAll("td");
    var dateTableRect = this.dateTableRef.current.getBoundingClientRect();
    var dateTableCellsMeta = [];
    dateTableCells.forEach(function (cellElement, index) {
      if (index % totalCellCount === 0) {
        dateTableCellsMeta.push([]);
      }

      var cellRect = cellElement.getBoundingClientRect();
      var validCellRect = (0, _utils.createCellElementMetaData)(dateTableRect, cellRect);
      dateTableCellsMeta[dateTableCellsMeta.length - 1].push(validCellRect);
    });
    return dateTableCellsMeta;
  };

  _proto.createAllDayPanelElementsMeta = function createAllDayPanelElementsMeta() {
    if (!this.allDayPanelRef.current) {
      return [];
    }

    var allDayPanelCells = this.allDayPanelRef.current.querySelectorAll("td");
    var allDayPanelRect = this.allDayPanelRef.current.getBoundingClientRect();
    var allDayPanelCellsMeta = [];
    allDayPanelCells.forEach(function (cellElement) {
      var cellRect = cellElement.getBoundingClientRect();
      allDayPanelCellsMeta.push((0, _utils.createCellElementMetaData)(allDayPanelRect, cellRect));
    });
    return allDayPanelCellsMeta;
  };

  _proto.componentWillUpdate = function componentWillUpdate(nextProps, nextState, context) {
    _InfernoComponent.prototype.componentWillUpdate.call(this);

    if (this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["groups"] !== nextProps["groups"] || this.props["groupOrientation"] !== nextProps["groupOrientation"]) {
      this.__getterCache["renderConfig"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"]) {
      this.__getterCache["viewDataGenerator"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"]) {
      this.__getterCache["dateHeaderDataGenerator"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"]) {
      this.__getterCache["timePanelDataGenerator"] = undefined;
    }

    if (this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["completeViewDataMap"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["viewDataMap"] = undefined;
    }

    if (this.props["groups"] !== nextProps["groups"] || this.props["type"] !== nextProps["type"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["viewData"] = undefined;
    }

    if (this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["completeDateHeaderData"] = undefined;
    }

    if (this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["dateHeaderData"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["groups"] !== nextProps["groups"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["completeTimePanelData"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["groups"] !== nextProps["groups"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["timePanelData"] = undefined;
    }

    if (this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["viewDataProvider"] = undefined;
    }

    if (this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["groupByDate"] !== nextProps["groupByDate"]) {
      this.__getterCache["groupPanelData"] = undefined;
    }
  };

  _proto.render = function render() {
    var props = this.props;
    return viewFunction({
      props: _extends({}, props, {
        dataCellTemplate: getTemplate(props.dataCellTemplate),
        dateCellTemplate: getTemplate(props.dateCellTemplate),
        timeCellTemplate: getTemplate(props.timeCellTemplate),
        resourceCellTemplate: getTemplate(props.resourceCellTemplate)
      }),
      groupPanelHeight: this.state.groupPanelHeight,
      headerEmptyCellWidth: this.state.headerEmptyCellWidth,
      tablesWidth: this.state.tablesWidth,
      dateTableRef: this.dateTableRef,
      allDayPanelRef: this.allDayPanelRef,
      timePanelRef: this.timePanelRef,
      groupPanelRef: this.groupPanelRef,
      layoutRef: this.layoutRef,
      renderConfig: this.renderConfig,
      groupOrientation: this.groupOrientation,
      isVerticalGrouping: this.isVerticalGrouping,
      isHorizontalGrouping: this.isHorizontalGrouping,
      isGroupedByDate: this.isGroupedByDate,
      layout: this.layout,
      isAllDayPanelVisible: this.isAllDayPanelVisible,
      viewDataGenerator: this.viewDataGenerator,
      dateHeaderDataGenerator: this.dateHeaderDataGenerator,
      timePanelDataGenerator: this.timePanelDataGenerator,
      startViewDate: this.startViewDate,
      completeViewDataMap: this.completeViewDataMap,
      viewDataMap: this.viewDataMap,
      viewData: this.viewData,
      completeDateHeaderData: this.completeDateHeaderData,
      dateHeaderData: this.dateHeaderData,
      completeTimePanelData: this.completeTimePanelData,
      timePanelData: this.timePanelData,
      viewDataProvider: this.viewDataProvider,
      groupPanelData: this.groupPanelData,
      headerPanelTemplate: this.headerPanelTemplate,
      dateTableTemplate: this.dateTableTemplate,
      timePanelTemplate: this.timePanelTemplate,
      isRenderHeaderEmptyCell: this.isRenderHeaderEmptyCell,
      isWorkSpaceWithOddCells: this.isWorkSpaceWithOddCells,
      classes: this.classes,
      isStandaloneAllDayPanel: this.isStandaloneAllDayPanel,
      isCalculateTablesWidth: this.isCalculateTablesWidth,
      createDateTableElementsMeta: this.createDateTableElementsMeta,
      createAllDayPanelElementsMeta: this.createAllDayPanelElementsMeta,
      restAttributes: this.restAttributes
    });
  };

  _createClass(WorkSpace, [{
    key: "renderConfig",
    get: function get() {
      var _this4 = this;

      if (this.__getterCache["renderConfig"] !== undefined) {
        return this.__getterCache["renderConfig"];
      }

      return this.__getterCache["renderConfig"] = function () {
        return (0, _work_space_config.getViewRenderConfigByType)(_this4.props.type, _this4.props.crossScrollingEnabled, _this4.props.intervalCount, _this4.props.groups, _this4.props.groupOrientation);
      }();
    }
  }, {
    key: "groupOrientation",
    get: function get() {
      var groupOrientation = this.props.groupOrientation;
      var defaultGroupOrientation = this.renderConfig.defaultGroupOrientation;
      return groupOrientation !== null && groupOrientation !== void 0 ? groupOrientation : defaultGroupOrientation;
    }
  }, {
    key: "isVerticalGrouping",
    get: function get() {
      return (0, _utils2.isVerticalGroupingApplied)(this.props.groups, this.groupOrientation);
    }
  }, {
    key: "isHorizontalGrouping",
    get: function get() {
      return (0, _utils2.isHorizontalGroupingApplied)(this.props.groups, this.groupOrientation);
    }
  }, {
    key: "isGroupedByDate",
    get: function get() {
      return (0, _utils2.isGroupingByDate)(this.props.groups, this.groupOrientation, this.props.groupByDate);
    }
  }, {
    key: "layout",
    get: function get() {
      return this.renderConfig.isCreateCrossScrolling ? _cross_scrolling_layout.CrossScrollingLayout : _ordinary_layout.OrdinaryLayout;
    }
  }, {
    key: "isAllDayPanelVisible",
    get: function get() {
      var showAllDayPanel = this.props.showAllDayPanel;
      var isAllDayPanelSupported = this.renderConfig.isAllDayPanelSupported;
      return isAllDayPanelSupported && showAllDayPanel;
    }
  }, {
    key: "viewDataGenerator",
    get: function get() {
      var _this5 = this;

      if (this.__getterCache["viewDataGenerator"] !== undefined) {
        return this.__getterCache["viewDataGenerator"];
      }

      return this.__getterCache["viewDataGenerator"] = function () {
        return (0, _utils3.getViewDataGeneratorByViewType)(_this5.props.type);
      }();
    }
  }, {
    key: "dateHeaderDataGenerator",
    get: function get() {
      var _this6 = this;

      if (this.__getterCache["dateHeaderDataGenerator"] !== undefined) {
        return this.__getterCache["dateHeaderDataGenerator"];
      }

      return this.__getterCache["dateHeaderDataGenerator"] = function () {
        return new _date_header_data_generator.DateHeaderDataGenerator(_this6.viewDataGenerator);
      }();
    }
  }, {
    key: "timePanelDataGenerator",
    get: function get() {
      var _this7 = this;

      if (this.__getterCache["timePanelDataGenerator"] !== undefined) {
        return this.__getterCache["timePanelDataGenerator"];
      }

      return this.__getterCache["timePanelDataGenerator"] = function () {
        return new _time_panel_data_generator.TimePanelDataGenerator(_this7.viewDataGenerator);
      }();
    }
  }, {
    key: "startViewDate",
    get: function get() {
      var _this$props3 = this.props,
          currentDate = _this$props3.currentDate,
          firstDayOfWeek = _this$props3.firstDayOfWeek,
          intervalCount = _this$props3.intervalCount,
          startDate = _this$props3.startDate,
          startDayHour = _this$props3.startDayHour,
          type = _this$props3.type;
      var options = {
        currentDate: currentDate,
        startDayHour: startDayHour,
        startDate: startDate,
        intervalCount: intervalCount,
        firstDayOfWeek: firstDayOfWeek
      };
      var viewDataGenerator = (0, _utils3.getViewDataGeneratorByViewType)(type);
      var startViewDate = viewDataGenerator.getStartViewDate(options);
      return startViewDate;
    }
  }, {
    key: "completeViewDataMap",
    get: function get() {
      var _this8 = this;

      if (this.__getterCache["completeViewDataMap"] !== undefined) {
        return this.__getterCache["completeViewDataMap"];
      }

      return this.__getterCache["completeViewDataMap"] = function () {
        var _this8$props = _this8.props,
            cellDuration = _this8$props.cellDuration,
            currentDate = _this8$props.currentDate,
            endDayHour = _this8$props.endDayHour,
            firstDayOfWeek = _this8$props.firstDayOfWeek,
            groupByDate = _this8$props.groupByDate,
            groups = _this8$props.groups,
            hoursInterval = _this8$props.hoursInterval,
            intervalCount = _this8$props.intervalCount,
            startDate = _this8$props.startDate,
            startDayHour = _this8$props.startDayHour,
            type = _this8$props.type;
        return _this8.viewDataGenerator.getCompleteViewDataMap({
          currentDate: currentDate,
          startDate: startDate,
          startDayHour: startDayHour,
          endDayHour: endDayHour,
          groupByDate: groupByDate,
          groups: groups,
          intervalCount: intervalCount,
          firstDayOfWeek: firstDayOfWeek,
          hoursInterval: hoursInterval,
          cellDuration: cellDuration,
          startViewDate: _this8.startViewDate,
          groupOrientation: _this8.groupOrientation,
          isVerticalGrouping: _this8.isVerticalGrouping,
          isHorizontalGrouping: _this8.isHorizontalGrouping,
          isGroupedByDate: _this8.isGroupedByDate,
          isAllDayPanelVisible: _this8.isAllDayPanelVisible,
          viewType: type,
          interval: _this8.viewDataGenerator.getInterval(hoursInterval)
        });
      }();
    }
  }, {
    key: "viewDataMap",
    get: function get() {
      var _this9 = this;

      if (this.__getterCache["viewDataMap"] !== undefined) {
        return this.__getterCache["viewDataMap"];
      }

      return this.__getterCache["viewDataMap"] = function () {
        return _this9.viewDataGenerator.generateViewDataMap(_this9.completeViewDataMap, {
          startRowIndex: 0,
          startCellIndex: 0,
          isVerticalGrouping: _this9.isVerticalGrouping,
          isAllDayPanelVisible: _this9.isAllDayPanelVisible
        });
      }();
    }
  }, {
    key: "viewData",
    get: function get() {
      var _this10 = this;

      if (this.__getterCache["viewData"] !== undefined) {
        return this.__getterCache["viewData"];
      }

      return this.__getterCache["viewData"] = function () {
        var groups = _this10.props.groups;
        return _this10.viewDataGenerator.getViewDataFromMap(_this10.completeViewDataMap, _this10.viewDataMap, {
          topVirtualRowHeight: 0,
          bottomVirtualRowHeight: 0,
          leftVirtualCellWidth: 0,
          rightVirtualCellWidth: 0,
          startRowIndex: 0,
          startCellIndex: 0,
          isProvideVirtualCellsWidth: _this10.renderConfig.isProvideVirtualCellsWidth,
          isVerticalGrouping: _this10.isVerticalGrouping,
          isAllDayPanelVisible: _this10.isAllDayPanelVisible,
          isGroupedAllDayPanel: (0, _base.calculateIsGroupedAllDayPanel)(groups, _this10.groupOrientation, _this10.isAllDayPanelVisible)
        });
      }();
    }
  }, {
    key: "completeDateHeaderData",
    get: function get() {
      var _this11 = this;

      if (this.__getterCache["completeDateHeaderData"] !== undefined) {
        return this.__getterCache["completeDateHeaderData"];
      }

      return this.__getterCache["completeDateHeaderData"] = function () {
        var _this11$props = _this11.props,
            currentDate = _this11$props.currentDate,
            endDayHour = _this11$props.endDayHour,
            groups = _this11$props.groups,
            hoursInterval = _this11$props.hoursInterval,
            intervalCount = _this11$props.intervalCount,
            startDayHour = _this11$props.startDayHour,
            viewType = _this11$props.type;
        return _this11.dateHeaderDataGenerator.getCompleteDateHeaderMap({
          isGenerateWeekDaysHeaderData: _this11.renderConfig.isGenerateWeekDaysHeaderData,
          isGroupedByDate: _this11.isGroupedByDate,
          groups: groups,
          groupOrientation: _this11.groupOrientation,
          isHorizontalGrouping: _this11.isHorizontalGrouping,
          startDayHour: startDayHour,
          endDayHour: endDayHour,
          hoursInterval: hoursInterval,
          intervalCount: intervalCount,
          headerCellTextFormat: _this11.renderConfig.headerCellTextFormat,
          getDateForHeaderText: _this11.renderConfig.getDateForHeaderText,
          interval: _this11.viewDataGenerator.getInterval(hoursInterval),
          startViewDate: _this11.startViewDate,
          currentDate: currentDate,
          viewType: viewType,
          today: new Date()
        }, _this11.completeViewDataMap);
      }();
    }
  }, {
    key: "dateHeaderData",
    get: function get() {
      var _this12 = this;

      if (this.__getterCache["dateHeaderData"] !== undefined) {
        return this.__getterCache["dateHeaderData"];
      }

      return this.__getterCache["dateHeaderData"] = function () {
        var _this12$props = _this12.props,
            endDayHour = _this12$props.endDayHour,
            groups = _this12$props.groups,
            hoursInterval = _this12$props.hoursInterval,
            startDayHour = _this12$props.startDayHour;
        return _this12.dateHeaderDataGenerator.generateDateHeaderData(_this12.completeDateHeaderData, _this12.completeViewDataMap, {
          isGenerateWeekDaysHeaderData: _this12.renderConfig.isGenerateWeekDaysHeaderData,
          cellWidth: 0,
          isProvideVirtualCellsWidth: _this12.renderConfig.isProvideVirtualCellsWidth,
          startDayHour: startDayHour,
          endDayHour: endDayHour,
          hoursInterval: hoursInterval,
          startCellIndex: 0,
          groups: groups,
          groupOrientation: _this12.groupOrientation,
          isGroupedByDate: _this12.isGroupedByDate
        });
      }();
    }
  }, {
    key: "completeTimePanelData",
    get: function get() {
      var _this13 = this;

      if (this.__getterCache["completeTimePanelData"] !== undefined) {
        return this.__getterCache["completeTimePanelData"];
      }

      return this.__getterCache["completeTimePanelData"] = function () {
        if (!_this13.renderConfig.isRenderTimePanel) {
          return undefined;
        }

        var _this13$props = _this13.props,
            cellDuration = _this13$props.cellDuration,
            currentDate = _this13$props.currentDate,
            endDayHour = _this13$props.endDayHour,
            hoursInterval = _this13$props.hoursInterval,
            intervalCount = _this13$props.intervalCount,
            startDayHour = _this13$props.startDayHour,
            type = _this13$props.type;
        return _this13.timePanelDataGenerator.getCompleteTimePanelMap({
          startViewDate: _this13.startViewDate,
          cellDuration: cellDuration,
          startDayHour: startDayHour,
          endDayHour: endDayHour,
          isVerticalGrouping: _this13.isVerticalGrouping,
          intervalCount: intervalCount,
          currentDate: currentDate,
          viewType: type,
          hoursInterval: hoursInterval
        }, _this13.completeViewDataMap);
      }();
    }
  }, {
    key: "timePanelData",
    get: function get() {
      var _this14 = this;

      if (this.__getterCache["timePanelData"] !== undefined) {
        return this.__getterCache["timePanelData"];
      }

      return this.__getterCache["timePanelData"] = function () {
        if (!_this14.completeTimePanelData) {
          return undefined;
        }

        return _this14.timePanelDataGenerator.generateTimePanelData(_this14.completeTimePanelData, {
          startRowIndex: 0,
          topVirtualRowHeight: 0,
          bottomVirtualRowHeight: 0,
          isGroupedAllDayPanel: (0, _base.calculateIsGroupedAllDayPanel)(_this14.props.groups, _this14.groupOrientation, _this14.isAllDayPanelVisible),
          isVerticalGrouping: _this14.isVerticalGrouping,
          isAllDayPanelVisible: _this14.isAllDayPanelVisible
        });
      }();
    }
  }, {
    key: "viewDataProvider",
    get: function get() {
      var _this15 = this;

      if (this.__getterCache["viewDataProvider"] !== undefined) {
        return this.__getterCache["viewDataProvider"];
      }

      return this.__getterCache["viewDataProvider"] = function () {
        var _this15$props = _this15.props,
            cellDuration = _this15$props.cellDuration,
            currentDate = _this15$props.currentDate,
            endDayHour = _this15$props.endDayHour,
            firstDayOfWeek = _this15$props.firstDayOfWeek,
            groups = _this15$props.groups,
            hoursInterval = _this15$props.hoursInterval,
            intervalCount = _this15$props.intervalCount,
            startDate = _this15$props.startDate,
            startDayHour = _this15$props.startDayHour,
            type = _this15$props.type;
        var viewDataProvider = new _view_data_provider.default(type);
        viewDataProvider.completeViewDataMap = _this15.completeViewDataMap;
        viewDataProvider.viewDataMap = _this15.viewDataMap;
        viewDataProvider.viewData = _this15.viewData;
        var generationOptions = prepareGenerationOptions({
          intervalCount: intervalCount,
          groups: groups,
          groupByDate: _this15.isGroupedByDate,
          groupOrientation: _this15.groupOrientation,
          startDayHour: startDayHour,
          endDayHour: endDayHour,
          currentDate: currentDate,
          startDate: startDate,
          firstDayOfWeek: firstDayOfWeek,
          hoursInterval: hoursInterval,
          type: type,
          cellDuration: cellDuration
        }, _this15.renderConfig, _this15.isAllDayPanelVisible);
        viewDataProvider.setViewOptions(generationOptions);
        viewDataProvider.createGroupedDataMapProvider();
        return viewDataProvider;
      }();
    }
  }, {
    key: "groupPanelData",
    get: function get() {
      var _this16 = this;

      if (this.__getterCache["groupPanelData"] !== undefined) {
        return this.__getterCache["groupPanelData"];
      }

      return this.__getterCache["groupPanelData"] = function () {
        var _this16$props = _this16.props,
            currentDate = _this16$props.currentDate,
            endDayHour = _this16$props.endDayHour,
            groups = _this16$props.groups,
            hoursInterval = _this16$props.hoursInterval,
            intervalCount = _this16$props.intervalCount,
            startDayHour = _this16$props.startDayHour,
            type = _this16$props.type;

        var columnCountPerGroup = _this16.viewDataGenerator.getCellCount({
          intervalCount: intervalCount,
          hoursInterval: hoursInterval,
          currentDate: currentDate,
          startDayHour: startDayHour,
          endDayHour: endDayHour,
          viewType: type
        });

        var groupPanelData = (0, _utils4.getGroupPanelData)(groups, columnCountPerGroup, _this16.isGroupedByDate, _this16.isGroupedByDate ? 1 : columnCountPerGroup);
        return groupPanelData;
      }();
    }
  }, {
    key: "headerPanelTemplate",
    get: function get() {
      var headerPanelTemplate = this.renderConfig.headerPanelTemplate;
      return headerPanelTemplate;
    }
  }, {
    key: "dateTableTemplate",
    get: function get() {
      var dateTableTemplate = this.renderConfig.dateTableTemplate;
      return dateTableTemplate;
    }
  }, {
    key: "timePanelTemplate",
    get: function get() {
      var timePanelTemplate = this.renderConfig.timePanelTemplate;
      return timePanelTemplate;
    }
  }, {
    key: "isRenderHeaderEmptyCell",
    get: function get() {
      return this.isVerticalGrouping || !!this.timePanelTemplate;
    }
  }, {
    key: "isWorkSpaceWithOddCells",
    get: function get() {
      return false;
    }
  }, {
    key: "classes",
    get: function get() {
      var _combineClasses;

      var _this$props4 = this.props,
          allDayPanelExpanded = _this$props4.allDayPanelExpanded,
          groups = _this$props4.groups,
          intervalCount = _this$props4.intervalCount;
      return (0, _combine_classes.combineClasses)((_combineClasses = {}, _defineProperty(_combineClasses, this.renderConfig.className, true), _defineProperty(_combineClasses, "dx-scheduler-work-space-count", intervalCount > 1), _defineProperty(_combineClasses, "dx-scheduler-work-space-odd-cells", !!this.isWorkSpaceWithOddCells), _defineProperty(_combineClasses, "dx-scheduler-work-space-all-day-collapsed", !allDayPanelExpanded && this.isAllDayPanelVisible), _defineProperty(_combineClasses, "dx-scheduler-work-space-all-day", this.isAllDayPanelVisible), _defineProperty(_combineClasses, "dx-scheduler-work-space-group-by-date", this.isGroupedByDate), _defineProperty(_combineClasses, "dx-scheduler-work-space-grouped", groups.length > 0), _defineProperty(_combineClasses, "dx-scheduler-work-space-vertical-grouped", this.isVerticalGrouping && this.renderConfig.defaultGroupOrientation !== "vertical"), _defineProperty(_combineClasses, "dx-scheduler-work-space-horizontal-grouped", (0, _utils2.isHorizontalGroupingApplied)(groups, this.groupOrientation) && this.renderConfig.defaultGroupOrientation === "vertical"), _defineProperty(_combineClasses, "dx-scheduler-group-column-count-one", this.isVerticalGrouping && groups.length === 1), _defineProperty(_combineClasses, "dx-scheduler-group-column-count-two", this.isVerticalGrouping && groups.length === 2), _defineProperty(_combineClasses, "dx-scheduler-group-column-count-three", this.isVerticalGrouping && groups.length === 3), _defineProperty(_combineClasses, "dx-scheduler-work-space-both-scrollbar", this.props.crossScrollingEnabled), _defineProperty(_combineClasses, "dx-scheduler-work-space", true), _combineClasses));
    }
  }, {
    key: "isStandaloneAllDayPanel",
    get: function get() {
      var groups = this.props.groups;
      return !(0, _utils2.isVerticalGroupingApplied)(groups, this.groupOrientation) && this.isAllDayPanelVisible;
    }
  }, {
    key: "isCalculateTablesWidth",
    get: function get() {
      return this.props.crossScrollingEnabled && this.renderConfig.defaultGroupOrientation !== "vertical";
    }
  }, {
    key: "restAttributes",
    get: function get() {
      var _this$props5 = this.props,
          accessKey = _this$props5.accessKey,
          activeStateEnabled = _this$props5.activeStateEnabled,
          allDayAppointments = _this$props5.allDayAppointments,
          allDayPanelExpanded = _this$props5.allDayPanelExpanded,
          allowMultipleCellSelection = _this$props5.allowMultipleCellSelection,
          appointments = _this$props5.appointments,
          cellDuration = _this$props5.cellDuration,
          className = _this$props5.className,
          crossScrollingEnabled = _this$props5.crossScrollingEnabled,
          currentDate = _this$props5.currentDate,
          dataCellTemplate = _this$props5.dataCellTemplate,
          dateCellTemplate = _this$props5.dateCellTemplate,
          disabled = _this$props5.disabled,
          endDayHour = _this$props5.endDayHour,
          firstDayOfWeek = _this$props5.firstDayOfWeek,
          focusStateEnabled = _this$props5.focusStateEnabled,
          groupByDate = _this$props5.groupByDate,
          groupOrientation = _this$props5.groupOrientation,
          groups = _this$props5.groups,
          height = _this$props5.height,
          hint = _this$props5.hint,
          hoursInterval = _this$props5.hoursInterval,
          hoverStateEnabled = _this$props5.hoverStateEnabled,
          indicatorTime = _this$props5.indicatorTime,
          indicatorUpdateInterval = _this$props5.indicatorUpdateInterval,
          intervalCount = _this$props5.intervalCount,
          onClick = _this$props5.onClick,
          onKeyDown = _this$props5.onKeyDown,
          onViewRendered = _this$props5.onViewRendered,
          resourceCellTemplate = _this$props5.resourceCellTemplate,
          rtlEnabled = _this$props5.rtlEnabled,
          schedulerHeight = _this$props5.schedulerHeight,
          schedulerWidth = _this$props5.schedulerWidth,
          scrolling = _this$props5.scrolling,
          selectedCellData = _this$props5.selectedCellData,
          shadeUntilCurrentTime = _this$props5.shadeUntilCurrentTime,
          showAllDayPanel = _this$props5.showAllDayPanel,
          showCurrentTimeIndicator = _this$props5.showCurrentTimeIndicator,
          startDate = _this$props5.startDate,
          startDayHour = _this$props5.startDayHour,
          tabIndex = _this$props5.tabIndex,
          timeCellTemplate = _this$props5.timeCellTemplate,
          type = _this$props5.type,
          visible = _this$props5.visible,
          width = _this$props5.width,
          restProps = _objectWithoutProperties(_this$props5, _excluded);

      return restProps;
    }
  }]);

  return WorkSpace;
}(_inferno2.InfernoComponent);

exports.WorkSpace = WorkSpace;
WorkSpace.defaultProps = _props.WorkSpaceProps;
