/**
* DevExtreme (esm/renovation/ui/scheduler/workspaces/base/work_space.js)
* Version: 21.2.3
* Build date: Thu Nov 11 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
var _excluded = ["accessKey", "activeStateEnabled", "allDayAppointments", "allDayPanelExpanded", "allowMultipleCellSelection", "appointments", "cellDuration", "className", "crossScrollingEnabled", "currentDate", "dataCellTemplate", "dateCellTemplate", "disabled", "endDayHour", "firstDayOfWeek", "focusStateEnabled", "groupByDate", "groupOrientation", "groups", "height", "hint", "hoursInterval", "hoverStateEnabled", "indicatorTime", "indicatorUpdateInterval", "intervalCount", "onClick", "onKeyDown", "onViewRendered", "resourceCellTemplate", "rtlEnabled", "schedulerHeight", "schedulerWidth", "scrolling", "selectedCellData", "shadeUntilCurrentTime", "showAllDayPanel", "showCurrentTimeIndicator", "startDate", "startDayHour", "tabIndex", "timeCellTemplate", "type", "visible", "width"];
import { createComponentVNode, normalizeProps } from "inferno";
import { InfernoEffect, InfernoComponent } from "@devextreme/runtime/inferno";
import { combineClasses } from "../../../../utils/combine_classes";
import { OrdinaryLayout } from "./ordinary_layout";
import ViewDataProvider from "../../../../../ui/scheduler/workspaces/view_model/view_data_provider";
import { createCellElementMetaData, getDateTableWidth, getTotalCellCount } from "./utils";
import { WorkSpaceProps } from "../props";
import { getViewRenderConfigByType } from "./work_space_config";
import { isGroupingByDate, isHorizontalGroupingApplied, isVerticalGroupingApplied } from "../utils";
import { CrossScrollingLayout } from "./cross_scrolling_layout";
import { getViewDataGeneratorByViewType } from "../../../../../ui/scheduler/workspaces/view_model/utils";
import { calculateIsGroupedAllDayPanel } from "../../view_model/to_test/views/utils/base";
import { DateHeaderDataGenerator } from "../../../../../ui/scheduler/workspaces/view_model/date_header_data_generator";
import { TimePanelDataGenerator } from "../../../../../ui/scheduler/workspaces/view_model/time_panel_data_generator";
import { getGroupPanelData } from "../../view_model/group_panel/utils";
export var prepareGenerationOptions = (workSpaceProps, renderConfig, isAllDayPanelVisible) => {
  var {
    cellDuration,
    currentDate,
    endDayHour,
    firstDayOfWeek,
    groupByDate,
    groupOrientation,
    groups,
    hoursInterval,
    intervalCount,
    startDate,
    startDayHour,
    type
  } = workSpaceProps;
  var {
    getDateForHeaderText,
    headerCellTextFormat,
    isGenerateWeekDaysHeaderData,
    isProvideVirtualCellsWidth,
    isRenderTimePanel
  } = renderConfig;
  return {
    startRowIndex: 0,
    startCellIndex: 0,
    groupOrientation,
    groupByDate,
    groups,
    isProvideVirtualCellsWidth,
    isAllDayPanelVisible,
    selectedCells: undefined,
    focusedCell: undefined,
    headerCellTextFormat,
    getDateForHeaderText,
    startDayHour,
    endDayHour,
    cellDuration,
    viewType: type,
    intervalCount,
    hoursInterval,
    currentDate,
    startDate,
    firstDayOfWeek,
    isGenerateTimePanelData: isRenderTimePanel,
    isGenerateWeekDaysHeaderData
  };
};
export var viewFunction = _ref => {
  var {
    allDayPanelRef,
    classes,
    dateHeaderData,
    dateTableRef,
    dateTableTemplate,
    groupOrientation,
    groupPanelData,
    groupPanelHeight,
    groupPanelRef,
    headerEmptyCellWidth,
    headerPanelTemplate,
    isAllDayPanelVisible,
    isGroupedByDate,
    isRenderHeaderEmptyCell,
    isStandaloneAllDayPanel,
    isVerticalGrouping,
    layout: Layout,
    layoutRef,
    props: {
      allDayAppointments,
      allDayPanelExpanded,
      appointments,
      dataCellTemplate,
      dateCellTemplate,
      groups,
      intervalCount,
      resourceCellTemplate,
      timeCellTemplate
    },
    renderConfig: {
      groupPanelClassName,
      isRenderDateHeader,
      scrollingDirection
    },
    tablesWidth,
    timePanelData,
    timePanelRef,
    timePanelTemplate,
    viewData
  } = _ref;
  return createComponentVNode(2, Layout, {
    "viewData": viewData,
    "dateHeaderData": dateHeaderData,
    "timePanelData": timePanelData,
    "groupPanelData": groupPanelData,
    "dataCellTemplate": dataCellTemplate,
    "dateCellTemplate": dateCellTemplate,
    "timeCellTemplate": timeCellTemplate,
    "resourceCellTemplate": resourceCellTemplate,
    "groups": groups,
    "groupByDate": isGroupedByDate,
    "groupOrientation": groupOrientation,
    "groupPanelClassName": groupPanelClassName,
    "intervalCount": intervalCount,
    "headerPanelTemplate": headerPanelTemplate,
    "dateTableTemplate": dateTableTemplate,
    "timePanelTemplate": timePanelTemplate,
    "isAllDayPanelCollapsed": !allDayPanelExpanded,
    "isAllDayPanelVisible": isAllDayPanelVisible,
    "isRenderDateHeader": isRenderDateHeader,
    "isRenderHeaderEmptyCell": isRenderHeaderEmptyCell,
    "isRenderGroupPanel": isVerticalGrouping,
    "isStandaloneAllDayPanel": isStandaloneAllDayPanel,
    "scrollingDirection": scrollingDirection,
    "groupPanelHeight": groupPanelHeight,
    "headerEmptyCellWidth": headerEmptyCellWidth,
    "tablesWidth": tablesWidth,
    "className": classes,
    "dateTableRef": dateTableRef,
    "allDayPanelRef": allDayPanelRef,
    "timePanelRef": timePanelRef,
    "groupPanelRef": groupPanelRef,
    "appointments": appointments,
    "allDayAppointments": allDayAppointments
  }, null, layoutRef);
};
import { createRef as infernoCreateRef } from "inferno";

var getTemplate = TemplateProp => TemplateProp && (TemplateProp.defaultProps ? props => normalizeProps(createComponentVNode(2, TemplateProp, _extends({}, props))) : TemplateProp);

export class WorkSpace extends InfernoComponent {
  constructor(props) {
    super(props);
    this.dateTableRef = infernoCreateRef();
    this.allDayPanelRef = infernoCreateRef();
    this.timePanelRef = infernoCreateRef();
    this.groupPanelRef = infernoCreateRef();
    this.layoutRef = infernoCreateRef();
    this.__getterCache = {};
    this.state = {
      groupPanelHeight: undefined,
      headerEmptyCellWidth: undefined,
      tablesWidth: undefined
    };
    this.groupPanelHeightEffect = this.groupPanelHeightEffect.bind(this);
    this.headerEmptyCellWidthEffect = this.headerEmptyCellWidthEffect.bind(this);
    this.tablesWidthEffect = this.tablesWidthEffect.bind(this);
    this.onViewRendered = this.onViewRendered.bind(this);
    this.createDateTableElementsMeta = this.createDateTableElementsMeta.bind(this);
    this.createAllDayPanelElementsMeta = this.createAllDayPanelElementsMeta.bind(this);
  }

  createEffects() {
    return [new InfernoEffect(this.groupPanelHeightEffect, [this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]), new InfernoEffect(this.headerEmptyCellWidthEffect, [this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]), new InfernoEffect(this.tablesWidthEffect, [this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]), new InfernoEffect(this.onViewRendered, [this.props.allDayPanelExpanded, this.props.cellDuration, this.props.crossScrollingEnabled, this.props.currentDate, this.props.endDayHour, this.props.firstDayOfWeek, this.props.groupByDate, this.props.groupOrientation, this.props.type, this.props.intervalCount, this.props.groups, this.props.hoursInterval, this.props.onViewRendered, this.props.scrolling, this.props.showAllDayPanel, this.props.startDate, this.props.startDayHour, this.state.tablesWidth])];
  }

  updateEffects() {
    var _this$_effects$, _this$_effects$2, _this$_effects$3, _this$_effects$4;

    (_this$_effects$ = this._effects[0]) === null || _this$_effects$ === void 0 ? void 0 : _this$_effects$.update([this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]);
    (_this$_effects$2 = this._effects[1]) === null || _this$_effects$2 === void 0 ? void 0 : _this$_effects$2.update([this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]);
    (_this$_effects$3 = this._effects[2]) === null || _this$_effects$3 === void 0 ? void 0 : _this$_effects$3.update([this.props, this.state.groupPanelHeight, this.state.headerEmptyCellWidth, this.state.tablesWidth, this.props.dataCellTemplate, this.props.dateCellTemplate, this.props.timeCellTemplate, this.props.resourceCellTemplate, this.props.intervalCount, this.props.groups, this.props.groupByDate, this.props.groupOrientation, this.props.crossScrollingEnabled, this.props.startDayHour, this.props.endDayHour, this.props.firstDayOfWeek, this.props.currentDate, this.props.startDate, this.props.hoursInterval, this.props.showAllDayPanel, this.props.allDayPanelExpanded, this.props.allowMultipleCellSelection, this.props.indicatorTime, this.props.indicatorUpdateInterval, this.props.shadeUntilCurrentTime, this.props.selectedCellData, this.props.scrolling, this.props.cellDuration, this.props.showCurrentTimeIndicator, this.props.schedulerHeight, this.props.schedulerWidth, this.props.type, this.props.onViewRendered, this.props.appointments, this.props.allDayAppointments, this.props.className, this.props.accessKey, this.props.activeStateEnabled, this.props.disabled, this.props.focusStateEnabled, this.props.height, this.props.hint, this.props.hoverStateEnabled, this.props.onClick, this.props.onKeyDown, this.props.rtlEnabled, this.props.tabIndex, this.props.visible, this.props.width]);
    (_this$_effects$4 = this._effects[3]) === null || _this$_effects$4 === void 0 ? void 0 : _this$_effects$4.update([this.props.allDayPanelExpanded, this.props.cellDuration, this.props.crossScrollingEnabled, this.props.currentDate, this.props.endDayHour, this.props.firstDayOfWeek, this.props.groupByDate, this.props.groupOrientation, this.props.type, this.props.intervalCount, this.props.groups, this.props.hoursInterval, this.props.onViewRendered, this.props.scrolling, this.props.showAllDayPanel, this.props.startDate, this.props.startDayHour, this.state.tablesWidth]);
  }

  groupPanelHeightEffect() {
    this.setState(__state_argument => {
      var _this$dateTableRef$cu;

      return {
        groupPanelHeight: (_this$dateTableRef$cu = this.dateTableRef.current) === null || _this$dateTableRef$cu === void 0 ? void 0 : _this$dateTableRef$cu.getBoundingClientRect().height
      };
    });
  }

  headerEmptyCellWidthEffect() {
    var _this$timePanelRef$cu, _this$timePanelRef$cu2, _this$groupPanelRef$c, _this$groupPanelRef$c2;

    var timePanelWidth = (_this$timePanelRef$cu = (_this$timePanelRef$cu2 = this.timePanelRef.current) === null || _this$timePanelRef$cu2 === void 0 ? void 0 : _this$timePanelRef$cu2.getBoundingClientRect().width) !== null && _this$timePanelRef$cu !== void 0 ? _this$timePanelRef$cu : 0;
    var groupPanelWidth = (_this$groupPanelRef$c = (_this$groupPanelRef$c2 = this.groupPanelRef.current) === null || _this$groupPanelRef$c2 === void 0 ? void 0 : _this$groupPanelRef$c2.getBoundingClientRect().width) !== null && _this$groupPanelRef$c !== void 0 ? _this$groupPanelRef$c : 0;
    this.setState(__state_argument => ({
      headerEmptyCellWidth: timePanelWidth + groupPanelWidth
    }));
  }

  tablesWidthEffect() {
    if (this.isCalculateTablesWidth) {
      var {
        currentDate,
        endDayHour,
        groups,
        hoursInterval,
        intervalCount,
        startDayHour,
        type: viewType
      } = this.props;
      this.setState(__state_argument => ({
        tablesWidth: getDateTableWidth(this.layoutRef.current.getScrollableWidth(), this.dateTableRef.current, this.viewDataProvider, {
          intervalCount,
          currentDate,
          viewType,
          hoursInterval,
          startDayHour,
          endDayHour,
          groups,
          groupOrientation: this.groupOrientation
        })
      }));
    }
  }

  onViewRendered() {
    var {
      allDayPanelExpanded,
      cellDuration,
      crossScrollingEnabled,
      currentDate,
      endDayHour,
      firstDayOfWeek,
      groupByDate,
      groupOrientation,
      groups,
      hoursInterval,
      intervalCount,
      onViewRendered,
      scrolling,
      showAllDayPanel,
      startDate,
      startDayHour,
      type: viewType
    } = this.props;
    var tableWidths = getDateTableWidth(this.layoutRef.current.getScrollableWidth(), this.dateTableRef.current, this.viewDataProvider, {
      intervalCount,
      currentDate,
      viewType,
      hoursInterval,
      startDayHour,
      endDayHour,
      groups,
      groupOrientation: this.groupOrientation
    });

    if (!this.isCalculateTablesWidth || tableWidths === this.state.tablesWidth) {
      var cellCount = this.viewDataProvider.getCellCount({
        intervalCount,
        currentDate,
        viewType,
        hoursInterval,
        startDayHour,
        endDayHour
      });
      var totalCellCount = getTotalCellCount(cellCount, this.groupOrientation, groups);
      var dateTableCellsMeta = this.createDateTableElementsMeta(totalCellCount);
      var allDayPanelCellsMeta = this.createAllDayPanelElementsMeta();
      onViewRendered({
        viewDataProvider: this.viewDataProvider,
        cellsMetaData: {
          dateTableCellsMeta,
          allDayPanelCellsMeta
        },
        viewDataProviderValidationOptions: {
          intervalCount,
          currentDate,
          type: viewType,
          hoursInterval,
          startDayHour,
          endDayHour,
          groups,
          groupOrientation,
          groupByDate,
          crossScrollingEnabled,
          firstDayOfWeek,
          startDate,
          showAllDayPanel,
          allDayPanelExpanded,
          scrolling,
          cellDuration
        }
      });
    }
  }

  get renderConfig() {
    if (this.__getterCache["renderConfig"] !== undefined) {
      return this.__getterCache["renderConfig"];
    }

    return this.__getterCache["renderConfig"] = (() => {
      return getViewRenderConfigByType(this.props.type, this.props.crossScrollingEnabled, this.props.intervalCount, this.props.groups, this.props.groupOrientation);
    })();
  }

  get groupOrientation() {
    var {
      groupOrientation
    } = this.props;
    var {
      defaultGroupOrientation
    } = this.renderConfig;
    return groupOrientation !== null && groupOrientation !== void 0 ? groupOrientation : defaultGroupOrientation;
  }

  get isVerticalGrouping() {
    return isVerticalGroupingApplied(this.props.groups, this.groupOrientation);
  }

  get isHorizontalGrouping() {
    return isHorizontalGroupingApplied(this.props.groups, this.groupOrientation);
  }

  get isGroupedByDate() {
    return isGroupingByDate(this.props.groups, this.groupOrientation, this.props.groupByDate);
  }

  get layout() {
    return this.renderConfig.isCreateCrossScrolling ? CrossScrollingLayout : OrdinaryLayout;
  }

  get isAllDayPanelVisible() {
    var {
      showAllDayPanel
    } = this.props;
    var {
      isAllDayPanelSupported
    } = this.renderConfig;
    return isAllDayPanelSupported && showAllDayPanel;
  }

  get viewDataGenerator() {
    if (this.__getterCache["viewDataGenerator"] !== undefined) {
      return this.__getterCache["viewDataGenerator"];
    }

    return this.__getterCache["viewDataGenerator"] = (() => {
      return getViewDataGeneratorByViewType(this.props.type);
    })();
  }

  get dateHeaderDataGenerator() {
    if (this.__getterCache["dateHeaderDataGenerator"] !== undefined) {
      return this.__getterCache["dateHeaderDataGenerator"];
    }

    return this.__getterCache["dateHeaderDataGenerator"] = (() => {
      return new DateHeaderDataGenerator(this.viewDataGenerator);
    })();
  }

  get timePanelDataGenerator() {
    if (this.__getterCache["timePanelDataGenerator"] !== undefined) {
      return this.__getterCache["timePanelDataGenerator"];
    }

    return this.__getterCache["timePanelDataGenerator"] = (() => {
      return new TimePanelDataGenerator(this.viewDataGenerator);
    })();
  }

  get startViewDate() {
    var {
      currentDate,
      firstDayOfWeek,
      intervalCount,
      startDate,
      startDayHour,
      type
    } = this.props;
    var options = {
      currentDate,
      startDayHour,
      startDate,
      intervalCount,
      firstDayOfWeek
    };
    var viewDataGenerator = getViewDataGeneratorByViewType(type);
    var startViewDate = viewDataGenerator.getStartViewDate(options);
    return startViewDate;
  }

  get completeViewDataMap() {
    if (this.__getterCache["completeViewDataMap"] !== undefined) {
      return this.__getterCache["completeViewDataMap"];
    }

    return this.__getterCache["completeViewDataMap"] = (() => {
      var {
        cellDuration,
        currentDate,
        endDayHour,
        firstDayOfWeek,
        groupByDate,
        groups,
        hoursInterval,
        intervalCount,
        startDate,
        startDayHour,
        type
      } = this.props;
      return this.viewDataGenerator.getCompleteViewDataMap({
        currentDate,
        startDate,
        startDayHour,
        endDayHour,
        groupByDate,
        groups,
        intervalCount,
        firstDayOfWeek,
        hoursInterval,
        cellDuration,
        startViewDate: this.startViewDate,
        groupOrientation: this.groupOrientation,
        isVerticalGrouping: this.isVerticalGrouping,
        isHorizontalGrouping: this.isHorizontalGrouping,
        isGroupedByDate: this.isGroupedByDate,
        isAllDayPanelVisible: this.isAllDayPanelVisible,
        viewType: type,
        interval: this.viewDataGenerator.getInterval(hoursInterval)
      });
    })();
  }

  get viewDataMap() {
    if (this.__getterCache["viewDataMap"] !== undefined) {
      return this.__getterCache["viewDataMap"];
    }

    return this.__getterCache["viewDataMap"] = (() => {
      return this.viewDataGenerator.generateViewDataMap(this.completeViewDataMap, {
        startRowIndex: 0,
        startCellIndex: 0,
        isVerticalGrouping: this.isVerticalGrouping,
        isAllDayPanelVisible: this.isAllDayPanelVisible
      });
    })();
  }

  get viewData() {
    if (this.__getterCache["viewData"] !== undefined) {
      return this.__getterCache["viewData"];
    }

    return this.__getterCache["viewData"] = (() => {
      var {
        groups
      } = this.props;
      return this.viewDataGenerator.getViewDataFromMap(this.completeViewDataMap, this.viewDataMap, {
        topVirtualRowHeight: 0,
        bottomVirtualRowHeight: 0,
        leftVirtualCellWidth: 0,
        rightVirtualCellWidth: 0,
        startRowIndex: 0,
        startCellIndex: 0,
        isProvideVirtualCellsWidth: this.renderConfig.isProvideVirtualCellsWidth,
        isVerticalGrouping: this.isVerticalGrouping,
        isAllDayPanelVisible: this.isAllDayPanelVisible,
        isGroupedAllDayPanel: calculateIsGroupedAllDayPanel(groups, this.groupOrientation, this.isAllDayPanelVisible)
      });
    })();
  }

  get completeDateHeaderData() {
    if (this.__getterCache["completeDateHeaderData"] !== undefined) {
      return this.__getterCache["completeDateHeaderData"];
    }

    return this.__getterCache["completeDateHeaderData"] = (() => {
      var {
        currentDate,
        endDayHour,
        groups,
        hoursInterval,
        intervalCount,
        startDayHour,
        type: viewType
      } = this.props;
      return this.dateHeaderDataGenerator.getCompleteDateHeaderMap({
        isGenerateWeekDaysHeaderData: this.renderConfig.isGenerateWeekDaysHeaderData,
        isGroupedByDate: this.isGroupedByDate,
        groups,
        groupOrientation: this.groupOrientation,
        isHorizontalGrouping: this.isHorizontalGrouping,
        startDayHour,
        endDayHour,
        hoursInterval,
        intervalCount,
        headerCellTextFormat: this.renderConfig.headerCellTextFormat,
        getDateForHeaderText: this.renderConfig.getDateForHeaderText,
        interval: this.viewDataGenerator.getInterval(hoursInterval),
        startViewDate: this.startViewDate,
        currentDate,
        viewType,
        today: new Date()
      }, this.completeViewDataMap);
    })();
  }

  get dateHeaderData() {
    if (this.__getterCache["dateHeaderData"] !== undefined) {
      return this.__getterCache["dateHeaderData"];
    }

    return this.__getterCache["dateHeaderData"] = (() => {
      var {
        endDayHour,
        groups,
        hoursInterval,
        startDayHour
      } = this.props;
      return this.dateHeaderDataGenerator.generateDateHeaderData(this.completeDateHeaderData, this.completeViewDataMap, {
        isGenerateWeekDaysHeaderData: this.renderConfig.isGenerateWeekDaysHeaderData,
        cellWidth: 0,
        isProvideVirtualCellsWidth: this.renderConfig.isProvideVirtualCellsWidth,
        startDayHour,
        endDayHour,
        hoursInterval,
        startCellIndex: 0,
        groups,
        groupOrientation: this.groupOrientation,
        isGroupedByDate: this.isGroupedByDate
      });
    })();
  }

  get completeTimePanelData() {
    if (this.__getterCache["completeTimePanelData"] !== undefined) {
      return this.__getterCache["completeTimePanelData"];
    }

    return this.__getterCache["completeTimePanelData"] = (() => {
      if (!this.renderConfig.isRenderTimePanel) {
        return undefined;
      }

      var {
        cellDuration,
        currentDate,
        endDayHour,
        hoursInterval,
        intervalCount,
        startDayHour,
        type
      } = this.props;
      return this.timePanelDataGenerator.getCompleteTimePanelMap({
        startViewDate: this.startViewDate,
        cellDuration,
        startDayHour,
        endDayHour,
        isVerticalGrouping: this.isVerticalGrouping,
        intervalCount,
        currentDate,
        viewType: type,
        hoursInterval
      }, this.completeViewDataMap);
    })();
  }

  get timePanelData() {
    if (this.__getterCache["timePanelData"] !== undefined) {
      return this.__getterCache["timePanelData"];
    }

    return this.__getterCache["timePanelData"] = (() => {
      if (!this.completeTimePanelData) {
        return undefined;
      }

      return this.timePanelDataGenerator.generateTimePanelData(this.completeTimePanelData, {
        startRowIndex: 0,
        topVirtualRowHeight: 0,
        bottomVirtualRowHeight: 0,
        isGroupedAllDayPanel: calculateIsGroupedAllDayPanel(this.props.groups, this.groupOrientation, this.isAllDayPanelVisible),
        isVerticalGrouping: this.isVerticalGrouping,
        isAllDayPanelVisible: this.isAllDayPanelVisible
      });
    })();
  }

  get viewDataProvider() {
    if (this.__getterCache["viewDataProvider"] !== undefined) {
      return this.__getterCache["viewDataProvider"];
    }

    return this.__getterCache["viewDataProvider"] = (() => {
      var {
        cellDuration,
        currentDate,
        endDayHour,
        firstDayOfWeek,
        groups,
        hoursInterval,
        intervalCount,
        startDate,
        startDayHour,
        type
      } = this.props;
      var viewDataProvider = new ViewDataProvider(type);
      viewDataProvider.completeViewDataMap = this.completeViewDataMap;
      viewDataProvider.viewDataMap = this.viewDataMap;
      viewDataProvider.viewData = this.viewData;
      var generationOptions = prepareGenerationOptions({
        intervalCount,
        groups,
        groupByDate: this.isGroupedByDate,
        groupOrientation: this.groupOrientation,
        startDayHour,
        endDayHour,
        currentDate,
        startDate,
        firstDayOfWeek,
        hoursInterval,
        type,
        cellDuration
      }, this.renderConfig, this.isAllDayPanelVisible);
      viewDataProvider.setViewOptions(generationOptions);
      viewDataProvider.createGroupedDataMapProvider();
      return viewDataProvider;
    })();
  }

  get groupPanelData() {
    if (this.__getterCache["groupPanelData"] !== undefined) {
      return this.__getterCache["groupPanelData"];
    }

    return this.__getterCache["groupPanelData"] = (() => {
      var {
        currentDate,
        endDayHour,
        groups,
        hoursInterval,
        intervalCount,
        startDayHour,
        type
      } = this.props;
      var columnCountPerGroup = this.viewDataGenerator.getCellCount({
        intervalCount,
        hoursInterval,
        currentDate,
        startDayHour,
        endDayHour,
        viewType: type
      });
      var groupPanelData = getGroupPanelData(groups, columnCountPerGroup, this.isGroupedByDate, this.isGroupedByDate ? 1 : columnCountPerGroup);
      return groupPanelData;
    })();
  }

  get headerPanelTemplate() {
    var {
      headerPanelTemplate
    } = this.renderConfig;
    return headerPanelTemplate;
  }

  get dateTableTemplate() {
    var {
      dateTableTemplate
    } = this.renderConfig;
    return dateTableTemplate;
  }

  get timePanelTemplate() {
    var {
      timePanelTemplate
    } = this.renderConfig;
    return timePanelTemplate;
  }

  get isRenderHeaderEmptyCell() {
    return this.isVerticalGrouping || !!this.timePanelTemplate;
  }

  get isWorkSpaceWithOddCells() {
    return false;
  }

  get classes() {
    var {
      allDayPanelExpanded,
      groups,
      intervalCount
    } = this.props;
    return combineClasses({
      [this.renderConfig.className]: true,
      "dx-scheduler-work-space-count": intervalCount > 1,
      "dx-scheduler-work-space-odd-cells": !!this.isWorkSpaceWithOddCells,
      "dx-scheduler-work-space-all-day-collapsed": !allDayPanelExpanded && this.isAllDayPanelVisible,
      "dx-scheduler-work-space-all-day": this.isAllDayPanelVisible,
      "dx-scheduler-work-space-group-by-date": this.isGroupedByDate,
      "dx-scheduler-work-space-grouped": groups.length > 0,
      "dx-scheduler-work-space-vertical-grouped": this.isVerticalGrouping && this.renderConfig.defaultGroupOrientation !== "vertical",
      "dx-scheduler-work-space-horizontal-grouped": isHorizontalGroupingApplied(groups, this.groupOrientation) && this.renderConfig.defaultGroupOrientation === "vertical",
      "dx-scheduler-group-column-count-one": this.isVerticalGrouping && groups.length === 1,
      "dx-scheduler-group-column-count-two": this.isVerticalGrouping && groups.length === 2,
      "dx-scheduler-group-column-count-three": this.isVerticalGrouping && groups.length === 3,
      "dx-scheduler-work-space-both-scrollbar": this.props.crossScrollingEnabled,
      "dx-scheduler-work-space": true
    });
  }

  get isStandaloneAllDayPanel() {
    var {
      groups
    } = this.props;
    return !isVerticalGroupingApplied(groups, this.groupOrientation) && this.isAllDayPanelVisible;
  }

  get isCalculateTablesWidth() {
    return this.props.crossScrollingEnabled && this.renderConfig.defaultGroupOrientation !== "vertical";
  }

  createDateTableElementsMeta(totalCellCount) {
    var dateTableCells = this.dateTableRef.current.querySelectorAll("td");
    var dateTableRect = this.dateTableRef.current.getBoundingClientRect();
    var dateTableCellsMeta = [];
    dateTableCells.forEach((cellElement, index) => {
      if (index % totalCellCount === 0) {
        dateTableCellsMeta.push([]);
      }

      var cellRect = cellElement.getBoundingClientRect();
      var validCellRect = createCellElementMetaData(dateTableRect, cellRect);
      dateTableCellsMeta[dateTableCellsMeta.length - 1].push(validCellRect);
    });
    return dateTableCellsMeta;
  }

  createAllDayPanelElementsMeta() {
    if (!this.allDayPanelRef.current) {
      return [];
    }

    var allDayPanelCells = this.allDayPanelRef.current.querySelectorAll("td");
    var allDayPanelRect = this.allDayPanelRef.current.getBoundingClientRect();
    var allDayPanelCellsMeta = [];
    allDayPanelCells.forEach(cellElement => {
      var cellRect = cellElement.getBoundingClientRect();
      allDayPanelCellsMeta.push(createCellElementMetaData(allDayPanelRect, cellRect));
    });
    return allDayPanelCellsMeta;
  }

  get restAttributes() {
    var _this$props = this.props,
        restProps = _objectWithoutPropertiesLoose(_this$props, _excluded);

    return restProps;
  }

  componentWillUpdate(nextProps, nextState, context) {
    super.componentWillUpdate();

    if (this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["groups"] !== nextProps["groups"] || this.props["groupOrientation"] !== nextProps["groupOrientation"]) {
      this.__getterCache["renderConfig"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"]) {
      this.__getterCache["viewDataGenerator"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"]) {
      this.__getterCache["dateHeaderDataGenerator"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"]) {
      this.__getterCache["timePanelDataGenerator"] = undefined;
    }

    if (this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["completeViewDataMap"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["viewDataMap"] = undefined;
    }

    if (this.props["groups"] !== nextProps["groups"] || this.props["type"] !== nextProps["type"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["viewData"] = undefined;
    }

    if (this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["completeDateHeaderData"] = undefined;
    }

    if (this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["dateHeaderData"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["groups"] !== nextProps["groups"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["completeTimePanelData"] = undefined;
    }

    if (this.props["type"] !== nextProps["type"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["groups"] !== nextProps["groups"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["startDate"] !== nextProps["startDate"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["timePanelData"] = undefined;
    }

    if (this.props["cellDuration"] !== nextProps["cellDuration"] || this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["firstDayOfWeek"] !== nextProps["firstDayOfWeek"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDate"] !== nextProps["startDate"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["groupByDate"] !== nextProps["groupByDate"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["showAllDayPanel"] !== nextProps["showAllDayPanel"]) {
      this.__getterCache["viewDataProvider"] = undefined;
    }

    if (this.props["currentDate"] !== nextProps["currentDate"] || this.props["endDayHour"] !== nextProps["endDayHour"] || this.props["groups"] !== nextProps["groups"] || this.props["hoursInterval"] !== nextProps["hoursInterval"] || this.props["intervalCount"] !== nextProps["intervalCount"] || this.props["startDayHour"] !== nextProps["startDayHour"] || this.props["type"] !== nextProps["type"] || this.props["groupOrientation"] !== nextProps["groupOrientation"] || this.props["crossScrollingEnabled"] !== nextProps["crossScrollingEnabled"] || this.props["groupByDate"] !== nextProps["groupByDate"]) {
      this.__getterCache["groupPanelData"] = undefined;
    }
  }

  render() {
    var props = this.props;
    return viewFunction({
      props: _extends({}, props, {
        dataCellTemplate: getTemplate(props.dataCellTemplate),
        dateCellTemplate: getTemplate(props.dateCellTemplate),
        timeCellTemplate: getTemplate(props.timeCellTemplate),
        resourceCellTemplate: getTemplate(props.resourceCellTemplate)
      }),
      groupPanelHeight: this.state.groupPanelHeight,
      headerEmptyCellWidth: this.state.headerEmptyCellWidth,
      tablesWidth: this.state.tablesWidth,
      dateTableRef: this.dateTableRef,
      allDayPanelRef: this.allDayPanelRef,
      timePanelRef: this.timePanelRef,
      groupPanelRef: this.groupPanelRef,
      layoutRef: this.layoutRef,
      renderConfig: this.renderConfig,
      groupOrientation: this.groupOrientation,
      isVerticalGrouping: this.isVerticalGrouping,
      isHorizontalGrouping: this.isHorizontalGrouping,
      isGroupedByDate: this.isGroupedByDate,
      layout: this.layout,
      isAllDayPanelVisible: this.isAllDayPanelVisible,
      viewDataGenerator: this.viewDataGenerator,
      dateHeaderDataGenerator: this.dateHeaderDataGenerator,
      timePanelDataGenerator: this.timePanelDataGenerator,
      startViewDate: this.startViewDate,
      completeViewDataMap: this.completeViewDataMap,
      viewDataMap: this.viewDataMap,
      viewData: this.viewData,
      completeDateHeaderData: this.completeDateHeaderData,
      dateHeaderData: this.dateHeaderData,
      completeTimePanelData: this.completeTimePanelData,
      timePanelData: this.timePanelData,
      viewDataProvider: this.viewDataProvider,
      groupPanelData: this.groupPanelData,
      headerPanelTemplate: this.headerPanelTemplate,
      dateTableTemplate: this.dateTableTemplate,
      timePanelTemplate: this.timePanelTemplate,
      isRenderHeaderEmptyCell: this.isRenderHeaderEmptyCell,
      isWorkSpaceWithOddCells: this.isWorkSpaceWithOddCells,
      classes: this.classes,
      isStandaloneAllDayPanel: this.isStandaloneAllDayPanel,
      isCalculateTablesWidth: this.isCalculateTablesWidth,
      createDateTableElementsMeta: this.createDateTableElementsMeta,
      createAllDayPanelElementsMeta: this.createAllDayPanelElementsMeta,
      restAttributes: this.restAttributes
    });
  }

}
WorkSpace.defaultProps = WorkSpaceProps;
