/**
* DevExtreme (dx.all.d.ts)
* Version: 21.2.3
* Build date: Thu Nov 11 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
declare global {
  interface JQuery<TElement = HTMLElement> {}
  interface JQueryEventObject {}
  interface JQueryPromise<T> {}
}
declare module DevExpress {
  /**
   * [descr:AnimationConfig]
   */
  export type AnimationConfig = {
    /**
     * [descr:AnimationConfig.complete]
     */
    complete?: (
      $element: DevExpress.core.DxElement,
      config: AnimationConfig
    ) => void;
    /**
     * [descr:AnimationConfig.delay]
     */
    delay?: number;
    /**
     * [descr:AnimationConfig.direction]
     */
    direction?: 'bottom' | 'left' | 'right' | 'top';
    /**
     * [descr:AnimationConfig.duration]
     */
    duration?: number;
    /**
     * [descr:AnimationConfig.easing]
     */
    easing?: string;
    /**
     * [descr:AnimationConfig.from]
     */
    from?: DevExpress.animation.AnimationState;
    /**
     * [descr:AnimationConfig.staggerDelay]
     */
    staggerDelay?: number;
    /**
     * [descr:AnimationConfig.start]
     */
    start?: (
      $element: DevExpress.core.DxElement,
      config: AnimationConfig
    ) => void;
    /**
     * [descr:AnimationConfig.to]
     */
    to?: DevExpress.animation.AnimationState;
    /**
     * [descr:AnimationConfig.type]
     */
    type?:
      | 'css'
      | 'fade'
      | 'fadeIn'
      | 'fadeOut'
      | 'pop'
      | 'slide'
      | 'slideIn'
      | 'slideOut';
  };
  /**
   * [descr:animationPresets]
   */
  export const animationPresets: {
    /**
     * [descr:animationPresets.applyChanges()]
     */
    applyChanges(): void;
    /**
     * [descr:animationPresets.clear()]
     */
    clear(): void;
    /**
     * [descr:animationPresets.clear(name)]
     */
    clear(name: string): void;
    /**
     * [descr:animationPresets.getPreset(name)]
     */
    getPreset(name: string): AnimationConfig;
    /**
     * [descr:animationPresets.registerDefaultPresets()]
     */
    registerDefaultPresets(): void;
    /**
     * [descr:animationPresets.registerPreset(name, config)]
     */
    registerPreset(
      name: string,
      config: { animation: AnimationConfig; device?: Device }
    ): void;
    /**
     * [descr:animationPresets.resetToDefaults()]
     */
    resetToDefaults(): void;
  };
  /**
   * [descr:Component]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class Component<TProperties> {
    constructor(options?: TProperties);
    /**
     * [descr:Component.beginUpdate()]
     */
    beginUpdate(): void;
    /**
     * [descr:Component.endUpdate()]
     */
    endUpdate(): void;
    /**
     * [descr:Component.instance()]
     */
    instance(): this;
    /**
     * [descr:Component.off(eventName)]
     */
    off(eventName: string): this;
    /**
     * [descr:Component.off(eventName, eventHandler)]
     */
    off(eventName: string, eventHandler: Function): this;
    /**
     * [descr:Component.on(eventName, eventHandler)]
     */
    on(eventName: string, eventHandler: Function): this;
    /**
     * [descr:Component.on(events)]
     */
    on(events: { [key: string]: Function }): this;
    /**
     * [descr:Component.option()]
     */
    option(): TProperties;
    /**
     * [descr:Component.option(optionName)]
     */
    option<TPropertyName extends string>(
      optionName: TPropertyName
    ): TPropertyName extends keyof TProperties
      ? TProperties[TPropertyName]
      : unknown;
    /**
     * [descr:Component.option(optionName, optionValue)]
     */
    option<TPropertyName extends string>(
      optionName: TPropertyName,
      optionValue: TPropertyName extends keyof TProperties
        ? TProperties[TPropertyName]
        : unknown
    ): void;
    /**
     * [descr:Component.option(options)]
     */
    option(options: Partial<TProperties>): void;
    /**
     * [descr:Component.resetOption(optionName)]
     */
    resetOption(optionName: string): void;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ComponentOptions<TComponent> {
    /**
     * [descr:ComponentOptions.onDisposing]
     */
    onDisposing?: (e: { component: TComponent }) => void;
    /**
     * [descr:ComponentOptions.onInitialized]
     */
    onInitialized?: (e: {
      component?: TComponent;
      element?: DevExpress.core.DxElement;
    }) => void;
    /**
     * [descr:ComponentOptions.onOptionChanged]
     */
    onOptionChanged?: (e: {
      component?: TComponent;
      name?: string;
      fullName?: string;
      value?: any;
    }) => void;
  }
  /**
   * [descr:config()]
   */
  export function config(): globalConfig;
  /**
   * [descr:config(config)]
   */
  export function config(config: globalConfig): void;
  /**
   * [descr:DataHelperMixin]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class DataHelperMixin {
    /**
     * [descr:DataHelperMixin.getDataSource()]
     */
    getDataSource(): DevExpress.data.DataSource;
  }
  /**
   * [descr:Device]
   */
  export type Device = {
    /**
     * [descr:Device.android]
     */
    android?: boolean;
    /**
     * [descr:Device.deviceType]
     */
    deviceType?: 'phone' | 'tablet' | 'desktop';
    /**
     * [descr:Device.generic]
     */
    generic?: boolean;
    /**
     * [descr:Device.grade]
     */
    grade?: 'A' | 'B' | 'C';
    /**
     * [descr:Device.ios]
     */
    ios?: boolean;
    /**
     * [descr:Device.phone]
     */
    phone?: boolean;
    /**
     * [descr:Device.platform]
     */
    platform?: 'android' | 'ios' | 'generic';
    /**
     * [descr:Device.tablet]
     */
    tablet?: boolean;
    /**
     * [descr:Device.version]
     */
    version?: Array<number>;
  };
  /**
    * [descr:devices]
    * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
    */
   export const devices: DevicesObject;
  /**
   * [descr:DevicesObject]
   */
  export class DevicesObject {
    constructor(options?: { window?: Window });
    /**
     * [descr:DevicesObject.current()]
     */
    current(): Device;
    /**
     * [descr:DevicesObject.current(deviceName)]
     */
    current(deviceName: string | Device): void;
    /**
     * [descr:DevicesObject.off(eventName)]
     */
    off(eventName: DevExpress.core.EventName): this;
    /**
     * [descr:DevicesObject.off(eventName, eventHandler)]
     */
    off(eventName: DevExpress.core.EventName, eventHandler: Function): this;
    /**
     * [descr:DevicesObject.on(eventName, eventHandler)]
     */
    on(eventName: DevExpress.core.EventName, eventHandler: Function): this;
    /**
     * [descr:DevicesObject.on(events)]
     */
    on(events: { [key in DevExpress.core.EventName]?: Function }): this;
    /**
     * [descr:DevicesObject.orientation()]
     */
    orientation(): 'portrait' | 'landscape' | undefined;
    /**
     * [descr:DevicesObject.real()]
     */
    real(): Device;
    isSimulator(): boolean;
  }
  /**
   * [descr:DOMComponent]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class DOMComponent<
    TProperties = DevExpress.DOMComponent.Properties
  > extends Component<TProperties> {
    _templateManager: DevExpress.core.TemplateManager;

    _cancelOptionChange?: string;

    constructor(
      element: DevExpress.core.UserDefinedElement,
      options?: TProperties
    );

    /**
     * [descr:DOMComponent.getInstance(element)]
     */
    static getInstance(
      element: DevExpress.core.UserDefinedElement
    ): DOMComponent<DevExpress.DOMComponent.Properties>;

    /**
     * [descr:DOMComponent.defaultOptions(rule)]
     */
    static defaultOptions<TProperties = DevExpress.DOMComponent.Properties>(
      rule: DevExpress.core.DefaultOptionsRule<TProperties>
    ): void;

    /**
     * [descr:DOMComponent.dispose()]
     */
    dispose(): void;
    /**
     * [descr:DOMComponent.element()]
     */
    element(): DevExpress.core.DxElement;

    $element(): DevExpress.core.UserDefinedElement;
    _getTemplate(template: unknown): DevExpress.core.FunctionTemplate;
    _invalidate(): void;
    _refresh(): void;
    _notifyOptionChanged(
      fullName: string,
      value: unknown,
      previousValue: unknown
    ): void;
    _createElement(element: HTMLElement): void;
  }
  module DOMComponent {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface DOMComponentInstance extends DOMComponent<Properties> {}
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type Properties = DOMComponentOptions<DOMComponentInstance>;
  }
  /* eslint-disable no-underscore-dangle */

  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface DOMComponentOptions<TComponent>
    extends ComponentOptions<TComponent> {
    /**
     * [descr:DOMComponentOptions.bindingOptions]
     */
    bindingOptions?: { [key: string]: any };
    /**
     * [descr:DOMComponentOptions.elementAttr]
     */
    elementAttr?: { [key: string]: any };
    /**
     * [descr:DOMComponentOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:DOMComponentOptions.onDisposing]
     */
    onDisposing?: (e: {
      component?: TComponent;
      element?: DevExpress.core.DxElement;
      model?: any;
    }) => void;
    /**
     * [descr:DOMComponentOptions.onOptionChanged]
     */
    onOptionChanged?: (e: {
      component?: TComponent;
      element?: DevExpress.core.DxElement;
      model?: any;
      name?: string;
      fullName?: string;
      value?: any;
    }) => void;
    /**
     * [descr:DOMComponentOptions.rtlEnabled]
     */
    rtlEnabled?: boolean;
    /**
     * [descr:DOMComponentOptions.width]
     */
    width?: number | string | (() => number | string);
  }
  /**
   * [descr:dxSchedulerTimeZone]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSchedulerTimeZone {
    /**
     * [descr:dxSchedulerTimeZone.id]
     */
    id: string;
    /**
     * [descr:dxSchedulerTimeZone.offset]
     */
    offset: number;
    /**
     * [descr:dxSchedulerTimeZone.title]
     */
    title: string;
  }
  /**
   * [descr:EndpointSelector]
   */
  export class EndpointSelector {
    constructor(options: any);
    /**
     * [descr:EndpointSelector.urlFor(key)]
     */
    urlFor(key: string): string;
  }
  /**
   * [descr:ExportLoadPanel]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ExportLoadPanel {
    /**
     * [descr:ExportLoadPanel.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:ExportLoadPanel.text]
     */
    text?: string;
    /**
     * [descr:ExportLoadPanel.width]
     */
    width?: number;
    /**
     * [descr:ExportLoadPanel.height]
     */
    height?: number;
    /**
     * [descr:ExportLoadPanel.showIndicator]
     */
    showIndicator?: boolean;
    /**
     * [descr:ExportLoadPanel.indicatorSrc]
     */
    indicatorSrc?: string;
    /**
     * [descr:ExportLoadPanel.showPane]
     */
    showPane?: boolean;
    /**
     * [descr:ExportLoadPanel.shading]
     */
    shading?: boolean;
    /**
     * [descr:ExportLoadPanel.shadingColor]
     */
    shadingColor?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  type ExternalFormat = any;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface FormatObject {
    /**
     * [descr:Format.currency]
     */
    currency?: string;
    /**
     * [descr:Format.formatter]
     */
    formatter?: (value: number | Date) => string;
    /**
     * [descr:Format.parser]
     */
    parser?: (value: string) => number | Date;
    /**
     * [descr:Format.precision]
     */
    precision?: number;
    /**
     * [descr:Format.type]
     */
    type?: PredefinedFormat | string;
  }
  /**
   * [descr:fx]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export const fx: {
    /**
     * [descr:fx.animate(element, config)]
     */
    animate(
      element: Element,
      config: AnimationConfig
    ): DevExpress.core.utils.DxPromise<void>;

    /**
     * [descr:fx.isAnimating(element)]
     */
    isAnimating(element: Element): boolean;

    /**
     * [descr:fx.stop(element, jumpToEnd)]
     */
    stop(element: Element, jumpToEnd: boolean): void;
  };
  /**
   * [descr:globalConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface globalConfig {
    /**
     * [descr:globalConfig.decimalSeparator]
     * @deprecated [depNote:globalConfig.decimalSeparator]
     */
    decimalSeparator?: string;
    /**
     * [descr:globalConfig.defaultCurrency]
     */
    defaultCurrency?: string;
    /**
     * [descr:globalConfig.editorStylingMode]
     */
    editorStylingMode?: 'outlined' | 'underlined' | 'filled';
    /**
     * [descr:globalConfig.floatingActionButtonConfig]
     */
    floatingActionButtonConfig?: {
      /**
       * [descr:globalConfig.floatingActionButtonConfig.closeIcon]
       */
      closeIcon?: string;
      /**
       * [descr:globalConfig.floatingActionButtonConfig.direction]
       */
      direction?: 'auto' | 'up' | 'down';
      /**
       * [descr:globalConfig.floatingActionButtonConfig.icon]
       */
      icon?: string;
      /**
       * [descr:globalConfig.floatingActionButtonConfig.label]
       */
      label?: string;
      /**
       * [descr:globalConfig.floatingActionButtonConfig.maxSpeedDialActionCount]
       */
      maxSpeedDialActionCount?: number;
      /**
       * [descr:globalConfig.floatingActionButtonConfig.position]
       */
      position?:
        | 'bottom'
        | 'center'
        | 'left'
        | 'left bottom'
        | 'left top'
        | 'right'
        | 'right bottom'
        | 'right top'
        | 'top'
        | PositionConfig
        | Function;
      /**
       * [descr:globalConfig.floatingActionButtonConfig.shading]
       */
      shading?: boolean;
    };
    /**
     * [descr:globalConfig.forceIsoDateParsing]
     */
    forceIsoDateParsing?: boolean;
    /**
     * [descr:globalConfig.oDataFilterToLower]
     */
    oDataFilterToLower?: boolean;
    /**
     * [descr:globalConfig.rtlEnabled]
     */
    rtlEnabled?: boolean;
    /**
     * [descr:globalConfig.serverDecimalSeparator]
     */
    serverDecimalSeparator?: string;
    /**
     * [descr:globalConfig.thousandsSeparator]
     * @deprecated [depNote:globalConfig.thousandsSeparator]
     */
    thousandsSeparator?: string;
    /**
     * [descr:globalConfig.useLegacyStoreResult]
     */
    useLegacyStoreResult?: boolean;
    /**
     * [descr:globalConfig.useLegacyVisibleIndex]
     */
    useLegacyVisibleIndex?: boolean;
  }
  /**
   * [descr:hideTopOverlay()]
   */
  export function hideTopOverlay(): boolean;
  /**
   * [descr:PositionConfig]
   */
  export interface PositionConfig {
    /**
     * [descr:PositionConfig.at]
     */
    at?:
      | 'bottom'
      | 'center'
      | 'left'
      | 'left bottom'
      | 'left top'
      | 'right'
      | 'right bottom'
      | 'right top'
      | 'top'
      | {
          /**
           * [descr:PositionConfig.at.x]
           */
          x?: 'center' | 'left' | 'right';
          /**
           * [descr:PositionConfig.at.y]
           */
          y?: 'bottom' | 'center' | 'top';
        };
    /**
     * [descr:PositionConfig.boundary]
     */
    boundary?: string | DevExpress.core.UserDefinedElement | Window;
    /**
     * [descr:PositionConfig.boundaryOffset]
     */
    boundaryOffset?:
      | string
      | {
          /**
           * [descr:PositionConfig.boundaryOffset.x]
           */
          x?: number;
          /**
           * [descr:PositionConfig.boundaryOffset.y]
           */
          y?: number;
        };
    /**
     * [descr:PositionConfig.collision]
     */
    collision?:
      | 'fit'
      | 'fit flip'
      | 'fit flipfit'
      | 'fit none'
      | 'flip'
      | 'flip fit'
      | 'flip none'
      | 'flipfit'
      | 'flipfit fit'
      | 'flipfit none'
      | 'none'
      | 'none fit'
      | 'none flip'
      | 'none flipfit'
      | {
          /**
           * [descr:PositionConfig.collision.x]
           */
          x?: 'fit' | 'flip' | 'flipfit' | 'none';
          /**
           * [descr:PositionConfig.collision.y]
           */
          y?: 'fit' | 'flip' | 'flipfit' | 'none';
        };
    /**
     * [descr:PositionConfig.my]
     */
    my?:
      | 'bottom'
      | 'center'
      | 'left'
      | 'left bottom'
      | 'left top'
      | 'right'
      | 'right bottom'
      | 'right top'
      | 'top'
      | {
          /**
           * [descr:PositionConfig.my.x]
           */
          x?: 'center' | 'left' | 'right';
          /**
           * [descr:PositionConfig.my.y]
           */
          y?: 'bottom' | 'center' | 'top';
        };
    /**
     * [descr:PositionConfig.of]
     */
    of?: string | DevExpress.core.UserDefinedElement | Window;
    /**
     * [descr:PositionConfig.offset]
     */
    offset?:
      | string
      | {
          /**
           * [descr:PositionConfig.offset.x]
           */
          x?: number;
          /**
           * [descr:PositionConfig.offset.y]
           */
          y?: number;
        };
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  type PredefinedFormat =
    | 'billions'
    | 'currency'
    | 'day'
    | 'decimal'
    | 'exponential'
    | 'fixedPoint'
    | 'largeNumber'
    | 'longDate'
    | 'longTime'
    | 'millions'
    | 'millisecond'
    | 'month'
    | 'monthAndDay'
    | 'monthAndYear'
    | 'percent'
    | 'quarter'
    | 'quarterAndYear'
    | 'shortDate'
    | 'shortTime'
    | 'thousands'
    | 'trillions'
    | 'year'
    | 'dayOfWeek'
    | 'hour'
    | 'longDateLongTime'
    | 'minute'
    | 'second'
    | 'shortDateShortTime';
  /**
   * [descr:registerComponent(name, componentClass)]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export function registerComponent<TComponent>(
    name: string,
    componentClass: DevExpress.core.ComponentFactory<TComponent>
  ): void;
  /**
   * [descr:registerComponent(name, namespace, componentClass)]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export function registerComponent<TComponent>(
    name: string,
    namespace: {
      [key: string]: DevExpress.core.ComponentFactory<DOMComponent>;
    },
    componentClass: DevExpress.core.ComponentFactory<TComponent>
  ): void;
  /**
   * [descr:setTemplateEngine(name)]
   */
  export function setTemplateEngine(templateEngineName: string): void;
  /**
   * [descr:setTemplateEngine(options)]
   */
  export function setTemplateEngine(templateEngineOptions: {
    compile?: Function;
    render?: Function;
  }): void;
  /**
   * [descr:TransitionExecutor]
   */
  export class TransitionExecutor {
    /**
     * [descr:TransitionExecutor.enter(elements, animation)]
     */
    enter(
      elements: DevExpress.core.UserDefinedElementsArray,
      animation: AnimationConfig | string
    ): void;
    /**
     * [descr:TransitionExecutor.leave(elements, animation)]
     */
    leave(
      elements: DevExpress.core.UserDefinedElementsArray,
      animation: AnimationConfig | string
    ): void;
    /**
     * [descr:TransitionExecutor.reset()]
     */
    reset(): void;
    /**
     * [descr:TransitionExecutor.start()]
     */
    start(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:TransitionExecutor.stop()]
     */
    stop(): void;
  }
  /**
   * [descr:validationEngine]
   */
  export class validationEngine {
    /**
     * [descr:validationEngine.getGroupConfig()]
     */
    static getGroupConfig(): any;
    /**
     * [descr:validationEngine.getGroupConfig(group)]
     */
    static getGroupConfig(group: string | any): any;
    /**
     * [descr:validationEngine.registerModelForValidation(model)]
     */
    static registerModelForValidation(model: any): void;
    /**
     * [descr:validationEngine.resetGroup()]
     */
    static resetGroup(): void;
    /**
     * [descr:validationEngine.resetGroup(group)]
     */
    static resetGroup(group: string | any): void;
    /**
     * [descr:validationEngine.unregisterModelForValidation(model)]
     */
    static unregisterModelForValidation(model: any): void;
    /**
     * [descr:validationEngine.validateGroup()]
     */
    static validateGroup(): DevExpress.ui.dxValidationGroup.ValidationResult;
    /**
     * [descr:validationEngine.validateGroup(group)]
     */
    static validateGroup(
      group: string | any
    ): DevExpress.ui.dxValidationGroup.ValidationResult;
    /**
     * [descr:validationEngine.validateModel(model)]
     */
    static validateModel(model: any): any;
  }
}
declare module DevExpress.animation {
  /**
   * @deprecated Use the AnimationConfig type instead
   */
  export type animationConfig = AnimationConfig;
  /**
   * [descr:AnimationState]
   */
  export type AnimationState =
    | string
    | number
    | {
        /**
         * [descr:AnimationState.opacity]
         */
        opacity: number;
      }
    | {
        /**
         * [descr:AnimationState.scale]
         */
        scale: number;
      }
    | {
        /**
         * [descr:AnimationState.position]
         */
        position: PositionConfig;
      }
    | {
        /**
         * [descr:AnimationState.left]
         */
        left: number;
      }
    | {
        /**
         * [descr:AnimationState.top]
         */
        top: number;
      };
  /**
   * @deprecated Use the PositionConfig type instead
   */
  export interface positionConfig extends PositionConfig {}
}
declare module DevExpress.core {
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  interface Component<TProperties> {
    // eslint-disable-line @typescript-eslint/no-unused-vars
    _optionsByReference: Record<string, unknown>;
    _deprecatedOptions: Record<string, unknown>;
    _options: {
      silent(path: any, value: any): void;
    };
    _createActionByOption(
      optionName: string,
      config: Record<string, unknown>
    ): (...args: any[]) => any;
    _dispose(): void;
    _getDefaultOptions(): Record<string, unknown>;
    _initOptions(options: Record<string, unknown>): void;
    _init(): void;
    _initializeComponent(): void;
    _optionChanging(name: string, value: unknown, prevValue: unknown): void;
    _optionChanged(args: { name: string; value: unknown }): void;
    _setOptionsByReference(): void;
    _setDeprecatedOptions(): void;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  type ComponentFactory<TComponent> = {
    new (
      element: UserDefinedElement,
      options?: Record<string, unknown>
    ): TComponent;
    getInstance(element: UserDefinedElement): TComponent;
  };
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface Condition {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  interface Condition extends JQueryEventObject {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type DeepPartial<T> = {
    [P in keyof T]?: DeepPartial<T[P]>;
  };
  export type DefaultOptionsRule<T> = {
    device?: Device | Device[] | ((device: Device) => boolean);
    options: DeepPartial<T>;
  };
  /**
   * [descr:dxElement]
   * @deprecated [depNote:dxElement]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type dxElement = DxElement<HTMLElement>;
  /**
   * [descr:DxElement]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type DxElement<T extends Element = HTMLElement> = {} extends Condition
    ? T
    : ElementWrapper<T>;
  /**
   * [descr:dxSVGElement]
   * @deprecated [depNote:dxSVGElement]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type dxSVGElement = DxElement<SVGElement>;
  /**
   * [descr:dxTemplate]
   */
  export type dxTemplate = Template;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTemplateOptions {
    /**
     * [descr:dxTemplateOptions.name]
     */
    name?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ElementsArrayWrapper<T extends Element> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  interface ElementsArrayWrapper<T extends Element> extends JQuery<T> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ElementWrapper<T extends Element> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  interface ElementWrapper<T extends Element> extends JQuery<T> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  type EventName = 'orientationChanged';
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class FunctionTemplate {
    render(template: {
      container: unknown;
      model?: object;
      transclude?: boolean;
    }): DxElement;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PromiseType<T> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  interface PromiseType<T> extends JQueryPromise<T> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type Skip<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
  /**
   * [descr:template]
   */
  export type template = string | Function | UserDefinedElement;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class Template {
    constructor(options?: dxTemplateOptions);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class TemplateManager {
    anonymousTemplateName: string;
    addDefaultTemplates(templates: Record<string, unknown>): void;
  }
  /**
   * [descr:UserDefinedElement]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type UserDefinedElement<T extends Element = Element> =
    {} extends Condition ? T : ElementWrapper<T> | T;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type UserDefinedElementsArray = {} extends Condition
    ? Array<Element>
    : ElementsArrayWrapper<Element>;
}
declare module DevExpress.core.utils {
  /**
   * [descr:DxPromise]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type DxPromise<T = void> = {} extends PromiseType<T>
    ? Promise<T>
    : PromiseType<T>;
}
declare module DevExpress.data {
  /**
   * [descr:Utils.applyChanges(data, changes, options)]
   */
  export function applyChanges(
    data: Array<any>,
    changes: Array<any>,
    options?: { keyExpr?: string | Array<string>; immutable?: boolean }
  ): Array<any>;
  /**
   * [descr:ArrayStore]
   */
  export class ArrayStore<TItem = any, TKey = any> extends Store<TItem, TKey> {
    constructor(options?: DevExpress.data.ArrayStore.Options<TItem, TKey>);
    /**
     * [descr:ArrayStore.clear()]
     */
    clear(): void;
    /**
     * [descr:ArrayStore.createQuery()]
     */
    createQuery(): Query;
  }
  module ArrayStore {
    export type Options<TItem = any, TKey = any> = ArrayStoreOptions<
      TItem,
      TKey
    >;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ArrayStoreOptions<TItem = any, TKey = any>
    extends DevExpress.data.Store.Options<TItem, TKey> {
    /**
     * [descr:ArrayStoreOptions.data]
     */
    data?: Array<TItem>;
  }
  /**
   * [descr:Utils.base64_encode(input)]
   */
  export function base64_encode(input: string | Array<number>): string;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  type BaseGroupDescriptor<T> = {
    selector: KeySelector<T>;
  };
  /**
   * [descr:CustomStore]
   */
  export class CustomStore<TItem = any, TKey = any> extends Store<TItem, TKey> {
    constructor(options?: DevExpress.data.CustomStore.Options<TItem, TKey>);
    /**
     * [descr:CustomStore.clearRawDataCache()]
     */
    clearRawDataCache(): void;
  }
  module CustomStore {
    export type Options<TItem = any, TKey = any> = CustomStoreOptions<
      TItem,
      TKey
    >;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CustomStoreOptions<TItem = any, TKey = any>
    extends DevExpress.data.Store.Options<TItem, TKey> {
    /**
     * [descr:CustomStoreOptions.byKey]
     */
    byKey?: (key: TKey) => PromiseLike<TItem>;
    /**
     * [descr:CustomStoreOptions.cacheRawData]
     */
    cacheRawData?: boolean;
    /**
     * [descr:CustomStoreOptions.insert]
     */
    insert?: (values: TItem) => PromiseLike<TItem>;
    /**
     * [descr:CustomStoreOptions.load]
     */
    load: (options: LoadOptions<TItem>) => PromiseLike<TItem> | Array<TItem>;
    /**
     * [descr:CustomStoreOptions.loadMode]
     */
    loadMode?: 'processed' | 'raw';
    /**
     * [descr:CustomStoreOptions.remove]
     */
    remove?: (key: TKey) => PromiseLike<void>;
    /**
     * [descr:CustomStoreOptions.totalCount]
     */
    totalCount?: (loadOptions: {
      filter?: FilterDescriptor | Array<FilterDescriptor>;
      group?: GroupDescriptor<TItem> | Array<GroupDescriptor<TItem>>;
    }) => PromiseLike<number>;
    /**
     * [descr:CustomStoreOptions.update]
     */
    update?: (key: TKey, values: TItem) => PromiseLike<any>;
    /**
     * [descr:CustomStoreOptions.useDefaultSearch]
     */
    useDefaultSearch?: boolean;
  }
  /**
   * [descr:DataSource]
   */
  export class DataSource<TItem = any, TKey = any> {
    constructor(data: Array<TItem>);
    constructor(
      options:
        | DevExpress.data.CustomStore.Options<TItem, TKey>
        | DevExpress.data.DataSource.Options<any, any, TItem, TKey>
    );
    constructor(store: Store<TItem, TKey>);
    constructor(url: string);
    /**
     * [descr:DataSource.cancel(operationId)]
     */
    cancel(operationId: number): boolean;
    /**
     * [descr:DataSource.dispose()]
     */
    dispose(): void;
    /**
     * [descr:DataSource.filter()]
     */
    filter(): FilterDescriptor | Array<FilterDescriptor>;
    /**
     * [descr:DataSource.filter(filterExpr)]
     */
    filter(filterExpr: FilterDescriptor | Array<FilterDescriptor>): void;
    /**
     * [descr:DataSource.group()]
     */
    group(): GroupDescriptor<TItem> | Array<GroupDescriptor<TItem>>;
    /**
     * [descr:DataSource.group(groupExpr)]
     */
    group(
      groupExpr: GroupDescriptor<TItem> | Array<GroupDescriptor<TItem>>
    ): void;
    /**
     * [descr:DataSource.isLastPage()]
     */
    isLastPage(): boolean;
    /**
     * [descr:DataSource.isLoaded()]
     */
    isLoaded(): boolean;
    /**
     * [descr:DataSource.isLoading()]
     */
    isLoading(): boolean;
    /**
     * [descr:DataSource.items()]
     */
    items(): Array<any>;
    /**
     * [descr:DataSource.key()]
     */
    key(): string | Array<string>;
    /**
     * [descr:DataSource.load()]
     */
    load(): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:DataSource.loadOptions()]
     */
    loadOptions(): LoadOptions<TItem>;
    /**
     * [descr:DataSource.off(eventName)]
     */
    off(eventName: DevExpress.data.DataSource.EventName): this;
    /**
     * [descr:DataSource.off(eventName, eventHandler)]
     */
    off(
      eventName: DevExpress.data.DataSource.EventName,
      eventHandler: Function
    ): this;
    /**
     * [descr:DataSource.on(eventName, eventHandler)]
     */
    on(
      eventName: DevExpress.data.DataSource.EventName,
      eventHandler: Function
    ): this;
    /**
     * [descr:DataSource.on(events)]
     */
    on(
      events: { [key in DevExpress.data.DataSource.EventName]?: Function }
    ): this;
    /**
     * [descr:DataSource.pageIndex()]
     */
    pageIndex(): number;
    /**
     * [descr:DataSource.pageIndex(newIndex)]
     */
    pageIndex(newIndex: number): void;
    /**
     * [descr:DataSource.pageSize()]
     */
    pageSize(): number;
    /**
     * [descr:DataSource.pageSize(value)]
     */
    pageSize(value: number): void;
    /**
     * [descr:DataSource.paginate()]
     */
    paginate(): boolean;
    /**
     * [descr:DataSource.paginate(value)]
     */
    paginate(value: boolean): void;
    /**
     * [descr:DataSource.reload()]
     */
    reload(): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:DataSource.requireTotalCount()]
     */
    requireTotalCount(): boolean;
    /**
     * [descr:DataSource.requireTotalCount(value)]
     */
    requireTotalCount(value: boolean): void;
    /**
     * [descr:DataSource.searchExpr()]
     */
    searchExpr(): string & Function & Array<string | Function>;
    /**
     * [descr:DataSource.searchExpr(expr)]
     */
    searchExpr(expr: string | Function | Array<string | Function>): void;
    /**
     * [descr:DataSource.searchOperation()]
     */
    searchOperation(): string;
    /**
     * [descr:DataSource.searchOperation(op)]
     */
    searchOperation(op: string): void;
    /**
     * [descr:DataSource.searchValue()]
     */
    searchValue(): any;
    /**
     * [descr:DataSource.searchValue(value)]
     */
    searchValue(value: any): void;
    /**
     * [descr:DataSource.select()]
     */
    select(): SelectDescriptor<TItem>;
    /**
     * [descr:DataSource.select(expr)]
     */
    select(expr: SelectDescriptor<TItem>): void;
    /**
     * [descr:DataSource.sort()]
     */
    sort(): SortDescriptor<TItem> | Array<SortDescriptor<TItem>>;
    /**
     * [descr:DataSource.sort(sortExpr)]
     */
    sort(sortExpr: SortDescriptor<TItem> | Array<SortDescriptor<TItem>>): void;
    /**
     * [descr:DataSource.store()]
     */
    store(): Store<TItem, TKey>;
    /**
     * [descr:DataSource.totalCount()]
     */
    totalCount(): number;
  }
  module DataSource {
    /**
     * [descr:DataSourceLike]
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type DataSourceLike<TItem, TKey = any> =
      | string
      | Array<TItem>
      | Store<TItem, TKey>
      | DataSourceOptionsStub<any, any, TItem>
      | DataSource<TItem, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface DataSourceOptionsStub<
      TStoreItem = any,
      TMappedItem = TStoreItem,
      TItem = TMappedItem
    > {
      customQueryParams?: any;
      expand?: Array<string> | string;
      filter?: FilterDescriptor | Array<FilterDescriptor>;
      group?: GroupDescriptor<TItem> | Array<GroupDescriptor<TItem>>;
      map?: (dataItem: TStoreItem) => TMappedItem;
      onChanged?: (e: { readonly changes?: Array<TMappedItem> }) => void;
      onLoadError?: (error: { readonly message?: string }) => void;
      onLoadingChanged?: (isLoading: boolean) => void;
      pageSize?: number;
      paginate?: boolean;
      postProcess?: (data: Array<TMappedItem>) => Array<TItem>;
      pushAggregationTimeout?: number;
      requireTotalCount?: boolean;
      reshapeOnPush?: boolean;
      searchExpr?: string | Function | Array<string | Function>;
      searchOperation?: SearchOperation;
      searchValue?: any;
      select?: SelectDescriptor<TItem>;
      sort?: SortDescriptor<TItem> | Array<SortDescriptor<TItem>>;
      store?:
        | Array<TStoreItem>
        | Store<TStoreItem, any>
        | (DevExpress.data.ArrayStore.Options<TStoreItem, any> & {
            type: 'array';
          })
        | (DevExpress.data.LocalStore.Options<TStoreItem, any> & {
            type: 'local';
          })
        | (DevExpress.data.ODataStore.Options<TStoreItem, any> & {
            type: 'odata';
          })
        | DevExpress.data.CustomStore.Options<TStoreItem, any>;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type EventName = 'changed' | 'loadError' | 'loadingChanged';
    export type Options<
      TStoreItem = any,
      TMappedItem = TStoreItem,
      TItem = TMappedItem,
      TKey = any
    > = DataSourceOptions<TStoreItem, TItem, TMappedItem, TKey>;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface DataSourceOptions<
    TStoreItem = any,
    TMappedItem = TStoreItem,
    TItem = TMappedItem,
    TKey = any
  > {
    /**
     * [descr:DataSourceOptions.customQueryParams]
     */
    customQueryParams?: any;
    /**
     * [descr:DataSourceOptions.expand]
     */
    expand?: Array<string> | string;
    /**
     * [descr:DataSourceOptions.filter]
     */
    filter?: FilterDescriptor | Array<FilterDescriptor>;
    /**
     * [descr:DataSourceOptions.group]
     */
    group?: GroupDescriptor<TItem> | Array<GroupDescriptor<TItem>>;
    /**
     * [descr:DataSourceOptions.map]
     */
    map?: (dataItem: TStoreItem) => TMappedItem;
    /**
     * [descr:DataSourceOptions.onChanged]
     */
    onChanged?: (e: { readonly changes?: Array<TMappedItem> }) => void;
    /**
     * [descr:DataSourceOptions.onLoadError]
     */
    onLoadError?: (error: { readonly message?: string }) => void;
    /**
     * [descr:DataSourceOptions.onLoadingChanged]
     */
    onLoadingChanged?: (isLoading: boolean) => void;
    /**
     * [descr:DataSourceOptions.pageSize]
     */
    pageSize?: number;
    /**
     * [descr:DataSourceOptions.paginate]
     */
    paginate?: boolean;
    /**
     * [descr:DataSourceOptions.postProcess]
     */
    postProcess?: (data: Array<TMappedItem>) => Array<TItem>;
    /**
     * [descr:DataSourceOptions.pushAggregationTimeout]
     */
    pushAggregationTimeout?: number;
    /**
     * [descr:DataSourceOptions.requireTotalCount]
     */
    requireTotalCount?: boolean;
    /**
     * [descr:DataSourceOptions.reshapeOnPush]
     */
    reshapeOnPush?: boolean;
    /**
     * [descr:DataSourceOptions.searchExpr]
     */
    searchExpr?: string | Function | Array<string | Function>;
    /**
     * [descr:DataSourceOptions.searchOperation]
     */
    searchOperation?: SearchOperation;
    /**
     * [descr:DataSourceOptions.searchValue]
     */
    searchValue?: any;
    /**
     * [descr:DataSourceOptions.select]
     */
    select?: SelectDescriptor<TItem>;
    /**
     * [descr:DataSourceOptions.sort]
     */
    sort?: SortDescriptor<TItem> | Array<SortDescriptor<TItem>>;
    /**
     * [descr:DataSourceOptions.store]
     */
    store?:
      | Array<TStoreItem>
      | Store<TStoreItem, TKey>
      | (DevExpress.data.ArrayStore.Options<TStoreItem, TKey> & {
          type: 'array';
        })
      | (DevExpress.data.LocalStore.Options<TStoreItem, TKey> & {
          type: 'local';
        })
      | (DevExpress.data.ODataStore.Options<TStoreItem, TKey> & {
          type: 'odata';
        })
      | DevExpress.data.CustomStore.Options<TStoreItem, TKey>;
  }
  /**
   * [descr:EdmLiteral]
   */
  export class EdmLiteral {
    constructor(value: string);
    /**
     * [descr:EdmLiteral.valueOf()]
     */
    valueOf(): string;
  }
  /**
   * [descr:Utils.errorHandler]
   * @deprecated [depNote:Utils.errorHandler]
   */
  export function errorHandler(e: Error): void;
  /**
   * [descr:FilterDescriptor]
   */
  export type FilterDescriptor = any;
  /**
   * [descr:GroupDescriptor]
   */
  export type GroupDescriptor<T> =
    | KeySelector<T>
    | (BaseGroupDescriptor<T> & {
        desc?: boolean;
      });
  /**
   * [descr:Guid]
   */
  export class Guid {
    constructor();
    constructor(value: string);
    /**
     * [descr:Guid.toString()]
     */
    toString(): string;
    /**
     * [descr:Guid.valueOf()]
     */
    valueOf(): string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  type KeySelector<T> =
    | string
    | ((source: T) => string | number | Date | Object);
  /**
   * [descr:LoadOptions]
   */
  export interface LoadOptions<T = any> {
    /**
     * [descr:LoadOptions.customQueryParams]
     */
    customQueryParams?: any;
    /**
     * [descr:LoadOptions.expand]
     */
    expand?: Array<string>;
    /**
     * [descr:LoadOptions.filter]
     */
    filter?: FilterDescriptor | Array<FilterDescriptor>;
    /**
     * [descr:LoadOptions.group]
     */
    group?: GroupDescriptor<T> | Array<GroupDescriptor<T>>;
    /**
     * [descr:LoadOptions.groupSummary]
     */
    groupSummary?: SummaryDescriptor<T> | Array<SummaryDescriptor<T>>;
    /**
     * [descr:LoadOptions.parentIds]
     */
    parentIds?: Array<any>;
    /**
     * [descr:LoadOptions.requireGroupCount]
     */
    requireGroupCount?: boolean;
    /**
     * [descr:LoadOptions.requireTotalCount]
     */
    requireTotalCount?: boolean;
    /**
     * [descr:LoadOptions.searchExpr]
     */
    searchExpr?: string | Function | Array<string | Function>;
    /**
     * [descr:LoadOptions.searchOperation]
     */
    searchOperation?: SearchOperation;
    /**
     * [descr:LoadOptions.searchValue]
     */
    searchValue?: any;
    /**
     * [descr:LoadOptions.select]
     */
    select?: SelectDescriptor<T>;
    /**
     * [descr:LoadOptions.skip]
     */
    skip?: number;
    /**
     * [descr:LoadOptions.sort]
     */
    sort?: SortDescriptor<T> | Array<SortDescriptor<T>>;
    /**
     * [descr:LoadOptions.take]
     */
    take?: number;
    /**
     * [descr:LoadOptions.totalSummary]
     */
    totalSummary?: SummaryDescriptor<T> | Array<SummaryDescriptor<T>>;
    /**
     * [descr:LoadOptions.userData]
     */
    userData?: any;
  }
  /**
   * [descr:LocalStore]
   */
  export class LocalStore<TItem = any, TKey = any> extends ArrayStore<
    TItem,
    TKey
  > {
    constructor(options?: DevExpress.data.LocalStore.Options<TItem, TKey>);
    /**
     * [descr:LocalStore.clear()]
     */
    clear(): void;
  }
  module LocalStore {
    export type Options<TItem = any, TKey = any> = LocalStoreOptions<
      TItem,
      TKey
    >;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface LocalStoreOptions<TItem = any, TKey = any>
    extends ArrayStoreOptions<TItem, TKey> {
    /**
     * [descr:LocalStoreOptions.flushInterval]
     */
    flushInterval?: number;
    /**
     * [descr:LocalStoreOptions.immediate]
     */
    immediate?: boolean;
    /**
     * [descr:LocalStoreOptions.name]
     */
    name?: string;
  }
  /**
   * [descr:ODataContext]
   */
  export class ODataContext {
    constructor(options?: ODataContextOptions);
    /**
     * [descr:ODataContext.get(operationName, params)]
     */
    get(
      operationName: string,
      params: any
    ): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:ODataContext.invoke(operationName, params, httpMethod)]
     */
    invoke(
      operationName: string,
      params: any,
      httpMethod: DevExpress.data.ODataContext.HttpMethod
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:ODataContext.objectLink(entityAlias, key)]
     */
    objectLink(entityAlias: string, key: any | string | number): any;
  }
  module ODataContext {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type HttpMethod = 'GET' | 'POST' | 'PATCH' | 'MERGE';
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ODataRequestOptions {
      accepts: any;
      async: boolean;
      contentType: string | boolean;
      data: any;
      dataType: string;
      headers: any;
      jsonp?: boolean;
      method: string;
      timeout: number;
      url: string;
      xhrFields: any;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ODataContextOptions {
    /**
     * [descr:ODataContextOptions.beforeSend]
     */
    beforeSend?: (options: {
      url: string;
      async: boolean;
      method: string;
      timeout: number;
      params: any;
      payload: any;
      headers: any;
    }) => void;
    /**
     * [descr:ODataContextOptions.deserializeDates]
     */
    deserializeDates?: boolean;
    /**
     * [descr:ODataContextOptions.entities]
     */
    entities?: any;
    /**
     * [descr:ODataContextOptions.errorHandler]
     */
    errorHandler?: (e: {
      httpStatus: number;
      errorDetails: any;
      requestOptions: DevExpress.data.ODataContext.ODataRequestOptions;
    }) => void;
    /**
     * [descr:ODataContextOptions.filterToLower]
     */
    filterToLower?: boolean;
    /**
     * [descr:ODataContextOptions.jsonp]
     */
    jsonp?: boolean;
    /**
     * [descr:ODataContextOptions.url]
     */
    url?: string;
    /**
     * [descr:ODataContextOptions.version]
     */
    version?: number;
    /**
     * [descr:ODataContextOptions.withCredentials]
     */
    withCredentials?: boolean;
  }
  /**
   * [descr:ODataStore]
   */
  export class ODataStore<TItem = any, TKey = any> extends Store<TItem, TKey> {
    constructor(options?: DevExpress.data.ODataStore.Options<TItem, TKey>);
    byKey(key: TKey): DevExpress.core.utils.DxPromise<TItem>;
    /**
     * [descr:ODataStore.byKey(key, extraOptions)]
     */
    byKey(
      key: TKey,
      extraOptions: {
        expand?: string | Array<string>;
        select?: string | Array<string>;
      }
    ): DevExpress.core.utils.DxPromise<TItem>;
    /**
     * [descr:ODataStore.createQuery(loadOptions)]
     */
    createQuery(loadOptions?: {
      expand?: string | Array<string>;
      requireTotalCount?: boolean;
      customQueryParams?: any;
    }): Query;

    /**
     * [descr:ODataStore.insert(values)]
     */
    insert(
      values: TItem
    ): DevExpress.core.utils.DxPromise<TItem> &
      DevExpress.data.ODataStore.PromiseExtension<TItem>;
  }
  module ODataStore {
    export type Options<TItem = any, TKey = any> = ODataStoreOptions<
      TItem,
      TKey
    >;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface PromiseExtension<T> {
      then<TResult1 = T, TResult2 = never>(
        onFulfilled?:
          | ((
              value: T,
              extraParameters?: T
            ) => TResult1 | PromiseLike<TResult1>)
          | undefined
          | null,
        onRejected?:
          | ((reason: any) => TResult2 | PromiseLike<TResult2>)
          | undefined
          | null
      ): Promise<TResult1 | TResult2>;
    }
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ODataStoreOptions<TItem = any, TKey = any>
    extends DevExpress.data.Store.Options<TItem, TKey> {
    /**
     * [descr:ODataStoreOptions.beforeSend]
     */
    beforeSend?: (options: {
      url: string;
      async: boolean;
      method: string;
      timeout: number;
      params: any;
      payload: any;
      headers: any;
    }) => void;
    /**
     * [descr:ODataStoreOptions.deserializeDates]
     */
    deserializeDates?: boolean;
    /**
     * [descr:ODataStoreOptions.errorHandler]
     */
    errorHandler?: (e: {
      httpStatus: number;
      errorDetails: any;
      requestOptions: DevExpress.data.ODataContext.ODataRequestOptions;
    }) => void;
    /**
     * [descr:ODataStoreOptions.fieldTypes]
     */
    fieldTypes?: any;
    /**
     * [descr:ODataStoreOptions.filterToLower]
     */
    filterToLower?: boolean;
    /**
     * [descr:ODataStoreOptions.jsonp]
     */
    jsonp?: boolean;
    /**
     * [descr:ODataStoreOptions.keyType]
     */
    keyType?:
      | 'String'
      | 'Int32'
      | 'Int64'
      | 'Guid'
      | 'Boolean'
      | 'Single'
      | 'Decimal'
      | any;
    /**
     * [descr:ODataStoreOptions.onLoading]
     */
    onLoading?: (loadOptions: LoadOptions<TItem>) => void;
    /**
     * [descr:ODataStoreOptions.url]
     */
    url?: string;
    /**
     * [descr:ODataStoreOptions.version]
     */
    version?: number;
    /**
     * [descr:ODataStoreOptions.withCredentials]
     */
    withCredentials?: boolean;
  }
  /**
   * [descr:PivotGridDataSource]
   */
  export class PivotGridDataSource {
    constructor(options?: DevExpress.data.PivotGridDataSource.Options);
    /**
     * [descr:PivotGridDataSource.collapseAll(id)]
     */
    collapseAll(id: number | string): void;
    /**
     * [descr:PivotGridDataSource.collapseHeaderItem(area, path)]
     */
    collapseHeaderItem(area: string, path: Array<string | number | Date>): void;
    /**
     * [descr:PivotGridDataSource.createDrillDownDataSource(options)]
     */
    createDrillDownDataSource(options: {
      columnPath?: Array<string | number | Date>;
      rowPath?: Array<string | number | Date>;
      dataIndex?: number;
      maxRowCount?: number;
      customColumns?: Array<string>;
    }): DataSource;
    /**
     * [descr:PivotGridDataSource.dispose()]
     */
    dispose(): void;
    /**
     * [descr:PivotGridDataSource.expandAll(id)]
     */
    expandAll(id: number | string): void;
    /**
     * [descr:PivotGridDataSource.expandHeaderItem(area, path)]
     */
    expandHeaderItem(area: string, path: Array<any>): void;
    /**
     * [descr:PivotGridDataSource.field(id)]
     */
    field(id: number | string): any;
    /**
     * [descr:PivotGridDataSource.field(id, options)]
     */
    field(id: number | string, options: any): void;
    /**
     * [descr:PivotGridDataSource.fields()]
     */
    fields(): Array<DevExpress.data.PivotGridDataSource.Field>;
    /**
     * [descr:PivotGridDataSource.fields(fields)]
     */
    fields(fields: Array<DevExpress.data.PivotGridDataSource.Field>): void;
    /**
     * [descr:PivotGridDataSource.filter()]
     */
    filter(): any;
    /**
     * [descr:PivotGridDataSource.filter(filterExpr)]
     */
    filter(filterExpr: any): void;
    /**
     * [descr:PivotGridDataSource.getAreaFields(area, collectGroups)]
     */
    getAreaFields(
      area: string,
      collectGroups: boolean
    ): Array<DevExpress.data.PivotGridDataSource.Field>;
    /**
     * [descr:PivotGridDataSource.getData()]
     */
    getData(): any;
    /**
     * [descr:PivotGridDataSource.isLoading()]
     */
    isLoading(): boolean;
    /**
     * [descr:PivotGridDataSource.load()]
     */
    load(): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:PivotGridDataSource.off(eventName)]
     */
    off(eventName: DevExpress.data.PivotGridDataSource.EventName): this;
    /**
     * [descr:PivotGridDataSource.off(eventName, eventHandler)]
     */
    off(
      eventName: DevExpress.data.PivotGridDataSource.EventName,
      eventHandler: Function
    ): this;
    /**
     * [descr:PivotGridDataSource.on(eventName, eventHandler)]
     */
    on(
      eventName: DevExpress.data.PivotGridDataSource.EventName,
      eventHandler: Function
    ): this;
    /**
     * [descr:PivotGridDataSource.on(events)]
     */
    on(
      events: {
        [key in DevExpress.data.PivotGridDataSource.EventName]?: Function;
      }
    ): this;
    /**
     * [descr:PivotGridDataSource.reload()]
     */
    reload(): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:PivotGridDataSource.state()]
     */
    state(): any;
    /**
     * [descr:PivotGridDataSource.state(state)]
     */
    state(state: any): void;
  }
  module PivotGridDataSource {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type EventName =
      | 'changed'
      | 'fieldsPrepared'
      | 'loadError'
      | 'loadingChanged';
    export type Options = PivotGridDataSourceOptions;
  }
  /**
   * @deprecated Use Field instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PivotGridDataSourceField {
    /**
     * [descr:PivotGridDataSourceOptions.fields.allowCrossGroupCalculation]
     */
    allowCrossGroupCalculation?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.allowExpandAll]
     */
    allowExpandAll?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.allowFiltering]
     */
    allowFiltering?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.allowSorting]
     */
    allowSorting?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.allowSortingBySummary]
     */
    allowSortingBySummary?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.area]
     */
    area?: 'column' | 'data' | 'filter' | 'row' | undefined;
    /**
     * [descr:PivotGridDataSourceOptions.fields.areaIndex]
     */
    areaIndex?: number;
    /**
     * [descr:PivotGridDataSourceOptions.fields.calculateCustomSummary]
     */
    calculateCustomSummary?: (options: {
      summaryProcess?: string;
      value?: any;
      totalValue?: any;
    }) => void;
    /**
     * [descr:PivotGridDataSourceOptions.fields.calculateSummaryValue]
     */
    calculateSummaryValue?: (e: DevExpress.ui.dxPivotGridSummaryCell) => number;
    /**
     * [descr:PivotGridDataSourceOptions.fields.caption]
     */
    caption?: string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.customizeText]
     */
    customizeText?: (cellInfo: {
      value?: string | number | Date;
      valueText?: string;
    }) => string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.dataField]
     */
    dataField?: string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.dataType]
     */
    dataType?: 'date' | 'number' | 'string';
    /**
     * [descr:PivotGridDataSourceOptions.fields.displayFolder]
     */
    displayFolder?: string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.expanded]
     */
    expanded?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.filterType]
     */
    filterType?: 'exclude' | 'include';
    /**
     * [descr:PivotGridDataSourceOptions.fields.filterValues]
     */
    filterValues?: Array<any>;
    /**
     * [descr:PivotGridDataSourceOptions.fields.format]
     */
    format?: DevExpress.ui.Format;
    /**
     * [descr:PivotGridDataSourceOptions.fields.groupIndex]
     */
    groupIndex?: number;
    /**
     * [descr:PivotGridDataSourceOptions.fields.groupInterval]
     */
    groupInterval?: 'day' | 'dayOfWeek' | 'month' | 'quarter' | 'year' | number;
    /**
     * [descr:PivotGridDataSourceOptions.fields.groupName]
     */
    groupName?: string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.headerFilter]
     */
    headerFilter?: { allowSearch?: boolean; height?: number; width?: number };
    /**
     * [descr:PivotGridDataSourceOptions.fields.isMeasure]
     */
    isMeasure?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.name]
     */
    name?: string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.runningTotal]
     */
    runningTotal?: 'column' | 'row';
    /**
     * [descr:PivotGridDataSourceOptions.fields.selector]
     */
    selector?: Function;
    /**
     * [descr:PivotGridDataSourceOptions.fields.showGrandTotals]
     */
    showGrandTotals?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.showTotals]
     */
    showTotals?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.showValues]
     */
    showValues?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.sortBy]
     */
    sortBy?: 'displayText' | 'value' | 'none';
    /**
     * [descr:PivotGridDataSourceOptions.fields.sortBySummaryField]
     */
    sortBySummaryField?: string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.sortBySummaryPath]
     */
    sortBySummaryPath?: Array<number | string>;
    /**
     * [descr:PivotGridDataSourceOptions.fields.sortOrder]
     */
    sortOrder?: 'asc' | 'desc';
    /**
     * [descr:PivotGridDataSourceOptions.fields.sortingMethod]
     */
    sortingMethod?: (
      a: { value?: string | number; children?: Array<any> },
      b: { value?: string | number; children?: Array<any> }
    ) => number;
    /**
     * [descr:PivotGridDataSourceOptions.fields.summaryDisplayMode]
     */
    summaryDisplayMode?:
      | 'absoluteVariation'
      | 'percentOfColumnGrandTotal'
      | 'percentOfColumnTotal'
      | 'percentOfGrandTotal'
      | 'percentOfRowGrandTotal'
      | 'percentOfRowTotal'
      | 'percentVariation';
    /**
     * [descr:PivotGridDataSourceOptions.fields.summaryType]
     */
    summaryType?: 'avg' | 'count' | 'custom' | 'max' | 'min' | 'sum' | string;
    /**
     * [descr:PivotGridDataSourceOptions.fields.visible]
     */
    visible?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.fields.width]
     */
    width?: number;
    /**
     * [descr:PivotGridDataSourceOptions.fields.wordWrapEnabled]
     */
    wordWrapEnabled?: boolean;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PivotGridDataSourceOptions {
    /**
     * [descr:PivotGridDataSourceOptions.fields]
     */
    fields?: Array<DevExpress.data.PivotGridDataSource.Field>;
    /**
     * [descr:PivotGridDataSourceOptions.filter]
     */
    filter?: string | Array<any> | Function;
    /**
     * [descr:PivotGridDataSourceOptions.onChanged]
     */
    onChanged?: Function;
    /**
     * [descr:PivotGridDataSourceOptions.onFieldsPrepared]
     */
    onFieldsPrepared?: (
      fields: Array<DevExpress.data.PivotGridDataSource.Field>
    ) => void;
    /**
     * [descr:PivotGridDataSourceOptions.onLoadError]
     */
    onLoadError?: (error: any) => void;
    /**
     * [descr:PivotGridDataSourceOptions.onLoadingChanged]
     */
    onLoadingChanged?: (isLoading: boolean) => void;
    /**
     * [descr:PivotGridDataSourceOptions.paginate]
     */
    paginate?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.remoteOperations]
     */
    remoteOperations?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.retrieveFields]
     */
    retrieveFields?: boolean;
    /**
     * [descr:PivotGridDataSourceOptions.store]
     */
    store?:
      | Store
      | DevExpress.data.Store.Options
      | XmlaStore
      | XmlaStoreOptions
      | Array<{
          /**
           * [descr:PivotGridDataSourceOptions.store.type]
           */
          type?: 'array' | 'local' | 'odata' | 'xmla';
        }>
      | {
          /**
           * [descr:PivotGridDataSourceOptions.store.type]
           */
          type?: 'array' | 'local' | 'odata' | 'xmla';
        };
  }
  /**
   * [descr:Utils.query(array)]
   */
  export function query(array: Array<any>): Query;
  /**
   * [descr:Utils.query(url, queryOptions)]
   */
  export function query(url: string, queryOptions: any): Query;
  /**
   * [descr:Query]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface Query {
    /**
     * [descr:Query.aggregate(seed, step, finalize)]
     */
    aggregate(
      seed: any,
      step: Function,
      finalize: Function
    ): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:Query.aggregate(step)]
     */
    aggregate(step: Function): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:Query.avg()]
     */
    avg(): DevExpress.core.utils.DxPromise<number>;
    /**
     * [descr:Query.avg(getter)]
     */
    avg(getter: any): DevExpress.core.utils.DxPromise<number>;
    /**
     * [descr:Query.count()]
     */
    count(): DevExpress.core.utils.DxPromise<number>;
    /**
     * [descr:Query.enumerate()]
     */
    enumerate(): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:Query.filter(criteria)]
     */
    filter(criteria: Array<any>): Query;
    /**
     * [descr:Query.filter(predicate)]
     */
    filter(predicate: Function): Query;
    /**
     * [descr:Query.groupBy(getter)]
     */
    groupBy(getter: any): Query;
    /**
     * [descr:Query.max()]
     */
    max(): DevExpress.core.utils.DxPromise<number | Date>;
    /**
     * [descr:Query.max(getter)]
     */
    max(getter: any): DevExpress.core.utils.DxPromise<number | Date>;
    /**
     * [descr:Query.min()]
     */
    min(): DevExpress.core.utils.DxPromise<number | Date>;
    /**
     * [descr:Query.min(getter)]
     */
    min(getter: any): DevExpress.core.utils.DxPromise<number | Date>;
    /**
     * [descr:Query.select(getter)]
     */
    select(getter: any): Query;
    /**
     * [descr:Query.slice(skip, take)]
     */
    slice(skip: number, take?: number): Query;
    /**
     * [descr:Query.sortBy(getter)]
     */
    sortBy(getter: any): Query;
    /**
     * [descr:Query.sortBy(getter, desc)]
     */
    sortBy(getter: any, desc: boolean): Query;
    /**
     * [descr:Query.sum()]
     */
    sum(): DevExpress.core.utils.DxPromise<number>;
    /**
     * [descr:Query.sum(getter)]
     */
    sum(getter: any): DevExpress.core.utils.DxPromise<number>;
    /**
     * [descr:Query.thenBy(getter)]
     */
    thenBy(getter: any): Query;
    /**
     * [descr:Query.thenBy(getter, desc)]
     */
    thenBy(getter: any, desc: boolean): Query;
    /**
     * [descr:Query.toArray()]
     */
    toArray(): Array<any>;
  }
  /**
   * [descr:SearchOperation]
   */
  export type SearchOperation =
    | '='
    | '<>'
    | '>'
    | '>='
    | '<'
    | '<='
    | 'startswith'
    | 'endswith'
    | 'contains'
    | 'notcontains';
  /**
   * [descr:SelectDescriptor]
   */
  export type SelectDescriptor<T> =
    | string
    | Array<string>
    | ((source: T) => any);
  /**
   * [descr:Utils.setErrorHandler]
   */
  export function setErrorHandler(handler: (e: Error) => void): void;
  /**
   * [descr:SortDescriptor]
   */
  export type SortDescriptor<T> = GroupDescriptor<T>;
  /**
   * [descr:Store]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class Store<TItem = any, TKey = any> {
    constructor(options?: DevExpress.data.Store.Options<TItem, TKey>);
    /**
     * [descr:Store.byKey(key)]
     */
    byKey(
      key: TKey,
      extraOptions?: LoadOptions<TItem>
    ): DevExpress.core.utils.DxPromise<TItem>;
    /**
     * [descr:Store.insert(values)]
     */
    insert(values: TItem): DevExpress.core.utils.DxPromise<TItem>;
    /**
     * [descr:Store.key()]
     */
    key(): string | Array<string>;
    /**
     * [descr:Store.keyOf(obj)]
     */
    keyOf(obj: TItem): TKey;
    /**
     * [descr:Store.load()]
     */
    load(): DevExpress.core.utils.DxPromise<Array<TItem>>;
    /**
     * [descr:Store.load(options)]
     */
    load(
      options: LoadOptions<TItem>
    ): DevExpress.core.utils.DxPromise<Array<TItem>>;
    /**
     * [descr:Store.off(eventName)]
     */
    off(eventName: DevExpress.data.Store.EventName): this;
    /**
     * [descr:Store.off(eventName, eventHandler)]
     */
    off(
      eventName: DevExpress.data.Store.EventName,
      eventHandler: Function
    ): this;
    /**
     * [descr:Store.on(eventName, eventHandler)]
     */
    on(
      eventName: DevExpress.data.Store.EventName,
      eventHandler: Function
    ): this;
    /**
     * [descr:Store.on(events)]
     */
    on(events: { [key in DevExpress.data.Store.EventName]?: Function }): this;
    /**
     * [descr:Store.push(changes)]
     */
    push(
      changes: Array<{
        type: 'insert' | 'update' | 'remove';
        data?: DevExpress.core.DeepPartial<TItem>;
        key?: TKey;
        index?: number;
      }>
    ): void;
    /**
     * [descr:Store.remove(key)]
     */
    remove(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:Store.totalCount(options)]
     */
    totalCount(obj: {
      filter?: FilterDescriptor | Array<FilterDescriptor>;
      group?: GroupDescriptor<TItem> | Array<GroupDescriptor<TItem>>;
    }): DevExpress.core.utils.DxPromise<number>;
    /**
     * [descr:Store.update(key, values)]
     */
    update(
      key: TKey,
      values: DevExpress.core.DeepPartial<TItem>
    ): DevExpress.core.utils.DxPromise<TItem>;
  }
  module Store {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type EventName =
      | 'loaded'
      | 'loading'
      | 'inserted'
      | 'inserting'
      | 'updated'
      | 'updating'
      | 'push'
      | 'removed'
      | 'removing'
      | 'modified'
      | 'modifying';
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type Options<TItem = any, TKey = any> = StoreOptions<TItem, TKey>;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface StoreOptions<TItem = any, TKey = any> {
    /**
     * [descr:StoreOptions.errorHandler]
     */
    errorHandler?: Function;
    /**
     * [descr:StoreOptions.key]
     */
    key?: string | Array<string>;
    /**
     * [descr:StoreOptions.onInserted]
     */
    onInserted?: (values: TItem, key: TKey) => void;
    /**
     * [descr:StoreOptions.onInserting]
     */
    onInserting?: (values: TItem) => void;
    /**
     * [descr:StoreOptions.onLoaded]
     */
    onLoaded?: (result: Array<TItem>, loadOptions: LoadOptions<TItem>) => void;
    /**
     * [descr:StoreOptions.onLoading]
     */
    onLoading?: (loadOptions: LoadOptions<TItem>) => void;
    /**
     * [descr:StoreOptions.onModified]
     */
    onModified?: Function;
    /**
     * [descr:StoreOptions.onModifying]
     */
    onModifying?: Function;
    /**
     * [descr:StoreOptions.onPush]
     */
    onPush?: (changes: Array<TItem>) => void;
    /**
     * [descr:StoreOptions.onRemoved]
     */
    onRemoved?: (key: TKey) => void;
    /**
     * [descr:StoreOptions.onRemoving]
     */
    onRemoving?: (key: TKey) => void;
    /**
     * [descr:StoreOptions.onUpdated]
     */
    onUpdated?: (key: TKey, values: TItem) => void;
    /**
     * [descr:StoreOptions.onUpdating]
     */
    onUpdating?: (key: TKey, values: TItem) => void;
  }
  /**
   * [descr:SummaryDescriptor]
   */
  export type SummaryDescriptor<T> =
    | KeySelector<T>
    | (BaseGroupDescriptor<T> & {
        summaryType?: 'sum' | 'avg' | 'min' | 'max' | 'count';
      });
  /**
   * [descr:XmlaStore]
   */
  export class XmlaStore {
    constructor(options?: XmlaStoreOptions);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface XmlaStoreOptions {
    /**
     * [descr:XmlaStoreOptions.beforeSend]
     */
    beforeSend?: (options: {
      url?: string;
      method?: string;
      headers?: any;
      xhrFields?: any;
      data?: string;
      dataType?: string;
    }) => void;
    /**
     * [descr:XmlaStoreOptions.catalog]
     */
    catalog?: string;
    /**
     * [descr:XmlaStoreOptions.cube]
     */
    cube?: string;
    /**
     * [descr:XmlaStoreOptions.url]
     */
    url?: string;
  }
}
declare module DevExpress.data.PivotGridDataSource {
  export type Field = PivotGridDataSourceField;
}
declare module DevExpress.data.utils {
  /**
   * [descr:Utils.compileGetter(expr)]
   */
  export function compileGetter(expr: string | Array<string>): Function;
  /**
   * [descr:Utils.compileSetter(expr)]
   */
  export function compileSetter(expr: string | Array<string>): Function;
}
declare module DevExpress.data.utils.odata {
  /**
   * [descr:Utils.keyConverters]
   */
  export var keyConverters: any;
}
declare module DevExpress.events {
  export interface Cancelable {
    cancel?: boolean;
  }
  export interface ChangedOptionInfo {
    readonly name: string;
    readonly fullName: string;
    readonly value?: any;
    readonly previousValue?: any;
  }
  /* eslint-enable @typescript-eslint/no-empty-interface */
  /**
   * [descr:DxEvent]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type DxEvent<TNativeEvent = Event> = {} extends EventType
    ? EventObject & TNativeEvent
    : EventType;
  /**
   * [descr:event]
   * @deprecated [depNote:event]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type event = DxEvent;
  export interface EventInfo<TComponent> {
    readonly component: TComponent;
    readonly element: DevExpress.core.DxElement;
    readonly model?: any;
  }
  /**
   * [descr:EventObject]
   */
  export class EventObject {
    /**
     * [descr:EventObject.currentTarget]
     */
    currentTarget: Element;

    /**
     * [descr:EventObject.data]
     */
    data: any;

    /**
     * [descr:EventObject.delegateTarget]
     */
    delegateTarget: Element;

    /**
     * [descr:EventObject.target]
     */
    target: Element;
    /**
     * [descr:EventObject.isDefaultPrevented()]
     */
    isDefaultPrevented(): boolean;
    /**
     * [descr:EventObject.isImmediatePropagationStopped()]
     */
    isImmediatePropagationStopped(): boolean;
    /**
     * [descr:EventObject.isPropagationStopped()]
     */
    isPropagationStopped(): boolean;
    /**
     * [descr:EventObject.preventDefault()]
     */
    preventDefault(): void;
    /**
     * [descr:EventObject.stopImmediatePropagation()]
     */
    stopImmediatePropagation(): void;
    /**
     * [descr:EventObject.stopPropagation()]
     */
    stopPropagation(): void;
  }
  /**
   * [descr:handler(event, extraParameters)]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export function eventsHandler(event: DxEvent, extraParameters: any): boolean;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface EventType {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  interface EventType extends JQueryEventObject {
    cancel?: boolean;
  }
  export interface InitializedEventInfo<TComponent> {
    readonly component?: TComponent;
    readonly element?: DevExpress.core.DxElement;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ItemInfo<TItemData = any> {
    readonly itemData?: TItemData;
    readonly itemElement: DevExpress.core.DxElement;
    readonly itemIndex: number;
  }
  export interface NativeEventInfo<TComponent, TNativeEvent = Event> {
    readonly component: TComponent;
    readonly element: DevExpress.core.DxElement;
    readonly model?: any;
    readonly event?: DxEvent<TNativeEvent>;
  }
  /**
   * [descr:events.off(element)]
   */
  export function off(element: Element | Array<Element>): void;
  /**
   * [descr:events.off(element, eventName)]
   */
  export function off(
    element: Element | Array<Element>,
    eventName: string
  ): void;
  /**
   * [descr:events.off(element, eventName, handler)]
   */
  export function off(
    element: Element | Array<Element>,
    eventName: string,
    handler: Function
  ): void;
  /**
   * [descr:events.off(element, eventName, selector)]
   */
  export function off(
    element: Element | Array<Element>,
    eventName: string,
    selector: string
  ): void;
  /**
   * [descr:events.off(element, eventName, selector, handler)]
   */
  export function off(
    element: Element | Array<Element>,
    eventName: string,
    selector: string,
    handler: Function
  ): void;
  /**
   * [descr:events.on(element, eventName, data, handler)]
   */
  export function on(
    element: Element | Array<Element>,
    eventName: string,
    data: any,
    handler: Function
  ): void;
  /**
   * [descr:events.on(element, eventName, handler)]
   */
  export function on(
    element: Element | Array<Element>,
    eventName: string,
    handler: Function
  ): void;
  /**
   * [descr:events.on(element, eventName, selector, data, handler)]
   */
  export function on(
    element: Element | Array<Element>,
    eventName: string,
    selector: string,
    data: any,
    handler: Function
  ): void;
  /**
   * [descr:events.on(element, eventName, selector, handler)]
   */
  export function on(
    element: Element | Array<Element>,
    eventName: string,
    selector: string,
    handler: Function
  ): void;
  /**
   * [descr:events.one(element, eventName, data, handler)]
   */
  export function one(
    element: Element | Array<Element>,
    eventName: string,
    data: any,
    handler: Function
  ): void;
  /**
   * [descr:events.one(element, eventName, handler)]
   */
  export function one(
    element: Element | Array<Element>,
    eventName: string,
    handler: Function
  ): void;
  /**
   * [descr:events.one(element, eventName, selector, data, handler)]
   */
  export function one(
    element: Element | Array<Element>,
    eventName: string,
    selector: string,
    data: any,
    handler: Function
  ): void;
  /**
   * [descr:events.one(element, eventName, selector, handler)]
   */
  export function one(
    element: Element | Array<Element>,
    eventName: string,
    selector: string,
    handler: Function
  ): void;
  /**
   * [descr:events.trigger(element, event)]
   */
  export function trigger(
    element: Element | Array<Element>,
    event: string | DxEvent
  ): void;
  /**
   * [descr:events.trigger(element, event, extraParameters)]
   */
  export function trigger(
    element: Element | Array<Element>,
    event: string | DxEvent,
    extraParameters: any
  ): void;
  /**
   * [descr:events.triggerHandler(element, event)]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export function triggerHandler(
    element: Element | Array<Element>,
    event: string | DxEvent
  ): void;
  /**
   * [descr:events.triggerHandler(element, event, extraParameters)]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export function triggerHandler(
    element: Element | Array<Element>,
    event: string | DxEvent,
    extraParameters: any
  ): void;
}
declare module DevExpress.excelExporter {
  /**
   * [descr:CellAddress]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CellAddress {
    /**
     * [descr:CellAddress.row]
     */
    row?: number;
    /**
     * [descr:CellAddress.column]
     */
    column?: number;
  }
  /**
   * [descr:CellRange]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CellRange {
    /**
     * [descr:CellRange.from]
     */
    from?: CellAddress;
    /**
     * [descr:CellRange.to]
     */
    to?: CellAddress;
  }
  export type DataGridCell = ExcelDataGridCell;
  /**
   * @deprecated Use DataGridCell instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ExcelDataGridCell {
    /**
     * [descr:ExcelDataGridCell.column]
     */
    column?: DevExpress.ui.dxDataGrid.Column;
    /**
     * [descr:ExcelDataGridCell.data]
     */
    data?: any;
    /**
     * [descr:ExcelDataGridCell.groupIndex]
     */
    groupIndex?: number;
    /**
     * [descr:ExcelDataGridCell.groupSummaryItems]
     */
    groupSummaryItems?: Array<{
      /**
       * [descr:ExcelDataGridCell.groupSummaryItems.name]
       */
      name?: string;
      /**
       * [descr:ExcelDataGridCell.groupSummaryItems.value]
       */
      value?: any;
    }>;
    /**
     * [descr:ExcelDataGridCell.rowType]
     */
    rowType?: string;
    /**
     * [descr:ExcelDataGridCell.totalSummaryItemName]
     */
    totalSummaryItemName?: string;
    /**
     * [descr:ExcelDataGridCell.value]
     */
    value?: any;
  }
  /**
   * [descr:ExcelExportBaseProps]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ExcelExportBaseProps {
    /**
     * [descr:ExcelExportBaseProps.worksheet]
     */
    worksheet?: object;
    /**
     * [descr:ExcelExportBaseProps.topLeftCell]
     */
    topLeftCell?: CellAddress | string;
    /**
     * [descr:ExcelExportBaseProps.keepColumnWidths]
     */
    keepColumnWidths?: boolean;
    /**
     * [descr:ExcelExportBaseProps.loadPanel]
     */
    loadPanel?: ExportLoadPanel;
  }
  /**
   * [descr:ExcelExportDataGridProps]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ExcelExportDataGridProps extends ExcelExportBaseProps {
    /**
     * [descr:ExcelExportDataGridProps.component]
     */
    component?: DevExpress.ui.dxDataGrid;
    /**
     * [descr:ExcelExportDataGridProps.selectedRowsOnly]
     */
    selectedRowsOnly?: boolean;
    /**
     * [descr:ExcelExportDataGridProps.autoFilterEnabled]
     */
    autoFilterEnabled?: boolean;
    /**
     * [descr:ExcelExportDataGridProps.customizeCell]
     */
    customizeCell?: (options: {
      gridCell?: DataGridCell;
      excelCell?: any;
    }) => void;
  }
  /**
   * [descr:ExcelExportPivotGridProps]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ExcelExportPivotGridProps extends ExcelExportBaseProps {
    /**
     * [descr:ExcelExportPivotGridProps.component]
     */
    component?: DevExpress.ui.dxPivotGrid;
    /**
     * [descr:ExcelExportPivotGridProps.mergeRowFieldValues]
     */
    mergeRowFieldValues?: boolean;
    /**
     * [descr:ExcelExportPivotGridProps.mergeColumnFieldValues]
     */
    mergeColumnFieldValues?: boolean;
    /**
     * [descr:ExcelExportPivotGridProps.customizeCell]
     */
    customizeCell?: (options: {
      pivotCell?: PivotGridCell;
      excelCell?: any;
    }) => void;
  }
  /**
   * @deprecated Use PivotGridCell instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ExcelPivotGridCell extends DevExpress.ui.dxPivotGrid.Cell {
    /**
     * [descr:ExcelPivotGridCell.area]
     */
    area?: string;
    /**
     * [descr:ExcelPivotGridCell.rowIndex]
     */
    rowIndex?: number;
    /**
     * [descr:ExcelPivotGridCell.columnIndex]
     */
    columnIndex?: number;
  }
  /**
   * [descr:excelExporter.exportDataGrid(options)]
   */
  export function exportDataGrid(
    options: ExcelExportDataGridProps
  ): DevExpress.core.utils.DxPromise<CellRange>;
  /**
   * [descr:excelExporter.exportPivotGrid(options)]
   */
  export function exportPivotGrid(
    options: ExcelExportPivotGridProps
  ): DevExpress.core.utils.DxPromise<CellRange>;
  export type PivotGridCell = ExcelPivotGridCell;
}
declare module DevExpress.exporter {
  /**
   * [descr:ExcelFont]
   * @deprecated [depNote:ExcelFont]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ExcelFont {
    /**
     * [descr:ExcelFont.bold]
     */
    bold?: boolean;
    /**
     * [descr:ExcelFont.color]
     */
    color?: string;
    /**
     * [descr:ExcelFont.italic]
     */
    italic?: boolean;
    /**
     * [descr:ExcelFont.name]
     */
    name?: string;
    /**
     * [descr:ExcelFont.size]
     */
    size?: number;
    /**
     * [descr:ExcelFont.underline]
     */
    underline?:
      | 'double'
      | 'doubleAccounting'
      | 'none'
      | 'single'
      | 'singleAccounting';
  }
}
declare module DevExpress.fileManagement {
  /**
   * [descr:CustomFileSystemProvider]
   */
  export class CustomFileSystemProvider extends FileSystemProviderBase {
    constructor(
      options?: DevExpress.fileManagement.CustomFileSystemProvider.Options
    );
  }
  module CustomFileSystemProvider {
    export type Options = CustomFileSystemProviderOptions;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CustomFileSystemProviderOptions
    extends FileSystemProviderBaseOptions<CustomFileSystemProvider> {
    /**
     * [descr:CustomFileSystemProviderOptions.abortFileUpload]
     */
    abortFileUpload?: (
      file: File,
      uploadInfo: UploadInfo,
      destinationDirectory: FileSystemItem
    ) => PromiseLike<any> | any;

    /**
     * [descr:CustomFileSystemProviderOptions.copyItem]
     */
    copyItem?: (
      item: FileSystemItem,
      destinationDirectory: FileSystemItem
    ) => PromiseLike<any> | any;

    /**
     * [descr:CustomFileSystemProviderOptions.createDirectory]
     */
    createDirectory?: (
      parentDirectory: FileSystemItem,
      name: string
    ) => PromiseLike<any> | any;

    /**
     * [descr:CustomFileSystemProviderOptions.deleteItem]
     */
    deleteItem?: (item: FileSystemItem) => PromiseLike<any> | any;

    /**
     * [descr:CustomFileSystemProviderOptions.downloadItems]
     */
    downloadItems?: (items: Array<FileSystemItem>) => void;

    /**
     * [descr:CustomFileSystemProviderOptions.getItems]
     */
    getItems?: (
      parentDirectory: FileSystemItem
    ) => PromiseLike<Array<any>> | Array<any>;

    /**
     * [descr:CustomFileSystemProviderOptions.getItemsContent]
     */
    getItemsContent?: (items: Array<FileSystemItem>) => PromiseLike<any> | any;

    /**
     * [descr:CustomFileSystemProviderOptions.hasSubDirectoriesExpr]
     */
    hasSubDirectoriesExpr?: string | Function;

    /**
     * [descr:CustomFileSystemProviderOptions.moveItem]
     */
    moveItem?: (
      item: FileSystemItem,
      destinationDirectory: FileSystemItem
    ) => PromiseLike<any> | any;

    /**
     * [descr:CustomFileSystemProviderOptions.renameItem]
     */
    renameItem?: (
      item: FileSystemItem,
      newName: string
    ) => PromiseLike<any> | any;

    /**
     * [descr:CustomFileSystemProviderOptions.uploadFileChunk]
     */
    uploadFileChunk?: (
      file: File,
      uploadInfo: UploadInfo,
      destinationDirectory: FileSystemItem
    ) => PromiseLike<any> | any;
  }
  /**
   * [descr:FileSystemError]
   */
  export class FileSystemError {
    constructor(
      errorCode?: number,
      fileSystemItem?: FileSystemItem,
      errorText?: string
    );
    /**
     * [descr:FileSystemError.fileSystemItem]
     */
    fileSystemItem?: FileSystemItem;

    /**
     * [descr:FileSystemError.errorCode]
     */
    errorCode?: number;

    /**
     * [descr:FileSystemError.errorText]
     */
    errorText?: string;
  }
  /**
   * [descr:FileSystemItem]
   */
  export class FileSystemItem {
    constructor(path: string, isDirectory: boolean, pathKeys?: Array<string>);

    /**
     * [descr:FileSystemItem.path]
     */
    path: string;

    /**
     * [descr:FileSystemItem.pathKeys]
     */
    pathKeys: Array<string>;

    /**
     * [descr:FileSystemItem.key]
     */
    key: string;

    /**
     * [descr:FileSystemItem.name]
     */
    name: string;

    /**
     * [descr:FileSystemItem.dateModified]
     */
    dateModified: Date;

    /**
     * [descr:FileSystemItem.size]
     */
    size: number;

    /**
     * [descr:FileSystemItem.isDirectory]
     */
    isDirectory: boolean;

    /**
     * [descr:FileSystemItem.hasSubDirectories]
     */
    hasSubDirectories: boolean;

    /**
     * [descr:FileSystemItem.thumbnail]
     */
    thumbnail: string;

    /**
     * [descr:FileSystemItem.dataItem]
     */
    dataItem: any;

    /**
     * [descr:FileSystemItem.getFileExtension()]
     */
    getFileExtension(): string;
  }
  /**
   * [descr:FileSystemProviderBase]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class FileSystemProviderBase {
    constructor(options?: FileSystemProviderBaseOptions);
    /**
     * [descr:FileSystemProviderBase.getItems()]
     */
    getItems(
      parentDirectory: FileSystemItem
    ): DevExpress.core.utils.DxPromise<Array<FileSystemItem>>;

    /**
     * [descr:FileSystemProviderBase.renameItem()]
     */
    renameItem(
      item: FileSystemItem,
      newName: string
    ): DevExpress.core.utils.DxPromise<any>;

    /**
     * [descr:FileSystemProviderBase.createDirectory()]
     */
    createDirectory(
      parentDirectory: FileSystemItem,
      name: string
    ): DevExpress.core.utils.DxPromise<any>;

    /**
     * [descr:FileSystemProviderBase.deleteItems()]
     */
    deleteItems(
      items: Array<FileSystemItem>
    ): Array<DevExpress.core.utils.DxPromise<any>>;

    /**
     * [descr:FileSystemProviderBase.moveItems()]
     */
    moveItems(
      items: Array<FileSystemItem>,
      destinationDirectory: FileSystemItem
    ): Array<DevExpress.core.utils.DxPromise<any>>;

    /**
     * [descr:FileSystemProviderBase.copyItems()]
     */
    copyItems(
      items: Array<FileSystemItem>,
      destinationDirectory: FileSystemItem
    ): Array<DevExpress.core.utils.DxPromise<any>>;

    /**
     * [descr:FileSystemProviderBase.uploadFileChunk()]
     */
    uploadFileChunk(
      fileData: File,
      uploadInfo: UploadInfo,
      destinationDirectory: FileSystemItem
    ): DevExpress.core.utils.DxPromise<any>;

    /**
     * [descr:FileSystemProviderBase.abortFileUpload()]
     */
    abortFileUpload(
      fileData: File,
      uploadInfo: UploadInfo,
      destinationDirectory: FileSystemItem
    ): DevExpress.core.utils.DxPromise<any>;

    /**
     * [descr:FileSystemProviderBase.downloadItems()]
     */
    downloadItems(items: Array<FileSystemItem>): void;

    /**
     * [descr:FileSystemProviderBase.getItemsContent()]
     */
    getItemsContent(
      items: Array<FileSystemItem>
    ): DevExpress.core.utils.DxPromise<any>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface FileSystemProviderBaseOptions<T = FileSystemProviderBase> {
    /**
     * [descr:FileSystemProviderBaseOptions.dateModifiedExpr]
     */
    dateModifiedExpr?: string | Function;
    /**
     * [descr:FileSystemProviderBaseOptions.isDirectoryExpr]
     */
    isDirectoryExpr?: string | Function;
    /**
     * [descr:FileSystemProviderBaseOptions.keyExpr]
     */
    keyExpr?: string | Function;
    /**
     * [descr:FileSystemProviderBaseOptions.nameExpr]
     */
    nameExpr?: string | Function;
    /**
     * [descr:FileSystemProviderBaseOptions.sizeExpr]
     */
    sizeExpr?: string | Function;
    /**
     * [descr:FileSystemProviderBaseOptions.thumbnailExpr]
     */
    thumbnailExpr?: string | Function;
  }
  /**
   * [descr:ObjectFileSystemProvider]
   */
  export class ObjectFileSystemProvider extends FileSystemProviderBase {
    constructor(
      options?: DevExpress.fileManagement.ObjectFileSystemProvider.Options
    );
  }
  module ObjectFileSystemProvider {
    export type Options = ObjectFileSystemProviderOptions;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ObjectFileSystemProviderOptions
    extends FileSystemProviderBaseOptions<ObjectFileSystemProvider> {
    /**
     * [descr:ObjectFileSystemProviderOptions.contentExpr]
     */
    contentExpr?: string | Function;
    /**
     * [descr:ObjectFileSystemProviderOptions.data]
     */
    data?: Array<any>;
    /**
     * [descr:ObjectFileSystemProviderOptions.itemsExpr]
     */
    itemsExpr?: string | Function;
  }
  /**
   * [descr:RemoteFileSystemProvider]
   */
  export class RemoteFileSystemProvider extends FileSystemProviderBase {
    constructor(
      options?: DevExpress.fileManagement.RemoteFileSystemProvider.Options
    );
  }
  module RemoteFileSystemProvider {
    export type Options = RemoteFileSystemProviderOptions;
  }
  /**
   * @deprecated Use Options instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface RemoteFileSystemProviderOptions
    extends FileSystemProviderBaseOptions<RemoteFileSystemProvider> {
    /**
     * [descr:RemoteFileSystemProviderOptions.beforeAjaxSend]
     */
    beforeAjaxSend?: (options: {
      headers?: any;
      xhrFields?: any;
      formData?: any;
    }) => void;
    /**
     * [descr:RemoteFileSystemProviderOptions.beforeSubmit]
     */
    beforeSubmit?: (options: { formData?: any }) => void;
    /**
     * [descr:RemoteFileSystemProviderOptions.endpointUrl]
     */
    endpointUrl?: string;
    /**
     * [descr:RemoteFileSystemProviderOptions.hasSubDirectoriesExpr]
     */
    hasSubDirectoriesExpr?: string | Function;
    /**
     * [descr:RemoteFileSystemProviderOptions.requestHeaders]
     */
    requestHeaders?: any;
  }
  /**
   * [descr:UploadInfo]
   */
  export interface UploadInfo {
    /**
     * [descr:UploadInfo.bytesUploaded]
     */
    bytesUploaded: number;

    /**
     * [descr:UploadInfo.chunkCount]
     */
    chunkCount: number;

    /**
     * [descr:UploadInfo.customData]
     */
    customData: any;

    /**
     * [descr:UploadInfo.chunkBlob]
     */
    chunkBlob: Blob;

    /**
     * [descr:UploadInfo.chunkIndex]
     */
    chunkIndex: number;
  }
}
declare module DevExpress.localization {
  /**
   * [descr:localization.formatDate(value, format)]
   */
  export function formatDate(value: Date, format: DevExpress.ui.Format): string;
  /**
   * [descr:localization.formatMessage(key, value)]
   */
  export function formatMessage(key: string, ...values: Array<string>): string;
  /**
   * [descr:localization.formatNumber(value, format)]
   */
  export function formatNumber(
    value: number,
    format: DevExpress.ui.Format
  ): string;
  /**
   * [descr:localization.loadMessages(messages)]
   */
  export function loadMessages(messages: any): void;
  /**
   * [descr:localization.locale()]
   */
  export function locale(): string;
  /**
   * [descr:localization.locale(locale)]
   */
  export function locale(locale: string): void;
  /**
   * [descr:localization.parseDate(text, format)]
   */
  export function parseDate(text: string, format: DevExpress.ui.Format): Date;
  /**
   * [descr:localization.parseNumber(text, format)]
   */
  export function parseNumber(
    text: string,
    format: DevExpress.ui.Format
  ): number;
}
declare module DevExpress.pdfExporter {
  /**
   * [descr:pdfExporter.exportDataGrid(options)]
   */
  export function exportDataGrid(
    options: PdfExportDataGridProps
  ): DevExpress.core.utils.DxPromise<void>;
  /**
   * [descr:pdfExporter.exportGantt(options)]
   */
  export function exportGantt(
    options: PdfExportGanttProps
  ): DevExpress.core.utils.DxPromise<any>;
  /**
   * [descr:PdfDataGridCell]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PdfDataGridCell {
    /**
     * [descr:PdfDataGridCell.column]
     */
    column?: DevExpress.ui.dxDataGrid.Column;
    /**
     * [descr:PdfDataGridCell.data]
     */
    data?: any;
    /**
     * [descr:PdfDataGridCell.groupIndex]
     */
    groupIndex?: number;
    /**
     * [descr:PdfDataGridCell.groupSummaryItems]
     */
    groupSummaryItems?: Array<{
      /**
       * [descr:PdfDataGridCell.groupSummaryItems.name]
       */
      name?: string;
      /**
       * [descr:PdfDataGridCell.groupSummaryItems.value]
       */
      value?: any;
    }>;
    /**
     * [descr:PdfDataGridCell.rowType]
     */
    rowType?: string;
    /**
     * [descr:PdfDataGridCell.totalSummaryItemName]
     */
    totalSummaryItemName?: string;
    /**
     * [descr:PdfDataGridCell.value]
     */
    value?: any;
  }
  /**
   * [descr:PdfExportDataGridProps]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PdfExportDataGridProps {
    /**
     * [descr:PdfExportDataGridProps.jsPDFDocument]
     */
    jsPDFDocument?: object;
    /**
     * [descr:PdfExportDataGridProps.autoTableOptions]
     */
    autoTableOptions?: object;
    /**
     * [descr:PdfExportDataGridProps.component]
     */
    component?: DevExpress.ui.dxDataGrid;
    /**
     * [descr:PdfExportDataGridProps.selectedRowsOnly]
     */
    selectedRowsOnly?: boolean;
    /**
     * [descr:PdfExportDataGridProps.keepColumnWidths]
     */
    keepColumnWidths?: boolean;
    /**
     * [descr:PdfExportDataGridProps.customizeCell]
     */
    customizeCell?: (options: {
      gridCell?: PdfDataGridCell;
      pdfCell?: any;
    }) => void;
    /**
     * [descr:PdfExportDataGridProps.loadPanel]
     */
    loadPanel?: ExportLoadPanel;
  }
  /**
   * [descr:PdfExportGanttProps]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PdfExportGanttProps {
    /**
     * [descr:PdfExportGanttProps.createDocumentMethod]
     */
    createDocumentMethod?: (options: any) => object;
    /**
     * [descr:PdfExportGanttProps.jsPDFDocument]
     */
    jsPDFDocument?: object;
    /**
     * [descr:PdfExportGanttProps.component]
     */
    component?: DevExpress.ui.dxGantt;
    /**
     * [descr:PdfExportGanttProps.format]
     */
    format?: string | object;
    /**
     * [descr:PdfExportGanttProps.landscape]
     */
    landscape?: boolean;
    /**
     * [descr:PdfExportGanttProps.fileName]
     */
    fileName?: string;
    /**
     * [descr:PdfExportGanttProps.margins]
     */
    margins?: object;
    /**
     * [descr:PdfExportGanttProps.exportMode]
     */
    exportMode?: 'all' | 'treeList' | 'chart';
    /**
     * [descr:PdfExportGanttProps.dateRange]
     */
    dateRange?: 'all' | 'visible' | object;
  }
}
declare module DevExpress.ui {
  /**
   * [descr:AsyncRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface AsyncRule {
    /**
     * [descr:AsyncRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:AsyncRule.message]
     */
    message?: string;
    /**
     * [descr:AsyncRule.reevaluate]
     */
    reevaluate?: boolean;
    /**
     * [descr:AsyncRule.type]
     */
    type: 'async';
    /**
     * [descr:AsyncRule.validationCallback]
     */
    validationCallback?: (options: ValidationCallbackData) => PromiseLike<any>;
  }
  /**
   * [descr:ColCountResponsible]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ColCountResponsible {
    /**
     * [descr:ColCountResponsible.lg]
     */
    lg?: number;
    /**
     * [descr:ColCountResponsible.md]
     */
    md?: number;
    /**
     * [descr:ColCountResponsible.sm]
     */
    sm?: number;
    /**
     * [descr:ColCountResponsible.xs]
     */
    xs?: number;
  }
  /**
   * [descr:CollectionWidget]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class CollectionWidget<
    TProperties extends CollectionWidgetOptions<any, TItem, TKey>,
    TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
    TKey = any
  > extends Widget<TProperties> {
    getDataSource(): DevExpress.data.DataSource<TItem, TKey>;
  }
  module CollectionWidget {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type ItemLike = string | CollectionWidgetItem | any;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface SelectionChangedInfo<TItem extends ItemLike = any> {
      readonly addedItems: Array<TItem>;
      readonly removedItems: Array<TItem>;
    }
  }
  /**
   * [descr:CollectionWidgetItem]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CollectionWidgetItem {
    /**
     * [descr:CollectionWidgetItem.disabled]
     */
    disabled?: boolean;
    /**
     * [descr:CollectionWidgetItem.html]
     */
    html?: string;
    /**
     * [descr:CollectionWidgetItem.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          itemData: this,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:CollectionWidgetItem.text]
     */
    text?: string;
    /**
     * [descr:CollectionWidgetItem.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CollectionWidgetOptions<
    TComponent extends CollectionWidget<any, TItem, TKey> | any,
    TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
    TKey = any
  > extends WidgetOptions<TComponent> {
    /**
     * [descr:CollectionWidgetOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:CollectionWidgetOptions.itemHoldTimeout]
     */
    itemHoldTimeout?: number;
    /**
     * [descr:CollectionWidgetOptions.itemTemplate]
     */
    itemTemplate?:
      | DevExpress.core.template
      | ((
          itemData: TItem,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:CollectionWidgetOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:CollectionWidgetOptions.keyExpr]
     */
    keyExpr?: string | Function;
    /**
     * [descr:CollectionWidgetOptions.noDataText]
     */
    noDataText?: string;
    /**
     * [descr:CollectionWidgetOptions.onItemClick]
     */
    onItemClick?:
      | ((
          e: DevExpress.events.NativeEventInfo<
            TComponent,
            KeyboardEvent | MouseEvent | PointerEvent
          > &
            DevExpress.events.ItemInfo<TItem>
        ) => void)
      | string;
    /**
     * [descr:CollectionWidgetOptions.onItemContextMenu]
     */
    onItemContextMenu?: (
      e: DevExpress.events.NativeEventInfo<
        TComponent,
        MouseEvent | PointerEvent | TouchEvent
      > &
        DevExpress.events.ItemInfo<TItem>
    ) => void;
    /**
     * [descr:CollectionWidgetOptions.onItemHold]
     */
    onItemHold?: (
      e: DevExpress.events.NativeEventInfo<
        TComponent,
        MouseEvent | PointerEvent | TouchEvent
      > &
        DevExpress.events.ItemInfo<TItem>
    ) => void;
    /**
     * [descr:CollectionWidgetOptions.onItemRendered]
     */
    onItemRendered?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.events.ItemInfo<TItem>
    ) => void;
    /**
     * [descr:CollectionWidgetOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>
    ) => void;
    /**
     * [descr:CollectionWidgetOptions.selectedIndex]
     */
    selectedIndex?: number;
    /**
     * [descr:CollectionWidgetOptions.selectedItem]
     */
    selectedItem?: TItem;
    /**
     * [descr:CollectionWidgetOptions.selectedItemKeys]
     */
    selectedItemKeys?: Array<TKey>;
    /**
     * [descr:CollectionWidgetOptions.selectedItems]
     */
    selectedItems?: Array<TItem>;
  }
  /**
   * [descr:CompareRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CompareRule {
    /**
     * [descr:CompareRule.comparisonTarget]
     */
    comparisonTarget?: () => any;
    /**
     * [descr:CompareRule.comparisonType]
     */
    comparisonType?: '!=' | '!==' | '<' | '<=' | '==' | '===' | '>' | '>=';
    /**
     * [descr:CompareRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:CompareRule.message]
     */
    message?: string;
    /**
     * [descr:CompareRule.type]
     */
    type: 'compare';
  }
  export interface CustomDialogOptions {
    title?: string;
    messageHtml?: string;
    buttons?: Array<dxButtonOptions>;
    showTitle?: boolean;
    message?: string;
    dragEnabled?: boolean;
  }
  /**
   * [descr:CustomRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CustomRule {
    /**
     * [descr:CustomRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:CustomRule.message]
     */
    message?: string;
    /**
     * [descr:CustomRule.reevaluate]
     */
    reevaluate?: boolean;
    /**
     * [descr:CustomRule.type]
     */
    type: 'custom';
    /**
     * [descr:CustomRule.validationCallback]
     */
    validationCallback?: (options: ValidationCallbackData) => boolean;
  }
  /**
   * [descr:DataExpressionMixin]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class DataExpressionMixin {
    constructor(options?: DataExpressionMixinOptions);
    getDataSource(): DevExpress.data.DataSource;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface DataExpressionMixinOptions<T = DataExpressionMixin> {
    /**
     * [descr:DataExpressionMixinOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<
      CollectionWidgetItem | any
    >;
    /**
     * [descr:DataExpressionMixinOptions.displayExpr]
     */
    displayExpr?: string | ((item: any) => string);
    /**
     * [descr:DataExpressionMixinOptions.itemTemplate]
     */
    itemTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:DataExpressionMixinOptions.items]
     */
    items?: Array<CollectionWidgetItem | any>;
    /**
     * [descr:DataExpressionMixinOptions.value]
     */
    value?: any;
    /**
     * [descr:DataExpressionMixinOptions.valueExpr]
     */
    valueExpr?: string | ((item: any) => string | number | boolean);
  }
  /**
   * [descr:DraggableBase]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface DraggableBase {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface DraggableBaseOptions<TComponent>
    extends DOMComponentOptions<TComponent> {
    /**
     * [descr:DraggableBaseOptions.autoScroll]
     */
    autoScroll?: boolean;
    /**
     * [descr:DraggableBaseOptions.boundary]
     */
    boundary?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:DraggableBaseOptions.container]
     */
    container?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:DraggableBaseOptions.cursorOffset]
     */
    cursorOffset?:
      | string
      | {
          /**
           * [descr:DraggableBaseOptions.cursorOffset.x]
           */
          x?: number;
          /**
           * [descr:DraggableBaseOptions.cursorOffset.y]
           */
          y?: number;
        };
    /**
     * [descr:DraggableBaseOptions.data]
     */
    data?: any;
    /**
     * [descr:DraggableBaseOptions.dragDirection]
     */
    dragDirection?: 'both' | 'horizontal' | 'vertical';
    /**
     * [descr:DraggableBaseOptions.group]
     */
    group?: string;
    /**
     * [descr:DraggableBaseOptions.handle]
     */
    handle?: string;
    /**
     * [descr:DraggableBaseOptions.scrollSensitivity]
     */
    scrollSensitivity?: number;
    /**
     * [descr:DraggableBaseOptions.scrollSpeed]
     */
    scrollSpeed?: number;
  }
  /**
   * [descr:dxAccordion]
   */
  export class dxAccordion<
    TItem extends DevExpress.ui.dxAccordion.ItemLike = any,
    TKey = any
  > extends CollectionWidget<dxAccordionOptions<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxAccordion.collapseItem(index)]
     */
    collapseItem(index: number): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxAccordion.expandItem(index)]
     */
    expandItem(index: number): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxAccordion.updateDimensions()]
     */
    updateDimensions(): DevExpress.core.utils.DxPromise<void>;
  }
  module dxAccordion {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxAccordion<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxAccordion<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      ItemTitleClickEvent: ItemTitleClickEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxAccordion<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxAccordion<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxAccordion<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxAccordion<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxAccordion<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemTitleClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxAccordion<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxAccordion<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxAccordionOptions<TItem, TKey>;
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxAccordion<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxAccordionItem extends CollectionWidgetItem {
    /**
     * [descr:dxAccordionItem.icon]
     */
    icon?: string;
    /**
     * [descr:dxAccordionItem.title]
     */
    title?: string;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxAccordionOptions<
    TItem extends DevExpress.ui.dxAccordion.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<dxAccordion<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxAccordionOptions.animationDuration]
     */
    animationDuration?: number;
    /**
     * [descr:dxAccordionOptions.collapsible]
     */
    collapsible?: boolean;
    /**
     * [descr:dxAccordionOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxAccordionOptions.deferRendering]
     */
    deferRendering?: boolean;
    /**
     * [descr:dxAccordionOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxAccordionOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxAccordionOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxAccordionOptions.itemTemplate]
     */
    itemTemplate?:
      | DevExpress.core.template
      | ((
          itemData: TItem,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxAccordionOptions.itemTitleTemplate]
     */
    itemTitleTemplate?:
      | DevExpress.core.template
      | ((
          itemData: TItem,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxAccordionOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxAccordionOptions.multiple]
     */
    multiple?: boolean;
    /**
     * [descr:dxAccordionOptions.onItemTitleClick]
     */
    onItemTitleClick?:
      | ((
          e: DevExpress.ui.dxAccordion.ItemTitleClickEvent<TItem, TKey>
        ) => void)
      | string;
    /**
     * [descr:dxAccordionOptions.repaintChangesOnly]
     */
    repaintChangesOnly?: boolean;
    /**
     * [descr:dxAccordionOptions.selectedIndex]
     */
    selectedIndex?: number;
  }
  /**
   * [descr:dxActionSheet]
   */
  export class dxActionSheet<
    TItem extends DevExpress.ui.dxActionSheet.ItemLike<TKey> = any,
    TKey = any
  > extends CollectionWidget<dxActionSheetOptions<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxActionSheet.hide()]
     */
    hide(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxActionSheet.show()]
     */
    show(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxActionSheet.toggle(showing)]
     */
    toggle(showing: boolean): DevExpress.core.utils.DxPromise<void>;
  }
  module dxActionSheet {
    export type CancelClickEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxActionSheet<TItem, TKey>>;
    export type ContentReadyEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxActionSheet<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxActionSheet<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike<TKey>, TKey> = {
      Properties: Properties<TItem, TKey>;
      CancelClickEvent: CancelClickEvent<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxActionSheet<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxActionSheet<TItem, TKey>,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxActionSheet<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxActionSheet<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike<TKey> = string | Item<TKey> | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxActionSheet<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxActionSheet<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = dxActionSheetOptions<TItem, TKey>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxActionSheetItem<TKey = any> extends CollectionWidgetItem {
    /**
     * [descr:dxActionSheetItem.icon]
     */
    icon?: string;
    /**
     * [descr:dxActionSheetItem.onClick]
     */
    onClick?:
      | ((
          e: DevExpress.events.NativeEventInfo<
            dxActionSheet<this, TKey>,
            MouseEvent | PointerEvent
          >
        ) => void)
      | string;
    /**
     * [descr:dxActionSheetItem.type]
     */
    type?: 'back' | 'danger' | 'default' | 'normal' | 'success';
    /**
     * [descr:dxActionSheetItem.stylingMode]
     */
    stylingMode?: 'text' | 'outlined' | 'contained';
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxActionSheetOptions<
    TItem extends DevExpress.ui.dxActionSheet.ItemLike<TKey> = any,
    TKey = any
  > extends CollectionWidgetOptions<dxActionSheet<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxActionSheetOptions.cancelText]
     */
    cancelText?: string;
    /**
     * [descr:dxActionSheetOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxActionSheetOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxActionSheetOptions.onCancelClick]
     */
    onCancelClick?:
      | ((e: DevExpress.ui.dxActionSheet.CancelClickEvent<TItem, TKey>) => void)
      | string;
    /**
     * [descr:dxActionSheetOptions.showCancelButton]
     */
    showCancelButton?: boolean;
    /**
     * [descr:dxActionSheetOptions.showTitle]
     */
    showTitle?: boolean;
    /**
     * [descr:dxActionSheetOptions.target]
     */
    target?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxActionSheetOptions.title]
     */
    title?: string;
    /**
     * [descr:dxActionSheetOptions.usePopover]
     */
    usePopover?: boolean;
    /**
     * [descr:dxActionSheetOptions.visible]
     */
    visible?: boolean;
  }
  /**
   * [descr:dxAutocomplete]
   */
  export class dxAutocomplete extends dxDropDownList<dxAutocompleteOptions> {}
  module dxAutocomplete {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      Event
    >;
    export type ClosedEvent = DevExpress.events.EventInfo<dxAutocomplete>;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxAutocomplete>;
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      ClipboardEvent
    >;
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      ClipboardEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxAutocomplete>;
    export type DropDownButtonTemplateData =
      DevExpress.ui.dxDropDownEditor.DropDownButtonTemplateDataModel;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxAutocomplete>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      UIEvent
    >;
    export type ItemClickEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      DevExpress.events.ItemInfo;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      KeyboardEvent
    >;
    export type OpenedEvent = DevExpress.events.EventInfo<dxAutocomplete>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxAutocomplete> &
        DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      ClipboardEvent
    >;
    export type Properties = dxAutocompleteOptions;
    export type SelectionChangedEvent =
      DevExpress.events.EventInfo<dxAutocomplete> &
        DevExpress.ui.dxDropDownList.SelectionChangedInfo;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxAutocomplete,
      KeyboardEvent | MouseEvent | PointerEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxAutocompleteOptions
    extends dxDropDownListOptions<dxAutocomplete> {
    /**
     * [descr:dxAutocompleteOptions.maxItemCount]
     */
    maxItemCount?: number;
    /**
     * [descr:dxAutocompleteOptions.minSearchLength]
     */
    minSearchLength?: number;
    /**
     * [descr:dxAutocompleteOptions.showDropDownButton]
     */
    showDropDownButton?: boolean;
    /**
     * [descr:dxAutocompleteOptions.value]
     */
    value?: string;

    /**
     * [descr:dxAutocompleteOptions.dropDownOptions]
     */
    dropDownOptions?: DevExpress.ui.dxPopup.Properties;
  }
  /**
   * [descr:dxBox]
   */
  export class dxBox<
    TItem extends DevExpress.ui.dxBox.ItemLike<TKey> = any,
    TKey = any
  > extends CollectionWidget<dxBoxOptions<TItem, TKey>, TItem, TKey> {}
  module dxBox {
    export type ContentReadyEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxBox<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxBox<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike<TKey>, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxBox<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike<TKey> = string | Item<TKey> | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxBox<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike<TKey> = any,
      TKey = any
    > = dxBoxOptions<TItem, TKey>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxBoxItem<TKey = any> extends CollectionWidgetItem {
    /**
     * [descr:dxBoxItem.baseSize]
     */
    baseSize?: number | 'auto';
    /**
     * [descr:dxBoxItem.box]
     */
    box?: dxBoxOptions<any, TKey>;
    /**
     * [descr:dxBoxItem.ratio]
     */
    ratio?: number;
    /**
     * [descr:dxBoxItem.shrink]
     */
    shrink?: number;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxBoxOptions<
    TItem extends DevExpress.ui.dxBox.ItemLike<TKey> = any,
    TKey = any
  > extends CollectionWidgetOptions<dxBox<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxBoxOptions.align]
     */
    align?: 'center' | 'end' | 'space-around' | 'space-between' | 'start';
    /**
     * [descr:dxBoxOptions.crossAlign]
     */
    crossAlign?: 'center' | 'end' | 'start' | 'stretch';
    /**
     * [descr:dxBoxOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxBoxOptions.direction]
     */
    direction?: 'col' | 'row';
    /**
     * [descr:dxBoxOptions.items]
     */
    items?: Array<TItem>;
  }
  /**
   * [descr:dxButton]
   */
  export class dxButton extends Widget<dxButtonOptions> {}
  module dxButton {
    export type ClickEvent = DevExpress.events.NativeEventInfo<
      dxButton,
      KeyboardEvent | MouseEvent | PointerEvent
    > & {
      validationGroup?: any;
    };
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxButton>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxButton>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxButton>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxButton> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxButtonOptions;
    export type TemplateData = {
      readonly text?: string;
      readonly icon?: string;
    };
  }
  /**
   * [descr:dxButtonGroup]
   */
  export class dxButtonGroup extends Widget<dxButtonGroupOptions> {}
  module dxButtonGroup {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxButtonGroup>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxButtonGroup>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxButtonGroup>;
    export type ItemClickEvent =
      DevExpress.events.NativeEventInfo<dxButtonGroup> &
        DevExpress.events.ItemInfo;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxButtonGroup> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxButtonGroupOptions;
    export type SelectionChangedEvent =
      DevExpress.events.EventInfo<dxButtonGroup> &
        DevExpress.ui.CollectionWidget.SelectionChangedInfo;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxButtonGroupItem extends CollectionWidgetItem {
    /**
     * [descr:dxButtonGroupItem.hint]
     */
    hint?: string;
    /**
     * [descr:dxButtonGroupItem.icon]
     */
    icon?: string;
    /**
     * [descr:dxButtonGroupItem.type]
     */
    type?: 'back' | 'danger' | 'default' | 'normal' | 'success';

    /**
     * [descr:dxButtonGroupItem.elementAttr]
     */
    elementAttr?: { [key: string]: any };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxButtonGroupOptions extends WidgetOptions<dxButtonGroup> {
    /**
     * [descr:dxButtonGroupOptions.buttonTemplate]
     */
    buttonTemplate?:
      | DevExpress.core.template
      | ((
          buttonData: any,
          buttonContent: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxButtonGroupOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxButtonGroupOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxButtonGroupOptions.items]
     */
    items?: Array<DevExpress.ui.dxButtonGroup.Item>;
    /**
     * [descr:dxButtonGroupOptions.keyExpr]
     */
    keyExpr?: string | Function;
    /**
     * [descr:dxButtonGroupOptions.onItemClick]
     */
    onItemClick?: (e: DevExpress.ui.dxButtonGroup.ItemClickEvent) => void;
    /**
     * [descr:dxButtonGroupOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.ui.dxButtonGroup.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxButtonGroupOptions.selectedItemKeys]
     */
    selectedItemKeys?: Array<any>;
    /**
     * [descr:dxButtonGroupOptions.selectedItems]
     */
    selectedItems?: Array<any>;
    /**
     * [descr:dxButtonGroupOptions.selectionMode]
     */
    selectionMode?: 'multiple' | 'single' | 'none';
    /**
     * [descr:dxButtonGroupOptions.stylingMode]
     */
    stylingMode?: 'text' | 'outlined' | 'contained';
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxButtonOptions extends WidgetOptions<dxButton> {
    /**
     * [descr:dxButtonOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxButtonOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxButtonOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxButtonOptions.icon]
     */
    icon?: string;
    /**
     * [descr:dxButtonOptions.onClick]
     */
    onClick?: (e: DevExpress.ui.dxButton.ClickEvent) => void;
    /**
     * [descr:dxButtonOptions.stylingMode]
     */
    stylingMode?: 'text' | 'outlined' | 'contained';
    /**
     * [descr:dxButtonOptions.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          data: DevExpress.ui.dxButton.TemplateData,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxButtonOptions.text]
     */
    text?: string;
    /**
     * [descr:dxButtonOptions.type]
     */
    type?: 'back' | 'danger' | 'default' | 'normal' | 'success';
    /**
     * [descr:dxButtonOptions.useSubmitBehavior]
     */
    useSubmitBehavior?: boolean;
    /**
     * [descr:dxButtonOptions.validationGroup]
     */
    validationGroup?: string;
  }
  /**
   * [descr:dxCalendar]
   */
  export class dxCalendar extends Editor<dxCalendarOptions> {}
  module dxCalendar {
    export type CellTemplateData = {
      readonly date: Date;
      readonly view: string;
      readonly text?: string;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ComponentDisabledDate<T> {
      component: T;
      readonly date: Date;
      readonly view: string;
    }
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxCalendar>;
    export type DisabledDate = ComponentDisabledDate<dxCalendar>;
    export type Properties = dxCalendarOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxCalendar,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxCalendarOptions extends EditorOptions<dxCalendar> {
    /**
     * [descr:dxCalendarOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxCalendarOptions.cellTemplate]
     */
    cellTemplate?:
      | DevExpress.core.template
      | ((
          itemData: DevExpress.ui.dxCalendar.CellTemplateData,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxCalendarOptions.dateSerializationFormat]
     */
    dateSerializationFormat?: string;
    /**
     * [descr:dxCalendarOptions.disabledDates]
     */
    disabledDates?:
      | Array<Date>
      | ((data: DevExpress.ui.dxCalendar.DisabledDate) => boolean);
    /**
     * [descr:dxCalendarOptions.firstDayOfWeek]
     */
    firstDayOfWeek?: 0 | 1 | 2 | 3 | 4 | 5 | 6;
    /**
     * [descr:dxCalendarOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxCalendarOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxCalendarOptions.max]
     */
    max?: Date | number | string;
    /**
     * [descr:dxCalendarOptions.maxZoomLevel]
     */
    maxZoomLevel?: 'century' | 'decade' | 'month' | 'year';
    /**
     * [descr:dxCalendarOptions.min]
     */
    min?: Date | number | string;
    /**
     * [descr:dxCalendarOptions.minZoomLevel]
     */
    minZoomLevel?: 'century' | 'decade' | 'month' | 'year';
    /**
     * [descr:dxCalendarOptions.name]
     */
    name?: string;
    /**
     * [descr:dxCalendarOptions.showTodayButton]
     */
    showTodayButton?: boolean;
    /**
     * [descr:dxCalendarOptions.value]
     */
    value?: Date | number | string;
    /**
     * [descr:dxCalendarOptions.zoomLevel]
     */
    zoomLevel?: 'century' | 'decade' | 'month' | 'year';
  }
  /**
   * [descr:dxCheckBox]
   */
  export class dxCheckBox extends Editor<dxCheckBoxOptions> {
    /**
     * [descr:dxCheckBox.blur()]
     */
    blur(): void;
  }
  module dxCheckBox {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxCheckBox>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxCheckBox>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxCheckBox>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxCheckBox> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxCheckBoxOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxCheckBox,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxCheckBoxOptions extends EditorOptions<dxCheckBox> {
    /**
     * [descr:dxCheckBoxOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxCheckBoxOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxCheckBoxOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxCheckBoxOptions.iconSize]
     */
    iconSize?: number | string;
    /**
     * [descr:dxCheckBoxOptions.name]
     */
    name?: string;
    /**
     * [descr:dxCheckBoxOptions.text]
     */
    text?: string;
    /**
     * [descr:dxCheckBoxOptions.value]
     */
    value?: boolean | null | undefined;
  }
  /**
   * [descr:dxColorBox]
   */
  export class dxColorBox extends dxDropDownEditor<dxColorBoxOptions> {}
  module dxColorBox {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      Event
    >;
    export type ClosedEvent = DevExpress.events.EventInfo<dxColorBox>;
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      ClipboardEvent
    >;
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      ClipboardEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxColorBox>;
    export type DropDownButtonTemplateData =
      DevExpress.ui.dxDropDownEditor.DropDownButtonTemplateDataModel;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxColorBox>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      UIEvent
    >;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      KeyboardEvent
    >;
    export type OpenedEvent = DevExpress.events.EventInfo<dxColorBox>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxColorBox> &
      DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      ClipboardEvent
    >;
    export type Properties = dxColorBoxOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxColorBox,
      KeyboardEvent | MouseEvent | PointerEvent | UIEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxColorBoxOptions
    extends dxDropDownEditorOptions<dxColorBox> {
    /**
     * [descr:dxColorBoxOptions.applyButtonText]
     */
    applyButtonText?: string;
    /**
     * [descr:dxColorBoxOptions.applyValueMode]
     */
    applyValueMode?: 'instantly' | 'useButtons';
    /**
     * [descr:dxColorBoxOptions.cancelButtonText]
     */
    cancelButtonText?: string;
    /**
     * [descr:dxColorBoxOptions.editAlphaChannel]
     */
    editAlphaChannel?: boolean;
    /**
     * [descr:dxColorBoxOptions.fieldTemplate]
     */
    fieldTemplate?:
      | DevExpress.core.template
      | ((
          value: string,
          fieldElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxColorBoxOptions.keyStep]
     */
    keyStep?: number;
    /**
     * [descr:dxColorBoxOptions.value]
     */
    value?: string;

    /**
     * [descr:dxColorBoxOptions.dropDownOptions]
     */
    dropDownOptions?: DevExpress.ui.dxPopup.Properties;
  }
  /**
   * [descr:dxContextMenu]
   */
  export class dxContextMenu<TKey = any> extends dxMenuBase<
    dxContextMenuOptions<TKey>,
    dxContextMenuItem,
    TKey
  > {
    /**
     * [descr:dxContextMenu.hide()]
     */
    hide(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxContextMenu.show()]
     */
    show(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxContextMenu.toggle(showing)]
     */
    toggle(showing: boolean): DevExpress.core.utils.DxPromise<void>;
  }
  module dxContextMenu {
    export type ContentReadyEvent<TKey = any> = DevExpress.events.EventInfo<
      dxContextMenu<TKey>
    >;
    export type DisposingEvent<TKey = any> = DevExpress.events.EventInfo<
      dxContextMenu<TKey>
    >;
    export type ExplicitTypes<TKey = any> = {
      Properties: Properties<TKey>;
      ContentReadyEvent: ContentReadyEvent<TKey>;
      DisposingEvent: DisposingEvent<TKey>;
      HiddenEvent: HiddenEvent<TKey>;
      HidingEvent: HidingEvent<TKey>;
      InitializedEvent: InitializedEvent<TKey>;
      ItemClickEvent: ItemClickEvent<TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TKey>;
      OptionChangedEvent: OptionChangedEvent<TKey>;
      PositioningEvent: PositioningEvent<TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TKey>;
      ShowingEvent: ShowingEvent<TKey>;
      ShownEvent: ShownEvent<TKey>;
    };
    export type HiddenEvent<TKey = any> = DevExpress.events.EventInfo<
      dxContextMenu<TKey>
    >;
    export type HidingEvent<TKey = any> = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxContextMenu<TKey>>;
    export type InitializedEvent<TKey = any> =
      DevExpress.events.InitializedEventInfo<dxContextMenu<TKey>>;
    export type ItemClickEvent<TKey = any> = DevExpress.events.NativeEventInfo<
      dxContextMenu<TKey>
    > &
      DevExpress.events.ItemInfo<Item>;
    export type ItemContextMenuEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxContextMenu<TKey>> &
        DevExpress.events.ItemInfo<Item>;
    export type ItemRenderedEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxContextMenu<TKey>> &
        DevExpress.events.ItemInfo<Item>;
    export type OptionChangedEvent<TKey = any> = DevExpress.events.EventInfo<
      dxContextMenu<TKey>
    > &
      DevExpress.events.ChangedOptionInfo;
    export type PositioningEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxContextMenu<TKey>> & {
        readonly position: PositionConfig;
      };
    export type Properties<TKey = any> = dxContextMenuOptions<TKey>;
    export type SelectionChangedEvent<TKey = any> = DevExpress.events.EventInfo<
      dxContextMenu<TKey>
    > &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<Item>;
    export type ShowingEvent<TKey = any> = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxContextMenu<TKey>>;
    export type ShownEvent<TKey = any> = DevExpress.events.EventInfo<
      dxContextMenu<TKey>
    >;
  }
  /**
   * @deprecated Use DevExpress.ui.dxContextMenu.Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxContextMenuItem extends dxMenuBaseItem {
    /**
     * [descr:dxContextMenuItem.items]
     */
    items?: Array<DevExpress.ui.dxContextMenu.Item>;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxContextMenuOptions<TKey = any>
    extends dxMenuBaseOptions<dxContextMenu<TKey>, dxContextMenuItem, TKey> {
    /**
     * [descr:dxContextMenuOptions.closeOnOutsideClick]
     */
    closeOnOutsideClick?:
      | boolean
      | ((event: DevExpress.events.DxEvent) => boolean);
    /**
     * [descr:dxContextMenuOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<
      DevExpress.ui.dxContextMenu.Item,
      TKey
    >;
    /**
     * [descr:dxContextMenuOptions.items]
     */
    items?: Array<DevExpress.ui.dxContextMenu.Item>;
    /**
     * [descr:dxContextMenuOptions.onHidden]
     */
    onHidden?: (e: DevExpress.ui.dxContextMenu.HiddenEvent<TKey>) => void;
    /**
     * [descr:dxContextMenuOptions.onHiding]
     */
    onHiding?: (e: DevExpress.ui.dxContextMenu.HidingEvent<TKey>) => void;
    /**
     * [descr:dxContextMenuOptions.onPositioning]
     */
    onPositioning?: (
      e: DevExpress.ui.dxContextMenu.PositioningEvent<TKey>
    ) => void;
    /**
     * [descr:dxContextMenuOptions.onShowing]
     */
    onShowing?: (e: DevExpress.ui.dxContextMenu.ShowingEvent<TKey>) => void;
    /**
     * [descr:dxContextMenuOptions.onShown]
     */
    onShown?: (e: DevExpress.ui.dxContextMenu.ShownEvent<TKey>) => void;
    /**
     * [descr:dxContextMenuOptions.position]
     */
    position?: PositionConfig;
    /**
     * [descr:dxContextMenuOptions.showEvent]
     */
    showEvent?:
      | {
          /**
           * [descr:dxContextMenuOptions.showEvent.delay]
           */
          delay?: number;
          /**
           * [descr:dxContextMenuOptions.showEvent.name]
           */
          name?: string;
        }
      | string;
    /**
     * [descr:dxContextMenuOptions.submenuDirection]
     */
    submenuDirection?: 'auto' | 'left' | 'right';
    /**
     * [descr:dxContextMenuOptions.target]
     */
    target?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxContextMenuOptions.visible]
     */
    visible?: boolean;
  }
  /**
   * [descr:dxDataGrid]
   */
  export class dxDataGrid<TRowData = any, TKey = any>
    extends Widget<dxDataGridOptions<TRowData, TKey>>
    implements GridBase<TRowData, TKey>
  {
    /**
     * [descr:dxDataGrid.addColumn(columnOptions)]
     */
    addColumn(
      columnOptions: DevExpress.ui.dxDataGrid.Column<TRowData, TKey> | string
    ): void;
    /**
     * [descr:dxDataGrid.addRow()]
     */
    addRow(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxDataGrid.clearGrouping()]
     */
    clearGrouping(): void;
    /**
     * [descr:dxDataGrid.collapseAll(groupIndex)]
     */
    collapseAll(groupIndex?: number): void;
    /**
     * [descr:dxDataGrid.collapseRow(key)]
     */
    collapseRow(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxDataGrid.expandAll(groupIndex)]
     */
    expandAll(groupIndex?: number): void;
    /**
     * [descr:dxDataGrid.expandRow(key)]
     */
    expandRow(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxDataGrid.exportToExcel(selectionOnly)]
     * @deprecated [depNote:dxDataGrid.exportToExcel(selectionOnly)]
     */
    exportToExcel(selectionOnly: boolean): void;
    /**
     * [descr:dxDataGrid.getSelectedRowKeys()]
     */
    getSelectedRowKeys(): Array<TKey> &
      DevExpress.core.utils.DxPromise<Array<TKey>>;
    /**
     * [descr:dxDataGrid.getSelectedRowsData()]
     */
    getSelectedRowsData(): Array<TRowData> &
      DevExpress.core.utils.DxPromise<Array<TRowData>>;
    /**
     * [descr:dxDataGrid.getTotalSummaryValue(summaryItemName)]
     */
    getTotalSummaryValue(summaryItemName: string): any;
    /**
     * [descr:dxDataGrid.getVisibleColumns()]
     */
    getVisibleColumns(): Array<DevExpress.ui.dxDataGrid.Column<TRowData, TKey>>;
    /**
     * [descr:dxDataGrid.getVisibleColumns(headerLevel)]
     */
    getVisibleColumns(
      headerLevel: number
    ): Array<DevExpress.ui.dxDataGrid.Column<TRowData, TKey>>;
    /**
     * [descr:dxDataGrid.getVisibleRows()]
     */
    getVisibleRows(): Array<DevExpress.ui.dxDataGrid.Row<TRowData, TKey>>;
    /**
     * [descr:dxDataGrid.isRowExpanded(key)]
     */
    isRowExpanded(key: TKey): boolean;
    /**
     * [descr:dxDataGrid.isRowSelected(data)]
     */
    isRowSelected(data: TRowData): boolean;
    isRowSelected(key: TKey): boolean;
    /**
     * [descr:dxDataGrid.totalCount()]
     */
    totalCount(): number;

    beginCustomLoading(messageText: string): void;
    byKey(key: TKey): DevExpress.core.utils.DxPromise<TRowData>;
    cancelEditData(): void;
    cellValue(rowIndex: number, dataField: string): any;
    cellValue(rowIndex: number, dataField: string, value: any): void;
    cellValue(rowIndex: number, visibleColumnIndex: number): any;
    cellValue(rowIndex: number, visibleColumnIndex: number, value: any): void;
    clearFilter(): void;
    clearFilter(filterName: string): void;
    clearSelection(): void;
    clearSorting(): void;
    closeEditCell(): void;
    collapseAdaptiveDetailRow(): void;
    columnCount(): number;
    columnOption(id: number | string): any;
    columnOption(id: number | string, optionName: string): any;
    columnOption(
      id: number | string,
      optionName: string,
      optionValue: any
    ): void;
    columnOption(id: number | string, options: any): void;
    deleteColumn(id: number | string): void;
    deleteRow(rowIndex: number): void;
    deselectAll(): DevExpress.core.utils.DxPromise<void>;
    deselectRows(
      keys: Array<TKey>
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    editCell(rowIndex: number, dataField: string): void;
    editCell(rowIndex: number, visibleColumnIndex: number): void;
    editRow(rowIndex: number): void;
    endCustomLoading(): void;
    expandAdaptiveDetailRow(key: TKey): void;
    filter(): any;
    filter(filterExpr: any): void;
    focus(): void;
    focus(element: DevExpress.core.UserDefinedElement): void;
    getCellElement(
      rowIndex: number,
      dataField: string
    ): DevExpress.core.DxElement | undefined;
    getCellElement(
      rowIndex: number,
      visibleColumnIndex: number
    ): DevExpress.core.DxElement | undefined;
    getCombinedFilter(): any;
    getCombinedFilter(returnDataField: boolean): any;
    getDataSource(): DevExpress.data.DataSource<TRowData, TKey>;
    getKeyByRowIndex(rowIndex: number): TKey | undefined;
    getRowElement(
      rowIndex: number
    ): DevExpress.core.UserDefinedElementsArray | undefined;
    getRowIndexByKey(key: TKey): number;
    getScrollable(): dxScrollable;
    getVisibleColumnIndex(id: number | string): number;
    hasEditData(): boolean;
    hideColumnChooser(): void;
    isAdaptiveDetailRowExpanded(key: TKey): boolean;
    isRowFocused(key: TKey): boolean;
    keyOf(obj: TRowData): TKey;
    navigateToRow(key: TKey): DevExpress.core.utils.DxPromise<void>;
    pageCount(): number;
    pageIndex(): number;
    pageIndex(newIndex: number): DevExpress.core.utils.DxPromise<void>;
    pageSize(): number;
    pageSize(value: number): void;
    refresh(): DevExpress.core.utils.DxPromise<void>;
    refresh(changesOnly: boolean): DevExpress.core.utils.DxPromise<void>;
    repaintRows(rowIndexes: Array<number>): void;
    saveEditData(): DevExpress.core.utils.DxPromise<void>;
    searchByText(text: string): void;
    selectAll(): DevExpress.core.utils.DxPromise<void>;
    selectRows(
      keys: Array<TKey>,
      preserve: boolean
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    selectRowsByIndexes(
      indexes: Array<number>
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    showColumnChooser(): void;
    state(): any;
    state(state: any): void;
    undeleteRow(rowIndex: number): void;
    updateDimensions(): void;
  }
  module dxDataGrid {
    export type AdaptiveDetailRowPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      AdaptiveDetailRowPreparingInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface AdaptiveDetailRowPreparingInfo {
      readonly formOptions: any;
    }
    export type CellClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly value?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly cellElement: DevExpress.core.DxElement;
      readonly row: Row<TRowData, TKey>;
    };
    export type CellDblClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly value?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly cellElement: DevExpress.core.DxElement;
      readonly row: Row<TRowData, TKey>;
    };
    export type CellHoverChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly eventType: string;
      readonly data: TRowData;
      readonly key: TKey;
      readonly value?: any;
      readonly text: string;
      readonly displayValue?: any;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly rowType: string;
      readonly cellElement: DevExpress.core.DxElement;
      readonly row: Row<TRowData, TKey>;
    };
    export type CellPreparedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly value?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly row: Row<TRowData, TKey>;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
      readonly isNewRow?: boolean;
      readonly cellElement: DevExpress.core.DxElement;
      readonly watch?: Function;
      readonly oldValue?: any;
    };
    export type Column<TRowData = any, TKey = any> = dxDataGridColumn<
      TRowData,
      TKey
    >;
    /**
     * [descr:GridBaseColumn]
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnBase<TRowData = any> {
      /**
       * [descr:GridBaseColumn.alignment]
       */
      alignment?: 'center' | 'left' | 'right';
      /**
       * [descr:GridBaseColumn.allowEditing]
       */
      allowEditing?: boolean;
      /**
       * [descr:GridBaseColumn.allowFiltering]
       */
      allowFiltering?: boolean;
      /**
       * [descr:GridBaseColumn.allowFixing]
       */
      allowFixing?: boolean;
      /**
       * [descr:GridBaseColumn.allowHeaderFiltering]
       */
      allowHeaderFiltering?: boolean;
      /**
       * [descr:GridBaseColumn.allowHiding]
       */
      allowHiding?: boolean;
      /**
       * [descr:GridBaseColumn.allowReordering]
       */
      allowReordering?: boolean;
      /**
       * [descr:GridBaseColumn.allowResizing]
       */
      allowResizing?: boolean;
      /**
       * [descr:GridBaseColumn.allowSearch]
       */
      allowSearch?: boolean;
      /**
       * [descr:GridBaseColumn.allowSorting]
       */
      allowSorting?: boolean;
      /**
       * [descr:GridBaseColumn.calculateCellValue]
       */
      calculateCellValue?: (rowData: TRowData) => any;
      /**
       * [descr:GridBaseColumn.calculateDisplayValue]
       */
      calculateDisplayValue?: string | ((rowData: TRowData) => any);
      /**
       * [descr:GridBaseColumn.calculateFilterExpression]
       */
      calculateFilterExpression?: (
        filterValue: any,
        selectedFilterOperation: string,
        target: string
      ) => string | Array<any> | Function;
      /**
       * [descr:GridBaseColumn.calculateSortValue]
       */
      calculateSortValue?: string | ((rowData: TRowData) => any);
      /**
       * [descr:GridBaseColumn.caption]
       */
      caption?: string;
      /**
       * [descr:GridBaseColumn.cssClass]
       */
      cssClass?: string;
      /**
       * [descr:GridBaseColumn.customizeText]
       */
      customizeText?: (cellInfo: ColumnCustomizeTextArg) => string;
      /**
       * [descr:GridBaseColumn.dataField]
       */
      dataField?: string;
      /**
       * [descr:GridBaseColumn.dataType]
       */
      dataType?:
        | 'string'
        | 'number'
        | 'date'
        | 'boolean'
        | 'object'
        | 'datetime';
      /**
       * [descr:GridBaseColumn.editorOptions]
       */
      editorOptions?: any;
      /**
       * [descr:GridBaseColumn.encodeHtml]
       */
      encodeHtml?: boolean;
      /**
       * [descr:GridBaseColumn.falseText]
       */
      falseText?: string;
      /**
       * [descr:GridBaseColumn.filterOperations]
       */
      filterOperations?: Array<
        | '='
        | '<>'
        | '<'
        | '<='
        | '>'
        | '>='
        | 'contains'
        | 'endswith'
        | 'isblank'
        | 'isnotblank'
        | 'notcontains'
        | 'startswith'
        | 'between'
        | 'anyof'
        | 'noneof'
      >;
      /**
       * [descr:GridBaseColumn.filterType]
       */
      filterType?: 'exclude' | 'include';
      /**
       * [descr:GridBaseColumn.filterValue]
       */
      filterValue?: any;
      /**
       * [descr:GridBaseColumn.filterValues]
       */
      filterValues?: Array<any>;
      /**
       * [descr:GridBaseColumn.fixed]
       */
      fixed?: boolean;
      /**
       * [descr:GridBaseColumn.fixedPosition]
       */
      fixedPosition?: 'left' | 'right';
      /**
       * [descr:GridBaseColumn.formItem]
       */
      formItem?: dxFormSimpleItem;
      /**
       * [descr:GridBaseColumn.format]
       */
      format?: Format;
      /**
       * [descr:GridBaseColumn.headerFilter]
       */
      headerFilter?: ColumnHeaderFilter;
      /**
       * [descr:GridBaseColumn.hidingPriority]
       */
      hidingPriority?: number;
      /**
       * [descr:GridBaseColumn.isBand]
       */
      isBand?: boolean;
      /**
       * [descr:GridBaseColumn.lookup]
       */
      lookup?: ColumnLookup;
      /**
       * [descr:GridBaseColumn.minWidth]
       */
      minWidth?: number;
      /**
       * [descr:GridBaseColumn.name]
       */
      name?: string;
      /**
       * [descr:GridBaseColumn.ownerBand]
       */
      ownerBand?: number;
      /**
       * [descr:GridBaseColumn.renderAsync]
       */
      renderAsync?: boolean;
      /**
       * [descr:GridBaseColumn.selectedFilterOperation]
       */
      selectedFilterOperation?:
        | '<'
        | '<='
        | '<>'
        | '='
        | '>'
        | '>='
        | 'between'
        | 'contains'
        | 'endswith'
        | 'notcontains'
        | 'startswith';
      /**
       * [descr:GridBaseColumn.setCellValue]
       */
      setCellValue?: (
        newData: DevExpress.core.DeepPartial<TRowData>,
        value: any,
        currentRowData: TRowData
      ) => void | PromiseLike<void>;
      /**
       * [descr:GridBaseColumn.showEditorAlways]
       */
      showEditorAlways?: boolean;
      /**
       * [descr:GridBaseColumn.showInColumnChooser]
       */
      showInColumnChooser?: boolean;
      /**
       * [descr:GridBaseColumn.sortIndex]
       */
      sortIndex?: number;
      /**
       * [descr:GridBaseColumn.sortOrder]
       */
      sortOrder?: 'asc' | 'desc';
      /**
       * [descr:GridBaseColumn.sortingMethod]
       */
      sortingMethod?: (value1: any, value2: any) => number;
      /**
       * [descr:GridBaseColumn.trueText]
       */
      trueText?: string;
      /**
       * [descr:GridBaseColumn.validationRules]
       */
      validationRules?: Array<
        | RequiredRule
        | NumericRule
        | RangeRule
        | StringLengthRule
        | CustomRule
        | CompareRule
        | PatternRule
        | EmailRule
        | AsyncRule
      >;
      /**
       * [descr:GridBaseColumn.visible]
       */
      visible?: boolean;
      /**
       * [descr:GridBaseColumn.visibleIndex]
       */
      visibleIndex?: number;
      /**
       * [descr:GridBaseColumn.width]
       */
      width?: number | string;
    }
    export type ColumnButton<
      TRowData = any,
      TKey = any
    > = dxDataGridColumnButton<TRowData, TKey>;
    /**
     * [descr:GridBaseColumnButton]
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnButtonBase {
      /**
       * [descr:GridBaseColumnButton.cssClass]
       */
      cssClass?: string;
      /**
       * [descr:GridBaseColumnButton.hint]
       */
      hint?: string;
      /**
       * [descr:GridBaseColumnButton.icon]
       */
      icon?: string;
      /**
       * [descr:GridBaseColumnButton.text]
       */
      text?: string;
    }
    export type ColumnButtonClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> & {
      row?: Row<TRowData, TKey>;
      column?: Column<TRowData, TKey>;
    };
    export type ColumnButtonTemplateData<TRowData = any, TKey = any> = {
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly data?: TRowData;
      readonly key?: TKey;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly row: Row<TRowData, TKey>;
    };
    export type ColumnCellTemplateData<TRowData = any, TKey = any> = {
      readonly data?: TRowData;
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly value?: any;
      readonly oldValue?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly row: Row<TRowData, TKey>;
      readonly rowType: string;
      readonly watch?: Function;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnChooser {
      /**
       * [descr:GridBaseOptions.columnChooser.allowSearch]
       */
      allowSearch?: boolean;
      /**
       * [descr:GridBaseOptions.columnChooser.emptyPanelText]
       */
      emptyPanelText?: string;
      /**
       * [descr:GridBaseOptions.columnChooser.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:GridBaseOptions.columnChooser.height]
       */
      height?: number;
      /**
       * [descr:GridBaseOptions.columnChooser.mode]
       */
      mode?: 'dragAndDrop' | 'select';
      /**
       * [descr:GridBaseOptions.columnChooser.searchTimeout]
       */
      searchTimeout?: number;
      /**
       * [descr:GridBaseOptions.columnChooser.title]
       */
      title?: string;
      /**
       * [descr:GridBaseOptions.columnChooser.width]
       */
      width?: number;
      /**
       * [descr:GridBaseOptions.columnChooser.sortOrder]
       */
      sortOrder?: 'asc' | 'desc';
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnCustomizeTextArg {
      value?: string | number | Date;
      valueText?: string;
      target?: string;
      groupInterval?: string | number;
    }
    export type ColumnEditCellTemplateData<TRowData = any, TKey = any> = {
      readonly setValue?: any;
      readonly data?: TRowData;
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly value?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly row: Row<TRowData, TKey>;
      readonly rowType: string;
      readonly watch?: Function;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnFixing {
      /**
       * [descr:GridBaseOptions.columnFixing.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:GridBaseOptions.columnFixing.texts]
       */
      texts?: ColumnFixingTexts;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnFixingTexts {
      /**
       * [descr:GridBaseOptions.columnFixing.texts.fix]
       */
      fix?: string;
      /**
       * [descr:GridBaseOptions.columnFixing.texts.leftPosition]
       */
      leftPosition?: string;
      /**
       * [descr:GridBaseOptions.columnFixing.texts.rightPosition]
       */
      rightPosition?: string;
      /**
       * [descr:GridBaseOptions.columnFixing.texts.unfix]
       */
      unfix?: string;
    }
    export type ColumnGroupCellTemplateData<TRowData = any, TKey = any> = {
      readonly data?: GroupData<TRowData>;
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly value?: any;
      readonly text: string;
      readonly displayValue?: any;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly row: Row<GroupData<TRowData>, GroupKey>;
      readonly summaryItems: Array<any>;
      readonly groupContinuesMessage?: string;
      readonly groupContinuedMessage?: string;
    };
    export type ColumnHeaderCellTemplateData<TRowData = any, TKey = any> = {
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnHeaderFilter {
      /**
       * [descr:GridBaseColumn.headerFilter.allowSearch]
       */
      allowSearch?: boolean;
      /**
       * [descr:GridBaseColumn.headerFilter.dataSource]
       */
      dataSource?:
        | DevExpress.ui.dxFilterBuilder.FilterLookupDataSource<any>
        | ((options: {
            component?: any;
            dataSource?: DevExpress.data.DataSource.Options;
          }) => void);
      /**
       * [descr:GridBaseColumn.headerFilter.groupInterval]
       */
      groupInterval?:
        | 'day'
        | 'hour'
        | 'minute'
        | 'month'
        | 'quarter'
        | 'second'
        | 'year'
        | number;
      /**
       * [descr:GridBaseColumn.headerFilter.height]
       */
      height?: number;
      /**
       * [descr:GridBaseColumn.headerFilter.searchMode]
       */
      searchMode?: 'contains' | 'startswith' | 'equals';
      /**
       * [descr:GridBaseColumn.headerFilter.width]
       */
      width?: number;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ColumnLookup {
      /**
       * [descr:GridBaseColumn.lookup.allowClearing]
       */
      allowClearing?: boolean;
      /**
       * [descr:GridBaseColumn.lookup.dataSource]
       */
      dataSource?:
        | DevExpress.ui.dxFilterBuilder.FilterLookupDataSource<any>
        | ((options: {
            data?: any;
            key?: any;
          }) => DevExpress.ui.dxFilterBuilder.FilterLookupDataSource<any>);
      /**
       * [descr:GridBaseColumn.lookup.displayExpr]
       */
      displayExpr?: string | ((data: any) => string);
      /**
       * [descr:GridBaseColumn.lookup.valueExpr]
       */
      valueExpr?: string;
      /**
       * [descr:GridBaseColumn.lookup.calculateCellValue]
       */
      calculateCellValue?: (rowData: any) => any;
    }
    export type ContentReadyEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>>;
    export type ContextMenuPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      items?: Array<any>;
      readonly target: string;
      readonly targetElement: DevExpress.core.DxElement;
      readonly columnIndex: number;
      readonly column?: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly row?: Row<TRowData, TKey>;
    };
    export type CustomSummaryInfo<TRowData = any, TKey = any> = {
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly name?: string;
      readonly summaryProcess: string;
      readonly value?: any;
      totalValue?: any;
      readonly groupIndex?: number;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface DataChangeInfo<TRowData = any, TKey = any> {
      readonly changes: Array<DataChange<TRowData, TKey>>;
    }
    export type DataErrorOccurredEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      DataErrorOccurredInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface DataErrorOccurredInfo {
      readonly error?: Error;
    }
    export type DataRowTemplateData<TRowData = any, TKey = any> = {
      readonly key: TKey;
      readonly data: TRowData;
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly values: Array<any>;
      readonly rowIndex: number;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type DefaultToolbarItemName =
      | 'addRowButton'
      | 'applyFilterButton'
      | 'columnChooserButton'
      | 'exportButton'
      | 'groupPanel'
      | 'revertButton'
      | 'saveButton'
      | 'searchPanel';
    export type DisposingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface DragDropInfo {
      readonly dropInsideItem: boolean;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface DragReorderInfo {
      readonly dropInsideItem: boolean;
      promise?: PromiseLike<void>;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface DragStartEventInfo<
      T extends GridBase<TRowData, TKey>,
      TRowData = any,
      TKey = any
    > {
      readonly component: T;
      readonly event: DevExpress.events.DxEvent;
      itemData?: TRowData;
      readonly itemElement: DevExpress.core.DxElement;
      readonly fromIndex: number;
      readonly fromData?: any;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface dxDataGridSortByGroupSummaryInfoItem {
      /**
       * [descr:dxDataGridOptions.sortByGroupSummaryInfo.groupColumn]
       */
      groupColumn?: string;
      /**
       * [descr:dxDataGridOptions.sortByGroupSummaryInfo.sortOrder]
       */
      sortOrder?: 'asc' | 'desc';
      /**
       * [descr:dxDataGridOptions.sortByGroupSummaryInfo.summaryItem]
       */
      summaryItem?: string | number;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type dxDataGridToolbar = Toolbar;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type dxDataGridToolbarItem = ToolbarItem;
    export type EditCanceledEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      DataChangeInfo<TRowData, TKey>;
    export type EditCancelingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      DataChangeInfo<TRowData, TKey>;
    export type Editing<TRowData = any, TKey = any> = EditingBase<
      TRowData,
      TKey
    > & {
      /**
       * [descr:dxDataGridOptions.editing.allowAdding]
       */
      allowAdding?: boolean;
      /**
       * [descr:dxDataGridOptions.editing.allowDeleting]
       */
      allowDeleting?:
        | boolean
        | ((options: {
            component?: dxDataGrid<TRowData, TKey>;
            row?: Row<TRowData, TKey>;
          }) => boolean);
      /**
       * [descr:dxDataGridOptions.editing.allowUpdating]
       */
      allowUpdating?:
        | boolean
        | ((options: {
            component?: dxDataGrid<TRowData, TKey>;
            row?: Row<TRowData, TKey>;
          }) => boolean);
      /**
       * [descr:dxDataGridOptions.editing.texts]
       */
      texts?: any;
      /**
       * [descr:dxDataGridOptions.editing.newRowPosition]
       */
      newRowPosition?:
        | 'first'
        | 'last'
        | 'pageBottom'
        | 'pageTop'
        | 'viewportBottom'
        | 'viewportTop';
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface EditingBase<TRowData = any, TKey = any> {
      /**
       * [descr:GridBaseOptions.editing.confirmDelete]
       */
      confirmDelete?: boolean;
      /**
       * [descr:GridBaseOptions.editing.changes]
       */
      changes?: Array<DataChange<TRowData, TKey>>;
      /**
       * [descr:GridBaseOptions.editing.editColumnName]
       */
      editColumnName?: string;
      /**
       * [descr:GridBaseOptions.editing.editRowKey]
       */
      editRowKey?: TKey;
      /**
       * [descr:GridBaseOptions.editing.form]
       */
      form?: dxFormOptions;
      /**
       * [descr:GridBaseOptions.editing.mode]
       */
      mode?: 'batch' | 'cell' | 'row' | 'form' | 'popup';
      /**
       * [descr:GridBaseOptions.editing.popup]
       */
      popup?: DevExpress.ui.dxPopup.Properties;
      /**
       * [descr:GridBaseOptions.editing.refreshMode]
       */
      refreshMode?: 'full' | 'reshape' | 'repaint';
      /**
       * [descr:GridBaseOptions.editing.selectTextOnEditStart]
       */
      selectTextOnEditStart?: boolean;
      /**
       * [descr:GridBaseOptions.editing.startEditAction]
       */
      startEditAction?: 'click' | 'dblClick';
      /**
       * [descr:GridBaseOptions.editing.texts]
       */
      texts?: EditingTextsBase;
      /**
       * [descr:GridBaseOptions.editing.useIcons]
       */
      useIcons?: boolean;
    }
    export type EditingStartEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
        readonly data: TRowData;
        readonly key: TKey;
        readonly column?: Column<TRowData, TKey>;
      };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface EditingTextsBase {
      /**
       * [descr:GridBaseOptions.editing.texts.addRow]
       */
      addRow?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.cancelAllChanges]
       */
      cancelAllChanges?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.cancelRowChanges]
       */
      cancelRowChanges?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.confirmDeleteMessage]
       */
      confirmDeleteMessage?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.confirmDeleteTitle]
       */
      confirmDeleteTitle?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.deleteRow]
       */
      deleteRow?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.editRow]
       */
      editRow?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.saveAllChanges]
       */
      saveAllChanges?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.saveRowChanges]
       */
      saveRowChanges?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.undeleteRow]
       */
      undeleteRow?: string;
      /**
       * [descr:GridBaseOptions.editing.texts.validationCancelChanges]
       */
      validationCancelChanges?: string;
    }
    export type EditorPreparedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly parentType: string;
      readonly value?: any;
      readonly setValue?: any;
      readonly updateValueTimeout?: number;
      readonly width?: number;
      readonly disabled: boolean;
      readonly rtlEnabled: boolean;
      readonly editorElement: DevExpress.core.DxElement;
      readonly readOnly: boolean;
      readonly dataField?: string;
      readonly row?: Row<TRowData, TKey>;
    };
    export type EditorPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly parentType: string;
      readonly value?: any;
      readonly setValue?: any;
      readonly updateValueTimeout?: number;
      readonly width?: number;
      readonly disabled: boolean;
      readonly rtlEnabled: boolean;
      cancel: boolean;
      readonly editorElement: DevExpress.core.DxElement;
      readonly readOnly: boolean;
      editorName: string;
      editorOptions: any;
      readonly dataField?: string;
      readonly row?: Row<TRowData, TKey>;
    };
    export type ExcelCellInfo<TRowData = any, TKey = any> = {
      readonly component: dxDataGrid<TRowData, TKey>;
      horizontalAlignment?:
        | 'center'
        | 'centerContinuous'
        | 'distributed'
        | 'fill'
        | 'general'
        | 'justify'
        | 'left'
        | 'right';
      verticalAlignment?:
        | 'bottom'
        | 'center'
        | 'distributed'
        | 'justify'
        | 'top';
      wrapTextEnabled?: boolean;
      backgroundColor?: string;
      fillPatternType?:
        | 'darkDown'
        | 'darkGray'
        | 'darkGrid'
        | 'darkHorizontal'
        | 'darkTrellis'
        | 'darkUp'
        | 'darkVertical'
        | 'gray0625'
        | 'gray125'
        | 'lightDown'
        | 'lightGray'
        | 'lightGrid'
        | 'lightHorizontal'
        | 'lightTrellis'
        | 'lightUp'
        | 'lightVertical'
        | 'mediumGray'
        | 'none'
        | 'solid';
      fillPatternColor?: string;
      font?: DevExpress.exporter.ExcelFont;
      readonly value?: string | number | Date;
      numberFormat?: string;
      gridCell?: DevExpress.excelExporter.DataGridCell;
    };
    export type ExplicitTypes<TRowData, TKey> = {
      AdaptiveDetailRowPreparingEvent: AdaptiveDetailRowPreparingEvent<
        TRowData,
        TKey
      >;
      CellClickEvent: CellClickEvent<TRowData, TKey>;
      CellDblClickEvent: CellDblClickEvent<TRowData, TKey>;
      CellHoverChangedEvent: CellHoverChangedEvent<TRowData, TKey>;
      CellPreparedEvent: CellPreparedEvent<TRowData, TKey>;
      Column: Column<TRowData, TKey>;
      ColumnButton: ColumnButton<TRowData, TKey>;
      ColumnButtonClickEvent: ColumnButtonClickEvent<TRowData, TKey>;
      ColumnButtonTemplateData: ColumnButtonTemplateData<TRowData, TKey>;
      ColumnCellTemplateData: ColumnCellTemplateData<TRowData, TKey>;
      ColumnEditCellTemplateData: ColumnEditCellTemplateData<TRowData, TKey>;
      ColumnGroupCellTemplateData: ColumnGroupCellTemplateData<TRowData, TKey>;
      ColumnHeaderCellTemplateData: ColumnHeaderCellTemplateData<
        TRowData,
        TKey
      >;
      ContentReadyEvent: ContentReadyEvent<TRowData, TKey>;
      ContextMenuPreparingEvent: ContextMenuPreparingEvent<TRowData, TKey>;
      CustomSummaryInfo: CustomSummaryInfo<TRowData, TKey>;
      DataErrorOccurredEvent: DataErrorOccurredEvent<TRowData, TKey>;
      DataRowTemplateData: DataRowTemplateData<TRowData, TKey>;
      DisposingEvent: DisposingEvent<TRowData, TKey>;
      EditCanceledEvent: EditCanceledEvent<TRowData, TKey>;
      EditCancelingEvent: EditCancelingEvent<TRowData, TKey>;
      Editing: Editing<TRowData, TKey>;
      EditingStartEvent: EditingStartEvent<TRowData, TKey>;
      EditorPreparedEvent: EditorPreparedEvent<TRowData, TKey>;
      EditorPreparingEvent: EditorPreparingEvent<TRowData, TKey>;
      ExcelCellInfo: ExcelCellInfo<TRowData, TKey>;
      Export: Export<TRowData, TKey>;
      ExportedEvent: ExportedEvent<TRowData, TKey>;
      ExportingEvent: ExportingEvent<TRowData, TKey>;
      ExportTexts: ExportTexts;
      FileSavingEvent: FileSavingEvent<TRowData, TKey>;
      FocusedCellChangedEvent: FocusedCellChangedEvent<TRowData, TKey>;
      FocusedCellChangingEvent: FocusedCellChangingEvent<TRowData, TKey>;
      FocusedRowChangedEvent: FocusedRowChangedEvent<TRowData, TKey>;
      FocusedRowChangingEvent: FocusedRowChangingEvent<TRowData, TKey>;
      GroupData: GroupData<TRowData>;
      Grouping: Grouping;
      GroupingTexts: GroupingTexts;
      GroupPanel: GroupPanel;
      InitializedEvent: InitializedEvent<TRowData, TKey>;
      InitNewRowEvent: InitNewRowEvent<TRowData, TKey>;
      KeyDownEvent: KeyDownEvent<TRowData, TKey>;
      MasterDetail: MasterDetail<TRowData, TKey>;
      MasterDetailTemplateData: MasterDetailTemplateData<TRowData, TKey>;
      OptionChangedEvent: OptionChangedEvent<TRowData, TKey>;
      Properties: Properties<TRowData, TKey>;
      Row: Row<TRowData, TKey>;
      RowClickEvent: RowClickEvent<TRowData, TKey>;
      RowCollapsedEvent: RowCollapsedEvent<TRowData, TKey>;
      RowCollapsingEvent: RowCollapsingEvent<TRowData, TKey>;
      RowDblClickEvent: RowDblClickEvent<TRowData, TKey>;
      RowDraggingAddEvent: RowDraggingAddEvent<TRowData, TKey>;
      RowDraggingChangeEvent: RowDraggingChangeEvent<TRowData, TKey>;
      RowDraggingEndEvent: RowDraggingEndEvent<TRowData, TKey>;
      RowDraggingMoveEvent: RowDraggingMoveEvent<TRowData, TKey>;
      RowDraggingRemoveEvent: RowDraggingRemoveEvent<TRowData, TKey>;
      RowDraggingReorderEvent: RowDraggingReorderEvent<TRowData, TKey>;
      RowDraggingStartEvent: RowDraggingStartEvent<TRowData, TKey>;
      RowDraggingTemplateData: RowDraggingTemplateData<TRowData>;
      RowExpandedEvent: RowExpandedEvent<TRowData, TKey>;
      RowExpandingEvent: RowExpandingEvent<TRowData, TKey>;
      RowInsertedEvent: RowInsertedEvent<TRowData, TKey>;
      RowInsertingEvent: RowInsertingEvent<TRowData, TKey>;
      RowPreparedEvent: RowPreparedEvent<TRowData, TKey>;
      RowRemovedEvent: RowRemovedEvent<TRowData, TKey>;
      RowRemovingEvent: RowRemovingEvent<TRowData, TKey>;
      RowTemplateData: RowTemplateData<TRowData, TKey>;
      RowUpdatedEvent: RowUpdatedEvent<TRowData, TKey>;
      RowUpdatingEvent: RowUpdatingEvent<TRowData, TKey>;
      RowValidatingEvent: RowValidatingEvent<TRowData, TKey>;
      SavedEvent: SavedEvent<TRowData, TKey>;
      SavingEvent: SavingEvent<TRowData, TKey>;
      Scrolling: Scrolling;
      Selection: Selection;
      SelectionChangedEvent: SelectionChangedEvent<TRowData, TKey>;
      Summary: Summary<TRowData, TKey>;
      SummaryGroupItem: SummaryGroupItem;
      SummaryItemTextInfo: SummaryItemTextInfo;
      SummaryTexts: SummaryTexts;
      SummaryTotalItem: SummaryTotalItem;
      Toolbar: Toolbar;
      ToolbarItem: ToolbarItem;
      ToolbarPreparingEvent: ToolbarPreparingEvent<TRowData, TKey>;
    };
    export type Export<TRowData = any, TKey = any> = {
      /**
       * [descr:dxDataGridOptions.export.allowExportSelectedData]
       */
      allowExportSelectedData?: boolean;
      /**
       * [descr:dxDataGridOptions.export.customizeExcelCell]
       * @deprecated [depNote:dxDataGridOptions.export.customizeExcelCell]
       */
      customizeExcelCell?: (options: ExcelCellInfo<TRowData, TKey>) => void;
      /**
       * [descr:dxDataGridOptions.export.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxDataGridOptions.export.excelFilterEnabled]
       * @deprecated [depNote:dxDataGridOptions.export.excelFilterEnabled]
       */
      excelFilterEnabled?: boolean;
      /**
       * [descr:dxDataGridOptions.export.excelWrapTextEnabled]
       * @deprecated [depNote:dxDataGridOptions.export.excelWrapTextEnabled]
       */
      excelWrapTextEnabled?: boolean;
      /**
       * [descr:dxDataGridOptions.export.fileName]
       * @deprecated [depNote:dxDataGridOptions.export.fileName]
       */
      fileName?: string;
      /**
       * [descr:dxDataGridOptions.export.ignoreExcelErrors]
       * @deprecated [depNote:dxDataGridOptions.export.ignoreExcelErrors]
       */
      ignoreExcelErrors?: boolean;
      /**
       * [descr:dxDataGridOptions.export.proxyUrl]
       * @deprecated [depNote:dxDataGridOptions.export.proxyUrl]
       */
      proxyUrl?: string;
      /**
       * [descr:dxDataGridOptions.export.texts]
       */
      texts?: ExportTexts;
    };
    export type ExportedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>>;
    export type ExportingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
        fileName?: string;
      };
    export type ExportTexts = {
      /**
       * [descr:dxDataGridOptions.export.texts.exportAll]
       */
      exportAll?: string;
      /**
       * [descr:dxDataGridOptions.export.texts.exportSelectedRows]
       */
      exportSelectedRows?: string;
      /**
       * [descr:dxDataGridOptions.export.texts.exportTo]
       */
      exportTo?: string;
    };
    export type FileSavingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable & {
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly element: DevExpress.core.DxElement;
      fileName?: string;
      format?: string;
      readonly data: Blob;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface FilterPanel<
      T extends GridBase<TRowData, TKey>,
      TRowData = any,
      TKey = any
    > {
      /**
       * [descr:GridBaseOptions.filterPanel.customizeText]
       */
      customizeText?: (e: FilterPanelCustomizeTextArg<T>) => string;
      /**
       * [descr:GridBaseOptions.filterPanel.filterEnabled]
       */
      filterEnabled?: boolean;
      /**
       * [descr:GridBaseOptions.filterPanel.texts]
       */
      texts?: FilterPanelTexts;
      /**
       * [descr:GridBaseOptions.filterPanel.visible]
       */
      visible?: boolean;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface FilterPanelCustomizeTextArg<T> {
      readonly component: T;
      readonly filterValue: any;
      readonly text: string;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface FilterPanelTexts {
      /**
       * [descr:GridBaseOptions.filterPanel.texts.clearFilter]
       */
      clearFilter?: string;
      /**
       * [descr:GridBaseOptions.filterPanel.texts.createFilter]
       */
      createFilter?: string;
      /**
       * [descr:GridBaseOptions.filterPanel.texts.filterEnabledHint]
       */
      filterEnabledHint?: string;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface FilterRow {
      /**
       * [descr:GridBaseOptions.filterRow.applyFilter]
       */
      applyFilter?: 'auto' | 'onClick';
      /**
       * [descr:GridBaseOptions.filterRow.applyFilterText]
       */
      applyFilterText?: string;
      /**
       * [descr:GridBaseOptions.filterRow.betweenEndText]
       */
      betweenEndText?: string;
      /**
       * [descr:GridBaseOptions.filterRow.betweenStartText]
       */
      betweenStartText?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions]
       */
      operationDescriptions?: FilterRowOperationDescriptions;
      /**
       * [descr:GridBaseOptions.filterRow.resetOperationText]
       */
      resetOperationText?: string;
      /**
       * [descr:GridBaseOptions.filterRow.showAllText]
       */
      showAllText?: string;
      /**
       * [descr:GridBaseOptions.filterRow.showOperationChooser]
       */
      showOperationChooser?: boolean;
      /**
       * [descr:GridBaseOptions.filterRow.visible]
       */
      visible?: boolean;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface FilterRowOperationDescriptions {
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.between]
       */
      between?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.contains]
       */
      contains?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.endsWith]
       */
      endsWith?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.equal]
       */
      equal?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.greaterThan]
       */
      greaterThan?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.greaterThanOrEqual]
       */
      greaterThanOrEqual?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.lessThan]
       */
      lessThan?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.lessThanOrEqual]
       */
      lessThanOrEqual?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.notContains]
       */
      notContains?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.notEqual]
       */
      notEqual?: string;
      /**
       * [descr:GridBaseOptions.filterRow.operationDescriptions.startsWith]
       */
      startsWith?: string;
    }
    export type FocusedCellChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly cellElement: DevExpress.core.DxElement;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly row?: Row<TRowData, TKey>;
      readonly column?: Column<TRowData, TKey>;
    };
    export type FocusedCellChangingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> & {
        readonly cellElement: DevExpress.core.DxElement;
        readonly prevColumnIndex: number;
        readonly prevRowIndex: number;
        newColumnIndex: number;
        newRowIndex: number;
        readonly rows: Array<Row<TRowData, TKey>>;
        readonly columns: Array<Column<TRowData, TKey>>;
        isHighlighted: boolean;
      };
    export type FocusedRowChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly rowElement: DevExpress.core.DxElement;
      readonly rowIndex: number;
      readonly row?: Row<TRowData, TKey>;
    };
    export type FocusedRowChangingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> & {
        readonly rowElement: DevExpress.core.DxElement;
        readonly prevRowIndex: number;
        newRowIndex: number;
        readonly rows: Array<Row<TRowData, TKey>>;
      };
    export type GroupData<TRowData> = {
      key: any;
      items: Array<TRowData> | Array<GroupData<TRowData>> | null;
      /**
       * @deprecated Attention! This property is for internal purposes only.
       */
      collapsedItems?: Array<TRowData> | Array<GroupData<TRowData>>;
      /**
       * @deprecated Attention! This property is for internal purposes only.
       */
      aggregates?: Array<any>;
      /**
       * @deprecated Attention! This property is for internal purposes only.
       */
      summary?: Array<any>;
      /**
       * @deprecated Attention! This property is for internal purposes only.
       */
      isContinuation?: boolean;
      /**
       * @deprecated Attention! This property is for internal purposes only.
       */
      isContinuationOnNextPage?: boolean;
    };
    export type Grouping = {
      /**
       * [descr:dxDataGridOptions.grouping.allowCollapsing]
       */
      allowCollapsing?: boolean;
      /**
       * [descr:dxDataGridOptions.grouping.autoExpandAll]
       */
      autoExpandAll?: boolean;
      /**
       * [descr:dxDataGridOptions.grouping.contextMenuEnabled]
       */
      contextMenuEnabled?: boolean;
      /**
       * [descr:dxDataGridOptions.grouping.expandMode]
       */
      expandMode?: 'buttonClick' | 'rowClick';
      /**
       * [descr:dxDataGridOptions.grouping.texts]
       */
      texts?: GroupingTexts;
    };
    export type GroupingTexts = {
      /**
       * [descr:dxDataGridOptions.grouping.texts.groupByThisColumn]
       */
      groupByThisColumn?: string;
      /**
       * [descr:dxDataGridOptions.grouping.texts.groupContinuedMessage]
       */
      groupContinuedMessage?: string;
      /**
       * [descr:dxDataGridOptions.grouping.texts.groupContinuesMessage]
       */
      groupContinuesMessage?: string;
      /**
       * [descr:dxDataGridOptions.grouping.texts.ungroup]
       */
      ungroup?: string;
      /**
       * [descr:dxDataGridOptions.grouping.texts.ungroupAll]
       */
      ungroupAll?: string;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type GroupKey = any[];
    export type GroupPanel = {
      /**
       * [descr:dxDataGridOptions.groupPanel.allowColumnDragging]
       */
      allowColumnDragging?: boolean;
      /**
       * [descr:dxDataGridOptions.groupPanel.emptyPanelText]
       */
      emptyPanelText?: string;
      /**
       * [descr:dxDataGridOptions.groupPanel.visible]
       */
      visible?: boolean | 'auto';
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface HeaderFilter {
      /**
       * [descr:GridBaseOptions.headerFilter.allowSearch]
       */
      allowSearch?: boolean;
      /**
       * [descr:GridBaseOptions.headerFilter.height]
       */
      height?: number;
      /**
       * [descr:GridBaseOptions.headerFilter.searchTimeout]
       */
      searchTimeout?: number;
      /**
       * [descr:GridBaseOptions.headerFilter.texts]
       */
      texts?: HeaderFilterTexts;
      /**
       * [descr:GridBaseOptions.headerFilter.visible]
       */
      visible?: boolean;
      /**
       * [descr:GridBaseOptions.headerFilter.width]
       */
      width?: number;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface HeaderFilterTexts {
      /**
       * [descr:GridBaseOptions.headerFilter.texts.cancel]
       */
      cancel?: string;
      /**
       * [descr:GridBaseOptions.headerFilter.texts.emptyValue]
       */
      emptyValue?: string;
      /**
       * [descr:GridBaseOptions.headerFilter.texts.ok]
       */
      ok?: string;
    }
    export type InitializedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxDataGrid<TRowData, TKey>>;
    export type InitNewRowEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      NewRowInfo<TRowData>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface KeyboardNavigation {
      /**
       * [descr:GridBaseOptions.keyboardNavigation.editOnKeyPress]
       */
      editOnKeyPress?: boolean;
      /**
       * [descr:GridBaseOptions.keyboardNavigation.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:GridBaseOptions.keyboardNavigation.enterKeyAction]
       */
      enterKeyAction?: 'startEdit' | 'moveFocus';
      /**
       * [descr:GridBaseOptions.keyboardNavigation.enterKeyDirection]
       */
      enterKeyDirection?: 'none' | 'column' | 'row';
    }
    export type KeyDownEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> &
      KeyDownInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface KeyDownInfo {
      handled: boolean;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface LoadPanel {
      /**
       * [descr:GridBaseOptions.loadPanel.enabled]
       */
      enabled?: boolean | 'auto';
      /**
       * [descr:GridBaseOptions.loadPanel.height]
       */
      height?: number;
      /**
       * [descr:GridBaseOptions.loadPanel.indicatorSrc]
       */
      indicatorSrc?: string;
      /**
       * [descr:GridBaseOptions.loadPanel.shading]
       */
      shading?: boolean;
      /**
       * [descr:GridBaseOptions.loadPanel.shadingColor]
       */
      shadingColor?: string;
      /**
       * [descr:GridBaseOptions.loadPanel.showIndicator]
       */
      showIndicator?: boolean;
      /**
       * [descr:GridBaseOptions.loadPanel.showPane]
       */
      showPane?: boolean;
      /**
       * [descr:GridBaseOptions.loadPanel.text]
       */
      text?: string;
      /**
       * [descr:GridBaseOptions.loadPanel.width]
       */
      width?: number;
    }
    export type MasterDetail<TRowData = any, TKey = any> = {
      /**
       * [descr:dxDataGridOptions.masterDetail.autoExpandAll]
       */
      autoExpandAll?: boolean;
      /**
       * [descr:dxDataGridOptions.masterDetail.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxDataGridOptions.masterDetail.template]
       */
      template?:
        | DevExpress.core.template
        | ((
            detailElement: DevExpress.core.DxElement,
            detailInfo: MasterDetailTemplateData<TRowData, TKey>
          ) => any);
    };
    export type MasterDetailTemplateData<TRowData = any, TKey = any> = {
      readonly key: TKey;
      readonly data: TRowData;
      readonly watch?: Function;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface NewRowInfo<TRowData = any> {
      data: TRowData;
      promise?: PromiseLike<void>;
    }
    export type OptionChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface Pager {
      /**
       * [descr:GridBaseOptions.pager.allowedPageSizes]
       */
      allowedPageSizes?: Array<number | 'all'> | 'auto';
      /**
       * [descr:GridBaseOptions.pager.displayMode]
       */
      displayMode?: 'adaptive' | 'compact' | 'full';
      /**
       * [descr:GridBaseOptions.pager.infoText]
       */
      infoText?: string;
      /**
       * [descr:GridBaseOptions.pager.showInfo]
       */
      showInfo?: boolean;
      /**
       * [descr:GridBaseOptions.pager.showNavigationButtons]
       */
      showNavigationButtons?: boolean;
      /**
       * [descr:GridBaseOptions.pager.showPageSizeSelector]
       */
      showPageSizeSelector?: boolean;
      /**
       * [descr:GridBaseOptions.pager.visible]
       */
      visible?: boolean | 'auto';
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface PagingBase {
      /**
       * [descr:GridBaseOptions.paging.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:GridBaseOptions.paging.pageIndex]
       */
      pageIndex?: number;
      /**
       * [descr:GridBaseOptions.paging.pageSize]
       */
      pageSize?: number;
    }
    export type Properties<TRowData = any, TKey = any> = dxDataGridOptions<
      TRowData,
      TKey
    >;
    /**
     * [descr:dxDataGridRowObject]
     */
    export type Row<TRowData = any, TKey = any> = {
      /**
       * [descr:dxDataGridRowObject.data]
       */
      readonly data: TRowData;
      /**
       * [descr:dxDataGridRowObject.groupIndex]
       */
      readonly groupIndex?: number;
      /**
       * [descr:dxDataGridRowObject.isEditing]
       */
      readonly isEditing?: boolean;
      /**
       * [descr:dxDataGridRowObject.isExpanded]
       */
      readonly isExpanded?: boolean;
      /**
       * [descr:dxDataGridRowObject.isNewRow]
       */
      readonly isNewRow?: boolean;
      /**
       * [descr:dxDataGridRowObject.isSelected]
       */
      readonly isSelected?: boolean;
      /**
       * [descr:dxDataGridRowObject.key]
       */
      readonly key: TKey;
      /**
       * [descr:dxDataGridRowObject.rowIndex]
       */
      readonly rowIndex: number;
      /**
       * [descr:dxDataGridRowObject.rowType]
       */
      readonly rowType: string;
      /**
       * [descr:dxDataGridRowObject.values]
       */
      readonly values: Array<any>;
    };
    export type RowClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly values: Array<any>;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
      readonly isNewRow?: boolean;
      readonly groupIndex?: number;
      readonly rowElement: DevExpress.core.DxElement;
      readonly handled: boolean;
    };
    export type RowCollapsedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowKeyInfo<TKey>;
    export type RowCollapsingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowKeyInfo<TKey>;
    export type RowDblClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly values: Array<any>;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
      readonly isNewRow?: boolean;
      readonly groupIndex?: number;
      readonly rowElement: DevExpress.core.DxElement;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowDragging<
      T extends GridBase<TRowData, TKey>,
      TRowData = any,
      TKey = any
    > {
      /**
       * [descr:GridBaseOptions.rowDragging.allowDropInsideItem]
       */
      allowDropInsideItem?: boolean;
      /**
       * [descr:GridBaseOptions.rowDragging.allowReordering]
       */
      allowReordering?: boolean;
      /**
       * [descr:GridBaseOptions.rowDragging.autoScroll]
       */
      autoScroll?: boolean;
      /**
       * [descr:GridBaseOptions.rowDragging.boundary]
       */
      boundary?: string | DevExpress.core.UserDefinedElement;
      /**
       * [descr:GridBaseOptions.rowDragging.container]
       */
      container?: string | DevExpress.core.UserDefinedElement;
      /**
       * [descr:GridBaseOptions.rowDragging.cursorOffset]
       */
      cursorOffset?:
        | string
        | {
            /**
             * [descr:GridBaseOptions.rowDragging.cursorOffset.x]
             */
            x?: number;
            /**
             * [descr:GridBaseOptions.rowDragging.cursorOffset.y]
             */
            y?: number;
          };
      /**
       * [descr:GridBaseOptions.rowDragging.data]
       */
      data?: any;
      /**
       * [descr:GridBaseOptions.rowDragging.dragDirection]
       */
      dragDirection?: 'both' | 'horizontal' | 'vertical';
      /**
       * [descr:GridBaseOptions.rowDragging.dragTemplate]
       */
      dragTemplate?:
        | DevExpress.core.template
        | ((
            dragInfo: RowDraggingTemplateData<TRowData>,
            containerElement: DevExpress.core.DxElement
          ) => string | DevExpress.core.UserDefinedElement);
      /**
       * [descr:GridBaseOptions.rowDragging.dropFeedbackMode]
       */
      dropFeedbackMode?: 'push' | 'indicate';
      /**
       * [descr:GridBaseOptions.rowDragging.filter]
       */
      filter?: string;
      /**
       * [descr:GridBaseOptions.rowDragging.group]
       */
      group?: string;
      /**
       * [descr:GridBaseOptions.rowDragging.handle]
       */
      handle?: string;
      /**
       * [descr:GridBaseOptions.rowDragging.onAdd]
       */
      onAdd?: (
        e: RowDraggingEventInfo<T, TRowData, TKey> & DragDropInfo
      ) => void;
      /**
       * [descr:GridBaseOptions.rowDragging.onDragChange]
       */
      onDragChange?: (
        e: DevExpress.events.Cancelable &
          RowDraggingEventInfo<T, TRowData, TKey> &
          DragDropInfo
      ) => void;
      /**
       * [descr:GridBaseOptions.rowDragging.onDragEnd]
       */
      onDragEnd?: (
        e: DevExpress.events.Cancelable &
          RowDraggingEventInfo<T, TRowData, TKey> &
          DragDropInfo
      ) => void;
      /**
       * [descr:GridBaseOptions.rowDragging.onDragMove]
       */
      onDragMove?: (
        e: DevExpress.events.Cancelable &
          RowDraggingEventInfo<T, TRowData, TKey> &
          DragDropInfo
      ) => void;
      /**
       * [descr:GridBaseOptions.rowDragging.onDragStart]
       */
      onDragStart?: (
        e: DevExpress.events.Cancelable & DragStartEventInfo<T, TRowData, TKey>
      ) => void;
      /**
       * [descr:GridBaseOptions.rowDragging.onRemove]
       */
      onRemove?: (e: RowDraggingEventInfo<T, TRowData, TKey>) => void;
      /**
       * [descr:GridBaseOptions.rowDragging.onReorder]
       */
      onReorder?: (
        e: RowDraggingEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey> &
          DragReorderInfo
      ) => void;
      /**
       * [descr:GridBaseOptions.rowDragging.scrollSensitivity]
       */
      scrollSensitivity?: number;
      /**
       * [descr:GridBaseOptions.rowDragging.scrollSpeed]
       */
      scrollSpeed?: number;
      /**
       * [descr:GridBaseOptions.rowDragging.showDragIcons]
       */
      showDragIcons?: boolean;
    }
    export type RowDraggingAddEvent<
      TRowData = any,
      TKey = any
    > = RowDraggingEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey> &
      DragDropInfo;
    export type RowDraggingChangeEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      RowDraggingEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey> &
      DragDropInfo;
    export type RowDraggingEndEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      RowDraggingEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey> &
      DragDropInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowDraggingEventInfo<
      T extends GridBase<TRowData, TKey>,
      TRowData = any,
      TKey = any
    > {
      readonly component: T;
      readonly event: DevExpress.events.DxEvent;
      readonly itemData?: TRowData;
      readonly itemElement: DevExpress.core.DxElement;
      readonly fromIndex: number;
      readonly toIndex: number;
      readonly fromComponent: dxSortable | dxDraggable;
      readonly toComponent: dxSortable | dxDraggable;
      readonly fromData?: any;
      readonly toData?: any;
    }
    export type RowDraggingMoveEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      RowDraggingEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey> &
      DragDropInfo;
    export type RowDraggingRemoveEvent<
      TRowData = any,
      TKey = any
    > = RowDraggingEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey>;
    export type RowDraggingReorderEvent<
      TRowData = any,
      TKey = any
    > = RowDraggingEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey> &
      DragReorderInfo;
    export type RowDraggingStartEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DragStartEventInfo<dxDataGrid<TRowData, TKey>, TRowData, TKey>;
    export type RowDraggingTemplateData<TRowData = any> =
      RowDraggingTemplateDataModel<TRowData>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowDraggingTemplateDataModel<TRowData = any> {
      readonly itemData: TRowData;
      readonly itemElement: DevExpress.core.DxElement;
    }
    export type RowExpandedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowKeyInfo<TKey>;
    export type RowExpandingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowKeyInfo<TKey>;
    export type RowInsertedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowInsertedInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowInsertedInfo<TRowData = any, TKey = any> {
      readonly data: TRowData;
      readonly key: TKey;
      readonly error: Error;
    }
    export type RowInsertingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowInsertingInfo<TRowData>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowInsertingInfo<TRowData = any> {
      data: TRowData;
      cancel: boolean | PromiseLike<void>;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowKeyInfo<TKey = any> {
      readonly key: TKey;
    }
    export type RowPreparedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly values: Array<any>;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly groupIndex?: number;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
      readonly isNewRow?: boolean;
      readonly rowElement: DevExpress.core.DxElement;
    };
    export type RowRemovedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowRemovedInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowRemovedInfo<TRowData = any, TKey = any> {
      readonly data: TRowData;
      readonly key: TKey;
      readonly error: Error;
    }
    export type RowRemovingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowRemovingInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowRemovingInfo<TRowData = any, TKey = any> {
      readonly data: TRowData;
      readonly key: TKey;
      cancel: boolean | PromiseLike<void>;
    }
    export type RowTemplateData<TRowData = any, TKey = any> = {
      readonly key: TKey;
      readonly data: TRowData;
      readonly component: dxDataGrid<TRowData, TKey>;
      readonly values: Array<any>;
      readonly rowIndex: number;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly isSelected?: boolean;
      readonly rowType: string;
      readonly groupIndex?: number;
      readonly isExpanded?: boolean;
    };
    export type RowUpdatedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowUpdatedInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowUpdatedInfo<TRowData = any, TKey = any> {
      readonly data: TRowData;
      readonly key: TKey;
      readonly error: Error;
    }
    export type RowUpdatingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowUpdatingInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowUpdatingInfo<TRowData = any, TKey = any> {
      readonly oldData: TRowData;
      newData: DevExpress.core.DeepPartial<TRowData>;
      readonly key: TKey;
      cancel: boolean | PromiseLike<void>;
    }
    export type RowValidatingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      RowValidatingInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface RowValidatingInfo<TRowData = any, TKey = any> {
      readonly brokenRules: Array<
        | RequiredRule
        | NumericRule
        | RangeRule
        | StringLengthRule
        | CustomRule
        | CompareRule
        | PatternRule
        | EmailRule
        | AsyncRule
      >;
      isValid: boolean;
      readonly key: TKey;
      readonly newData: DevExpress.core.DeepPartial<TRowData>;
      readonly oldData: TRowData;
      errorText: string;
      promise?: PromiseLike<void>;
    }
    export type SavedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      DataChangeInfo<TRowData, TKey>;
    export type SavingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      SavingInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface SavingInfo<TRowData = any, TKey = any> {
      changes: Array<DataChange<TRowData, TKey>>;
      promise?: PromiseLike<void>;
      cancel: boolean;
    }
    export type Scrolling = ScrollingBase & {
      /**
       * [descr:dxDataGridOptions.scrolling.mode]
       */
      mode?: 'infinite' | 'standard' | 'virtual';
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ScrollingBase {
      /**
       * [descr:GridBaseOptions.scrolling.columnRenderingMode]
       */
      columnRenderingMode?: 'standard' | 'virtual';
      /**
       * [descr:GridBaseOptions.scrolling.preloadEnabled]
       */
      preloadEnabled?: boolean;
      /**
       * [descr:GridBaseOptions.scrolling.rowRenderingMode]
       */
      rowRenderingMode?: 'standard' | 'virtual';
      /**
       * [descr:GridBaseOptions.scrolling.scrollByContent]
       */
      scrollByContent?: boolean;
      /**
       * [descr:GridBaseOptions.scrolling.scrollByThumb]
       */
      scrollByThumb?: boolean;
      /**
       * [descr:GridBaseOptions.scrolling.showScrollbar]
       */
      showScrollbar?: 'always' | 'never' | 'onHover' | 'onScroll';
      /**
       * [descr:GridBaseOptions.scrolling.useNative]
       */
      useNative?: boolean | 'auto';
      /**
       * [descr:GridBaseOptions.scrolling.renderAsync]
       */
      renderAsync?: boolean;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface SearchPanel {
      /**
       * [descr:GridBaseOptions.searchPanel.highlightCaseSensitive]
       */
      highlightCaseSensitive?: boolean;
      /**
       * [descr:GridBaseOptions.searchPanel.highlightSearchText]
       */
      highlightSearchText?: boolean;
      /**
       * [descr:GridBaseOptions.searchPanel.placeholder]
       */
      placeholder?: string;
      /**
       * [descr:GridBaseOptions.searchPanel.searchVisibleColumnsOnly]
       */
      searchVisibleColumnsOnly?: boolean;
      /**
       * [descr:GridBaseOptions.searchPanel.text]
       */
      text?: string;
      /**
       * [descr:GridBaseOptions.searchPanel.visible]
       */
      visible?: boolean;
      /**
       * [descr:GridBaseOptions.searchPanel.width]
       */
      width?: number;
    }
    export type Selection = SelectionBase & {
      /**
       * [descr:dxDataGridOptions.selection.deferred]
       */
      deferred?: boolean;
      /**
       * [descr:dxDataGridOptions.selection.selectAllMode]
       */
      selectAllMode?: 'allPages' | 'page';
      /**
       * [descr:dxDataGridOptions.selection.showCheckBoxesMode]
       */
      showCheckBoxesMode?: 'always' | 'none' | 'onClick' | 'onLongTap';
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface SelectionBase {
      /**
       * [descr:GridBaseOptions.selection.allowSelectAll]
       */
      allowSelectAll?: boolean;
      /**
       * [descr:GridBaseOptions.selection.mode]
       */
      mode?: 'multiple' | 'none' | 'single';
    }
    export type SelectionChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      SelectionChangedInfo<TRowData, TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface SelectionChangedInfo<TRowData = any, TKey = any> {
      readonly currentSelectedRowKeys: Array<TKey>;
      readonly currentDeselectedRowKeys: Array<TKey>;
      readonly selectedRowKeys: Array<TKey>;
      readonly selectedRowsData: Array<TRowData>;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface Sorting {
      /**
       * [descr:GridBaseOptions.sorting.ascendingText]
       */
      ascendingText?: string;
      /**
       * [descr:GridBaseOptions.sorting.clearText]
       */
      clearText?: string;
      /**
       * [descr:GridBaseOptions.sorting.descendingText]
       */
      descendingText?: string;
      /**
       * [descr:GridBaseOptions.sorting.mode]
       */
      mode?: 'multiple' | 'none' | 'single';
      /**
       * [descr:GridBaseOptions.sorting.showSortIndexes]
       */
      showSortIndexes?: boolean;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface StateStoring {
      /**
       * [descr:GridBaseOptions.stateStoring.customLoad]
       */
      customLoad?: () => PromiseLike<any>;
      /**
       * [descr:GridBaseOptions.stateStoring.customSave]
       */
      customSave?: (gridState: any) => any;
      /**
       * [descr:GridBaseOptions.stateStoring.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:GridBaseOptions.stateStoring.savingTimeout]
       */
      savingTimeout?: number;
      /**
       * [descr:GridBaseOptions.stateStoring.storageKey]
       */
      storageKey?: string;
      /**
       * [descr:GridBaseOptions.stateStoring.type]
       */
      type?: 'custom' | 'localStorage' | 'sessionStorage';
    }
    export type Summary<TRowData = any, TKey = any> = {
      /**
       * [descr:dxDataGridOptions.summary.calculateCustomSummary]
       */
      calculateCustomSummary?: (
        options: CustomSummaryInfo<TRowData, TKey>
      ) => void;
      /**
       * [descr:dxDataGridOptions.summary.groupItems]
       */
      groupItems?: Array<SummaryGroupItem>;
      /**
       * [descr:dxDataGridOptions.summary.recalculateWhileEditing]
       */
      recalculateWhileEditing?: boolean;
      /**
       * [descr:dxDataGridOptions.summary.skipEmptyValues]
       */
      skipEmptyValues?: boolean;
      /**
       * [descr:dxDataGridOptions.summary.texts]
       */
      texts?: SummaryTexts;
      /**
       * [descr:dxDataGridOptions.summary.totalItems]
       */
      totalItems?: Array<SummaryTotalItem>;
    };
    export type SummaryGroupItem = {
      /**
       * [descr:dxDataGridOptions.summary.groupItems.alignByColumn]
       */
      alignByColumn?: boolean;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.column]
       */
      column?: string;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.customizeText]
       */
      customizeText?: (itemInfo: SummaryItemTextInfo) => string;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.displayFormat]
       */
      displayFormat?: string;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.name]
       */
      name?: string;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.showInColumn]
       */
      showInColumn?: string;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.showInGroupFooter]
       */
      showInGroupFooter?: boolean;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.skipEmptyValues]
       */
      skipEmptyValues?: boolean;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.summaryType]
       */
      summaryType?: 'avg' | 'count' | 'custom' | 'max' | 'min' | 'sum' | string;
      /**
       * [descr:dxDataGridOptions.summary.groupItems.valueFormat]
       */
      valueFormat?: Format;
    };
    export type SummaryItemTextInfo = {
      readonly value?: string | number | Date;
      readonly valueText: string;
    };
    export type SummaryTexts = {
      /**
       * [descr:dxDataGridOptions.summary.texts.avg]
       */
      avg?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.avgOtherColumn]
       */
      avgOtherColumn?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.count]
       */
      count?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.max]
       */
      max?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.maxOtherColumn]
       */
      maxOtherColumn?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.min]
       */
      min?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.minOtherColumn]
       */
      minOtherColumn?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.sum]
       */
      sum?: string;
      /**
       * [descr:dxDataGridOptions.summary.texts.sumOtherColumn]
       */
      sumOtherColumn?: string;
    };
    export type SummaryTotalItem = {
      /**
       * [descr:dxDataGridOptions.summary.totalItems.alignment]
       */
      alignment?: 'center' | 'left' | 'right';
      /**
       * [descr:dxDataGridOptions.summary.totalItems.column]
       */
      column?: string;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.cssClass]
       */
      cssClass?: string;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.customizeText]
       */
      customizeText?: (itemInfo: SummaryItemTextInfo) => string;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.displayFormat]
       */
      displayFormat?: string;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.name]
       */
      name?: string;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.showInColumn]
       */
      showInColumn?: string;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.skipEmptyValues]
       */
      skipEmptyValues?: boolean;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.summaryType]
       */
      summaryType?: 'avg' | 'count' | 'custom' | 'max' | 'min' | 'sum' | string;
      /**
       * [descr:dxDataGridOptions.summary.totalItems.valueFormat]
       */
      valueFormat?: Format;
    };
    export type ToolbarPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxDataGrid<TRowData, TKey>> &
      ToolbarPreparingInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ToolbarPreparingInfo {
      toolbarOptions: dxToolbarOptions;
    }
  }
  /**
   * @deprecated Use the DevExpress.ui.dxDataGrid.Column type instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDataGridColumn<TRowData = any, TKey = any>
    extends DevExpress.ui.dxDataGrid.ColumnBase<TRowData> {
    /**
     * [descr:dxDataGridColumn.allowExporting]
     */
    allowExporting?: boolean;
    /**
     * [descr:dxDataGridColumn.allowGrouping]
     */
    allowGrouping?: boolean;
    /**
     * [descr:dxDataGridColumn.autoExpandGroup]
     */
    autoExpandGroup?: boolean;
    /**
     * [descr:dxDataGridColumn.buttons]
     */
    buttons?: Array<
      | 'cancel'
      | 'delete'
      | 'edit'
      | 'save'
      | 'undelete'
      | DevExpress.ui.dxDataGrid.ColumnButton<TRowData, TKey>
    >;
    /**
     * [descr:dxDataGridColumn.calculateGroupValue]
     */
    calculateGroupValue?: string | ((rowData: TRowData) => any);
    /**
     * [descr:dxDataGridColumn.cellTemplate]
     */
    cellTemplate?:
      | DevExpress.core.template
      | ((
          cellElement: DevExpress.core.DxElement,
          cellInfo: DevExpress.ui.dxDataGrid.ColumnCellTemplateData<
            TRowData,
            TKey
          >
        ) => any);
    /**
     * [descr:dxDataGridColumn.columns]
     */
    columns?: Array<DevExpress.ui.dxDataGrid.Column<TRowData, TKey> | string>;
    /**
     * [descr:dxDataGridColumn.editCellTemplate]
     */
    editCellTemplate?:
      | DevExpress.core.template
      | ((
          cellElement: DevExpress.core.DxElement,
          cellInfo: DevExpress.ui.dxDataGrid.ColumnEditCellTemplateData<
            TRowData,
            TKey
          >
        ) => any);
    /**
     * [descr:dxDataGridColumn.groupCellTemplate]
     */
    groupCellTemplate?:
      | DevExpress.core.template
      | ((
          cellElement: DevExpress.core.DxElement,
          cellInfo: DevExpress.ui.dxDataGrid.ColumnGroupCellTemplateData<
            TRowData,
            TKey
          >
        ) => any);
    /**
     * [descr:dxDataGridColumn.groupIndex]
     */
    groupIndex?: number;
    /**
     * [descr:dxDataGridColumn.headerCellTemplate]
     */
    headerCellTemplate?:
      | DevExpress.core.template
      | ((
          columnHeader: DevExpress.core.DxElement,
          headerInfo: DevExpress.ui.dxDataGrid.ColumnHeaderCellTemplateData<
            TRowData,
            TKey
          >
        ) => any);
    /**
     * [descr:dxDataGridColumn.showWhenGrouped]
     */
    showWhenGrouped?: boolean;
    /**
     * [descr:dxDataGridColumn.type]
     */
    type?:
      | 'adaptive'
      | 'buttons'
      | 'detailExpand'
      | 'groupExpand'
      | 'selection'
      | 'drag';
  }
  /**
   * @deprecated Use the DataGrid's ColumnButton type instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDataGridColumnButton<TRowData = any, TKey = any>
    extends DevExpress.ui.dxDataGrid.ColumnButtonBase {
    /**
     * [descr:dxDataGridColumnButton.name]
     */
    name?: 'cancel' | 'delete' | 'edit' | 'save' | 'undelete' | string;
    /**
     * [descr:dxDataGridColumnButton.onClick]
     */
    onClick?: (
      e: DevExpress.ui.dxDataGrid.ColumnButtonClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridColumnButton.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          cellElement: DevExpress.core.DxElement,
          cellInfo: DevExpress.ui.dxDataGrid.ColumnButtonTemplateData<
            TRowData,
            TKey
          >
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDataGridColumnButton.visible]
     */
    visible?:
      | boolean
      | ((options: {
          component?: dxDataGrid<TRowData, TKey>;
          row?: DevExpress.ui.dxDataGrid.Row<TRowData, TKey>;
          column?: DevExpress.ui.dxDataGrid.Column<TRowData, TKey>;
        }) => boolean);
    /**
     * [descr:dxDataGridColumnButton.disabled]
     */
    disabled?:
      | boolean
      | ((options: {
          component?: dxDataGrid<TRowData, TKey>;
          row?: DevExpress.ui.dxDataGrid.Row<TRowData, TKey>;
          column?: DevExpress.ui.dxDataGrid.Column<TRowData, TKey>;
        }) => boolean);
  }
  /**
   * @deprecated Use DevExpress.ui.dxDataGrid.Editing instead
   */
  export type dxDataGridEditing<
    TRowData,
    TKey = any
  > = DevExpress.ui.dxDataGrid.Editing<TRowData, TKey>;
  /**
   * @deprecated use Properties instead
   */
  export interface dxDataGridOptions<TRowData = any, TKey = any>
    extends GridBaseOptions<dxDataGrid<TRowData, TKey>, TRowData, TKey> {
    /**
     * [descr:dxDataGridOptions.columns]
     */
    columns?: Array<DevExpress.ui.dxDataGrid.Column<TRowData, TKey> | string>;
    /**
     * [descr:dxDataGridOptions.customizeColumns]
     */
    customizeColumns?: (
      columns: Array<DevExpress.ui.dxDataGrid.Column<TRowData, TKey>>
    ) => void;
    /**
     * [descr:dxDataGridOptions.customizeExportData]
     * @deprecated [depNote:dxDataGridOptions.customizeExportData]
     */
    customizeExportData?: (
      columns: Array<DevExpress.ui.dxDataGrid.Column<TRowData, TKey>>,
      rows: Array<DevExpress.ui.dxDataGrid.Row<TRowData, TKey>>
    ) => void;
    /**
     * [descr:dxDataGridOptions.editing]
     */
    editing?: DevExpress.ui.dxDataGrid.Editing<TRowData, TKey>;
    /**
     * [descr:dxDataGridOptions.export]
     */
    export?: DevExpress.ui.dxDataGrid.Export<TRowData, TKey>;
    /**
     * [descr:dxDataGridOptions.groupPanel]
     */
    groupPanel?: DevExpress.ui.dxDataGrid.GroupPanel;
    /**
     * [descr:dxDataGridOptions.grouping]
     */
    grouping?: DevExpress.ui.dxDataGrid.Grouping;
    /**
     * [descr:dxDataGridOptions.keyExpr]
     */
    keyExpr?: string | Array<string>;
    /**
     * [descr:dxDataGridOptions.masterDetail]
     */
    masterDetail?: DevExpress.ui.dxDataGrid.MasterDetail<TRowData, TKey>;
    /**
     * [descr:dxDataGridOptions.onCellClick]
     */
    onCellClick?: (
      e: DevExpress.ui.dxDataGrid.CellClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onCellDblClick]
     */
    onCellDblClick?: (
      e: DevExpress.ui.dxDataGrid.CellDblClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onCellHoverChanged]
     */
    onCellHoverChanged?: (
      e: DevExpress.ui.dxDataGrid.CellHoverChangedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onCellPrepared]
     */
    onCellPrepared?: (
      e: DevExpress.ui.dxDataGrid.CellPreparedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onContextMenuPreparing]
     */
    onContextMenuPreparing?: (
      e: DevExpress.ui.dxDataGrid.ContextMenuPreparingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onEditingStart]
     */
    onEditingStart?: (
      e: DevExpress.ui.dxDataGrid.EditingStartEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onEditorPrepared]
     */
    onEditorPrepared?: (
      options: DevExpress.ui.dxDataGrid.EditorPreparedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onEditorPreparing]
     */
    onEditorPreparing?: (
      e: DevExpress.ui.dxDataGrid.EditorPreparingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onExported]
     * @deprecated [depNote:dxDataGridOptions.onExported]
     */
    onExported?: (
      e: DevExpress.ui.dxDataGrid.ExportedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onExporting]
     */
    onExporting?: (
      e: DevExpress.ui.dxDataGrid.ExportingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onFileSaving]
     * @deprecated [depNote:dxDataGridOptions.onFileSaving]
     */
    onFileSaving?: (
      e: DevExpress.ui.dxDataGrid.FileSavingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onFocusedCellChanged]
     */
    onFocusedCellChanged?: (
      e: DevExpress.ui.dxDataGrid.FocusedCellChangedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onFocusedCellChanging]
     */
    onFocusedCellChanging?: (
      e: DevExpress.ui.dxDataGrid.FocusedCellChangingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onFocusedRowChanged]
     */
    onFocusedRowChanged?: (
      e: DevExpress.ui.dxDataGrid.FocusedRowChangedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onFocusedRowChanging]
     */
    onFocusedRowChanging?: (
      e: DevExpress.ui.dxDataGrid.FocusedRowChangingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onRowClick]
     */
    onRowClick?: (
      e: DevExpress.ui.dxDataGrid.RowClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onRowDblClick]
     */
    onRowDblClick?: (
      e: DevExpress.ui.dxDataGrid.RowDblClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.onRowPrepared]
     */
    onRowPrepared?: (
      e: DevExpress.ui.dxDataGrid.RowPreparedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxDataGridOptions.remoteOperations]
     */
    remoteOperations?:
      | boolean
      | {
          /**
           * [descr:dxDataGridOptions.remoteOperations.filtering]
           */
          filtering?: boolean;
          /**
           * [descr:dxDataGridOptions.remoteOperations.groupPaging]
           */
          groupPaging?: boolean;
          /**
           * [descr:dxDataGridOptions.remoteOperations.grouping]
           */
          grouping?: boolean;
          /**
           * [descr:dxDataGridOptions.remoteOperations.paging]
           */
          paging?: boolean;
          /**
           * [descr:dxDataGridOptions.remoteOperations.sorting]
           */
          sorting?: boolean;
          /**
           * [descr:dxDataGridOptions.remoteOperations.summary]
           */
          summary?: boolean;
        }
      | 'auto';
    /**
     * [descr:dxDataGridOptions.rowTemplate]
     */
    rowTemplate?:
      | DevExpress.core.template
      | ((
          rowElement: DevExpress.core.DxElement,
          rowInfo: DevExpress.ui.dxDataGrid.RowTemplateData<TRowData, TKey>
        ) => any);
    /**
     * [descr:dxDataGridOptions.dataRowTemplate]
     */
    dataRowTemplate?:
      | DevExpress.core.template
      | ((
          rowElement: DevExpress.core.DxElement,
          rowInfo: DevExpress.ui.dxDataGrid.DataRowTemplateData<TRowData, TKey>
        ) => any);
    /**
     * [descr:dxDataGridOptions.scrolling]
     */
    scrolling?: DevExpress.ui.dxDataGrid.Scrolling;
    /**
     * [descr:dxDataGridOptions.selection]
     */
    selection?: DevExpress.ui.dxDataGrid.Selection;
    /**
     * [descr:dxDataGridOptions.selectionFilter]
     */
    selectionFilter?: string | Array<any> | Function;
    /**
     * [descr:dxDataGridOptions.sortByGroupSummaryInfo]
     */
    sortByGroupSummaryInfo?: Array<DevExpress.ui.dxDataGrid.dxDataGridSortByGroupSummaryInfoItem>;
    /**
     * [descr:dxDataGridOptions.summary]
     */
    summary?: DevExpress.ui.dxDataGrid.Summary<TRowData, TKey>;
    /**
     * [descr:dxDataGridOptions.toolbar]
     */
    toolbar?: DevExpress.ui.dxDataGrid.Toolbar;
  }
  /**
   * @deprecated Use DevExpress.ui.dxDataGrid.Scrolling instead
   */
  export type dxDataGridScrolling = DevExpress.ui.dxDataGrid.Scrolling;
  /**
   * @deprecated Use DevExpress.ui.dxDataGrid.Selection instead
   */
  export type dxDataGridSelection = DevExpress.ui.dxDataGrid.Selection;
  /**
   * [descr:dxDateBox]
   */
  export class dxDateBox extends dxDropDownEditor<dxDateBoxOptions> {
    /**
     * [descr:dxDateBox.close()]
     */
    close(): void;
    /**
     * [descr:dxDateBox.open()]
     */
    open(): void;
  }
  module dxDateBox {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      Event
    >;
    export type ClosedEvent = DevExpress.events.EventInfo<dxDateBox>;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxDateBox>;
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      ClipboardEvent
    >;
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      ClipboardEvent
    >;
    export type DisabledDate =
      DevExpress.ui.dxCalendar.ComponentDisabledDate<dxDateBox>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxDateBox>;
    export type DropDownButtonTemplateData =
      DevExpress.ui.dxDropDownEditor.DropDownButtonTemplateDataModel;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxDateBox>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      UIEvent
    >;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      KeyboardEvent
    >;
    export type OpenedEvent = DevExpress.events.EventInfo<dxDateBox>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxDateBox> &
      DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      ClipboardEvent
    >;
    export type Properties = dxDateBoxOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxDateBox,
      KeyboardEvent | MouseEvent | PointerEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDateBoxOptions extends dxDropDownEditorOptions<dxDateBox> {
    /**
     * [descr:dxDateBoxOptions.adaptivityEnabled]
     */
    adaptivityEnabled?: boolean;
    /**
     * [descr:dxDateBoxOptions.applyButtonText]
     */
    applyButtonText?: string;
    /**
     * [descr:dxDateBoxOptions.calendarOptions]
     */
    calendarOptions?: dxCalendarOptions;
    /**
     * [descr:dxDateBoxOptions.cancelButtonText]
     */
    cancelButtonText?: string;
    /**
     * [descr:dxDateBoxOptions.dateOutOfRangeMessage]
     */
    dateOutOfRangeMessage?: string;
    /**
     * [descr:dxDateBoxOptions.dateSerializationFormat]
     */
    dateSerializationFormat?: string;
    /**
     * [descr:dxDateBoxOptions.disabledDates]
     */
    disabledDates?:
      | Array<Date>
      | ((data: DevExpress.ui.dxDateBox.DisabledDate) => boolean);
    /**
     * [descr:dxDateBoxOptions.displayFormat]
     */
    displayFormat?: Format;
    /**
     * [descr:dxDateBoxOptions.interval]
     */
    interval?: number;
    /**
     * [descr:dxDateBoxOptions.invalidDateMessage]
     */
    invalidDateMessage?: string;
    /**
     * [descr:dxDateBoxOptions.max]
     */
    max?: Date | number | string;
    /**
     * [descr:dxDateBoxOptions.min]
     */
    min?: Date | number | string;
    /**
     * [descr:dxDateBoxOptions.pickerType]
     */
    pickerType?: 'calendar' | 'list' | 'native' | 'rollers';
    /**
     * [descr:dxDateBoxOptions.placeholder]
     */
    placeholder?: string;
    /**
     * [descr:dxDateBoxOptions.showAnalogClock]
     */
    showAnalogClock?: boolean;
    /**
     * [descr:dxDateBoxOptions.type]
     */
    type?: 'date' | 'datetime' | 'time';
    /**
     * [descr:dxDateBoxOptions.useMaskBehavior]
     */
    useMaskBehavior?: boolean;
    /**
     * [descr:dxDateBoxOptions.value]
     */
    value?: Date | number | string;

    /**
     * [descr:dxDateBoxOptions.dropDownOptions]
     */
    dropDownOptions?: DevExpress.ui.dxPopup.Properties;
  }
  /**
   * [descr:dxDeferRendering]
   */
  export class dxDeferRendering extends Widget<dxDeferRenderingOptions> {}
  module dxDeferRendering {
    export type ContentReadyEvent =
      DevExpress.events.EventInfo<dxDeferRendering>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxDeferRendering>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxDeferRendering>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxDeferRendering> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxDeferRenderingOptions;
    export type RenderedEvent = DevExpress.events.EventInfo<dxDeferRendering>;
    export type ShownEvent = DevExpress.events.EventInfo<dxDeferRendering>;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDeferRenderingOptions
    extends WidgetOptions<dxDeferRendering> {
    /**
     * [descr:dxDeferRenderingOptions.animation]
     */
    animation?: AnimationConfig;
    /**
     * [descr:dxDeferRenderingOptions.onRendered]
     */
    onRendered?: (e: {
      component?: dxDeferRendering;
      element?: DevExpress.core.DxElement;
      model?: any;
    }) => void;
    /**
     * [descr:dxDeferRenderingOptions.onShown]
     */
    onShown?: (e: {
      component?: dxDeferRendering;
      element?: DevExpress.core.DxElement;
      model?: any;
    }) => void;
    /**
     * [descr:dxDeferRenderingOptions.renderWhen]
     */
    renderWhen?: PromiseLike<void> | boolean;
    /**
     * [descr:dxDeferRenderingOptions.showLoadIndicator]
     */
    showLoadIndicator?: boolean;
    /**
     * [descr:dxDeferRenderingOptions.staggerItemSelector]
     */
    staggerItemSelector?: string;
  }
  /**
   * [descr:dxDiagram]
   */
  export class dxDiagram extends Widget<dxDiagramOptions> {
    /**
     * [descr:dxDiagram.getNodeDataSource()]
     */
    getNodeDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxDiagram.getEdgeDataSource()]
     */
    getEdgeDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxDiagram.getItemByKey(key)]
     */
    getItemByKey(key: Object): DevExpress.ui.dxDiagram.Item;
    /**
     * [descr:dxDiagram.getItemById(id)]
     */
    getItemById(id: string): DevExpress.ui.dxDiagram.Item;
    /**
     * [descr:dxDiagram.getItems()]
     */
    getItems(): Array<DevExpress.ui.dxDiagram.Item>;
    /**
     * [descr:dxDiagram.getSelectedItems()]
     */
    getSelectedItems(): Array<DevExpress.ui.dxDiagram.Item>;
    /**
     * [descr:dxDiagram.setSelectedItems(items)]
     */
    setSelectedItems(items: Array<DevExpress.ui.dxDiagram.Item>): void;
    /**
     * [descr:dxDiagram.scrollToItem(item)]
     */
    scrollToItem(item: DevExpress.ui.dxDiagram.Item): void;
    /**
     * [descr:dxDiagram.export()]
     */
    export(): string;
    /**
     * [descr:dxDiagram.exportTo(format, callback)]
     */
    exportTo(format: 'svg' | 'png' | 'jpg', callback: Function): void;
    /**
     * [descr:dxDiagram.import(data, updateExistingItemsOnly)]
     */
    import(data: string, updateExistingItemsOnly?: boolean): void;
    /**
     * [descr:dxDiagram.updateToolbox()]
     */
    updateToolbox(): void;
    /**
     * [descr:dxDiagram.fitToContent()]
     */
    fitToContent(): void;
    /**
     * [descr:dxDiagram.fitToWidth()]
     */
    fitToWidth(): void;
  }
  module dxDiagram {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxDiagram>;
    export type CustomCommandEvent = {
      readonly component: dxDiagram;
      readonly element: DevExpress.core.DxElement;
      readonly name: string;
    };
    export type CustomShapeTemplateData = {
      readonly item: dxDiagramShape;
    };
    export type CustomShapeToolboxTemplateData = {
      readonly item: dxDiagramShape;
    };
    export type DisposingEvent = DevExpress.events.EventInfo<dxDiagram>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxDiagram>;
    export type ItemClickEvent = DevExpress.events.EventInfo<dxDiagram> & {
      readonly item: Item;
    };
    export type ItemDblClickEvent = DevExpress.events.EventInfo<dxDiagram> & {
      readonly item: Item;
    };
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxDiagram> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxDiagramOptions;
    export type RequestEditOperationEvent =
      DevExpress.events.EventInfo<dxDiagram> & {
        readonly operation:
          | 'addShape'
          | 'addShapeFromToolbox'
          | 'deleteShape'
          | 'deleteConnector'
          | 'changeConnection'
          | 'changeConnectorPoints';
        readonly args:
          | dxDiagramAddShapeArgs
          | dxDiagramAddShapeFromToolboxArgs
          | dxDiagramDeleteShapeArgs
          | dxDiagramDeleteConnectorArgs
          | dxDiagramChangeConnectionArgs
          | dxDiagramChangeConnectorPointsArgs
          | dxDiagramBeforeChangeShapeTextArgs
          | dxDiagramChangeShapeTextArgs
          | dxDiagramBeforeChangeConnectorTextArgs
          | dxDiagramChangeConnectorTextArgs
          | dxDiagramResizeShapeArgs
          | dxDiagramMoveShapeArgs;
        readonly reason: 'checkUIElementAvailability' | 'modelModification';
        allowed?: boolean;
      };
    export type RequestLayoutUpdateEvent =
      DevExpress.events.EventInfo<dxDiagram> & {
        readonly changes: any[];
        allowed?: boolean;
      };
    export type SelectionChangedEvent =
      DevExpress.events.EventInfo<dxDiagram> & {
        readonly items: Array<Item>;
      };
  }
  /**
   * [descr:dxDiagramAddShapeArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramAddShapeArgs {
    /**
     * [descr:dxDiagramAddShapeArgs.shape]
     */
    shape?: dxDiagramShape;
    /**
     * [descr:dxDiagramAddShapeArgs.position]
     */
    position?: {
      /**
       * [descr:dxDiagramAddShapeArgs.position.x]
       */
      x?: number;
      /**
       * [descr:dxDiagramAddShapeArgs.position.y]
       */
      y?: number;
    };
  }
  /**
   * [descr:dxDiagramAddShapeFromToolboxArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramAddShapeFromToolboxArgs {
    /**
     * [descr:dxDiagramAddShapeFromToolboxArgs.shapeType]
     */
    shapeType?:
      | 'text'
      | 'rectangle'
      | 'ellipse'
      | 'cross'
      | 'triangle'
      | 'diamond'
      | 'heart'
      | 'pentagon'
      | 'hexagon'
      | 'octagon'
      | 'star'
      | 'arrowLeft'
      | 'arrowTop'
      | 'arrowRight'
      | 'arrowBottom'
      | 'arrowNorthSouth'
      | 'arrowEastWest'
      | 'process'
      | 'decision'
      | 'terminator'
      | 'predefinedProcess'
      | 'document'
      | 'multipleDocuments'
      | 'manualInput'
      | 'preparation'
      | 'data'
      | 'database'
      | 'hardDisk'
      | 'internalStorage'
      | 'paperTape'
      | 'manualOperation'
      | 'delay'
      | 'storedData'
      | 'display'
      | 'merge'
      | 'connector'
      | 'or'
      | 'summingJunction'
      | 'verticalContainer'
      | 'horizontalContainer'
      | 'cardWithImageOnLeft'
      | 'cardWithImageOnTop'
      | 'cardWithImageOnRight'
      | string;
  }
  /**
   * [descr:dxDiagramBeforeChangeConnectorTextArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramBeforeChangeConnectorTextArgs {
    /**
     * [descr:dxDiagramBeforeChangeConnectorTextArgs.connector]
     */
    connector?: dxDiagramConnector;
    /**
     * [descr:dxDiagramBeforeChangeConnectorTextArgs.index]
     */
    index?: number;
  }
  /**
   * [descr:dxDiagramBeforeChangeShapeTextArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramBeforeChangeShapeTextArgs {
    /**
     * [descr:dxDiagramBeforeChangeShapeTextArgs.shape]
     */
    shape?: dxDiagramShape;
  }
  /**
   * [descr:dxDiagramChangeConnectionArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramChangeConnectionArgs {
    /**
     * [descr:dxDiagramChangeConnectionArgs.newShape]
     */
    newShape?: dxDiagramShape;
    /**
     * [descr:dxDiagramChangeConnectionArgs.oldShape]
     */
    oldShape?: dxDiagramShape;
    /**
     * [descr:dxDiagramChangeConnectionArgs.connector]
     */
    connector?: dxDiagramConnector;
    /**
     * [descr:dxDiagramChangeConnectionArgs.connectionPointIndex]
     */
    connectionPointIndex?: number;
    /**
     * [descr:dxDiagramChangeConnectionArgs.connectorPosition]
     */
    connectorPosition?: 'start' | 'end';
  }
  /**
   * [descr:dxDiagramChangeConnectorPointsArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramChangeConnectorPointsArgs {
    /**
     * [descr:dxDiagramChangeConnectorPointsArgs.connector]
     */
    connector?: dxDiagramConnector;
    /**
     * [descr:dxDiagramChangeConnectorPointsArgs.newPoints]
     */
    newPoints?: Array<{
      /**
       * [descr:dxDiagramChangeConnectorPointsArgs.newPoints.x]
       */
      x?: number;
      /**
       * [descr:dxDiagramChangeConnectorPointsArgs.newPoints.y]
       */
      y?: number;
    }>;
    /**
     * [descr:dxDiagramChangeConnectorPointsArgs.oldPoints]
     */
    oldPoints?: Array<{
      /**
       * [descr:dxDiagramChangeConnectorPointsArgs.oldPoints.x]
       */
      x?: number;
      /**
       * [descr:dxDiagramChangeConnectorPointsArgs.oldPoints.y]
       */
      y?: number;
    }>;
  }
  /**
   * [descr:dxDiagramChangeConnectorTextArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramChangeConnectorTextArgs {
    /**
     * [descr:dxDiagramChangeConnectorTextArgs.connector]
     */
    connector?: dxDiagramConnector;
    /**
     * [descr:dxDiagramChangeConnectorTextArgs.index]
     */
    index?: number;
    /**
     * [descr:dxDiagramChangeConnectorTextArgs.text]
     */
    text?: string;
  }
  /**
   * [descr:dxDiagramChangeShapeTextArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramChangeShapeTextArgs {
    /**
     * [descr:dxDiagramChangeShapeTextArgs.shape]
     */
    shape?: dxDiagramShape;
    /**
     * [descr:dxDiagramChangeShapeTextArgs.text]
     */
    text?: string;
  }
  /**
   * [descr:dxDiagramConnector]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramConnector extends DevExpress.ui.dxDiagram.Item {
    /**
     * [descr:dxDiagramConnector.fromKey]
     */
    fromKey?: any;
    /**
     * [descr:dxDiagramConnector.fromId]
     */
    fromId?: string;
    /**
     * [descr:dxDiagramConnector.fromPointIndex]
     */
    fromPointIndex?: number;
    /**
     * [descr:dxDiagramConnector.points]
     */
    points?: Array<{
      /**
       * [descr:dxDiagramConnector.points.x]
       */
      x?: number;
      /**
       * [descr:dxDiagramConnector.points.y]
       */
      y?: number;
    }>;

    /**
     * [descr:dxDiagramConnector.texts]
     */
    texts?: Array<string>;
    /**
     * [descr:dxDiagramConnector.toKey]
     */
    toKey?: any;
    /**
     * [descr:dxDiagramConnector.toId]
     */
    toId?: string;
    /**
     * [descr:dxDiagramConnector.toPointIndex]
     */
    toPointIndex?: number;
  }
  /**
   * [descr:dxDiagramCustomCommand]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramCustomCommand {
    /**
     * [descr:dxDiagramCustomCommand.name]
     */
    name?: string;
    /**
     * [descr:dxDiagramCustomCommand.text]
     */
    text?: string;
    /**
     * [descr:dxDiagramCustomCommand.icon]
     */
    icon?: string;
    /**
     * [descr:dxDiagramCustomCommand.items]
     */
    items?: Array<dxDiagramCustomCommand>;
  }
  /**
   * [descr:dxDiagramDeleteConnectorArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramDeleteConnectorArgs {
    /**
     * [descr:dxDiagramDeleteConnectorArgs.connector]
     */
    connector?: dxDiagramConnector;
  }
  /**
   * [descr:dxDiagramDeleteShapeArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramDeleteShapeArgs {
    /**
     * [descr:dxDiagramDeleteShapeArgs.shape]
     */
    shape?: dxDiagramShape;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramItem {
    /**
     * [descr:dxDiagramItem.dataItem]
     */
    dataItem?: any;
    /**
     * [descr:dxDiagramItem.id]
     */
    id?: string;
    /**
     * [descr:dxDiagramItem.key]
     */
    key?: Object;
    /**
     * [descr:dxDiagramItem.itemType]
     */
    itemType?: 'shape' | 'connector';
  }
  /**
   * [descr:dxDiagramMoveShapeArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramMoveShapeArgs {
    /**
     * [descr:dxDiagramMoveShapeArgs.shape]
     */
    shape?: dxDiagramShape;
    /**
     * [descr:dxDiagramMoveShapeArgs.newPosition]
     */
    newPosition?: {
      /**
       * [descr:dxDiagramMoveShapeArgs.newPosition.x]
       */
      x?: number;
      /**
       * [descr:dxDiagramMoveShapeArgs.newPosition.y]
       */
      y?: number;
    };
    /**
     * [descr:dxDiagramMoveShapeArgs.oldPosition]
     */
    oldPosition?: {
      /**
       * [descr:dxDiagramMoveShapeArgs.oldPosition.x]
       */
      x?: number;
      /**
       * [descr:dxDiagramMoveShapeArgs.oldPosition.y]
       */
      y?: number;
    };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramOptions extends WidgetOptions<dxDiagram> {
    /**
     * [descr:dxDiagramOptions.autoZoomMode]
     */
    autoZoomMode?: 'fitContent' | 'fitWidth' | 'disabled';
    /**
     * [descr:dxDiagramOptions.contextMenu]
     */
    contextMenu?: {
      /**
       * [descr:dxDiagramOptions.contextMenu.commands]
       */
      commands?: Array<
        | 'separator'
        | 'exportSvg'
        | 'exportPng'
        | 'exportJpg'
        | 'undo'
        | 'redo'
        | 'cut'
        | 'copy'
        | 'paste'
        | 'selectAll'
        | 'delete'
        | 'fontName'
        | 'fontSize'
        | 'bold'
        | 'italic'
        | 'underline'
        | 'lineStyle'
        | 'lineWidth'
        | 'textAlignLeft'
        | 'textAlignCenter'
        | 'textAlignRight'
        | 'lock'
        | 'unlock'
        | 'sendToBack'
        | 'bringToFront'
        | 'insertShapeImage'
        | 'editShapeImage'
        | 'deleteShapeImage'
        | 'connectorLineType'
        | 'connectorLineStart'
        | 'connectorLineEnd'
        | 'layoutTreeTopToBottom'
        | 'layoutTreeBottomToTop'
        | 'layoutTreeLeftToRight'
        | 'layoutTreeRightToLeft'
        | 'layoutLayeredTopToBottom'
        | 'layoutLayeredBottomToTop'
        | 'layoutLayeredLeftToRight'
        | 'layoutLayeredRightToLeft'
        | 'fullScreen'
        | 'zoomLevel'
        | 'showGrid'
        | 'snapToGrid'
        | 'gridSize'
        | 'units'
        | 'pageSize'
        | 'pageOrientation'
        | 'pageColor'
        | 'simpleView'
        | 'toolbox'
      >;
      /**
       * [descr:dxDiagramOptions.contextMenu.enabled]
       */
      enabled?: boolean;
    };
    /**
     * [descr:dxDiagramOptions.contextToolbox]
     */
    contextToolbox?: {
      /**
       * [descr:dxDiagramOptions.contextToolbox.category]
       */
      category?:
        | 'general'
        | 'flowchart'
        | 'orgChart'
        | 'containers'
        | 'custom'
        | string;
      /**
       * [descr:dxDiagramOptions.contextToolbox.displayMode]
       */
      displayMode?: 'icons' | 'texts';
      /**
       * [descr:dxDiagramOptions.contextToolbox.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxDiagramOptions.contextToolbox.shapeIconsPerRow]
       */
      shapeIconsPerRow?: number;
      /**
       * [descr:dxDiagramOptions.contextToolbox.shapes]
       */
      shapes?:
        | Array<
            | 'text'
            | 'rectangle'
            | 'ellipse'
            | 'cross'
            | 'triangle'
            | 'diamond'
            | 'heart'
            | 'pentagon'
            | 'hexagon'
            | 'octagon'
            | 'star'
            | 'arrowLeft'
            | 'arrowTop'
            | 'arrowRight'
            | 'arrowBottom'
            | 'arrowNorthSouth'
            | 'arrowEastWest'
            | 'process'
            | 'decision'
            | 'terminator'
            | 'predefinedProcess'
            | 'document'
            | 'multipleDocuments'
            | 'manualInput'
            | 'preparation'
            | 'data'
            | 'database'
            | 'hardDisk'
            | 'internalStorage'
            | 'paperTape'
            | 'manualOperation'
            | 'delay'
            | 'storedData'
            | 'display'
            | 'merge'
            | 'connector'
            | 'or'
            | 'summingJunction'
            | 'verticalContainer'
            | 'horizontalContainer'
            | 'cardWithImageOnLeft'
            | 'cardWithImageOnTop'
            | 'cardWithImageOnRight'
          >
        | Array<string>;
      /**
       * [descr:dxDiagramOptions.contextToolbox.width]
       */
      width?: number;
    };
    /**
     * [descr:dxDiagramOptions.onCustomCommand]
     */
    onCustomCommand?: (e: DevExpress.ui.dxDiagram.CustomCommandEvent) => void;
    /**
     * [descr:dxDiagramOptions.customShapeTemplate]
     */
    customShapeTemplate?:
      | DevExpress.core.template
      | ((
          container: DevExpress.core.DxElement<SVGElement>,
          data: DevExpress.ui.dxDiagram.CustomShapeTemplateData
        ) => any);
    /**
     * [descr:dxDiagramOptions.customShapeToolboxTemplate]
     */
    customShapeToolboxTemplate?:
      | DevExpress.core.template
      | ((
          container: DevExpress.core.DxElement<SVGElement>,
          data: DevExpress.ui.dxDiagram.CustomShapeToolboxTemplateData
        ) => any);
    /**
     * [descr:dxDiagramOptions.customShapes]
     */
    customShapes?: Array<{
      /**
       * [descr:dxDiagramOptions.customShapes.allowEditImage]
       */
      allowEditImage?: boolean;
      /**
       * [descr:dxDiagramOptions.customShapes.allowEditText]
       */
      allowEditText?: boolean;
      /**
       * [descr:dxDiagramOptions.customShapes.allowResize]
       */
      allowResize?: boolean;
      /**
       * [descr:dxDiagramOptions.customShapes.backgroundImageHeight]
       */
      backgroundImageHeight?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.backgroundImageLeft]
       */
      backgroundImageLeft?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.backgroundImageTop]
       */
      backgroundImageTop?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.backgroundImageUrl]
       */
      backgroundImageUrl?: string;
      /**
       * [descr:dxDiagramOptions.customShapes.backgroundImageToolboxUrl]
       */
      backgroundImageToolboxUrl?: string;
      /**
       * [descr:dxDiagramOptions.customShapes.backgroundImageWidth]
       */
      backgroundImageWidth?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.baseType]
       */
      baseType?:
        | 'text'
        | 'rectangle'
        | 'ellipse'
        | 'cross'
        | 'triangle'
        | 'diamond'
        | 'heart'
        | 'pentagon'
        | 'hexagon'
        | 'octagon'
        | 'star'
        | 'arrowLeft'
        | 'arrowTop'
        | 'arrowRight'
        | 'arrowBottom'
        | 'arrowNorthSouth'
        | 'arrowEastWest'
        | 'process'
        | 'decision'
        | 'terminator'
        | 'predefinedProcess'
        | 'document'
        | 'multipleDocuments'
        | 'manualInput'
        | 'preparation'
        | 'data'
        | 'database'
        | 'hardDisk'
        | 'internalStorage'
        | 'paperTape'
        | 'manualOperation'
        | 'delay'
        | 'storedData'
        | 'display'
        | 'merge'
        | 'connector'
        | 'or'
        | 'summingJunction'
        | 'verticalContainer'
        | 'horizontalContainer'
        | 'cardWithImageOnLeft'
        | 'cardWithImageOnTop'
        | 'cardWithImageOnRight'
        | string;
      /**
       * [descr:dxDiagramOptions.customShapes.category]
       */
      category?: string;
      /**
       * [descr:dxDiagramOptions.customShapes.connectionPoints]
       */
      connectionPoints?: Array<{
        /**
         * [descr:dxDiagramOptions.customShapes.connectionPoints.x]
         */
        x?: number;
        /**
         * [descr:dxDiagramOptions.customShapes.connectionPoints.y]
         */
        y?: number;
      }>;
      /**
       * [descr:dxDiagramOptions.customShapes.defaultHeight]
       */
      defaultHeight?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.defaultImageUrl]
       */
      defaultImageUrl?: string;
      /**
       * [descr:dxDiagramOptions.customShapes.defaultText]
       */
      defaultText?: string;
      /**
       * [descr:dxDiagramOptions.customShapes.defaultWidth]
       */
      defaultWidth?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.imageHeight]
       */
      imageHeight?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.imageLeft]
       */
      imageLeft?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.imageTop]
       */
      imageTop?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.imageWidth]
       */
      imageWidth?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.keepRatioOnAutoSize]
       */
      keepRatioOnAutoSize?: boolean;
      /**
       * [descr:dxDiagramOptions.customShapes.maxHeight]
       */
      maxHeight?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.maxWidth]
       */
      maxWidth?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.minHeight]
       */
      minHeight?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.minWidth]
       */
      minWidth?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.template]
       */
      template?:
        | DevExpress.core.template
        | ((
            container: DevExpress.core.DxElement<SVGElement>,
            data: DevExpress.ui.dxDiagram.CustomShapeTemplateData
          ) => any);
      /**
       * [descr:dxDiagramOptions.customShapes.templateHeight]
       */
      templateHeight?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.templateLeft]
       */
      templateLeft?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.templateTop]
       */
      templateTop?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.templateWidth]
       */
      templateWidth?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.textHeight]
       */
      textHeight?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.textLeft]
       */
      textLeft?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.textTop]
       */
      textTop?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.textWidth]
       */
      textWidth?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.title]
       */
      title?: string;
      /**
       * [descr:dxDiagramOptions.customShapes.toolboxTemplate]
       */
      toolboxTemplate?:
        | DevExpress.core.template
        | ((
            container: DevExpress.core.DxElement<SVGElement>,
            data: DevExpress.ui.dxDiagram.CustomShapeToolboxTemplateData
          ) => any);
      /**
       * [descr:dxDiagramOptions.customShapes.toolboxWidthToHeightRatio]
       */
      toolboxWidthToHeightRatio?: number;
      /**
       * [descr:dxDiagramOptions.customShapes.type]
       */
      type?: string;
    }>;
    /**
     * [descr:dxDiagramOptions.defaultItemProperties]
     */
    defaultItemProperties?: {
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.style]
       */
      style?: Object;
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.textStyle]
       */
      textStyle?: Object;
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.connectorLineType]
       */
      connectorLineType?: 'straight' | 'orthogonal';
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.connectorLineStart]
       */
      connectorLineStart?:
        | 'none'
        | 'arrow'
        | 'outlinedTriangle'
        | 'filledTriangle';
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.connectorLineEnd]
       */
      connectorLineEnd?:
        | 'none'
        | 'arrow'
        | 'outlinedTriangle'
        | 'filledTriangle';
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.shapeMinWidth]
       */
      shapeMinWidth?: number;
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.shapeMaxWidth]
       */
      shapeMaxWidth?: number;
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.shapeMinHeight]
       */
      shapeMinHeight?: number;
      /**
       * [descr:dxDiagramOptions.defaultItemProperties.shapeMaxHeight]
       */
      shapeMaxHeight?: number;
    };
    /**
     * [descr:dxDiagramOptions.editing]
     */
    editing?: {
      /**
       * [descr:dxDiagramOptions.editing.allowAddShape]
       */
      allowAddShape?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowDeleteShape]
       */
      allowDeleteShape?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowDeleteConnector]
       */
      allowDeleteConnector?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowChangeConnection]
       */
      allowChangeConnection?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowChangeConnectorPoints]
       */
      allowChangeConnectorPoints?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowChangeConnectorText]
       */
      allowChangeConnectorText?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowChangeShapeText]
       */
      allowChangeShapeText?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowResizeShape]
       */
      allowResizeShape?: boolean;
      /**
       * [descr:dxDiagramOptions.editing.allowMoveShape]
       */
      allowMoveShape?: boolean;
    };
    /**
     * [descr:dxDiagramOptions.edges]
     */
    edges?: {
      /**
       * [descr:dxDiagramOptions.edges.customDataExpr]
       */
      customDataExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.dataSource]
       */
      dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
      /**
       * [descr:dxDiagramOptions.edges.fromExpr]
       */
      fromExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.fromLineEndExpr]
       */
      fromLineEndExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.fromPointIndexExpr]
       */
      fromPointIndexExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.keyExpr]
       */
      keyExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.lineTypeExpr]
       */
      lineTypeExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.lockedExpr]
       */
      lockedExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.pointsExpr]
       */
      pointsExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.styleExpr]
       */
      styleExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.textExpr]
       */
      textExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.textStyleExpr]
       */
      textStyleExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.toExpr]
       */
      toExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.toLineEndExpr]
       */
      toLineEndExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.toPointIndexExpr]
       */
      toPointIndexExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.edges.zIndexExpr]
       */
      zIndexExpr?: string | ((data: any, value?: any) => any);
    };
    /**
     * [descr:dxDiagramOptions.export]
     */
    export?: {
      /**
       * [descr:dxDiagramOptions.export.fileName]
       */
      fileName?: string;
      /**
       * [descr:dxDiagramOptions.export.proxyUrl]
       * @deprecated [depNote:dxDiagramOptions.export.proxyUrl]
       */
      proxyUrl?: string;
    };
    /**
     * [descr:dxDiagramOptions.fullScreen]
     */
    fullScreen?: boolean;
    /**
     * [descr:dxDiagramOptions.gridSize]
     */
    gridSize?:
      | number
      | {
          /**
           * [descr:dxDiagramOptions.gridSize.items]
           */
          items?: Array<number>;
          /**
           * [descr:dxDiagramOptions.gridSize.value]
           */
          value?: number;
        };
    /**
     * [descr:dxDiagramOptions.nodes]
     */
    nodes?: {
      /**
       * [descr:dxDiagramOptions.nodes.autoLayout]
       */
      autoLayout?:
        | 'off'
        | 'tree'
        | 'layered'
        | {
            /**
             * [descr:dxDiagramOptions.nodes.autoLayout.orientation]
             */
            orientation?: 'vertical' | 'horizontal';
            /**
             * [descr:dxDiagramOptions.nodes.autoLayout.type]
             */
            type?: 'off' | 'tree' | 'layered';
          };
      /**
       * [descr:dxDiagramOptions.nodes.autoSizeEnabled]
       */
      autoSizeEnabled?: boolean;
      /**
       * [descr:dxDiagramOptions.nodes.containerKeyExpr]
       */
      containerKeyExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.containerChildrenExpr]
       */
      containerChildrenExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.customDataExpr]
       */
      customDataExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.dataSource]
       */
      dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
      /**
       * [descr:dxDiagramOptions.nodes.heightExpr]
       */
      heightExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.imageUrlExpr]
       */
      imageUrlExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.itemsExpr]
       */
      itemsExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.keyExpr]
       */
      keyExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.leftExpr]
       */
      leftExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.lockedExpr]
       */
      lockedExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.parentKeyExpr]
       */
      parentKeyExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.styleExpr]
       */
      styleExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.textExpr]
       */
      textExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.textStyleExpr]
       */
      textStyleExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.topExpr]
       */
      topExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.typeExpr]
       */
      typeExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.widthExpr]
       */
      widthExpr?: string | ((data: any, value?: any) => any);
      /**
       * [descr:dxDiagramOptions.nodes.zIndexExpr]
       */
      zIndexExpr?: string | ((data: any, value?: any) => any);
    };
    /**
     * [descr:dxDiagramOptions.hasChanges]
     */
    hasChanges?: boolean;
    /**
     * [descr:dxDiagramOptions.onItemClick]
     */
    onItemClick?: (e: DevExpress.ui.dxDiagram.ItemClickEvent) => void;
    /**
     * [descr:dxDiagramOptions.onItemDblClick]
     */
    onItemDblClick?: (e: DevExpress.ui.dxDiagram.ItemDblClickEvent) => void;
    /**
     * [descr:dxDiagramOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.ui.dxDiagram.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxDiagramOptions.onRequestEditOperation]
     */
    onRequestEditOperation?: (
      e: DevExpress.ui.dxDiagram.RequestEditOperationEvent
    ) => void;
    /**
     * [descr:dxDiagramOptions.onRequestLayoutUpdate]
     */
    onRequestLayoutUpdate?: (
      e: DevExpress.ui.dxDiagram.RequestLayoutUpdateEvent
    ) => void;
    /**
     * [descr:dxDiagramOptions.pageColor]
     */
    pageColor?: string;
    /**
     * [descr:dxDiagramOptions.pageOrientation]
     */
    pageOrientation?: 'portrait' | 'landscape';
    /**
     * [descr:dxDiagramOptions.pageSize]
     */
    pageSize?: {
      /**
       * [descr:dxDiagramOptions.pageSize.height]
       */
      height?: number;
      /**
       * [descr:dxDiagramOptions.pageSize.items]
       */
      items?: Array<{
        /**
         * [descr:dxDiagramOptions.pageSize.items.height]
         */
        height?: number;
        /**
         * [descr:dxDiagramOptions.pageSize.items.text]
         */
        text?: string;
        /**
         * [descr:dxDiagramOptions.pageSize.items.width]
         */
        width?: number;
      }>;
      /**
       * [descr:dxDiagramOptions.pageSize.width]
       */
      width?: number;
    };
    /**
     * [descr:dxDiagramOptions.propertiesPanel]
     */
    propertiesPanel?: {
      /**
       * [descr:dxDiagramOptions.propertiesPanel.tabs]
       */
      tabs?: Array<{
        /**
         * [descr:dxDiagramOptions.propertiesPanel.tabs.commands]
         */
        commands?: Array<
          | 'separator'
          | 'exportSvg'
          | 'exportPng'
          | 'exportJpg'
          | 'undo'
          | 'redo'
          | 'cut'
          | 'copy'
          | 'paste'
          | 'selectAll'
          | 'delete'
          | 'fontName'
          | 'fontSize'
          | 'bold'
          | 'italic'
          | 'underline'
          | 'fontColor'
          | 'lineStyle'
          | 'lineWidth'
          | 'lineColor'
          | 'fillColor'
          | 'textAlignLeft'
          | 'textAlignCenter'
          | 'textAlignRight'
          | 'lock'
          | 'unlock'
          | 'sendToBack'
          | 'bringToFront'
          | 'insertShapeImage'
          | 'editShapeImage'
          | 'deleteShapeImage'
          | 'connectorLineType'
          | 'connectorLineStart'
          | 'connectorLineEnd'
          | 'layoutTreeTopToBottom'
          | 'layoutTreeBottomToTop'
          | 'layoutTreeLeftToRight'
          | 'layoutTreeRightToLeft'
          | 'layoutLayeredTopToBottom'
          | 'layoutLayeredBottomToTop'
          | 'layoutLayeredLeftToRight'
          | 'layoutLayeredRightToLeft'
          | 'fullScreen'
          | 'zoomLevel'
          | 'showGrid'
          | 'snapToGrid'
          | 'gridSize'
          | 'units'
          | 'pageSize'
          | 'pageOrientation'
          | 'pageColor'
          | 'simpleView'
          | 'toolbox'
        >;
        /**
         * [descr:dxDiagramOptions.propertiesPanel.tabs.groups]
         */
        groups?: Array<{
          /**
           * [descr:dxDiagramOptions.propertiesPanel.tabs.groups.commands]
           */
          commands?: Array<
            | 'separator'
            | 'exportSvg'
            | 'exportPng'
            | 'exportJpg'
            | 'undo'
            | 'redo'
            | 'cut'
            | 'copy'
            | 'paste'
            | 'selectAll'
            | 'delete'
            | 'fontName'
            | 'fontSize'
            | 'bold'
            | 'italic'
            | 'underline'
            | 'fontColor'
            | 'lineStyle'
            | 'lineWidth'
            | 'lineColor'
            | 'fillColor'
            | 'textAlignLeft'
            | 'textAlignCenter'
            | 'textAlignRight'
            | 'lock'
            | 'unlock'
            | 'sendToBack'
            | 'bringToFront'
            | 'insertShapeImage'
            | 'editShapeImage'
            | 'deleteShapeImage'
            | 'connectorLineType'
            | 'connectorLineStart'
            | 'connectorLineEnd'
            | 'layoutTreeTopToBottom'
            | 'layoutTreeBottomToTop'
            | 'layoutTreeLeftToRight'
            | 'layoutTreeRightToLeft'
            | 'layoutLayeredTopToBottom'
            | 'layoutLayeredBottomToTop'
            | 'layoutLayeredLeftToRight'
            | 'layoutLayeredRightToLeft'
            | 'fullScreen'
            | 'zoomLevel'
            | 'showGrid'
            | 'snapToGrid'
            | 'gridSize'
            | 'units'
            | 'pageSize'
            | 'pageOrientation'
            | 'pageColor'
            | 'simpleView'
            | 'toolbox'
          >;
          /**
           * [descr:dxDiagramOptions.propertiesPanel.tabs.groups.title]
           */
          title?: string;
        }>;
        /**
         * [descr:dxDiagramOptions.propertiesPanel.tabs.title]
         */
        title?: string;
      }>;
      /**
       * [descr:dxDiagramOptions.propertiesPanel.visibility]
       */
      visibility?: 'auto' | 'visible' | 'collapsed' | 'disabled';
    };
    /**
     * [descr:dxDiagramOptions.readOnly]
     */
    readOnly?: boolean;
    /**
     * [descr:dxDiagramOptions.showGrid]
     */
    showGrid?: boolean;
    /**
     * [descr:dxDiagramOptions.simpleView]
     */
    simpleView?: boolean;
    /**
     * [descr:dxDiagramOptions.useNativeScrolling]
     */
    useNativeScrolling?: boolean;
    /**
     * [descr:dxDiagramOptions.snapToGrid]
     */
    snapToGrid?: boolean;
    /**
     * [descr:dxDiagramOptions.mainToolbar]
     */
    mainToolbar?: {
      /**
       * [descr:dxDiagramOptions.mainToolbar.commands]
       */
      commands?: Array<
        | 'separator'
        | 'exportSvg'
        | 'exportPng'
        | 'exportJpg'
        | 'undo'
        | 'redo'
        | 'cut'
        | 'copy'
        | 'paste'
        | 'selectAll'
        | 'delete'
        | 'fontName'
        | 'fontSize'
        | 'bold'
        | 'italic'
        | 'underline'
        | 'fontColor'
        | 'lineStyle'
        | 'lineWidth'
        | 'lineColor'
        | 'fillColor'
        | 'textAlignLeft'
        | 'textAlignCenter'
        | 'textAlignRight'
        | 'lock'
        | 'unlock'
        | 'sendToBack'
        | 'bringToFront'
        | 'insertShapeImage'
        | 'editShapeImage'
        | 'deleteShapeImage'
        | 'connectorLineType'
        | 'connectorLineStart'
        | 'connectorLineEnd'
        | 'layoutTreeTopToBottom'
        | 'layoutTreeBottomToTop'
        | 'layoutTreeLeftToRight'
        | 'layoutTreeRightToLeft'
        | 'layoutLayeredTopToBottom'
        | 'layoutLayeredBottomToTop'
        | 'layoutLayeredLeftToRight'
        | 'layoutLayeredRightToLeft'
        | 'fullScreen'
        | 'zoomLevel'
        | 'showGrid'
        | 'snapToGrid'
        | 'gridSize'
        | 'units'
        | 'pageSize'
        | 'pageOrientation'
        | 'pageColor'
        | 'simpleView'
        | 'toolbox'
      >;
      /**
       * [descr:dxDiagramOptions.mainToolbar.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxDiagramOptions.historyToolbar]
     */
    historyToolbar?: {
      /**
       * [descr:dxDiagramOptions.historyToolbar.commands]
       */
      commands?: Array<
        | 'separator'
        | 'exportSvg'
        | 'exportPng'
        | 'exportJpg'
        | 'undo'
        | 'redo'
        | 'cut'
        | 'copy'
        | 'paste'
        | 'selectAll'
        | 'delete'
        | 'fontName'
        | 'fontSize'
        | 'bold'
        | 'italic'
        | 'underline'
        | 'fontColor'
        | 'lineStyle'
        | 'lineWidth'
        | 'lineColor'
        | 'fillColor'
        | 'textAlignLeft'
        | 'textAlignCenter'
        | 'textAlignRight'
        | 'lock'
        | 'unlock'
        | 'sendToBack'
        | 'bringToFront'
        | 'insertShapeImage'
        | 'editShapeImage'
        | 'deleteShapeImage'
        | 'connectorLineType'
        | 'connectorLineStart'
        | 'connectorLineEnd'
        | 'layoutTreeTopToBottom'
        | 'layoutTreeBottomToTop'
        | 'layoutTreeLeftToRight'
        | 'layoutTreeRightToLeft'
        | 'layoutLayeredTopToBottom'
        | 'layoutLayeredBottomToTop'
        | 'layoutLayeredLeftToRight'
        | 'layoutLayeredRightToLeft'
        | 'fullScreen'
        | 'zoomLevel'
        | 'showGrid'
        | 'snapToGrid'
        | 'gridSize'
        | 'units'
        | 'pageSize'
        | 'pageOrientation'
        | 'pageColor'
        | 'simpleView'
        | 'toolbox'
      >;
      /**
       * [descr:dxDiagramOptions.historyToolbar.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxDiagramOptions.viewToolbar]
     */
    viewToolbar?: {
      /**
       * [descr:dxDiagramOptions.viewToolbar.commands]
       */
      commands?: Array<
        | 'separator'
        | 'exportSvg'
        | 'exportPng'
        | 'exportJpg'
        | 'undo'
        | 'redo'
        | 'cut'
        | 'copy'
        | 'paste'
        | 'selectAll'
        | 'delete'
        | 'fontName'
        | 'fontSize'
        | 'bold'
        | 'italic'
        | 'underline'
        | 'fontColor'
        | 'lineStyle'
        | 'lineWidth'
        | 'lineColor'
        | 'fillColor'
        | 'textAlignLeft'
        | 'textAlignCenter'
        | 'textAlignRight'
        | 'lock'
        | 'unlock'
        | 'sendToBack'
        | 'bringToFront'
        | 'insertShapeImage'
        | 'editShapeImage'
        | 'deleteShapeImage'
        | 'connectorLineType'
        | 'connectorLineStart'
        | 'connectorLineEnd'
        | 'layoutTreeTopToBottom'
        | 'layoutTreeBottomToTop'
        | 'layoutTreeLeftToRight'
        | 'layoutTreeRightToLeft'
        | 'layoutLayeredTopToBottom'
        | 'layoutLayeredBottomToTop'
        | 'layoutLayeredLeftToRight'
        | 'layoutLayeredRightToLeft'
        | 'fullScreen'
        | 'zoomLevel'
        | 'showGrid'
        | 'snapToGrid'
        | 'gridSize'
        | 'units'
        | 'pageSize'
        | 'pageOrientation'
        | 'pageColor'
        | 'simpleView'
        | 'toolbox'
      >;
      /**
       * [descr:dxDiagramOptions.viewToolbar.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxDiagramOptions.toolbox]
     */
    toolbox?: {
      /**
       * [descr:dxDiagramOptions.toolbox.groups]
       */
      groups?:
        | Array<{
            /**
             * [descr:dxDiagramOptions.toolbox.groups.category]
             */
            category?:
              | 'general'
              | 'flowchart'
              | 'orgChart'
              | 'containers'
              | 'custom'
              | string;
            /**
             * [descr:dxDiagramOptions.toolbox.groups.displayMode]
             */
            displayMode?: 'icons' | 'texts';
            /**
             * [descr:dxDiagramOptions.toolbox.groups.expanded]
             */
            expanded?: boolean;
            /**
             * [descr:dxDiagramOptions.toolbox.groups.shapes]
             */
            shapes?:
              | Array<
                  | 'text'
                  | 'rectangle'
                  | 'ellipse'
                  | 'cross'
                  | 'triangle'
                  | 'diamond'
                  | 'heart'
                  | 'pentagon'
                  | 'hexagon'
                  | 'octagon'
                  | 'star'
                  | 'arrowLeft'
                  | 'arrowTop'
                  | 'arrowRight'
                  | 'arrowBottom'
                  | 'arrowNorthSouth'
                  | 'arrowEastWest'
                  | 'process'
                  | 'decision'
                  | 'terminator'
                  | 'predefinedProcess'
                  | 'document'
                  | 'multipleDocuments'
                  | 'manualInput'
                  | 'preparation'
                  | 'data'
                  | 'database'
                  | 'hardDisk'
                  | 'internalStorage'
                  | 'paperTape'
                  | 'manualOperation'
                  | 'delay'
                  | 'storedData'
                  | 'display'
                  | 'merge'
                  | 'connector'
                  | 'or'
                  | 'summingJunction'
                  | 'verticalContainer'
                  | 'horizontalContainer'
                  | 'cardWithImageOnLeft'
                  | 'cardWithImageOnTop'
                  | 'cardWithImageOnRight'
                >
              | Array<string>;
            /**
             * [descr:dxDiagramOptions.toolbox.groups.title]
             */
            title?: string;
          }>
        | Array<'general' | 'flowchart' | 'orgChart' | 'containers' | 'custom'>;
      /**
       * [descr:dxDiagramOptions.toolbox.shapeIconsPerRow]
       */
      shapeIconsPerRow?: number;
      /**
       * [descr:dxDiagramOptions.toolbox.showSearch]
       */
      showSearch?: boolean;
      /**
       * [descr:dxDiagramOptions.toolbox.visibility]
       */
      visibility?: 'auto' | 'visible' | 'collapsed' | 'disabled';
      /**
       * [descr:dxDiagramOptions.toolbox.width]
       */
      width?: number;
    };
    /**
     * [descr:dxDiagramOptions.units]
     */
    units?: 'in' | 'cm' | 'px';
    /**
     * [descr:dxDiagramOptions.viewUnits]
     */
    viewUnits?: 'in' | 'cm' | 'px';
    /**
     * [descr:dxDiagramOptions.zoomLevel]
     */
    zoomLevel?:
      | number
      | {
          /**
           * [descr:dxDiagramOptions.zoomLevel.items]
           */
          items?: Array<number>;
          /**
           * [descr:dxDiagramOptions.zoomLevel.value]
           */
          value?: number;
        };
  }
  /**
   * [descr:dxDiagramResizeShapeArgs]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramResizeShapeArgs {
    /**
     * [descr:dxDiagramResizeShapeArgs.shape]
     */
    shape?: dxDiagramShape;
    /**
     * [descr:dxDiagramResizeShapeArgs.newSize]
     */
    newSize?: {
      /**
       * [descr:dxDiagramResizeShapeArgs.newSize.height]
       */
      height?: number;
      /**
       * [descr:dxDiagramResizeShapeArgs.newSize.width]
       */
      width?: number;
    };
    /**
     * [descr:dxDiagramResizeShapeArgs.oldSize]
     */
    oldSize?: {
      /**
       * [descr:dxDiagramResizeShapeArgs.oldSize.height]
       */
      height?: number;
      /**
       * [descr:dxDiagramResizeShapeArgs.oldSize.width]
       */
      width?: number;
    };
  }
  /**
   * [descr:dxDiagramShape]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDiagramShape extends DevExpress.ui.dxDiagram.Item {
    /**
     * [descr:dxDiagramShape.text]
     */
    text?: string;
    /**
     * [descr:dxDiagramShape.type]
     */
    type?:
      | 'text'
      | 'rectangle'
      | 'ellipse'
      | 'cross'
      | 'triangle'
      | 'diamond'
      | 'heart'
      | 'pentagon'
      | 'hexagon'
      | 'octagon'
      | 'star'
      | 'arrowLeft'
      | 'arrowTop'
      | 'arrowRight'
      | 'arrowBottom'
      | 'arrowNorthSouth'
      | 'arrowEastWest'
      | 'process'
      | 'decision'
      | 'terminator'
      | 'predefinedProcess'
      | 'document'
      | 'multipleDocuments'
      | 'manualInput'
      | 'preparation'
      | 'data'
      | 'database'
      | 'hardDisk'
      | 'internalStorage'
      | 'paperTape'
      | 'manualOperation'
      | 'delay'
      | 'storedData'
      | 'display'
      | 'merge'
      | 'connector'
      | 'or'
      | 'summingJunction'
      | 'verticalContainer'
      | 'horizontalContainer'
      | 'cardWithImageOnLeft'
      | 'cardWithImageOnTop'
      | 'cardWithImageOnRight'
      | string;
    /**
     * [descr:dxDiagramShape.position]
     */
    position?: {
      /**
       * [descr:dxDiagramShape.position.x]
       */
      x?: number;
      /**
       * [descr:dxDiagramShape.position.y]
       */
      y?: number;
    };

    /**
     * [descr:dxDiagramShape.size]
     */
    size?: {
      /**
       * [descr:dxDiagramShape.size.height]
       */
      height?: number;
      /**
       * [descr:dxDiagramShape.size.width]
       */
      width?: number;
    };
    /**
     * [descr:dxDiagramShape.attachedConnectorIds]
     */
    attachedConnectorIds?: Array<String>;
    /**
     * [descr:dxDiagramShape.containerId]
     */
    containerId?: string;
    /**
     * [descr:dxDiagramShape.containerChildItemIds]
     */
    containerChildItemIds?: Array<String>;
    /**
     * [descr:dxDiagramShape.containerExpanded]
     */
    containerExpanded?: boolean;
  }
  /**
   * [descr:dxDraggable]
   */
  export class dxDraggable
    extends DOMComponent<dxDraggableOptions>
    implements DraggableBase {}
  module dxDraggable {
    export type DisposingEvent = DevExpress.events.EventInfo<dxDraggable>;
    export type DragEndEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxDraggable> & {
        readonly itemData?: any;
        readonly itemElement?: DevExpress.core.DxElement;
        readonly fromComponent: dxSortable | dxDraggable;
        readonly toComponent: dxSortable | dxDraggable;
        readonly fromData?: any;
        readonly toData?: any;
      };
    export type DragMoveEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxDraggable> & {
        readonly itemData?: any;
        readonly itemElement?: DevExpress.core.DxElement;
        readonly fromComponent: dxSortable | dxDraggable;
        readonly toComponent: dxSortable | dxDraggable;
        readonly fromData?: any;
        readonly toData?: any;
      };
    export type DragStartEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxDraggable> & {
        itemData?: any;
        readonly itemElement?: DevExpress.core.DxElement;
        readonly fromData?: any;
      };
    export type DragTemplateData = {
      readonly itemData?: any;
      readonly itemElement: DevExpress.core.DxElement;
    };
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxDraggable>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxDraggable> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxDraggableOptions;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDraggableOptions
    extends DraggableBaseOptions<dxDraggable> {
    /**
     * [descr:dxDraggableOptions.clone]
     */
    clone?: boolean;
    /**
     * [descr:dxDraggableOptions.dragTemplate]
     */
    dragTemplate?:
      | DevExpress.core.template
      | ((
          dragInfo: DevExpress.ui.dxDraggable.DragTemplateData,
          containerElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDraggableOptions.onDragEnd]
     */
    onDragEnd?: (e: DevExpress.ui.dxDraggable.DragEndEvent) => void;
    /**
     * [descr:dxDraggableOptions.onDragMove]
     */
    onDragMove?: (e: DevExpress.ui.dxDraggable.DragMoveEvent) => void;
    /**
     * [descr:dxDraggableOptions.onDragStart]
     */
    onDragStart?: (e: DevExpress.ui.dxDraggable.DragStartEvent) => void;
  }
  /**
   * [descr:dxDrawer]
   */
  export class dxDrawer extends Widget<dxDrawerOptions> {
    /**
     * [descr:dxDrawer.content()]
     */
    content(): DevExpress.core.DxElement;
    /**
     * [descr:dxDrawer.hide()]
     */
    hide(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxDrawer.show()]
     */
    show(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxDrawer.toggle()]
     */
    toggle(): DevExpress.core.utils.DxPromise<void>;
  }
  module dxDrawer {
    export type DisposingEvent = DevExpress.events.EventInfo<dxDrawer>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxDrawer>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxDrawer> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxDrawerOptions;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDrawerOptions extends WidgetOptions<dxDrawer> {
    /**
     * [descr:dxDrawerOptions.animationDuration]
     */
    animationDuration?: number;
    /**
     * [descr:dxDrawerOptions.animationEnabled]
     */
    animationEnabled?: boolean;
    /**
     * [descr:dxDrawerOptions.closeOnOutsideClick]
     */
    closeOnOutsideClick?:
      | boolean
      | ((event: DevExpress.events.DxEvent) => boolean);
    /**
     * [descr:dxDrawerOptions.maxSize]
     */
    maxSize?: number;
    /**
     * [descr:dxDrawerOptions.minSize]
     */
    minSize?: number;
    /**
     * [descr:dxDrawerOptions.opened]
     */
    opened?: boolean;
    /**
     * [descr:dxDrawerOptions.openedStateMode]
     */
    openedStateMode?: 'overlap' | 'shrink' | 'push';
    /**
     * [descr:dxDrawerOptions.position]
     */
    position?: 'left' | 'right' | 'top' | 'bottom' | 'before' | 'after';
    /**
     * [descr:dxDrawerOptions.revealMode]
     */
    revealMode?: 'slide' | 'expand';
    /**
     * [descr:dxDrawerOptions.shading]
     */
    shading?: boolean;
    /**
     * [descr:dxDrawerOptions.target]
     * @deprecated [depNote:dxDrawerOptions.target]
     */
    target?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxDrawerOptions.template]
     */
    template?:
      | DevExpress.core.template
      | ((Element: DevExpress.core.DxElement) => any);
  }
  /**
   * [descr:dxDropDownBox]
   */
  export class dxDropDownBox extends dxDropDownEditor<dxDropDownBoxOptions> {
    getDataSource(): DevExpress.data.DataSource;
  }
  module dxDropDownBox {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      Event
    >;
    export type ClosedEvent = DevExpress.events.EventInfo<dxDropDownBox>;
    export type ContentTemplateData = {
      component: dxDropDownBox;
      readonly value?: any;
    };
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      ClipboardEvent
    >;
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      ClipboardEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxDropDownBox>;
    export type DropDownButtonTemplateData =
      DevExpress.ui.dxDropDownEditor.DropDownButtonTemplateDataModel;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxDropDownBox>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      UIEvent
    >;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      KeyboardEvent
    >;
    export type OpenedEvent = DevExpress.events.EventInfo<dxDropDownBox>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxDropDownBox> &
        DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      ClipboardEvent
    >;
    export type Properties = dxDropDownBoxOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxDropDownBox,
      KeyboardEvent | MouseEvent | PointerEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDropDownBoxOptions
    extends DataExpressionMixinOptions<dxDropDownBox>,
      dxDropDownEditorOptions<dxDropDownBox> {
    /**
     * [descr:dxDropDownBoxOptions.acceptCustomValue]
     */
    acceptCustomValue?: boolean;
    /**
     * [descr:dxDropDownBoxOptions.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          templateData: DevExpress.ui.dxDropDownBox.ContentTemplateData,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDropDownBoxOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
    /**
     * [descr:dxDropDownBoxOptions.displayValueFormatter]
     */
    displayValueFormatter?: (value: string | Array<any>) => string;
    /**
     * [descr:dxDropDownBoxOptions.fieldTemplate]
     */
    fieldTemplate?:
      | DevExpress.core.template
      | ((
          value: any,
          fieldElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDropDownBoxOptions.items]
     */
    items?: Array<any>;
    /**
     * [descr:dxDropDownBoxOptions.openOnFieldClick]
     */
    openOnFieldClick?: boolean;
    /**
     * [descr:dxDropDownBoxOptions.valueChangeEvent]
     */
    valueChangeEvent?: string;

    /**
     * [descr:dxDropDownBoxOptions.dropDownOptions]
     */
    dropDownOptions?: DevExpress.ui.dxPopup.Properties;
  }
  /**
   * [descr:dxDropDownButton]
   */
  export class dxDropDownButton extends Widget<dxDropDownButtonOptions> {
    /**
     * [descr:dxDropDownButton.close()]
     */
    close(): DevExpress.core.utils.DxPromise<void>;
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxDropDownButton.open()]
     */
    open(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxDropDownButton.toggle()]
     */
    toggle(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxDropDownButton.toggle(visibility)]
     */
    toggle(visibility: boolean): DevExpress.core.utils.DxPromise<void>;
  }
  module dxDropDownButton {
    export type ButtonClickEvent = DevExpress.events.NativeEventInfo<
      dxDropDownButton,
      KeyboardEvent | MouseEvent | PointerEvent
    > & {
      readonly selectedItem?: any;
    };
    export type ContentReadyEvent =
      DevExpress.events.EventInfo<dxDropDownButton>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxDropDownButton>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxDropDownButton>;
    export type ItemClickEvent = DevExpress.events.NativeEventInfo<
      dxDropDownButton,
      KeyboardEvent | MouseEvent | PointerEvent
    > & {
      readonly itemData?: any;
      readonly itemElement: DevExpress.core.DxElement;
    };
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxDropDownButton> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxDropDownButtonOptions;
    export type SelectionChangedEvent =
      DevExpress.events.EventInfo<dxDropDownButton> & {
        readonly item: any;
        readonly previousItem: any;
      };
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDropDownButtonItem extends DevExpress.ui.dxList.Item {
    /**
     * [descr:dxDropDownButtonItem.onClick]
     */
    onClick?:
      | ((e: DevExpress.ui.dxDropDownButton.ItemClickEvent) => void)
      | string;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDropDownButtonOptions
    extends WidgetOptions<dxDropDownButton> {
    /**
     * [descr:dxDropDownButtonOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<
      DevExpress.ui.dxDropDownButton.Item | any
    >;
    /**
     * [descr:dxDropDownButtonOptions.deferRendering]
     */
    deferRendering?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.displayExpr]
     */
    displayExpr?: string | ((itemData: any) => string);
    /**
     * [descr:dxDropDownButtonOptions.dropDownContentTemplate]
     */
    dropDownContentTemplate?:
      | DevExpress.core.template
      | ((
          data: Array<string | number | any> | DevExpress.data.DataSource,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDropDownButtonOptions.dropDownOptions]
     */
    dropDownOptions?: DevExpress.ui.dxPopup.Properties;
    /**
     * [descr:dxDropDownButtonOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.icon]
     */
    icon?: string;
    /**
     * [descr:dxDropDownButtonOptions.itemTemplate]
     */
    itemTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDropDownButtonOptions.items]
     */
    items?: Array<DevExpress.ui.dxDropDownButton.Item | any>;
    /**
     * [descr:dxDropDownButtonOptions.keyExpr]
     */
    keyExpr?: string;
    /**
     * [descr:dxDropDownButtonOptions.noDataText]
     */
    noDataText?: string;
    /**
     * [descr:dxDropDownButtonOptions.onButtonClick]
     */
    onButtonClick?:
      | ((e: DevExpress.ui.dxDropDownButton.ButtonClickEvent) => void)
      | string;
    /**
     * [descr:dxDropDownButtonOptions.onItemClick]
     */
    onItemClick?:
      | ((e: DevExpress.ui.dxDropDownButton.ItemClickEvent) => void)
      | string;
    /**
     * [descr:dxDropDownButtonOptions.onSelectionChanged]
     */
    onSelectionChanged?:
      | ((e: DevExpress.ui.dxDropDownButton.SelectionChangedEvent) => void)
      | string;
    /**
     * [descr:dxDropDownButtonOptions.opened]
     */
    opened?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.selectedItem]
     */
    selectedItem?: string | number | any;
    /**
     * [descr:dxDropDownButtonOptions.selectedItemKey]
     */
    selectedItemKey?: string | number;
    /**
     * [descr:dxDropDownButtonOptions.showArrowIcon]
     */
    showArrowIcon?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.splitButton]
     */
    splitButton?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.stylingMode]
     */
    stylingMode?: 'text' | 'outlined' | 'contained';
    /**
     * [descr:dxDropDownButtonOptions.text]
     */
    text?: string;
    /**
     * [descr:dxDropDownButtonOptions.useSelectMode]
     */
    useSelectMode?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.wrapItemText]
     */
    wrapItemText?: boolean;
    /**
     * [descr:dxDropDownButtonOptions.useItemTextAsTitle]
     */
    useItemTextAsTitle?: boolean;
  }
  /**
   * [descr:dxDropDownEditor]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxDropDownEditor<TProperties> extends dxTextBox<TProperties> {
    /**
     * [descr:dxDropDownEditor.close()]
     */
    close(): void;
    /**
     * [descr:dxDropDownEditor.content()]
     */
    content(): DevExpress.core.DxElement;
    /**
     * [descr:dxDropDownEditor.field()]
     */
    field(): DevExpress.core.DxElement;
    /**
     * [descr:dxDropDownEditor.open()]
     */
    open(): void;
  }
  module dxDropDownEditor {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface DropDownButtonTemplateDataModel {
      readonly text?: string;
      readonly icon?: string;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDropDownEditorOptions<TComponent>
    extends dxTextBoxOptions<TComponent> {
    /**
     * [descr:dxDropDownEditorOptions.acceptCustomValue]
     */
    acceptCustomValue?: boolean;
    /**
     * [descr:dxDropDownEditorOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxDropDownEditorOptions.applyValueMode]
     */
    applyValueMode?: 'instantly' | 'useButtons';
    /**
     * [descr:dxDropDownEditorOptions.dropDownOptions]
     */
    dropDownOptions?:
      | DevExpress.ui.dxPopup.Properties
      | DevExpress.ui.dxPopover.Properties;
    /**
     * [descr:dxDropDownEditorOptions.buttons]
     */
    buttons?: Array<'clear' | 'dropDown' | dxTextEditorButton>;
    /**
     * [descr:dxDropDownEditorOptions.deferRendering]
     */
    deferRendering?: boolean;
    /**
     * [descr:dxDropDownEditorOptions.dropDownButtonTemplate]
     */
    dropDownButtonTemplate?:
      | DevExpress.core.template
      | ((
          buttonData: DevExpress.ui.dxDropDownEditor.DropDownButtonTemplateDataModel,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDropDownEditorOptions.onClosed]
     */
    onClosed?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:dxDropDownEditorOptions.onOpened]
     */
    onOpened?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:dxDropDownEditorOptions.openOnFieldClick]
     */
    openOnFieldClick?: boolean;
    /**
     * [descr:dxDropDownEditorOptions.opened]
     */
    opened?: boolean;
    /**
     * [descr:dxDropDownEditorOptions.showDropDownButton]
     */
    showDropDownButton?: boolean;
    /**
     * [descr:dxDropDownEditorOptions.value]
     */
    value?: any;
  }
  /**
   * [descr:dxDropDownList]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxDropDownList<
    TProperties
  > extends dxDropDownEditor<TProperties> {
    getDataSource(): DevExpress.data.DataSource;
  }
  module dxDropDownList {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface SelectionChangedInfo<T = any> {
      readonly selectedItem: T;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxDropDownListOptions<TComponent>
    extends DataExpressionMixinOptions<TComponent>,
      dxDropDownEditorOptions<TComponent> {
    /**
     * [descr:dxDropDownListOptions.displayValue]
     */
    displayValue?: string;
    /**
     * [descr:dxDropDownListOptions.groupTemplate]
     */
    groupTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxDropDownListOptions.grouped]
     */
    grouped?: boolean;
    /**
     * [descr:dxDropDownListOptions.minSearchLength]
     */
    minSearchLength?: number;
    /**
     * [descr:dxDropDownListOptions.noDataText]
     */
    noDataText?: string;
    /**
     * [descr:dxDropDownListOptions.onItemClick]
     */
    onItemClick?: (
      e: DevExpress.events.NativeEventInfo<
        TComponent,
        KeyboardEvent | MouseEvent | PointerEvent
      > &
        DevExpress.events.ItemInfo
    ) => void;
    /**
     * [descr:dxDropDownListOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDropDownList.SelectionChangedInfo
    ) => void;
    /**
     * [descr:dxDropDownListOptions.onValueChanged]
     */
    onValueChanged?: (
      e: DevExpress.events.NativeEventInfo<
        TComponent,
        KeyboardEvent | MouseEvent | PointerEvent | Event
      > &
        DevExpress.ui.Editor.ValueChangedInfo
    ) => void;
    /**
     * [descr:dxDropDownListOptions.searchEnabled]
     */
    searchEnabled?: boolean;
    /**
     * [descr:dxDropDownListOptions.searchExpr]
     */
    searchExpr?: string | Function | Array<string | Function>;
    /**
     * [descr:dxDropDownListOptions.searchMode]
     */
    searchMode?: 'contains' | 'startswith';
    /**
     * [descr:dxDropDownListOptions.searchTimeout]
     */
    searchTimeout?: number;
    /**
     * [descr:dxDropDownListOptions.selectedItem]
     */
    selectedItem?: any;
    /**
     * [descr:dxDropDownListOptions.showDataBeforeSearch]
     */
    showDataBeforeSearch?: boolean;
    /**
     * [descr:dxDropDownListOptions.value]
     */
    value?: any;
    /**
     * [descr:dxDropDownListOptions.valueChangeEvent]
     */
    valueChangeEvent?: string;
    /**
     * [descr:dxDropDownListOptions.wrapItemText]
     */
    wrapItemText?: boolean;
    /**
     * [descr:dxDropDownListOptions.useItemTextAsTitle]
     */
    useItemTextAsTitle?: boolean;
  }
  /**
   * [descr:dxFileManager]
   */
  export class dxFileManager extends Widget<dxFileManagerOptions> {
    /**
     * [descr:dxFileManager.getCurrentDirectory()]
     */
    getCurrentDirectory(): any;
    /**
     * [descr:dxFileManager.getSelectedItems()]
     */
    getSelectedItems(): Array<any>;
    /**
     * [descr:dxFileManager.refresh()]
     */
    refresh(): DevExpress.core.utils.DxPromise<any>;
  }
  module dxFileManager {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface ActionEventInfo {
      errorCode?: number;
      errorText: string;
      cancel: boolean | PromiseLike<void>;
    }
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxFileManager>;
    export type ContextMenuItemClickEvent =
      DevExpress.events.NativeEventInfo<dxFileManager> & {
        readonly itemData: any;
        readonly itemElement: DevExpress.core.DxElement;
        readonly itemIndex: number;
        readonly fileSystemItem?: DevExpress.fileManagement.FileSystemItem;
        readonly viewArea: 'navPane' | 'itemView';
      };
    export type ContextMenuShowingEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxFileManager> & {
        readonly fileSystemItem?: DevExpress.fileManagement.FileSystemItem;
        readonly targetElement?: DevExpress.core.DxElement;
        readonly viewArea: 'navPane' | 'itemView';
      };
    export type CurrentDirectoryChangedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly directory: DevExpress.fileManagement.FileSystemItem;
      };
    export type DirectoryCreatedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly parentDirectory: DevExpress.fileManagement.FileSystemItem;
        readonly name: string;
      };
    export type DirectoryCreatingEvent =
      DevExpress.events.EventInfo<dxFileManager> &
        ActionEventInfo & {
          readonly parentDirectory: DevExpress.fileManagement.FileSystemItem;
          readonly name: string;
        };
    export type DisposingEvent = DevExpress.events.EventInfo<dxFileManager>;
    export type ErrorOccurredEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly errorCode?: number;
        errorText?: string;
        readonly fileSystemItem?: DevExpress.fileManagement.FileSystemItem;
      };
    export type FileUploadedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly fileData: File;
        readonly parentDirectory: DevExpress.fileManagement.FileSystemItem;
      };
    export type FileUploadingEvent =
      DevExpress.events.EventInfo<dxFileManager> &
        ActionEventInfo & {
          readonly fileData: File;
          readonly destinationDirectory: DevExpress.fileManagement.FileSystemItem;
        };
    export type FocusedItemChangedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly item?: DevExpress.fileManagement.FileSystemItem;
        readonly itemElement?: DevExpress.core.DxElement;
      };
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxFileManager>;
    export type ItemCopiedEvent = DevExpress.events.EventInfo<dxFileManager> & {
      readonly sourceItem: DevExpress.fileManagement.FileSystemItem;
      readonly parentDirectory: DevExpress.fileManagement.FileSystemItem;
      readonly itemName: string;
      readonly itemPath: string;
    };
    export type ItemCopyingEvent = DevExpress.events.EventInfo<dxFileManager> &
      ActionEventInfo & {
        readonly item: DevExpress.fileManagement.FileSystemItem;
        readonly destinationDirectory: DevExpress.fileManagement.FileSystemItem;
      };
    export type ItemDeletedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly item: DevExpress.fileManagement.FileSystemItem;
      };
    export type ItemDeletingEvent = DevExpress.events.EventInfo<dxFileManager> &
      ActionEventInfo & {
        readonly item: DevExpress.fileManagement.FileSystemItem;
      };
    export type ItemDownloadingEvent =
      DevExpress.events.EventInfo<dxFileManager> &
        ActionEventInfo & {
          readonly item: DevExpress.fileManagement.FileSystemItem;
        };
    export type ItemMovedEvent = DevExpress.events.EventInfo<dxFileManager> & {
      readonly sourceItem: DevExpress.fileManagement.FileSystemItem;
      readonly parentDirectory: DevExpress.fileManagement.FileSystemItem;
      readonly itemName: string;
      readonly itemPath: string;
    };
    export type ItemMovingEvent = DevExpress.events.EventInfo<dxFileManager> &
      ActionEventInfo & {
        readonly item: DevExpress.fileManagement.FileSystemItem;
        readonly destinationDirectory: DevExpress.fileManagement.FileSystemItem;
      };
    export type ItemRenamedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly sourceItem: DevExpress.fileManagement.FileSystemItem;
        readonly itemName: string;
      };
    export type ItemRenamingEvent = DevExpress.events.EventInfo<dxFileManager> &
      ActionEventInfo & {
        readonly item: DevExpress.fileManagement.FileSystemItem;
        readonly newName: string;
      };
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxFileManager> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxFileManagerOptions;
    export type SelectedFileOpenedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly file: DevExpress.fileManagement.FileSystemItem;
      };
    export type SelectionChangedEvent =
      DevExpress.events.EventInfo<dxFileManager> & {
        readonly currentSelectedItemKeys: Array<string>;
        readonly currentDeselectedItemKeys: Array<string>;
        readonly selectedItems: Array<DevExpress.fileManagement.FileSystemItem>;
        readonly selectedItemKeys: Array<string>;
      };
    export type ToolbarItemClickEvent =
      DevExpress.events.NativeEventInfo<dxFileManager> & {
        readonly itemData: any;
        readonly itemElement: DevExpress.core.DxElement;
        readonly itemIndex: number;
      };
  }
  /**
   * [descr:dxFileManagerContextMenu]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFileManagerContextMenu {
    /**
     * [descr:dxFileManagerContextMenu.items]
     */
    items?: Array<
      | DevExpress.ui.dxFileManager.ContextMenuItem
      | 'create'
      | 'upload'
      | 'refresh'
      | 'download'
      | 'move'
      | 'copy'
      | 'rename'
      | 'delete'
    >;
  }
  /**
   * @deprecated Use DevExpress.ui.dxFileManager.ContextMenuItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFileManagerContextMenuItem
    extends DevExpress.ui.dxContextMenu.Item {
    /**
     * [descr:dxFileManagerContextMenuItem.items]
     */
    items?: Array<DevExpress.ui.dxFileManager.ContextMenuItem>;
    /**
     * [descr:dxFileManagerContextMenuItem.name]
     */
    name?:
      | 'create'
      | 'upload'
      | 'refresh'
      | 'download'
      | 'move'
      | 'copy'
      | 'rename'
      | 'delete'
      | string;
    /**
     * [descr:dxFileManagerContextMenuItem.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFileManagerContextMenuItem.template]
     */
    template?:
      | DevExpress.core.template
      | (() => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * [descr:dxFileManagerDetailsColumn]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFileManagerDetailsColumn {
    /**
     * [descr:dxFileManagerDetailsColumn.alignment]
     */
    alignment?: 'center' | 'left' | 'right' | undefined;
    /**
     * [descr:dxFileManagerDetailsColumn.caption]
     */
    caption?: string;
    /**
     * [descr:dxFileManagerDetailsColumn.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxFileManagerDetailsColumn.dataField]
     */
    dataField?: string;
    /**
     * [descr:dxFileManagerDetailsColumn.dataType]
     */
    dataType?: 'string' | 'number' | 'date' | 'boolean' | 'object' | 'datetime';
    /**
     * [descr:dxFileManagerDetailsColumn.hidingPriority]
     */
    hidingPriority?: number;
    /**
     * [descr:dxFileManagerDetailsColumn.sortIndex]
     */
    sortIndex?: number;
    /**
     * [descr:dxFileManagerDetailsColumn.sortOrder]
     */
    sortOrder?: 'asc' | 'desc' | undefined;
    /**
     * [descr:dxFileManagerDetailsColumn.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFileManagerDetailsColumn.visibleIndex]
     */
    visibleIndex?: number;
    /**
     * [descr:dxFileManagerDetailsColumn.width]
     */
    width?: number | string;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFileManagerOptions extends WidgetOptions<dxFileManager> {
    /**
     * [descr:dxFileManagerOptions.allowedFileExtensions]
     */
    allowedFileExtensions?: Array<string>;
    /**
     * [descr:dxFileManagerOptions.contextMenu]
     */
    contextMenu?: dxFileManagerContextMenu;
    /**
     * [descr:dxFileManagerOptions.currentPath]
     */
    currentPath?: string;
    /**
     * [descr:dxFileManagerOptions.currentPathKeys]
     */
    currentPathKeys?: Array<string>;
    /**
     * [descr:dxFileManagerOptions.customizeDetailColumns]
     */
    customizeDetailColumns?: (
      columns: Array<dxFileManagerDetailsColumn>
    ) => Array<dxFileManagerDetailsColumn>;
    /**
     * [descr:dxFileManagerOptions.customizeThumbnail]
     */
    customizeThumbnail?: (
      fileSystemItem: DevExpress.fileManagement.FileSystemItem
    ) => string;
    /**
     * [descr:dxFileManagerOptions.fileSystemProvider]
     */
    fileSystemProvider?: any;
    /**
     * [descr:dxFileManagerOptions.itemView]
     */
    itemView?: {
      /**
       * [descr:dxFileManagerOptions.itemView.details]
       */
      details?: {
        /**
         * [descr:dxFileManagerOptions.itemView.details.columns]
         */
        columns?: Array<dxFileManagerDetailsColumn | string>;
      };
      /**
       * [descr:dxFileManagerOptions.itemView.mode]
       */
      mode?: 'details' | 'thumbnails';
      /**
       * [descr:dxFileManagerOptions.itemView.showFolders]
       */
      showFolders?: boolean;
      /**
       * [descr:dxFileManagerOptions.itemView.showParentFolder]
       */
      showParentFolder?: boolean;
    };
    /**
     * [descr:dxFileManagerOptions.notifications]
     */
    notifications?: {
      /**
       * [descr:dxFileManagerOptions.notifications.showPanel]
       */
      showPanel?: boolean;
      /**
       * [descr:dxFileManagerOptions.notifications.showPopup]
       */
      showPopup?: boolean;
    };
    /**
     * [descr:dxFileManagerOptions.onContextMenuItemClick]
     */
    onContextMenuItemClick?: (
      e: DevExpress.ui.dxFileManager.ContextMenuItemClickEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onContextMenuShowing]
     */
    onContextMenuShowing?: (
      e: DevExpress.ui.dxFileManager.ContextMenuShowingEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onCurrentDirectoryChanged]
     */
    onCurrentDirectoryChanged?: (
      e: DevExpress.ui.dxFileManager.CurrentDirectoryChangedEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onSelectedFileOpened]
     */
    onSelectedFileOpened?: (
      e: DevExpress.ui.dxFileManager.SelectedFileOpenedEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.ui.dxFileManager.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onToolbarItemClick]
     */
    onToolbarItemClick?: (
      e: DevExpress.ui.dxFileManager.ToolbarItemClickEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onFocusedItemChanged]
     */
    onFocusedItemChanged?: (
      e: DevExpress.ui.dxFileManager.FocusedItemChangedEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onErrorOccurred]
     */
    onErrorOccurred?: (
      e: DevExpress.ui.dxFileManager.ErrorOccurredEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onDirectoryCreating]
     */
    onDirectoryCreating?: (
      e: DevExpress.ui.dxFileManager.DirectoryCreatingEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onDirectoryCreated]
     */
    onDirectoryCreated?: (
      e: DevExpress.ui.dxFileManager.DirectoryCreatedEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onItemRenaming]
     */
    onItemRenaming?: (e: DevExpress.ui.dxFileManager.ItemRenamingEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemRenamed]
     */
    onItemRenamed?: (e: DevExpress.ui.dxFileManager.ItemRenamedEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemMoving]
     */
    onItemMoving?: (e: DevExpress.ui.dxFileManager.ItemMovingEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemMoved]
     */
    onItemMoved?: (e: DevExpress.ui.dxFileManager.ItemMovedEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemCopying]
     */
    onItemCopying?: (e: DevExpress.ui.dxFileManager.ItemCopyingEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemCopied]
     */
    onItemCopied?: (e: DevExpress.ui.dxFileManager.ItemCopiedEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemDeleting]
     */
    onItemDeleting?: (e: DevExpress.ui.dxFileManager.ItemDeletingEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemDeleted]
     */
    onItemDeleted?: (e: DevExpress.ui.dxFileManager.ItemDeletedEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onFileUploading]
     */
    onFileUploading?: (
      e: DevExpress.ui.dxFileManager.FileUploadingEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.onFileUploaded]
     */
    onFileUploaded?: (e: DevExpress.ui.dxFileManager.FileUploadedEvent) => void;
    /**
     * [descr:dxFileManagerOptions.onItemDownloading]
     */
    onItemDownloading?: (
      e: DevExpress.ui.dxFileManager.ItemDownloadingEvent
    ) => void;
    /**
     * [descr:dxFileManagerOptions.permissions]
     */
    permissions?: {
      /**
       * [descr:dxFileManagerOptions.permissions.copy]
       */
      copy?: boolean;
      /**
       * [descr:dxFileManagerOptions.permissions.create]
       */
      create?: boolean;
      /**
       * [descr:dxFileManagerOptions.permissions.download]
       */
      download?: boolean;
      /**
       * [descr:dxFileManagerOptions.permissions.move]
       */
      move?: boolean;
      /**
       * [descr:dxFileManagerOptions.permissions.delete]
       */
      delete?: boolean;
      /**
       * [descr:dxFileManagerOptions.permissions.rename]
       */
      rename?: boolean;
      /**
       * [descr:dxFileManagerOptions.permissions.upload]
       */
      upload?: boolean;
    };
    /**
     * [descr:dxFileManagerOptions.rootFolderName]
     */
    rootFolderName?: string;
    /**
     * [descr:dxFileManagerOptions.selectionMode]
     */
    selectionMode?: 'multiple' | 'single';
    /**
     * [descr:dxFileManagerOptions.selectedItemKeys]
     */
    selectedItemKeys?: Array<string>;
    /**
     * [descr:dxFileManagerOptions.focusedItemKey]
     */
    focusedItemKey?: string;
    /**
     * [descr:dxFileManagerOptions.toolbar]
     */
    toolbar?: dxFileManagerToolbar;
    /**
     * [descr:dxFileManagerOptions.upload]
     */
    upload?: {
      /**
       * [descr:dxFileManagerOptions.upload.maxFileSize]
       */
      maxFileSize?: number;
      /**
       * [descr:dxFileManagerOptions.upload.chunkSize]
       */
      chunkSize?: number;
    };
  }
  /**
   * [descr:dxFileManagerToolbar]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFileManagerToolbar {
    /**
     * [descr:dxFileManagerToolbar.fileSelectionItems]
     */
    fileSelectionItems?: Array<
      | DevExpress.ui.dxFileManager.ToolbarItem
      | 'showNavPane'
      | 'create'
      | 'upload'
      | 'refresh'
      | 'switchView'
      | 'download'
      | 'move'
      | 'copy'
      | 'rename'
      | 'delete'
      | 'clearSelection'
      | 'separator'
    >;
    /**
     * [descr:dxFileManagerToolbar.items]
     */
    items?: Array<
      | DevExpress.ui.dxFileManager.ToolbarItem
      | 'showNavPane'
      | 'create'
      | 'upload'
      | 'refresh'
      | 'switchView'
      | 'download'
      | 'move'
      | 'copy'
      | 'rename'
      | 'delete'
      | 'clearSelection'
      | 'separator'
    >;
  }
  /**
   * @deprecated Use ToolbarItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFileManagerToolbarItem
    extends DevExpress.ui.dxToolbar.Item {
    /**
     * [descr:dxFileManagerToolbarItem.icon]
     */
    icon?: string;
    /**
     * [descr:dxFileManagerToolbarItem.location]
     */
    location?: 'after' | 'before' | 'center';
    /**
     * [descr:dxFileManagerToolbarItem.name]
     */
    name?:
      | 'showNavPane'
      | 'create'
      | 'upload'
      | 'refresh'
      | 'switchView'
      | 'download'
      | 'move'
      | 'copy'
      | 'rename'
      | 'delete'
      | 'clearSelection'
      | 'separator'
      | string;
    /**
     * [descr:dxFileManagerToolbarItem.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFileManagerToolbarItem.html]
     */
    html?: string;
    /**
     * [descr:dxFileManagerToolbarItem.template]
     */
    template?:
      | DevExpress.core.template
      | (() => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxFileManagerToolbarItem.menuItemTemplate]
     */
    menuItemTemplate?:
      | DevExpress.core.template
      | (() => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * [descr:dxFileUploader]
   */
  export class dxFileUploader extends Editor<dxFileUploaderOptions> {
    /**
     * [descr:dxFileUploader.upload()]
     */
    upload(): void;
    /**
     * [descr:dxFileUploader.upload(fileIndex)]
     */
    upload(fileIndex: number): void;
    /**
     * [descr:dxFileUploader.upload(file)]
     */
    upload(file: File): void;
    /**
     * [descr:dxFileUploader.abortUpload()]
     */
    abortUpload(): void;
    /**
     * [descr:dxFileUploader.abortUpload(fileIndex)]
     */
    abortUpload(fileIndex: number): void;
    /**
     * [descr:dxFileUploader.abortUpload(file)]
     */
    abortUpload(file: File): void;
    /**
     * [descr:dxFileUploader.removeFile(fileIndex)]
     */
    removeFile(fileIndex: number): void;
    /**
     * [descr:dxFileUploader.removeFile(file)]
     */
    removeFile(file: File): void;
  }
  module dxFileUploader {
    export type BeforeSendEvent =
      DevExpress.events.EventInfo<dxFileUploader> & {
        readonly request: XMLHttpRequest;
        readonly file: File;
        readonly uploadInfo?: DevExpress.fileManagement.UploadInfo;
      };
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxFileUploader>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxFileUploader>;
    export type DropZoneEnterEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly dropZoneElement: DevExpress.core.DxElement;
      };
    export type DropZoneLeaveEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly dropZoneElement: DevExpress.core.DxElement;
      };
    export type FilesUploadedEvent =
      DevExpress.events.EventInfo<dxFileUploader>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxFileUploader>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxFileUploader> &
        DevExpress.events.ChangedOptionInfo;
    export type ProgressEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly file: File;
        readonly segmentSize: number;
        readonly bytesLoaded: number;
        readonly bytesTotal: number;
        readonly request: XMLHttpRequest;
      };
    export type Properties = dxFileUploaderOptions;
    export type UploadAbortedEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly file: File;
        readonly request: XMLHttpRequest;
        message: string;
      };
    export type UploadedEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly file: File;
        readonly request: XMLHttpRequest;
        message: string;
      };
    export type UploadErrorEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly file: File;
        readonly request: XMLHttpRequest;
        readonly error: any;
        message: string;
      };
    export type UploadStartedEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly file: File;
        readonly request: XMLHttpRequest;
      };
    export type ValueChangedEvent =
      DevExpress.events.NativeEventInfo<dxFileUploader> & {
        readonly value?: Array<File>;
        readonly previousValue?: Array<File>;
      };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFileUploaderOptions extends EditorOptions<dxFileUploader> {
    /**
     * [descr:dxFileUploaderOptions.abortUpload]
     */
    abortUpload?: (
      file: File,
      uploadInfo?: DevExpress.fileManagement.UploadInfo
    ) => PromiseLike<any> | any;
    /**
     * [descr:dxFileUploaderOptions.accept]
     */
    accept?: string;
    /**
     * [descr:dxFileUploaderOptions.allowCanceling]
     */
    allowCanceling?: boolean;
    /**
     * [descr:dxFileUploaderOptions.allowedFileExtensions]
     */
    allowedFileExtensions?: Array<string>;
    /**
     * [descr:dxFileUploaderOptions.chunkSize]
     */
    chunkSize?: number;
    /**
     * [descr:dxFileUploaderOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxFileUploaderOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxFileUploaderOptions.invalidFileExtensionMessage]
     */
    invalidFileExtensionMessage?: string;
    /**
     * [descr:dxFileUploaderOptions.invalidMaxFileSizeMessage]
     */
    invalidMaxFileSizeMessage?: string;
    /**
     * [descr:dxFileUploaderOptions.invalidMinFileSizeMessage]
     */
    invalidMinFileSizeMessage?: string;
    /**
     * [descr:dxFileUploaderOptions.inputAttr]
     */
    inputAttr?: any;
    /**
     * [descr:dxFileUploaderOptions.labelText]
     */
    labelText?: string;
    /**
     * [descr:dxFileUploaderOptions.maxFileSize]
     */
    maxFileSize?: number;
    /**
     * [descr:dxFileUploaderOptions.minFileSize]
     */
    minFileSize?: number;
    /**
     * [descr:dxFileUploaderOptions.multiple]
     */
    multiple?: boolean;
    /**
     * [descr:dxFileUploaderOptions.name]
     */
    name?: string;
    /**
     * [descr:dxFileUploaderOptions.onBeforeSend]
     */
    onBeforeSend?: (e: DevExpress.ui.dxFileUploader.BeforeSendEvent) => void;
    /**
     * [descr:dxFileUploaderOptions.onDropZoneEnter]
     */
    onDropZoneEnter?: (
      e: DevExpress.ui.dxFileUploader.DropZoneEnterEvent
    ) => void;
    /**
     * [descr:dxFileUploaderOptions.onDropZoneLeave]
     */
    onDropZoneLeave?: (
      e: DevExpress.ui.dxFileUploader.DropZoneLeaveEvent
    ) => void;
    /**
     * [descr:dxFileUploaderOptions.onFilesUploaded]
     */
    onFilesUploaded?: (
      e: DevExpress.ui.dxFileUploader.FilesUploadedEvent
    ) => void;
    /**
     * [descr:dxFileUploaderOptions.onProgress]
     */
    onProgress?: (e: DevExpress.ui.dxFileUploader.ProgressEvent) => void;
    /**
     * [descr:dxFileUploaderOptions.onUploadAborted]
     */
    onUploadAborted?: (
      e: DevExpress.ui.dxFileUploader.UploadAbortedEvent
    ) => void;
    /**
     * [descr:dxFileUploaderOptions.onUploadError]
     */
    onUploadError?: (e: DevExpress.ui.dxFileUploader.UploadErrorEvent) => void;
    /**
     * [descr:dxFileUploaderOptions.onUploadStarted]
     */
    onUploadStarted?: (
      e: DevExpress.ui.dxFileUploader.UploadStartedEvent
    ) => void;
    /**
     * [descr:dxFileUploaderOptions.onUploaded]
     */
    onUploaded?: (e: DevExpress.ui.dxFileUploader.UploadedEvent) => void;
    /**
     * [descr:dxFileUploaderOptions.onValueChanged]
     */
    onValueChanged?: (
      e: DevExpress.ui.dxFileUploader.ValueChangedEvent
    ) => void;
    /**
     * [descr:dxFileUploaderOptions.progress]
     */
    progress?: number;
    /**
     * [descr:dxFileUploaderOptions.readyToUploadMessage]
     */
    readyToUploadMessage?: string;
    /**
     * [descr:dxFileUploaderOptions.selectButtonText]
     */
    selectButtonText?: string;
    /**
     * [descr:dxFileUploaderOptions.showFileList]
     */
    showFileList?: boolean;
    /**
     * [descr:dxFileUploaderOptions.dialogTrigger]
     */
    dialogTrigger?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxFileUploaderOptions.dropZone]
     */
    dropZone?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxFileUploaderOptions.uploadButtonText]
     */
    uploadButtonText?: string;
    /**
     * [descr:dxFileUploaderOptions.uploadChunk]
     */
    uploadChunk?: (
      file: File,
      uploadInfo: DevExpress.fileManagement.UploadInfo
    ) => PromiseLike<any> | any;
    /**
     * [descr:dxFileUploaderOptions.uploadFailedMessage]
     */
    uploadFailedMessage?: string;
    /**
     * [descr:dxFileUploaderOptions.uploadAbortedMessage]
     */
    uploadAbortedMessage?: string;
    /**
     * [descr:dxFileUploaderOptions.uploadFile]
     */
    uploadFile?: (
      file: File,
      progressCallback: Function
    ) => PromiseLike<any> | any;
    /**
     * [descr:dxFileUploaderOptions.uploadHeaders]
     */
    uploadHeaders?: any;
    /**
     * [descr:dxFileUploaderOptions.uploadCustomData]
     */
    uploadCustomData?: any;
    /**
     * [descr:dxFileUploaderOptions.uploadMethod]
     */
    uploadMethod?: 'POST' | 'PUT';
    /**
     * [descr:dxFileUploaderOptions.uploadMode]
     */
    uploadMode?: 'instantly' | 'useButtons' | 'useForm';
    /**
     * [descr:dxFileUploaderOptions.uploadUrl]
     */
    uploadUrl?: string;
    /**
     * [descr:dxFileUploaderOptions.uploadedMessage]
     */
    uploadedMessage?: string;
    /**
     * [descr:dxFileUploaderOptions.value]
     */
    value?: Array<File>;
  }
  /**
   * [descr:dxFilterBuilder]
   */
  export class dxFilterBuilder extends Widget<dxFilterBuilderOptions> {
    /**
     * [descr:dxFilterBuilder.getFilterExpression()]
     */
    getFilterExpression(): string | Array<any> | Function;
  }
  module dxFilterBuilder {
    export type ContentReadyEvent =
      DevExpress.events.EventInfo<dxFilterBuilder>;
    export type CustomOperationEditorTemplate = {
      readonly value?: string | number | Date;
      readonly field: dxFilterBuilderField;
      readonly setValue: Function;
    };
    export type DisposingEvent = DevExpress.events.EventInfo<dxFilterBuilder>;
    export type EditorPreparedEvent =
      DevExpress.events.EventInfo<dxFilterBuilder> & {
        readonly value?: any;
        readonly setValue: any;
        readonly editorElement: DevExpress.core.DxElement;
        readonly editorName: string;
        readonly dataField?: string;
        readonly filterOperation?: string;
        readonly updateValueTimeout?: number;
        readonly width?: number;
        readonly readOnly: boolean;
        readonly disabled: boolean;
        readonly rtlEnabled: boolean;
      };
    export type EditorPreparingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxFilterBuilder> & {
        readonly value?: any;
        readonly setValue: any;
        readonly editorElement?: DevExpress.core.DxElement;
        editorName: string;
        editorOptions?: any;
        readonly dataField?: string;
        readonly filterOperation?: string;
        updateValueTimeout?: number;
        readonly width?: number;
        readonly readOnly: boolean;
        readonly disabled: boolean;
        readonly rtlEnabled: boolean;
      };
    export type FieldEditorTemplate = {
      readonly value?: string | number | Date;
      readonly filterOperation?: string;
      readonly field: dxFilterBuilderField;
      readonly setValue: Function;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type FilterLookupDataSource<T> = Exclude<
      DevExpress.data.DataSource.DataSourceLike<T>,
      string | DevExpress.data.DataSource
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxFilterBuilder>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxFilterBuilder> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxFilterBuilderOptions;
    export type ValueChangedEvent =
      DevExpress.events.EventInfo<dxFilterBuilder> & {
        readonly value?: any;
        readonly previousValue?: any;
      };
  }
  /**
   * [descr:dxFilterBuilderCustomOperation]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFilterBuilderCustomOperation {
    /**
     * [descr:dxFilterBuilderCustomOperation.calculateFilterExpression]
     */
    calculateFilterExpression?: (
      filterValue: any,
      field: dxFilterBuilderField
    ) => string | Array<any> | Function;
    /**
     * [descr:dxFilterBuilderCustomOperation.caption]
     */
    caption?: string;
    /**
     * [descr:dxFilterBuilderCustomOperation.customizeText]
     */
    customizeText?: (fieldInfo: {
      value?: string | number | Date;
      valueText?: string;
      field?: dxFilterBuilderField;
    }) => string;
    /**
     * [descr:dxFilterBuilderCustomOperation.dataTypes]
     */
    dataTypes?: Array<
      'string' | 'number' | 'date' | 'boolean' | 'object' | 'datetime'
    >;
    /**
     * [descr:dxFilterBuilderCustomOperation.editorTemplate]
     */
    editorTemplate?:
      | DevExpress.core.template
      | ((
          conditionInfo: DevExpress.ui.dxFilterBuilder.CustomOperationEditorTemplate,
          container: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxFilterBuilderCustomOperation.hasValue]
     */
    hasValue?: boolean;
    /**
     * [descr:dxFilterBuilderCustomOperation.icon]
     */
    icon?: string;
    /**
     * [descr:dxFilterBuilderCustomOperation.name]
     */
    name?: string;
  }
  /**
   * [descr:dxFilterBuilderField]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFilterBuilderField {
    /**
     * [descr:dxFilterBuilderField.calculateFilterExpression]
     */
    calculateFilterExpression?: (
      filterValue: any,
      selectedFilterOperation: string
    ) => string | Array<any> | Function;
    /**
     * [descr:dxFilterBuilderField.caption]
     */
    caption?: string;
    /**
     * [descr:dxFilterBuilderField.customizeText]
     */
    customizeText?: (fieldInfo: {
      value?: string | number | Date;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxFilterBuilderField.dataField]
     */
    dataField?: string;
    /**
     * [descr:dxFilterBuilderField.dataType]
     */
    dataType?: 'string' | 'number' | 'date' | 'boolean' | 'object' | 'datetime';
    /**
     * [descr:dxFilterBuilderField.editorOptions]
     */
    editorOptions?: any;
    /**
     * [descr:dxFilterBuilderField.editorTemplate]
     */
    editorTemplate?:
      | DevExpress.core.template
      | ((
          conditionInfo: DevExpress.ui.dxFilterBuilder.FieldEditorTemplate,
          container: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxFilterBuilderField.falseText]
     */
    falseText?: string;
    /**
     * [descr:dxFilterBuilderField.filterOperations]
     */
    filterOperations?: Array<
      | '='
      | '<>'
      | '<'
      | '<='
      | '>'
      | '>='
      | 'contains'
      | 'endswith'
      | 'isblank'
      | 'isnotblank'
      | 'notcontains'
      | 'startswith'
      | 'between'
      | string
    >;
    /**
     * [descr:dxFilterBuilderField.format]
     */
    format?: Format;
    /**
     * [descr:dxFilterBuilderField.lookup]
     */
    lookup?: {
      /**
       * [descr:dxFilterBuilderField.lookup.allowClearing]
       */
      allowClearing?: boolean;
      /**
       * [descr:dxFilterBuilderField.lookup.dataSource]
       */
      dataSource?: DevExpress.ui.dxFilterBuilder.FilterLookupDataSource<any>;
      /**
       * [descr:dxFilterBuilderField.lookup.displayExpr]
       */
      displayExpr?: string | ((data: any) => string);
      /**
       * [descr:dxFilterBuilderField.lookup.valueExpr]
       */
      valueExpr?: string | ((data: any) => string | number | boolean);
    };
    /**
     * [descr:dxFilterBuilderField.name]
     */
    name?: string;
    /**
     * [descr:dxFilterBuilderField.trueText]
     */
    trueText?: string;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFilterBuilderOptions
    extends WidgetOptions<dxFilterBuilder> {
    /**
     * [descr:dxFilterBuilderOptions.allowHierarchicalFields]
     */
    allowHierarchicalFields?: boolean;
    /**
     * [descr:dxFilterBuilderOptions.customOperations]
     */
    customOperations?: Array<dxFilterBuilderCustomOperation>;
    /**
     * [descr:dxFilterBuilderOptions.fields]
     */
    fields?: Array<dxFilterBuilderField>;
    /**
     * [descr:dxFilterBuilderOptions.filterOperationDescriptions]
     */
    filterOperationDescriptions?: {
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.between]
       */
      between?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.contains]
       */
      contains?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.endsWith]
       */
      endsWith?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.equal]
       */
      equal?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.greaterThan]
       */
      greaterThan?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.greaterThanOrEqual]
       */
      greaterThanOrEqual?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.isBlank]
       */
      isBlank?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.isNotBlank]
       */
      isNotBlank?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.lessThan]
       */
      lessThan?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.lessThanOrEqual]
       */
      lessThanOrEqual?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.notContains]
       */
      notContains?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.notEqual]
       */
      notEqual?: string;
      /**
       * [descr:dxFilterBuilderOptions.filterOperationDescriptions.startsWith]
       */
      startsWith?: string;
    };
    /**
     * [descr:dxFilterBuilderOptions.groupOperationDescriptions]
     */
    groupOperationDescriptions?: {
      /**
       * [descr:dxFilterBuilderOptions.groupOperationDescriptions.and]
       */
      and?: string;
      /**
       * [descr:dxFilterBuilderOptions.groupOperationDescriptions.notAnd]
       */
      notAnd?: string;
      /**
       * [descr:dxFilterBuilderOptions.groupOperationDescriptions.notOr]
       */
      notOr?: string;
      /**
       * [descr:dxFilterBuilderOptions.groupOperationDescriptions.or]
       */
      or?: string;
    };
    /**
     * [descr:dxFilterBuilderOptions.groupOperations]
     */
    groupOperations?: Array<'and' | 'or' | 'notAnd' | 'notOr'>;
    /**
     * [descr:dxFilterBuilderOptions.maxGroupLevel]
     */
    maxGroupLevel?: number;
    /**
     * [descr:dxFilterBuilderOptions.onEditorPrepared]
     */
    onEditorPrepared?: (
      e: DevExpress.ui.dxFilterBuilder.EditorPreparedEvent
    ) => void;
    /**
     * [descr:dxFilterBuilderOptions.onEditorPreparing]
     */
    onEditorPreparing?: (
      e: DevExpress.ui.dxFilterBuilder.EditorPreparingEvent
    ) => void;
    /**
     * [descr:dxFilterBuilderOptions.onValueChanged]
     */
    onValueChanged?: (
      e: DevExpress.ui.dxFilterBuilder.ValueChangedEvent
    ) => void;
    /**
     * [descr:dxFilterBuilderOptions.value]
     */
    value?: string | Array<any> | Function;
  }
  /**
   * [descr:dxForm]
   */
  export class dxForm extends Widget<dxFormOptions> {
    /**
     * [descr:dxForm.getButton(name)]
     */
    getButton(name: string): dxButton | undefined;
    /**
     * [descr:dxForm.getEditor(dataField)]
     */
    getEditor(dataField: string): Editor | undefined;
    /**
     * [descr:dxForm.itemOption(id)]
     */
    itemOption(id: string): any;
    /**
     * [descr:dxForm.itemOption(id, option, value)]
     */
    itemOption(id: string, option: string, value: any): void;
    /**
     * [descr:dxForm.itemOption(id, options)]
     */
    itemOption(id: string, options: any): void;
    /**
     * [descr:dxForm.resetValues()]
     */
    resetValues(): void;
    /**
     * [descr:dxForm.updateData(data)]
     */
    updateData(data: any): void;
    /**
     * [descr:dxForm.updateData(dataField, value)]
     */
    updateData(dataField: string, value: any): void;
    /**
     * [descr:dxForm.updateDimensions()]
     */
    updateDimensions(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxForm.validate()]
     */
    validate(): DevExpress.ui.dxValidationGroup.ValidationResult;
  }
  module dxForm {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxForm>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxForm>;
    export type EditorEnterKeyEvent = DevExpress.events.EventInfo<dxForm> & {
      readonly dataField?: string;
    };
    export type FieldDataChangedEvent = DevExpress.events.EventInfo<dxForm> & {
      readonly dataField?: string;
      readonly value?: any;
    };
    export type GroupItemTemplateData = {
      readonly component: dxForm;
      readonly formData?: any;
    };
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxForm>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxForm> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxFormOptions;
    export type SimpleItemTemplateData = {
      readonly component: dxForm;
      readonly dataField?: string;
      readonly editorOptions?: any;
      readonly editorType?: string;
      readonly name?: string;
    };
  }
  /**
   * @deprecated Use ButtonItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFormButtonItem {
    /**
     * [descr:dxFormButtonItem.buttonOptions]
     */
    buttonOptions?: dxButtonOptions;
    /**
     * [descr:dxFormButtonItem.colSpan]
     */
    colSpan?: number;
    /**
     * [descr:dxFormButtonItem.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxFormButtonItem.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxFormButtonItem.itemType]
     */
    itemType?: 'empty' | 'group' | 'simple' | 'tabbed' | 'button';
    /**
     * [descr:dxFormButtonItem.name]
     */
    name?: string;
    /**
     * [descr:dxFormButtonItem.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'center' | 'top';
    /**
     * [descr:dxFormButtonItem.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFormButtonItem.visibleIndex]
     */
    visibleIndex?: number;
  }
  /**
   * @deprecated Use EmptyItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFormEmptyItem {
    /**
     * [descr:dxFormEmptyItem.colSpan]
     */
    colSpan?: number;
    /**
     * [descr:dxFormEmptyItem.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxFormEmptyItem.itemType]
     */
    itemType?: 'empty' | 'group' | 'simple' | 'tabbed' | 'button';
    /**
     * [descr:dxFormEmptyItem.name]
     */
    name?: string;
    /**
     * [descr:dxFormEmptyItem.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFormEmptyItem.visibleIndex]
     */
    visibleIndex?: number;
  }
  /**
   * @deprecated Use GroupItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFormGroupItem {
    /**
     * [descr:dxFormGroupItem.alignItemLabels]
     */
    alignItemLabels?: boolean;
    /**
     * [descr:dxFormGroupItem.caption]
     */
    caption?: string;
    /**
     * [descr:dxFormGroupItem.colCount]
     */
    colCount?: number;
    /**
     * [descr:dxFormGroupItem.colCountByScreen]
     */
    colCountByScreen?: any;
    /**
     * [descr:dxFormGroupItem.colSpan]
     */
    colSpan?: number;
    /**
     * [descr:dxFormGroupItem.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxFormGroupItem.itemType]
     */
    itemType?: 'empty' | 'group' | 'simple' | 'tabbed' | 'button';
    /**
     * [descr:dxFormGroupItem.items]
     */
    items?: Array<DevExpress.ui.dxForm.Item>;
    /**
     * [descr:dxFormGroupItem.name]
     */
    name?: string;
    /**
     * [descr:dxFormGroupItem.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          data: DevExpress.ui.dxForm.GroupItemTemplateData,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxFormGroupItem.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFormGroupItem.visibleIndex]
     */
    visibleIndex?: number;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFormOptions extends WidgetOptions<dxForm> {
    /**
     * [descr:dxFormOptions.alignItemLabels]
     */
    alignItemLabels?: boolean;
    /**
     * [descr:dxFormOptions.alignItemLabelsInAllGroups]
     */
    alignItemLabelsInAllGroups?: boolean;
    /**
     * [descr:dxFormOptions.colCount]
     */
    colCount?: number | 'auto';
    /**
     * [descr:dxFormOptions.colCountByScreen]
     */
    colCountByScreen?: any;
    /**
     * [descr:dxFormOptions.customizeItem]
     */
    customizeItem?: (item: DevExpress.ui.dxForm.Item) => void;
    /**
     * [descr:dxFormOptions.formData]
     */
    formData?: any;
    /**
     * [descr:dxFormOptions.items]
     */
    items?: Array<DevExpress.ui.dxForm.Item>;
    /**
     * [descr:dxFormOptions.labelLocation]
     */
    labelLocation?: 'left' | 'right' | 'top';
    /**
     * [descr:dxFormOptions.labelMode]
     */
    labelMode?: 'outside' | 'floating' | 'static' | 'hidden';
    /**
     * [descr:dxFormOptions.minColWidth]
     */
    minColWidth?: number;
    /**
     * [descr:dxFormOptions.onEditorEnterKey]
     */
    onEditorEnterKey?: (e: DevExpress.ui.dxForm.EditorEnterKeyEvent) => void;
    /**
     * [descr:dxFormOptions.onFieldDataChanged]
     */
    onFieldDataChanged?: (
      e: DevExpress.ui.dxForm.FieldDataChangedEvent
    ) => void;
    /**
     * [descr:dxFormOptions.optionalMark]
     */
    optionalMark?: string;
    /**
     * [descr:dxFormOptions.readOnly]
     */
    readOnly?: boolean;
    /**
     * [descr:dxFormOptions.requiredMark]
     */
    requiredMark?: string;
    /**
     * [descr:dxFormOptions.requiredMessage]
     */
    requiredMessage?: string;
    /**
     * [descr:dxFormOptions.screenByWidth]
     */
    screenByWidth?: Function;
    /**
     * [descr:dxFormOptions.scrollingEnabled]
     */
    scrollingEnabled?: boolean;
    /**
     * [descr:dxFormOptions.showColonAfterLabel]
     */
    showColonAfterLabel?: boolean;
    /**
     * [descr:dxFormOptions.showOptionalMark]
     */
    showOptionalMark?: boolean;
    /**
     * [descr:dxFormOptions.showRequiredMark]
     */
    showRequiredMark?: boolean;
    /**
     * [descr:dxFormOptions.showValidationSummary]
     */
    showValidationSummary?: boolean;
    /**
     * [descr:dxFormOptions.validationGroup]
     */
    validationGroup?: string;
  }
  /**
   * @deprecated Use SimpleItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFormSimpleItem {
    /**
     * [descr:dxFormSimpleItem.colSpan]
     */
    colSpan?: number;
    /**
     * [descr:dxFormSimpleItem.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxFormSimpleItem.dataField]
     */
    dataField?: string;
    /**
     * [descr:dxFormSimpleItem.editorOptions]
     */
    editorOptions?: any;
    /**
     * [descr:dxFormSimpleItem.editorType]
     */
    editorType?:
      | 'dxAutocomplete'
      | 'dxCalendar'
      | 'dxCheckBox'
      | 'dxColorBox'
      | 'dxDateBox'
      | 'dxDropDownBox'
      | 'dxHtmlEditor'
      | 'dxLookup'
      | 'dxNumberBox'
      | 'dxRadioGroup'
      | 'dxRangeSlider'
      | 'dxSelectBox'
      | 'dxSlider'
      | 'dxSwitch'
      | 'dxTagBox'
      | 'dxTextArea'
      | 'dxTextBox';
    /**
     * [descr:dxFormSimpleItem.helpText]
     */
    helpText?: string;
    /**
     * [descr:dxFormSimpleItem.isRequired]
     */
    isRequired?: boolean;
    /**
     * [descr:dxFormSimpleItem.itemType]
     */
    itemType?: 'empty' | 'group' | 'simple' | 'tabbed' | 'button';
    /**
     * [descr:dxFormSimpleItem.label]
     */
    label?: {
      /**
       * [descr:dxFormSimpleItem.label.alignment]
       */
      alignment?: 'center' | 'left' | 'right';
      /**
       * [descr:dxFormSimpleItem.label.location]
       */
      location?: 'left' | 'right' | 'top';
      /**
       * [descr:dxFormSimpleItem.label.showColon]
       */
      showColon?: boolean;
      /**
       * [descr:dxFormSimpleItem.label.text]
       */
      text?: string;
      /**
       * [descr:dxFormSimpleItem.label.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxFormSimpleItem.name]
     */
    name?: string;
    /**
     * [descr:dxFormSimpleItem.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          data: DevExpress.ui.dxForm.SimpleItemTemplateData,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxFormSimpleItem.validationRules]
     */
    validationRules?: Array<
      | RequiredRule
      | NumericRule
      | RangeRule
      | StringLengthRule
      | CustomRule
      | CompareRule
      | PatternRule
      | EmailRule
      | AsyncRule
    >;
    /**
     * [descr:dxFormSimpleItem.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFormSimpleItem.visibleIndex]
     */
    visibleIndex?: number;
  }
  /**
   * @deprecated Use TabbedItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFormTabbedItem {
    /**
     * [descr:dxFormTabbedItem.colSpan]
     */
    colSpan?: number;
    /**
     * [descr:dxFormTabbedItem.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxFormTabbedItem.itemType]
     */
    itemType?: 'empty' | 'group' | 'simple' | 'tabbed' | 'button';
    /**
     * [descr:dxFormTabbedItem.name]
     */
    name?: string;
    /**
     * [descr:dxFormTabbedItem.tabPanelOptions]
     */
    tabPanelOptions?: dxTabPanelOptions;
    /**
     * [descr:dxFormTabbedItem.tabs]
     */
    tabs?: Array<{
      /**
       * [descr:dxFormTabbedItem.tabs.alignItemLabels]
       */
      alignItemLabels?: boolean;
      /**
       * [descr:dxFormTabbedItem.tabs.badge]
       */
      badge?: string;
      /**
       * [descr:dxFormTabbedItem.tabs.colCount]
       */
      colCount?: number;
      /**
       * [descr:dxFormTabbedItem.tabs.colCountByScreen]
       */
      colCountByScreen?: any;
      /**
       * [descr:dxFormTabbedItem.tabs.disabled]
       */
      disabled?: boolean;
      /**
       * [descr:dxFormTabbedItem.tabs.icon]
       */
      icon?: string;
      /**
       * [descr:dxFormTabbedItem.tabs.items]
       */
      items?: Array<DevExpress.ui.dxForm.Item>;
      /**
       * [descr:dxFormTabbedItem.tabs.tabTemplate]
       */
      tabTemplate?:
        | DevExpress.core.template
        | ((
            tabData: any,
            tabIndex: number,
            tabElement: DevExpress.core.DxElement
          ) => any);
      /**
       * [descr:dxFormTabbedItem.tabs.template]
       */
      template?:
        | DevExpress.core.template
        | ((
            tabData: any,
            tabIndex: number,
            tabElement: DevExpress.core.DxElement
          ) => any);
      /**
       * [descr:dxFormTabbedItem.tabs.title]
       */
      title?: string;
    }>;
    /**
     * [descr:dxFormTabbedItem.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxFormTabbedItem.visibleIndex]
     */
    visibleIndex?: number;
  }
  /**
   * [descr:dxGallery]
   */
  export class dxGallery<
    TItem extends DevExpress.ui.dxGallery.ItemLike = any,
    TKey = any
  > extends CollectionWidget<dxGalleryOptions<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxGallery.goToItem(itemIndex, animation)]
     */
    goToItem(
      itemIndex: number,
      animation: boolean
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxGallery.nextItem(animation)]
     */
    nextItem(animation: boolean): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxGallery.prevItem(animation)]
     */
    prevItem(animation: boolean): DevExpress.core.utils.DxPromise<void>;
  }
  module dxGallery {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxGallery<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxGallery<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxGallery<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxGallery<TItem, TKey>,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxGallery<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxGallery<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxGallery<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxGallery<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxGalleryOptions<TItem, TKey>;
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxGallery<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGalleryItem extends CollectionWidgetItem {
    /**
     * [descr:dxGalleryItem.imageAlt]
     */
    imageAlt?: string;
    /**
     * [descr:dxGalleryItem.imageSrc]
     */
    imageSrc?: string;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxGalleryOptions<
    TItem extends DevExpress.ui.dxGallery.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<dxGallery<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxGalleryOptions.animationDuration]
     */
    animationDuration?: number;
    /**
     * [descr:dxGalleryOptions.animationEnabled]
     */
    animationEnabled?: boolean;
    /**
     * [descr:dxGalleryOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxGalleryOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxGalleryOptions.indicatorEnabled]
     */
    indicatorEnabled?: boolean;
    /**
     * [descr:dxGalleryOptions.initialItemWidth]
     */
    initialItemWidth?: number;
    /**
     * [descr:dxGalleryOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxGalleryOptions.loop]
     */
    loop?: boolean;
    /**
     * [descr:dxGalleryOptions.noDataText]
     */
    noDataText?: string;
    /**
     * [descr:dxGalleryOptions.selectedIndex]
     */
    selectedIndex?: number;
    /**
     * [descr:dxGalleryOptions.showIndicator]
     */
    showIndicator?: boolean;
    /**
     * [descr:dxGalleryOptions.showNavButtons]
     */
    showNavButtons?: boolean;
    /**
     * [descr:dxGalleryOptions.slideshowDelay]
     */
    slideshowDelay?: number;
    /**
     * [descr:dxGalleryOptions.stretchImages]
     */
    stretchImages?: boolean;
    /**
     * [descr:dxGalleryOptions.swipeEnabled]
     */
    swipeEnabled?: boolean;
    /**
     * [descr:dxGalleryOptions.wrapAround]
     */
    wrapAround?: boolean;
  }
  /**
   * [descr:dxGantt]
   */
  export class dxGantt extends Widget<dxGanttOptions> {
    /**
     * [descr:dxGantt.getTaskData(key)]
     */
    getTaskData(key: any): any;
    /**
     * [descr:dxGantt.getDependencyData(key)]
     */
    getDependencyData(key: any): any;
    /**
     * [descr:dxGantt.getResourceData(key)]
     */
    getResourceData(key: any): any;
    /**
     * [descr:dxGantt.getResourceAssignmentData(key)]
     */
    getResourceAssignmentData(key: any): any;
    /**
     * [descr:dxGantt.insertTask(data)]
     */
    insertTask(data: any): void;
    /**
     * [descr:dxGantt.deleteTask(key)]
     */
    deleteTask(key: any): void;
    /**
     * [descr:dxGantt.updateTask(key, data)]
     */
    updateTask(key: any, data: any): void;
    /**
     * [descr:dxGantt.insertDependency(data)]
     */
    insertDependency(data: any): void;
    /**
     * [descr:dxGantt.deleteDependency(key)]
     */
    deleteDependency(key: any): void;
    /**
     * [descr:dxGantt.insertResource(data, taskKeys)]
     */
    insertResource(data: any, taskKeys?: Array<any>): void;
    /**
     * [descr:dxGantt.deleteResource(key)]
     */
    deleteResource(key: any): void;
    /**
     * [descr:dxGantt.assignResourceToTask(resourceKey, taskKey)]
     */
    assignResourceToTask(resourceKey: any, taskKey: any): void;
    /**
     * [descr:dxGantt.unassignResourceFromTask(resourceKey, taskKey)]
     */
    unassignResourceFromTask(resourceKey: any, taskKey: any): void;
    /**
     * [descr:dxGantt.getTaskResources(key)]
     */
    getTaskResources(key: any): Array<any>;
    /**
     * [descr:dxGantt.getVisibleTaskKeys()]
     */
    getVisibleTaskKeys(): Array<any>;
    /**
     * [descr:dxGantt.getVisibleDependencyKeys()]
     */
    getVisibleDependencyKeys(): Array<any>;
    /**
     * [descr:dxGantt.getVisibleResourceKeys()]
     */
    getVisibleResourceKeys(): Array<any>;
    /**
     * [descr:dxGantt.getVisibleResourceAssignmentKeys()]
     */
    getVisibleResourceAssignmentKeys(): Array<any>;
    /**
     * [descr:dxGantt.updateDimensions()]
     */
    updateDimensions(): void;
    /**
     * [descr:dxGantt.scrollToDate(date)]
     */
    scrollToDate(date: Date | Number | string): void;
    /**
     * [descr:dxGantt.showResourceManagerDialog()]
     */
    showResourceManagerDialog(): void;
    /**
     * [descr:dxGantt.expandAll()]
     */
    expandAll(): void;
    /**
     * [descr:dxGantt.collapseAll()]
     */
    collapseAll(): void;
    /**
     * [descr:dxGantt.expandAllToLevel(level)]
     */
    expandAllToLevel(level: Number): void;
    /**
     * [descr:dxGantt.expandToTask(key)]
     */
    expandToTask(key: any): void;
    /**
     * [descr:dxGantt.collapseTask(key)]
     */
    collapseTask(key: any): void;
    /**
     * [descr:dxGantt.expandTask(key)]
     */
    expandTask(key: any): void;
    /**
     * [descr:dxGantt.refresh()]
     */
    refresh(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxGantt.showResources(value)]
     */
    showResources(value: boolean): void;
    /**
     * [descr:dxGantt.showDependencies(value)]
     */
    showDependencies(value: boolean): void;
    /**
     * [descr:dxGantt.zoomIn()]
     */
    zoomIn(): void;
    /**
     * [descr:dxGantt.zoomOut()]
     */
    zoomOut(): void;
    /**
     * [descr:dxGantt.unassignAllResourcesFromTask(taskKey)]
     */
    unassignAllResourcesFromTask(taskKey: any): void;
    /**
     * [descr:dxGantt.showTaskDetailsDialog(taskKey)]
     */
    showTaskDetailsDialog(taskKey: any): void;
  }
  module dxGantt {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxGantt>;
    export type ContextMenuPreparingEvent = DevExpress.events.Cancelable & {
      readonly component?: dxGantt;
      readonly element?: DevExpress.core.DxElement;
      readonly event?: DevExpress.events.DxEvent;
      readonly targetKey?: any;
      readonly targetType?: string;
      readonly data?: any;
      readonly items?: Array<any>;
    };
    export type CustomCommandEvent = {
      readonly component?: dxGantt;
      readonly element?: DevExpress.core.DxElement;
      readonly name: string;
    };
    export type DependencyDeletedEvent =
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
      };
    export type DependencyDeletingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
      };
    export type DependencyInsertedEvent =
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
      };
    export type DependencyInsertingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
      };
    export type DisposingEvent = DevExpress.events.EventInfo<dxGantt>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxGantt>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxGantt> &
      DevExpress.events.ChangedOptionInfo;
    export type ProgressTooltipTemplateData = {
      readonly progress: number;
    };
    export type Properties = dxGanttOptions;
    export type ResourceAssignedEvent = DevExpress.events.EventInfo<dxGantt> & {
      readonly values: any;
      readonly key: any;
    };
    export type ResourceAssigningEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
      };
    export type ResourceDeletedEvent = DevExpress.events.EventInfo<dxGantt> & {
      readonly values: any;
      readonly key: any;
    };
    export type ResourceDeletingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
      };
    export type ResourceInsertedEvent = DevExpress.events.EventInfo<dxGantt> & {
      readonly values: any;
      readonly key: any;
    };
    export type ResourceInsertingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
      };
    export type ResourceManagerDialogShowingEvent =
      DevExpress.events.Cancelable &
        DevExpress.events.EventInfo<dxGantt> & {
          readonly values: Array<any>;
        };
    export type ResourceUnassignedEvent =
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
      };
    export type ResourceUnassigningEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
      };
    export type SelectionChangedEvent = DevExpress.events.EventInfo<dxGantt> & {
      readonly selectedRowKey?: any;
    };
    export type TaskClickEvent = DevExpress.events.NativeEventInfo<dxGantt> & {
      readonly key?: any;
      readonly data?: any;
    };
    export type TaskContentTemplateData = {
      readonly cellSize: any;
      readonly isMilestone: boolean;
      readonly taskData: any;
      readonly taskHTML: any;
      readonly taskPosition: any;
      readonly taskResources: Array<any>;
      readonly taskSize: any;
    };
    export type TaskDblClickEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxGantt> & {
        readonly key?: any;
        readonly data?: any;
      };
    export type TaskDeletedEvent = DevExpress.events.EventInfo<dxGantt> & {
      readonly values: any;
      readonly key: any;
    };
    export type TaskDeletingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
      };
    export type TaskEditDialogShowingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
        readonly key: any;
        readonly readOnlyFields?: Array<string>;
        readonly hiddenFields?: Array<string>;
      };
    export type TaskInsertedEvent = DevExpress.events.EventInfo<dxGantt> & {
      readonly value?: any;
      readonly key: any;
    };
    export type TaskInsertingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly values: any;
      };
    export type TaskMovingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly newValues: any;
        readonly values: any;
        readonly key: any;
      };
    export type TaskUpdatedEvent = DevExpress.events.EventInfo<dxGantt> & {
      readonly values: any;
      readonly key: any;
    };
    export type TaskUpdatingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxGantt> & {
        readonly newValues: any;
        readonly values: any;
        readonly key: any;
      };
    export type TimeTooltipTemplateData = {
      readonly start: Date;
      readonly end: Date;
    };
  }
  /**
   * [descr:dxGanttContextMenu]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttContextMenu {
    /**
     * [descr:dxGanttContextMenu.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:dxGanttContextMenu.items]
     */
    items?: Array<
      | DevExpress.ui.dxGantt.ContextMenuItem
      | 'undo'
      | 'redo'
      | 'expandAll'
      | 'collapseAll'
      | 'addTask'
      | 'deleteTask'
      | 'zoomIn'
      | 'zoomOut'
      | 'deleteDependency'
      | 'taskDetails'
      | 'resourceManager'
    >;
  }
  /**
   * @deprecated Use ContextMenuItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttContextMenuItem
    extends DevExpress.ui.dxContextMenu.Item {
    /**
     * [descr:dxGanttContextMenuItem.name]
     */
    name?:
      | 'undo'
      | 'redo'
      | 'expandAll'
      | 'collapseAll'
      | 'addTask'
      | 'deleteTask'
      | 'zoomIn'
      | 'zoomOut'
      | 'deleteDependency'
      | 'taskDetails'
      | 'resourceManager'
      | string;
  }
  /**
   * [descr:dxGanttFilterRow]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttFilterRow {
    /**
     * [descr:dxGanttFilterRow.betweenEndText]
     */
    betweenEndText?: string;
    /**
     * [descr:dxGanttFilterRow.betweenStartText]
     */
    betweenStartText?: string;
    /**
     * [descr:dxGanttFilterRow.operationDescriptions]
     */
    operationDescriptions?: dxGanttFilterRowOperationDescriptions;
    /**
     * [descr:dxGanttFilterRow.resetOperationText]
     */
    resetOperationText?: string;
    /**
     * [descr:dxGanttFilterRow.showAllText]
     */
    showAllText?: string;
    /**
     * [descr:dxGanttFilterRow.showOperationChooser]
     */
    showOperationChooser?: boolean;
    /**
     * [descr:dxGanttFilterRow.visible]
     */
    visible?: boolean;
  }
  /**
   * [descr:dxGanttFilterRowOperationDescriptions]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttFilterRowOperationDescriptions {
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.between]
     */
    between?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.contains]
     */
    contains?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.endsWith]
     */
    endsWith?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.equal]
     */
    equal?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.greaterThan]
     */
    greaterThan?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.greaterThanOrEqual]
     */
    greaterThanOrEqual?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.lessThan]
     */
    lessThan?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.lessThanOrEqual]
     */
    lessThanOrEqual?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.notContains]
     */
    notContains?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.notEqual]
     */
    notEqual?: string;
    /**
     * [descr:dxGanttFilterRowOperationDescriptions.startsWith]
     */
    startsWith?: string;
  }
  /**
   * [descr:dxGanttHeaderFilter]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttHeaderFilter {
    /**
     * [descr:dxGanttHeaderFilter.allowSearch]
     */
    allowSearch?: boolean;
    /**
     * [descr:dxGanttHeaderFilter.height]
     */
    height?: number;
    /**
     * [descr:dxGanttHeaderFilter.searchTimeout]
     */
    searchTimeout?: number;
    /**
     * [descr:dxGanttHeaderFilter.texts]
     */
    texts?: dxGanttHeaderFilterTexts;
    /**
     * [descr:dxGanttHeaderFilter.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxGanttHeaderFilter.width]
     */
    width?: number;
  }
  /**
   * [descr:dxGanttHeaderFilterTexts]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttHeaderFilterTexts {
    /**
     * [descr:dxGanttHeaderFilterTexts.cancel]
     */
    cancel?: string;
    /**
     * [descr:dxGanttHeaderFilterTexts.emptyValue]
     */
    emptyValue?: string;
    /**
     * [descr:dxGanttHeaderFilterTexts.ok]
     */
    ok?: string;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttOptions extends WidgetOptions<dxGantt> {
    /**
     * [descr:dxGanttOptions.allowSelection]
     */
    allowSelection?: boolean;
    /**
     * [descr:dxGanttOptions.columns]
     */
    columns?: Array<DevExpress.ui.dxTreeList.Column | string>;
    /**
     * [descr:dxGanttOptions.dependencies]
     */
    dependencies?: {
      /**
       * [descr:dxGanttOptions.dependencies.dataSource]
       */
      dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
      /**
       * [descr:dxGanttOptions.dependencies.keyExpr]
       */
      keyExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.dependencies.predecessorIdExpr]
       */
      predecessorIdExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.dependencies.successorIdExpr]
       */
      successorIdExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.dependencies.typeExpr]
       */
      typeExpr?: string | Function;
    };
    /**
     * [descr:dxGanttOptions.editing]
     */
    editing?: {
      /**
       * [descr:dxGanttOptions.editing.allowDependencyAdding]
       */
      allowDependencyAdding?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowDependencyDeleting]
       */
      allowDependencyDeleting?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowResourceAdding]
       */
      allowResourceAdding?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowResourceDeleting]
       */
      allowResourceDeleting?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowResourceUpdating]
       */
      allowResourceUpdating?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowTaskAdding]
       */
      allowTaskAdding?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowTaskDeleting]
       */
      allowTaskDeleting?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowTaskResourceUpdating]
       */
      allowTaskResourceUpdating?: boolean;
      /**
       * [descr:dxGanttOptions.editing.allowTaskUpdating]
       */
      allowTaskUpdating?: boolean;
      /**
       * [descr:dxGanttOptions.editing.enabled]
       */
      enabled?: boolean;
    };
    /**
     * [descr:dxGanttOptions.validation]
     */
    validation?: {
      /**
       * [descr:dxGanttOptions.validation.validateDependencies]
       */
      validateDependencies?: boolean;
      /**
       * [descr:dxGanttOptions.validation.autoUpdateParentTasks]
       */
      autoUpdateParentTasks?: boolean;
      /**
       * [descr:dxGanttOptions.validation.enablePredecessorGap]
       */
      enablePredecessorGap?: boolean;
    };
    /**
     * [descr:dxGanttOptions.sorting]
     */
    sorting?: dxGanttSorting;
    /**
     * [descr:dxGanttOptions.filterRow]
     */
    filterRow?: dxGanttFilterRow;
    /**
     * [descr:dxGanttOptions.headerFilter]
     */
    headerFilter?: dxGanttHeaderFilter;
    /**
     * [descr:dxGanttOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.ui.dxGantt.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onCustomCommand]
     */
    onCustomCommand?: (e: DevExpress.ui.dxGantt.CustomCommandEvent) => void;
    /**
     * [descr:dxGanttOptions.onContextMenuPreparing]
     */
    onContextMenuPreparing?: (
      e: DevExpress.ui.dxGantt.ContextMenuPreparingEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onTaskInserting]
     */
    onTaskInserting?: (e: DevExpress.ui.dxGantt.TaskInsertingEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskInserted]
     */
    onTaskInserted?: (e: DevExpress.ui.dxGantt.TaskInsertedEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskDeleting]
     */
    onTaskDeleting?: (e: DevExpress.ui.dxGantt.TaskDeletingEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskDeleted]
     */
    onTaskDeleted?: (e: DevExpress.ui.dxGantt.TaskDeletedEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskUpdating]
     */
    onTaskUpdating?: (e: DevExpress.ui.dxGantt.TaskUpdatingEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskUpdated]
     */
    onTaskUpdated?: (e: DevExpress.ui.dxGantt.TaskUpdatedEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskMoving]
     */
    onTaskMoving?: (e: DevExpress.ui.dxGantt.TaskMovingEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskEditDialogShowing]
     */
    onTaskEditDialogShowing?: (
      e: DevExpress.ui.dxGantt.TaskEditDialogShowingEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceManagerDialogShowing]
     */
    onResourceManagerDialogShowing?: (
      e: DevExpress.ui.dxGantt.ResourceManagerDialogShowingEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onDependencyInserting]
     */
    onDependencyInserting?: (
      e: DevExpress.ui.dxGantt.DependencyInsertingEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onDependencyInserted]
     */
    onDependencyInserted?: (
      e: DevExpress.ui.dxGantt.DependencyInsertedEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onDependencyDeleting]
     */
    onDependencyDeleting?: (
      e: DevExpress.ui.dxGantt.DependencyDeletingEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onDependencyDeleted]
     */
    onDependencyDeleted?: (
      e: DevExpress.ui.dxGantt.DependencyDeletedEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceInserting]
     */
    onResourceInserting?: (
      e: DevExpress.ui.dxGantt.ResourceInsertingEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceInserted]
     */
    onResourceInserted?: (
      e: DevExpress.ui.dxGantt.ResourceInsertedEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceDeleting]
     */
    onResourceDeleting?: (
      e: DevExpress.ui.dxGantt.ResourceDeletingEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceDeleted]
     */
    onResourceDeleted?: (e: DevExpress.ui.dxGantt.ResourceDeletedEvent) => void;
    /**
     * [descr:dxGanttOptions.onResourceAssigning]
     */
    onResourceAssigning?: (
      e: DevExpress.ui.dxGantt.ResourceAssigningEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceAssigned]
     */
    onResourceAssigned?: (
      e: DevExpress.ui.dxGantt.ResourceAssignedEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceUnassigning]
     */
    onResourceUnassigning?: (
      e: DevExpress.ui.dxGantt.ResourceUnassigningEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onResourceUnassigned]
     */
    onResourceUnassigned?: (
      e: DevExpress.ui.dxGantt.ResourceUnassignedEvent
    ) => void;
    /**
     * [descr:dxGanttOptions.onTaskClick]
     */
    onTaskClick?: (e: DevExpress.ui.dxGantt.TaskClickEvent) => void;
    /**
     * [descr:dxGanttOptions.onTaskDblClick]
     */
    onTaskDblClick?: (e: DevExpress.ui.dxGantt.TaskDblClickEvent) => void;
    /**
     * [descr:dxGanttOptions.resourceAssignments]
     */
    resourceAssignments?: {
      /**
       * [descr:dxGanttOptions.resourceAssignments.dataSource]
       */
      dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
      /**
       * [descr:dxGanttOptions.resourceAssignments.keyExpr]
       */
      keyExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.resourceAssignments.resourceIdExpr]
       */
      resourceIdExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.resourceAssignments.taskIdExpr]
       */
      taskIdExpr?: string | Function;
    };
    /**
     * [descr:dxGanttOptions.resources]
     */
    resources?: {
      /**
       * [descr:dxGanttOptions.resources.colorExpr]
       */
      colorExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.resources.dataSource]
       */
      dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
      /**
       * [descr:dxGanttOptions.resources.keyExpr]
       */
      keyExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.resources.textExpr]
       */
      textExpr?: string | Function;
    };
    /**
     * [descr:dxGanttOptions.scaleType]
     */
    scaleType?:
      | 'auto'
      | 'minutes'
      | 'hours'
      | 'days'
      | 'weeks'
      | 'months'
      | 'quarters'
      | 'years';
    /**
     * [descr:dxGanttOptions.scaleTypeRange]
     */
    scaleTypeRange?: {
      /**
       * [descr:dxGanttOptions.scaleTypeRange.min]
       */
      min?:
        | 'minutes'
        | 'hours'
        | 'days'
        | 'weeks'
        | 'months'
        | 'quarters'
        | 'years';
      /**
       * [descr:dxGanttOptions.scaleTypeRange.max]
       */
      max?:
        | 'minutes'
        | 'hours'
        | 'days'
        | 'weeks'
        | 'months'
        | 'quarters'
        | 'years';
    };
    /**
     * [descr:dxGanttOptions.selectedRowKey]
     */
    selectedRowKey?: any;
    /**
     * [descr:dxGanttOptions.showResources]
     */
    showResources?: boolean;
    /**
     * [descr:dxGanttOptions.showDependencies]
     */
    showDependencies?: boolean;
    /**
     * [descr:dxGanttOptions.showRowLines]
     */
    showRowLines?: boolean;
    /**
     * [descr:dxGanttOptions.taskListWidth]
     */
    taskListWidth?: number;
    /**
     * [descr:dxGanttOptions.taskTitlePosition]
     */
    taskTitlePosition?: 'inside' | 'outside' | 'none';
    /**
     * [descr:dxGanttOptions.firstDayOfWeek]
     */
    firstDayOfWeek?: 0 | 1 | 2 | 3 | 4 | 5 | 6;
    /**
     * [descr:dxGanttOptions.tasks]
     */
    tasks?: {
      /**
       * [descr:dxGanttOptions.tasks.colorExpr]
       */
      colorExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.tasks.dataSource]
       */
      dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
      /**
       * [descr:dxGanttOptions.tasks.endExpr]
       */
      endExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.tasks.keyExpr]
       */
      keyExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.tasks.parentIdExpr]
       */
      parentIdExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.tasks.progressExpr]
       */
      progressExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.tasks.startExpr]
       */
      startExpr?: string | Function;
      /**
       * [descr:dxGanttOptions.tasks.titleExpr]
       */
      titleExpr?: string | Function;
    };
    /**
     * [descr:dxGanttOptions.toolbar]
     */
    toolbar?: dxGanttToolbar;
    /**
     * [descr:dxGanttOptions.contextMenu]
     */
    contextMenu?: dxGanttContextMenu;
    /**
     * [descr:dxGanttOptions.stripLines]
     */
    stripLines?: Array<dxGanttStripLine>;
    /**
     * [descr:dxGanttOptions.taskTooltipContentTemplate]
     */
    taskTooltipContentTemplate?:
      | DevExpress.core.template
      | ((
          container: DevExpress.core.DxElement,
          task: any
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxGanttOptions.taskTimeTooltipContentTemplate]
     */
    taskTimeTooltipContentTemplate?:
      | DevExpress.core.template
      | ((
          container: DevExpress.core.DxElement,
          item: DevExpress.ui.dxGantt.TimeTooltipTemplateData
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxGanttOptions.taskProgressTooltipContentTemplate]
     */
    taskProgressTooltipContentTemplate?:
      | DevExpress.core.template
      | ((
          container: DevExpress.core.DxElement,
          item: DevExpress.ui.dxGantt.ProgressTooltipTemplateData
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxGanttOptions.taskContentTemplate]
     */
    taskContentTemplate?:
      | DevExpress.core.template
      | ((
          container: DevExpress.core.DxElement,
          item: DevExpress.ui.dxGantt.TaskContentTemplateData
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxGanttOptions.rootValue]
     */
    rootValue?: any;
    /**
     * [descr:dxGanttOptions.startDateRange]
     */
    startDateRange?: Date;
    /**
     * [descr:dxGanttOptions.endDateRange]
     */
    endDateRange?: Date;
  }
  /**
   * [descr:dxGanttSorting]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttSorting {
    /**
     * [descr:dxGanttSorting.ascendingText]
     */
    ascendingText?: string;
    /**
     * [descr:dxGanttSorting.clearText]
     */
    clearText?: string;
    /**
     * [descr:dxGanttSorting.descendingText]
     */
    descendingText?: string;
    /**
     * [descr:dxGanttSorting.mode]
     */
    mode?: 'multiple' | 'none' | 'single';
    /**
     * [descr:dxGanttSorting.showSortIndexes]
     */
    showSortIndexes?: boolean;
  }
  /**
   * [descr:dxGanttStripLine]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttStripLine {
    /**
     * [descr:dxGanttStripLine.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxGanttStripLine.end]
     */
    end?: Date | number | string | (() => Date | number | string);
    /**
     * [descr:dxGanttStripLine.start]
     */
    start?: Date | number | string | (() => Date | number | string);
    /**
     * [descr:dxGanttStripLine.title]
     */
    title?: string;
  }
  /**
   * [descr:dxGanttToolbar]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttToolbar {
    /**
     * [descr:dxGanttToolbar.items]
     */
    items?: Array<
      | DevExpress.ui.dxGantt.ToolbarItem
      | 'separator'
      | 'undo'
      | 'redo'
      | 'expandAll'
      | 'collapseAll'
      | 'addTask'
      | 'deleteTask'
      | 'zoomIn'
      | 'zoomOut'
      | 'taskDetails'
      | 'fullScreen'
      | 'resourceManager'
      | 'showResources'
      | 'showDependencies'
    >;
  }
  /**
   * @deprecated Use ToolbarItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxGanttToolbarItem extends DevExpress.ui.dxToolbar.Item {
    /**
     * [descr:dxGanttToolbarItem.name]
     */
    name?:
      | 'separator'
      | 'undo'
      | 'redo'
      | 'expandAll'
      | 'collapseAll'
      | 'addTask'
      | 'deleteTask'
      | 'zoomIn'
      | 'zoomOut'
      | 'taskDetails'
      | 'fullScreen'
      | 'resourceManager'
      | 'toggleResources'
      | 'toggleDependencies'
      | string;
    /**
     * [descr:dxGanttToolbarItem.location]
     */
    location?: 'after' | 'before' | 'center';
  }
  /**
   * [descr:dxHtmlEditor]
   */
  export class dxHtmlEditor extends Editor<dxHtmlEditorOptions> {
    /**
     * [descr:dxHtmlEditor.blur()]
     */
    blur(): void;
    /**
     * [descr:dxHtmlEditor.clearHistory()]
     */
    clearHistory(): void;
    /**
     * [descr:dxHtmlEditor.delete(index, length)]
     */
    delete(index: number, length: number): void;
    /**
     * [descr:dxHtmlEditor.format(formatName, formatValue)]
     */
    format(
      formatName:
        | 'background'
        | 'bold'
        | 'color'
        | 'font'
        | 'italic'
        | 'link'
        | 'size'
        | 'strike'
        | 'script'
        | 'underline'
        | 'blockquote'
        | 'header'
        | 'indent'
        | 'list'
        | 'align'
        | 'code-block'
        | string,
      formatValue: any
    ): void;
    /**
     * [descr:dxHtmlEditor.formatLine(index, length, formatName, formatValue)]
     */
    formatLine(
      index: number,
      length: number,
      formatName:
        | 'background'
        | 'bold'
        | 'color'
        | 'font'
        | 'italic'
        | 'link'
        | 'size'
        | 'strike'
        | 'script'
        | 'underline'
        | 'blockquote'
        | 'header'
        | 'indent'
        | 'list'
        | 'align'
        | 'code-block'
        | string,
      formatValue: any
    ): void;
    /**
     * [descr:dxHtmlEditor.formatLine(index, length, formats)]
     */
    formatLine(index: number, length: number, formats: any): void;
    /**
     * [descr:dxHtmlEditor.formatText(index, length, formatName, formatValue)]
     */
    formatText(
      index: number,
      length: number,
      formatName:
        | 'background'
        | 'bold'
        | 'color'
        | 'font'
        | 'italic'
        | 'link'
        | 'size'
        | 'strike'
        | 'script'
        | 'underline'
        | 'blockquote'
        | 'header'
        | 'indent'
        | 'list'
        | 'align'
        | 'code-block'
        | string,
      formatValue: any
    ): void;
    /**
     * [descr:dxHtmlEditor.formatText(index, length, formats)]
     */
    formatText(index: number, length: number, formats: any): void;
    /**
     * [descr:dxHtmlEditor.get(componentPath)]
     */
    get(componentPath: string): any;
    /**
     * [descr:dxHtmlEditor.getBounds(index, length)]
     */
    getBounds(index: number, length: number): any;
    /**
     * [descr:dxHtmlEditor.getFormat()]
     */
    getFormat(): any;
    /**
     * [descr:dxHtmlEditor.getFormat(index, length)]
     */
    getFormat(index: number, length: number): any;
    /**
     * [descr:dxHtmlEditor.getLength()]
     */
    getLength(): number;
    /**
     * [descr:dxHtmlEditor.getModule(moduleName)]
     */
    getModule(moduleName: string): any;
    /**
     * [descr:dxHtmlEditor.getQuillInstance()]
     */
    getQuillInstance(): any;
    /**
     * [descr:dxHtmlEditor.getSelection()]
     */
    getSelection(focus?: boolean | undefined): any;
    /**
     * [descr:dxHtmlEditor.getText(index, length)]
     */
    getText(index: number, length: number): string;
    /**
     * [descr:dxHtmlEditor.insertEmbed(index, type, config)]
     */
    insertEmbed(index: number, type: string, config: any): void;
    /**
     * [descr:dxHtmlEditor.insertText(index, text, formatName, formatValue)]
     */
    insertText(
      index: number,
      text: string,
      formatName:
        | 'background'
        | 'bold'
        | 'color'
        | 'font'
        | 'italic'
        | 'link'
        | 'size'
        | 'strike'
        | 'script'
        | 'underline'
        | 'blockquote'
        | 'header'
        | 'indent'
        | 'list'
        | 'align'
        | 'code-block'
        | string,
      formatValue: any
    ): void;
    /**
     * [descr:dxHtmlEditor.insertText(index, text, formats)]
     */
    insertText(index: number, text: string, formats: any): void;
    /**
     * [descr:dxHtmlEditor.redo()]
     */
    redo(): void;
    /**
     * [descr:dxHtmlEditor.register(components)]
     */
    register(modules: any): void;
    /**
     * [descr:dxHtmlEditor.removeFormat(index, length)]
     */
    removeFormat(index: number, length: number): void;
    /**
     * [descr:dxHtmlEditor.setSelection(index, length)]
     */
    setSelection(index: number, length: number): void;
    /**
     * [descr:dxHtmlEditor.undo()]
     */
    undo(): void;
  }
  module dxHtmlEditor {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxHtmlEditor>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxHtmlEditor>;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxHtmlEditor,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxHtmlEditor,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxHtmlEditor>;
    export interface MentionTemplateData {
      readonly marker: string;
      readonly id?: string | number;
      readonly value?: any;
    }
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxHtmlEditor> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxHtmlEditorOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxHtmlEditor,
      KeyboardEvent | ClipboardEvent
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * [descr:dxHtmlEditorMediaResizing]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorMediaResizing {
    /**
     * [descr:dxHtmlEditorMediaResizing.allowedTargets]
     */
    allowedTargets?: Array<string>;
    /**
     * [descr:dxHtmlEditorMediaResizing.enabled]
     */
    enabled?: boolean;
  }
  /**
   * [descr:dxHtmlEditorMention]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorMention {
    /**
     * [descr:dxHtmlEditorMention.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<string>;
    /**
     * [descr:dxHtmlEditorMention.displayExpr]
     */
    displayExpr?: string | ((item: any) => string);
    /**
     * [descr:dxHtmlEditorMention.itemTemplate]
     */
    itemTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxHtmlEditorMention.marker]
     */
    marker?: string;
    /**
     * [descr:dxHtmlEditorMention.minSearchLength]
     */
    minSearchLength?: number;
    /**
     * [descr:dxHtmlEditorMention.searchExpr]
     */
    searchExpr?: string | Function | Array<string | Function>;
    /**
     * [descr:dxHtmlEditorMention.searchTimeout]
     */
    searchTimeout?: number;
    /**
     * [descr:dxHtmlEditorMention.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          mentionData: DevExpress.ui.dxHtmlEditor.MentionTemplateData,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxHtmlEditorMention.valueExpr]
     */
    valueExpr?: string | Function;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorOptions extends EditorOptions<dxHtmlEditor> {
    /**
     * [descr:dxHtmlEditorOptions.allowSoftLineBreak]
     */
    allowSoftLineBreak?: boolean;
    /**
     * [descr:dxHtmlEditorOptions.customizeModules]
     */
    customizeModules?: (config: any) => void;
    /**
     * [descr:dxHtmlEditorOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxHtmlEditorOptions.mediaResizing]
     */
    mediaResizing?: dxHtmlEditorMediaResizing;
    /**
     * [descr:dxHtmlEditorOptions.tableResizing]
     */
    tableResizing?: dxHtmlEditorTableResizing;
    /**
     * [descr:dxHtmlEditorOptions.mentions]
     */
    mentions?: Array<dxHtmlEditorMention>;
    /**
     * [descr:dxHtmlEditorOptions.tableContextMenu]
     */
    tableContextMenu?: dxHtmlEditorTableContextMenu;
    /**
     * [descr:dxHtmlEditorOptions.name]
     */
    name?: string;
    /**
     * [descr:dxHtmlEditorOptions.onFocusIn]
     */
    onFocusIn?: (e: DevExpress.ui.dxHtmlEditor.FocusInEvent) => void;
    /**
     * [descr:dxHtmlEditorOptions.onFocusOut]
     */
    onFocusOut?: (e: DevExpress.ui.dxHtmlEditor.FocusOutEvent) => void;
    /**
     * [descr:dxHtmlEditorOptions.placeholder]
     */
    placeholder?: string;
    /**
     * [descr:dxHtmlEditorOptions.toolbar]
     */
    toolbar?: dxHtmlEditorToolbar;
    /**
     * [descr:dxHtmlEditorOptions.valueType]
     */
    valueType?: 'html' | 'markdown';
    /**
     * [descr:dxHtmlEditorOptions.variables]
     */
    variables?: dxHtmlEditorVariables;
    /**
     * [descr:dxHtmlEditorOptions.stylingMode]
     */
    stylingMode?: 'outlined' | 'underlined' | 'filled';
  }
  /**
   * [descr:dxHtmlEditorTableContextMenu]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorTableContextMenu {
    /**
     * [descr:dxHtmlEditorTableContextMenu.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:dxHtmlEditorTableContextMenu.items]
     */
    items?: Array<
      | DevExpress.ui.dxHtmlEditor.ContextMenuItem
      | 'background'
      | 'bold'
      | 'color'
      | 'font'
      | 'italic'
      | 'link'
      | 'image'
      | 'strike'
      | 'subscript'
      | 'superscript'
      | 'underline'
      | 'blockquote'
      | 'header'
      | 'increaseIndent'
      | 'decreaseIndent'
      | 'orderedList'
      | 'bulletList'
      | 'alignLeft'
      | 'alignCenter'
      | 'alignRight'
      | 'alignJustify'
      | 'codeBlock'
      | 'variable'
      | 'separator'
      | 'undo'
      | 'redo'
      | 'clear'
      | 'insertTable'
      | 'insertRowAbove'
      | 'insertRowBelow'
      | 'insertColumnLeft'
      | 'insertColumnRight'
      | 'deleteColumn'
      | 'deleteRow'
      | 'deleteTable'
    >;
  }
  /**
   * @deprecated Use DevExpress.ui.dxHtmlEditor.ContextMenuItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorTableContextMenuItem
    extends DevExpress.ui.dxMenu.MenuBasePlainItem {
    /**
     * [descr:dxHtmlEditorTableContextMenuItem.name]
     */
    name?:
      | 'background'
      | 'bold'
      | 'color'
      | 'font'
      | 'italic'
      | 'link'
      | 'image'
      | 'strike'
      | 'subscript'
      | 'superscript'
      | 'underline'
      | 'blockquote'
      | 'header'
      | 'increaseIndent'
      | 'decreaseIndent'
      | 'orderedList'
      | 'bulletList'
      | 'alignLeft'
      | 'alignCenter'
      | 'alignRight'
      | 'alignJustify'
      | 'codeBlock'
      | 'variable'
      | 'separator'
      | 'undo'
      | 'redo'
      | 'clear'
      | 'insertTable'
      | 'insertRowAbove'
      | 'insertRowBelow'
      | 'insertColumnLeft'
      | 'insertColumnRight'
      | 'deleteColumn'
      | 'deleteRow'
      | 'deleteTable';
    /**
     * [descr:dxHtmlEditorTableContextMenuItem.items]
     */
    items?: Array<
      | DevExpress.ui.dxHtmlEditor.ContextMenuItem
      | 'background'
      | 'bold'
      | 'color'
      | 'font'
      | 'italic'
      | 'link'
      | 'image'
      | 'strike'
      | 'subscript'
      | 'superscript'
      | 'underline'
      | 'blockquote'
      | 'header'
      | 'increaseIndent'
      | 'decreaseIndent'
      | 'orderedList'
      | 'bulletList'
      | 'alignLeft'
      | 'alignCenter'
      | 'alignRight'
      | 'alignJustify'
      | 'codeBlock'
      | 'variable'
      | 'separator'
      | 'undo'
      | 'redo'
      | 'clear'
      | 'insertTable'
      | 'insertRowAbove'
      | 'insertRowBelow'
      | 'insertColumnLeft'
      | 'insertColumnRight'
      | 'deleteColumn'
      | 'deleteRow'
      | 'deleteTable'
    >;
  }
  /**
   * [descr:dxHtmlEditorTableResizing]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorTableResizing {
    /**
     * [descr:dxHtmlEditorTableResizing.minColumnWidth]
     */
    minColumnWidth?: number;
    /**
     * [descr:dxHtmlEditorTableResizing.minRowHeight]
     */
    minRowHeight?: number;
    /**
     * [descr:dxHtmlEditorTableResizing.enabled]
     */
    enabled?: boolean;
  }
  /**
   * [descr:dxHtmlEditorToolbar]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorToolbar {
    /**
     * [descr:dxHtmlEditorToolbar.container]
     */
    container?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxHtmlEditorToolbar.items]
     */
    items?: Array<
      | DevExpress.ui.dxHtmlEditor.ToolbarItem
      | 'background'
      | 'bold'
      | 'color'
      | 'font'
      | 'italic'
      | 'link'
      | 'image'
      | 'size'
      | 'strike'
      | 'subscript'
      | 'superscript'
      | 'underline'
      | 'blockquote'
      | 'header'
      | 'increaseIndent'
      | 'decreaseIndent'
      | 'orderedList'
      | 'bulletList'
      | 'alignLeft'
      | 'alignCenter'
      | 'alignRight'
      | 'alignJustify'
      | 'codeBlock'
      | 'variable'
      | 'separator'
      | 'undo'
      | 'redo'
      | 'clear'
      | 'insertTable'
      | 'insertHeaderRow'
      | 'insertRowAbove'
      | 'insertRowBelow'
      | 'insertColumnLeft'
      | 'insertColumnRight'
      | 'deleteColumn'
      | 'deleteRow'
      | 'deleteTable'
      | 'cellProperties'
      | 'tableProperties'
    >;
    /**
     * [descr:dxHtmlEditorToolbar.multiline]
     */
    multiline?: boolean;
  }
  /**
   * @deprecated Use ToolbarItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorToolbarItem
    extends DevExpress.ui.dxToolbar.Item {
    /**
     * [descr:dxHtmlEditorToolbarItem.name]
     */
    name?:
      | 'background'
      | 'bold'
      | 'color'
      | 'font'
      | 'italic'
      | 'link'
      | 'image'
      | 'size'
      | 'strike'
      | 'subscript'
      | 'superscript'
      | 'underline'
      | 'blockquote'
      | 'header'
      | 'increaseIndent'
      | 'decreaseIndent'
      | 'orderedList'
      | 'bulletList'
      | 'alignLeft'
      | 'alignCenter'
      | 'alignRight'
      | 'alignJustify'
      | 'codeBlock'
      | 'variable'
      | 'separator'
      | 'undo'
      | 'redo'
      | 'clear'
      | 'insertTable'
      | 'insertHeaderRow'
      | 'insertRowAbove'
      | 'insertRowBelow'
      | 'insertColumnLeft'
      | 'insertColumnRight'
      | 'deleteColumn'
      | 'deleteRow'
      | 'deleteTable'
      | 'cellProperties'
      | 'tableProperties'
      | string;
    /**
     * [descr:dxHtmlEditorToolbarItem.formatName]
     * @deprecated [depNote:dxHtmlEditorToolbarItem.formatName]
     */
    formatName?:
      | 'background'
      | 'bold'
      | 'color'
      | 'font'
      | 'italic'
      | 'link'
      | 'image'
      | 'size'
      | 'strike'
      | 'subscript'
      | 'superscript'
      | 'underline'
      | 'blockquote'
      | 'header'
      | 'increaseIndent'
      | 'decreaseIndent'
      | 'orderedList'
      | 'bulletList'
      | 'alignLeft'
      | 'alignCenter'
      | 'alignRight'
      | 'alignJustify'
      | 'codeBlock'
      | 'variable'
      | 'separator'
      | 'undo'
      | 'redo'
      | 'clear'
      | 'insertTable'
      | 'insertHeaderRow'
      | 'insertRowAbove'
      | 'insertRowBelow'
      | 'insertColumnLeft'
      | 'insertColumnRight'
      | 'deleteColumn'
      | 'deleteRow'
      | 'deleteTable'
      | 'cellProperties'
      | 'tableProperties'
      | string;
    /**
     * [descr:dxHtmlEditorToolbarItem.acceptedValues]
     */
    acceptedValues?: Array<string | number | boolean>;
    /**
     * [descr:dxHtmlEditorToolbarItem.formatValues]
     * @deprecated [depNote:dxHtmlEditorToolbarItem.formatValues]
     */
    formatValues?: Array<string | number | boolean>;
    /**
     * [descr:dxHtmlEditorToolbarItem.location]
     */
    location?: 'after' | 'before' | 'center';
  }
  /**
   * [descr:dxHtmlEditorVariables]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxHtmlEditorVariables {
    /**
     * [descr:dxHtmlEditorVariables.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<string>;
    /**
     * [descr:dxHtmlEditorVariables.escapeChar]
     */
    escapeChar?: string | Array<string>;
  }
  /**
   * [descr:dxItem]
   */
  export var dxItem: any;
  /**
   * [descr:dxList]
   */
  export class dxList<
    TItem extends DevExpress.ui.dxList.ItemLike = any,
    TKey = any
  > extends CollectionWidget<dxListOptions<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxList.clientHeight()]
     */
    clientHeight(): number;
    /**
     * [descr:dxList.collapseGroup(groupIndex)]
     */
    collapseGroup(groupIndex: number): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxList.deleteItem(itemElement)]
     */
    deleteItem(itemElement: Element): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxList.deleteItem(itemIndex)]
     */
    deleteItem(itemIndex: number | any): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxList.expandGroup(groupIndex)]
     */
    expandGroup(groupIndex: number): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxList.isItemSelected(itemElement)]
     */
    isItemSelected(itemElement: Element): boolean;
    /**
     * [descr:dxList.isItemSelected(itemIndex)]
     */
    isItemSelected(itemIndex: number | any): boolean;
    /**
     * [descr:dxList.reload()]
     */
    reload(): void;
    /**
     * [descr:dxList.reorderItem(itemElement, toItemElement)]
     */
    reorderItem(
      itemElement: Element,
      toItemElement: Element
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxList.reorderItem(itemIndex, toItemIndex)]
     */
    reorderItem(
      itemIndex: number | any,
      toItemIndex: number | any
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxList.scrollBy(distance)]
     */
    scrollBy(distance: number): void;
    /**
     * [descr:dxList.scrollHeight()]
     */
    scrollHeight(): number;
    /**
     * [descr:dxList.scrollTo(location)]
     */
    scrollTo(location: number): void;
    /**
     * [descr:dxList.scrollToItem(itemElement)]
     */
    scrollToItem(itemElement: Element): void;
    /**
     * [descr:dxList.scrollToItem(itemIndex)]
     */
    scrollToItem(itemIndex: number | any): void;
    /**
     * [descr:dxList.scrollTop()]
     */
    scrollTop(): number;
    /**
     * [descr:dxList.selectAll()]
     */
    selectAll(): void;
    /**
     * [descr:dxList.selectItem(itemElement)]
     */
    selectItem(itemElement: Element): void;
    /**
     * [descr:dxList.selectItem(itemIndex)]
     */
    selectItem(itemIndex: number | any): void;
    /**
     * [descr:dxList.unselectAll()]
     */
    unselectAll(): void;
    /**
     * [descr:dxList.unselectItem(itemElement)]
     */
    unselectItem(itemElement: Element): void;
    /**
     * [descr:dxList.unselectItem(itemIndex)]
     */
    unselectItem(itemIndex: number | any): void;
    /**
     * [descr:dxList.updateDimensions()]
     */
    updateDimensions(): DevExpress.core.utils.DxPromise<void>;
  }
  module dxList {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      GroupRenderedEvent: GroupRenderedEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemDeletedEvent: ItemDeletedEvent<TItem, TKey>;
      ItemDeletingEvent: ItemDeletingEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      ItemReorderedEvent: ItemReorderedEvent<TItem, TKey>;
      ItemSwipeEvent: ItemSwipeEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      PageLoadingEvent: PageLoadingEvent<TItem, TKey>;
      PullRefreshEvent: PullRefreshEvent<TItem, TKey>;
      ScrollEvent: ScrollEvent<TItem, TKey>;
      SelectAllValueChangedEvent: SelectAllValueChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
    };
    export type GroupRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> & {
      readonly groupData?: any;
      readonly groupElement?: DevExpress.core.DxElement;
      readonly groupIndex?: number;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxList<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxList<TItem, TKey>,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      ListItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxList<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      ListItemInfo<TItem>;
    export type ItemDeletedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> & ListItemInfo<TItem>;
    export type ItemDeletingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> &
      ListItemInfo<TItem> & {
        cancel?: boolean | PromiseLike<void>;
      };
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxList<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      ListItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends Item | any = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemReorderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> &
      ListItemInfo<TItem> & {
        readonly fromIndex: number;
        readonly toIndex: number;
      };
    export type ItemSwipeEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxList<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      ListItemInfo<TItem> & {
        readonly direction: string;
      };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface ListItemInfo<TItem extends ItemLike> {
      readonly itemData?: TItem;
      readonly itemElement: DevExpress.core.DxElement;
      readonly itemIndex: number | { group: number; item: number };
    }
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type PageLoadingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>>;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxListOptions<TItem, TKey>;
    export type PullRefreshEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>>;
    export type ScrollEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxList<TItem, TKey>, Event> &
      ScrollInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ScrollInfo {
      readonly scrollOffset?: any;
      readonly reachedLeft: boolean;
      readonly reachedRight: boolean;
      readonly reachedTop: boolean;
      readonly reachedBottom: boolean;
    }
    export type SelectAllValueChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> & {
      readonly value: boolean;
    };
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxList<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxListItem extends CollectionWidgetItem {
    /**
     * [descr:dxListItem.badge]
     */
    badge?: string;
    /**
     * [descr:dxListItem.icon]
     */
    icon?: string;
    /**
     * [descr:dxListItem.key]
     */
    key?: string;
    /**
     * [descr:dxListItem.showChevron]
     */
    showChevron?: boolean;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxListOptions<
    TItem extends DevExpress.ui.dxList.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<dxList<TItem, TKey>, TItem, TKey>,
      SearchBoxMixinOptions {
    /**
     * [descr:dxListOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxListOptions.allowItemDeleting]
     */
    allowItemDeleting?: boolean;
    /**
     * [descr:dxListOptions.bounceEnabled]
     */
    bounceEnabled?: boolean;
    /**
     * [descr:dxListOptions.collapsibleGroups]
     */
    collapsibleGroups?: boolean;
    /**
     * [descr:dxListOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxListOptions.displayExpr]
     */
    displayExpr?: string | ((item: TItem) => string);
    /**
     * [descr:dxListOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxListOptions.groupTemplate]
     */
    groupTemplate?:
      | DevExpress.core.template
      | ((
          groupData: any,
          groupIndex: number,
          groupElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxListOptions.grouped]
     */
    grouped?: boolean;
    /**
     * [descr:dxListOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxListOptions.indicateLoading]
     */
    indicateLoading?: boolean;
    /**
     * [descr:dxListOptions.itemDeleteMode]
     */
    itemDeleteMode?:
      | 'context'
      | 'slideButton'
      | 'slideItem'
      | 'static'
      | 'swipe'
      | 'toggle';
    /**
     * [descr:dxListOptions.itemDragging]
     */
    itemDragging?: dxSortableOptions;
    /**
     * [descr:dxListOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxListOptions.menuItems]
     */
    menuItems?: Array<{
      /**
       * [descr:dxListOptions.menuItems.action]
       */
      action?: (itemElement: DevExpress.core.DxElement, itemData: TItem) => any;
      /**
       * [descr:dxListOptions.menuItems.text]
       */
      text?: string;
    }>;
    /**
     * [descr:dxListOptions.menuMode]
     */
    menuMode?: 'context' | 'slide';
    /**
     * [descr:dxListOptions.nextButtonText]
     */
    nextButtonText?: string;
    /**
     * [descr:dxListOptions.onGroupRendered]
     */
    onGroupRendered?: (
      e: DevExpress.ui.dxList.GroupRenderedEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.onItemClick]
     */
    onItemClick?:
      | ((e: DevExpress.ui.dxList.ItemClickEvent<TItem, TKey>) => void)
      | string;
    /**
     * [descr:dxListOptions.onItemContextMenu]
     */
    onItemContextMenu?: (
      e: DevExpress.ui.dxList.ItemContextMenuEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.onItemDeleted]
     */
    onItemDeleted?: (
      e: DevExpress.ui.dxList.ItemDeletedEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.onItemDeleting]
     */
    onItemDeleting?: (
      e: DevExpress.ui.dxList.ItemDeletingEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.onItemHold]
     */
    onItemHold?: (e: DevExpress.ui.dxList.ItemHoldEvent<TItem, TKey>) => void;
    /**
     * [descr:dxListOptions.onItemReordered]
     */
    onItemReordered?: (
      e: DevExpress.ui.dxList.ItemReorderedEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.onItemSwipe]
     */
    onItemSwipe?: (e: DevExpress.ui.dxList.ItemSwipeEvent<TItem, TKey>) => void;
    /**
     * [descr:dxListOptions.onPageLoading]
     */
    onPageLoading?: (
      e: DevExpress.ui.dxList.PageLoadingEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.onPullRefresh]
     */
    onPullRefresh?: (
      e: DevExpress.ui.dxList.PullRefreshEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.onScroll]
     */
    onScroll?: (e: DevExpress.ui.dxList.ScrollEvent<TItem, TKey>) => void;
    /**
     * [descr:dxListOptions.onSelectAllValueChanged]
     */
    onSelectAllValueChanged?: (
      e: DevExpress.ui.dxList.SelectAllValueChangedEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxListOptions.pageLoadMode]
     */
    pageLoadMode?: 'nextButton' | 'scrollBottom';
    /**
     * [descr:dxListOptions.pageLoadingText]
     */
    pageLoadingText?: string;
    /**
     * [descr:dxListOptions.pullRefreshEnabled]
     */
    pullRefreshEnabled?: boolean;
    /**
     * [descr:dxListOptions.pulledDownText]
     */
    pulledDownText?: string;
    /**
     * [descr:dxListOptions.pullingDownText]
     */
    pullingDownText?: string;
    /**
     * [descr:dxListOptions.refreshingText]
     */
    refreshingText?: string;
    /**
     * [descr:dxListOptions.repaintChangesOnly]
     */
    repaintChangesOnly?: boolean;
    /**
     * [descr:dxListOptions.scrollByContent]
     */
    scrollByContent?: boolean;
    /**
     * [descr:dxListOptions.scrollByThumb]
     */
    scrollByThumb?: boolean;
    /**
     * [descr:dxListOptions.scrollingEnabled]
     */
    scrollingEnabled?: boolean;
    /**
     * [descr:dxListOptions.selectAllMode]
     */
    selectAllMode?: 'allPages' | 'page';
    /**
     * [descr:dxListOptions.selectionMode]
     */
    selectionMode?: 'all' | 'multiple' | 'none' | 'single';
    /**
     * [descr:dxListOptions.showScrollbar]
     */
    showScrollbar?: 'always' | 'never' | 'onHover' | 'onScroll';
    /**
     * [descr:dxListOptions.showSelectionControls]
     */
    showSelectionControls?: boolean;
    /**
     * [descr:dxListOptions.selectAllText]
     */
    selectAllText?: string;
    /**
     * [descr:dxListOptions.useNativeScrolling]
     */
    useNativeScrolling?: boolean;
  }
  /**
   * [descr:dxLoadIndicator]
   */
  export class dxLoadIndicator extends Widget<dxLoadIndicatorOptions> {}
  module dxLoadIndicator {
    export type ContentReadyEvent =
      DevExpress.events.EventInfo<dxLoadIndicator>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxLoadIndicator>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxLoadIndicator>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxLoadIndicator> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxLoadIndicatorOptions;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLoadIndicatorOptions
    extends WidgetOptions<dxLoadIndicator> {
    /**
     * [descr:dxLoadIndicatorOptions.indicatorSrc]
     */
    indicatorSrc?: string;
  }
  /**
   * [descr:dxLoadPanel]
   */
  export class dxLoadPanel extends dxOverlay<dxLoadPanelOptions> {}
  module dxLoadPanel {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxLoadPanel>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxLoadPanel>;
    export type HiddenEvent = DevExpress.events.EventInfo<dxLoadPanel>;
    export type HidingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxLoadPanel>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxLoadPanel>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxLoadPanel> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxLoadPanelOptions;
    export type ShowingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxLoadPanel>;
    export type ShownEvent = DevExpress.events.EventInfo<dxLoadPanel>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLoadPanelAnimation extends dxOverlayAnimation {
    /**
     * [descr:dxLoadPanelOptions.animation.hide]
     */
    hide?: AnimationConfig;
    /**
     * [descr:dxLoadPanelOptions.animation.show]
     */
    show?: AnimationConfig;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLoadPanelOptions extends dxOverlayOptions<dxLoadPanel> {
    /**
     * [descr:dxLoadPanelOptions.animation]
     */
    animation?: dxLoadPanelAnimation;
    /**
     * [descr:dxLoadPanelOptions.container]
     */
    container?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxLoadPanelOptions.delay]
     */
    delay?: number;
    /**
     * [descr:dxLoadPanelOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxLoadPanelOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxLoadPanelOptions.indicatorSrc]
     */
    indicatorSrc?: string;
    /**
     * [descr:dxLoadPanelOptions.maxHeight]
     */
    maxHeight?: number | string | (() => number | string);
    /**
     * [descr:dxLoadPanelOptions.maxWidth]
     */
    maxWidth?: number | string | (() => number | string);
    /**
     * [descr:dxLoadPanelOptions.message]
     */
    message?: string;
    /**
     * [descr:dxLoadPanelOptions.position]
     */
    position?:
      | 'bottom'
      | 'center'
      | 'left'
      | 'left bottom'
      | 'left top'
      | 'right'
      | 'right bottom'
      | 'right top'
      | 'top'
      | PositionConfig
      | Function;
    /**
     * [descr:dxLoadPanelOptions.shadingColor]
     */
    shadingColor?: string;
    /**
     * [descr:dxLoadPanelOptions.showIndicator]
     */
    showIndicator?: boolean;
    /**
     * [descr:dxLoadPanelOptions.showPane]
     */
    showPane?: boolean;
    /**
     * [descr:dxLoadPanelOptions.width]
     */
    width?: number | string | (() => number | string);
  }
  /**
   * [descr:dxLookup]
   */
  export class dxLookup extends dxDropDownList<dxLookupOptions> {}
  module dxLookup {
    export type ClosedEvent = DevExpress.events.EventInfo<dxLookup>;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxLookup>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxLookup>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxLookup>;
    export type ItemClickEvent = DevExpress.events.NativeEventInfo<
      dxLookup,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      DevExpress.events.ItemInfo;
    export type OpenedEvent = DevExpress.events.EventInfo<dxLookup>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxLookup> &
      DevExpress.events.ChangedOptionInfo;
    export type PageLoadingEvent = DevExpress.events.EventInfo<dxLookup>;
    export type Properties = dxLookupOptions;
    export type PullRefreshEvent = DevExpress.events.EventInfo<dxLookup>;
    export type ScrollEvent = DevExpress.events.NativeEventInfo<
      dxLookup,
      MouseEvent | Event
    > &
      DevExpress.ui.dxList.ScrollInfo;
    export type SelectionChangedEvent = DevExpress.events.EventInfo<dxLookup> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo;
    export type TitleRenderedEvent = DevExpress.events.EventInfo<dxLookup> &
      DevExpress.ui.dxPopup.TitleRenderedInfo;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxLookup,
      KeyboardEvent | MouseEvent | PointerEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLookupOptions extends dxDropDownListOptions<dxLookup> {
    /**
     * [descr:dxLookupOptions.applyButtonText]
     */
    applyButtonText?: string;
    /**
     * [descr:dxLookupOptions.applyValueMode]
     */
    applyValueMode?: 'instantly' | 'useButtons';
    /**
     * [descr:dxLookupOptions.cancelButtonText]
     */
    cancelButtonText?: string;
    /**
     * [descr:dxLookupOptions.cleanSearchOnOpening]
     */
    cleanSearchOnOpening?: boolean;
    /**
     * [descr:dxLookupOptions.clearButtonText]
     */
    clearButtonText?: string;
    /**
     * [descr:dxLookupOptions.fieldTemplate]
     */
    fieldTemplate?:
      | DevExpress.core.template
      | ((
          selectedItem: any,
          fieldElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxLookupOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxLookupOptions.fullScreen]
     * @deprecated [depNote:dxLookupOptions.fullScreen]
     */
    fullScreen?: boolean;
    /**
     * [descr:dxLookupOptions.groupTemplate]
     */
    groupTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxLookupOptions.grouped]
     */
    grouped?: boolean;
    /**
     * [descr:dxLookupOptions.nextButtonText]
     */
    nextButtonText?: string;
    /**
     * [descr:dxLookupOptions.onPageLoading]
     */
    onPageLoading?: (e: DevExpress.ui.dxLookup.PageLoadingEvent) => void;
    /**
     * [descr:dxLookupOptions.onPullRefresh]
     */
    onPullRefresh?: (e: DevExpress.ui.dxLookup.PullRefreshEvent) => void;
    /**
     * [descr:dxLookupOptions.onScroll]
     */
    onScroll?: (e: DevExpress.ui.dxLookup.ScrollEvent) => void;
    /**
     * [descr:dxLookupOptions.onValueChanged]
     */
    onValueChanged?: (e: DevExpress.ui.dxLookup.ValueChangedEvent) => void;
    /**
     * [descr:dxLookupOptions.pageLoadMode]
     */
    pageLoadMode?: 'nextButton' | 'scrollBottom';
    /**
     * [descr:dxLookupOptions.pageLoadingText]
     */
    pageLoadingText?: string;
    /**
     * [descr:dxLookupOptions.placeholder]
     */
    placeholder?: string;
    /**
     * [descr:dxLookupOptions.pullRefreshEnabled]
     */
    pullRefreshEnabled?: boolean;
    /**
     * [descr:dxLookupOptions.pulledDownText]
     */
    pulledDownText?: string;
    /**
     * [descr:dxLookupOptions.pullingDownText]
     */
    pullingDownText?: string;
    /**
     * [descr:dxLookupOptions.refreshingText]
     */
    refreshingText?: string;
    /**
     * [descr:dxLookupOptions.searchEnabled]
     */
    searchEnabled?: boolean;
    /**
     * [descr:dxLookupOptions.searchPlaceholder]
     */
    searchPlaceholder?: string;
    /**
     * [descr:dxLookupOptions.showCancelButton]
     */
    showCancelButton?: boolean;
    /**
     * [descr:dxLookupOptions.showClearButton]
     */
    showClearButton?: boolean;
    /**
     * [descr:dxLookupOptions.useNativeScrolling]
     */
    useNativeScrolling?: boolean;
    /**
     * [descr:dxLookupOptions.usePopover]
     */
    usePopover?: boolean;
    /**
     * [descr:dxLookupOptions.dropDownCentered]
     */
    dropDownCentered?: boolean;
    /**
     * [descr:dxLookupOptions.dropDownOptions]
     */
    dropDownOptions?: DevExpress.ui.dxPopover.Properties;
  }
  /**
   * [descr:dxMap]
   */
  export class dxMap extends Widget<dxMapOptions> {
    /**
     * [descr:dxMap.addMarker(markerOptions)]
     */
    addMarker(
      markerOptions: any | Array<any>
    ): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:dxMap.addRoute(routeOptions)]
     */
    addRoute(options: any | Array<any>): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:dxMap.removeMarker(marker)]
     */
    removeMarker(
      marker: any | number | Array<any>
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxMap.removeRoute(route)]
     */
    removeRoute(
      route: any | number | Array<any>
    ): DevExpress.core.utils.DxPromise<void>;
  }
  module dxMap {
    export type ClickEvent = DevExpress.events.NativeEventInfo<
      dxMap,
      MouseEvent | PointerEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxMap>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxMap>;
    export type MarkerAddedEvent = DevExpress.events.EventInfo<dxMap> & {
      readonly options: any;
      originalMarker: any;
    };
    export type MarkerRemovedEvent = DevExpress.events.EventInfo<dxMap> & {
      readonly options?: any;
    };
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxMap> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxMapOptions;
    export type ReadyEvent = DevExpress.events.EventInfo<dxMap> & {
      originalMap: any;
    };
    export type RouteAddedEvent = DevExpress.events.EventInfo<dxMap> & {
      readonly options: any;
      originalRoute: any;
    };
    export type RouteRemovedEvent = DevExpress.events.EventInfo<dxMap> & {
      readonly options?: any;
    };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxMapOptions extends WidgetOptions<dxMap> {
    /**
     * [descr:dxMapOptions.apiKey]
     */
    apiKey?:
      | string
      | {
          /**
           * [descr:dxMapOptions.apiKey.bing]
           */
          bing?: string;
          /**
           * [descr:dxMapOptions.apiKey.google]
           */
          google?: string;
          /**
           * [descr:dxMapOptions.apiKey.googleStatic]
           */
          googleStatic?: string;
        };
    /**
     * [descr:dxMapOptions.autoAdjust]
     */
    autoAdjust?: boolean;
    /**
     * [descr:dxMapOptions.center]
     */
    center?: any | string | Array<number>;
    /**
     * [descr:dxMapOptions.controls]
     */
    controls?: boolean;
    /**
     * [descr:dxMapOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxMapOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxMapOptions.markerIconSrc]
     */
    markerIconSrc?: string;
    /**
     * [descr:dxMapOptions.markers]
     */
    markers?: Array<{
      /**
       * [descr:dxMapOptions.markers.iconSrc]
       */
      iconSrc?: string;
      /**
       * [descr:dxMapOptions.markers.location]
       */
      location?: any | string | Array<number>;
      /**
       * [descr:dxMapOptions.markers.onClick]
       */
      onClick?: Function;
      /**
       * [descr:dxMapOptions.markers.tooltip]
       */
      tooltip?:
        | string
        | {
            /**
             * [descr:dxMapOptions.markers.tooltip.isShown]
             */
            isShown?: boolean;
            /**
             * [descr:dxMapOptions.markers.tooltip.text]
             */
            text?: string;
          };
    }>;
    /**
     * [descr:dxMapOptions.onClick]
     */
    onClick?: ((e: DevExpress.ui.dxMap.ClickEvent) => void) | string;
    /**
     * [descr:dxMapOptions.onMarkerAdded]
     */
    onMarkerAdded?: (e: DevExpress.ui.dxMap.MarkerAddedEvent) => void;
    /**
     * [descr:dxMapOptions.onMarkerRemoved]
     */
    onMarkerRemoved?: (e: DevExpress.ui.dxMap.MarkerRemovedEvent) => void;
    /**
     * [descr:dxMapOptions.onReady]
     */
    onReady?: (e: DevExpress.ui.dxMap.ReadyEvent) => void;
    /**
     * [descr:dxMapOptions.onRouteAdded]
     */
    onRouteAdded?: (e: DevExpress.ui.dxMap.RouteAddedEvent) => void;
    /**
     * [descr:dxMapOptions.onRouteRemoved]
     */
    onRouteRemoved?: (e: DevExpress.ui.dxMap.RouteRemovedEvent) => void;
    /**
     * [descr:dxMapOptions.provider]
     */
    provider?: 'bing' | 'google' | 'googleStatic';
    /**
     * [descr:dxMapOptions.routes]
     */
    routes?: Array<{
      /**
       * [descr:dxMapOptions.routes.color]
       */
      color?: string;
      /**
       * [descr:dxMapOptions.routes.locations]
       */
      locations?: Array<any>;
      /**
       * [descr:dxMapOptions.routes.mode]
       */
      mode?: 'driving' | 'walking';
      /**
       * [descr:dxMapOptions.routes.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxMapOptions.routes.weight]
       */
      weight?: number;
    }>;
    /**
     * [descr:dxMapOptions.type]
     */
    type?: 'hybrid' | 'roadmap' | 'satellite';
    /**
     * [descr:dxMapOptions.width]
     */
    width?: number | string | (() => number | string);
    /**
     * [descr:dxMapOptions.zoom]
     */
    zoom?: number;
  }
  /**
   * [descr:dxMenu]
   */
  export class dxMenu<TKey = any> extends dxMenuBase<
    dxMenuOptions<TKey>,
    dxMenuItem,
    TKey
  > {}
  module dxMenu {
    export type ContentReadyEvent<TKey = any> = DevExpress.events.EventInfo<
      dxMenu<TKey>
    >;
    export type DisposingEvent<TKey = any> = DevExpress.events.EventInfo<
      dxMenu<TKey>
    >;
    export type ExplicitTypes<TKey = any> = {
      Properties: Properties<TKey>;
      ContentReadyEvent: ContentReadyEvent<TKey>;
      DisposingEvent: DisposingEvent<TKey>;
      InitializedEvent: InitializedEvent<TKey>;
      ItemClickEvent: ItemClickEvent<TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TKey>;
      OptionChangedEvent: OptionChangedEvent<TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TKey>;
      SubmenuHiddenEvent: SubmenuHiddenEvent<TKey>;
      SubmenuHidingEvent: SubmenuHidingEvent<TKey>;
      SubmenuShowingEvent: SubmenuShowingEvent<TKey>;
      SubmenuShownEvent: SubmenuShownEvent<TKey>;
    };
    export type InitializedEvent<TKey = any> =
      DevExpress.events.InitializedEventInfo<dxMenu<TKey>>;
    export type ItemClickEvent<TKey = any> = DevExpress.events.NativeEventInfo<
      dxMenu<TKey>
    > &
      DevExpress.events.ItemInfo<Item>;
    export type ItemContextMenuEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxMenu<TKey>> &
        DevExpress.events.ItemInfo<Item>;
    export type ItemRenderedEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxMenu<TKey>> &
        DevExpress.events.ItemInfo<Item>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface MenuBasePlainItem extends CollectionWidgetItem {
      /**
       * [descr:dxMenuBaseItem.beginGroup]
       */
      beginGroup?: boolean;
      /**
       * [descr:dxMenuBaseItem.closeMenuOnClick]
       */
      closeMenuOnClick?: boolean;
      /**
       * [descr:dxMenuBaseItem.disabled]
       */
      disabled?: boolean;
      /**
       * [descr:dxMenuBaseItem.icon]
       */
      icon?: string;
      /**
       * [descr:dxMenuBaseItem.selectable]
       */
      selectable?: boolean;
      /**
       * [descr:dxMenuBaseItem.selected]
       */
      selected?: boolean;
      /**
       * [descr:dxMenuBaseItem.text]
       */
      text?: string;
      /**
       * [descr:dxMenuBaseItem.visible]
       */
      visible?: boolean;
    }
    export type OptionChangedEvent<TKey = any> = DevExpress.events.EventInfo<
      dxMenu<TKey>
    > &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<TKey = any> = dxMenuOptions<TKey>;
    export type SelectionChangedEvent<TKey = any> = DevExpress.events.EventInfo<
      dxMenu<TKey>
    > &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<Item>;
    export type SubmenuHiddenEvent<TKey = any> = DevExpress.events.EventInfo<
      dxMenu<TKey>
    > & {
      readonly rootItem?: DevExpress.core.DxElement;
    };
    export type SubmenuHidingEvent<TKey = any> = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxMenu<TKey>> & {
        readonly rootItem?: DevExpress.core.DxElement;
      };
    export type SubmenuShowingEvent<TKey = any> = DevExpress.events.EventInfo<
      dxMenu<TKey>
    > & {
      readonly rootItem?: DevExpress.core.DxElement;
    };
    export type SubmenuShownEvent<TKey = any> = DevExpress.events.EventInfo<
      dxMenu<TKey>
    > & {
      readonly rootItem?: DevExpress.core.DxElement;
    };
  }
  /**
   * [descr:dxMenuBase]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxMenuBase<
    TProperties extends dxMenuBaseOptions<any, TItem, TKey>,
    TItem extends dxMenuBaseItem = dxMenuBaseItem,
    TKey = any
  > extends HierarchicalCollectionWidget<TProperties, TItem, TKey> {
    /**
     * [descr:dxMenuBase.selectItem(itemElement)]
     */
    selectItem(itemElement: Element): void;
    /**
     * [descr:dxMenuBase.unselectItem(itemElement)]
     */
    unselectItem(itemElement: Element): void;
  }
  /**
   * [descr:dxMenuBaseItem]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxMenuBaseItem
    extends DevExpress.ui.dxMenu.MenuBasePlainItem {
    /**
     * [descr:dxMenuBaseItem.items]
     */
    items?: Array<dxMenuBaseItem>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxMenuBaseOptions<
    TComponent extends dxMenuBase<any, TItem, TKey>,
    TItem extends dxMenuBaseItem = dxMenuBaseItem,
    TKey = any
  > extends DevExpress.core.Skip<
      HierarchicalCollectionWidgetOptions<TComponent, TItem, TKey>,
      'dataSource'
    > {
    /**
     * [descr:dxMenuBaseOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxMenuBaseOptions.animation]
     */
    animation?: {
      /**
       * [descr:dxMenuBaseOptions.animation.hide]
       */
      hide?: AnimationConfig;
      /**
       * [descr:dxMenuBaseOptions.animation.show]
       */
      show?: AnimationConfig;
    };
    /**
     * [descr:dxMenuBaseOptions.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxMenuBaseOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxMenuBaseOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxMenuBaseOptions.selectByClick]
     */
    selectByClick?: boolean;
    /**
     * [descr:dxMenuBaseOptions.selectionMode]
     */
    selectionMode?: 'none' | 'single';
    /**
     * [descr:dxMenuBaseOptions.showSubmenuMode]
     */
    showSubmenuMode?:
      | {
          /**
           * [descr:dxMenuBaseOptions.showSubmenuMode.delay]
           */
          delay?:
            | {
                /**
                 * [descr:dxMenuBaseOptions.showSubmenuMode.delay.hide]
                 */
                hide?: number;
                /**
                 * [descr:dxMenuBaseOptions.showSubmenuMode.delay.show]
                 */
                show?: number;
              }
            | number;
          /**
           * [descr:dxMenuBaseOptions.showSubmenuMode.name]
           */
          name?: 'onClick' | 'onHover';
        }
      | 'onClick'
      | 'onHover';
  }
  /**
   * @deprecated Use DevExpress.ui.dxMenu.Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxMenuItem extends dxMenuBaseItem {
    /**
     * [descr:dxMenuItem.items]
     */
    items?: Array<DevExpress.ui.dxMenu.Item>;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxMenuOptions<TKey = any>
    extends dxMenuBaseOptions<dxMenu<TKey>, dxMenuItem, TKey> {
    /**
     * [descr:dxMenuOptions.adaptivityEnabled]
     */
    adaptivityEnabled?: boolean;
    /**
     * [descr:dxMenuOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<
      DevExpress.ui.dxMenu.Item,
      TKey
    >;
    /**
     * [descr:dxMenuOptions.hideSubmenuOnMouseLeave]
     */
    hideSubmenuOnMouseLeave?: boolean;
    /**
     * [descr:dxMenuOptions.items]
     */
    items?: Array<DevExpress.ui.dxMenu.Item>;
    /**
     * [descr:dxMenuOptions.onSubmenuHidden]
     */
    onSubmenuHidden?: (
      e: DevExpress.ui.dxMenu.SubmenuHiddenEvent<TKey>
    ) => void;
    /**
     * [descr:dxMenuOptions.onSubmenuHiding]
     */
    onSubmenuHiding?: (
      e: DevExpress.ui.dxMenu.SubmenuHidingEvent<TKey>
    ) => void;
    /**
     * [descr:dxMenuOptions.onSubmenuShowing]
     */
    onSubmenuShowing?: (
      e: DevExpress.ui.dxMenu.SubmenuShowingEvent<TKey>
    ) => void;
    /**
     * [descr:dxMenuOptions.onSubmenuShown]
     */
    onSubmenuShown?: (e: DevExpress.ui.dxMenu.SubmenuShownEvent<TKey>) => void;
    /**
     * [descr:dxMenuOptions.orientation]
     */
    orientation?: 'horizontal' | 'vertical';
    /**
     * [descr:dxMenuOptions.showFirstSubmenuMode]
     */
    showFirstSubmenuMode?:
      | {
          /**
           * [descr:dxMenuOptions.showFirstSubmenuMode.delay]
           */
          delay?:
            | {
                /**
                 * [descr:dxMenuOptions.showFirstSubmenuMode.delay.hide]
                 */
                hide?: number;
                /**
                 * [descr:dxMenuOptions.showFirstSubmenuMode.delay.show]
                 */
                show?: number;
              }
            | number;
          /**
           * [descr:dxMenuOptions.showFirstSubmenuMode.name]
           */
          name?: 'onClick' | 'onHover';
        }
      | 'onClick'
      | 'onHover';
    /**
     * [descr:dxMenuOptions.submenuDirection]
     */
    submenuDirection?: 'auto' | 'leftOrTop' | 'rightOrBottom';
  }
  /**
   * [descr:dxMultiView]
   */
  export class dxMultiView<
    TProperties extends dxMultiViewOptions<TItem, TKey> = dxMultiViewOptions<
      any,
      any
    >,
    TItem extends DevExpress.ui.dxMultiView.ItemLike = any,
    TKey = any
  > extends CollectionWidget<TProperties, TItem, TKey> {}
  module dxMultiView {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<MultiViewInstance<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<MultiViewInstance<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<MultiViewInstance<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<MultiViewInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<MultiViewInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<MultiViewInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<MultiViewInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface MultiViewInstance<TItem, TKey>
      extends dxMultiView<Properties<TItem, TKey>, TItem, TKey> {}
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<MultiViewInstance<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxMultiViewBaseOptions<MultiViewInstance<TItem, TKey>, TItem, TKey>;
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<MultiViewInstance<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxMultiViewBaseOptions<
    TComponent extends dxMultiView<any, TItem, TKey> = dxMultiView<
      any,
      any,
      any
    >,
    TItem extends DevExpress.ui.dxMultiView.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<TComponent, TItem, TKey> {
    /**
     * [descr:dxMultiViewOptions.animationEnabled]
     */
    animationEnabled?: boolean;
    /**
     * [descr:dxMultiViewOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxMultiViewOptions.deferRendering]
     */
    deferRendering?: boolean;
    /**
     * [descr:dxMultiViewOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxMultiViewOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxMultiViewOptions.loop]
     */
    loop?: boolean;
    /**
     * [descr:dxMultiViewOptions.selectedIndex]
     */
    selectedIndex?: number;
    /**
     * [descr:dxMultiViewOptions.swipeEnabled]
     */
    swipeEnabled?: boolean;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxMultiViewItem extends CollectionWidgetItem {}
  /**
   * @deprecated use DevExpress.ui.dxMultiView.Properties instead
   */
  export interface dxMultiViewOptions<
    TItem extends DevExpress.ui.dxMultiView.ItemLike = any,
    TKey = any
  > extends DevExpress.ui.dxMultiView.Properties<TItem, TKey> {}
  /**
   * [descr:dxNavBar]
   * @deprecated [depNote:dxNavBar]
   */
  export class dxNavBar<
    TItem extends DevExpress.ui.dxNavBar.ItemLike = any,
    TKey = any
  > extends dxTabs<dxNavBarOptions<TItem, TKey>, TItem, TKey> {}
  module dxNavBar {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxNavBar<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxNavBar<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxNavBar<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxNavBar<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxNavBar<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxNavBar<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxNavBar<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxNavBar<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxNavBarOptions<TItem, TKey>;
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxNavBar<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxNavBarItem extends DevExpress.ui.dxTabs.Item {
    /**
     * [descr:dxNavBarItem.badge]
     */
    badge?: string;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxNavBarOptions<
    TItem extends DevExpress.ui.dxNavBar.ItemLike = any,
    TKey = any
  > extends dxTabsBaseOptions<dxNavBar<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxNavBarOptions.scrollByContent]
     */
    scrollByContent?: boolean;
  }
  /**
   * [descr:dxNumberBox]
   */
  export class dxNumberBox extends dxTextEditor<dxNumberBoxOptions> {}
  module dxNumberBox {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      Event
    >;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxNumberBox>;
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      ClipboardEvent
    >;
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      ClipboardEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxNumberBox>;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxNumberBox>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      UIEvent
    >;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      KeyboardEvent
    >;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxNumberBox> &
      DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      ClipboardEvent
    >;
    export type Properties = dxNumberBoxOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxNumberBox,
      KeyboardEvent | MouseEvent | PointerEvent | TouchEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxNumberBoxOptions extends dxTextEditorOptions<dxNumberBox> {
    /**
     * [descr:dxNumberBoxOptions.buttons]
     */
    buttons?: Array<'clear' | 'spins' | dxTextEditorButton>;
    /**
     * [descr:dxNumberBoxOptions.format]
     */
    format?: Format;
    /**
     * [descr:dxNumberBoxOptions.invalidValueMessage]
     */
    invalidValueMessage?: string;
    /**
     * [descr:dxNumberBoxOptions.max]
     */
    max?: number;
    /**
     * [descr:dxNumberBoxOptions.min]
     */
    min?: number;
    /**
     * [descr:dxNumberBoxOptions.mode]
     */
    mode?: 'number' | 'text' | 'tel';
    /**
     * [descr:dxNumberBoxOptions.showSpinButtons]
     */
    showSpinButtons?: boolean;
    /**
     * [descr:dxNumberBoxOptions.step]
     */
    step?: number;
    /**
     * [descr:dxNumberBoxOptions.useLargeSpinButtons]
     */
    useLargeSpinButtons?: boolean;
    /**
     * [descr:dxNumberBoxOptions.value]
     */
    value?: number;
  }
  /**
   * [descr:dxOverlay]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxOverlay<TProperties> extends Widget<TProperties> {
    /**
     * [descr:dxOverlay.content()]
     */
    content(): DevExpress.core.DxElement;
    /**
     * [descr:dxOverlay.hide()]
     */
    hide(): DevExpress.core.utils.DxPromise<boolean>;
    /**
     * [descr:dxOverlay.repaint()]
     */
    repaint(): void;
    /**
     * [descr:dxOverlay.show()]
     */
    show(): DevExpress.core.utils.DxPromise<boolean>;
    /**
     * [descr:dxOverlay.toggle(showing)]
     */
    toggle(showing: boolean): DevExpress.core.utils.DxPromise<boolean>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxOverlayAnimation {
    /**
     * [descr:dxOverlayOptions.animation.hide]
     */
    hide?: AnimationConfig;
    /**
     * [descr:dxOverlayOptions.animation.show]
     */
    show?: AnimationConfig;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxOverlayOptions<TComponent>
    extends WidgetOptions<TComponent> {
    /**
     * [descr:dxOverlayOptions.animation]
     */
    animation?: dxOverlayAnimation;
    /**
     * [descr:dxOverlayOptions.closeOnOutsideClick]
     */
    closeOnOutsideClick?:
      | boolean
      | ((
          event: DevExpress.events.DxEvent<
            MouseEvent | PointerEvent | TouchEvent
          >
        ) => boolean);
    /**
     * [descr:dxOverlayOptions.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxOverlayOptions.copyRootClassesToWrapper]
     * @deprecated [depNote:dxOverlayOptions.copyRootClassesToWrapper]
     */
    copyRootClassesToWrapper?: boolean;
    /**
     * [descr:dxOverlayOptions.deferRendering]
     */
    deferRendering?: boolean;
    /**
     * [descr:dxOverlayOptions.dragAndResizeArea]
     */
    dragAndResizeArea?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxOverlayOptions.dragEnabled]
     */
    dragEnabled?: boolean;
    /**
     * [descr:dxOverlayOptions.dragOutsideBoundary]
     */
    dragOutsideBoundary?: boolean;
    /**
     * [descr:dxOverlayOptions.elementAttr]
     * @deprecated [depNote:dxOverlayOptions.elementAttr]
     */
    elementAttr?: any;
    /**
     * [descr:dxOverlayOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxOverlayOptions.maxHeight]
     */
    maxHeight?: number | string | (() => number | string);
    /**
     * [descr:dxOverlayOptions.maxWidth]
     */
    maxWidth?: number | string | (() => number | string);
    /**
     * [descr:dxOverlayOptions.minHeight]
     */
    minHeight?: number | string | (() => number | string);
    /**
     * [descr:dxOverlayOptions.minWidth]
     */
    minWidth?: number | string | (() => number | string);
    /**
     * [descr:dxOverlayOptions.onHidden]
     */
    onHidden?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:dxOverlayOptions.onHiding]
     */
    onHiding?: (
      e: DevExpress.events.Cancelable & DevExpress.events.EventInfo<TComponent>
    ) => void;
    /**
     * [descr:dxOverlayOptions.onShowing]
     */
    onShowing?: (
      e: DevExpress.events.Cancelable & DevExpress.events.EventInfo<TComponent>
    ) => void;
    /**
     * [descr:dxOverlayOptions.onShown]
     */
    onShown?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:dxOverlayOptions.position]
     */
    position?: any;
    /**
     * [descr:dxOverlayOptions.shading]
     */
    shading?: boolean;
    /**
     * [descr:dxOverlayOptions.shadingColor]
     */
    shadingColor?: string;
    /**
     * [descr:dxOverlayOptions.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxOverlayOptions.width]
     */
    width?: number | string | (() => number | string);
    /**
     * [descr:dxOverlayOptions.wrapperAttr]
     */
    wrapperAttr?: any;
    /**
     * [descr:dxOverlayOptions.hideOnParentScroll]
     */
    hideOnParentScroll?: boolean;
  }
  /**
   * [descr:dxPivotGrid]
   */
  export class dxPivotGrid extends Widget<dxPivotGridOptions> {
    /**
     * [descr:dxPivotGrid.bindChart(chart, integrationOptions)]
     */
    bindChart(
      chart: string | DevExpress.core.DxElement | any,
      integrationOptions: {
        inverted?: boolean;
        dataFieldsDisplayMode?: string;
        putDataFieldsInto?: string;
        alternateDataFields?: boolean;
        processCell?: Function;
        customizeChart?: Function;
        customizeSeries?: Function;
      }
    ): Function & null;
    /**
     * [descr:dxPivotGrid.exportToExcel()]
     * @deprecated [depNote:dxPivotGrid.exportToExcel()]
     */
    exportToExcel(): void;
    /**
     * [descr:dxPivotGrid.getDataSource()]
     */
    getDataSource(): DevExpress.data.PivotGridDataSource;
    /**
     * [descr:dxPivotGrid.getFieldChooserPopup()]
     */
    getFieldChooserPopup(): dxPopup;
    /**
     * [descr:dxPivotGrid.updateDimensions()]
     */
    updateDimensions(): void;
  }
  module dxPivotGrid {
    export type CellClickEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxPivotGrid> & {
        readonly area?: string;
        readonly cellElement?: DevExpress.core.DxElement;
        readonly cell?: Cell;
        readonly rowIndex?: number;
        readonly columnIndex?: number;
        readonly columnFields?: Array<DevExpress.data.PivotGridDataSource.Field>;
        readonly rowFields?: Array<DevExpress.data.PivotGridDataSource.Field>;
        readonly dataFields?: Array<DevExpress.data.PivotGridDataSource.Field>;
      };
    export type CellPreparedEvent = DevExpress.events.EventInfo<dxPivotGrid> & {
      readonly area?: string;
      readonly cellElement?: DevExpress.core.DxElement;
      readonly cell?: Cell;
      readonly rowIndex?: number;
      readonly columnIndex?: number;
    };
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxPivotGrid>;
    export type ContextMenuPreparingEvent =
      DevExpress.events.EventInfo<dxPivotGrid> & {
        readonly area?: string;
        readonly cell?: Cell;
        readonly cellElement?: DevExpress.core.DxElement;
        readonly columnIndex?: number;
        readonly rowIndex?: number;
        readonly dataFields?: Array<DevExpress.data.PivotGridDataSource.Field>;
        readonly rowFields?: Array<DevExpress.data.PivotGridDataSource.Field>;
        readonly columnFields?: Array<DevExpress.data.PivotGridDataSource.Field>;
        readonly field?: DevExpress.data.PivotGridDataSource.Field;
        items?: Array<any>;
      };
    export type DisposingEvent = DevExpress.events.EventInfo<dxPivotGrid>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxPivotGrid>;
    export type ExportingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxPivotGrid> & {
        fileName?: string;
      };
    export type FileSavingEvent = DevExpress.events.Cancelable & {
      readonly component: dxPivotGrid;
      readonly element: DevExpress.core.DxElement;
      readonly data?: Blob;
      readonly format?: string;
      fileName?: string;
    };
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxPivotGrid>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxPivotGrid> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxPivotGridOptions;
  }
  /**
   * [descr:dxPivotGridFieldChooser]
   */
  export class dxPivotGridFieldChooser extends Widget<dxPivotGridFieldChooserOptions> {
    /**
     * [descr:dxPivotGridFieldChooser.applyChanges()]
     */
    applyChanges(): void;
    /**
     * [descr:dxPivotGridFieldChooser.cancelChanges()]
     */
    cancelChanges(): void;
    /**
     * [descr:dxPivotGridFieldChooser.getDataSource()]
     */
    getDataSource(): DevExpress.data.PivotGridDataSource;
    /**
     * [descr:dxPivotGridFieldChooser.updateDimensions()]
     */
    updateDimensions(): void;
  }
  module dxPivotGridFieldChooser {
    export type ContentReadyEvent =
      DevExpress.events.EventInfo<dxPivotGridFieldChooser>;
    export type ContextMenuPreparingEvent =
      DevExpress.events.EventInfo<dxPivotGridFieldChooser> & {
        readonly area?: string;
        readonly field?: DevExpress.data.PivotGridDataSource.Field;
        readonly event?: DevExpress.events.DxEvent;
        items?: Array<any>;
      };
    export type DisposingEvent =
      DevExpress.events.EventInfo<dxPivotGridFieldChooser>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxPivotGridFieldChooser>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxPivotGridFieldChooser> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxPivotGridFieldChooserOptions;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPivotGridFieldChooserOptions
    extends WidgetOptions<dxPivotGridFieldChooser> {
    /**
     * [descr:dxPivotGridFieldChooserOptions.allowSearch]
     */
    allowSearch?: boolean;
    /**
     * [descr:dxPivotGridFieldChooserOptions.applyChangesMode]
     */
    applyChangesMode?: 'instantly' | 'onDemand';
    /**
     * [descr:dxPivotGridFieldChooserOptions.dataSource]
     */
    dataSource?: DevExpress.data.PivotGridDataSource;
    /**
     * [descr:dxPivotGridFieldChooserOptions.headerFilter]
     */
    headerFilter?: {
      /**
       * [descr:dxPivotGridFieldChooserOptions.headerFilter.allowSearch]
       */
      allowSearch?: boolean;
      /**
       * [descr:dxPivotGridFieldChooserOptions.headerFilter.height]
       */
      height?: number;
      /**
       * [descr:dxPivotGridFieldChooserOptions.headerFilter.searchTimeout]
       */
      searchTimeout?: number;
      /**
       * [descr:dxPivotGridFieldChooserOptions.headerFilter.showRelevantValues]
       */
      showRelevantValues?: boolean;
      /**
       * [descr:dxPivotGridFieldChooserOptions.headerFilter.texts]
       */
      texts?: {
        /**
         * [descr:dxPivotGridFieldChooserOptions.headerFilter.texts.cancel]
         */
        cancel?: string;
        /**
         * [descr:dxPivotGridFieldChooserOptions.headerFilter.texts.emptyValue]
         */
        emptyValue?: string;
        /**
         * [descr:dxPivotGridFieldChooserOptions.headerFilter.texts.ok]
         */
        ok?: string;
      };
      /**
       * [descr:dxPivotGridFieldChooserOptions.headerFilter.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPivotGridFieldChooserOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxPivotGridFieldChooserOptions.layout]
     */
    layout?: 0 | 1 | 2;
    /**
     * [descr:dxPivotGridFieldChooserOptions.onContextMenuPreparing]
     */
    onContextMenuPreparing?: (
      e: DevExpress.ui.dxPivotGridFieldChooser.ContextMenuPreparingEvent
    ) => void;
    /**
     * [descr:dxPivotGridFieldChooserOptions.searchTimeout]
     */
    searchTimeout?: number;
    /**
     * [descr:dxPivotGridFieldChooserOptions.state]
     */
    state?: any;
    /**
     * [descr:dxPivotGridFieldChooserOptions.texts]
     */
    texts?: {
      /**
       * [descr:dxPivotGridFieldChooserOptions.texts.allFields]
       */
      allFields?: string;
      /**
       * [descr:dxPivotGridFieldChooserOptions.texts.columnFields]
       */
      columnFields?: string;
      /**
       * [descr:dxPivotGridFieldChooserOptions.texts.dataFields]
       */
      dataFields?: string;
      /**
       * [descr:dxPivotGridFieldChooserOptions.texts.filterFields]
       */
      filterFields?: string;
      /**
       * [descr:dxPivotGridFieldChooserOptions.texts.rowFields]
       */
      rowFields?: string;
    };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPivotGridOptions extends WidgetOptions<dxPivotGrid> {
    /**
     * [descr:dxPivotGridOptions.allowExpandAll]
     */
    allowExpandAll?: boolean;
    /**
     * [descr:dxPivotGridOptions.allowFiltering]
     */
    allowFiltering?: boolean;
    /**
     * [descr:dxPivotGridOptions.allowSorting]
     */
    allowSorting?: boolean;
    /**
     * [descr:dxPivotGridOptions.allowSortingBySummary]
     */
    allowSortingBySummary?: boolean;
    /**
     * [descr:dxPivotGridOptions.dataFieldArea]
     */
    dataFieldArea?: 'column' | 'row';
    /**
     * [descr:dxPivotGridOptions.dataSource]
     */
    dataSource?:
      | Array<any>
      | DevExpress.data.PivotGridDataSource
      | DevExpress.data.PivotGridDataSource.Options;
    /**
     * [descr:dxPivotGridOptions.encodeHtml]
     */
    encodeHtml?: boolean;
    /**
     * [descr:dxPivotGridOptions.export]
     */
    export?: {
      /**
       * [descr:dxPivotGridOptions.export.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxPivotGridOptions.export.fileName]
       * @deprecated [depNote:dxPivotGridOptions.export.fileName]
       */
      fileName?: string;
      /**
       * [descr:dxPivotGridOptions.export.ignoreExcelErrors]
       * @deprecated [depNote:dxPivotGridOptions.export.ignoreExcelErrors]
       */
      ignoreExcelErrors?: boolean;
      /**
       * [descr:dxPivotGridOptions.export.proxyUrl]
       * @deprecated [depNote:dxPivotGridOptions.export.proxyUrl]
       */
      proxyUrl?: string;
    };
    /**
     * [descr:dxPivotGridOptions.fieldChooser]
     */
    fieldChooser?: {
      /**
       * [descr:dxPivotGridOptions.fieldChooser.allowSearch]
       */
      allowSearch?: boolean;
      /**
       * [descr:dxPivotGridOptions.fieldChooser.applyChangesMode]
       */
      applyChangesMode?: 'instantly' | 'onDemand';
      /**
       * [descr:dxPivotGridOptions.fieldChooser.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxPivotGridOptions.fieldChooser.height]
       */
      height?: number;
      /**
       * [descr:dxPivotGridOptions.fieldChooser.layout]
       */
      layout?: 0 | 1 | 2;
      /**
       * [descr:dxPivotGridOptions.fieldChooser.searchTimeout]
       */
      searchTimeout?: number;
      /**
       * [descr:dxPivotGridOptions.fieldChooser.texts]
       */
      texts?: {
        /**
         * [descr:dxPivotGridOptions.fieldChooser.texts.allFields]
         */
        allFields?: string;
        /**
         * [descr:dxPivotGridOptions.fieldChooser.texts.columnFields]
         */
        columnFields?: string;
        /**
         * [descr:dxPivotGridOptions.fieldChooser.texts.dataFields]
         */
        dataFields?: string;
        /**
         * [descr:dxPivotGridOptions.fieldChooser.texts.filterFields]
         */
        filterFields?: string;
        /**
         * [descr:dxPivotGridOptions.fieldChooser.texts.rowFields]
         */
        rowFields?: string;
      };
      /**
       * [descr:dxPivotGridOptions.fieldChooser.title]
       */
      title?: string;
      /**
       * [descr:dxPivotGridOptions.fieldChooser.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPivotGridOptions.fieldPanel]
     */
    fieldPanel?: {
      /**
       * [descr:dxPivotGridOptions.fieldPanel.allowFieldDragging]
       */
      allowFieldDragging?: boolean;
      /**
       * [descr:dxPivotGridOptions.fieldPanel.showColumnFields]
       */
      showColumnFields?: boolean;
      /**
       * [descr:dxPivotGridOptions.fieldPanel.showDataFields]
       */
      showDataFields?: boolean;
      /**
       * [descr:dxPivotGridOptions.fieldPanel.showFilterFields]
       */
      showFilterFields?: boolean;
      /**
       * [descr:dxPivotGridOptions.fieldPanel.showRowFields]
       */
      showRowFields?: boolean;
      /**
       * [descr:dxPivotGridOptions.fieldPanel.texts]
       */
      texts?: {
        /**
         * [descr:dxPivotGridOptions.fieldPanel.texts.columnFieldArea]
         */
        columnFieldArea?: string;
        /**
         * [descr:dxPivotGridOptions.fieldPanel.texts.dataFieldArea]
         */
        dataFieldArea?: string;
        /**
         * [descr:dxPivotGridOptions.fieldPanel.texts.filterFieldArea]
         */
        filterFieldArea?: string;
        /**
         * [descr:dxPivotGridOptions.fieldPanel.texts.rowFieldArea]
         */
        rowFieldArea?: string;
      };
      /**
       * [descr:dxPivotGridOptions.fieldPanel.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxPivotGridOptions.headerFilter]
     */
    headerFilter?: {
      /**
       * [descr:dxPivotGridOptions.headerFilter.allowSearch]
       */
      allowSearch?: boolean;
      /**
       * [descr:dxPivotGridOptions.headerFilter.height]
       */
      height?: number;
      /**
       * [descr:dxPivotGridOptions.headerFilter.searchTimeout]
       */
      searchTimeout?: number;
      /**
       * [descr:dxPivotGridOptions.headerFilter.showRelevantValues]
       */
      showRelevantValues?: boolean;
      /**
       * [descr:dxPivotGridOptions.headerFilter.texts]
       */
      texts?: {
        /**
         * [descr:dxPivotGridOptions.headerFilter.texts.cancel]
         */
        cancel?: string;
        /**
         * [descr:dxPivotGridOptions.headerFilter.texts.emptyValue]
         */
        emptyValue?: string;
        /**
         * [descr:dxPivotGridOptions.headerFilter.texts.ok]
         */
        ok?: string;
      };
      /**
       * [descr:dxPivotGridOptions.headerFilter.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPivotGridOptions.hideEmptySummaryCells]
     */
    hideEmptySummaryCells?: boolean;
    /**
     * [descr:dxPivotGridOptions.loadPanel]
     */
    loadPanel?: {
      /**
       * [descr:dxPivotGridOptions.loadPanel.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxPivotGridOptions.loadPanel.height]
       */
      height?: number;
      /**
       * [descr:dxPivotGridOptions.loadPanel.indicatorSrc]
       */
      indicatorSrc?: string;
      /**
       * [descr:dxPivotGridOptions.loadPanel.shading]
       */
      shading?: boolean;
      /**
       * [descr:dxPivotGridOptions.loadPanel.shadingColor]
       */
      shadingColor?: string;
      /**
       * [descr:dxPivotGridOptions.loadPanel.showIndicator]
       */
      showIndicator?: boolean;
      /**
       * [descr:dxPivotGridOptions.loadPanel.showPane]
       */
      showPane?: boolean;
      /**
       * [descr:dxPivotGridOptions.loadPanel.text]
       */
      text?: string;
      /**
       * [descr:dxPivotGridOptions.loadPanel.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPivotGridOptions.onCellClick]
     */
    onCellClick?: (e: DevExpress.ui.dxPivotGrid.CellClickEvent) => void;
    /**
     * [descr:dxPivotGridOptions.onCellPrepared]
     */
    onCellPrepared?: (e: DevExpress.ui.dxPivotGrid.CellPreparedEvent) => void;
    /**
     * [descr:dxPivotGridOptions.onContextMenuPreparing]
     */
    onContextMenuPreparing?: (
      e: DevExpress.ui.dxPivotGrid.ContextMenuPreparingEvent
    ) => void;
    /**
     * [descr:dxPivotGridOptions.onExported]
     * @deprecated [depNote:dxPivotGridOptions.onExported]
     */
    onExported?: (e: DevExpress.ui.dxPivotGrid.ExportedEvent) => void;
    /**
     * [descr:dxPivotGridOptions.onExporting]
     */
    onExporting?: (e: DevExpress.ui.dxPivotGrid.ExportingEvent) => void;
    /**
     * [descr:dxPivotGridOptions.onFileSaving]
     * @deprecated [depNote:dxPivotGridOptions.onFileSaving]
     */
    onFileSaving?: (e: DevExpress.ui.dxPivotGrid.FileSavingEvent) => void;
    /**
     * [descr:dxPivotGridOptions.rowHeaderLayout]
     */
    rowHeaderLayout?: 'standard' | 'tree';
    /**
     * [descr:dxPivotGridOptions.scrolling]
     */
    scrolling?: {
      /**
       * [descr:dxPivotGridOptions.scrolling.mode]
       */
      mode?: 'standard' | 'virtual';
      /**
       * [descr:dxPivotGridOptions.scrolling.useNative]
       */
      useNative?: boolean | 'auto';
    };
    /**
     * [descr:dxPivotGridOptions.showBorders]
     */
    showBorders?: boolean;
    /**
     * [descr:dxPivotGridOptions.showColumnGrandTotals]
     */
    showColumnGrandTotals?: boolean;
    /**
     * [descr:dxPivotGridOptions.showColumnTotals]
     */
    showColumnTotals?: boolean;
    /**
     * [descr:dxPivotGridOptions.showRowGrandTotals]
     */
    showRowGrandTotals?: boolean;
    /**
     * [descr:dxPivotGridOptions.showRowTotals]
     */
    showRowTotals?: boolean;
    /**
     * [descr:dxPivotGridOptions.showTotalsPrior]
     */
    showTotalsPrior?: 'both' | 'columns' | 'none' | 'rows';
    /**
     * [descr:dxPivotGridOptions.stateStoring]
     */
    stateStoring?: {
      /**
       * [descr:dxPivotGridOptions.stateStoring.customLoad]
       */
      customLoad?: () => PromiseLike<any>;
      /**
       * [descr:dxPivotGridOptions.stateStoring.customSave]
       */
      customSave?: (state: any) => any;
      /**
       * [descr:dxPivotGridOptions.stateStoring.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxPivotGridOptions.stateStoring.savingTimeout]
       */
      savingTimeout?: number;
      /**
       * [descr:dxPivotGridOptions.stateStoring.storageKey]
       */
      storageKey?: string;
      /**
       * [descr:dxPivotGridOptions.stateStoring.type]
       */
      type?: 'custom' | 'localStorage' | 'sessionStorage';
    };
    /**
     * [descr:dxPivotGridOptions.texts]
     */
    texts?: {
      /**
       * [descr:dxPivotGridOptions.texts.collapseAll]
       */
      collapseAll?: string;
      /**
       * [descr:dxPivotGridOptions.texts.dataNotAvailable]
       */
      dataNotAvailable?: string;
      /**
       * [descr:dxPivotGridOptions.texts.expandAll]
       */
      expandAll?: string;
      /**
       * [descr:dxPivotGridOptions.texts.exportToExcel]
       */
      exportToExcel?: string;
      /**
       * [descr:dxPivotGridOptions.texts.grandTotal]
       */
      grandTotal?: string;
      /**
       * [descr:dxPivotGridOptions.texts.noData]
       */
      noData?: string;
      /**
       * [descr:dxPivotGridOptions.texts.removeAllSorting]
       */
      removeAllSorting?: string;
      /**
       * [descr:dxPivotGridOptions.texts.showFieldChooser]
       */
      showFieldChooser?: string;
      /**
       * [descr:dxPivotGridOptions.texts.sortColumnBySummary]
       */
      sortColumnBySummary?: string;
      /**
       * [descr:dxPivotGridOptions.texts.sortRowBySummary]
       */
      sortRowBySummary?: string;
      /**
       * [descr:dxPivotGridOptions.texts.total]
       */
      total?: string;
    };
    /**
     * [descr:dxPivotGridOptions.wordWrapEnabled]
     */
    wordWrapEnabled?: boolean;
  }
  /**
   * @deprecated Use Cell instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPivotGridPivotGridCell {
    /**
     * [descr:dxPivotGridPivotGridCell.columnPath]
     */
    columnPath?: Array<string | number | Date>;
    /**
     * [descr:dxPivotGridPivotGridCell.columnType]
     */
    columnType?: 'D' | 'T' | 'GT';
    /**
     * [descr:dxPivotGridPivotGridCell.dataIndex]
     */
    dataIndex?: number;
    /**
     * [descr:dxPivotGridPivotGridCell.expanded]
     */
    expanded?: boolean;
    /**
     * [descr:dxPivotGridPivotGridCell.path]
     */
    path?: Array<string | number | Date>;
    /**
     * [descr:dxPivotGridPivotGridCell.rowPath]
     */
    rowPath?: Array<string | number | Date>;
    /**
     * [descr:dxPivotGridPivotGridCell.rowType]
     */
    rowType?: 'D' | 'T' | 'GT';
    /**
     * [descr:dxPivotGridPivotGridCell.text]
     */
    text?: string;
    /**
     * [descr:dxPivotGridPivotGridCell.type]
     */
    type?: 'D' | 'T' | 'GT';
    /**
     * [descr:dxPivotGridPivotGridCell.value]
     */
    value?: any;
  }
  /**
   * [descr:dxPivotGridSummaryCell]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  interface dxPivotGridSummaryCell {
    /**
     * [descr:dxPivotGridSummaryCell.child(direction, fieldValue)]
     */
    child(
      direction: string,
      fieldValue: number | string
    ): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.children(direction)]
     */
    children(direction: string): Array<dxPivotGridSummaryCell>;
    /**
     * [descr:dxPivotGridSummaryCell.field(area)]
     */
    field(area: string): DevExpress.data.PivotGridDataSource.Field;
    /**
     * [descr:dxPivotGridSummaryCell.grandTotal()]
     */
    grandTotal(): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.grandTotal(direction)]
     */
    grandTotal(direction: string): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.isPostProcessed(field)]
     */
    isPostProcessed(
      field: DevExpress.data.PivotGridDataSource.Field | string
    ): boolean;
    /**
     * [descr:dxPivotGridSummaryCell.next(direction)]
     */
    next(direction: string): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.next(direction, allowCrossGroup)]
     */
    next(direction: string, allowCrossGroup: boolean): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.parent(direction)]
     */
    parent(direction: string): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.prev(direction)]
     */
    prev(direction: string): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.prev(direction, allowCrossGroup)]
     */
    prev(direction: string, allowCrossGroup: boolean): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.slice(field, value)]
     */
    slice(
      field: DevExpress.data.PivotGridDataSource.Field,
      value: number | string
    ): dxPivotGridSummaryCell;
    /**
     * [descr:dxPivotGridSummaryCell.value()]
     */
    value(): any;
    /**
     * [descr:dxPivotGridSummaryCell.value(field)]
     */
    value(field: DevExpress.data.PivotGridDataSource.Field | string): any;
    /**
     * [descr:dxPivotGridSummaryCell.value(field, postProcessed)]
     */
    value(
      field: DevExpress.data.PivotGridDataSource.Field | string,
      postProcessed: boolean
    ): any;
    /**
     * [descr:dxPivotGridSummaryCell.value(postProcessed)]
     */
    value(postProcessed: boolean): any;
  }
  /**
   * [descr:dxPopover]
   */
  export class dxPopover<
    TProperties = DevExpress.ui.dxPopover.Properties
  > extends dxPopup<TProperties> {
    show(): DevExpress.core.utils.DxPromise<boolean>;
    /**
     * [descr:dxPopover.show(target)]
     */
    show(
      target: string | DevExpress.core.UserDefinedElement
    ): DevExpress.core.utils.DxPromise<boolean>;
  }
  module dxPopover {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxPopover>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxPopover>;
    export type HiddenEvent = DevExpress.events.EventInfo<dxPopover>;
    export type HidingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxPopover>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxPopover>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxPopover> &
      DevExpress.events.ChangedOptionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface PopoverInstance extends dxPopover<Properties> {}
    export type Properties = dxPopoverOptions<PopoverInstance>;
    export type ShowingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxPopover>;
    export type ShownEvent = DevExpress.events.EventInfo<dxPopover>;
    export type TitleRenderedEvent = DevExpress.events.EventInfo<dxPopup> &
      DevExpress.ui.dxPopup.TitleRenderedInfo;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPopoverAnimation extends dxPopupAnimation {
    /**
     * [descr:dxPopoverOptions.animation.hide]
     */
    hide?: AnimationConfig;
    /**
     * [descr:dxPopoverOptions.animation.show]
     */
    show?: AnimationConfig;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPopoverOptions<TComponent>
    extends dxPopupOptions<TComponent> {
    /**
     * [descr:dxPopoverOptions.animation]
     */
    animation?: dxPopoverAnimation;
    /**
     * [descr:dxPopoverOptions.closeOnOutsideClick]
     */
    closeOnOutsideClick?:
      | boolean
      | ((
          event: DevExpress.events.DxEvent<
            MouseEvent | PointerEvent | TouchEvent
          >
        ) => boolean);
    /**
     * [descr:dxPopoverOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxPopoverOptions.hideEvent]
     */
    hideEvent?:
      | {
          /**
           * [descr:dxPopoverOptions.hideEvent.delay]
           */
          delay?: number;
          /**
           * [descr:dxPopoverOptions.hideEvent.name]
           */
          name?: string;
        }
      | string;
    /**
     * [descr:dxPopoverOptions.position]
     */
    position?: 'bottom' | 'left' | 'right' | 'top' | PositionConfig;
    /**
     * [descr:dxPopoverOptions.shading]
     */
    shading?: boolean;
    /**
     * [descr:dxPopoverOptions.showEvent]
     */
    showEvent?:
      | {
          /**
           * [descr:dxPopoverOptions.showEvent.delay]
           */
          delay?: number;
          /**
           * [descr:dxPopoverOptions.showEvent.name]
           */
          name?: string;
        }
      | string;
    /**
     * [descr:dxPopoverOptions.showTitle]
     */
    showTitle?: boolean;
    /**
     * [descr:dxPopoverOptions.target]
     */
    target?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxPopoverOptions.width]
     */
    width?: number | string | (() => number | string);
    /**
     * [descr:dxPopoverOptions.hideOnParentScroll]
     */
    hideOnParentScroll?: boolean;
  }
  /**
   * [descr:dxPopup]
   */
  export class dxPopup<
    TProperties = DevExpress.ui.dxPopup.Properties
  > extends dxOverlay<TProperties> {}
  module dxPopup {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxPopup>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxPopup>;
    export type HiddenEvent = DevExpress.events.EventInfo<dxPopup>;
    export type HidingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxPopup>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxPopup>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxPopup> &
      DevExpress.events.ChangedOptionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface PopupInstance extends dxPopup<Properties> {}
    export type Properties = dxPopupOptions<PopupInstance>;
    export type ResizeEndEvent = DevExpress.events.NativeEventInfo<
      dxPopup,
      MouseEvent | TouchEvent
    > &
      DevExpress.ui.dxResizable.ResizeInfo;
    export type ResizeEvent = DevExpress.events.NativeEventInfo<
      dxPopup,
      MouseEvent | TouchEvent
    > &
      DevExpress.ui.dxResizable.ResizeInfo;
    export type ResizeStartEvent = DevExpress.events.NativeEventInfo<
      dxPopup,
      MouseEvent | TouchEvent
    > &
      DevExpress.ui.dxResizable.ResizeInfo;
    export type ShowingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxPopup>;
    export type ShownEvent = DevExpress.events.EventInfo<dxPopup>;
    export type TitleRenderedEvent = DevExpress.events.EventInfo<dxPopup> &
      TitleRenderedInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface TitleRenderedInfo {
      readonly titleElement: DevExpress.core.DxElement;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPopupAnimation extends dxOverlayAnimation {
    /**
     * [descr:dxPopupOptions.animation.hide]
     */
    hide?: AnimationConfig;
    /**
     * [descr:dxPopupOptions.animation.show]
     */
    show?: AnimationConfig;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPopupOptions<TComponent>
    extends dxOverlayOptions<TComponent> {
    /**
     * [descr:dxPopupOptions.animation]
     */
    animation?: dxPopupAnimation;
    /**
     * [descr:dxPopupOptions.container]
     */
    container?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:dxPopupOptions.dragEnabled]
     */
    dragEnabled?: boolean;
    /**
     * [descr:dxPopupOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxPopupOptions.fullScreen]
     */
    fullScreen?: boolean;
    /**
     * [descr:dxPopupOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxPopupOptions.onResize]
     */
    onResize?: (e: DevExpress.ui.dxPopup.ResizeEvent) => void;
    /**
     * [descr:dxPopupOptions.onResizeEnd]
     */
    onResizeEnd?: (e: DevExpress.ui.dxPopup.ResizeEndEvent) => void;
    /**
     * [descr:dxPopupOptions.onResizeStart]
     */
    onResizeStart?: (e: DevExpress.ui.dxPopup.ResizeStartEvent) => void;
    /**
     * [descr:dxPopupOptions.onTitleRendered]
     */
    onTitleRendered?: (e: DevExpress.ui.dxPopup.TitleRenderedEvent) => void;
    /**
     * [descr:dxPopupOptions.position]
     */
    position?:
      | 'bottom'
      | 'center'
      | 'left'
      | 'left bottom'
      | 'left top'
      | 'right'
      | 'right bottom'
      | 'right top'
      | 'top'
      | PositionConfig
      | Function;
    /**
     * [descr:dxPopupOptions.resizeEnabled]
     */
    resizeEnabled?: boolean;
    /**
     * [descr:dxPopupOptions.restorePosition]
     */
    restorePosition?: boolean;
    /**
     * [descr:dxPopupOptions.showCloseButton]
     */
    showCloseButton?: boolean;
    /**
     * [descr:dxPopupOptions.showTitle]
     */
    showTitle?: boolean;
    /**
     * [descr:dxPopupOptions.title]
     */
    title?: string;
    /**
     * [descr:dxPopupOptions.titleTemplate]
     */
    titleTemplate?:
      | DevExpress.core.template
      | ((
          titleElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxPopupOptions.toolbarItems]
     */
    toolbarItems?: Array<DevExpress.ui.dxPopup.ToolbarItem>;
    /**
     * [descr:dxPopupOptions.width]
     */
    width?: number | string | (() => number | string);
  }
  /**
   * @deprecated Use ToolbarItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPopupToolbarItem {
    /**
     * [descr:dxPopupOptions.toolbarItems.disabled]
     */
    disabled?: boolean;
    /**
     * [descr:dxPopupOptions.toolbarItems.html]
     */
    html?: string;
    /**
     * [descr:dxPopupOptions.toolbarItems.location]
     */
    location?: 'after' | 'before' | 'center';
    /**
     * [descr:dxPopupOptions.toolbarItems.options]
     */
    options?: any;
    /**
     * [descr:dxPopupOptions.toolbarItems.template]
     */
    template?: DevExpress.core.template;
    /**
     * [descr:dxPopupOptions.toolbarItems.text]
     */
    text?: string;
    /**
     * [descr:dxPopupOptions.toolbarItems.toolbar]
     */
    toolbar?: 'bottom' | 'top';
    /**
     * [descr:dxPopupOptions.toolbarItems.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxPopupOptions.toolbarItems.widget]
     */
    widget?:
      | 'dxAutocomplete'
      | 'dxButton'
      | 'dxCheckBox'
      | 'dxDateBox'
      | 'dxMenu'
      | 'dxSelectBox'
      | 'dxTabs'
      | 'dxTextBox'
      | 'dxButtonGroup'
      | 'dxDropDownButton';
  }
  /**
   * [descr:dxProgressBar]
   */
  export class dxProgressBar extends dxTrackBar<dxProgressBarOptions> {}
  module dxProgressBar {
    export type CompleteEvent =
      DevExpress.events.NativeEventInfo<dxProgressBar>;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxProgressBar>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxProgressBar>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxProgressBar>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxProgressBar> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxProgressBarOptions;
    export type ValueChangedEvent =
      DevExpress.events.NativeEventInfo<dxProgressBar> &
        DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxProgressBarOptions
    extends dxTrackBarOptions<dxProgressBar> {
    /**
     * [descr:dxProgressBarOptions.onComplete]
     */
    onComplete?: (e: DevExpress.ui.dxProgressBar.CompleteEvent) => void;
    /**
     * [descr:dxProgressBarOptions.showStatus]
     */
    showStatus?: boolean;
    /**
     * [descr:dxProgressBarOptions.statusFormat]
     */
    statusFormat?: string | ((ratio: number, value: number) => string);
    /**
     * [descr:dxProgressBarOptions.value]
     */
    value?: number | boolean;
  }
  /**
   * [descr:dxRadioGroup]
   */
  export class dxRadioGroup extends Editor<dxRadioGroupOptions> {
    getDataSource(): DevExpress.data.DataSource;
  }
  module dxRadioGroup {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxRadioGroup>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxRadioGroup>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxRadioGroup>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxRadioGroup> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxRadioGroupOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxRadioGroup,
      MouseEvent | PointerEvent | KeyboardEvent
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxRadioGroupOptions
    extends EditorOptions<dxRadioGroup>,
      DataExpressionMixinOptions<dxRadioGroup> {
    /**
     * [descr:dxRadioGroupOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxRadioGroupOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxRadioGroupOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxRadioGroupOptions.layout]
     */
    layout?: 'horizontal' | 'vertical';
    /**
     * [descr:dxRadioGroupOptions.name]
     */
    name?: string;
    /**
     * [descr:dxRadioGroupOptions.value]
     */
    value?: any;
  }
  /**
   * [descr:dxRangeSlider]
   */
  export class dxRangeSlider extends dxTrackBar<dxRangeSliderOptions> {}
  module dxRangeSlider {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxRangeSlider>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxRangeSlider>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxRangeSlider>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxRangeSlider> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxRangeSliderOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxRangeSlider,
      KeyboardEvent | MouseEvent | PointerEvent | TouchEvent | UIEvent
    > &
      DevExpress.ui.Editor.ValueChangedInfo & {
        readonly start?: number;
        readonly end?: number;
        readonly value?: Array<number>;
      };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxRangeSliderOptions
    extends dxSliderBaseOptions<dxRangeSlider> {
    /**
     * [descr:dxRangeSliderOptions.end]
     */
    end?: number;
    /**
     * [descr:dxRangeSliderOptions.endName]
     */
    endName?: string;
    /**
     * [descr:dxRangeSliderOptions.onValueChanged]
     */
    onValueChanged?: (e: DevExpress.ui.dxRangeSlider.ValueChangedEvent) => void;
    /**
     * [descr:dxRangeSliderOptions.start]
     */
    start?: number;
    /**
     * [descr:dxRangeSliderOptions.startName]
     */
    startName?: string;
    /**
     * [descr:dxRangeSliderOptions.value]
     */
    value?: Array<number>;
  }
  /**
   * [descr:dxRecurrenceEditor]
   */
  export class dxRecurrenceEditor extends Editor<dxRecurrenceEditorOptions> {}
  module dxRecurrenceEditor {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type Properties = dxRecurrenceEditorOptions;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxRecurrenceEditorOptions
    extends EditorOptions<dxRecurrenceEditor> {
    /**
     * [descr:dxRecurrenceEditorOptions.value]
     */
    value?: string;
  }
  /**
   * [descr:dxResizable]
   */
  export class dxResizable extends DOMComponent<dxResizableOptions> {}
  module dxResizable {
    export type DisposingEvent = DevExpress.events.EventInfo<dxResizable>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxResizable>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxResizable> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxResizableOptions;
    export type ResizeEndEvent = DevExpress.events.NativeEventInfo<
      dxResizable,
      MouseEvent | TouchEvent
    > &
      ResizeInfo;
    export type ResizeEvent = DevExpress.events.NativeEventInfo<
      dxResizable,
      MouseEvent | TouchEvent
    > &
      ResizeInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ResizeInfo {
      readonly width: number;
      readonly height: number;
      handles: {
        readonly left: boolean;
        readonly top: boolean;
        readonly right: boolean;
        readonly bottom: boolean;
      };
    }
    export type ResizeStartEvent = DevExpress.events.NativeEventInfo<
      dxResizable,
      MouseEvent | TouchEvent
    > &
      ResizeInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxResizableOptions extends DOMComponentOptions<dxResizable> {
    /**
     * [descr:dxResizableOptions.handles]
     */
    handles?: 'bottom' | 'left' | 'right' | 'top' | 'all' | string;
    /**
     * [descr:dxResizableOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxResizableOptions.maxHeight]
     */
    maxHeight?: number;
    /**
     * [descr:dxResizableOptions.maxWidth]
     */
    maxWidth?: number;
    /**
     * [descr:dxResizableOptions.minHeight]
     */
    minHeight?: number;
    /**
     * [descr:dxResizableOptions.minWidth]
     */
    minWidth?: number;
    /**
     * [descr:dxResizableOptions.onResize]
     */
    onResize?: (e: DevExpress.ui.dxResizable.ResizeEvent) => void;
    /**
     * [descr:dxResizableOptions.onResizeEnd]
     */
    onResizeEnd?: (e: DevExpress.ui.dxResizable.ResizeEndEvent) => void;
    /**
     * [descr:dxResizableOptions.onResizeStart]
     */
    onResizeStart?: (e: DevExpress.ui.dxResizable.ResizeStartEvent) => void;
    /**
     * [descr:dxResizableOptions.width]
     */
    width?: number | string | (() => number | string);
  }
  /**
   * [descr:dxResponsiveBox]
   */
  export class dxResponsiveBox<
    TItem extends DevExpress.ui.dxResponsiveBox.ItemLike = any,
    TKey = any
  > extends CollectionWidget<
    dxResponsiveBoxOptions<TItem, TKey>,
    TItem,
    TKey
  > {}
  module dxResponsiveBox {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxResponsiveBox<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxResponsiveBox<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxResponsiveBox<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxResponsiveBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxResponsiveBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxResponsiveBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxResponsiveBox<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxResponsiveBox<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxResponsiveBoxOptions<TItem, TKey>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxResponsiveBoxItem extends CollectionWidgetItem {
    /**
     * [descr:dxResponsiveBoxItem.location]
     */
    location?:
      | {
          /**
           * [descr:dxResponsiveBoxItem.location.col]
           */
          col?: number;
          /**
           * [descr:dxResponsiveBoxItem.location.colspan]
           */
          colspan?: number;
          /**
           * [descr:dxResponsiveBoxItem.location.row]
           */
          row?: number;
          /**
           * [descr:dxResponsiveBoxItem.location.rowspan]
           */
          rowspan?: number;
          /**
           * [descr:dxResponsiveBoxItem.location.screen]
           */
          screen?: string;
        }
      | Array<{
          col?: number;
          colspan?: number;
          row?: number;
          rowspan?: number;
          screen?: string;
        }>;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxResponsiveBoxOptions<
    TItem extends DevExpress.ui.dxResponsiveBox.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<dxResponsiveBox<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxResponsiveBoxOptions.cols]
     */
    cols?: Array<{
      /**
       * [descr:dxResponsiveBoxOptions.cols.baseSize]
       */
      baseSize?: number | 'auto';
      /**
       * [descr:dxResponsiveBoxOptions.cols.ratio]
       */
      ratio?: number;
      /**
       * [descr:dxResponsiveBoxOptions.cols.screen]
       */
      screen?: string;
      /**
       * [descr:dxResponsiveBoxOptions.cols.shrink]
       */
      shrink?: number;
    }>;
    /**
     * [descr:dxResponsiveBoxOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxResponsiveBoxOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxResponsiveBoxOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxResponsiveBoxOptions.rows]
     */
    rows?: Array<{
      /**
       * [descr:dxResponsiveBoxOptions.rows.baseSize]
       */
      baseSize?: number | 'auto';
      /**
       * [descr:dxResponsiveBoxOptions.rows.ratio]
       */
      ratio?: number;
      /**
       * [descr:dxResponsiveBoxOptions.rows.screen]
       */
      screen?: string;
      /**
       * [descr:dxResponsiveBoxOptions.rows.shrink]
       */
      shrink?: number;
    }>;
    /**
     * [descr:dxResponsiveBoxOptions.screenByWidth]
     */
    screenByWidth?: Function;
    /**
     * [descr:dxResponsiveBoxOptions.singleColumnScreen]
     */
    singleColumnScreen?: string;
    /**
     * [descr:dxResponsiveBoxOptions.width]
     */
    width?: number | string | (() => number | string);
  }
  /**
   * [descr:dxScheduler]
   */
  export class dxScheduler extends Widget<dxSchedulerOptions> {
    /**
     * [descr:dxScheduler.addAppointment(appointment)]
     */
    addAppointment(appointment: dxSchedulerAppointment): void;
    /**
     * [descr:dxScheduler.deleteAppointment(appointment)]
     */
    deleteAppointment(appointment: dxSchedulerAppointment): void;
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxScheduler.getEndViewDate()]
     */
    getEndViewDate(): Date;
    /**
     * [descr:dxScheduler.getStartViewDate()]
     */
    getStartViewDate(): Date;
    /**
     * [descr:dxScheduler.hideAppointmentPopup(saveChanges)]
     */
    hideAppointmentPopup(saveChanges?: boolean): void;
    /**
     * [descr:dxScheduler.hideAppointmentTooltip()]
     */
    hideAppointmentTooltip(): void;
    /**
     * [descr:dxScheduler.scrollTo(date, group, allDay)]
     */
    scrollTo(date: Date, group?: object, allDay?: boolean): void;
    /**
     * [descr:dxScheduler.scrollToTime(hours, minutes, date)]
     * @deprecated [depNote:dxScheduler.scrollToTime(hours, minutes, date)]
     */
    scrollToTime(hours: number, minutes: number, date?: Date): void;
    /**
     * [descr:dxScheduler.showAppointmentPopup(appointmentData, createNewAppointment, currentAppointmentData)]
     */
    showAppointmentPopup(
      appointmentData?: dxSchedulerAppointment,
      createNewAppointment?: boolean,
      currentAppointmentData?: dxSchedulerAppointment
    ): void;
    /**
     * [descr:dxScheduler.showAppointmentTooltip(appointmentData, target, currentAppointmentData)]
     */
    showAppointmentTooltip(
      appointmentData: dxSchedulerAppointment,
      target: string | DevExpress.core.UserDefinedElement,
      currentAppointmentData?: dxSchedulerAppointment
    ): void;
    /**
     * [descr:dxScheduler.updateAppointment(target, appointment)]
     */
    updateAppointment(
      target: dxSchedulerAppointment,
      appointment: dxSchedulerAppointment
    ): void;
  }
  module dxScheduler {
    export type Appointment = dxSchedulerAppointment;
    export type AppointmentAddedEvent =
      DevExpress.events.EventInfo<dxScheduler> & {
        readonly appointmentData: dxSchedulerAppointment;
        readonly error?: Error;
      };
    export type AppointmentAddingEvent =
      DevExpress.events.EventInfo<dxScheduler> & {
        readonly appointmentData: dxSchedulerAppointment;
        cancel: boolean | PromiseLike<boolean>;
      };
    export type AppointmentClickEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<
        dxScheduler,
        KeyboardEvent | MouseEvent | PointerEvent
      > &
      TargetedAppointmentInfo & {
        readonly appointmentElement: DevExpress.core.DxElement;
      };
    export type AppointmentCollectorTemplateData = {
      readonly appointmentCount: number;
      readonly isCompact: boolean;
    };
    export type AppointmentContextMenuEvent = DevExpress.events.NativeEventInfo<
      dxScheduler,
      MouseEvent | PointerEvent | TouchEvent
    > &
      TargetedAppointmentInfo & {
        readonly appointmentElement: DevExpress.core.DxElement;
      };
    export type AppointmentDblClickEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<
        dxScheduler,
        MouseEvent | PointerEvent
      > &
      TargetedAppointmentInfo & {
        readonly appointmentElement: DevExpress.core.DxElement;
      };
    export type AppointmentDeletedEvent =
      DevExpress.events.EventInfo<dxScheduler> & {
        readonly appointmentData: dxSchedulerAppointment;
        readonly error?: Error;
      };
    export type AppointmentDeletingEvent =
      DevExpress.events.EventInfo<dxScheduler> & {
        readonly appointmentData: dxSchedulerAppointment;
        cancel: boolean | PromiseLike<boolean>;
      };
    export type AppointmentDraggingAddEvent = AppointmentDraggingEvent & {
      readonly fromComponent?: dxSortable | dxDraggable;
      readonly toComponent?: dxSortable | dxDraggable;
      readonly toData?: any;
    };
    export type AppointmentDraggingEndEvent = DevExpress.events.Cancelable &
      AppointmentDraggingEvent & {
        readonly fromComponent?: dxSortable | dxDraggable;
        readonly toComponent?: dxSortable | dxDraggable;
        readonly toData?: any;
      };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface AppointmentDraggingEvent {
      readonly component: dxScheduler;
      readonly event?: DevExpress.events.DxEvent<MouseEvent | TouchEvent>;
      readonly itemData?: any;
      readonly itemElement?: DevExpress.core.DxElement;
      readonly fromData?: any;
    }
    export type AppointmentDraggingMoveEvent = DevExpress.events.Cancelable &
      AppointmentDraggingEvent & {
        readonly fromComponent?: dxSortable | dxDraggable;
        readonly toComponent?: dxSortable | dxDraggable;
        readonly toData?: any;
      };
    export type AppointmentDraggingRemoveEvent = AppointmentDraggingEvent & {
      readonly fromComponent?: dxSortable | dxDraggable;
      readonly toComponent?: dxSortable | dxDraggable;
    };
    export type AppointmentDraggingStartEvent = DevExpress.events.Cancelable &
      AppointmentDraggingEvent;
    export type AppointmentFormOpeningEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxScheduler> & {
        readonly appointmentData?: dxSchedulerAppointment;
        readonly form: dxForm;
        readonly popup: dxPopup;
      };
    export type AppointmentRenderedEvent =
      DevExpress.events.EventInfo<dxScheduler> &
        TargetedAppointmentInfo & {
          readonly appointmentElement: DevExpress.core.DxElement;
        };
    export type AppointmentTemplateData = TargetedAppointmentInfo;
    export type AppointmentTooltipTemplateData = TargetedAppointmentInfo;
    export type AppointmentUpdatedEvent =
      DevExpress.events.EventInfo<dxScheduler> & {
        readonly appointmentData: dxSchedulerAppointment;
        readonly error?: Error;
      };
    export type AppointmentUpdatingEvent =
      DevExpress.events.EventInfo<dxScheduler> & {
        readonly oldData: any;
        readonly newData: any;
        cancel?: boolean | PromiseLike<boolean>;
      };
    export type CellClickEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<
        dxScheduler,
        KeyboardEvent | MouseEvent | PointerEvent
      > & {
        readonly cellData: any;
        readonly cellElement: DevExpress.core.DxElement;
      };
    export type CellContextMenuEvent = DevExpress.events.NativeEventInfo<
      dxScheduler,
      MouseEvent | PointerEvent | TouchEvent
    > & {
      readonly cellData: any;
      readonly cellElement: DevExpress.core.DxElement;
    };
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxScheduler>;
    export type DateNavigatorTextInfo = {
      readonly startDate: Date;
      readonly endDate: Date;
      readonly text: string;
    };
    export type DisposingEvent = DevExpress.events.EventInfo<dxScheduler>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxScheduler>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxScheduler> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSchedulerOptions;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface TargetedAppointmentInfo {
      readonly appointmentData: dxSchedulerAppointment;
      readonly targetedAppointmentData?: dxSchedulerAppointment;
    }
  }
  /**
   * @deprecated Use the Scheduler's Appointment type instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSchedulerAppointment extends CollectionWidgetItem {
    /**
     * [descr:dxSchedulerAppointment.allDay]
     */
    allDay?: boolean;
    /**
     * [descr:dxSchedulerAppointment.description]
     */
    description?: string;
    /**
     * [descr:dxSchedulerAppointment.disabled]
     */
    disabled?: boolean;
    /**
     * [descr:dxSchedulerAppointment.endDate]
     */
    endDate?: Date | string;
    /**
     * [descr:dxSchedulerAppointment.endDateTimeZone]
     */
    endDateTimeZone?: string;
    /**
     * [descr:dxSchedulerAppointment.html]
     */
    html?: string;
    /**
     * [descr:dxSchedulerAppointment.recurrenceException]
     */
    recurrenceException?: string;
    /**
     * [descr:dxSchedulerAppointment.recurrenceRule]
     */
    recurrenceRule?: string;
    /**
     * [descr:dxSchedulerAppointment.startDate]
     */
    startDate?: Date | string;
    /**
     * [descr:dxSchedulerAppointment.startDateTimeZone]
     */
    startDateTimeZone?: string;
    /**
     * [descr:dxSchedulerAppointment.template]
     */
    template?: DevExpress.core.template;
    /**
     * [descr:dxSchedulerAppointment.text]
     */
    text?: string;
    /**
     * [descr:dxSchedulerAppointment.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSchedulerOptions extends WidgetOptions<dxScheduler> {
    /**
     * [descr:dxSchedulerOptions.adaptivityEnabled]
     */
    adaptivityEnabled?: boolean;
    /**
     * [descr:dxSchedulerOptions.allDayExpr]
     */
    allDayExpr?: string;
    /**
     * [descr:dxSchedulerOptions.appointmentCollectorTemplate]
     */
    appointmentCollectorTemplate?:
      | DevExpress.core.template
      | ((
          data: DevExpress.ui.dxScheduler.AppointmentCollectorTemplateData,
          collectorElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.appointmentDragging]
     */
    appointmentDragging?: {
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.autoScroll]
       */
      autoScroll?: boolean;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.data]
       */
      data?: any;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.group]
       */
      group?: string;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.onAdd]
       */
      onAdd?: (
        e: DevExpress.ui.dxScheduler.AppointmentDraggingAddEvent
      ) => void;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.onDragEnd]
       */
      onDragEnd?: (
        e: DevExpress.ui.dxScheduler.AppointmentDraggingEndEvent
      ) => void;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.onDragMove]
       */
      onDragMove?: (
        e: DevExpress.ui.dxScheduler.AppointmentDraggingMoveEvent
      ) => void;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.onDragStart]
       */
      onDragStart?: (
        e: DevExpress.ui.dxScheduler.AppointmentDraggingStartEvent
      ) => void;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.onRemove]
       */
      onRemove?: (
        e: DevExpress.ui.dxScheduler.AppointmentDraggingRemoveEvent
      ) => void;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.scrollSensitivity]
       */
      scrollSensitivity?: number;
      /**
       * [descr:dxSchedulerOptions.appointmentDragging.scrollSpeed]
       */
      scrollSpeed?: number;
    };
    /**
     * [descr:dxSchedulerOptions.appointmentTemplate]
     */
    appointmentTemplate?:
      | DevExpress.core.template
      | ((
          model: DevExpress.ui.dxScheduler.AppointmentTemplateData,
          itemIndex: number,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.appointmentTooltipTemplate]
     */
    appointmentTooltipTemplate?:
      | DevExpress.core.template
      | ((
          model: DevExpress.ui.dxScheduler.AppointmentTemplateData,
          itemIndex: number,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.cellDuration]
     */
    cellDuration?: number;
    /**
     * [descr:dxSchedulerOptions.crossScrollingEnabled]
     */
    crossScrollingEnabled?: boolean;
    /**
     * [descr:dxSchedulerOptions.currentDate]
     */
    currentDate?: Date | number | string;
    /**
     * [descr:dxSchedulerOptions.currentView]
     */
    currentView?:
      | 'agenda'
      | 'day'
      | 'month'
      | 'timelineDay'
      | 'timelineMonth'
      | 'timelineWeek'
      | 'timelineWorkWeek'
      | 'week'
      | 'workWeek';
    /**
     * [descr:dxSchedulerOptions.customizeDateNavigatorText]
     */
    customizeDateNavigatorText?: (
      info: DevExpress.ui.dxScheduler.DateNavigatorTextInfo
    ) => string;
    /**
     * [descr:dxSchedulerOptions.dataCellTemplate]
     */
    dataCellTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<DevExpress.ui.dxScheduler.Appointment>;
    /**
     * [descr:dxSchedulerOptions.dateCellTemplate]
     */
    dateCellTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.dateSerializationFormat]
     */
    dateSerializationFormat?: string;
    /**
     * [descr:dxSchedulerOptions.descriptionExpr]
     */
    descriptionExpr?: string;
    /**
     * [descr:dxSchedulerOptions.dropDownAppointmentTemplate]
     * @deprecated [depNote:dxSchedulerOptions.dropDownAppointmentTemplate]
     */
    dropDownAppointmentTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          contentElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.editing]
     */
    editing?:
      | boolean
      | {
          /**
           * [descr:dxSchedulerOptions.editing.allowAdding]
           */
          allowAdding?: boolean;
          /**
           * [descr:dxSchedulerOptions.editing.allowDeleting]
           */
          allowDeleting?: boolean;
          /**
           * [descr:dxSchedulerOptions.editing.allowDragging]
           */
          allowDragging?: boolean;
          /**
           * [descr:dxSchedulerOptions.editing.allowResizing]
           */
          allowResizing?: boolean;
          /**
           * [descr:dxSchedulerOptions.editing.allowTimeZoneEditing]
           */
          allowTimeZoneEditing?: boolean;
          /**
           * [descr:dxSchedulerOptions.editing.allowUpdating]
           */
          allowUpdating?: boolean;
        };
    /**
     * [descr:dxSchedulerOptions.endDateExpr]
     */
    endDateExpr?: string;
    /**
     * [descr:dxSchedulerOptions.endDateTimeZoneExpr]
     */
    endDateTimeZoneExpr?: string;
    /**
     * [descr:dxSchedulerOptions.endDayHour]
     */
    endDayHour?: number;
    /**
     * [descr:dxSchedulerOptions.firstDayOfWeek]
     */
    firstDayOfWeek?: 0 | 1 | 2 | 3 | 4 | 5 | 6;
    /**
     * [descr:dxSchedulerOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxSchedulerOptions.groupByDate]
     */
    groupByDate?: boolean;
    /**
     * [descr:dxSchedulerOptions.groups]
     */
    groups?: Array<string>;
    /**
     * [descr:dxSchedulerOptions.indicatorUpdateInterval]
     */
    indicatorUpdateInterval?: number;
    /**
     * [descr:dxSchedulerOptions.max]
     */
    max?: Date | number | string;
    /**
     * [descr:dxSchedulerOptions.maxAppointmentsPerCell]
     */
    maxAppointmentsPerCell?: number | 'auto' | 'unlimited';
    /**
     * [descr:dxSchedulerOptions.min]
     */
    min?: Date | number | string;
    /**
     * [descr:dxSchedulerOptions.noDataText]
     */
    noDataText?: string;
    /**
     * [descr:dxSchedulerOptions.onAppointmentAdded]
     */
    onAppointmentAdded?: (
      e: DevExpress.ui.dxScheduler.AppointmentAddedEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onAppointmentAdding]
     */
    onAppointmentAdding?: (
      e: DevExpress.ui.dxScheduler.AppointmentAddingEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onAppointmentClick]
     */
    onAppointmentClick?:
      | ((e: DevExpress.ui.dxScheduler.AppointmentClickEvent) => void)
      | string;
    /**
     * [descr:dxSchedulerOptions.onAppointmentContextMenu]
     */
    onAppointmentContextMenu?:
      | ((e: DevExpress.ui.dxScheduler.AppointmentContextMenuEvent) => void)
      | string;
    /**
     * [descr:dxSchedulerOptions.onAppointmentDblClick]
     */
    onAppointmentDblClick?:
      | ((e: DevExpress.ui.dxScheduler.AppointmentDblClickEvent) => void)
      | string;
    /**
     * [descr:dxSchedulerOptions.onAppointmentDeleted]
     */
    onAppointmentDeleted?: (
      e: DevExpress.ui.dxScheduler.AppointmentDeletedEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onAppointmentDeleting]
     */
    onAppointmentDeleting?: (
      e: DevExpress.ui.dxScheduler.AppointmentDeletingEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onAppointmentFormOpening]
     */
    onAppointmentFormOpening?: (
      e: DevExpress.ui.dxScheduler.AppointmentFormOpeningEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onAppointmentRendered]
     */
    onAppointmentRendered?: (
      e: DevExpress.ui.dxScheduler.AppointmentRenderedEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onAppointmentUpdated]
     */
    onAppointmentUpdated?: (
      e: DevExpress.ui.dxScheduler.AppointmentUpdatedEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onAppointmentUpdating]
     */
    onAppointmentUpdating?: (
      e: DevExpress.ui.dxScheduler.AppointmentUpdatingEvent
    ) => void;
    /**
     * [descr:dxSchedulerOptions.onCellClick]
     */
    onCellClick?:
      | ((e: DevExpress.ui.dxScheduler.CellClickEvent) => void)
      | string;
    /**
     * [descr:dxSchedulerOptions.onCellContextMenu]
     */
    onCellContextMenu?:
      | ((e: DevExpress.ui.dxScheduler.CellContextMenuEvent) => void)
      | string;
    /**
     * [descr:dxSchedulerOptions.recurrenceEditMode]
     */
    recurrenceEditMode?: 'dialog' | 'occurrence' | 'series';
    /**
     * [descr:dxSchedulerOptions.recurrenceExceptionExpr]
     */
    recurrenceExceptionExpr?: string;
    /**
     * [descr:dxSchedulerOptions.recurrenceRuleExpr]
     */
    recurrenceRuleExpr?: string;
    /**
     * [descr:dxSchedulerOptions.remoteFiltering]
     */
    remoteFiltering?: boolean;
    /**
     * [descr:dxSchedulerOptions.resourceCellTemplate]
     */
    resourceCellTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.resources]
     */
    resources?: Array<{
      /**
       * [descr:dxSchedulerOptions.resources.allowMultiple]
       */
      allowMultiple?: boolean;
      /**
       * [descr:dxSchedulerOptions.resources.colorExpr]
       */
      colorExpr?: string;
      /**
       * [descr:dxSchedulerOptions.resources.dataSource]
       */
      dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
      /**
       * [descr:dxSchedulerOptions.resources.displayExpr]
       */
      displayExpr?: string | ((resource: any) => string);
      /**
       * [descr:dxSchedulerOptions.resources.fieldExpr]
       */
      fieldExpr?: string;
      /**
       * [descr:dxSchedulerOptions.resources.label]
       */
      label?: string;
      /**
       * [descr:dxSchedulerOptions.resources.useColorAsDefault]
       */
      useColorAsDefault?: boolean;
      /**
       * [descr:dxSchedulerOptions.resources.valueExpr]
       */
      valueExpr?: string | Function;
    }>;
    /**
     * [descr:dxSchedulerOptions.scrolling]
     */
    scrolling?: dxSchedulerScrolling;
    /**
     * [descr:dxSchedulerOptions.selectedCellData]
     */
    selectedCellData?: Array<any>;
    /**
     * [descr:dxSchedulerOptions.shadeUntilCurrentTime]
     */
    shadeUntilCurrentTime?: boolean;
    /**
     * [descr:dxSchedulerOptions.showAllDayPanel]
     */
    showAllDayPanel?: boolean;
    /**
     * [descr:dxSchedulerOptions.showCurrentTimeIndicator]
     */
    showCurrentTimeIndicator?: boolean;
    /**
     * [descr:dxSchedulerOptions.startDateExpr]
     */
    startDateExpr?: string;
    /**
     * [descr:dxSchedulerOptions.startDateTimeZoneExpr]
     */
    startDateTimeZoneExpr?: string;
    /**
     * [descr:dxSchedulerOptions.startDayHour]
     */
    startDayHour?: number;
    /**
     * [descr:dxSchedulerOptions.textExpr]
     */
    textExpr?: string;
    /**
     * [descr:dxSchedulerOptions.timeCellTemplate]
     */
    timeCellTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSchedulerOptions.timeZone]
     */
    timeZone?: string;
    /**
     * [descr:dxSchedulerOptions.useDropDownViewSwitcher]
     */
    useDropDownViewSwitcher?: boolean;
    /**
     * [descr:dxSchedulerOptions.views]
     */
    views?: Array<
      | 'day'
      | 'week'
      | 'workWeek'
      | 'month'
      | 'timelineDay'
      | 'timelineWeek'
      | 'timelineWorkWeek'
      | 'timelineMonth'
      | 'agenda'
      | {
          /**
           * [descr:dxSchedulerOptions.views.agendaDuration]
           */
          agendaDuration?: number;
          /**
           * [descr:dxSchedulerOptions.views.appointmentCollectorTemplate]
           */
          appointmentCollectorTemplate?:
            | DevExpress.core.template
            | ((
                data: DevExpress.ui.dxScheduler.AppointmentCollectorTemplateData,
                collectorElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.appointmentTemplate]
           */
          appointmentTemplate?:
            | DevExpress.core.template
            | ((
                model: DevExpress.ui.dxScheduler.AppointmentTemplateData,
                itemIndex: number,
                contentElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.appointmentTooltipTemplate]
           */
          appointmentTooltipTemplate?:
            | DevExpress.core.template
            | ((
                model: DevExpress.ui.dxScheduler.AppointmentTooltipTemplateData,
                itemIndex: number,
                contentElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.dropDownAppointmentTemplate]
           * @deprecated [depNote:dxSchedulerOptions.views.dropDownAppointmentTemplate]
           */
          dropDownAppointmentTemplate?:
            | DevExpress.core.template
            | ((
                itemData: any,
                itemIndex: number,
                contentElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.cellDuration]
           */
          cellDuration?: number;
          /**
           * [descr:dxSchedulerOptions.views.dataCellTemplate]
           */
          dataCellTemplate?:
            | DevExpress.core.template
            | ((
                itemData: any,
                itemIndex: number,
                itemElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.dateCellTemplate]
           */
          dateCellTemplate?:
            | DevExpress.core.template
            | ((
                itemData: any,
                itemIndex: number,
                itemElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.endDayHour]
           */
          endDayHour?: number;
          /**
           * [descr:dxSchedulerOptions.views.firstDayOfWeek]
           */
          firstDayOfWeek?: 0 | 1 | 2 | 3 | 4 | 5 | 6;
          /**
           * [descr:dxSchedulerOptions.views.groupByDate]
           */
          groupByDate?: boolean;
          /**
           * [descr:dxSchedulerOptions.views.groupOrientation]
           */
          groupOrientation?: 'horizontal' | 'vertical';
          /**
           * [descr:dxSchedulerOptions.views.groups]
           */
          groups?: Array<string>;
          /**
           * [descr:dxSchedulerOptions.views.intervalCount]
           */
          intervalCount?: number;
          /**
           * [descr:dxSchedulerOptions.views.maxAppointmentsPerCell]
           */
          maxAppointmentsPerCell?: number | 'auto' | 'unlimited';
          /**
           * [descr:dxSchedulerOptions.views.name]
           */
          name?: string;
          /**
           * [descr:dxSchedulerOptions.views.resourceCellTemplate]
           */
          resourceCellTemplate?:
            | DevExpress.core.template
            | ((
                itemData: any,
                itemIndex: number,
                itemElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.startDate]
           */
          startDate?: Date | number | string;
          /**
           * [descr:dxSchedulerOptions.views.startDayHour]
           */
          startDayHour?: number;
          /**
           * [descr:dxSchedulerOptions.views.timeCellTemplate]
           */
          timeCellTemplate?:
            | DevExpress.core.template
            | ((
                itemData: any,
                itemIndex: number,
                itemElement: DevExpress.core.DxElement
              ) => string | DevExpress.core.UserDefinedElement);
          /**
           * [descr:dxSchedulerOptions.views.type]
           */
          type?:
            | 'agenda'
            | 'day'
            | 'month'
            | 'timelineDay'
            | 'timelineMonth'
            | 'timelineWeek'
            | 'timelineWorkWeek'
            | 'week'
            | 'workWeek';
          /**
           * [descr:dxSchedulerOptions.views.scrolling]
           */
          scrolling?: dxSchedulerScrolling;
        }
    >;
  }
  /**
   * [descr:dxSchedulerScrolling]
   */
  export interface dxSchedulerScrolling {
    /**
     * [descr:dxSchedulerScrolling.mode]
     */
    mode?: 'standard' | 'virtual';
  }
  /**
   * [descr:dxScrollable]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxScrollable<
    TProperties = DevExpress.ui.dxScrollable.Properties
  > extends DOMComponent<TProperties> {
    /**
     * [descr:dxScrollable.clientHeight()]
     */
    clientHeight(): number;
    /**
     * [descr:dxScrollable.clientWidth()]
     */
    clientWidth(): number;
    /**
     * [descr:dxScrollable.content()]
     */
    content(): DevExpress.core.DxElement;
    /**
     * [descr:dxScrollable.scrollBy(distance)]
     */
    scrollBy(distance: number | any): void;
    /**
     * [descr:dxScrollable.scrollHeight()]
     */
    scrollHeight(): number;
    /**
     * [descr:dxScrollable.scrollLeft()]
     */
    scrollLeft(): number;
    /**
     * [descr:dxScrollable.scrollOffset()]
     */
    scrollOffset(): any;
    /**
     * [descr:dxScrollable.scrollTo(targetLocation)]
     */
    scrollTo(targetLocation: number | any): void;
    /**
     * [descr:dxScrollable.scrollToElement(element)]
     */
    scrollToElement(element: DevExpress.core.UserDefinedElement): void;
    /**
     * [descr:dxScrollable.scrollTop()]
     */
    scrollTop(): number;
    /**
     * [descr:dxScrollable.scrollWidth()]
     */
    scrollWidth(): number;
    /**
     * [descr:dxScrollable.update()]
     */
    update(): DevExpress.core.utils.DxPromise<void>;
  }
  module dxScrollable {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type Properties = dxScrollableOptions<ScrollableInstance>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface ScrollableInstance extends dxScrollable<Properties> {}
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ScrollEventInfo<T>
      extends DevExpress.events.NativeEventInfo<T> {
      readonly scrollOffset?: any;
      readonly reachedLeft?: boolean;
      readonly reachedRight?: boolean;
      readonly reachedTop?: boolean;
      readonly reachedBottom?: boolean;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxScrollableOptions<TComponent>
    extends DOMComponentOptions<TComponent> {
    /**
     * [descr:dxScrollableOptions.bounceEnabled]
     */
    bounceEnabled?: boolean;
    /**
     * [descr:dxScrollableOptions.direction]
     */
    direction?: 'both' | 'horizontal' | 'vertical';
    /**
     * [descr:dxScrollableOptions.disabled]
     */
    disabled?: boolean;
    /**
     * [descr:dxScrollableOptions.onScroll]
     */
    onScroll?: (
      e: DevExpress.ui.dxScrollable.ScrollEventInfo<TComponent>
    ) => void;
    /**
     * [descr:dxScrollableOptions.onUpdated]
     */
    onUpdated?: (
      e: DevExpress.ui.dxScrollable.ScrollEventInfo<TComponent>
    ) => void;
    /**
     * [descr:dxScrollableOptions.scrollByContent]
     */
    scrollByContent?: boolean;
    /**
     * [descr:dxScrollableOptions.scrollByThumb]
     */
    scrollByThumb?: boolean;
    /**
     * [descr:dxScrollableOptions.showScrollbar]
     */
    showScrollbar?: 'onScroll' | 'onHover' | 'always' | 'never';
    /**
     * [descr:dxScrollableOptions.useNative]
     */
    useNative?: boolean;
  }
  /**
   * [descr:dxScrollView]
   */
  export class dxScrollView extends dxScrollable<dxScrollViewOptions> {
    /**
     * [descr:dxScrollView.refresh()]
     */
    refresh(): void;
    /**
     * [descr:dxScrollView.release(preventScrollBottom)]
     */
    release(
      preventScrollBottom: boolean
    ): DevExpress.core.utils.DxPromise<void>;
  }
  module dxScrollView {
    export type DisposingEvent = DevExpress.events.EventInfo<dxScrollView>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxScrollView>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxScrollView> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxScrollViewOptions;
    export type PullDownEvent = DevExpress.events.EventInfo<dxScrollView>;
    export type ReachBottomEvent = DevExpress.events.EventInfo<dxScrollView>;
    export type ScrollEvent =
      DevExpress.ui.dxScrollable.ScrollEventInfo<dxScrollView>;
    export type UpdatedEvent =
      DevExpress.ui.dxScrollable.ScrollEventInfo<dxScrollView>;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxScrollViewOptions
    extends dxScrollableOptions<dxScrollView> {
    /**
     * [descr:dxScrollViewOptions.onPullDown]
     */
    onPullDown?: (e: DevExpress.ui.dxScrollView.PullDownEvent) => void;
    /**
     * [descr:dxScrollViewOptions.onReachBottom]
     */
    onReachBottom?: (e: DevExpress.ui.dxScrollView.ReachBottomEvent) => void;
    /**
     * [descr:dxScrollViewOptions.pulledDownText]
     */
    pulledDownText?: string;
    /**
     * [descr:dxScrollViewOptions.pullingDownText]
     */
    pullingDownText?: string;
    /**
     * [descr:dxScrollViewOptions.reachBottomText]
     */
    reachBottomText?: string;
    /**
     * [descr:dxScrollViewOptions.refreshingText]
     */
    refreshingText?: string;
  }
  /**
   * [descr:dxSelectBox]
   */
  export class dxSelectBox<
    TProperties = DevExpress.ui.dxSelectBox.Properties
  > extends dxDropDownList<TProperties> {}
  module dxSelectBox {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      Event
    >;
    export type ClosedEvent = DevExpress.events.EventInfo<dxSelectBox>;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxSelectBox>;
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      ClipboardEvent
    >;
    export type CustomItemCreatingEvent =
      DevExpress.events.EventInfo<dxSelectBox> & CustomItemCreatingInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface CustomItemCreatingInfo {
      readonly text?: string;
      customItem?: string | any | PromiseLike<any>;
    }
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      ClipboardEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxSelectBox>;
    export type DropDownButtonTemplateData =
      DevExpress.ui.dxDropDownEditor.DropDownButtonTemplateDataModel;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSelectBox>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      UIEvent
    >;
    export type ItemClickEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      KeyboardEvent | MouseEvent | PointerEvent
    > &
      DevExpress.events.ItemInfo;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      KeyboardEvent
    >;
    export type OpenedEvent = DevExpress.events.EventInfo<dxSelectBox>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxSelectBox> &
      DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      ClipboardEvent
    >;
    export type Properties = dxSelectBoxOptions<SelectBoxInstance>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface SelectBoxInstance extends dxSelectBox<Properties> {}
    export type SelectionChangedEvent =
      DevExpress.events.EventInfo<dxSelectBox> &
        DevExpress.ui.dxDropDownList.SelectionChangedInfo;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxSelectBox,
      KeyboardEvent | MouseEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSelectBoxOptions<TComponent>
    extends dxDropDownListOptions<TComponent> {
    /**
     * [descr:dxSelectBoxOptions.acceptCustomValue]
     */
    acceptCustomValue?: boolean;
    /**
     * [descr:dxSelectBoxOptions.fieldTemplate]
     */
    fieldTemplate?:
      | DevExpress.core.template
      | ((
          selectedItem: any,
          fieldElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSelectBoxOptions.onCustomItemCreating]
     */
    onCustomItemCreating?: (
      e: DevExpress.ui.dxSelectBox.CustomItemCreatingEvent
    ) => void;
    /**
     * [descr:dxSelectBoxOptions.openOnFieldClick]
     */
    openOnFieldClick?: boolean;
    /**
     * [descr:dxSelectBoxOptions.placeholder]
     */
    placeholder?: string;
    /**
     * [descr:dxSelectBoxOptions.showDropDownButton]
     */
    showDropDownButton?: boolean;
    /**
     * [descr:dxSelectBoxOptions.showSelectionControls]
     */
    showSelectionControls?: boolean;
    /**
     * [descr:dxSelectBoxOptions.valueChangeEvent]
     */
    valueChangeEvent?: string;

    /**
     * [descr:dxSelectBoxOptions.dropDownOptions]
     */
    dropDownOptions?: DevExpress.ui.dxPopup.Properties;
  }
  /**
   * [descr:dxSlideOut]
   * @deprecated [depNote:dxSlideOut]
   */
  export class dxSlideOut<
    TItem extends DevExpress.ui.dxSlideOut.ItemLike = any,
    TKey = any
  > extends CollectionWidget<dxSlideOutOptions<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxSlideOut.hideMenu()]
     */
    hideMenu(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxSlideOut.showMenu()]
     */
    showMenu(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxSlideOut.toggleMenuVisibility(showing)]
     */
    toggleMenuVisibility(
      showing?: boolean
    ): DevExpress.core.utils.DxPromise<void>;
  }
  module dxSlideOut {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxSlideOut<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxSlideOut<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      MenuGroupRenderedEvent: MenuGroupRenderedEvent<TItem, TKey>;
      MenuItemRenderedEvent: MenuItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxSlideOut<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxSlideOut<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxSlideOut<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxSlideOut<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxSlideOut<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type MenuGroupRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxSlideOut<TItem, TKey>>;
    export type MenuItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxSlideOut<TItem, TKey>>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxSlideOut<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxSlideOutOptions<TItem, TKey>;
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxSlideOut<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSlideOutItem extends CollectionWidgetItem {
    /**
     * [descr:dxSlideOutItem.menuTemplate]
     */
    menuTemplate?:
      | DevExpress.core.template
      | (() => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxSlideOutOptions<
    TItem extends DevExpress.ui.dxSlideOut.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<dxSlideOut<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxSlideOutOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxSlideOutOptions.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          container: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSlideOutOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxSlideOutOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxSlideOutOptions.menuGroupTemplate]
     */
    menuGroupTemplate?:
      | DevExpress.core.template
      | ((
          groupData: any,
          groupIndex: number,
          groupElement: any
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSlideOutOptions.menuGrouped]
     */
    menuGrouped?: boolean;
    /**
     * [descr:dxSlideOutOptions.menuItemTemplate]
     */
    menuItemTemplate?:
      | DevExpress.core.template
      | ((
          itemData: TItem,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSlideOutOptions.menuPosition]
     */
    menuPosition?: 'inverted' | 'normal';
    /**
     * [descr:dxSlideOutOptions.menuVisible]
     */
    menuVisible?: boolean;
    /**
     * [descr:dxSlideOutOptions.onMenuGroupRendered]
     */
    onMenuGroupRendered?: (
      e: DevExpress.ui.dxSlideOut.MenuGroupRenderedEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxSlideOutOptions.onMenuItemRendered]
     */
    onMenuItemRendered?: (
      e: DevExpress.ui.dxSlideOut.MenuItemRenderedEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxSlideOutOptions.selectedIndex]
     */
    selectedIndex?: number;
    /**
     * [descr:dxSlideOutOptions.swipeEnabled]
     */
    swipeEnabled?: boolean;
  }
  /**
   * [descr:dxSlideOutView]
   * @deprecated [depNote:dxSlideOutView]
   */
  export class dxSlideOutView extends Widget<dxSlideOutViewOptions> {
    /**
     * [descr:dxSlideOutView.content()]
     */
    content(): DevExpress.core.DxElement;
    /**
     * [descr:dxSlideOutView.hideMenu()]
     */
    hideMenu(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxSlideOutView.menuContent()]
     */
    menuContent(): DevExpress.core.DxElement;
    /**
     * [descr:dxSlideOutView.showMenu()]
     */
    showMenu(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxSlideOutView.toggleMenuVisibility(showing)]
     */
    toggleMenuVisibility(
      showing?: boolean
    ): DevExpress.core.utils.DxPromise<void>;
  }
  module dxSlideOutView {
    export type DisposingEvent = DevExpress.events.EventInfo<dxSlideOutView>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSlideOutView>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxSlideOutView> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSlideOutViewOptions;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSlideOutViewOptions extends WidgetOptions<dxSlideOutView> {
    /**
     * [descr:dxSlideOutViewOptions.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((contentElement: DevExpress.core.DxElement) => any);
    /**
     * [descr:dxSlideOutViewOptions.menuPosition]
     */
    menuPosition?: 'inverted' | 'normal';
    /**
     * [descr:dxSlideOutViewOptions.menuTemplate]
     */
    menuTemplate?:
      | DevExpress.core.template
      | ((menuElement: DevExpress.core.DxElement) => any);
    /**
     * [descr:dxSlideOutViewOptions.menuVisible]
     */
    menuVisible?: boolean;
    /**
     * [descr:dxSlideOutViewOptions.swipeEnabled]
     */
    swipeEnabled?: boolean;
  }
  /**
   * [descr:dxSlider]
   */
  export class dxSlider extends dxTrackBar<dxSliderOptions> {}
  module dxSlider {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxSlider>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxSlider>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSlider>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxSlider> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSliderOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxSlider,
      KeyboardEvent | MouseEvent | PointerEvent | TouchEvent | UIEvent
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * [descr:dxSliderBase]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSliderBaseOptions<TComponent>
    extends dxTrackBarOptions<TComponent> {
    /**
     * [descr:dxSliderBaseOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxSliderBaseOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxSliderBaseOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxSliderBaseOptions.keyStep]
     */
    keyStep?: number;
    /**
     * [descr:dxSliderBaseOptions.label]
     */
    label?: {
      /**
       * [descr:dxSliderBaseOptions.label.format]
       */
      format?: Format;
      /**
       * [descr:dxSliderBaseOptions.label.position]
       */
      position?: 'bottom' | 'top';
      /**
       * [descr:dxSliderBaseOptions.label.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxSliderBaseOptions.name]
     */
    name?: string;
    /**
     * [descr:dxSliderBaseOptions.showRange]
     */
    showRange?: boolean;
    /**
     * [descr:dxSliderBaseOptions.step]
     */
    step?: number;
    /**
     * [descr:dxSliderBaseOptions.tooltip]
     */
    tooltip?: {
      /**
       * [descr:dxSliderBaseOptions.tooltip.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxSliderBaseOptions.tooltip.format]
       */
      format?: Format;
      /**
       * [descr:dxSliderBaseOptions.tooltip.position]
       */
      position?: 'bottom' | 'top';
      /**
       * [descr:dxSliderBaseOptions.tooltip.showMode]
       */
      showMode?: 'always' | 'onHover';
    };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSliderOptions extends dxSliderBaseOptions<dxSlider> {
    /**
     * [descr:dxSliderOptions.value]
     */
    value?: number;
  }
  /**
   * [descr:dxSortable]
   */
  export class dxSortable
    extends DOMComponent<dxSortableOptions>
    implements DraggableBase
  {
    /**
     * [descr:dxSortable.update()]
     */
    update(): void;
  }
  module dxSortable {
    export interface AddEvent {
      readonly component: dxSortable;
      readonly element: DevExpress.core.DxElement;
      readonly model?: any;
      readonly event: DevExpress.events.DxEvent;
      readonly itemData?: any;
      readonly itemElement: DevExpress.core.DxElement;
      readonly fromIndex: number;
      readonly toIndex: number;
      readonly fromComponent: dxSortable | dxDraggable;
      readonly toComponent: dxSortable | dxDraggable;
      readonly fromData?: any;
      readonly toData?: any;
      readonly dropInsideItem: boolean;
    }
    export type DisposingEvent = DevExpress.events.EventInfo<dxSortable>;
    export type DragChangeEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxSortable> & {
        readonly itemData?: any;
        readonly itemElement: DevExpress.core.DxElement;
        readonly fromIndex?: number;
        readonly toIndex?: number;
        readonly fromComponent?: dxSortable | dxDraggable;
        readonly toComponent?: dxSortable | dxDraggable;
        readonly fromData?: any;
        readonly toData?: any;
        readonly dropInsideItem?: boolean;
      };
    export type DragEndEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxSortable> & {
        readonly itemData?: any;
        readonly itemElement: DevExpress.core.DxElement;
        readonly fromIndex: number;
        readonly toIndex: number;
        readonly fromComponent: dxSortable | dxDraggable;
        readonly toComponent: dxSortable | dxDraggable;
        readonly fromData?: any;
        readonly toData?: any;
        readonly dropInsideItem: boolean;
      };
    export type DragMoveEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxSortable> & {
        readonly itemData?: any;
        readonly itemElement: DevExpress.core.DxElement;
        readonly fromIndex: number;
        readonly toIndex: number;
        readonly fromComponent: dxSortable | dxDraggable;
        readonly toComponent: dxSortable | dxDraggable;
        readonly fromData?: any;
        readonly toData?: any;
        readonly dropInsideItem: boolean;
      };
    export type DragStartEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxSortable> & {
        itemData?: any;
        readonly itemElement: DevExpress.core.DxElement;
        readonly fromIndex: number;
        readonly fromData?: any;
      };
    export interface DragTemplateData {
      readonly itemData?: any;
      readonly itemElement: DevExpress.core.DxElement;
      readonly fromIndex: number;
    }
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSortable>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxSortable> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSortableOptions;
    export type RemoveEvent = DevExpress.events.NativeEventInfo<dxSortable> & {
      readonly itemData?: any;
      readonly itemElement: DevExpress.core.DxElement;
      readonly fromIndex: number;
      readonly toIndex: number;
      readonly fromComponent: dxSortable | dxDraggable;
      readonly toComponent: dxSortable | dxDraggable;
      readonly fromData?: any;
      readonly toData?: any;
    };
    export type ReorderEvent = DevExpress.events.NativeEventInfo<dxSortable> & {
      readonly itemData?: any;
      readonly itemElement: DevExpress.core.DxElement;
      readonly fromIndex: number;
      readonly toIndex: number;
      readonly fromComponent: dxSortable | dxDraggable;
      readonly toComponent: dxSortable | dxDraggable;
      readonly fromData?: any;
      readonly toData?: any;
      readonly dropInsideItem: boolean;
      promise?: PromiseLike<void>;
    };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSortableOptions extends DraggableBaseOptions<dxSortable> {
    /**
     * [descr:dxSortableOptions.allowDropInsideItem]
     */
    allowDropInsideItem?: boolean;
    /**
     * [descr:dxSortableOptions.allowReordering]
     */
    allowReordering?: boolean;
    /**
     * [descr:dxSortableOptions.dragTemplate]
     */
    dragTemplate?:
      | DevExpress.core.template
      | ((
          dragInfo: DevExpress.ui.dxSortable.DragTemplateData,
          containerElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSortableOptions.dropFeedbackMode]
     */
    dropFeedbackMode?: 'push' | 'indicate';
    /**
     * [descr:dxSortableOptions.filter]
     */
    filter?: string;
    /**
     * [descr:dxSortableOptions.itemOrientation]
     */
    itemOrientation?: 'horizontal' | 'vertical';
    /**
     * [descr:dxSortableOptions.moveItemOnDrop]
     */
    moveItemOnDrop?: boolean;
    /**
     * [descr:dxSortableOptions.onAdd]
     */
    onAdd?: (e: DevExpress.ui.dxSortable.AddEvent) => void;
    /**
     * [descr:dxSortableOptions.onDragChange]
     */
    onDragChange?: (e: DevExpress.ui.dxSortable.DragChangeEvent) => void;
    /**
     * [descr:dxSortableOptions.onDragEnd]
     */
    onDragEnd?: (e: DevExpress.ui.dxSortable.DragEndEvent) => void;
    /**
     * [descr:dxSortableOptions.onDragMove]
     */
    onDragMove?: (e: DevExpress.ui.dxSortable.DragMoveEvent) => void;
    /**
     * [descr:dxSortableOptions.onDragStart]
     */
    onDragStart?: (e: DevExpress.ui.dxSortable.DragStartEvent) => void;
    /**
     * [descr:dxSortableOptions.onRemove]
     */
    onRemove?: (e: DevExpress.ui.dxSortable.RemoveEvent) => void;
    /**
     * [descr:dxSortableOptions.onReorder]
     */
    onReorder?: (e: DevExpress.ui.dxSortable.ReorderEvent) => void;
  }
  /**
   * [descr:dxSpeedDialAction]
   */
  export class dxSpeedDialAction extends Widget<dxSpeedDialActionOptions> {}
  module dxSpeedDialAction {
    export type ClickEvent = DevExpress.events.NativeEventInfo<
      dxSpeedDialAction,
      MouseEvent | PointerEvent
    > & {
      actionElement?: DevExpress.core.DxElement;
    };
    export type ContentReadyEvent =
      DevExpress.events.EventInfo<dxSpeedDialAction> & {
        actionElement?: DevExpress.core.DxElement;
      };
    export type DisposingEvent = DevExpress.events.EventInfo<dxSpeedDialAction>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSpeedDialAction>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxSpeedDialAction> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSpeedDialActionOptions;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSpeedDialActionOptions
    extends WidgetOptions<dxSpeedDialAction> {
    /**
     * [descr:dxSpeedDialActionOptions.icon]
     */
    icon?: string;
    /**
     * [descr:dxSpeedDialActionOptions.index]
     */
    index?: number;
    /**
     * [descr:dxSpeedDialActionOptions.label]
     */
    label?: string;
    /**
     * [descr:dxSpeedDialActionOptions.onClick]
     */
    onClick?: (e: DevExpress.ui.dxSpeedDialAction.ClickEvent) => void;
    /**
     * [descr:dxSpeedDialActionOptions.onContentReady]
     */
    onContentReady?: (
      e: DevExpress.ui.dxSpeedDialAction.ContentReadyEvent
    ) => void;
    /**
     * [descr:dxSpeedDialActionOptions.visible]
     */
    visible?: boolean;
  }
  /**
   * [descr:dxSwitch]
   */
  export class dxSwitch extends Editor<dxSwitchOptions> {}
  module dxSwitch {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxSwitch>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxSwitch>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSwitch>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxSwitch> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSwitchOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxSwitch,
      KeyboardEvent | MouseEvent | PointerEvent | TouchEvent | UIEvent
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSwitchOptions extends EditorOptions<dxSwitch> {
    /**
     * [descr:dxSwitchOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxSwitchOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxSwitchOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxSwitchOptions.name]
     */
    name?: string;
    /**
     * [descr:dxSwitchOptions.switchedOffText]
     */
    switchedOffText?: string;
    /**
     * [descr:dxSwitchOptions.switchedOnText]
     */
    switchedOnText?: string;
    /**
     * [descr:dxSwitchOptions.value]
     */
    value?: boolean;
  }
  /**
   * [descr:dxTabPanel]
   */
  export class dxTabPanel<
    TItem extends DevExpress.ui.dxTabPanel.ItemLike = any,
    TKey = any
  > extends dxMultiView<dxTabPanelOptions<TItem, TKey>, TItem, TKey> {}
  module dxTabPanel {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTabPanel<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTabPanel<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
      TitleClickEvent: TitleClickEvent<TItem, TKey>;
      TitleHoldEvent: TitleHoldEvent<TItem, TKey>;
      TitleRenderedEvent: TitleRenderedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxTabPanel<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTabPanel<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTabPanel<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTabPanel<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTabPanel<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTabPanel<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxTabPanelOptions<TItem, TKey>;
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTabPanel<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface TabPanelItemInfo<TItem extends ItemLike> {
      readonly itemData?: TItem;
      readonly itemElement?: DevExpress.core.DxElement;
    }
    export type TitleClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTabPanel<TItem, TKey>> &
      TabPanelItemInfo<TItem>;
    export type TitleHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTabPanel<TItem, TKey>> &
      TabPanelItemInfo<TItem>;
    export type TitleRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTabPanel<TItem, TKey>> &
      TabPanelItemInfo<TItem>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTabPanelItem extends DevExpress.ui.dxMultiView.Item {
    /**
     * [descr:dxTabPanelItem.badge]
     */
    badge?: string;
    /**
     * [descr:dxTabPanelItem.icon]
     */
    icon?: string;
    /**
     * [descr:dxTabPanelItem.tabTemplate]
     */
    tabTemplate?:
      | DevExpress.core.template
      | (() => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxTabPanelItem.title]
     */
    title?: string;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxTabPanelOptions<
    TItem extends DevExpress.ui.dxTabPanel.ItemLike = any,
    TKey = any
  > extends dxMultiViewBaseOptions<dxTabPanel<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxTabPanelOptions.animationEnabled]
     */
    animationEnabled?: boolean;
    /**
     * [descr:dxTabPanelOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxTabPanelOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxTabPanelOptions.itemTitleTemplate]
     */
    itemTitleTemplate?:
      | DevExpress.core.template
      | ((
          itemData: TItem,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxTabPanelOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxTabPanelOptions.onTitleClick]
     */
    onTitleClick?:
      | ((e: DevExpress.ui.dxTabPanel.TitleClickEvent<TItem, TKey>) => void)
      | string;
    /**
     * [descr:dxTabPanelOptions.onTitleHold]
     */
    onTitleHold?: (
      e: DevExpress.ui.dxTabPanel.TitleHoldEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxTabPanelOptions.onTitleRendered]
     */
    onTitleRendered?: (
      e: DevExpress.ui.dxTabPanel.TitleRenderedEvent<TItem, TKey>
    ) => void;
    /**
     * [descr:dxTabPanelOptions.repaintChangesOnly]
     */
    repaintChangesOnly?: boolean;
    /**
     * [descr:dxTabPanelOptions.scrollByContent]
     */
    scrollByContent?: boolean;
    /**
     * [descr:dxTabPanelOptions.scrollingEnabled]
     */
    scrollingEnabled?: boolean;
    /**
     * [descr:dxTabPanelOptions.showNavButtons]
     */
    showNavButtons?: boolean;
    /**
     * [descr:dxTabPanelOptions.swipeEnabled]
     */
    swipeEnabled?: boolean;
  }
  /**
   * [descr:dxTabs]
   */
  export class dxTabs<
    TProperties extends dxTabsOptions<TItem, TKey> = dxTabsOptions<any, any>,
    TItem extends DevExpress.ui.dxTabs.ItemLike = any,
    TKey = any
  > extends CollectionWidget<TProperties, TItem, TKey> {}
  module dxTabs {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<TabsInstance<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<TabsInstance<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<TabsInstance<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<TabsInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<TabsInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<TabsInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<TabsInstance<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<TabsInstance<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxTabsBaseOptions<TabsInstance<TItem, TKey>, TItem, TKey>;
    export type SelectionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<TabsInstance<TItem, TKey>> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface TabsInstance<TItem, TKey>
      extends dxTabs<Properties<TItem, TKey>, TItem, TKey> {}
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTabsBaseOptions<
    TComponent extends dxTabs<any, TItem, TKey> = dxTabs<any, any, any>,
    TItem extends DevExpress.ui.dxTabs.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<TComponent, TItem, TKey> {
    /**
     * [descr:dxTabsOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxTabsOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxTabsOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxTabsOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxTabsOptions.repaintChangesOnly]
     */
    repaintChangesOnly?: boolean;
    /**
     * [descr:dxTabsOptions.scrollByContent]
     */
    scrollByContent?: boolean;
    /**
     * [descr:dxTabsOptions.scrollingEnabled]
     */
    scrollingEnabled?: boolean;
    /**
     * [descr:dxTabsOptions.selectionMode]
     */
    selectionMode?: 'multiple' | 'single';
    /**
     * [descr:dxTabsOptions.showNavButtons]
     */
    showNavButtons?: boolean;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTabsItem extends CollectionWidgetItem {
    /**
     * [descr:dxTabsItem.badge]
     */
    badge?: string;
    /**
     * [descr:dxTabsItem.icon]
     */
    icon?: string;
  }
  /**
   * @deprecated use DevExpress.ui.dxTabs.Properties instead
   */
  export interface dxTabsOptions<
    TItem extends DevExpress.ui.dxTabs.ItemLike = any,
    TKey = any
  > extends DevExpress.ui.dxTabs.Properties<TItem, TKey> {}
  /**
   * [descr:dxTagBox]
   */
  export class dxTagBox extends dxSelectBox<dxTagBoxOptions> {}
  module dxTagBox {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      Event
    >;
    export type ClosedEvent = DevExpress.events.EventInfo<dxTagBox>;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxTagBox>;
    export type CustomItemCreatingEvent =
      DevExpress.events.EventInfo<dxTagBox> &
        DevExpress.ui.dxSelectBox.CustomItemCreatingInfo;
    export type DisposingEvent = DevExpress.events.EventInfo<dxTagBox>;
    export type DropDownButtonTemplateData =
      DevExpress.ui.dxDropDownEditor.DropDownButtonTemplateDataModel;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxTagBox>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      UIEvent
    >;
    export type ItemClickEvent = DevExpress.events.NativeEventInfo<dxTagBox> &
      DevExpress.events.ItemInfo;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      KeyboardEvent
    >;
    export type MultiTagPreparingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTagBox> & {
        readonly multiTagElement: DevExpress.core.DxElement;
        readonly selectedItems?: Array<string | number | any>;
        text?: string;
      };
    export type OpenedEvent = DevExpress.events.EventInfo<dxTagBox>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxTagBox> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxTagBoxOptions;
    export type SelectAllValueChangedEvent =
      DevExpress.events.EventInfo<dxTagBox> & {
        readonly value: boolean;
      };
    export type SelectionChangedEvent = DevExpress.events.EventInfo<dxTagBox> &
      DevExpress.ui.CollectionWidget.SelectionChangedInfo<
        string | number | any
      >;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxTagBox,
      KeyboardEvent | MouseEvent | PointerEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTagBoxOptions
    extends Pick<
      dxSelectBoxOptions<dxTagBox>,
      Exclude<keyof dxSelectBoxOptions<dxTagBox>, 'onSelectionChanged'>
    > {
    /**
     * [descr:dxTagBoxOptions.applyValueMode]
     */
    applyValueMode?: 'instantly' | 'useButtons';
    /**
     * [descr:dxTagBoxOptions.hideSelectedItems]
     */
    hideSelectedItems?: boolean;
    /**
     * [descr:dxTagBoxOptions.maxDisplayedTags]
     */
    maxDisplayedTags?: number;
    /**
     * [descr:dxTagBoxOptions.multiline]
     */
    multiline?: boolean;
    /**
     * [descr:dxTagBoxOptions.onMultiTagPreparing]
     */
    onMultiTagPreparing?: (
      e: DevExpress.ui.dxTagBox.MultiTagPreparingEvent
    ) => void;
    /**
     * [descr:dxTagBoxOptions.onSelectAllValueChanged]
     */
    onSelectAllValueChanged?: (
      e: DevExpress.ui.dxTagBox.SelectAllValueChangedEvent
    ) => void;
    /**
     * [descr:dxTagBoxOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.ui.dxTagBox.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxTagBoxOptions.selectAllMode]
     */
    selectAllMode?: 'allPages' | 'page';
    /**
     * [descr:dxTagBoxOptions.selectedItems]
     */
    selectedItems?: Array<string | number | any>;
    /**
     * [descr:dxTagBoxOptions.selectAllText]
     */
    selectAllText?: string;
    /**
     * [descr:dxTagBoxOptions.showDropDownButton]
     */
    showDropDownButton?: boolean;
    /**
     * [descr:dxTagBoxOptions.maxFilterQueryLength]
     */
    maxFilterQueryLength?: number;
    /**
     * [descr:dxTagBoxOptions.showMultiTagOnly]
     */
    showMultiTagOnly?: boolean;
    /**
     * [descr:dxTagBoxOptions.tagTemplate]
     */
    tagTemplate?:
      | DevExpress.core.template
      | ((
          itemData: any,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxTagBoxOptions.value]
     */
    value?: Array<string | number | any>;
  }
  /**
   * [descr:dxTextArea]
   */
  export class dxTextArea extends dxTextBox<dxTextAreaOptions> {}
  module dxTextArea {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      Event
    >;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxTextArea>;
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      ClipboardEvent
    >;
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      ClipboardEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxTextArea>;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxTextArea>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      UIEvent
    >;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      KeyboardEvent
    >;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxTextArea> &
      DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      ClipboardEvent
    >;
    export type Properties = dxTextAreaOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxTextArea,
      KeyboardEvent | MouseEvent | PointerEvent | TouchEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTextAreaOptions extends dxTextBoxOptions<dxTextArea> {
    /**
     * [descr:dxTextAreaOptions.autoResizeEnabled]
     */
    autoResizeEnabled?: boolean;
    /**
     * [descr:dxTextAreaOptions.maxHeight]
     */
    maxHeight?: number | string;
    /**
     * [descr:dxTextAreaOptions.minHeight]
     */
    minHeight?: number | string;
    /**
     * [descr:dxTextAreaOptions.spellcheck]
     */
    spellcheck?: boolean;
  }
  /**
   * [descr:dxTextBox]
   */
  export class dxTextBox<
    TProperties = DevExpress.ui.dxTextBox.Properties
  > extends dxTextEditor<TProperties> {}
  module dxTextBox {
    export type ChangeEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      Event
    >;
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxTextBox>;
    export type CopyEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      ClipboardEvent
    >;
    export type CutEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      ClipboardEvent
    >;
    export type DisposingEvent = DevExpress.events.EventInfo<dxTextBox>;
    export type EnterKeyEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      KeyboardEvent
    >;
    export type FocusInEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      FocusEvent
    >;
    export type FocusOutEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      FocusEvent
    >;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxTextBox>;
    export type InputEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      UIEvent
    >;
    export type KeyDownEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      KeyboardEvent
    >;
    export type KeyPressEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      KeyboardEvent
    >;
    export type KeyUpEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      KeyboardEvent
    >;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxTextBox> &
      DevExpress.events.ChangedOptionInfo;
    export type PasteEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      ClipboardEvent
    >;
    export type Properties = dxTextBoxOptions<TextBoxInstance>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface TextBoxInstance extends dxTextBox<Properties> {}
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxTextBox,
      KeyboardEvent | MouseEvent | PointerEvent | TouchEvent | Event
    > &
      DevExpress.ui.Editor.ValueChangedInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTextBoxOptions<TComponent>
    extends dxTextEditorOptions<TComponent> {
    /**
     * [descr:dxTextBoxOptions.maxLength]
     */
    maxLength?: string | number;
    /**
     * [descr:dxTextBoxOptions.mode]
     */
    mode?: 'email' | 'password' | 'search' | 'tel' | 'text' | 'url';
    /**
     * [descr:dxTextBoxOptions.value]
     */
    value?: string;
  }
  /**
   * [descr:dxTextEditor]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxTextEditor<
    TProperties = DevExpress.ui.dxTextEditor.Properties
  > extends Editor<TProperties> {
    /**
     * [descr:dxTextEditor.blur()]
     */
    blur(): void;
    /**
     * [descr:dxTextEditor.focus()]
     */
    focus(): void;
    /**
     * [descr:dxTextEditor.getButton(name)]
     */
    getButton(name: string): dxButton | undefined;
  }
  module dxTextEditor {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type Properties = dxTextEditorOptions<TextEditorInstance>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface TextEditorInstance extends dxTextEditor<Properties> {}
  }
  /**
   * [descr:dxTextEditorButton]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTextEditorButton {
    /**
     * [descr:dxTextEditorButton.location]
     */
    location?: 'after' | 'before';
    /**
     * [descr:dxTextEditorButton.name]
     */
    name?: string;
    /**
     * [descr:dxTextEditorButton.options]
     */
    options?: dxButtonOptions;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTextEditorOptions<TComponent>
    extends EditorOptions<TComponent> {
    /**
     * [descr:dxTextEditorOptions.buttons]
     */
    buttons?: Array<string | 'clear' | dxTextEditorButton>;
    /**
     * [descr:dxTextEditorOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxTextEditorOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxTextEditorOptions.inputAttr]
     */
    inputAttr?: any;
    /**
     * [descr:dxTextEditorOptions.label]
     */
    label?: string;
    /**
     * [descr:dxTextEditorOptions.labelMode]
     */
    labelMode?: 'static' | 'floating' | 'hidden';
    /**
     * [descr:dxTextEditorOptions.mask]
     */
    mask?: string;
    /**
     * [descr:dxTextEditorOptions.maskChar]
     */
    maskChar?: string;
    /**
     * [descr:dxTextEditorOptions.maskInvalidMessage]
     */
    maskInvalidMessage?: string;
    /**
     * [descr:dxTextEditorOptions.maskRules]
     */
    maskRules?: any;
    /**
     * [descr:dxTextEditorOptions.name]
     */
    name?: string;
    /**
     * [descr:dxTextEditorOptions.onChange]
     */
    onChange?: (
      e: DevExpress.events.NativeEventInfo<TComponent, Event>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onCopy]
     */
    onCopy?: (
      e: DevExpress.events.NativeEventInfo<TComponent, ClipboardEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onCut]
     */
    onCut?: (
      e: DevExpress.events.NativeEventInfo<TComponent, ClipboardEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onEnterKey]
     */
    onEnterKey?: (
      e: DevExpress.events.NativeEventInfo<TComponent, KeyboardEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onFocusIn]
     */
    onFocusIn?: (
      e: DevExpress.events.NativeEventInfo<TComponent, FocusEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onFocusOut]
     */
    onFocusOut?: (
      e: DevExpress.events.NativeEventInfo<TComponent, FocusEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onInput]
     */
    onInput?: (
      e: DevExpress.events.NativeEventInfo<TComponent, UIEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onKeyDown]
     */
    onKeyDown?: (
      e: DevExpress.events.NativeEventInfo<TComponent, KeyboardEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onKeyUp]
     */
    onKeyUp?: (
      e: DevExpress.events.NativeEventInfo<TComponent, KeyboardEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.onPaste]
     */
    onPaste?: (
      e: DevExpress.events.NativeEventInfo<TComponent, ClipboardEvent>
    ) => void;
    /**
     * [descr:dxTextEditorOptions.placeholder]
     */
    placeholder?: string;
    /**
     * [descr:dxTextEditorOptions.showClearButton]
     */
    showClearButton?: boolean;
    /**
     * [descr:dxTextEditorOptions.showMaskMode]
     */
    showMaskMode?: 'always' | 'onFocus';
    /**
     * [descr:dxTextEditorOptions.spellcheck]
     */
    spellcheck?: boolean;
    /**
     * [descr:dxTextEditorOptions.stylingMode]
     */
    stylingMode?: 'outlined' | 'underlined' | 'filled';
    /**
     * [descr:dxTextEditorOptions.text]
     */
    text?: string;
    /**
     * [descr:dxTextEditorOptions.useMaskedValue]
     */
    useMaskedValue?: boolean;
    /**
     * [descr:dxTextEditorOptions.value]
     */
    value?: any;
    /**
     * [descr:dxTextEditorOptions.valueChangeEvent]
     */
    valueChangeEvent?: string;
  }
  /**
   * [descr:dxTileView]
   */
  export class dxTileView<
    TItem extends DevExpress.ui.dxTileView.ItemLike = any,
    TKey = any
  > extends CollectionWidget<dxTileViewOptions<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxTileView.scrollPosition()]
     */
    scrollPosition(): number;
  }
  module dxTileView {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTileView<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTileView<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxTileView<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTileView<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTileView<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTileView<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTileView<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTileView<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxTileViewOptions<TItem, TKey>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTileViewItem extends CollectionWidgetItem {
    /**
     * [descr:dxTileViewItem.heightRatio]
     */
    heightRatio?: number;
    /**
     * [descr:dxTileViewItem.widthRatio]
     */
    widthRatio?: number;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxTileViewOptions<
    TItem extends DevExpress.ui.dxTileView.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<dxTileView<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxTileViewOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:dxTileViewOptions.baseItemHeight]
     */
    baseItemHeight?: number;
    /**
     * [descr:dxTileViewOptions.baseItemWidth]
     */
    baseItemWidth?: number;
    /**
     * [descr:dxTileViewOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxTileViewOptions.direction]
     */
    direction?: 'horizontal' | 'vertical';
    /**
     * [descr:dxTileViewOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:dxTileViewOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxTileViewOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:dxTileViewOptions.itemMargin]
     */
    itemMargin?: number;
    /**
     * [descr:dxTileViewOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxTileViewOptions.showScrollbar]
     */
    showScrollbar?: boolean;
  }
  /**
   * [descr:dxToast]
   */
  export class dxToast extends dxOverlay<dxToastOptions> {}
  module dxToast {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxToast>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxToast>;
    export type HiddenEvent = DevExpress.events.EventInfo<dxToast>;
    export type HidingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxToast>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxToast>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxToast> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxToastOptions;
    export type ShowingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxToast>;
    export type ShownEvent = DevExpress.events.EventInfo<dxToast>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxToastAnimation extends dxOverlayAnimation {
    /**
     * [descr:dxToastOptions.animation.hide]
     */
    hide?: AnimationConfig;
    /**
     * [descr:dxToastOptions.animation.show]
     */
    show?: AnimationConfig;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxToastOptions extends dxOverlayOptions<dxToast> {
    /**
     * [descr:dxToastOptions.animation]
     */
    animation?: dxToastAnimation;
    /**
     * [descr:dxToastOptions.closeOnClick]
     */
    closeOnClick?: boolean;
    /**
     * [descr:dxToastOptions.closeOnOutsideClick]
     */
    closeOnOutsideClick?:
      | boolean
      | ((
          event: DevExpress.events.DxEvent<
            MouseEvent | PointerEvent | TouchEvent
          >
        ) => boolean);
    /**
     * [descr:dxToastOptions.closeOnSwipe]
     */
    closeOnSwipe?: boolean;
    /**
     * [descr:dxToastOptions.displayTime]
     */
    displayTime?: number;
    /**
     * [descr:dxToastOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:dxToastOptions.maxWidth]
     */
    maxWidth?: number | string | (() => number | string);
    /**
     * [descr:dxToastOptions.message]
     */
    message?: string;
    /**
     * [descr:dxToastOptions.minWidth]
     */
    minWidth?: number | string | (() => number | string);
    /**
     * [descr:dxToastOptions.position]
     */
    position?: PositionConfig | string;
    /**
     * [descr:dxToastOptions.shading]
     */
    shading?: boolean;
    /**
     * [descr:dxToastOptions.type]
     */
    type?: 'custom' | 'error' | 'info' | 'success' | 'warning';
    /**
     * [descr:dxToastOptions.width]
     */
    width?: number | string | (() => number | string);
  }
  /**
   * [descr:dxToolbar]
   */
  export class dxToolbar<
    TItem extends DevExpress.ui.dxToolbar.ItemLike = any,
    TKey = any
  > extends CollectionWidget<dxToolbarOptions<TItem, TKey>, TItem, TKey> {}
  module dxToolbar {
    export type ContentReadyEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxToolbar<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxToolbar<TItem, TKey>>;
    export type ExplicitTypes<TItem extends ItemLike, TKey> = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TItem, TKey>;
      ItemHoldEvent: ItemHoldEvent<TItem, TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxToolbar<TItem, TKey>>;
    export type ItemClickEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxToolbar<TItem, TKey>,
      MouseEvent | PointerEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemContextMenuEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxToolbar<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    export type ItemHoldEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxToolbar<TItem, TKey>,
      MouseEvent | PointerEvent | TouchEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type ItemLike = string | Item | any;
    export type ItemRenderedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxToolbar<TItem, TKey>> &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxToolbar<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends ItemLike = any,
      TKey = any
    > = dxToolbarOptions<TItem, TKey>;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxToolbarItem extends CollectionWidgetItem {
    /**
     * [descr:dxToolbarItem.cssClass]
     */
    cssClass?: string;
    /**
     * [descr:dxToolbarItem.locateInMenu]
     */
    locateInMenu?: 'always' | 'auto' | 'never';
    /**
     * [descr:dxToolbarItem.location]
     */
    location?: 'after' | 'before' | 'center';
    /**
     * [descr:dxToolbarItem.menuItemTemplate]
     */
    menuItemTemplate?:
      | DevExpress.core.template
      | (() => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxToolbarItem.options]
     */
    options?: any;
    /**
     * [descr:dxToolbarItem.showText]
     */
    showText?: 'always' | 'inMenu';
    /**
     * [descr:dxToolbarItem.widget]
     */
    widget?:
      | 'dxAutocomplete'
      | 'dxButton'
      | 'dxCheckBox'
      | 'dxDateBox'
      | 'dxMenu'
      | 'dxSelectBox'
      | 'dxTabs'
      | 'dxTextBox'
      | 'dxButtonGroup'
      | 'dxDropDownButton';
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxToolbarOptions<
    TItem extends DevExpress.ui.dxToolbar.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<dxToolbar<TItem, TKey>, TItem, TKey> {
    /**
     * [descr:dxToolbarOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TItem, TKey>;
    /**
     * [descr:dxToolbarOptions.items]
     */
    items?: Array<TItem>;
    /**
     * [descr:dxToolbarOptions.menuItemTemplate]
     */
    menuItemTemplate?:
      | DevExpress.core.template
      | ((
          itemData: TItem,
          itemIndex: number,
          itemElement: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxToolbarOptions.height]
     * @deprecated [depNote:dxToolbarOptions.height]
     */
    height?: number | string | (() => number | string);
  }
  /**
   * [descr:dxTooltip]
   */
  export class dxTooltip extends dxPopover<dxTooltipOptions> {}
  module dxTooltip {
    export type ContentReadyEvent = DevExpress.events.EventInfo<dxTooltip>;
    export type DisposingEvent = DevExpress.events.EventInfo<dxTooltip>;
    export type HiddenEvent = DevExpress.events.EventInfo<dxTooltip>;
    export type HidingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTooltip>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxTooltip>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxTooltip> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxTooltipOptions;
    export type ShowingEvent = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTooltip>;
    export type ShownEvent = DevExpress.events.EventInfo<dxTooltip>;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTooltipOptions extends dxPopoverOptions<dxTooltip> {}
  /**
   * [descr:dxTrackBar]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxTrackBar<TProperties> extends Editor<TProperties> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTrackBarOptions<TComponent>
    extends EditorOptions<TComponent> {
    /**
     * [descr:dxTrackBarOptions.max]
     */
    max?: number;
    /**
     * [descr:dxTrackBarOptions.min]
     */
    min?: number;
  }
  /**
   * [descr:dxTreeList]
   */
  export class dxTreeList<TRowData = any, TKey = any>
    extends Widget<dxTreeListOptions<TRowData, TKey>>
    implements GridBase<TRowData, TKey>
  {
    /**
     * [descr:dxTreeList.addColumn(columnOptions)]
     */
    addColumn(
      columnOptions: DevExpress.ui.dxTreeList.Column<TRowData, TKey> | string
    ): void;
    /**
     * [descr:dxTreeList.addRow()]
     */
    addRow(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeList.addRow(parentId)]
     */
    addRow(parentId: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeList.collapseRow(key)]
     */
    collapseRow(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeList.expandRow(key)]
     */
    expandRow(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeList.forEachNode(callback)]
     */
    forEachNode(callback: Function): void;
    /**
     * [descr:dxTreeList.forEachNode(nodes, callback)]
     */
    forEachNode(
      nodes: Array<DevExpress.ui.dxTreeList.Node<TRowData, TKey>>,
      callback: Function
    ): void;
    /**
     * [descr:dxTreeList.getNodeByKey(key)]
     */
    getNodeByKey(key: TKey): DevExpress.ui.dxTreeList.Node<TRowData, TKey>;
    /**
     * [descr:dxTreeList.getRootNode()]
     */
    getRootNode(): DevExpress.ui.dxTreeList.Node<TRowData, TKey>;
    /**
     * [descr:dxTreeList.getSelectedRowKeys()]
     */
    getSelectedRowKeys(): Array<TKey>;
    /**
     * [descr:dxTreeList.getSelectedRowKeys(mode)]
     */
    getSelectedRowKeys(mode: string): Array<TKey>;
    /**
     * [descr:dxTreeList.getSelectedRowsData()]
     */
    getSelectedRowsData(): Array<TRowData>;
    /**
     * [descr:dxTreeList.getSelectedRowsData(mode)]
     */
    getSelectedRowsData(mode: string): Array<TRowData>;
    /**
     * [descr:dxTreeList.getVisibleColumns()]
     */
    getVisibleColumns(): Array<DevExpress.ui.dxTreeList.Column<TRowData, TKey>>;
    /**
     * [descr:dxTreeList.getVisibleColumns(headerLevel)]
     */
    getVisibleColumns(
      headerLevel: number
    ): Array<DevExpress.ui.dxTreeList.Column<TRowData, TKey>>;
    /**
     * [descr:dxTreeList.getVisibleRows()]
     */
    getVisibleRows(): Array<DevExpress.ui.dxTreeList.Row<TRowData, TKey>>;
    /**
     * [descr:dxTreeList.isRowExpanded(key)]
     */
    isRowExpanded(key: TKey): boolean;
    /**
     * [descr:dxTreeList.loadDescendants()]
     */
    loadDescendants(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeList.loadDescendants(keys)]
     */
    loadDescendants(keys: Array<TKey>): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeList.loadDescendants(keys, childrenOnly)]
     */
    loadDescendants(
      keys: Array<TKey>,
      childrenOnly: boolean
    ): DevExpress.core.utils.DxPromise<void>;

    beginCustomLoading(messageText: string): void;
    byKey(key: TKey): DevExpress.core.utils.DxPromise<TRowData>;
    cancelEditData(): void;
    cellValue(rowIndex: number, dataField: string): any;
    cellValue(rowIndex: number, dataField: string, value: any): void;
    cellValue(rowIndex: number, visibleColumnIndex: number): any;
    cellValue(rowIndex: number, visibleColumnIndex: number, value: any): void;
    clearFilter(): void;
    clearFilter(filterName: string): void;
    clearSelection(): void;
    clearSorting(): void;
    closeEditCell(): void;
    collapseAdaptiveDetailRow(): void;
    columnCount(): number;
    columnOption(id: number | string): any;
    columnOption(id: number | string, optionName: string): any;
    columnOption(
      id: number | string,
      optionName: string,
      optionValue: any
    ): void;
    columnOption(id: number | string, options: any): void;
    deleteColumn(id: number | string): void;
    deleteRow(rowIndex: number): void;
    deselectAll(): DevExpress.core.utils.DxPromise<void>;
    deselectRows(
      keys: Array<TKey>
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    editCell(rowIndex: number, dataField: string): void;
    editCell(rowIndex: number, visibleColumnIndex: number): void;
    editRow(rowIndex: number): void;
    endCustomLoading(): void;
    expandAdaptiveDetailRow(key: TKey): void;
    filter(): any;
    filter(filterExpr: any): void;
    focus(): void;
    focus(element: DevExpress.core.UserDefinedElement): void;
    getCellElement(
      rowIndex: number,
      dataField: string
    ): DevExpress.core.DxElement | undefined;
    getCellElement(
      rowIndex: number,
      visibleColumnIndex: number
    ): DevExpress.core.DxElement | undefined;
    getCombinedFilter(): any;
    getCombinedFilter(returnDataField: boolean): any;
    getDataSource(): DevExpress.data.DataSource<TRowData, TKey>;
    getKeyByRowIndex(rowIndex: number): TKey | undefined;
    getRowElement(
      rowIndex: number
    ): DevExpress.core.UserDefinedElementsArray | undefined;
    getRowIndexByKey(key: TKey): number;
    getScrollable(): dxScrollable;
    getVisibleColumnIndex(id: number | string): number;
    hasEditData(): boolean;
    hideColumnChooser(): void;
    isAdaptiveDetailRowExpanded(key: TKey): boolean;
    isRowFocused(key: TKey): boolean;
    isRowSelected(key: TKey): boolean;
    keyOf(obj: TRowData): TKey;
    navigateToRow(key: TKey): DevExpress.core.utils.DxPromise<void>;
    pageCount(): number;
    pageIndex(): number;
    pageIndex(newIndex: number): DevExpress.core.utils.DxPromise<void>;
    pageSize(): number;
    pageSize(value: number): void;
    refresh(): DevExpress.core.utils.DxPromise<void>;
    refresh(changesOnly: boolean): DevExpress.core.utils.DxPromise<void>;
    repaintRows(rowIndexes: Array<number>): void;
    saveEditData(): DevExpress.core.utils.DxPromise<void>;
    searchByText(text: string): void;
    selectAll(): DevExpress.core.utils.DxPromise<void>;
    selectRows(
      keys: Array<TKey>,
      preserve: boolean
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    selectRowsByIndexes(
      indexes: Array<number>
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    showColumnChooser(): void;
    state(): any;
    state(state: any): void;
    undeleteRow(rowIndex: number): void;
    updateDimensions(): void;
  }
  module dxTreeList {
    export type AdaptiveDetailRowPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.AdaptiveDetailRowPreparingInfo;
    export type CellClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> &
      CellInfo<TRowData, TKey>;
    export type CellDblClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> &
      CellInfo<TRowData, TKey>;
    export type CellHoverChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      CellInfo<TRowData, TKey> & {
        readonly eventType: string;
      };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface CellInfo<TRowData = any, TKey = any> {
      readonly data: TRowData;
      readonly key: TKey;
      readonly value?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly cellElement: DevExpress.core.DxElement;
      readonly row: Row<TRowData, TKey>;
    }
    export type CellPreparedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      CellInfo<TRowData, TKey> & {
        readonly isSelected?: boolean;
        readonly isExpanded?: boolean;
        readonly isNewRow?: boolean;
        readonly watch?: Function;
        readonly oldValue?: any;
      };
    export type Column<TRowData = any, TKey = any> = dxTreeListColumn<
      TRowData,
      TKey
    >;
    export type ColumnButton<
      TRowData = any,
      TKey = any
    > = dxTreeListColumnButton<TRowData, TKey>;
    export type ColumnButtonClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> & {
      row?: Row<TRowData, TKey>;
      column?: Column<TRowData, TKey>;
    };
    export type ColumnButtonTemplateData<TRowData = any, TKey = any> = {
      readonly component: dxTreeList<TRowData, TKey>;
      readonly data: TRowData;
      readonly key: TKey;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly row: Row<TRowData, TKey>;
    };
    export type ColumnCellTemplateData<TRowData = any, TKey = any> = {
      readonly data: TRowData;
      readonly component: dxTreeList<TRowData, TKey>;
      readonly value?: any;
      readonly oldValue?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly row: Row<TRowData, TKey>;
      readonly rowType: string;
      readonly watch?: Function;
    };
    export type ColumnEditCellTemplateData<TRowData = any, TKey = any> = {
      readonly setValue?: any;
      readonly data: TRowData;
      readonly component: dxTreeList<TRowData, TKey>;
      readonly value?: any;
      readonly displayValue?: any;
      readonly text: string;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly column: Column<TRowData, TKey>;
      readonly row: Row<TRowData, TKey>;
      readonly rowType: string;
      readonly watch?: Function;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type ColumnHeaderCellTemplateData<TRowData = any, TKey = any> = {
      readonly component: dxTreeList<TRowData, TKey>;
      readonly columnIndex: number;
      readonly column: Column<TRowData, TKey>;
    };
    export type ContentReadyEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>>;
    export type ContextMenuPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
      items?: Array<any>;
      readonly target: string;
      readonly targetElement: DevExpress.core.DxElement;
      readonly columnIndex: number;
      readonly column?: Column<TRowData, TKey>;
      readonly rowIndex: number;
      readonly row?: Row<TRowData, TKey>;
    };
    export type DataErrorOccurredEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.DataErrorOccurredInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type DefaultToolbarItemName =
      | 'addRowButton'
      | 'applyFilterButton'
      | 'columnChooserButton'
      | 'revertButton'
      | 'saveButton'
      | 'searchPanel';
    export type DisposingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type dxTreeListToolbar = Toolbar;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type dxTreeListToolbarItem = ToolbarItem;
    export type EditCanceledEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.DataChangeInfo<TRowData, TKey>;
    export type EditCancelingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.DataChangeInfo<TRowData, TKey>;
    export interface Editing<TRowData = any, TKey = any>
      extends DevExpress.ui.dxDataGrid.EditingBase<TRowData, TKey> {
      /**
       * [descr:dxTreeListOptions.editing.allowAdding]
       */
      allowAdding?:
        | boolean
        | ((options: {
            readonly component: dxTreeList<TRowData, TKey>;
            readonly row?: Row<TRowData, TKey>;
          }) => boolean);
      /**
       * [descr:dxTreeListOptions.editing.allowDeleting]
       */
      allowDeleting?:
        | boolean
        | ((options: {
            readonly component: dxTreeList<TRowData, TKey>;
            readonly row?: Row<TRowData, TKey>;
          }) => boolean);
      /**
       * [descr:dxTreeListOptions.editing.allowUpdating]
       */
      allowUpdating?:
        | boolean
        | ((options: {
            readonly component: dxTreeList<TRowData, TKey>;
            readonly row?: Row<TRowData, TKey>;
          }) => boolean);
      /**
       * [descr:dxTreeListOptions.editing.texts]
       */
      texts?: EditingTexts;
    }
    export type EditingStartEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
        readonly data: TRowData;
        readonly key: TKey;
        readonly column: Column<TRowData, TKey>;
      };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface EditingTexts
      extends DevExpress.ui.dxDataGrid.EditingTextsBase {
      /**
       * [descr:dxTreeListOptions.editing.texts.addRowToNode]
       */
      addRowToNode?: string;
    }
    export type EditorPreparedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
      readonly parentType: string;
      readonly value?: any;
      readonly setValue?: any;
      readonly updateValueTimeout?: number;
      readonly width?: number;
      readonly disabled: boolean;
      readonly rtlEnabled: boolean;
      readonly editorElement: DevExpress.core.DxElement;
      readonly readOnly: boolean;
      readonly dataField?: string;
      readonly row?: Row<TRowData, TKey>;
    };
    export type EditorPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
        readonly parentType: string;
        readonly value?: any;
        readonly setValue?: any;
        updateValueTimeout?: number;
        readonly width?: number;
        readonly disabled: boolean;
        readonly rtlEnabled: boolean;
        readonly editorElement: DevExpress.core.DxElement;
        readonly readOnly: boolean;
        editorName: string;
        editorOptions: any;
        readonly dataField?: string;
        readonly row?: Row<TRowData, TKey>;
      };
    export type ExplicitTypes<TRowData, TKey> = {
      AdaptiveDetailRowPreparingEvent: AdaptiveDetailRowPreparingEvent<
        TRowData,
        TKey
      >;
      CellClickEvent: CellClickEvent<TRowData, TKey>;
      CellDblClickEvent: CellDblClickEvent<TRowData, TKey>;
      CellHoverChangedEvent: CellHoverChangedEvent<TRowData, TKey>;
      CellPreparedEvent: CellPreparedEvent<TRowData, TKey>;
      ColumnButtonClickEvent: ColumnButtonClickEvent<TRowData, TKey>;
      ColumnButtonTemplateData: ColumnButtonTemplateData<TRowData, TKey>;
      ColumnCellTemplateData: ColumnCellTemplateData<TRowData, TKey>;
      ColumnEditCellTemplateData: ColumnEditCellTemplateData<TRowData, TKey>;
      ContentReadyEvent: ContentReadyEvent<TRowData, TKey>;
      ContextMenuPreparingEvent: ContextMenuPreparingEvent<TRowData, TKey>;
      DataErrorOccurredEvent: DataErrorOccurredEvent<TRowData, TKey>;
      DisposingEvent: DisposingEvent<TRowData, TKey>;
      EditCanceledEvent: EditCanceledEvent<TRowData, TKey>;
      EditCancelingEvent: EditCancelingEvent<TRowData, TKey>;
      Editing: Editing<TRowData, TKey>;
      EditingStartEvent: EditingStartEvent<TRowData, TKey>;
      EditorPreparedEvent: EditorPreparedEvent<TRowData, TKey>;
      EditorPreparingEvent: EditorPreparingEvent<TRowData, TKey>;
      FocusedCellChangedEvent: FocusedCellChangedEvent<TRowData, TKey>;
      FocusedCellChangingEvent: FocusedCellChangingEvent<TRowData, TKey>;
      FocusedRowChangedEvent: FocusedRowChangedEvent<TRowData, TKey>;
      FocusedRowChangingEvent: FocusedRowChangingEvent<TRowData, TKey>;
      InitializedEvent: InitializedEvent<TRowData, TKey>;
      InitNewRowEvent: InitNewRowEvent<TRowData, TKey>;
      KeyDownEvent: KeyDownEvent<TRowData, TKey>;
      NodesInitializedEvent: NodesInitializedEvent<TRowData, TKey>;
      OptionChangedEvent: OptionChangedEvent<TRowData, TKey>;
      Properties: Properties<TRowData, TKey>;
      RowClickEvent: RowClickEvent<TRowData, TKey>;
      RowCollapsedEvent: RowCollapsedEvent<TRowData, TKey>;
      RowCollapsingEvent: RowCollapsingEvent<TRowData, TKey>;
      RowDblClickEvent: RowDblClickEvent<TRowData, TKey>;
      RowDraggingAddEvent: RowDraggingAddEvent<TRowData, TKey>;
      RowDraggingChangeEvent: RowDraggingChangeEvent<TRowData, TKey>;
      RowDraggingEndEvent: RowDraggingEndEvent<TRowData, TKey>;
      RowDraggingMoveEvent: RowDraggingMoveEvent<TRowData, TKey>;
      RowDraggingRemoveEvent: RowDraggingRemoveEvent<TRowData, TKey>;
      RowDraggingReorderEvent: RowDraggingReorderEvent<TRowData, TKey>;
      RowDraggingStartEvent: RowDraggingStartEvent<TRowData, TKey>;
      RowDraggingTemplateData: RowDraggingTemplateData<TRowData>;
      RowExpandedEvent: RowExpandedEvent<TRowData, TKey>;
      RowExpandingEvent: RowExpandingEvent<TRowData, TKey>;
      RowInsertedEvent: RowInsertedEvent<TRowData, TKey>;
      RowInsertingEvent: RowInsertingEvent<TRowData, TKey>;
      RowPreparedEvent: RowPreparedEvent<TRowData, TKey>;
      RowRemovedEvent: RowRemovedEvent<TRowData, TKey>;
      RowRemovingEvent: RowRemovingEvent<TRowData, TKey>;
      RowUpdatedEvent: RowUpdatedEvent<TRowData, TKey>;
      RowUpdatingEvent: RowUpdatingEvent<TRowData, TKey>;
      RowValidatingEvent: RowValidatingEvent<TRowData, TKey>;
      SavedEvent: SavedEvent<TRowData, TKey>;
      SavingEvent: SavingEvent<TRowData, TKey>;
      Scrolling: Scrolling;
      Selection: Selection;
      SelectionChangedEvent: SelectionChangedEvent<TRowData, TKey>;
      Toolbar: Toolbar;
      ToolbarItem: ToolbarItem;
      ToolbarPreparingEvent: ToolbarPreparingEvent<TRowData, TKey>;
    };
    export type FocusedCellChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
      readonly cellElement: DevExpress.core.DxElement;
      readonly columnIndex: number;
      readonly rowIndex: number;
      readonly row: Row<TRowData, TKey>;
      readonly column: Column<TRowData, TKey>;
    };
    export type FocusedCellChangingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> & {
        readonly cellElement: DevExpress.core.DxElement;
        readonly prevColumnIndex: number;
        readonly prevRowIndex: number;
        newColumnIndex: number;
        newRowIndex: number;
        readonly rows: Array<Row<TRowData, TKey>>;
        readonly columns: Array<Column<TRowData, TKey>>;
        isHighlighted: boolean;
      };
    export type FocusedRowChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
      readonly rowElement: DevExpress.core.DxElement;
      readonly rowIndex: number;
      readonly row: Row<TRowData, TKey>;
    };
    export type FocusedRowChangingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> & {
      readonly rowElement: DevExpress.core.DxElement;
      readonly prevRowIndex: number;
      newRowIndex: number;
      readonly rows: Array<Row<TRowData, TKey>>;
    };
    export type InitializedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<dxTreeList<TRowData, TKey>>;
    export type InitNewRowEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.NewRowInfo<TRowData>;
    export type KeyDownEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.KeyDownInfo;
    /**
     * [descr:dxTreeListNode]
     */
    export type Node<TRowData = any, TKey = any> = {
      /**
       * [descr:dxTreeListNode.children]
       */
      children?: Array<Node<TRowData, TKey>>;
      /**
       * [descr:dxTreeListNode.data]
       */
      data?: TRowData;
      /**
       * [descr:dxTreeListNode.hasChildren]
       */
      hasChildren?: boolean;
      /**
       * [descr:dxTreeListNode.key]
       */
      key: TKey;
      /**
       * [descr:dxTreeListNode.level]
       */
      level: number;
      /**
       * [descr:dxTreeListNode.parent]
       */
      parent?: Node<TRowData, TKey>;
      /**
       * [descr:dxTreeListNode.visible]
       */
      visible?: boolean;
    };
    export type NodesInitializedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
      readonly root: Node<TRowData, TKey>;
    };
    export type OptionChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface Paging extends DevExpress.ui.dxDataGrid.PagingBase {
      /**
       * [descr:dxTreeListOptions.paging.enabled]
       */
      enabled?: boolean;
    }
    export type Properties<TRowData = any, TKey = any> = dxTreeListOptions<
      TRowData,
      TKey
    >;
    /**
     * [descr:dxTreeListRowObject]
     */
    export type Row<TRowData = any, TKey = any> = {
      /**
       * [descr:dxTreeListRowObject.isEditing]
       */
      readonly isEditing?: boolean;
      /**
       * [descr:dxTreeListRowObject.isExpanded]
       */
      readonly isExpanded?: boolean;
      /**
       * [descr:dxTreeListRowObject.isNewRow]
       */
      readonly isNewRow?: boolean;
      /**
       * [descr:dxTreeListRowObject.isSelected]
       */
      readonly isSelected?: boolean;
      /**
       * [descr:dxTreeListRowObject.key]
       */
      readonly key: TKey;
      /**
       * [descr:dxTreeListRowObject.level]
       */
      readonly level: number;
      /**
       * [descr:dxTreeListRowObject.node]
       */
      readonly node: Node<TRowData, TKey>;
      /**
       * [descr:dxTreeListRowObject.rowIndex]
       */
      readonly rowIndex: number;
      /**
       * [descr:dxTreeListRowObject.rowType]
       */
      readonly rowType: string;
      /**
       * [descr:dxTreeListRowObject.values]
       */
      readonly values: Array<any>;
      /**
       * [descr:dxTreeListRowObject.data]
       */
      readonly data: TRowData;
    };
    export type RowClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly values: Array<any>;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
      readonly isNewRow?: boolean;
      readonly rowElement: DevExpress.core.DxElement;
      readonly handled: boolean;
      readonly node: Node<TRowData, TKey>;
      readonly level: number;
    };
    export type RowCollapsedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>;
    export type RowCollapsingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>;
    export type RowDblClickEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<dxTreeList<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly values: Array<any>;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
      readonly isNewRow?: boolean;
      readonly rowElement: DevExpress.core.DxElement;
    };
    export type RowDraggingAddEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.ui.dxDataGrid.RowDraggingEventInfo<
      dxTreeList<TRowData, TKey>,
      TRowData,
      TKey
    > &
      DevExpress.ui.dxDataGrid.DragDropInfo;
    export type RowDraggingChangeEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.ui.dxDataGrid.RowDraggingEventInfo<
        dxTreeList<TRowData, TKey>,
        TRowData,
        TKey
      > &
      DevExpress.ui.dxDataGrid.DragDropInfo;
    export type RowDraggingEndEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.ui.dxDataGrid.RowDraggingEventInfo<
        dxTreeList<TRowData, TKey>,
        TRowData,
        TKey
      > &
      DevExpress.ui.dxDataGrid.DragDropInfo;
    export type RowDraggingMoveEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.ui.dxDataGrid.RowDraggingEventInfo<
        dxTreeList<TRowData, TKey>,
        TRowData,
        TKey
      > &
      DevExpress.ui.dxDataGrid.DragDropInfo;
    export type RowDraggingRemoveEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.ui.dxDataGrid.RowDraggingEventInfo<
      dxTreeList<TRowData, TKey>,
      TRowData,
      TKey
    >;
    export type RowDraggingReorderEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.ui.dxDataGrid.RowDraggingEventInfo<
      dxTreeList<TRowData, TKey>,
      TRowData,
      TKey
    > &
      DevExpress.ui.dxDataGrid.DragReorderInfo;
    export type RowDraggingStartEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.ui.dxDataGrid.DragStartEventInfo<
        dxTreeList<TRowData, TKey>,
        TRowData,
        TKey
      >;
    export type RowDraggingTemplateData<TRowData = any> =
      DevExpress.ui.dxDataGrid.RowDraggingTemplateDataModel<TRowData>;
    export type RowExpandedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>;
    export type RowExpandingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.Cancelable &
      DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>;
    export type RowInsertedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowInsertedInfo<TRowData, TKey>;
    export type RowInsertingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowInsertingInfo<TRowData>;
    export type RowPreparedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> & {
      readonly data: TRowData;
      readonly key: TKey;
      readonly values: Array<any>;
      readonly columns: Array<Column<TRowData, TKey>>;
      readonly rowIndex: number;
      readonly rowType: string;
      readonly isSelected?: boolean;
      readonly isExpanded?: boolean;
      readonly isNewRow?: boolean;
      readonly rowElement: DevExpress.core.DxElement;
      readonly node: Node<TRowData, TKey>;
      readonly level: number;
    };
    export type RowRemovedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowRemovedInfo<TRowData, TKey>;
    export type RowRemovingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowRemovingInfo<TRowData, TKey>;
    export type RowUpdatedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowUpdatedInfo<TRowData, TKey>;
    export type RowUpdatingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowUpdatingInfo<TRowData, TKey>;
    export type RowValidatingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.RowValidatingInfo<TRowData, TKey>;
    export type SavedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.DataChangeInfo<TRowData, TKey>;
    export type SavingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.SavingInfo<TRowData, TKey>;
    export interface Scrolling extends DevExpress.ui.dxDataGrid.ScrollingBase {
      /**
       * [descr:dxTreeListOptions.scrolling.mode]
       */
      mode?: 'standard' | 'virtual';
    }
    export interface Selection extends DevExpress.ui.dxDataGrid.SelectionBase {
      /**
       * [descr:dxTreeListOptions.selection.recursive]
       */
      recursive?: boolean;
    }
    export type SelectionChangedEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.SelectionChangedInfo<TRowData, TKey>;
    export type ToolbarPreparingEvent<
      TRowData = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxTreeList<TRowData, TKey>> &
      DevExpress.ui.dxDataGrid.ToolbarPreparingInfo;
  }
  /**
   * @deprecated Use the DevExpress.ui.dxTreeList.Column type instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTreeListColumn<TRowData = any, TKey = any>
    extends DevExpress.ui.dxDataGrid.ColumnBase<TRowData> {
    /**
     * [descr:dxTreeListColumn.buttons]
     */
    buttons?: Array<
      | 'add'
      | 'cancel'
      | 'delete'
      | 'edit'
      | 'save'
      | 'undelete'
      | DevExpress.ui.dxTreeList.ColumnButton<TRowData, TKey>
    >;
    /**
     * [descr:dxTreeListColumn.cellTemplate]
     */
    cellTemplate?:
      | DevExpress.core.template
      | ((
          cellElement: DevExpress.core.DxElement,
          cellInfo: DevExpress.ui.dxTreeList.ColumnCellTemplateData<
            TRowData,
            TKey
          >
        ) => any);
    /**
     * [descr:dxTreeListColumn.columns]
     */
    columns?: Array<DevExpress.ui.dxTreeList.Column<TRowData, TKey> | string>;
    /**
     * [descr:dxTreeListColumn.editCellTemplate]
     */
    editCellTemplate?:
      | DevExpress.core.template
      | ((
          cellElement: DevExpress.core.DxElement,
          cellInfo: DevExpress.ui.dxTreeList.ColumnEditCellTemplateData<
            TRowData,
            TKey
          >
        ) => any);
    /**
     * [descr:dxTreeListColumn.headerCellTemplate]
     */
    headerCellTemplate?:
      | DevExpress.core.template
      | ((
          columnHeader: DevExpress.core.DxElement,
          headerInfo: DevExpress.ui.dxTreeList.ColumnHeaderCellTemplateData<
            TRowData,
            TKey
          >
        ) => any);
    /**
     * [descr:dxTreeListColumn.type]
     */
    type?: 'adaptive' | 'buttons' | 'drag';
  }
  /**
   * @deprecated Use the TreeList's ColumnButton type instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTreeListColumnButton<TRowData = any, TKey = any>
    extends DevExpress.ui.dxDataGrid.ColumnButtonBase {
    /**
     * [descr:dxTreeListColumnButton.name]
     */
    name?: 'add' | 'cancel' | 'delete' | 'edit' | 'save' | 'undelete' | string;
    /**
     * [descr:dxTreeListColumnButton.onClick]
     */
    onClick?: (
      e: DevExpress.ui.dxTreeList.ColumnButtonClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListColumnButton.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          cellElement: DevExpress.core.DxElement,
          cellInfo: DevExpress.ui.dxTreeList.ColumnButtonTemplateData<
            TRowData,
            TKey
          >
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxTreeListColumnButton.visible]
     */
    visible?:
      | boolean
      | ((options: {
          readonly component: dxTreeList<TRowData, TKey>;
          readonly row?: DevExpress.ui.dxTreeList.Row<TRowData, TKey>;
          readonly column: DevExpress.ui.dxTreeList.Column<TRowData, TKey>;
        }) => boolean);
    /**
     * [descr:dxTreeListColumnButton.disabled]
     */
    disabled?:
      | boolean
      | ((options: {
          readonly component: dxTreeList<TRowData, TKey>;
          readonly row?: DevExpress.ui.dxTreeList.Row<TRowData, TKey>;
          readonly column: DevExpress.ui.dxTreeList.Column<TRowData, TKey>;
        }) => boolean);
  }
  /**
   * @deprecated Use DevExpress.ui.dxTreeList.Editing instead
   */
  export type dxTreeListEditing<
    TRowData = any,
    TKey = any
  > = DevExpress.ui.dxTreeList.Editing<TRowData, TKey>;
  /**
   * @deprecated 
   */
  export type dxTreeListEditingTexts = DevExpress.ui.dxTreeList.EditingTexts;
  /**
   * @deprecated Use DevExpress.ui.dxTreeList.Node instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type dxTreeListNode<
    TRowData = any,
    TKey = any
  > = DevExpress.ui.dxTreeList.Node<TRowData, TKey>;
  /**
   * @deprecated use Properties instead
   */
  export interface dxTreeListOptions<TRowData = any, TKey = any>
    extends GridBaseOptions<dxTreeList<TRowData, TKey>, TRowData, TKey> {
    /**
     * [descr:dxTreeListOptions.autoExpandAll]
     */
    autoExpandAll?: boolean;
    /**
     * [descr:dxTreeListOptions.columns]
     */
    columns?: Array<DevExpress.ui.dxTreeList.Column<TRowData, TKey> | string>;
    /**
     * [descr:dxTreeListOptions.customizeColumns]
     */
    customizeColumns?: (
      columns: Array<DevExpress.ui.dxTreeList.Column<TRowData, TKey>>
    ) => void;
    /**
     * [descr:dxTreeListOptions.dataStructure]
     */
    dataStructure?: 'plain' | 'tree';
    /**
     * [descr:dxTreeListOptions.editing]
     */
    editing?: DevExpress.ui.dxTreeList.Editing<TRowData, TKey>;
    /**
     * [descr:dxTreeListOptions.expandNodesOnFiltering]
     */
    expandNodesOnFiltering?: boolean;
    /**
     * [descr:dxTreeListOptions.expandedRowKeys]
     */
    expandedRowKeys?: Array<TKey>;
    /**
     * [descr:dxTreeListOptions.filterMode]
     */
    filterMode?: 'fullBranch' | 'withAncestors' | 'matchOnly';
    /**
     * [descr:dxTreeListOptions.hasItemsExpr]
     */
    hasItemsExpr?: string | Function;
    /**
     * [descr:dxTreeListOptions.itemsExpr]
     */
    itemsExpr?: string | Function;
    /**
     * [descr:dxTreeListOptions.keyExpr]
     */
    keyExpr?: string | Function;
    /**
     * [descr:dxTreeListOptions.onCellClick]
     */
    onCellClick?: (
      e: DevExpress.ui.dxTreeList.CellClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onCellDblClick]
     */
    onCellDblClick?: (
      e: DevExpress.ui.dxTreeList.CellDblClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onCellHoverChanged]
     */
    onCellHoverChanged?: (
      e: DevExpress.ui.dxTreeList.CellHoverChangedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onCellPrepared]
     */
    onCellPrepared?: (
      e: DevExpress.ui.dxTreeList.CellPreparedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onContextMenuPreparing]
     */
    onContextMenuPreparing?: (
      e: DevExpress.ui.dxTreeList.ContextMenuPreparingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onEditingStart]
     */
    onEditingStart?: (
      e: DevExpress.ui.dxTreeList.EditingStartEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onEditorPrepared]
     */
    onEditorPrepared?: (
      options: DevExpress.ui.dxTreeList.EditorPreparedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onEditorPreparing]
     */
    onEditorPreparing?: (
      e: DevExpress.ui.dxTreeList.EditorPreparingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onFocusedCellChanged]
     */
    onFocusedCellChanged?: (
      e: DevExpress.ui.dxTreeList.FocusedCellChangedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onFocusedCellChanging]
     */
    onFocusedCellChanging?: (
      e: DevExpress.ui.dxTreeList.FocusedCellChangingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onFocusedRowChanged]
     */
    onFocusedRowChanged?: (
      e: DevExpress.ui.dxTreeList.FocusedRowChangedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onFocusedRowChanging]
     */
    onFocusedRowChanging?: (
      e: DevExpress.ui.dxTreeList.FocusedRowChangingEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onNodesInitialized]
     */
    onNodesInitialized?: (
      e: DevExpress.ui.dxTreeList.NodesInitializedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onRowClick]
     */
    onRowClick?: (
      e: DevExpress.ui.dxTreeList.RowClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onRowDblClick]
     */
    onRowDblClick?: (
      e: DevExpress.ui.dxTreeList.RowDblClickEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.onRowPrepared]
     */
    onRowPrepared?: (
      e: DevExpress.ui.dxTreeList.RowPreparedEvent<TRowData, TKey>
    ) => void;
    /**
     * [descr:dxTreeListOptions.paging]
     */
    paging?: DevExpress.ui.dxTreeList.Paging;
    /**
     * [descr:dxTreeListOptions.parentIdExpr]
     */
    parentIdExpr?: string | Function;
    /**
     * [descr:dxTreeListOptions.remoteOperations]
     */
    remoteOperations?:
      | {
          /**
           * [descr:dxTreeListOptions.remoteOperations.filtering]
           */
          filtering?: boolean;
          /**
           * [descr:dxTreeListOptions.remoteOperations.grouping]
           */
          grouping?: boolean;
          /**
           * [descr:dxTreeListOptions.remoteOperations.sorting]
           */
          sorting?: boolean;
        }
      | 'auto';
    /**
     * [descr:dxTreeListOptions.rootValue]
     */
    rootValue?: TKey;
    /**
     * [descr:dxTreeListOptions.scrolling]
     */
    scrolling?: DevExpress.ui.dxTreeList.Scrolling;
    /**
     * [descr:dxTreeListOptions.selection]
     */
    selection?: DevExpress.ui.dxTreeList.Selection;
    /**
     * [descr:dxTreeListOptions.toolbar]
     */
    toolbar?: DevExpress.ui.dxTreeList.Toolbar;
  }
  /**
   * @deprecated 
   */
  export type dxTreeListPaging = DevExpress.ui.dxTreeList.Paging;
  /**
   * @deprecated Use DevExpress.ui.dxTreeList.Scrolling instead
   */
  export type dxTreeListScrolling = DevExpress.ui.dxTreeList.Scrolling;
  /**
   * @deprecated Use DevExpress.ui.dxTreeList.Selection instead
   */
  export type dxTreeListSelection = DevExpress.ui.dxTreeList.Selection;
  /**
   * [descr:dxTreeView]
   */
  export class dxTreeView<TKey = any> extends HierarchicalCollectionWidget<
    dxTreeViewOptions<TKey>,
    dxTreeViewItem,
    TKey
  > {
    /**
     * [descr:dxTreeView.collapseAll()]
     */
    collapseAll(): void;
    /**
     * [descr:dxTreeView.collapseItem(itemData)]
     */
    collapseItem(
      itemData: DevExpress.ui.dxTreeView.Item
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.collapseItem(itemElement)]
     */
    collapseItem(itemElement: Element): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.collapseItem(key)]
     */
    collapseItem(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.expandAll()]
     */
    expandAll(): void;
    /**
     * [descr:dxTreeView.expandItem(itemData)]
     */
    expandItem(
      itemData: DevExpress.ui.dxTreeView.Item
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.expandItem(itemElement)]
     */
    expandItem(itemElement: Element): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.expandItem(key)]
     */
    expandItem(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.getNodes()]
     */
    getNodes(): Array<DevExpress.ui.dxTreeView.Node<TKey>>;
    /**
     * [descr:dxTreeView.getSelectedNodes()]
     */
    getSelectedNodes(): Array<DevExpress.ui.dxTreeView.Node<TKey>>;
    /**
     * [descr:dxTreeView.getSelectedNodeKeys()]
     */
    getSelectedNodeKeys(): Array<TKey>;
    /**
     * [descr:dxTreeView.selectAll()]
     */
    selectAll(): void;
    /**
     * [descr:dxTreeView.selectItem(itemData)]
     */
    selectItem(itemData: DevExpress.ui.dxTreeView.Item): boolean;
    /**
     * [descr:dxTreeView.selectItem(itemElement)]
     */
    selectItem(itemElement: Element): boolean;
    /**
     * [descr:dxTreeView.selectItem(key)]
     */
    selectItem(key: TKey): boolean;
    /**
     * [descr:dxTreeView.unselectAll()]
     */
    unselectAll(): void;
    /**
     * [descr:dxTreeView.unselectItem(itemData)]
     */
    unselectItem(itemData: DevExpress.ui.dxTreeView.Item): boolean;
    /**
     * [descr:dxTreeView.unselectItem(itemElement)]
     */
    unselectItem(itemElement: Element): boolean;
    /**
     * [descr:dxTreeView.unselectItem(key)]
     */
    unselectItem(key: TKey): boolean;
    /**
     * [descr:dxTreeView.updateDimensions()]
     */
    updateDimensions(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.scrollToItem(itemData)]
     */
    scrollToItem(
      itemData: DevExpress.ui.dxTreeView.Item
    ): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.scrollToItem(itemElement)]
     */
    scrollToItem(itemElement: Element): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:dxTreeView.scrollToItem(key)]
     */
    scrollToItem(key: TKey): DevExpress.core.utils.DxPromise<void>;
  }
  module dxTreeView {
    export type ContentReadyEvent<TKey = any> = DevExpress.events.EventInfo<
      dxTreeView<TKey>
    >;
    export type DisposingEvent<TKey = any> = DevExpress.events.EventInfo<
      dxTreeView<TKey>
    >;
    export type ExplicitTypes<TKey = any> = {
      Properties: Properties<TKey>;
      Node: Node<TKey>;
      ContentReadyEvent: ContentReadyEvent<TKey>;
      DisposingEvent: DisposingEvent<TKey>;
      InitializedEvent: InitializedEvent<TKey>;
      ItemClickEvent: ItemClickEvent<TKey>;
      ItemCollapsedEvent: ItemCollapsedEvent<TKey>;
      ItemContextMenuEvent: ItemContextMenuEvent<TKey>;
      ItemExpandedEvent: ItemExpandedEvent<TKey>;
      ItemHoldEvent: ItemHoldEvent<TKey>;
      ItemRenderedEvent: ItemRenderedEvent<TKey>;
      ItemSelectionChangedEvent: ItemSelectionChangedEvent<TKey>;
      OptionChangedEvent: OptionChangedEvent<TKey>;
      SelectAllValueChangedEvent: SelectAllValueChangedEvent<TKey>;
      SelectionChangedEvent: SelectionChangedEvent<TKey>;
    };
    export type InitializedEvent<TKey = any> =
      DevExpress.events.InitializedEventInfo<dxTreeView<TKey>>;
    export type ItemClickEvent<TKey = any> = DevExpress.events.NativeEventInfo<
      dxTreeView<TKey>
    > &
      ItemInfo<TKey>;
    export type ItemCollapsedEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxTreeView<TKey>> & ItemInfo<TKey>;
    export type ItemContextMenuEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxTreeView<TKey>> & ItemInfo<TKey>;
    export type ItemExpandedEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxTreeView<TKey>> & ItemInfo<TKey>;
    export type ItemHoldEvent<TKey = any> = DevExpress.events.NativeEventInfo<
      dxTreeView<TKey>
    > &
      ItemInfo<TKey>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface ItemInfo<TKey = any> {
      readonly itemData?: Item;
      readonly itemElement?: DevExpress.core.DxElement;
      readonly itemIndex?: number;
      readonly node?: Node<TKey>;
    }
    export type ItemRenderedEvent<TKey = any> =
      DevExpress.events.NativeEventInfo<dxTreeView<TKey>> & ItemInfo<TKey>;
    export type ItemSelectionChangedEvent<TKey = any> =
      DevExpress.events.EventInfo<dxTreeView<TKey>> & ItemInfo<TKey>;
    export type Node<TKey = any> = dxTreeViewNode<TKey>;
    export type OptionChangedEvent<TKey = any> = DevExpress.events.EventInfo<
      dxTreeView<TKey>
    > &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<TKey = any> = dxTreeViewOptions<TKey>;
    export type SelectAllValueChangedEvent<TKey = any> =
      DevExpress.events.EventInfo<dxTreeView<TKey>> & {
        readonly value?: boolean | undefined;
      };
    export type SelectionChangedEvent<TKey = any> = DevExpress.events.EventInfo<
      dxTreeView<TKey>
    >;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTreeViewItem extends CollectionWidgetItem {
    /**
     * [descr:dxTreeViewItem.expanded]
     */
    expanded?: boolean;
    /**
     * [descr:dxTreeViewItem.hasItems]
     */
    hasItems?: boolean;
    /**
     * [descr:dxTreeViewItem.icon]
     */
    icon?: string;
    /**
     * [descr:dxTreeViewItem.items]
     */
    items?: Array<dxTreeViewItem>;
    /**
     * [descr:dxTreeViewItem.id]
     */
    id?: number | string;
    /**
     * [descr:dxTreeViewItem.parentId]
     */
    parentId?: number | string;
    /**
     * [descr:dxTreeViewItem.selected]
     */
    selected?: boolean;
  }
  /**
   * [descr:dxTreeViewNode]
   * @deprecated [depNote:dxTreeViewNode]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTreeViewNode<TKey = any> {
    /**
     * [descr:dxTreeViewNode.children]
     */
    children?: Array<dxTreeViewNode<TKey>>;
    /**
     * [descr:dxTreeViewNode.disabled]
     */
    disabled?: boolean;
    /**
     * [descr:dxTreeViewNode.expanded]
     */
    expanded?: boolean;
    /**
     * [descr:dxTreeViewNode.itemData]
     */
    itemData?: DevExpress.ui.dxTreeView.Item;
    /**
     * [descr:dxTreeViewNode.key]
     */
    key?: TKey;
    /**
     * [descr:dxTreeViewNode.parent]
     */
    parent?: dxTreeViewNode<TKey>;
    /**
     * [descr:dxTreeViewNode.selected]
     */
    selected?: boolean;
    /**
     * [descr:dxTreeViewNode.text]
     */
    text?: string;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxTreeViewOptions<TKey = any>
    extends DevExpress.core.Skip<
        HierarchicalCollectionWidgetOptions<
          dxTreeView<TKey>,
          dxTreeViewItem,
          TKey
        >,
        'dataSource'
      >,
      SearchBoxMixinOptions {
    /**
     * [descr:dxTreeViewOptions.animationEnabled]
     */
    animationEnabled?: boolean;
    /**
     * [descr:dxTreeViewOptions.createChildren]
     */
    createChildren?: (
      parentNode: DevExpress.ui.dxTreeView.Node<TKey>
    ) => PromiseLike<any> | Array<any>;
    /**
     * [descr:dxTreeViewOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<
      DevExpress.ui.dxTreeView.Item,
      TKey
    >;
    /**
     * [descr:dxTreeViewOptions.dataStructure]
     */
    dataStructure?: 'plain' | 'tree';
    /**
     * [descr:dxTreeViewOptions.expandAllEnabled]
     */
    expandAllEnabled?: boolean;
    /**
     * [descr:dxTreeViewOptions.expandEvent]
     */
    expandEvent?: 'dblclick' | 'click';
    /**
     * [descr:dxTreeViewOptions.expandNodesRecursive]
     */
    expandNodesRecursive?: boolean;
    /**
     * [descr:dxTreeViewOptions.expandedExpr]
     */
    expandedExpr?: string | Function;
    /**
     * [descr:dxTreeViewOptions.hasItemsExpr]
     */
    hasItemsExpr?: string | Function;
    /**
     * [descr:dxTreeViewOptions.items]
     */
    items?: Array<DevExpress.ui.dxTreeView.Item>;
    /**
     * [descr:dxTreeViewOptions.onItemClick]
     */
    onItemClick?: (e: DevExpress.ui.dxTreeView.ItemClickEvent<TKey>) => void;
    /**
     * [descr:dxTreeViewOptions.onItemCollapsed]
     */
    onItemCollapsed?: (
      e: DevExpress.ui.dxTreeView.ItemCollapsedEvent<TKey>
    ) => void;
    /**
     * [descr:dxTreeViewOptions.onItemContextMenu]
     */
    onItemContextMenu?: (
      e: DevExpress.ui.dxTreeView.ItemContextMenuEvent<TKey>
    ) => void;
    /**
     * [descr:dxTreeViewOptions.onItemExpanded]
     */
    onItemExpanded?: (
      e: DevExpress.ui.dxTreeView.ItemExpandedEvent<TKey>
    ) => void;
    /**
     * [descr:dxTreeViewOptions.onItemHold]
     */
    onItemHold?: (e: DevExpress.ui.dxTreeView.ItemHoldEvent<TKey>) => void;
    /**
     * [descr:dxTreeViewOptions.onItemRendered]
     */
    onItemRendered?: (
      e: DevExpress.ui.dxTreeView.ItemRenderedEvent<TKey>
    ) => void;
    /**
     * [descr:dxTreeViewOptions.onItemSelectionChanged]
     */
    onItemSelectionChanged?: (
      e: DevExpress.ui.dxTreeView.ItemSelectionChangedEvent<TKey>
    ) => void;
    /**
     * [descr:dxTreeViewOptions.onSelectAllValueChanged]
     */
    onSelectAllValueChanged?: (
      e: DevExpress.ui.dxTreeView.SelectAllValueChangedEvent<TKey>
    ) => void;
    /**
     * [descr:dxTreeViewOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.ui.dxTreeView.SelectionChangedEvent<TKey>
    ) => void;
    /**
     * [descr:dxTreeViewOptions.parentIdExpr]
     */
    parentIdExpr?: string | Function;
    /**
     * [descr:dxTreeViewOptions.rootValue]
     */
    rootValue?: any;
    /**
     * [descr:dxTreeViewOptions.scrollDirection]
     */
    scrollDirection?: 'both' | 'horizontal' | 'vertical';
    /**
     * [descr:dxTreeViewOptions.selectAllText]
     */
    selectAllText?: string;
    /**
     * [descr:dxTreeViewOptions.selectByClick]
     */
    selectByClick?: boolean;
    /**
     * [descr:dxTreeViewOptions.selectNodesRecursive]
     */
    selectNodesRecursive?: boolean;
    /**
     * [descr:dxTreeViewOptions.selectionMode]
     */
    selectionMode?: 'multiple' | 'single';
    /**
     * [descr:dxTreeViewOptions.showCheckBoxesMode]
     */
    showCheckBoxesMode?: 'none' | 'normal' | 'selectAll';
    /**
     * [descr:dxTreeViewOptions.virtualModeEnabled]
     */
    virtualModeEnabled?: boolean;
    /**
     * [descr:dxTreeViewOptions.useNativeScrolling]
     */
    useNativeScrolling?: boolean;
  }
  /**
   * [descr:dxValidationGroup]
   */
  export class dxValidationGroup extends DOMComponent<dxValidationGroupOptions> {
    /**
     * [descr:dxValidationGroup.reset()]
     */
    reset(): void;
    /**
     * [descr:dxValidationGroup.validate()]
     */
    validate(): DevExpress.ui.dxValidationGroup.ValidationResult;
  }
  module dxValidationGroup {
    export type DisposingEvent = DevExpress.events.EventInfo<dxValidationGroup>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxValidationGroup>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxValidationGroup> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxValidationGroupOptions;
    export type ValidationResult = dxValidationGroupResult;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxValidationGroupOptions
    extends DOMComponentOptions<dxValidationGroup> {}
  /**
   * [descr:dxValidationGroupResult]
   * @deprecated [depNote:dxValidationGroupResult]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxValidationGroupResult {
    /**
     * [descr:dxValidationGroupResult.brokenRules]
     */
    brokenRules?: Array<
      | RequiredRule
      | NumericRule
      | RangeRule
      | StringLengthRule
      | CustomRule
      | CompareRule
      | PatternRule
      | EmailRule
      | AsyncRule
    >;
    /**
     * [descr:dxValidationGroupResult.complete]
     */
    complete?: DevExpress.core.utils.DxPromise<dxValidationGroupResult>;
    /**
     * [descr:dxValidationGroupResult.isValid]
     */
    isValid?: boolean;
    /**
     * [descr:dxValidationGroupResult.status]
     */
    status?: 'valid' | 'invalid' | 'pending';
    /**
     * [descr:dxValidationGroupResult.validators]
     */
    validators?: Array<any>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class dxValidationMessage extends dxOverlay<dxValidationMessageOptions> {}
  module dxValidationMessage {
    export type Properties = dxValidationMessageOptions;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxValidationMessageOptions
    extends dxOverlayOptions<dxValidationMessage> {
    mode?: string;

    validationErrors?: Array<object> | null;

    positionRequest?: string;

    boundary?: String | DevExpress.core.UserDefinedElement;

    offset?: object;
  }
  /**
   * [descr:dxValidationSummary]
   */
  export class dxValidationSummary<
    TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
    TKey = any
  > extends CollectionWidget<
    dxValidationSummaryOptions<TItem, TKey>,
    TItem,
    TKey
  > {}
  module dxValidationSummary {
    export type ContentReadyEvent<
      TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxValidationSummary<TItem, TKey>>;
    export type DisposingEvent<
      TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxValidationSummary<TItem, TKey>>;
    export type ExplicitTypes<
      TItem extends DevExpress.ui.CollectionWidget.ItemLike,
      TKey
    > = {
      Properties: Properties<TItem, TKey>;
      ContentReadyEvent: ContentReadyEvent<TItem, TKey>;
      DisposingEvent: DisposingEvent<TItem, TKey>;
      InitializedEvent: InitializedEvent<TItem, TKey>;
      ItemClickEvent: ItemClickEvent<TItem, TKey>;
      OptionChangedEvent: OptionChangedEvent<TItem, TKey>;
    };
    export type InitializedEvent<
      TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
      TKey = any
    > = DevExpress.events.InitializedEventInfo<
      dxValidationSummary<TItem, TKey>
    >;
    export type ItemClickEvent<
      TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
      TKey = any
    > = DevExpress.events.NativeEventInfo<
      dxValidationSummary<TItem, TKey>,
      MouseEvent | PointerEvent
    > &
      DevExpress.events.ItemInfo<TItem>;
    export type OptionChangedEvent<
      TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
      TKey = any
    > = DevExpress.events.EventInfo<dxValidationSummary<TItem, TKey>> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties<
      TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
      TKey = any
    > = dxValidationSummaryOptions<TItem, TKey>;
  }
  /**
   * @deprecated use Properties instead
   */
  export interface dxValidationSummaryOptions<
    TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<
      dxValidationSummary<TItem, TKey>,
      TItem,
      TKey
    > {
    /**
     * [descr:dxValidationSummaryOptions.validationGroup]
     */
    validationGroup?: string;
  }
  /**
   * [descr:dxValidator]
   */
  export class dxValidator extends DOMComponent<dxValidatorOptions> {
    /**
     * [descr:dxValidator.focus()]
     */
    focus(): void;
    /**
     * [descr:dxValidator.reset()]
     */
    reset(): void;
    /**
     * [descr:dxValidator.validate()]
     */
    validate(): DevExpress.ui.dxValidator.ValidationResult;
  }
  module dxValidator {
    export type DisposingEvent = DevExpress.events.EventInfo<dxValidator>;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxValidator>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxValidator> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxValidatorOptions;
    export type ValidatedEvent = {
      name?: string;
      isValid?: boolean;
      value?: any;
      validationRules?: Array<ValidationRule>;
      brokenRule?: ValidationRule;
      brokenRules?: ValidationRule;
      status?: 'valid' | 'invalid' | 'pending';
    };
    export type ValidationResult = dxValidatorResult;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxValidatorOptions extends DOMComponentOptions<dxValidator> {
    /**
     * [descr:dxValidatorOptions.adapter]
     */
    adapter?: {
      /**
       * [descr:dxValidatorOptions.adapter.applyValidationResults]
       */
      applyValidationResults?: Function;
      /**
       * [descr:dxValidatorOptions.adapter.bypass]
       */
      bypass?: Function;
      /**
       * [descr:dxValidatorOptions.adapter.focus]
       */
      focus?: Function;
      /**
       * [descr:dxValidatorOptions.adapter.getValue]
       */
      getValue?: Function;
      /**
       * [descr:dxValidatorOptions.adapter.reset]
       */
      reset?: Function;
      /**
       * [descr:dxValidatorOptions.adapter.validationRequestsCallbacks]
       */
      validationRequestsCallbacks?: Array<Function>;
    };
    /**
     * [descr:dxValidatorOptions.name]
     */
    name?: string;
    /**
     * [descr:dxValidatorOptions.onValidated]
     */
    onValidated?: (
      validatedInfo: DevExpress.ui.dxValidator.ValidatedEvent
    ) => void;
    /**
     * [descr:dxValidatorOptions.validationGroup]
     */
    validationGroup?: string;
    /**
     * [descr:dxValidatorOptions.validationRules]
     */
    validationRules?: Array<ValidationRule>;
  }
  /**
   * [descr:dxValidatorResult]
   * @deprecated [depNote:dxValidatorResult]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxValidatorResult {
    /**
     * [descr:dxValidatorResult.brokenRule]
     */
    brokenRule?: ValidationRule;
    /**
     * [descr:dxValidatorResult.brokenRules]
     */
    brokenRules?: Array<ValidationRule>;
    /**
     * [descr:dxValidatorResult.complete]
     */
    complete?: DevExpress.core.utils.DxPromise<dxValidatorResult>;
    /**
     * [descr:dxValidatorResult.isValid]
     */
    isValid?: boolean;
    /**
     * [descr:dxValidatorResult.pendingRules]
     */
    pendingRules?: Array<AsyncRule>;
    /**
     * [descr:dxValidatorResult.status]
     */
    status?: 'valid' | 'invalid' | 'pending';
    /**
     * [descr:dxValidatorResult.validationRules]
     */
    validationRules?: Array<ValidationRule>;
    /**
     * [descr:dxValidatorResult.value]
     */
    value?: any;
  }
  /**
   * [descr:Editor]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class Editor<
    TProperties = DevExpress.ui.Editor.Properties
  > extends Widget<TProperties> {
    /**
     * [descr:Editor.reset()]
     */
    reset(): void;
  }
  module Editor {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface EditorInstance extends Editor<Properties> {}
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    type Properties = EditorOptions<EditorInstance>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ValueChangedInfo {
      readonly previousValue?: any;
      readonly value?: any;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface EditorOptions<TComponent> extends WidgetOptions<TComponent> {
    /**
     * [descr:EditorOptions.isValid]
     */
    isValid?: boolean;
    /**
     * [descr:EditorOptions.onValueChanged]
     */
    onValueChanged?: (
      e: DevExpress.events.NativeEventInfo<
        TComponent,
        KeyboardEvent | MouseEvent | PointerEvent | TouchEvent | Event
      > &
        DevExpress.ui.Editor.ValueChangedInfo
    ) => void;
    /**
     * [descr:EditorOptions.readOnly]
     */
    readOnly?: boolean;
    /**
     * [descr:EditorOptions.validationError]
     */
    validationError?: any;
    /**
     * [descr:EditorOptions.validationErrors]
     */
    validationErrors?: Array<any>;
    /**
     * [descr:EditorOptions.validationMessageMode]
     */
    validationMessageMode?: 'always' | 'auto';
    /**
     * [descr:EditorOptions.validationStatus]
     */
    validationStatus?: 'valid' | 'invalid' | 'pending';
    /**
     * [descr:EditorOptions.value]
     */
    value?: any;
    /**
     * [descr:EditorOptions.stylingMode]
     */
    stylingMode?: 'outlined' | 'underlined' | 'filled';
  }
  /**
   * [descr:EmailRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface EmailRule {
    /**
     * [descr:EmailRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:EmailRule.message]
     */
    message?: string;
    /**
     * [descr:EmailRule.type]
     */
    type: 'email';
  }
  /**
   * [descr:Format]
   */
  export type Format =
    | FormatObject
    | PredefinedFormat
    | string
    | ((value: number | Date) => string)
    | ExternalFormat;
  /**
   * [descr:GridBase]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface GridBase<TRowData = any, TKey = any> {
    /**
     * [descr:GridBase.beginCustomLoading(messageText)]
     */
    beginCustomLoading(messageText: string): void;
    /**
     * [descr:GridBase.byKey(key)]
     */
    byKey(key: TKey): DevExpress.core.utils.DxPromise<TRowData>;
    /**
     * [descr:GridBase.cancelEditData()]
     */
    cancelEditData(): void;
    /**
     * [descr:GridBase.cellValue(rowIndex, dataField)]
     */
    cellValue(rowIndex: number, dataField: string): any;
    /**
     * [descr:GridBase.cellValue(rowIndex, dataField, value)]
     */
    cellValue(rowIndex: number, dataField: string, value: any): void;
    /**
     * [descr:GridBase.cellValue(rowIndex, visibleColumnIndex)]
     */
    cellValue(rowIndex: number, visibleColumnIndex: number): any;
    /**
     * [descr:GridBase.cellValue(rowIndex, visibleColumnIndex, value)]
     */
    cellValue(rowIndex: number, visibleColumnIndex: number, value: any): void;
    /**
     * [descr:GridBase.clearFilter()]
     */
    clearFilter(): void;
    /**
     * [descr:GridBase.clearFilter(filterName)]
     */
    clearFilter(filterName: string): void;
    /**
     * [descr:GridBase.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:GridBase.clearSorting()]
     */
    clearSorting(): void;
    /**
     * [descr:GridBase.closeEditCell()]
     */
    closeEditCell(): void;
    /**
     * [descr:GridBase.collapseAdaptiveDetailRow()]
     */
    collapseAdaptiveDetailRow(): void;
    /**
     * [descr:GridBase.columnCount()]
     */
    columnCount(): number;
    /**
     * [descr:GridBase.columnOption(id)]
     */
    columnOption(id: number | string): any;
    /**
     * [descr:GridBase.columnOption(id, optionName)]
     */
    columnOption(id: number | string, optionName: string): any;
    /**
     * [descr:GridBase.columnOption(id, optionName, optionValue)]
     */
    columnOption(
      id: number | string,
      optionName: string,
      optionValue: any
    ): void;
    /**
     * [descr:GridBase.columnOption(id, options)]
     */
    columnOption(id: number | string, options: any): void;
    /**
     * [descr:GridBase.deleteColumn(id)]
     */
    deleteColumn(id: number | string): void;
    /**
     * [descr:GridBase.deleteRow(rowIndex)]
     */
    deleteRow(rowIndex: number): void;
    /**
     * [descr:GridBase.deselectAll()]
     */
    deselectAll(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:GridBase.deselectRows(keys)]
     */
    deselectRows(keys: Array<any>): DevExpress.core.utils.DxPromise<any>;
    /**
     * [descr:GridBase.editCell(rowIndex, dataField)]
     */
    editCell(rowIndex: number, dataField: string): void;
    /**
     * [descr:GridBase.editCell(rowIndex, visibleColumnIndex)]
     */
    editCell(rowIndex: number, visibleColumnIndex: number): void;
    /**
     * [descr:GridBase.editRow(rowIndex)]
     */
    editRow(rowIndex: number): void;
    /**
     * [descr:GridBase.endCustomLoading()]
     */
    endCustomLoading(): void;
    /**
     * [descr:GridBase.expandAdaptiveDetailRow(key)]
     */
    expandAdaptiveDetailRow(key: TKey): void;
    /**
     * [descr:GridBase.filter()]
     */
    filter(): any;
    /**
     * [descr:GridBase.filter(filterExpr)]
     */
    filter(filterExpr: any): void;
    focus(): void;
    /**
     * [descr:GridBase.focus(element)]
     */
    focus(element: DevExpress.core.UserDefinedElement): void;
    /**
     * [descr:GridBase.getCellElement(rowIndex, dataField)]
     */
    getCellElement(
      rowIndex: number,
      dataField: string
    ): DevExpress.core.DxElement | undefined;
    /**
     * [descr:GridBase.getCellElement(rowIndex, visibleColumnIndex)]
     */
    getCellElement(
      rowIndex: number,
      visibleColumnIndex: number
    ): DevExpress.core.DxElement | undefined;
    /**
     * [descr:GridBase.getCombinedFilter()]
     */
    getCombinedFilter(): any;
    /**
     * [descr:GridBase.getCombinedFilter(returnDataField)]
     */
    getCombinedFilter(returnDataField: boolean): any;
    getDataSource(): DevExpress.data.DataSource<TRowData, TKey>;
    /**
     * [descr:GridBase.getKeyByRowIndex(rowIndex)]
     */
    getKeyByRowIndex(rowIndex: number): TKey | undefined;
    /**
     * [descr:GridBase.getRowElement(rowIndex)]
     */
    getRowElement(
      rowIndex: number
    ): DevExpress.core.UserDefinedElementsArray | undefined;
    /**
     * [descr:GridBase.getRowIndexByKey(key)]
     */
    getRowIndexByKey(key: TKey): number;
    /**
     * [descr:GridBase.getScrollable()]
     */
    getScrollable(): dxScrollable;
    /**
     * [descr:GridBase.getVisibleColumnIndex(id)]
     */
    getVisibleColumnIndex(id: number | string): number;
    /**
     * [descr:GridBase.hasEditData()]
     */
    hasEditData(): boolean;
    /**
     * [descr:GridBase.hideColumnChooser()]
     */
    hideColumnChooser(): void;
    /**
     * [descr:GridBase.isAdaptiveDetailRowExpanded(key)]
     */
    isAdaptiveDetailRowExpanded(key: TKey): boolean;
    /**
     * [descr:GridBase.isRowFocused(key)]
     */
    isRowFocused(key: TKey): boolean;
    /**
     * [descr:GridBase.isRowSelected(key)]
     */
    isRowSelected(key: TKey): boolean;
    /**
     * [descr:GridBase.keyOf(obj)]
     */
    keyOf(obj: TRowData): TKey;
    /**
     * [descr:GridBase.navigateToRow(key)]
     */
    navigateToRow(key: TKey): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:GridBase.pageCount()]
     */
    pageCount(): number;
    /**
     * [descr:GridBase.pageIndex()]
     */
    pageIndex(): number;
    /**
     * [descr:GridBase.pageIndex(newIndex)]
     */
    pageIndex(newIndex: number): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:GridBase.pageSize()]
     */
    pageSize(): number;
    /**
     * [descr:GridBase.pageSize(value)]
     */
    pageSize(value: number): void;
    /**
     * [descr:GridBase.refresh()]
     */
    refresh(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:GridBase.refresh(changesOnly)]
     */
    refresh(changesOnly: boolean): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:GridBase.repaintRows(rowIndexes)]
     */
    repaintRows(rowIndexes: Array<number>): void;
    /**
     * [descr:GridBase.saveEditData()]
     */
    saveEditData(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:GridBase.searchByText(text)]
     */
    searchByText(text: string): void;
    /**
     * [descr:GridBase.selectAll()]
     */
    selectAll(): DevExpress.core.utils.DxPromise<void>;
    /**
     * [descr:GridBase.selectRows(keys, preserve)]
     */
    selectRows(
      keys: Array<TKey>,
      preserve: boolean
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    /**
     * [descr:GridBase.selectRowsByIndexes(indexes)]
     */
    selectRowsByIndexes(
      indexes: Array<number>
    ): DevExpress.core.utils.DxPromise<Array<TRowData>>;
    /**
     * [descr:GridBase.showColumnChooser()]
     */
    showColumnChooser(): void;
    /**
     * [descr:GridBase.state()]
     */
    state(): any;
    /**
     * [descr:GridBase.state(state)]
     */
    state(state: any): void;
    /**
     * [descr:GridBase.undeleteRow(rowIndex)]
     */
    undeleteRow(rowIndex: number): void;
    /**
     * [descr:GridBase.updateDimensions()]
     */
    updateDimensions(): void;
  }
  /**
   * @deprecated 
   */
  export type GridBaseColumn<TRowData = any> =
    DevExpress.ui.dxDataGrid.ColumnBase<TRowData>;
  /**
   * @deprecated 
   */
  export type GridBaseColumnButton = DevExpress.ui.dxDataGrid.ColumnButtonBase;
  /**
   * @deprecated 
   */
  export type GridBaseEditing<
    TRowData = any,
    TKey = any
  > = DevExpress.ui.dxDataGrid.EditingBase<TRowData, TKey>;
  /**
   * @deprecated 
   */
  export type GridBaseEditingTexts = DevExpress.ui.dxDataGrid.EditingTextsBase;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface GridBaseOptions<
    TComponent extends GridBase<TRowData, TKey>,
    TRowData = any,
    TKey = any
  > extends WidgetOptions<TComponent> {
    /**
     * [descr:GridBaseOptions.allowColumnReordering]
     */
    allowColumnReordering?: boolean;
    /**
     * [descr:GridBaseOptions.allowColumnResizing]
     */
    allowColumnResizing?: boolean;
    /**
     * [descr:GridBaseOptions.autoNavigateToFocusedRow]
     */
    autoNavigateToFocusedRow?: boolean;
    /**
     * [descr:GridBaseOptions.cacheEnabled]
     */
    cacheEnabled?: boolean;
    /**
     * [descr:GridBaseOptions.cellHintEnabled]
     */
    cellHintEnabled?: boolean;
    /**
     * [descr:GridBaseOptions.columnAutoWidth]
     */
    columnAutoWidth?: boolean;
    /**
     * [descr:GridBaseOptions.columnChooser]
     */
    columnChooser?: DevExpress.ui.dxDataGrid.ColumnChooser;
    /**
     * [descr:GridBaseOptions.columnFixing]
     */
    columnFixing?: DevExpress.ui.dxDataGrid.ColumnFixing;
    /**
     * [descr:GridBaseOptions.columnHidingEnabled]
     */
    columnHidingEnabled?: boolean;
    /**
     * [descr:GridBaseOptions.columnMinWidth]
     */
    columnMinWidth?: number;
    /**
     * [descr:GridBaseOptions.columnResizingMode]
     */
    columnResizingMode?: 'nextColumn' | 'widget';
    /**
     * [descr:GridBaseOptions.columnWidth]
     */
    columnWidth?: number | 'auto';
    /**
     * [descr:GridBaseOptions.columns]
     */
    columns?: Array<DevExpress.ui.dxDataGrid.ColumnBase<TRowData> | string>;
    /**
     * [descr:GridBaseOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<TRowData, TKey>;
    /**
     * [descr:GridBaseOptions.dateSerializationFormat]
     */
    dateSerializationFormat?: string;
    /**
     * [descr:GridBaseOptions.editing]
     */
    editing?: DevExpress.ui.dxDataGrid.EditingBase<TRowData, TKey>;
    /**
     * [descr:GridBaseOptions.errorRowEnabled]
     */
    errorRowEnabled?: boolean;
    /**
     * [descr:GridBaseOptions.filterBuilder]
     */
    filterBuilder?: dxFilterBuilderOptions;
    /**
     * [descr:GridBaseOptions.filterBuilderPopup]
     */
    filterBuilderPopup?: DevExpress.ui.dxPopup.Properties;
    /**
     * [descr:GridBaseOptions.filterPanel]
     */
    filterPanel?: DevExpress.ui.dxDataGrid.FilterPanel<
      TComponent,
      TRowData,
      TKey
    >;
    /**
     * [descr:GridBaseOptions.filterRow]
     */
    filterRow?: DevExpress.ui.dxDataGrid.FilterRow;
    /**
     * [descr:GridBaseOptions.filterSyncEnabled]
     */
    filterSyncEnabled?: boolean | 'auto';
    /**
     * [descr:GridBaseOptions.filterValue]
     */
    filterValue?: string | Array<any> | Function;
    /**
     * [descr:GridBaseOptions.focusedColumnIndex]
     */
    focusedColumnIndex?: number;
    /**
     * [descr:GridBaseOptions.focusedRowEnabled]
     */
    focusedRowEnabled?: boolean;
    /**
     * [descr:GridBaseOptions.focusedRowIndex]
     */
    focusedRowIndex?: number;
    /**
     * [descr:GridBaseOptions.focusedRowKey]
     */
    focusedRowKey?: TKey;
    /**
     * [descr:GridBaseOptions.headerFilter]
     */
    headerFilter?: DevExpress.ui.dxDataGrid.HeaderFilter;
    /**
     * [descr:GridBaseOptions.highlightChanges]
     */
    highlightChanges?: boolean;
    /**
     * [descr:GridBaseOptions.keyboardNavigation]
     */
    keyboardNavigation?: DevExpress.ui.dxDataGrid.KeyboardNavigation;
    /**
     * [descr:GridBaseOptions.loadPanel]
     */
    loadPanel?: DevExpress.ui.dxDataGrid.LoadPanel;
    /**
     * [descr:GridBaseOptions.noDataText]
     */
    noDataText?: string;
    /**
     * [descr:GridBaseOptions.onAdaptiveDetailRowPreparing]
     */
    onAdaptiveDetailRowPreparing?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.AdaptiveDetailRowPreparingInfo
    ) => void;
    /**
     * [descr:GridBaseOptions.onDataErrorOccurred]
     */
    onDataErrorOccurred?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.DataErrorOccurredInfo
    ) => void;
    /**
     * [descr:GridBaseOptions.onEditCanceled]
     */
    onEditCanceled?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.DataChangeInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onEditCanceling]
     */
    onEditCanceling?: (
      e: DevExpress.events.Cancelable &
        DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.DataChangeInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onInitNewRow]
     */
    onInitNewRow?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.NewRowInfo<TRowData>
    ) => void;
    /**
     * [descr:GridBaseOptions.onKeyDown]
     */
    onKeyDown?: (
      e: DevExpress.events.NativeEventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.KeyDownInfo
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowCollapsed]
     */
    onRowCollapsed?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowCollapsing]
     */
    onRowCollapsing?: (
      e: DevExpress.events.Cancelable &
        DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowExpanded]
     */
    onRowExpanded?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowExpanding]
     */
    onRowExpanding?: (
      e: DevExpress.events.Cancelable &
        DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowKeyInfo<TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowInserted]
     */
    onRowInserted?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowInsertedInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowInserting]
     */
    onRowInserting?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowInsertingInfo<TRowData>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowRemoved]
     */
    onRowRemoved?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowRemovedInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowRemoving]
     */
    onRowRemoving?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowRemovingInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowUpdated]
     */
    onRowUpdated?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowUpdatedInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowUpdating]
     */
    onRowUpdating?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowUpdatingInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onRowValidating]
     */
    onRowValidating?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.RowValidatingInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onSaved]
     */
    onSaved?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.DataChangeInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onSaving]
     */
    onSaving?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.SavingInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.SelectionChangedInfo<TRowData, TKey>
    ) => void;
    /**
     * [descr:GridBaseOptions.onToolbarPreparing]
     * @deprecated [depNote:GridBaseOptions.onToolbarPreparing]
     */
    onToolbarPreparing?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.ui.dxDataGrid.ToolbarPreparingInfo
    ) => void;
    /**
     * [descr:GridBaseOptions.pager]
     */
    pager?: DevExpress.ui.dxDataGrid.Pager;
    /**
     * [descr:GridBaseOptions.paging]
     */
    paging?: DevExpress.ui.dxDataGrid.PagingBase;
    /**
     * [descr:GridBaseOptions.renderAsync]
     */
    renderAsync?: boolean;
    /**
     * [descr:GridBaseOptions.repaintChangesOnly]
     */
    repaintChangesOnly?: boolean;
    /**
     * [descr:GridBaseOptions.rowAlternationEnabled]
     */
    rowAlternationEnabled?: boolean;
    /**
     * [descr:GridBaseOptions.rowDragging]
     */
    rowDragging?: DevExpress.ui.dxDataGrid.RowDragging<
      TComponent,
      TRowData,
      TKey
    >;
    /**
     * [descr:GridBaseOptions.scrolling]
     */
    scrolling?: DevExpress.ui.dxDataGrid.ScrollingBase;
    /**
     * [descr:GridBaseOptions.searchPanel]
     */
    searchPanel?: DevExpress.ui.dxDataGrid.SearchPanel;
    /**
     * [descr:GridBaseOptions.selectedRowKeys]
     */
    selectedRowKeys?: Array<TKey>;
    /**
     * [descr:GridBaseOptions.selection]
     */
    selection?: DevExpress.ui.dxDataGrid.SelectionBase;
    /**
     * [descr:GridBaseOptions.showBorders]
     */
    showBorders?: boolean;
    /**
     * [descr:GridBaseOptions.showColumnHeaders]
     */
    showColumnHeaders?: boolean;
    /**
     * [descr:GridBaseOptions.showColumnLines]
     */
    showColumnLines?: boolean;
    /**
     * [descr:GridBaseOptions.showRowLines]
     */
    showRowLines?: boolean;
    /**
     * [descr:GridBaseOptions.sorting]
     */
    sorting?: DevExpress.ui.dxDataGrid.Sorting;
    /**
     * [descr:GridBaseOptions.stateStoring]
     */
    stateStoring?: DevExpress.ui.dxDataGrid.StateStoring;
    /**
     * [descr:GridBaseOptions.twoWayBindingEnabled]
     */
    twoWayBindingEnabled?: boolean;
    /**
     * [descr:GridBaseOptions.wordWrapEnabled]
     */
    wordWrapEnabled?: boolean;
  }
  /**
   * @deprecated 
   */
  export type GridBasePaging = DevExpress.ui.dxDataGrid.PagingBase;
  /**
   * @deprecated 
   */
  export type GridBaseScrolling = DevExpress.ui.dxDataGrid.ScrollingBase;
  /**
   * @deprecated 
   */
  export type GridBaseSelection = DevExpress.ui.dxDataGrid.SelectionBase;
  /**
   * [descr:HierarchicalCollectionWidget]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class HierarchicalCollectionWidget<
    TProperties extends HierarchicalCollectionWidgetOptions<any, TItem, TKey>,
    TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
    TKey = any
  > extends CollectionWidget<TProperties, TItem, TKey> {}
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface HierarchicalCollectionWidgetOptions<
    TComponent extends HierarchicalCollectionWidget<any, TItem, TKey>,
    TItem extends DevExpress.ui.CollectionWidget.ItemLike = any,
    TKey = any
  > extends CollectionWidgetOptions<TComponent, TItem, TKey> {
    /**
     * [descr:HierarchicalCollectionWidgetOptions.disabledExpr]
     */
    disabledExpr?: string | Function;
    /**
     * [descr:HierarchicalCollectionWidgetOptions.displayExpr]
     */
    displayExpr?: string | ((item: TItem) => string);
    /**
     * [descr:HierarchicalCollectionWidgetOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:HierarchicalCollectionWidgetOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:HierarchicalCollectionWidgetOptions.itemsExpr]
     */
    itemsExpr?: string | Function;
    /**
     * [descr:HierarchicalCollectionWidgetOptions.keyExpr]
     */
    keyExpr?: string | Function;
    /**
     * [descr:HierarchicalCollectionWidgetOptions.selectedExpr]
     */
    selectedExpr?: string | Function;
  }
  export interface MapLocation {
    /**
     * [descr:MapLocation.lat]
     */
    lat: number;
    /**
     * [descr:MapLocation.lng]
     */
    lng: number;
  }
  /**
   * [descr:ui.notify(message,type,displayTime)]
   */
  export function notify(
    message: string,
    type?: string,
    displayTime?: number
  ): void;
  /**
   * [descr:ui.notify(options,type,displayTime)]
   */
  export function notify(
    options: any,
    type?: string,
    displayTime?: number
  ): void;
  /**
   * [descr:NumericRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface NumericRule {
    /**
     * [descr:NumericRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:NumericRule.message]
     */
    message?: string;
    /**
     * [descr:NumericRule.type]
     */
    type: 'numeric';
  }
  /**
   * [descr:PatternRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PatternRule {
    /**
     * [descr:PatternRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:PatternRule.message]
     */
    message?: string;
    /**
     * [descr:PatternRule.pattern]
     */
    pattern?: RegExp | string;
    /**
     * [descr:PatternRule.type]
     */
    type: 'pattern';
  }
  /**
   * [descr:RangeRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface RangeRule {
    /**
     * [descr:RangeRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:RangeRule.max]
     */
    max?: Date | number;
    /**
     * [descr:RangeRule.message]
     */
    message?: string;
    /**
     * [descr:RangeRule.min]
     */
    min?: Date | number;
    /**
     * [descr:RangeRule.reevaluate]
     */
    reevaluate?: boolean;
    /**
     * [descr:RangeRule.type]
     */
    type: 'range';
  }
  /**
   * [descr:ui.repaintFloatingActionButton()]
   */
  export function repaintFloatingActionButton(): void;
  /**
   * [descr:RequiredRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface RequiredRule {
    /**
     * [descr:RequiredRule.message]
     */
    message?: string;
    /**
     * [descr:RequiredRule.trim]
     */
    trim?: boolean;
    /**
     * [descr:RequiredRule.type]
     */
    type: 'required';
  }
  /**
   * [descr:SearchBoxMixin]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class SearchBoxMixin {
    constructor(options?: SearchBoxMixinOptions);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface SearchBoxMixinOptions {
    /**
     * [descr:SearchBoxMixinOptions.searchEditorOptions]
     */
    searchEditorOptions?: DevExpress.ui.dxTextBox.Properties;
    /**
     * [descr:SearchBoxMixinOptions.searchEnabled]
     */
    searchEnabled?: boolean;
    /**
     * [descr:SearchBoxMixinOptions.searchExpr]
     */
    searchExpr?: string | Function | Array<string | Function>;
    /**
     * [descr:SearchBoxMixinOptions.searchMode]
     */
    searchMode?: 'contains' | 'startswith' | 'equals';
    /**
     * [descr:SearchBoxMixinOptions.searchTimeout]
     */
    searchTimeout?: number;
    /**
     * [descr:SearchBoxMixinOptions.searchValue]
     */
    searchValue?: string;
  }
  /**
   * [descr:StringLengthRule]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface StringLengthRule {
    /**
     * [descr:StringLengthRule.ignoreEmptyValue]
     */
    ignoreEmptyValue?: boolean;
    /**
     * [descr:StringLengthRule.max]
     */
    max?: number;
    /**
     * [descr:StringLengthRule.message]
     */
    message?: string;
    /**
     * [descr:StringLengthRule.min]
     */
    min?: number;
    /**
     * [descr:StringLengthRule.trim]
     */
    trim?: boolean;
    /**
     * [descr:StringLengthRule.type]
     */
    type: 'stringLength';
  }
  /**
   * [descr:ui.template]
   * @deprecated [depNote:ui.template]
   */
  export type Template = DevExpress.core.template;
  /**
   * [descr:ui.themes]
   */
  export class themes {
    /**
     * [descr:ui.themes.current()]
     */
    static current(): string;
    /**
     * [descr:ui.themes.current(themeName)]
     */
    static current(themeName: string): void;
    /**
     * [descr:ui.themes.ready(callback)]
     */
    static ready(callback: Function): void;
    /**
     * [descr:ui.themes.initialized(callback)]
     */
    static initialized(callback: Function): void;
    static isMaterial(theme: string): boolean;
  }
  export interface ValidationCallbackData {
    value?: string | number;
    rule: any;
    validator: any;
    data?: any;
    column?: any;
    formItem?: any;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type ValidationRule =
    | AsyncRule
    | CompareRule
    | CustomRule
    | EmailRule
    | NumericRule
    | PatternRule
    | RangeRule
    | RequiredRule
    | StringLengthRule;
  /**
   * [descr:Widget]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class Widget<TProperties> extends DOMComponent<TProperties> {
    /**
     * [descr:Widget.focus()]
     */
    focus(): void;
    /**
     * [descr:Widget.registerKeyHandler(key, handler)]
     */
    registerKeyHandler(key: string, handler: Function): void;
    /**
     * [descr:Widget.repaint()]
     */
    repaint(): void;
  }
  module Widget {
    /**
     * [descr:format]
     * @deprecated [depNote:format]
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type format = Format;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface WidgetOptions<TComponent>
    extends DOMComponentOptions<TComponent> {
    /**
     * [descr:WidgetOptions.accessKey]
     */
    accessKey?: string;
    /**
     * [descr:WidgetOptions.activeStateEnabled]
     */
    activeStateEnabled?: boolean;
    /**
     * [descr:WidgetOptions.disabled]
     */
    disabled?: boolean;
    /**
     * [descr:WidgetOptions.focusStateEnabled]
     */
    focusStateEnabled?: boolean;
    /**
     * [descr:WidgetOptions.hint]
     */
    hint?: string;
    /**
     * [descr:WidgetOptions.hoverStateEnabled]
     */
    hoverStateEnabled?: boolean;
    /**
     * [descr:WidgetOptions.onContentReady]
     */
    onContentReady?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:WidgetOptions.tabIndex]
     */
    tabIndex?: number;
    /**
     * [descr:WidgetOptions.visible]
     */
    visible?: boolean;
  }
}
declare module DevExpress.ui.dialog {
  /**
   * [descr:ui.dialog.alert(messageHtml,title)]
   */
  export function alert(
    messageHtml: string,
    title: string
  ): DevExpress.core.utils.DxPromise<void>;
  /**
   * [descr:ui.dialog.confirm(messageHtml,title)]
   */
  export function confirm(
    messageHtml: string,
    title: string
  ): DevExpress.core.utils.DxPromise<boolean>;
  /**
   * [descr:ui.dialog.custom(options)]
   */
  export function custom(options: CustomDialogOptions): any;
}
declare module DevExpress.ui.dxAccordion {
  export type Item = dxAccordionItem;
}
declare module DevExpress.ui.dxActionSheet {
  export type Item<TKey = any> = dxActionSheetItem<TKey>;
}
declare module DevExpress.ui.dxBox {
  export type Item<TKey = any> = dxBoxItem<TKey>;
}
declare module DevExpress.ui.dxButtonGroup {
  export type Item = dxButtonGroupItem;
}
declare module DevExpress.ui.dxContextMenu {
  export type Item = dxContextMenuItem;
}
declare module DevExpress.ui.dxDataGrid {
  /**
   * [descr:DataChange]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface DataChange<TRowData = any, TKey = any> {
    /**
     * [descr:DataChange.key]
     */
    key: TKey;
    /**
     * [descr:DataChange.type]
     */
    type: 'insert' | 'update' | 'remove';
    /**
     * [descr:DataChange.data]
     */
    data: DevExpress.core.DeepPartial<TRowData>;
    /**
     * [descr:DataChange.insertAfterKey]
     */
    insertAfterKey?: TKey;
    /**
     * [descr:DataChange.insertBeforeKey]
     */
    insertBeforeKey?: TKey;
  }
  /**
   * [descr:dxDataGridToolbar]
   */
  export type Toolbar = {
    /**
     * [descr:dxDataGridToolbar.items]
     */
    items?: Array<DefaultToolbarItemName | ToolbarItem>;
    /**
     * [descr:dxDataGridToolbar.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxDataGridToolbar.disabled]
     */
    disabled?: boolean;
  };
  /**
   * [descr:dxDataGridToolbarItem]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ToolbarItem extends dxToolbarItem {
    /**
     * [descr:dxDataGridToolbarItem.name]
     */
    name?: DefaultToolbarItemName | string;
    /**
     * [descr:dxDataGridToolbarItem.location]
     */
    location?: 'after' | 'before' | 'center';
  }
}
declare module DevExpress.ui.dxDiagram {
  export type Item = dxDiagramItem;
}
declare module DevExpress.ui.dxDropDownButton {
  export type Item = dxDropDownButtonItem;
}
declare module DevExpress.ui.dxFileManager {
  export type ContextMenuItem = dxFileManagerContextMenuItem;
  export type ToolbarItem = dxFileManagerToolbarItem;
}
declare module DevExpress.ui.dxForm {
  export type ButtonItem = dxFormButtonItem;
  export type EmptyItem = dxFormEmptyItem;
  export type GroupItem = dxFormGroupItem;
  export type Item =
    | SimpleItem
    | GroupItem
    | TabbedItem
    | EmptyItem
    | ButtonItem;
  export type SimpleItem = dxFormSimpleItem;
  export type TabbedItem = dxFormTabbedItem;
}
declare module DevExpress.ui.dxGallery {
  export type Item = dxGalleryItem;
}
declare module DevExpress.ui.dxGantt {
  export type ContextMenuItem = dxGanttContextMenuItem;
  export type ToolbarItem = dxGanttToolbarItem;
}
declare module DevExpress.ui.dxHtmlEditor {
  export type ContextMenuItem = dxHtmlEditorTableContextMenuItem;
  export type ToolbarItem = dxHtmlEditorToolbarItem;
}
declare module DevExpress.ui.dxList {
  export type Item = dxListItem;
}
declare module DevExpress.ui.dxMenu {
  export type Item = dxMenuItem;
}
declare module DevExpress.ui.dxMultiView {
  export type Item = dxMultiViewItem;
}
declare module DevExpress.ui.dxNavBar {
  export type Item = dxNavBarItem;
}
declare module DevExpress.ui.dxOverlay {
  /**
   * [descr:ui.dxOverlay.baseZIndex(zIndex)]
   */
  export function baseZIndex(zIndex: number): void;
}
declare module DevExpress.ui.dxPivotGrid {
  export type Cell = dxPivotGridPivotGridCell;
}
declare module DevExpress.ui.dxPopup {
  export type ToolbarItem = dxPopupToolbarItem;
}
declare module DevExpress.ui.dxResponsiveBox {
  export type Item = dxResponsiveBoxItem;
}
declare module DevExpress.ui.dxSlideOut {
  export type Item = dxSlideOutItem;
}
declare module DevExpress.ui.dxTabPanel {
  export type Item = dxTabPanelItem;
}
declare module DevExpress.ui.dxTabs {
  export type Item = dxTabsItem;
}
declare module DevExpress.ui.dxTileView {
  export type Item = dxTileViewItem;
}
declare module DevExpress.ui.dxToolbar {
  export type Item = dxToolbarItem;
}
declare module DevExpress.ui.dxTreeList {
  /**
   * [descr:dxTreeListToolbar]
   */
  export type Toolbar = {
    /**
     * [descr:dxTreeListToolbar.items]
     */
    items?: (DefaultToolbarItemName | ToolbarItem)[];
    /**
     * [descr:dxTreeListToolbar.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxTreeListToolbar.disabled]
     */
    disabled?: boolean;
  };
  /**
   * [descr:dxTreeListToolbarItem]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ToolbarItem extends dxToolbarItem {
    /**
     * [descr:dxTreeListToolbarItem.name]
     */
    name?: DefaultToolbarItemName | string;
    /**
     * [descr:dxTreeListToolbarItem.location]
     */
    location?: 'after' | 'before' | 'center';
  }
}
declare module DevExpress.ui.dxTreeView {
  export type Item = dxTreeViewItem;
}
declare module DevExpress.utils {
  /**
   * [descr:utils.cancelAnimationFrame(requestID)]
   */
  export function cancelAnimationFrame(requestID: number): void;
  /**
   * [descr:utils.getTimeZones(date)]
   */
  export function getTimeZones(date?: Date): Array<dxSchedulerTimeZone>;
  /**
   * [descr:utils.initMobileViewport(options)]
   */
  export function initMobileViewport(options: {
    allowZoom?: boolean;
    allowPan?: boolean;
    allowSelection?: boolean;
  }): void;
  /**
   * [descr:utils.requestAnimationFrame(callback)]
   */
  export function requestAnimationFrame(callback: Function): number;
}
declare module DevExpress.viz {
  /**
   * [descr:BarGaugeBarInfo]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BarGaugeBarInfo {
    /**
     * [descr:BarGaugeBarInfo.color]
     */
    color?: string;
    /**
     * [descr:BarGaugeBarInfo.index]
     */
    index?: number;
    /**
     * [descr:BarGaugeBarInfo.value]
     */
    value?: number;
  }
  /**
   * @deprecated Use LegendItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BarGaugeLegendItem extends BaseLegendItem {
    /**
     * [descr:BarGaugeLegendItem.item]
     */
    item?: BarGaugeBarInfo;
  }
  /**
   * [descr:BaseChart]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class BaseChart<TProperties> extends BaseWidget<TProperties> {
    /**
     * [descr:BaseChart.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:BaseChart.getAllSeries()]
     */
    getAllSeries(): Array<baseSeriesObject>;
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:BaseChart.getSeriesByName(seriesName)]
     */
    getSeriesByName(seriesName: any): chartSeriesObject;
    /**
     * [descr:BaseChart.getSeriesByPos(seriesIndex)]
     */
    getSeriesByPos(seriesIndex: number): chartSeriesObject;
    /**
     * [descr:BaseChart.hideTooltip()]
     */
    hideTooltip(): void;
    /**
     * [descr:BaseChart.refresh()]
     */
    refresh(): void;
    render(): void;
    /**
     * [descr:BaseChart.render(renderOptions)]
     */
    render(renderOptions: any): void;
  }
  module BaseChart {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface PointInteractionInfo {
      readonly target: basePointObject;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface TooltipInfo {
      target?: basePointObject | dxChartAnnotationConfig | any;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseChartAdaptiveLayout {
    /**
     * [descr:BaseChartOptions.adaptiveLayout.height]
     */
    height?: number;
    /**
     * [descr:BaseChartOptions.adaptiveLayout.keepLabels]
     */
    keepLabels?: boolean;
    /**
     * [descr:BaseChartOptions.adaptiveLayout.width]
     */
    width?: number;
  }
  /**
   * [descr:BaseChartAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseChartAnnotationConfig
    extends BaseWidgetAnnotationConfig {
    /**
     * [descr:BaseChartAnnotationConfig.argument]
     */
    argument?: number | Date | string;
    /**
     * [descr:BaseChartAnnotationConfig.series]
     */
    series?: string;
    /**
     * [descr:BaseChartAnnotationConfig.value]
     */
    value?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseChartLegend extends BaseLegend {
    /**
     * [descr:BaseChartOptions.legend.customizeItems]
     */
    customizeItems?: (
      items: Array<BaseChartLegendItem>
    ) => Array<BaseChartLegendItem>;
    /**
     * [descr:BaseChartOptions.legend.markerTemplate]
     */
    markerTemplate?:
      | DevExpress.core.template
      | ((
          legendItem: BaseChartLegendItem,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
  }
  /**
   * [descr:BaseChartLegendItem]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseChartLegendItem extends BaseLegendItem {
    /**
     * [descr:BaseChartLegendItem.series]
     */
    series?: baseSeriesObject;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseChartOptions<TComponent>
    extends BaseWidgetOptions<TComponent> {
    /**
     * [descr:BaseChartOptions.adaptiveLayout]
     */
    adaptiveLayout?: BaseChartAdaptiveLayout;
    /**
     * [descr:BaseChartOptions.animation]
     */
    animation?:
      | {
          /**
           * [descr:BaseChartOptions.animation.duration]
           */
          duration?: number;
          /**
           * [descr:BaseChartOptions.animation.easing]
           */
          easing?: 'easeOutCubic' | 'linear';
          /**
           * [descr:BaseChartOptions.animation.enabled]
           */
          enabled?: boolean;
          /**
           * [descr:BaseChartOptions.animation.maxPointCountSupported]
           */
          maxPointCountSupported?: number;
        }
      | boolean;
    /**
     * [descr:BaseChartOptions.customizeLabel]
     */
    customizeLabel?: (pointInfo: any) => dxChartSeriesTypesCommonSeriesLabel;
    /**
     * [descr:BaseChartOptions.customizePoint]
     */
    customizePoint?: (pointInfo: any) => dxChartSeriesTypesCommonSeriesPoint;
    /**
     * [descr:BaseChartOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
    /**
     * [descr:BaseChartOptions.legend]
     */
    legend?: BaseChartLegend;
    /**
     * [descr:BaseChartOptions.onDone]
     */
    onDone?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:BaseChartOptions.onPointClick]
     */
    onPointClick?:
      | ((
          e: DevExpress.events.NativeEventInfo<
            TComponent,
            MouseEvent | PointerEvent
          > &
            DevExpress.viz.BaseChart.PointInteractionInfo
        ) => void)
      | string;
    /**
     * [descr:BaseChartOptions.onPointHoverChanged]
     */
    onPointHoverChanged?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseChart.PointInteractionInfo
    ) => void;
    /**
     * [descr:BaseChartOptions.onPointSelectionChanged]
     */
    onPointSelectionChanged?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseChart.PointInteractionInfo
    ) => void;
    /**
     * [descr:BaseChartOptions.onTooltipHidden]
     */
    onTooltipHidden?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseChart.TooltipInfo
    ) => void;
    /**
     * [descr:BaseChartOptions.onTooltipShown]
     */
    onTooltipShown?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseChart.TooltipInfo
    ) => void;
    /**
     * [descr:BaseChartOptions.palette]
     */
    palette?: Array<string> | PaletteType;
    /**
     * [descr:BaseChartOptions.paletteExtensionMode]
     */
    paletteExtensionMode?: PaletteExtensionModeType;
    /**
     * [descr:BaseChartOptions.pointSelectionMode]
     */
    pointSelectionMode?: 'multiple' | 'single';
    /**
     * [descr:BaseChartOptions.series]
     */
    series?: any | Array<any>;
    /**
     * [descr:BaseChartOptions.tooltip]
     */
    tooltip?: BaseChartTooltip;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseChartTooltip extends BaseWidgetTooltip {
    /**
     * [descr:BaseChartOptions.tooltip.argumentFormat]
     */
    argumentFormat?: DevExpress.ui.Format;
    /**
     * [descr:BaseChartOptions.tooltip.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          pointInfo: any,
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:BaseChartOptions.tooltip.customizeTooltip]
     */
    customizeTooltip?: (pointInfo: any) => any;
    /**
     * [descr:BaseChartOptions.tooltip.shared]
     */
    shared?: boolean;
    /**
     * [descr:BaseChartOptions.tooltip.interactive]
     */
    interactive?: boolean;
  }
  /**
   * [descr:BaseGauge]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class BaseGauge<TProperties> extends BaseWidget<TProperties> {
    /**
     * [descr:BaseGauge.subvalues()]
     */
    subvalues(): Array<number>;
    /**
     * [descr:BaseGauge.subvalues(subvalues)]
     */
    subvalues(subvalues: Array<number>): void;
    /**
     * [descr:BaseGauge.value()]
     */
    value(): number;
    /**
     * [descr:BaseGauge.value(value)]
     */
    value(value: number): void;
  }
  module BaseGauge {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type GaugeIndicatorType =
      | 'circle'
      | 'rangeBar'
      | 'rectangle'
      | 'rectangleNeedle'
      | 'rhombus'
      | 'textCloud'
      | 'triangleMarker'
      | 'triangleNeedle'
      | 'twoColorNeedle';
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface TooltipInfo {
      target: any;
    }
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseGaugeAnimation {
    /**
     * [descr:BaseGaugeOptions.animation.duration]
     */
    duration?: number;
    /**
     * [descr:BaseGaugeOptions.animation.easing]
     */
    easing?: 'easeOutCubic' | 'linear';
    /**
     * [descr:BaseGaugeOptions.animation.enabled]
     */
    enabled?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseGaugeLoadingIndicator
    extends BaseWidgetLoadingIndicator {
    /**
     * [descr:BaseGaugeOptions.loadingIndicator.enabled]
     */
    enabled?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseGaugeOptions<TComponent>
    extends BaseWidgetOptions<TComponent> {
    /**
     * [descr:BaseGaugeOptions.animation]
     */
    animation?: BaseGaugeAnimation;
    /**
     * [descr:BaseGaugeOptions.containerBackgroundColor]
     */
    containerBackgroundColor?: string;
    /**
     * [descr:BaseGaugeOptions.loadingIndicator]
     */
    loadingIndicator?: BaseGaugeLoadingIndicator;
    /**
     * [descr:BaseGaugeOptions.onTooltipHidden]
     */
    onTooltipHidden?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseGauge.TooltipInfo
    ) => void;
    /**
     * [descr:BaseGaugeOptions.onTooltipShown]
     */
    onTooltipShown?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseGauge.TooltipInfo
    ) => void;
    /**
     * [descr:BaseGaugeOptions.rangeContainer]
     */
    rangeContainer?: BaseGaugeRangeContainer;
    /**
     * [descr:BaseGaugeOptions.scale]
     */
    scale?: BaseGaugeScale;
    /**
     * [descr:BaseGaugeOptions.subvalues]
     */
    subvalues?: Array<number>;
    /**
     * [descr:BaseGaugeOptions.tooltip]
     */
    tooltip?: BaseGaugeTooltip;
    /**
     * [descr:BaseGaugeOptions.value]
     */
    value?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseGaugeRangeContainer {
    /**
     * [descr:BaseGaugeOptions.rangeContainer.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:BaseGaugeOptions.rangeContainer.offset]
     */
    offset?: number;
    /**
     * [descr:BaseGaugeOptions.rangeContainer.palette]
     */
    palette?: Array<string> | PaletteType;
    /**
     * [descr:BaseGaugeOptions.rangeContainer.paletteExtensionMode]
     */
    paletteExtensionMode?: PaletteExtensionModeType;
    /**
     * [descr:BaseGaugeOptions.rangeContainer.ranges]
     */
    ranges?: Array<{
      /**
       * [descr:BaseGaugeOptions.rangeContainer.ranges.color]
       */
      color?: string;
      /**
       * [descr:BaseGaugeOptions.rangeContainer.ranges.endValue]
       */
      endValue?: number;
      /**
       * [descr:BaseGaugeOptions.rangeContainer.ranges.startValue]
       */
      startValue?: number;
    }>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseGaugeScale {
    /**
     * [descr:BaseGaugeOptions.scale.allowDecimals]
     */
    allowDecimals?: boolean;
    /**
     * [descr:BaseGaugeOptions.scale.customMinorTicks]
     */
    customMinorTicks?: Array<number>;
    /**
     * [descr:BaseGaugeOptions.scale.customTicks]
     */
    customTicks?: Array<number>;
    /**
     * [descr:BaseGaugeOptions.scale.endValue]
     */
    endValue?: number;
    /**
     * [descr:BaseGaugeOptions.scale.label]
     */
    label?: BaseGaugeScaleLabel;
    /**
     * [descr:BaseGaugeOptions.scale.minorTick]
     */
    minorTick?: {
      /**
       * [descr:BaseGaugeOptions.scale.minorTick.color]
       */
      color?: string;
      /**
       * [descr:BaseGaugeOptions.scale.minorTick.length]
       */
      length?: number;
      /**
       * [descr:BaseGaugeOptions.scale.minorTick.opacity]
       */
      opacity?: number;
      /**
       * [descr:BaseGaugeOptions.scale.minorTick.visible]
       */
      visible?: boolean;
      /**
       * [descr:BaseGaugeOptions.scale.minorTick.width]
       */
      width?: number;
    };
    /**
     * [descr:BaseGaugeOptions.scale.minorTickInterval]
     */
    minorTickInterval?: number;
    /**
     * [descr:BaseGaugeOptions.scale.scaleDivisionFactor]
     */
    scaleDivisionFactor?: number;
    /**
     * [descr:BaseGaugeOptions.scale.startValue]
     */
    startValue?: number;
    /**
     * [descr:BaseGaugeOptions.scale.tick]
     */
    tick?: {
      /**
       * [descr:BaseGaugeOptions.scale.tick.color]
       */
      color?: string;
      /**
       * [descr:BaseGaugeOptions.scale.tick.length]
       */
      length?: number;
      /**
       * [descr:BaseGaugeOptions.scale.tick.opacity]
       */
      opacity?: number;
      /**
       * [descr:BaseGaugeOptions.scale.tick.visible]
       */
      visible?: boolean;
      /**
       * [descr:BaseGaugeOptions.scale.tick.width]
       */
      width?: number;
    };
    /**
     * [descr:BaseGaugeOptions.scale.tickInterval]
     */
    tickInterval?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseGaugeScaleLabel {
    /**
     * [descr:BaseGaugeOptions.scale.label.customizeText]
     */
    customizeText?: (scaleValue: {
      value?: number;
      valueText?: string;
    }) => string;
    /**
     * [descr:BaseGaugeOptions.scale.label.font]
     */
    font?: Font;
    /**
     * [descr:BaseGaugeOptions.scale.label.format]
     */
    format?: DevExpress.ui.Format;
    /**
     * [descr:BaseGaugeOptions.scale.label.overlappingBehavior]
     */
    overlappingBehavior?: 'hide' | 'none';
    /**
     * [descr:BaseGaugeOptions.scale.label.useRangeColors]
     */
    useRangeColors?: boolean;
    /**
     * [descr:BaseGaugeOptions.scale.label.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseGaugeTooltip extends BaseWidgetTooltip {
    /**
     * [descr:BaseGaugeOptions.tooltip.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          scaleValue: { value?: number; valueText?: string },
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:BaseGaugeOptions.tooltip.customizeTooltip]
     */
    customizeTooltip?: (scaleValue: {
      value?: number;
      valueText?: string;
    }) => any;
    /**
     * [descr:BaseGaugeOptions.tooltip.interactive]
     */
    interactive?: boolean;
  }
  /**
   * [descr:baseLabelObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface baseLabelObject {
    /**
     * [descr:baseLabelObject.getBoundingRect()]
     */
    getBoundingRect(): any;
    /**
     * [descr:baseLabelObject.hide()]
     */
    hide(): void;
    /**
     * [descr:baseLabelObject.hide(holdInvisible)]
     */
    hide(holdInvisible: boolean): void;
    /**
     * [descr:baseLabelObject.isVisible()]
     */
    isVisible(): boolean;
    /**
     * [descr:baseLabelObject.show()]
     */
    show(): void;
    /**
     * [descr:baseLabelObject.show(holdVisible)]
     */
    show(holdVisible: boolean): void;
  }
  /**
   * [descr:BaseLegend]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseLegend {
    /**
     * [descr:BaseLegend.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:BaseLegend.border]
     */
    border?: {
      /**
       * [descr:BaseLegend.border.color]
       */
      color?: string;
      /**
       * [descr:BaseLegend.border.cornerRadius]
       */
      cornerRadius?: number;
      /**
       * [descr:BaseLegend.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:BaseLegend.border.opacity]
       */
      opacity?: number;
      /**
       * [descr:BaseLegend.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:BaseLegend.border.width]
       */
      width?: number;
    };
    /**
     * [descr:BaseLegend.columnCount]
     */
    columnCount?: number;
    /**
     * [descr:BaseLegend.columnItemSpacing]
     */
    columnItemSpacing?: number;
    /**
     * [descr:BaseLegend.font]
     */
    font?: Font;
    /**
     * [descr:BaseLegend.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:BaseLegend.itemTextPosition]
     */
    itemTextPosition?: 'bottom' | 'left' | 'right' | 'top';
    /**
     * [descr:BaseLegend.itemsAlignment]
     */
    itemsAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:BaseLegend.margin]
     */
    margin?:
      | number
      | {
          /**
           * [descr:BaseLegend.margin.bottom]
           */
          bottom?: number;
          /**
           * [descr:BaseLegend.margin.left]
           */
          left?: number;
          /**
           * [descr:BaseLegend.margin.right]
           */
          right?: number;
          /**
           * [descr:BaseLegend.margin.top]
           */
          top?: number;
        };
    /**
     * [descr:BaseLegend.markerSize]
     */
    markerSize?: number;
    /**
     * [descr:BaseLegend.orientation]
     */
    orientation?: 'horizontal' | 'vertical';
    /**
     * [descr:BaseLegend.paddingLeftRight]
     */
    paddingLeftRight?: number;
    /**
     * [descr:BaseLegend.paddingTopBottom]
     */
    paddingTopBottom?: number;
    /**
     * [descr:BaseLegend.rowCount]
     */
    rowCount?: number;
    /**
     * [descr:BaseLegend.rowItemSpacing]
     */
    rowItemSpacing?: number;
    /**
     * [descr:BaseLegend.title]
     */
    title?:
      | {
          /**
           * [descr:BaseLegend.title.font]
           */
          font?: Font;
          /**
           * [descr:BaseLegend.title.horizontalAlignment]
           */
          horizontalAlignment?: 'center' | 'left' | 'right';
          /**
           * [descr:BaseLegend.title.margin]
           */
          margin?: {
            /**
             * [descr:BaseLegend.title.margin.bottom]
             */
            bottom?: number;
            /**
             * [descr:BaseLegend.title.margin.left]
             */
            left?: number;
            /**
             * [descr:BaseLegend.title.margin.right]
             */
            right?: number;
            /**
             * [descr:BaseLegend.title.margin.top]
             */
            top?: number;
          };
          /**
           * [descr:BaseLegend.title.placeholderSize]
           */
          placeholderSize?: number;
          /**
           * [descr:BaseLegend.title.subtitle]
           */
          subtitle?:
            | {
                /**
                 * [descr:BaseLegend.title.subtitle.font]
                 */
                font?: Font;
                /**
                 * [descr:BaseLegend.title.subtitle.offset]
                 */
                offset?: number;
                /**
                 * [descr:BaseLegend.title.subtitle.text]
                 */
                text?: string;
              }
            | string;
          /**
           * [descr:BaseLegend.title.text]
           */
          text?: string;
          /**
           * [descr:BaseLegend.title.verticalAlignment]
           */
          verticalAlignment?: 'bottom' | 'top';
        }
      | string;
    /**
     * [descr:BaseLegend.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'top';
    /**
     * [descr:BaseLegend.visible]
     */
    visible?: boolean;
  }
  /**
   * [descr:BaseLegendItem]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseLegendItem {
    /**
     * [descr:BaseLegendItem.marker]
     */
    marker?: {
      /**
       * [descr:BaseLegendItem.marker.fill]
       */
      fill?: string;
      /**
       * [descr:BaseLegendItem.marker.opacity]
       */
      opacity?: number;
      /**
       * [descr:BaseLegendItem.marker.size]
       */
      size?: number;
      /**
       * [descr:BaseLegendItem.marker.state]
       */
      state?: 'normal' | 'hovered' | 'selected';
    };
    /**
     * [descr:BaseLegendItem.text]
     */
    text?: string;
    /**
     * [descr:BaseLegendItem.visible]
     */
    visible?: boolean;
  }
  /**
   * [descr:basePointObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface basePointObject {
    /**
     * [descr:basePointObject.clearHover()]
     */
    clearHover(): void;
    /**
     * [descr:basePointObject.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:basePointObject.data]
     */
    data?: any;
    /**
     * [descr:basePointObject.fullState]
     */
    fullState?: number;
    /**
     * [descr:basePointObject.getColor()]
     */
    getColor(): string;
    /**
     * [descr:basePointObject.getLabel()]
     */
    getLabel(): baseLabelObject & Array<baseLabelObject>;
    /**
     * [descr:basePointObject.hideTooltip()]
     */
    hideTooltip(): void;
    /**
     * [descr:basePointObject.hover()]
     */
    hover(): void;
    /**
     * [descr:basePointObject.isHovered()]
     */
    isHovered(): boolean;
    /**
     * [descr:basePointObject.isSelected()]
     */
    isSelected(): boolean;
    /**
     * [descr:basePointObject.originalArgument]
     */
    originalArgument?: string | number | Date;
    /**
     * [descr:basePointObject.originalValue]
     */
    originalValue?: string | number | Date;
    /**
     * [descr:basePointObject.select()]
     */
    select(): void;
    /**
     * [descr:basePointObject.series]
     */
    series?: any;
    /**
     * [descr:basePointObject.showTooltip()]
     */
    showTooltip(): void;
    /**
     * [descr:basePointObject.tag]
     */
    tag?: any;
  }
  /**
   * [descr:baseSeriesObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface baseSeriesObject {
    /**
     * [descr:baseSeriesObject.clearHover()]
     */
    clearHover(): void;
    /**
     * [descr:baseSeriesObject.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:baseSeriesObject.deselectPoint(point)]
     */
    deselectPoint(point: basePointObject): void;
    /**
     * [descr:baseSeriesObject.fullState]
     */
    fullState?: number;
    /**
     * [descr:baseSeriesObject.getAllPoints()]
     */
    getAllPoints(): Array<basePointObject>;
    /**
     * [descr:baseSeriesObject.getColor()]
     */
    getColor(): string;
    /**
     * [descr:baseSeriesObject.getPointByPos(positionIndex)]
     */
    getPointByPos(positionIndex: number): basePointObject;
    /**
     * [descr:baseSeriesObject.getPointsByArg(pointArg)]
     */
    getPointsByArg(pointArg: number | string | Date): Array<basePointObject>;
    /**
     * [descr:baseSeriesObject.getVisiblePoints()]
     */
    getVisiblePoints(): Array<basePointObject>;
    /**
     * [descr:baseSeriesObject.hide()]
     */
    hide(): void;
    /**
     * [descr:baseSeriesObject.hover()]
     */
    hover(): void;
    /**
     * [descr:baseSeriesObject.isHovered()]
     */
    isHovered(): boolean;
    /**
     * [descr:baseSeriesObject.isSelected()]
     */
    isSelected(): boolean;
    /**
     * [descr:baseSeriesObject.isVisible()]
     */
    isVisible(): boolean;
    /**
     * [descr:baseSeriesObject.name]
     */
    name?: any;
    /**
     * [descr:baseSeriesObject.select()]
     */
    select(): void;
    /**
     * [descr:baseSeriesObject.selectPoint(point)]
     */
    selectPoint(point: basePointObject): void;
    /**
     * [descr:baseSeriesObject.show()]
     */
    show(): void;
    /**
     * [descr:baseSeriesObject.tag]
     */
    tag?: any;
    /**
     * [descr:baseSeriesObject.type]
     */
    type?: string;
  }
  /**
   * [descr:BaseSparkline]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class BaseSparkline<TProperties> extends BaseWidget<TProperties> {
    /**
     * [descr:BaseSparkline.hideLoadingIndicator()]
     */
    hideLoadingIndicator(): void;
    /**
     * [descr:BaseSparkline.showLoadingIndicator()]
     */
    showLoadingIndicator(): void;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseSparklineOptions<TComponent>
    extends BaseWidgetOptions<TComponent> {
    /**
     * [descr:BaseSparklineOptions.export]
     */
    export?: BaseWidgetExport;
    /**
     * [descr:BaseSparklineOptions.loadingIndicator]
     */
    loadingIndicator?: BaseWidgetLoadingIndicator;
    /**
     * [descr:BaseSparklineOptions.onTooltipHidden]
     */
    onTooltipHidden?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:BaseSparklineOptions.onTooltipShown]
     */
    onTooltipShown?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:BaseSparklineOptions.redrawOnResize]
     */
    redrawOnResize?: boolean;
    /**
     * [descr:BaseSparklineOptions.title]
     */
    title?: BaseWidgetTitle | string;
    /**
     * [descr:BaseSparklineOptions.tooltip]
     */
    tooltip?: BaseSparklineTooltip;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseSparklineTooltip extends BaseWidgetTooltip {
    /**
     * [descr:BaseSparklineOptions.tooltip.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          pointsInfo: any,
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:BaseSparklineOptions.tooltip.customizeTooltip]
     */
    customizeTooltip?: (pointsInfo: any) => any;
    /**
     * [descr:BaseSparklineOptions.tooltip.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:BaseSparklineOptions.tooltip.interactive]
     */
    interactive?: boolean;
  }
  /**
   * [descr:BaseWidget]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export class BaseWidget<TProperties> extends DOMComponent<TProperties> {
    /**
     * [descr:BaseWidget.defaultOptions(rule)]
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    static defaultOptions<TProperties>(
      rule: DevExpress.core.DefaultOptionsRule<TProperties>
    ): void;
    /**
     * [descr:BaseWidget.exportTo(fileName, format)]
     */
    exportTo(fileName: string, format: string): void;
    /**
     * [descr:BaseWidget.getSize()]
     */
    getSize(): BaseWidgetSize;
    /**
     * [descr:BaseWidget.hideLoadingIndicator()]
     */
    hideLoadingIndicator(): void;
    /**
     * [descr:BaseWidget.print()]
     */
    print(): void;
    /**
     * [descr:BaseWidget.render()]
     */
    render(): void;
    /**
     * [descr:BaseWidget.showLoadingIndicator()]
     */
    showLoadingIndicator(): void;
    /**
     * [descr:BaseWidget.svg()]
     */
    svg(): string;
  }
  module BaseWidget {
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface ExportInfo {
      readonly fileName: string;
      readonly format: string;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type FileSavingEventInfo<T> = DevExpress.events.Cancelable & {
      readonly component: T;
      readonly element: DevExpress.core.DxElement;
      readonly fileName: string;
      readonly format: string;
      readonly data: Blob;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface IncidentInfo {
      readonly target: any;
    }
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type VizTextOverflowType = 'ellipsis' | 'hide' | 'none';
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type WordWrapType = 'normal' | 'breakWord' | 'none';
  }
  /**
   * [descr:BaseWidgetAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetAnnotationConfig {
    /**
     * [descr:BaseWidgetAnnotationConfig.allowDragging]
     */
    allowDragging?: boolean;
    /**
     * [descr:BaseWidgetAnnotationConfig.arrowLength]
     */
    arrowLength?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.arrowWidth]
     */
    arrowWidth?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.border]
     */
    border?: {
      /**
       * [descr:BaseWidgetAnnotationConfig.border.color]
       */
      color?: string;
      /**
       * [descr:BaseWidgetAnnotationConfig.border.cornerRadius]
       */
      cornerRadius?: number;
      /**
       * [descr:BaseWidgetAnnotationConfig.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:BaseWidgetAnnotationConfig.border.opacity]
       */
      opacity?: number;
      /**
       * [descr:BaseWidgetAnnotationConfig.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:BaseWidgetAnnotationConfig.border.width]
       */
      width?: number;
    };
    /**
     * [descr:BaseWidgetAnnotationConfig.color]
     */
    color?: string;
    /**
     * [descr:BaseWidgetAnnotationConfig.data]
     */
    data?: any;
    /**
     * [descr:BaseWidgetAnnotationConfig.description]
     */
    description?: string;
    /**
     * [descr:BaseWidgetAnnotationConfig.font]
     */
    font?: Font;
    /**
     * [descr:BaseWidgetAnnotationConfig.height]
     */
    height?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.image]
     */
    image?:
      | string
      | {
          /**
           * [descr:BaseWidgetAnnotationConfig.image.height]
           */
          height?: number;
          /**
           * [descr:BaseWidgetAnnotationConfig.image.url]
           */
          url?: string;
          /**
           * [descr:BaseWidgetAnnotationConfig.image.width]
           */
          width?: number;
        };
    /**
     * [descr:BaseWidgetAnnotationConfig.offsetX]
     */
    offsetX?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.offsetY]
     */
    offsetY?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.opacity]
     */
    opacity?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.paddingLeftRight]
     */
    paddingLeftRight?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.paddingTopBottom]
     */
    paddingTopBottom?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.shadow]
     */
    shadow?: {
      /**
       * [descr:BaseWidgetAnnotationConfig.shadow.blur]
       */
      blur?: number;
      /**
       * [descr:BaseWidgetAnnotationConfig.shadow.color]
       */
      color?: string;
      /**
       * [descr:BaseWidgetAnnotationConfig.shadow.offsetX]
       */
      offsetX?: number;
      /**
       * [descr:BaseWidgetAnnotationConfig.shadow.offsetY]
       */
      offsetY?: number;
      /**
       * [descr:BaseWidgetAnnotationConfig.shadow.opacity]
       */
      opacity?: number;
    };
    /**
     * [descr:BaseWidgetAnnotationConfig.text]
     */
    text?: string;
    /**
     * [descr:BaseWidgetAnnotationConfig.textOverflow]
     */
    textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
    /**
     * [descr:BaseWidgetAnnotationConfig.tooltipEnabled]
     */
    tooltipEnabled?: boolean;
    /**
     * [descr:BaseWidgetAnnotationConfig.type]
     */
    type?: 'text' | 'image' | 'custom';
    /**
     * [descr:BaseWidgetAnnotationConfig.width]
     */
    width?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.wordWrap]
     */
    wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
    /**
     * [descr:BaseWidgetAnnotationConfig.x]
     */
    x?: number;
    /**
     * [descr:BaseWidgetAnnotationConfig.y]
     */
    y?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetExport {
    /**
     * [descr:BaseWidgetOptions.export.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:BaseWidgetOptions.export.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:BaseWidgetOptions.export.fileName]
     */
    fileName?: string;
    /**
     * [descr:BaseWidgetOptions.export.formats]
     */
    formats?: Array<'GIF' | 'JPEG' | 'PDF' | 'PNG' | 'SVG'>;
    /**
     * [descr:BaseWidgetOptions.export.margin]
     */
    margin?: number;
    /**
     * [descr:BaseWidgetOptions.export.printingEnabled]
     */
    printingEnabled?: boolean;
    /**
     * [descr:BaseWidgetOptions.export.proxyUrl]
     * @deprecated [depNote:BaseWidgetOptions.export.proxyUrl]
     */
    proxyUrl?: string;
    /**
     * [descr:BaseWidgetOptions.export.svgToCanvas]
     */
    svgToCanvas?: (
      svg: SVGElement,
      canvas: HTMLCanvasElement
    ) => PromiseLike<void>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetLoadingIndicator {
    /**
     * [descr:BaseWidgetOptions.loadingIndicator.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:BaseWidgetOptions.loadingIndicator.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:BaseWidgetOptions.loadingIndicator.font]
     */
    font?: Font;
    /**
     * [descr:BaseWidgetOptions.loadingIndicator.show]
     */
    show?: boolean;
    /**
     * [descr:BaseWidgetOptions.loadingIndicator.text]
     */
    text?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetMargin {
    /**
     * [descr:BaseWidgetOptions.margin.bottom]
     */
    bottom?: number;
    /**
     * [descr:BaseWidgetOptions.margin.left]
     */
    left?: number;
    /**
     * [descr:BaseWidgetOptions.margin.right]
     */
    right?: number;
    /**
     * [descr:BaseWidgetOptions.margin.top]
     */
    top?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetOptions<TComponent>
    extends DOMComponentOptions<TComponent> {
    /**
     * [descr:BaseWidgetOptions.disabled]
     */
    disabled?: boolean;
    /**
     * [descr:BaseWidgetOptions.export]
     */
    export?: BaseWidgetExport;
    /**
     * [descr:BaseWidgetOptions.height]
     */
    height?: number | string | (() => number | string);
    /**
     * [descr:BaseWidgetOptions.loadingIndicator]
     */
    loadingIndicator?: BaseWidgetLoadingIndicator;
    /**
     * [descr:BaseWidgetOptions.margin]
     */
    margin?: BaseWidgetMargin;
    /**
     * [descr:BaseWidgetOptions.onDrawn]
     */
    onDrawn?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:BaseWidgetOptions.onExported]
     */
    onExported?: (e: DevExpress.events.EventInfo<TComponent>) => void;
    /**
     * [descr:BaseWidgetOptions.onExporting]
     */
    onExporting?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseWidget.ExportInfo
    ) => void;
    /**
     * [descr:BaseWidgetOptions.onFileSaving]
     */
    onFileSaving?: (
      e: DevExpress.viz.BaseWidget.FileSavingEventInfo<TComponent>
    ) => void;
    /**
     * [descr:BaseWidgetOptions.onIncidentOccurred]
     */
    onIncidentOccurred?: (
      e: DevExpress.events.EventInfo<TComponent> &
        DevExpress.viz.BaseWidget.IncidentInfo
    ) => void;
    /**
     * [descr:BaseWidgetOptions.pathModified]
     */
    pathModified?: boolean;
    /**
     * [descr:BaseWidgetOptions.redrawOnResize]
     */
    redrawOnResize?: boolean;
    /**
     * [descr:BaseWidgetOptions.rtlEnabled]
     */
    rtlEnabled?: boolean;
    /**
     * [descr:BaseWidgetOptions.size]
     */
    size?: BaseWidgetSize;
    /**
     * [descr:BaseWidgetOptions.theme]
     */
    theme?:
      | 'generic.dark'
      | 'generic.light'
      | 'generic.contrast'
      | 'generic.carmine'
      | 'generic.darkmoon'
      | 'generic.darkviolet'
      | 'generic.greenmist'
      | 'generic.softblue'
      | 'material.blue.light'
      | 'material.lime.light'
      | 'material.orange.light'
      | 'material.purple.light'
      | 'material.teal.light';
    /**
     * [descr:BaseWidgetOptions.title]
     */
    title?: BaseWidgetTitle | string;
    /**
     * [descr:BaseWidgetOptions.tooltip]
     */
    tooltip?: BaseWidgetTooltip;
    /**
     * [descr:BaseWidgetOptions.width]
     */
    width?: number | string | (() => number | string);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetSize {
    /**
     * [descr:BaseWidgetOptions.size.height]
     */
    height?: number;
    /**
     * [descr:BaseWidgetOptions.size.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetTitle {
    /**
     * [descr:BaseWidgetOptions.title.font]
     */
    font?: Font;
    /**
     * [descr:BaseWidgetOptions.title.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:BaseWidgetOptions.title.margin]
     */
    margin?:
      | number
      | {
          /**
           * [descr:BaseWidgetOptions.title.margin.bottom]
           */
          bottom?: number;
          /**
           * [descr:BaseWidgetOptions.title.margin.left]
           */
          left?: number;
          /**
           * [descr:BaseWidgetOptions.title.margin.right]
           */
          right?: number;
          /**
           * [descr:BaseWidgetOptions.title.margin.top]
           */
          top?: number;
        };
    /**
     * [descr:BaseWidgetOptions.title.placeholderSize]
     */
    placeholderSize?: number;
    /**
     * [descr:BaseWidgetOptions.title.subtitle]
     */
    subtitle?:
      | {
          /**
           * [descr:BaseWidgetOptions.title.subtitle.font]
           */
          font?: Font;
          /**
           * [descr:BaseWidgetOptions.title.subtitle.offset]
           */
          offset?: number;
          /**
           * [descr:BaseWidgetOptions.title.subtitle.text]
           */
          text?: string;
          /**
           * [descr:BaseWidgetOptions.title.subtitle.textOverflow]
           */
          textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
          /**
           * [descr:BaseWidgetOptions.title.subtitle.wordWrap]
           */
          wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
        }
      | string;
    /**
     * [descr:BaseWidgetOptions.title.text]
     */
    text?: string;
    /**
     * [descr:BaseWidgetOptions.title.textOverflow]
     */
    textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
    /**
     * [descr:BaseWidgetOptions.title.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'top';
    /**
     * [descr:BaseWidgetOptions.title.wordWrap]
     */
    wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface BaseWidgetTooltip {
    /**
     * [descr:BaseWidgetOptions.tooltip.arrowLength]
     */
    arrowLength?: number;
    /**
     * [descr:BaseWidgetOptions.tooltip.border]
     */
    border?: {
      /**
       * [descr:BaseWidgetOptions.tooltip.border.color]
       */
      color?: string;
      /**
       * [descr:BaseWidgetOptions.tooltip.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:BaseWidgetOptions.tooltip.border.opacity]
       */
      opacity?: number;
      /**
       * [descr:BaseWidgetOptions.tooltip.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:BaseWidgetOptions.tooltip.border.width]
       */
      width?: number;
    };
    /**
     * [descr:BaseWidgetOptions.tooltip.color]
     */
    color?: string;
    /**
     * [descr:BaseWidgetOptions.tooltip.container]
     */
    container?: string | DevExpress.core.UserDefinedElement;
    /**
     * [descr:BaseWidgetOptions.tooltip.cornerRadius]
     */
    cornerRadius?: number;
    /**
     * [descr:BaseWidgetOptions.tooltip.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:BaseWidgetOptions.tooltip.font]
     */
    font?: Font;
    /**
     * [descr:BaseWidgetOptions.tooltip.format]
     */
    format?: DevExpress.ui.Format;
    /**
     * [descr:BaseWidgetOptions.tooltip.opacity]
     */
    opacity?: number;
    /**
     * [descr:BaseWidgetOptions.tooltip.paddingLeftRight]
     */
    paddingLeftRight?: number;
    /**
     * [descr:BaseWidgetOptions.tooltip.paddingTopBottom]
     */
    paddingTopBottom?: number;
    /**
     * [descr:BaseWidgetOptions.tooltip.shadow]
     */
    shadow?: {
      /**
       * [descr:BaseWidgetOptions.tooltip.shadow.blur]
       */
      blur?: number;
      /**
       * [descr:BaseWidgetOptions.tooltip.shadow.color]
       */
      color?: string;
      /**
       * [descr:BaseWidgetOptions.tooltip.shadow.offsetX]
       */
      offsetX?: number;
      /**
       * [descr:BaseWidgetOptions.tooltip.shadow.offsetY]
       */
      offsetY?: number;
      /**
       * [descr:BaseWidgetOptions.tooltip.shadow.opacity]
       */
      opacity?: number;
    };
    /**
     * [descr:BaseWidgetOptions.tooltip.zIndex]
     */
    zIndex?: number;
  }
  /**
   * [descr:chartAxisObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface chartAxisObject {
    /**
     * [descr:chartAxisObject.visualRange()]
     */
    visualRange(): VizRange;
    /**
     * [descr:chartAxisObject.visualRange(visualRange)]
     */
    visualRange(visualRange: Array<number | string | Date> | VizRange): void;
  }
  /**
   * [descr:chartPointAggregationInfoObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface chartPointAggregationInfoObject {
    /**
     * [descr:chartPointAggregationInfoObject.aggregationInterval]
     */
    aggregationInterval?: any;
    /**
     * [descr:chartPointAggregationInfoObject.data]
     */
    data?: Array<any>;
    /**
     * [descr:chartPointAggregationInfoObject.intervalEnd]
     */
    intervalEnd?: any;
    /**
     * [descr:chartPointAggregationInfoObject.intervalStart]
     */
    intervalStart?: any;
  }
  /**
   * [descr:chartPointObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface chartPointObject extends basePointObject {
    /**
     * [descr:chartPointObject.aggregationInfo]
     */
    aggregationInfo?: chartPointAggregationInfoObject;
    /**
     * [descr:chartPointObject.getBoundingRect()]
     */
    getBoundingRect(): any;
    /**
     * [descr:chartPointObject.originalCloseValue]
     */
    originalCloseValue?: number | string;
    /**
     * [descr:chartPointObject.originalHighValue]
     */
    originalHighValue?: number | string;
    /**
     * [descr:chartPointObject.originalLowValue]
     */
    originalLowValue?: number | string;
    /**
     * [descr:chartPointObject.originalMinValue]
     */
    originalMinValue?: string | number | Date;
    /**
     * [descr:chartPointObject.originalOpenValue]
     */
    originalOpenValue?: number | string;
    /**
     * [descr:chartPointObject.size]
     */
    size?: number | string;
  }
  /**
   * [descr:ChartSeries]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ChartSeries extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:ChartSeries.name]
     */
    name?: string;
    /**
     * [descr:ChartSeries.tag]
     */
    tag?: any;
    /**
     * [descr:ChartSeries.type]
     */
    type?: ChartSeriesType;
  }
  /**
   * [descr:chartSeriesObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface chartSeriesObject extends baseSeriesObject {
    /**
     * [descr:chartSeriesObject.axis]
     */
    axis?: string;
    /**
     * [descr:chartSeriesObject.barOverlapGroup]
     */
    barOverlapGroup?: string;
    /**
     * [descr:chartSeriesObject.getArgumentAxis()]
     */
    getArgumentAxis(): chartAxisObject;
    /**
     * [descr:chartSeriesObject.getValueAxis()]
     */
    getValueAxis(): chartAxisObject;
    /**
     * [descr:chartSeriesObject.pane]
     */
    pane?: string;
    /**
     * [descr:chartSeriesObject.stack]
     */
    stack?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type ChartSeriesType =
    | 'area'
    | 'bar'
    | 'bubble'
    | 'candlestick'
    | 'fullstackedarea'
    | 'fullstackedbar'
    | 'fullstackedline'
    | 'fullstackedspline'
    | 'fullstackedsplinearea'
    | 'line'
    | 'rangearea'
    | 'rangebar'
    | 'scatter'
    | 'spline'
    | 'splinearea'
    | 'stackedarea'
    | 'stackedbar'
    | 'stackedline'
    | 'stackedspline'
    | 'stackedsplinearea'
    | 'steparea'
    | 'stepline'
    | 'stock';
  /**
   * [descr:CommonIndicator]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface CommonIndicator {
    /**
     * [descr:CommonIndicator.arrowLength]
     */
    arrowLength?: number;
    /**
     * [descr:CommonIndicator.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:CommonIndicator.baseValue]
     */
    baseValue?: number;
    /**
     * [descr:CommonIndicator.beginAdaptingAtRadius]
     */
    beginAdaptingAtRadius?: number;
    /**
     * [descr:CommonIndicator.color]
     */
    color?: string;
    /**
     * [descr:CommonIndicator.horizontalOrientation]
     */
    horizontalOrientation?: 'left' | 'right';
    /**
     * [descr:CommonIndicator.indentFromCenter]
     */
    indentFromCenter?: number;
    /**
     * [descr:CommonIndicator.length]
     */
    length?: number;
    /**
     * [descr:CommonIndicator.offset]
     */
    offset?: number;
    /**
     * [descr:CommonIndicator.palette]
     */
    palette?: Array<string> | PaletteType;
    /**
     * [descr:CommonIndicator.secondColor]
     */
    secondColor?: string;
    /**
     * [descr:CommonIndicator.secondFraction]
     */
    secondFraction?: number;
    /**
     * [descr:CommonIndicator.size]
     */
    size?: number;
    /**
     * [descr:CommonIndicator.spindleGapSize]
     */
    spindleGapSize?: number;
    /**
     * [descr:CommonIndicator.spindleSize]
     */
    spindleSize?: number;
    /**
     * [descr:CommonIndicator.text]
     */
    text?: {
      /**
       * [descr:CommonIndicator.text.customizeText]
       */
      customizeText?: (indicatedValue: {
        value?: number;
        valueText?: string;
      }) => string;
      /**
       * [descr:CommonIndicator.text.font]
       */
      font?: Font;
      /**
       * [descr:CommonIndicator.text.format]
       */
      format?: DevExpress.ui.Format;
      /**
       * [descr:CommonIndicator.text.indent]
       */
      indent?: number;
    };
    /**
     * [descr:CommonIndicator.verticalOrientation]
     */
    verticalOrientation?: 'bottom' | 'top';
    /**
     * [descr:CommonIndicator.width]
     */
    width?: number;
  }
  /**
   * [descr:viz.currentPalette()]
   */
  export function currentPalette(): string;
  /**
   * [descr:viz.currentPalette(paletteName)]
   */
  export function currentPalette(paletteName: string): void;
  /**
   * [descr:viz.currentTheme()]
   */
  export function currentTheme(): string;
  /**
   * [descr:viz.currentTheme(platform, colorScheme)]
   */
  export function currentTheme(platform: string, colorScheme: string): void;
  /**
   * [descr:viz.currentTheme(theme)]
   */
  export function currentTheme(theme: string): void;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type DashStyleType = 'dash' | 'dot' | 'longDash' | 'solid';
  /**
   * [descr:dxBarGauge]
   */
  export class dxBarGauge extends BaseWidget<dxBarGaugeOptions> {
    /**
     * [descr:dxBarGauge.values()]
     */
    values(): Array<number>;
    /**
     * [descr:dxBarGauge.values(newValues)]
     */
    values(values: Array<number>): void;
  }
  module dxBarGauge {
    export type DisposingEvent = DevExpress.events.EventInfo<dxBarGauge>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxBarGauge>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxBarGauge>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxBarGauge> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxBarGauge>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxBarGauge> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxBarGauge>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxBarGauge> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxBarGaugeOptions;
    export type TooltipHiddenEvent = DevExpress.events.EventInfo<dxBarGauge> &
      TooltipInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface TooltipInfo {
      target?: any;
    }
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxBarGauge> &
      TooltipInfo;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxBarGaugeLegend extends BaseLegend {
    /**
     * [descr:dxBarGaugeOptions.legend.customizeHint]
     */
    customizeHint?: (arg: { item?: BarGaugeBarInfo; text?: string }) => string;
    /**
     * [descr:dxBarGaugeOptions.legend.customizeItems]
     */
    customizeItems?: (
      items: Array<DevExpress.viz.dxBarGauge.LegendItem>
    ) => Array<DevExpress.viz.dxBarGauge.LegendItem>;
    /**
     * [descr:dxBarGaugeOptions.legend.customizeText]
     */
    customizeText?: (arg: { item?: BarGaugeBarInfo; text?: string }) => string;
    /**
     * [descr:dxBarGaugeOptions.legend.itemTextFormat]
     */
    itemTextFormat?: DevExpress.ui.Format;
    /**
     * [descr:dxBarGaugeOptions.legend.markerTemplate]
     */
    markerTemplate?:
      | DevExpress.core.template
      | ((
          legendItem: DevExpress.viz.dxBarGauge.LegendItem,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxBarGaugeOptions.legend.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxBarGaugeLoadingIndicator
    extends BaseWidgetLoadingIndicator {
    /**
     * [descr:dxBarGaugeOptions.loadingIndicator.enabled]
     */
    enabled?: boolean;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxBarGaugeOptions extends BaseWidgetOptions<dxBarGauge> {
    /**
     * [descr:dxBarGaugeOptions.animation]
     */
    animation?: any;
    /**
     * [descr:dxBarGaugeOptions.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:dxBarGaugeOptions.barSpacing]
     */
    barSpacing?: number;
    /**
     * [descr:dxBarGaugeOptions.baseValue]
     */
    baseValue?: number;
    /**
     * [descr:dxBarGaugeOptions.endValue]
     */
    endValue?: number;
    /**
     * [descr:dxBarGaugeOptions.geometry]
     */
    geometry?: {
      /**
       * [descr:dxBarGaugeOptions.geometry.endAngle]
       */
      endAngle?: number;
      /**
       * [descr:dxBarGaugeOptions.geometry.startAngle]
       */
      startAngle?: number;
    };
    /**
     * [descr:dxBarGaugeOptions.label]
     */
    label?: {
      /**
       * [descr:dxBarGaugeOptions.label.connectorColor]
       */
      connectorColor?: string;
      /**
       * [descr:dxBarGaugeOptions.label.connectorWidth]
       */
      connectorWidth?: number;
      /**
       * [descr:dxBarGaugeOptions.label.customizeText]
       */
      customizeText?: (barValue: {
        value?: number;
        valueText?: string;
      }) => string;
      /**
       * [descr:dxBarGaugeOptions.label.font]
       */
      font?: Font;
      /**
       * [descr:dxBarGaugeOptions.label.format]
       */
      format?: DevExpress.ui.Format;
      /**
       * [descr:dxBarGaugeOptions.label.indent]
       */
      indent?: number;
      /**
       * [descr:dxBarGaugeOptions.label.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxBarGaugeOptions.legend]
     */
    legend?: dxBarGaugeLegend;
    /**
     * [descr:dxBarGaugeOptions.loadingIndicator]
     */
    loadingIndicator?: dxBarGaugeLoadingIndicator;
    /**
     * [descr:dxBarGaugeOptions.onTooltipHidden]
     */
    onTooltipHidden?: (e: DevExpress.viz.dxBarGauge.TooltipHiddenEvent) => void;
    /**
     * [descr:dxBarGaugeOptions.onTooltipShown]
     */
    onTooltipShown?: (e: DevExpress.viz.dxBarGauge.TooltipShownEvent) => void;
    /**
     * [descr:dxBarGaugeOptions.palette]
     */
    palette?: Array<string> | PaletteType;
    /**
     * [descr:dxBarGaugeOptions.paletteExtensionMode]
     */
    paletteExtensionMode?: PaletteExtensionModeType;
    /**
     * [descr:dxBarGaugeOptions.relativeInnerRadius]
     */
    relativeInnerRadius?: number;
    /**
     * [descr:dxBarGaugeOptions.resolveLabelOverlapping]
     */
    resolveLabelOverlapping?: 'hide' | 'none';
    /**
     * [descr:dxBarGaugeOptions.startValue]
     */
    startValue?: number;
    /**
     * [descr:dxBarGaugeOptions.tooltip]
     */
    tooltip?: dxBarGaugeTooltip;
    /**
     * [descr:dxBarGaugeOptions.values]
     */
    values?: Array<number>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxBarGaugeTooltip extends BaseWidgetTooltip {
    /**
     * [descr:dxBarGaugeOptions.tooltip.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          scaleValue: { value?: number; valueText?: string; index?: number },
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxBarGaugeOptions.tooltip.customizeTooltip]
     */
    customizeTooltip?: (scaleValue: {
      value?: number;
      valueText?: string;
      index?: number;
    }) => any;
    /**
     * [descr:dxBarGaugeOptions.tooltip.interactive]
     */
    interactive?: boolean;
  }
  /**
   * [descr:dxBullet]
   */
  export class dxBullet extends BaseSparkline<dxBulletOptions> {}
  module dxBullet {
    export type DisposingEvent = DevExpress.events.EventInfo<dxBullet>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxBullet>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxBullet>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxBullet> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxBullet>;
    export type IncidentOccurredEvent = DevExpress.events.EventInfo<dxBullet> &
      DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxBullet>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxBullet> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxBulletOptions;
    export type TooltipHiddenEvent = DevExpress.events.EventInfo<dxBullet>;
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxBullet>;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxBulletOptions extends BaseSparklineOptions<dxBullet> {
    /**
     * [descr:dxBulletOptions.color]
     */
    color?: string;
    /**
     * [descr:dxBulletOptions.endScaleValue]
     */
    endScaleValue?: number;
    /**
     * [descr:dxBulletOptions.showTarget]
     */
    showTarget?: boolean;
    /**
     * [descr:dxBulletOptions.showZeroLevel]
     */
    showZeroLevel?: boolean;
    /**
     * [descr:dxBulletOptions.startScaleValue]
     */
    startScaleValue?: number;
    /**
     * [descr:dxBulletOptions.target]
     */
    target?: number;
    /**
     * [descr:dxBulletOptions.targetColor]
     */
    targetColor?: string;
    /**
     * [descr:dxBulletOptions.targetWidth]
     */
    targetWidth?: number;
    /**
     * [descr:dxBulletOptions.value]
     */
    value?: number;
  }
  /**
   * [descr:dxChart]
   */
  export class dxChart extends BaseChart<dxChartOptions> {
    /**
     * [descr:dxChart.getArgumentAxis()]
     */
    getArgumentAxis(): chartAxisObject;
    /**
     * [descr:dxChart.getValueAxis()]
     */
    getValueAxis(): chartAxisObject;
    /**
     * [descr:dxChart.getValueAxis(name)]
     */
    getValueAxis(name: string): chartAxisObject;
    /**
     * [descr:dxChart.resetVisualRange()]
     */
    resetVisualRange(): void;
    /**
     * [descr:dxChart.zoomArgument(startValue,endValue)]
     */
    zoomArgument(
      startValue: number | Date | string,
      endValue: number | Date | string
    ): void;
  }
  module dxChart {
    export type ArgumentAxisClickEvent = DevExpress.events.NativeEventInfo<
      dxChart,
      MouseEvent | PointerEvent
    > & {
      readonly argument: Date | number | string;
    };
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type ChartSingleValueSeriesAggregationMethodType =
      | 'avg'
      | 'count'
      | 'max'
      | 'min'
      | 'sum'
      | 'custom';
    export type DisposingEvent = DevExpress.events.EventInfo<dxChart>;
    export type DoneEvent = DevExpress.events.EventInfo<dxChart>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxChart>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxChart>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxChart> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxChart>;
    export type IncidentOccurredEvent = DevExpress.events.EventInfo<dxChart> &
      DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxChart>;
    export type LegendClickEvent = DevExpress.events.NativeEventInfo<
      dxChart,
      MouseEvent | PointerEvent
    > & {
      readonly target: chartSeriesObject;
    };
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxChart> &
      DevExpress.events.ChangedOptionInfo;
    export type PointClickEvent = DevExpress.events.NativeEventInfo<
      dxChart,
      MouseEvent | PointerEvent
    > &
      DevExpress.viz.BaseChart.PointInteractionInfo;
    export type PointHoverChangedEvent = DevExpress.events.EventInfo<dxChart> &
      DevExpress.viz.BaseChart.PointInteractionInfo;
    export type PointSelectionChangedEvent =
      DevExpress.events.EventInfo<dxChart> &
        DevExpress.viz.BaseChart.PointInteractionInfo;
    export type Properties = dxChartOptions;
    export type SeriesClickEvent = DevExpress.events.NativeEventInfo<
      dxChart,
      MouseEvent | PointerEvent
    > & {
      readonly target: chartSeriesObject;
    };
    export type SeriesHoverChangedEvent = DevExpress.events.EventInfo<dxChart> &
      SeriesInteractionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface SeriesInteractionInfo {
      target: chartSeriesObject;
    }
    export type SeriesSelectionChangedEvent =
      DevExpress.events.EventInfo<dxChart> & SeriesInteractionInfo;
    export type TooltipHiddenEvent = DevExpress.events.EventInfo<dxChart> &
      DevExpress.viz.BaseChart.TooltipInfo;
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxChart> &
      DevExpress.viz.BaseChart.TooltipInfo;
    export type ZoomEndEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxChart, MouseEvent | TouchEvent> & {
        readonly rangeStart: Date | number;
        readonly rangeEnd: Date | number;
        readonly axis: chartAxisObject;
        readonly range: VizRange;
        readonly previousRange: VizRange;
        readonly actionType: 'zoom' | 'pan';
        readonly zoomFactor: number;
        readonly shift: number;
      };
    export type ZoomStartEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<dxChart, MouseEvent | TouchEvent> & {
        readonly axis: chartAxisObject;
        readonly range: VizRange;
        readonly actionType?: 'zoom' | 'pan';
      };
  }
  /**
   * [descr:dxChartAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartAnnotationConfig
    extends dxChartCommonAnnotationConfig {
    /**
     * [descr:dxChartAnnotationConfig.name]
     */
    name?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxis extends dxChartCommonAxisSettings {
    /**
     * [descr:dxChartOptions.argumentAxis.aggregateByCategory]
     */
    aggregateByCategory?: boolean;
    /**
     * [descr:dxChartOptions.argumentAxis.aggregationGroupWidth]
     */
    aggregationGroupWidth?: number;
    /**
     * [descr:dxChartOptions.argumentAxis.aggregationInterval]
     */
    aggregationInterval?: VizTimeInterval;
    /**
     * [descr:dxChartOptions.argumentAxis.argumentType]
     */
    argumentType?: 'datetime' | 'numeric' | 'string';
    /**
     * [descr:dxChartOptions.argumentAxis.axisDivisionFactor]
     */
    axisDivisionFactor?: number;
    /**
     * [descr:dxChartOptions.argumentAxis.breaks]
     */
    breaks?: Array<ScaleBreak>;
    /**
     * [descr:dxChartOptions.argumentAxis.categories]
     */
    categories?: Array<number | string | Date>;
    /**
     * [descr:dxChartOptions.argumentAxis.constantLineStyle]
     */
    constantLineStyle?: dxChartArgumentAxisConstantLineStyle;
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines]
     */
    constantLines?: Array<dxChartArgumentAxisConstantLines>;
    /**
     * [descr:dxChartOptions.argumentAxis.endOnTick]
     */
    endOnTick?: boolean;
    /**
     * [descr:dxChartOptions.argumentAxis.holidays]
     */
    holidays?: Array<Date | string> | Array<number>;
    /**
     * [descr:dxChartOptions.argumentAxis.hoverMode]
     */
    hoverMode?: 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartOptions.argumentAxis.label]
     */
    label?: dxChartArgumentAxisLabel;
    /**
     * [descr:dxChartOptions.argumentAxis.linearThreshold]
     */
    linearThreshold?: number;
    /**
     * [descr:dxChartOptions.argumentAxis.logarithmBase]
     */
    logarithmBase?: number;
    /**
     * [descr:dxChartOptions.argumentAxis.minVisualRangeLength]
     */
    minVisualRangeLength?: VizTimeInterval;
    /**
     * [descr:dxChartOptions.argumentAxis.minorTickCount]
     */
    minorTickCount?: number;
    /**
     * [descr:dxChartOptions.argumentAxis.minorTickInterval]
     */
    minorTickInterval?: VizTimeInterval;
    /**
     * [descr:dxChartOptions.argumentAxis.position]
     */
    position?: 'bottom' | 'left' | 'right' | 'top';
    /**
     * [descr:dxChartOptions.argumentAxis.customPosition]
     */
    customPosition?: number | Date | string;
    /**
     * [descr:dxChartOptions.argumentAxis.customPositionAxis]
     */
    customPositionAxis?: string;
    /**
     * [descr:dxChartOptions.argumentAxis.offset]
     */
    offset?: number;
    /**
     * [descr:dxChartOptions.argumentAxis.singleWorkdays]
     */
    singleWorkdays?: Array<Date | string> | Array<number>;
    /**
     * [descr:dxChartOptions.argumentAxis.strips]
     */
    strips?: Array<dxChartArgumentAxisStrips>;
    /**
     * [descr:dxChartOptions.argumentAxis.tickInterval]
     */
    tickInterval?: VizTimeInterval;
    /**
     * [descr:dxChartOptions.argumentAxis.title]
     */
    title?: dxChartArgumentAxisTitle;
    /**
     * [descr:dxChartOptions.argumentAxis.type]
     */
    type?: 'continuous' | 'discrete' | 'logarithmic';
    /**
     * [descr:dxChartOptions.argumentAxis.visualRange]
     */
    visualRange?: VizRange | Array<number | string | Date>;
    /**
     * [descr:dxChartOptions.argumentAxis.visualRangeUpdateMode]
     */
    visualRangeUpdateMode?: 'auto' | 'keep' | 'reset' | 'shift';
    /**
     * [descr:dxChartOptions.argumentAxis.wholeRange]
     */
    wholeRange?: VizRange | Array<number | string | Date>;
    /**
     * [descr:dxChartOptions.argumentAxis.workWeek]
     */
    workWeek?: Array<number>;
    /**
     * [descr:dxChartOptions.argumentAxis.workdaysOnly]
     */
    workdaysOnly?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisConstantLines
    extends dxChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines.displayBehindSeries]
     */
    displayBehindSeries?: boolean;
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines.extendAxis]
     */
    extendAxis?: boolean;
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines.label]
     */
    label?: dxChartArgumentAxisConstantLinesLabel;
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines.value]
     */
    value?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisConstantLinesLabel
    extends dxChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines.label.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines.label.text]
     */
    text?: string;
    /**
     * [descr:dxChartOptions.argumentAxis.constantLines.label.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'center' | 'top';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisConstantLineStyle
    extends dxChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxChartOptions.argumentAxis.constantLineStyle.label]
     */
    label?: dxChartArgumentAxisConstantLineStyleLabel;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisConstantLineStyleLabel
    extends dxChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxChartOptions.argumentAxis.constantLineStyle.label.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartOptions.argumentAxis.constantLineStyle.label.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'center' | 'top';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisLabel
    extends dxChartCommonAxisSettingsLabel {
    /**
     * [descr:dxChartOptions.argumentAxis.label.customizeHint]
     */
    customizeHint?: (argument: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxChartOptions.argumentAxis.label.customizeText]
     */
    customizeText?: (argument: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxChartOptions.argumentAxis.label.format]
     */
    format?: DevExpress.ui.Format;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisStrips
    extends dxChartCommonAxisSettingsStripStyle {
    /**
     * [descr:dxChartOptions.argumentAxis.strips.color]
     */
    color?: string;
    /**
     * [descr:dxChartOptions.argumentAxis.strips.endValue]
     */
    endValue?: number | Date | string;
    /**
     * [descr:dxChartOptions.argumentAxis.strips.label]
     */
    label?: dxChartArgumentAxisStripsLabel;
    /**
     * [descr:dxChartOptions.argumentAxis.strips.startValue]
     */
    startValue?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisStripsLabel
    extends dxChartCommonAxisSettingsStripStyleLabel {
    /**
     * [descr:dxChartOptions.argumentAxis.strips.label.text]
     */
    text?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartArgumentAxisTitle
    extends dxChartCommonAxisSettingsTitle {
    /**
     * [descr:dxChartOptions.argumentAxis.title.text]
     */
    text?: string;
  }
  /**
   * [descr:dxChartCommonAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAnnotationConfig
    extends BaseChartAnnotationConfig {
    /**
     * [descr:dxChartCommonAnnotationConfig.axis]
     */
    axis?: string;
    /**
     * [descr:dxChartCommonAnnotationConfig.customizeTooltip]
     */
    customizeTooltip?: (annotation: dxChartAnnotationConfig | any) => any;
    /**
     * [descr:dxChartCommonAnnotationConfig.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          annotation: dxChartAnnotationConfig | any,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxChartCommonAnnotationConfig.tooltipTemplate]
     */
    tooltipTemplate?:
      | DevExpress.core.template
      | ((
          annotation: dxChartAnnotationConfig | any,
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAxisSettings {
    /**
     * [descr:dxChartOptions.commonAxisSettings.allowDecimals]
     */
    allowDecimals?: boolean;
    /**
     * [descr:dxChartOptions.commonAxisSettings.breakStyle]
     */
    breakStyle?: {
      /**
       * [descr:dxChartOptions.commonAxisSettings.breakStyle.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.commonAxisSettings.breakStyle.line]
       */
      line?: 'straight' | 'waved';
      /**
       * [descr:dxChartOptions.commonAxisSettings.breakStyle.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartOptions.commonAxisSettings.color]
     */
    color?: string;
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle]
     */
    constantLineStyle?: dxChartCommonAxisSettingsConstantLineStyle;
    /**
     * [descr:dxChartOptions.commonAxisSettings.discreteAxisDivisionMode]
     */
    discreteAxisDivisionMode?: 'betweenLabels' | 'crossLabels';
    /**
     * [descr:dxChartOptions.commonAxisSettings.endOnTick]
     */
    endOnTick?: boolean;
    /**
     * [descr:dxChartOptions.commonAxisSettings.grid]
     */
    grid?: {
      /**
       * [descr:dxChartOptions.commonAxisSettings.grid.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.commonAxisSettings.grid.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.grid.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartOptions.commonAxisSettings.grid.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartOptions.commonAxisSettings.inverted]
     */
    inverted?: boolean;
    /**
     * [descr:dxChartOptions.commonAxisSettings.label]
     */
    label?: dxChartCommonAxisSettingsLabel;
    /**
     * [descr:dxChartOptions.commonAxisSettings.maxValueMargin]
     */
    maxValueMargin?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.minValueMargin]
     */
    minValueMargin?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.minorGrid]
     */
    minorGrid?: {
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorGrid.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorGrid.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorGrid.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorGrid.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartOptions.commonAxisSettings.minorTick]
     */
    minorTick?: {
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorTick.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorTick.length]
       */
      length?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorTick.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorTick.shift]
       */
      shift?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorTick.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartOptions.commonAxisSettings.minorTick.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartOptions.commonAxisSettings.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.placeholderSize]
     */
    placeholderSize?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.stripStyle]
     */
    stripStyle?: dxChartCommonAxisSettingsStripStyle;
    /**
     * [descr:dxChartOptions.commonAxisSettings.tick]
     */
    tick?: {
      /**
       * [descr:dxChartOptions.commonAxisSettings.tick.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.commonAxisSettings.tick.length]
       */
      length?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.tick.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.tick.shift]
       */
      shift?: number;
      /**
       * [descr:dxChartOptions.commonAxisSettings.tick.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartOptions.commonAxisSettings.tick.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartOptions.commonAxisSettings.title]
     */
    title?: dxChartCommonAxisSettingsTitle;
    /**
     * [descr:dxChartOptions.commonAxisSettings.valueMarginsEnabled]
     */
    valueMarginsEnabled?: boolean;
    /**
     * [descr:dxChartOptions.commonAxisSettings.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxChartOptions.commonAxisSettings.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.color]
     */
    color?: string;
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.label]
     */
    label?: dxChartCommonAxisSettingsConstantLineStyleLabel;
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.paddingLeftRight]
     */
    paddingLeftRight?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.paddingTopBottom]
     */
    paddingTopBottom?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.label.font]
     */
    font?: Font;
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.label.position]
     */
    position?: 'inside' | 'outside';
    /**
     * [descr:dxChartOptions.commonAxisSettings.constantLineStyle.label.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAxisSettingsLabel {
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          data: object,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.alignment]
     */
    alignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.displayMode]
     */
    displayMode?: 'rotate' | 'stagger' | 'standard';
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.font]
     */
    font?: Font;
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.indentFromAxis]
     */
    indentFromAxis?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.overlappingBehavior]
     */
    overlappingBehavior?: 'rotate' | 'stagger' | 'none' | 'hide';
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.position]
     */
    position?: 'inside' | 'outside' | 'bottom' | 'left' | 'right' | 'top';
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.rotationAngle]
     */
    rotationAngle?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.staggeringSpacing]
     */
    staggeringSpacing?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.textOverflow]
     */
    textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxChartOptions.commonAxisSettings.label.wordWrap]
     */
    wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAxisSettingsStripStyle {
    /**
     * [descr:dxChartOptions.commonAxisSettings.stripStyle.label]
     */
    label?: dxChartCommonAxisSettingsStripStyleLabel;
    /**
     * [descr:dxChartOptions.commonAxisSettings.stripStyle.paddingLeftRight]
     */
    paddingLeftRight?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.stripStyle.paddingTopBottom]
     */
    paddingTopBottom?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAxisSettingsStripStyleLabel {
    /**
     * [descr:dxChartOptions.commonAxisSettings.stripStyle.label.font]
     */
    font?: Font;
    /**
     * [descr:dxChartOptions.commonAxisSettings.stripStyle.label.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartOptions.commonAxisSettings.stripStyle.label.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'center' | 'top';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonAxisSettingsTitle {
    /**
     * [descr:dxChartOptions.commonAxisSettings.title.alignment]
     */
    alignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartOptions.commonAxisSettings.title.font]
     */
    font?: Font;
    /**
     * [descr:dxChartOptions.commonAxisSettings.title.margin]
     */
    margin?: number;
    /**
     * [descr:dxChartOptions.commonAxisSettings.title.textOverflow]
     */
    textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
    /**
     * [descr:dxChartOptions.commonAxisSettings.title.wordWrap]
     */
    wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonPaneSettings {
    /**
     * [descr:dxChartOptions.commonPaneSettings.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:dxChartOptions.commonPaneSettings.border]
     */
    border?: {
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.bottom]
       */
      bottom?: boolean;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.left]
       */
      left?: boolean;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.right]
       */
      right?: boolean;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.top]
       */
      top?: boolean;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartOptions.commonPaneSettings.border.width]
       */
      width?: number;
    };
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartCommonSeriesSettings
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartOptions.commonSeriesSettings.area]
     */
    area?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.bar]
     */
    bar?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.bubble]
     */
    bubble?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.candlestick]
     */
    candlestick?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.fullstackedarea]
     */
    fullstackedarea?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.fullstackedbar]
     */
    fullstackedbar?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.fullstackedline]
     */
    fullstackedline?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.fullstackedspline]
     */
    fullstackedspline?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.fullstackedsplinearea]
     */
    fullstackedsplinearea?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.line]
     */
    line?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.rangearea]
     */
    rangearea?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.rangebar]
     */
    rangebar?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.scatter]
     */
    scatter?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.spline]
     */
    spline?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.splinearea]
     */
    splinearea?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.stackedarea]
     */
    stackedarea?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.stackedbar]
     */
    stackedbar?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.stackedline]
     */
    stackedline?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.stackedspline]
     */
    stackedspline?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.stackedsplinearea]
     */
    stackedsplinearea?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.steparea]
     */
    steparea?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.stepline]
     */
    stepline?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.stock]
     */
    stock?: any;
    /**
     * [descr:dxChartOptions.commonSeriesSettings.type]
     */
    type?: ChartSeriesType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartLegend extends BaseChartLegend {
    /**
     * [descr:dxChartOptions.legend.customizeHint]
     */
    customizeHint?: (seriesInfo: {
      seriesName?: any;
      seriesIndex?: number;
      seriesColor?: string;
    }) => string;
    /**
     * [descr:dxChartOptions.legend.customizeText]
     */
    customizeText?: (seriesInfo: {
      seriesName?: any;
      seriesIndex?: number;
      seriesColor?: string;
    }) => string;
    /**
     * [descr:dxChartOptions.legend.hoverMode]
     */
    hoverMode?: 'excludePoints' | 'includePoints' | 'none';
    /**
     * [descr:dxChartOptions.legend.position]
     */
    position?: 'inside' | 'outside';
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartOptions extends BaseChartOptions<dxChart> {
    /**
     * [descr:dxChartOptions.adjustOnZoom]
     */
    adjustOnZoom?: boolean;
    /**
     * [descr:dxChartOptions.annotations]
     */
    annotations?: Array<dxChartAnnotationConfig | any>;
    /**
     * [descr:dxChartOptions.argumentAxis]
     */
    argumentAxis?: dxChartArgumentAxis;
    /**
     * [descr:dxChartOptions.autoHidePointMarkers]
     */
    autoHidePointMarkers?: boolean;
    /**
     * [descr:dxChartOptions.barGroupPadding]
     */
    barGroupPadding?: number;
    /**
     * [descr:dxChartOptions.barGroupWidth]
     */
    barGroupWidth?: number;
    /**
     * [descr:dxChartOptions.commonAnnotationSettings]
     */
    commonAnnotationSettings?: dxChartCommonAnnotationConfig;
    /**
     * [descr:dxChartOptions.commonAxisSettings]
     */
    commonAxisSettings?: dxChartCommonAxisSettings;
    /**
     * [descr:dxChartOptions.commonPaneSettings]
     */
    commonPaneSettings?: dxChartCommonPaneSettings;
    /**
     * [descr:dxChartOptions.commonSeriesSettings]
     */
    commonSeriesSettings?: dxChartCommonSeriesSettings;
    /**
     * [descr:dxChartOptions.containerBackgroundColor]
     */
    containerBackgroundColor?: string;
    /**
     * [descr:dxChartOptions.crosshair]
     */
    crosshair?: {
      /**
       * [descr:dxChartOptions.crosshair.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.crosshair.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxChartOptions.crosshair.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxChartOptions.crosshair.horizontalLine]
       */
      horizontalLine?:
        | {
            /**
             * [descr:dxChartOptions.crosshair.horizontalLine.color]
             */
            color?: string;
            /**
             * [descr:dxChartOptions.crosshair.horizontalLine.dashStyle]
             */
            dashStyle?: DashStyleType;
            /**
             * [descr:dxChartOptions.crosshair.horizontalLine.label]
             */
            label?: {
              /**
               * [descr:dxChartOptions.crosshair.horizontalLine.label.backgroundColor]
               */
              backgroundColor?: string;
              /**
               * [descr:dxChartOptions.crosshair.horizontalLine.label.customizeText]
               */
              customizeText?: (info: {
                value?: Date | number | string;
                valueText?: string;
                point?: chartPointObject;
              }) => string;
              /**
               * [descr:dxChartOptions.crosshair.horizontalLine.label.font]
               */
              font?: Font;
              /**
               * [descr:dxChartOptions.crosshair.horizontalLine.label.format]
               */
              format?: DevExpress.ui.Format;
              /**
               * [descr:dxChartOptions.crosshair.horizontalLine.label.visible]
               */
              visible?: boolean;
            };
            /**
             * [descr:dxChartOptions.crosshair.horizontalLine.opacity]
             */
            opacity?: number;
            /**
             * [descr:dxChartOptions.crosshair.horizontalLine.visible]
             */
            visible?: boolean;
            /**
             * [descr:dxChartOptions.crosshair.horizontalLine.width]
             */
            width?: number;
          }
        | boolean;
      /**
       * [descr:dxChartOptions.crosshair.label]
       */
      label?: {
        /**
         * [descr:dxChartOptions.crosshair.label.backgroundColor]
         */
        backgroundColor?: string;
        /**
         * [descr:dxChartOptions.crosshair.label.customizeText]
         */
        customizeText?: (info: {
          value?: Date | number | string;
          valueText?: string;
          point?: chartPointObject;
        }) => string;
        /**
         * [descr:dxChartOptions.crosshair.label.font]
         */
        font?: Font;
        /**
         * [descr:dxChartOptions.crosshair.label.format]
         */
        format?: DevExpress.ui.Format;
        /**
         * [descr:dxChartOptions.crosshair.label.visible]
         */
        visible?: boolean;
      };
      /**
       * [descr:dxChartOptions.crosshair.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartOptions.crosshair.verticalLine]
       */
      verticalLine?:
        | {
            /**
             * [descr:dxChartOptions.crosshair.verticalLine.color]
             */
            color?: string;
            /**
             * [descr:dxChartOptions.crosshair.verticalLine.dashStyle]
             */
            dashStyle?: DashStyleType;
            /**
             * [descr:dxChartOptions.crosshair.verticalLine.label]
             */
            label?: {
              /**
               * [descr:dxChartOptions.crosshair.verticalLine.label.backgroundColor]
               */
              backgroundColor?: string;
              /**
               * [descr:dxChartOptions.crosshair.verticalLine.label.customizeText]
               */
              customizeText?: (info: {
                value?: Date | number | string;
                valueText?: string;
                point?: chartPointObject;
              }) => string;
              /**
               * [descr:dxChartOptions.crosshair.verticalLine.label.font]
               */
              font?: Font;
              /**
               * [descr:dxChartOptions.crosshair.verticalLine.label.format]
               */
              format?: DevExpress.ui.Format;
              /**
               * [descr:dxChartOptions.crosshair.verticalLine.label.visible]
               */
              visible?: boolean;
            };
            /**
             * [descr:dxChartOptions.crosshair.verticalLine.opacity]
             */
            opacity?: number;
            /**
             * [descr:dxChartOptions.crosshair.verticalLine.visible]
             */
            visible?: boolean;
            /**
             * [descr:dxChartOptions.crosshair.verticalLine.width]
             */
            width?: number;
          }
        | boolean;
      /**
       * [descr:dxChartOptions.crosshair.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartOptions.customizeAnnotation]
     */
    customizeAnnotation?: (
      annotation: dxChartAnnotationConfig | any
    ) => dxChartAnnotationConfig;
    /**
     * [descr:dxChartOptions.dataPrepareSettings]
     */
    dataPrepareSettings?: {
      /**
       * [descr:dxChartOptions.dataPrepareSettings.checkTypeForAllData]
       */
      checkTypeForAllData?: boolean;
      /**
       * [descr:dxChartOptions.dataPrepareSettings.convertToAxisDataType]
       */
      convertToAxisDataType?: boolean;
      /**
       * [descr:dxChartOptions.dataPrepareSettings.sortingMethod]
       */
      sortingMethod?: boolean | ((a: any, b: any) => number);
    };
    /**
     * [descr:dxChartOptions.defaultPane]
     */
    defaultPane?: string;
    /**
     * [descr:dxChartOptions.legend]
     */
    legend?: dxChartLegend;
    /**
     * [descr:dxChartOptions.maxBubbleSize]
     */
    maxBubbleSize?: number;
    /**
     * [descr:dxChartOptions.minBubbleSize]
     */
    minBubbleSize?: number;
    /**
     * [descr:dxChartOptions.negativesAsZeroes]
     */
    negativesAsZeroes?: boolean;
    /**
     * [descr:dxChartOptions.onArgumentAxisClick]
     */
    onArgumentAxisClick?:
      | ((e: DevExpress.viz.dxChart.ArgumentAxisClickEvent) => void)
      | string;
    /**
     * [descr:dxChartOptions.onLegendClick]
     */
    onLegendClick?:
      | ((e: DevExpress.viz.dxChart.LegendClickEvent) => void)
      | string;
    /**
     * [descr:dxChartOptions.onSeriesClick]
     */
    onSeriesClick?:
      | ((e: DevExpress.viz.dxChart.SeriesClickEvent) => void)
      | string;
    /**
     * [descr:dxChartOptions.onSeriesHoverChanged]
     */
    onSeriesHoverChanged?: (
      e: DevExpress.viz.dxChart.SeriesHoverChangedEvent
    ) => void;
    /**
     * [descr:dxChartOptions.onSeriesSelectionChanged]
     */
    onSeriesSelectionChanged?: (
      e: DevExpress.viz.dxChart.SeriesSelectionChangedEvent
    ) => void;
    /**
     * [descr:dxChartOptions.onZoomEnd]
     */
    onZoomEnd?: (e: DevExpress.viz.dxChart.ZoomEndEvent) => void;
    /**
     * [descr:dxChartOptions.onZoomStart]
     */
    onZoomStart?: (e: DevExpress.viz.dxChart.ZoomStartEvent) => void;
    /**
     * [descr:dxChartOptions.panes]
     */
    panes?: dxChartPanes | Array<dxChartPanes>;
    /**
     * [descr:dxChartOptions.resizePanesOnZoom]
     */
    resizePanesOnZoom?: boolean;
    /**
     * [descr:dxChartOptions.resolveLabelOverlapping]
     */
    resolveLabelOverlapping?: 'hide' | 'none' | 'stack';
    /**
     * [descr:dxChartOptions.rotated]
     */
    rotated?: boolean;
    /**
     * [descr:dxChartOptions.scrollBar]
     */
    scrollBar?: {
      /**
       * [descr:dxChartOptions.scrollBar.color]
       */
      color?: string;
      /**
       * [descr:dxChartOptions.scrollBar.offset]
       */
      offset?: number;
      /**
       * [descr:dxChartOptions.scrollBar.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartOptions.scrollBar.position]
       */
      position?: 'bottom' | 'left' | 'right' | 'top';
      /**
       * [descr:dxChartOptions.scrollBar.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartOptions.scrollBar.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartOptions.series]
     */
    series?: ChartSeries | Array<ChartSeries>;
    /**
     * [descr:dxChartOptions.seriesSelectionMode]
     */
    seriesSelectionMode?: 'multiple' | 'single';
    /**
     * [descr:dxChartOptions.seriesTemplate]
     */
    seriesTemplate?: {
      /**
       * [descr:dxChartOptions.seriesTemplate.customizeSeries]
       */
      customizeSeries?: (seriesName: any) => ChartSeries;
      /**
       * [descr:dxChartOptions.seriesTemplate.nameField]
       */
      nameField?: string;
    };
    /**
     * [descr:dxChartOptions.stickyHovering]
     */
    stickyHovering?: boolean;
    /**
     * [descr:dxChartOptions.synchronizeMultiAxes]
     */
    synchronizeMultiAxes?: boolean;
    /**
     * [descr:dxChartOptions.tooltip]
     */
    tooltip?: dxChartTooltip;
    /**
     * [descr:dxChartOptions.valueAxis]
     */
    valueAxis?: dxChartValueAxis | Array<dxChartValueAxis>;
    /**
     * [descr:dxChartOptions.zoomAndPan]
     */
    zoomAndPan?: {
      /**
       * [descr:dxChartOptions.zoomAndPan.allowMouseWheel]
       */
      allowMouseWheel?: boolean;
      /**
       * [descr:dxChartOptions.zoomAndPan.allowTouchGestures]
       */
      allowTouchGestures?: boolean;
      /**
       * [descr:dxChartOptions.zoomAndPan.argumentAxis]
       */
      argumentAxis?: 'both' | 'none' | 'pan' | 'zoom';
      /**
       * [descr:dxChartOptions.zoomAndPan.dragBoxStyle]
       */
      dragBoxStyle?: {
        /**
         * [descr:dxChartOptions.zoomAndPan.dragBoxStyle.color]
         */
        color?: string;
        /**
         * [descr:dxChartOptions.zoomAndPan.dragBoxStyle.opacity]
         */
        opacity?: number;
      };
      /**
       * [descr:dxChartOptions.zoomAndPan.dragToZoom]
       */
      dragToZoom?: boolean;
      /**
       * [descr:dxChartOptions.zoomAndPan.panKey]
       */
      panKey?: 'alt' | 'ctrl' | 'meta' | 'shift';
      /**
       * [descr:dxChartOptions.zoomAndPan.valueAxis]
       */
      valueAxis?: 'both' | 'none' | 'pan' | 'zoom';
    };
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartPanes extends dxChartCommonPaneSettings {
    /**
     * [descr:dxChartOptions.panes.height]
     */
    height?: number | string;
    /**
     * [descr:dxChartOptions.panes.name]
     */
    name?: string;
  }
  /**
   * [descr:dxChartSeriesTypes]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypes {
    /**
     * [descr:dxChartSeriesTypes.AreaSeries]
     */
    AreaSeries?: dxChartSeriesTypesAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.BarSeries]
     */
    BarSeries?: dxChartSeriesTypesBarSeries;
    /**
     * [descr:dxChartSeriesTypes.BubbleSeries]
     */
    BubbleSeries?: dxChartSeriesTypesBubbleSeries;
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries]
     */
    CandleStickSeries?: dxChartSeriesTypesCandleStickSeries;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries]
     */
    CommonSeries?: dxChartSeriesTypesCommonSeries;
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries]
     */
    FullStackedAreaSeries?: dxChartSeriesTypesFullStackedAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries]
     */
    FullStackedBarSeries?: dxChartSeriesTypesFullStackedBarSeries;
    /**
     * [descr:dxChartSeriesTypes.FullStackedLineSeries]
     */
    FullStackedLineSeries?: dxChartSeriesTypesFullStackedLineSeries;
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries]
     */
    FullStackedSplineAreaSeries?: dxChartSeriesTypesFullStackedSplineAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineSeries]
     */
    FullStackedSplineSeries?: dxChartSeriesTypesFullStackedSplineSeries;
    /**
     * [descr:dxChartSeriesTypes.LineSeries]
     */
    LineSeries?: dxChartSeriesTypesLineSeries;
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries]
     */
    RangeAreaSeries?: dxChartSeriesTypesRangeAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.RangeBarSeries]
     */
    RangeBarSeries?: dxChartSeriesTypesRangeBarSeries;
    /**
     * [descr:dxChartSeriesTypes.ScatterSeries]
     */
    ScatterSeries?: dxChartSeriesTypesScatterSeries;
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries]
     */
    SplineAreaSeries?: dxChartSeriesTypesSplineAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.SplineSeries]
     */
    SplineSeries?: dxChartSeriesTypesSplineSeries;
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries]
     */
    StackedAreaSeries?: dxChartSeriesTypesStackedAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries]
     */
    StackedBarSeries?: dxChartSeriesTypesStackedBarSeries;
    /**
     * [descr:dxChartSeriesTypes.StackedLineSeries]
     */
    StackedLineSeries?: dxChartSeriesTypesStackedLineSeries;
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries]
     */
    StackedSplineAreaSeries?: dxChartSeriesTypesStackedSplineAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.StackedSplineSeries]
     */
    StackedSplineSeries?: dxChartSeriesTypesStackedSplineSeries;
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries]
     */
    StepAreaSeries?: dxChartSeriesTypesStepAreaSeries;
    /**
     * [descr:dxChartSeriesTypes.StepLineSeries]
     */
    StepLineSeries?: dxChartSeriesTypesStepLineSeries;
    /**
     * [descr:dxChartSeriesTypes.StockSeries]
     */
    StockSeries?: dxChartSeriesTypesStockSeries;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.label]
     */
    label?: dxChartSeriesTypesAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.point]
     */
    point?: dxChartSeriesTypesAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.AreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesBarSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.BarSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesBarSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.BarSeries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.BarSeries.label]
     */
    label?: dxChartSeriesTypesBarSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.BarSeries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesBarSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.BarSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesBarSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.BarSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesBubbleSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.BubbleSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesBubbleSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.BubbleSeries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.BubbleSeries.label]
     */
    label?: dxChartSeriesTypesBubbleSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.BubbleSeries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesBubbleSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.BubbleSeries.aggregation.method]
     */
    method?: 'avg' | 'custom';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesBubbleSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.BubbleSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCandleStickSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesCandleStickSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.argumentField]
     */
    argumentField?: string;
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.hoverStyle]
     */
    hoverStyle?: dxChartSeriesTypesCandleStickSeriesHoverStyle;
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.label]
     */
    label?: dxChartSeriesTypesCandleStickSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.selectionStyle]
     */
    selectionStyle?: dxChartSeriesTypesCandleStickSeriesSelectionStyle;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCandleStickSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.aggregation.method]
     */
    method?: 'ohlc' | 'custom';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCandleStickSeriesHoverStyle
    extends dxChartSeriesTypesCommonSeriesHoverStyle {
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.hoverStyle.hatching]
     */
    hatching?: dxChartSeriesTypesCandleStickSeriesHoverStyleHatching;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCandleStickSeriesHoverStyleHatching
    extends dxChartSeriesTypesCommonSeriesHoverStyleHatching {
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.hoverStyle.hatching.direction]
     */
    direction?: HatchingDirectionType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCandleStickSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCandleStickSeriesSelectionStyle
    extends dxChartSeriesTypesCommonSeriesSelectionStyle {
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.selectionStyle.hatching]
     */
    hatching?: dxChartSeriesTypesCandleStickSeriesSelectionStyleHatching;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCandleStickSeriesSelectionStyleHatching
    extends dxChartSeriesTypesCommonSeriesSelectionStyleHatching {
    /**
     * [descr:dxChartSeriesTypes.CandleStickSeries.selectionStyle.hatching.direction]
     */
    direction?: HatchingDirectionType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesCommonSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.argumentField]
     */
    argumentField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.axis]
     */
    axis?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.barOverlapGroup]
     */
    barOverlapGroup?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.barPadding]
     */
    barPadding?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.barWidth]
     */
    barWidth?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.border]
     */
    border?: dxChartSeriesTypesCommonSeriesBorder;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.closeValueField]
     */
    closeValueField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.color]
     */
    color?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.cornerRadius]
     */
    cornerRadius?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.highValueField]
     */
    highValueField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverMode]
     */
    hoverMode?:
      | 'allArgumentPoints'
      | 'allSeriesPoints'
      | 'excludePoints'
      | 'includePoints'
      | 'nearestPoint'
      | 'none'
      | 'onlyPoint';
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle]
     */
    hoverStyle?: dxChartSeriesTypesCommonSeriesHoverStyle;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.ignoreEmptyPoints]
     */
    ignoreEmptyPoints?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.innerColor]
     */
    innerColor?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label]
     */
    label?: dxChartSeriesTypesCommonSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.lowValueField]
     */
    lowValueField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.maxLabelCount]
     */
    maxLabelCount?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.minBarSize]
     */
    minBarSize?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.openValueField]
     */
    openValueField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.pane]
     */
    pane?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point]
     */
    point?: dxChartSeriesTypesCommonSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.rangeValue1Field]
     */
    rangeValue1Field?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.rangeValue2Field]
     */
    rangeValue2Field?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.reduction]
     */
    reduction?: {
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.reduction.color]
       */
      color?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.reduction.level]
       */
      level?: 'close' | 'high' | 'low' | 'open';
    };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionMode]
     */
    selectionMode?:
      | 'allArgumentPoints'
      | 'allSeriesPoints'
      | 'excludePoints'
      | 'includePoints'
      | 'none'
      | 'onlyPoint';
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle]
     */
    selectionStyle?: dxChartSeriesTypesCommonSeriesSelectionStyle;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.showInLegend]
     */
    showInLegend?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.sizeField]
     */
    sizeField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.stack]
     */
    stack?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.tagField]
     */
    tagField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar]
     */
    valueErrorBar?: {
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.color]
       */
      color?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.displayMode]
       */
      displayMode?: 'auto' | 'high' | 'low' | 'none';
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.edgeLength]
       */
      edgeLength?: number;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.highValueField]
       */
      highValueField?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.lineWidth]
       */
      lineWidth?: number;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.lowValueField]
       */
      lowValueField?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.type]
       */
      type?: 'fixed' | 'percent' | 'stdDeviation' | 'stdError' | 'variance';
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.valueErrorBar.value]
       */
      value?: number;
    };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.valueField]
     */
    valueField?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.aggregation.calculate]
     */
    calculate?: (
      aggregationInfo: chartPointAggregationInfoObject,
      series: chartSeriesObject
    ) => any | Array<any>;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.aggregation.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.aggregation.method]
     */
    method?:
      | 'avg'
      | 'count'
      | 'max'
      | 'min'
      | 'ohlc'
      | 'range'
      | 'sum'
      | 'custom';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesBorder {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.border.color]
     */
    color?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.border.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.border.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.border.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesHoverStyle {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.border]
     */
    border?: dxChartSeriesTypesCommonSeriesHoverStyleBorder;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.color]
     */
    color?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.hatching]
     */
    hatching?: dxChartSeriesTypesCommonSeriesHoverStyleHatching;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesHoverStyleBorder {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.border.color]
     */
    color?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.border.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.border.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.border.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesHoverStyleHatching {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.hatching.direction]
     */
    direction?: HatchingDirectionType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.hatching.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.hatching.step]
     */
    step?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.hoverStyle.hatching.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.alignment]
     */
    alignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.argumentFormat]
     */
    argumentFormat?: DevExpress.ui.Format;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.border]
     */
    border?: {
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.label.border.color]
       */
      color?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.label.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.label.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.label.border.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.connector]
     */
    connector?: {
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.label.connector.color]
       */
      color?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.label.connector.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.label.connector.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.font]
     */
    font?: Font;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.format]
     */
    format?: DevExpress.ui.Format;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.horizontalOffset]
     */
    horizontalOffset?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.position]
     */
    position?: 'inside' | 'outside';
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.rotationAngle]
     */
    rotationAngle?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.showForZeroValues]
     */
    showForZeroValues?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.verticalOffset]
     */
    verticalOffset?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.label.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.border]
     */
    border?: {
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.border.color]
       */
      color?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.border.width]
       */
      width?: number;
    };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.color]
     */
    color?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.hoverMode]
     */
    hoverMode?: 'allArgumentPoints' | 'allSeriesPoints' | 'none' | 'onlyPoint';
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.hoverStyle]
     */
    hoverStyle?: {
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.hoverStyle.border]
       */
      border?: {
        /**
         * [descr:dxChartSeriesTypes.CommonSeries.point.hoverStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxChartSeriesTypes.CommonSeries.point.hoverStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxChartSeriesTypes.CommonSeries.point.hoverStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.hoverStyle.color]
       */
      color?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.hoverStyle.size]
       */
      size?: number;
    };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.image]
     */
    image?:
      | string
      | {
          /**
           * [descr:dxChartSeriesTypes.CommonSeries.point.image.height]
           */
          height?:
            | number
            | {
                /**
                 * [descr:dxChartSeriesTypes.CommonSeries.point.image.height.rangeMaxPoint]
                 */
                rangeMaxPoint?: number;
                /**
                 * [descr:dxChartSeriesTypes.CommonSeries.point.image.height.rangeMinPoint]
                 */
                rangeMinPoint?: number;
              };
          /**
           * [descr:dxChartSeriesTypes.CommonSeries.point.image.url]
           */
          url?:
            | string
            | {
                /**
                 * [descr:dxChartSeriesTypes.CommonSeries.point.image.url.rangeMaxPoint]
                 */
                rangeMaxPoint?: string;
                /**
                 * [descr:dxChartSeriesTypes.CommonSeries.point.image.url.rangeMinPoint]
                 */
                rangeMinPoint?: string;
              };
          /**
           * [descr:dxChartSeriesTypes.CommonSeries.point.image.width]
           */
          width?:
            | number
            | {
                /**
                 * [descr:dxChartSeriesTypes.CommonSeries.point.image.width.rangeMaxPoint]
                 */
                rangeMaxPoint?: number;
                /**
                 * [descr:dxChartSeriesTypes.CommonSeries.point.image.width.rangeMinPoint]
                 */
                rangeMinPoint?: number;
              };
        };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.selectionMode]
     */
    selectionMode?:
      | 'allArgumentPoints'
      | 'allSeriesPoints'
      | 'none'
      | 'onlyPoint';
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.selectionStyle]
     */
    selectionStyle?: {
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.selectionStyle.border]
       */
      border?: {
        /**
         * [descr:dxChartSeriesTypes.CommonSeries.point.selectionStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxChartSeriesTypes.CommonSeries.point.selectionStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxChartSeriesTypes.CommonSeries.point.selectionStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.selectionStyle.color]
       */
      color?: string;
      /**
       * [descr:dxChartSeriesTypes.CommonSeries.point.selectionStyle.size]
       */
      size?: number;
    };
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.size]
     */
    size?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.symbol]
     */
    symbol?:
      | 'circle'
      | 'cross'
      | 'polygon'
      | 'square'
      | 'triangleDown'
      | 'triangleUp';
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesSelectionStyle {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.border]
     */
    border?: dxChartSeriesTypesCommonSeriesSelectionStyleBorder;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.color]
     */
    color?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.hatching]
     */
    hatching?: dxChartSeriesTypesCommonSeriesSelectionStyleHatching;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesSelectionStyleBorder {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.border.color]
     */
    color?: string;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.border.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.border.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.border.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesCommonSeriesSelectionStyleHatching {
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.hatching.direction]
     */
    direction?: HatchingDirectionType;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.hatching.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.hatching.step]
     */
    step?: number;
    /**
     * [descr:dxChartSeriesTypes.CommonSeries.selectionStyle.hatching.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesFullStackedAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.label]
     */
    label?: dxChartSeriesTypesFullStackedAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.point]
     */
    point?: dxChartSeriesTypesFullStackedAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.FullStackedAreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedBarSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesFullStackedBarSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries.label]
     */
    label?: dxChartSeriesTypesFullStackedBarSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedBarSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedBarSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
    /**
     * [descr:dxChartSeriesTypes.FullStackedBarSeries.label.position]
     */
    position?: 'inside' | 'outside';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedLineSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.FullStackedLineSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesFullStackedLineSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.FullStackedLineSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.FullStackedLineSeries.label]
     */
    label?: dxChartSeriesTypesFullStackedLineSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.FullStackedLineSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedLineSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.FullStackedLineSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedLineSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.FullStackedLineSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedSplineAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesFullStackedSplineAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.label]
     */
    label?: dxChartSeriesTypesFullStackedSplineAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.point]
     */
    point?: dxChartSeriesTypesFullStackedSplineAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedSplineAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedSplineAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedSplineAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineAreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedSplineSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesFullStackedSplineSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineSeries.label]
     */
    label?: dxChartSeriesTypesFullStackedSplineSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedSplineSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesFullStackedSplineSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.FullStackedSplineSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesLineSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.LineSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesLineSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.LineSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.LineSeries.label]
     */
    label?: dxChartSeriesTypesLineSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.LineSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesLineSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.LineSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesLineSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.LineSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesRangeAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesRangeAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.label]
     */
    label?: dxChartSeriesTypesRangeAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.point]
     */
    point?: dxChartSeriesTypesRangeAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesRangeAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.aggregation.method]
     */
    method?: 'range' | 'custom';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesRangeAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesRangeAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.RangeAreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesRangeBarSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.RangeBarSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesRangeBarSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.RangeBarSeries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.RangeBarSeries.label]
     */
    label?: dxChartSeriesTypesRangeBarSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.RangeBarSeries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesRangeBarSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.RangeBarSeries.aggregation.method]
     */
    method?: 'range' | 'custom';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesRangeBarSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.RangeBarSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesScatterSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.ScatterSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesScatterSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.ScatterSeries.label]
     */
    label?: dxChartSeriesTypesScatterSeriesLabel;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesScatterSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.ScatterSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesScatterSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.ScatterSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesSplineAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesSplineAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.label]
     */
    label?: dxChartSeriesTypesSplineAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.point]
     */
    point?: dxChartSeriesTypesSplineAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesSplineAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesSplineAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesSplineAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.SplineAreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesSplineSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.SplineSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesSplineSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.SplineSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.SplineSeries.label]
     */
    label?: dxChartSeriesTypesSplineSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.SplineSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesSplineSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.SplineSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesSplineSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.SplineSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStackedAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.label]
     */
    label?: dxChartSeriesTypesStackedAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.point]
     */
    point?: dxChartSeriesTypesStackedAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.StackedAreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedBarSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStackedBarSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries.label]
     */
    label?: dxChartSeriesTypesStackedBarSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedBarSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedBarSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
    /**
     * [descr:dxChartSeriesTypes.StackedBarSeries.label.position]
     */
    position?: 'inside' | 'outside';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedLineSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StackedLineSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStackedLineSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StackedLineSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StackedLineSeries.label]
     */
    label?: dxChartSeriesTypesStackedLineSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StackedLineSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedLineSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StackedLineSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedLineSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StackedLineSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedSplineAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStackedSplineAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.label]
     */
    label?: dxChartSeriesTypesStackedSplineAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.point]
     */
    point?: dxChartSeriesTypesStackedSplineAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedSplineAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedSplineAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedSplineAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.StackedSplineAreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedSplineSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StackedSplineSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStackedSplineSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StackedSplineSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StackedSplineSeries.label]
     */
    label?: dxChartSeriesTypesStackedSplineSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StackedSplineSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedSplineSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StackedSplineSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStackedSplineSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StackedSplineSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStepAreaSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.border]
     */
    border?: dxChartSeriesTypesStepAreaSeriesBorder;
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.hoverStyle]
     */
    hoverStyle?: dxChartSeriesTypesStepAreaSeriesHoverStyle;
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.label]
     */
    label?: dxChartSeriesTypesStepAreaSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.point]
     */
    point?: dxChartSeriesTypesStepAreaSeriesPoint;
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.selectionStyle]
     */
    selectionStyle?: dxChartSeriesTypesStepAreaSeriesSelectionStyle;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesBorder
    extends dxChartSeriesTypesCommonSeriesBorder {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.border.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesHoverStyle
    extends dxChartSeriesTypesCommonSeriesHoverStyle {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.hoverStyle.border]
     */
    border?: dxChartSeriesTypesStepAreaSeriesHoverStyleBorder;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesHoverStyleBorder
    extends dxChartSeriesTypesCommonSeriesHoverStyleBorder {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.hoverStyle.border.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesPoint
    extends dxChartSeriesTypesCommonSeriesPoint {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesSelectionStyle
    extends dxChartSeriesTypesCommonSeriesSelectionStyle {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.selectionStyle.border]
     */
    border?: dxChartSeriesTypesStepAreaSeriesSelectionStyleBorder;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepAreaSeriesSelectionStyleBorder
    extends dxChartSeriesTypesCommonSeriesSelectionStyleBorder {
    /**
     * [descr:dxChartSeriesTypes.StepAreaSeries.selectionStyle.border.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepLineSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StepLineSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStepLineSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StepLineSeries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StepLineSeries.label]
     */
    label?: dxChartSeriesTypesStepLineSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StepLineSeries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepLineSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StepLineSeries.aggregation.method]
     */
    method?: DevExpress.viz.dxChart.ChartSingleValueSeriesAggregationMethodType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStepLineSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StepLineSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStockSeries
    extends dxChartSeriesTypesCommonSeries {
    /**
     * [descr:dxChartSeriesTypes.StockSeries.aggregation]
     */
    aggregation?: dxChartSeriesTypesStockSeriesAggregation;
    /**
     * [descr:dxChartSeriesTypes.StockSeries.argumentField]
     */
    argumentField?: string;
    /**
     * [descr:dxChartSeriesTypes.StockSeries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxChartSeriesTypes.StockSeries.label]
     */
    label?: dxChartSeriesTypesStockSeriesLabel;
    /**
     * [descr:dxChartSeriesTypes.StockSeries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStockSeriesAggregation
    extends dxChartSeriesTypesCommonSeriesAggregation {
    /**
     * [descr:dxChartSeriesTypes.StockSeries.aggregation.method]
     */
    method?: 'ohlc' | 'custom';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartSeriesTypesStockSeriesLabel
    extends dxChartSeriesTypesCommonSeriesLabel {
    /**
     * [descr:dxChartSeriesTypes.StockSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartTooltip extends BaseChartTooltip {
    /**
     * [descr:dxChartOptions.tooltip.location]
     */
    location?: 'center' | 'edge';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxis extends dxChartCommonAxisSettings {
    /**
     * [descr:dxChartOptions.valueAxis.autoBreaksEnabled]
     */
    autoBreaksEnabled?: boolean;
    /**
     * [descr:dxChartOptions.valueAxis.axisDivisionFactor]
     */
    axisDivisionFactor?: number;
    /**
     * [descr:dxChartOptions.valueAxis.breaks]
     */
    breaks?: Array<ScaleBreak>;
    /**
     * [descr:dxChartOptions.valueAxis.categories]
     */
    categories?: Array<number | string | Date>;
    /**
     * [descr:dxChartOptions.valueAxis.constantLineStyle]
     */
    constantLineStyle?: dxChartValueAxisConstantLineStyle;
    /**
     * [descr:dxChartOptions.valueAxis.constantLines]
     */
    constantLines?: Array<dxChartValueAxisConstantLines>;
    /**
     * [descr:dxChartOptions.valueAxis.endOnTick]
     */
    endOnTick?: boolean;
    /**
     * [descr:dxChartOptions.valueAxis.label]
     */
    label?: dxChartValueAxisLabel;
    /**
     * [descr:dxChartOptions.valueAxis.linearThreshold]
     */
    linearThreshold?: number;
    /**
     * [descr:dxChartOptions.valueAxis.logarithmBase]
     */
    logarithmBase?: number;
    /**
     * [descr:dxChartOptions.valueAxis.maxAutoBreakCount]
     */
    maxAutoBreakCount?: number;
    /**
     * [descr:dxChartOptions.valueAxis.minVisualRangeLength]
     */
    minVisualRangeLength?: VizTimeInterval;
    /**
     * [descr:dxChartOptions.valueAxis.minorTickCount]
     */
    minorTickCount?: number;
    /**
     * [descr:dxChartOptions.valueAxis.minorTickInterval]
     */
    minorTickInterval?: VizTimeInterval;
    /**
     * [descr:dxChartOptions.valueAxis.multipleAxesSpacing]
     */
    multipleAxesSpacing?: number;
    /**
     * [descr:dxChartOptions.valueAxis.name]
     */
    name?: string;
    /**
     * [descr:dxChartOptions.valueAxis.pane]
     */
    pane?: string;
    /**
     * [descr:dxChartOptions.valueAxis.position]
     */
    position?: 'bottom' | 'left' | 'right' | 'top';
    /**
     * [descr:dxChartOptions.valueAxis.customPosition]
     */
    customPosition?: number | Date | string;
    /**
     * [descr:dxChartOptions.valueAxis.offset]
     */
    offset?: number;
    /**
     * [descr:dxChartOptions.valueAxis.showZero]
     */
    showZero?: boolean;
    /**
     * [descr:dxChartOptions.valueAxis.strips]
     */
    strips?: Array<dxChartValueAxisStrips>;
    /**
     * [descr:dxChartOptions.valueAxis.synchronizedValue]
     */
    synchronizedValue?: number;
    /**
     * [descr:dxChartOptions.valueAxis.tickInterval]
     */
    tickInterval?: VizTimeInterval;
    /**
     * [descr:dxChartOptions.valueAxis.title]
     */
    title?: dxChartValueAxisTitle;
    /**
     * [descr:dxChartOptions.valueAxis.type]
     */
    type?: 'continuous' | 'discrete' | 'logarithmic';
    /**
     * [descr:dxChartOptions.valueAxis.valueType]
     */
    valueType?: 'datetime' | 'numeric' | 'string';
    /**
     * [descr:dxChartOptions.valueAxis.visualRange]
     */
    visualRange?: VizRange | Array<number | string | Date>;
    /**
     * [descr:dxChartOptions.valueAxis.visualRangeUpdateMode]
     */
    visualRangeUpdateMode?: 'auto' | 'keep' | 'reset' | 'shift';
    /**
     * [descr:dxChartOptions.valueAxis.wholeRange]
     */
    wholeRange?: VizRange | Array<number | string | Date>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisConstantLines
    extends dxChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxChartOptions.valueAxis.constantLines.displayBehindSeries]
     */
    displayBehindSeries?: boolean;
    /**
     * [descr:dxChartOptions.valueAxis.constantLines.extendAxis]
     */
    extendAxis?: boolean;
    /**
     * [descr:dxChartOptions.valueAxis.constantLines.label]
     */
    label?: dxChartValueAxisConstantLinesLabel;
    /**
     * [descr:dxChartOptions.valueAxis.constantLines.value]
     */
    value?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisConstantLinesLabel
    extends dxChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxChartOptions.valueAxis.constantLines.label.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartOptions.valueAxis.constantLines.label.text]
     */
    text?: string;
    /**
     * [descr:dxChartOptions.valueAxis.constantLines.label.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'center' | 'top';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisConstantLineStyle
    extends dxChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxChartOptions.valueAxis.constantLineStyle.label]
     */
    label?: dxChartValueAxisConstantLineStyleLabel;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisConstantLineStyleLabel
    extends dxChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxChartOptions.valueAxis.constantLineStyle.label.horizontalAlignment]
     */
    horizontalAlignment?: 'center' | 'left' | 'right';
    /**
     * [descr:dxChartOptions.valueAxis.constantLineStyle.label.verticalAlignment]
     */
    verticalAlignment?: 'bottom' | 'center' | 'top';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisLabel
    extends dxChartCommonAxisSettingsLabel {
    /**
     * [descr:dxChartOptions.valueAxis.label.customizeHint]
     */
    customizeHint?: (axisValue: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxChartOptions.valueAxis.label.customizeText]
     */
    customizeText?: (axisValue: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxChartOptions.valueAxis.label.format]
     */
    format?: DevExpress.ui.Format;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisStrips
    extends dxChartCommonAxisSettingsStripStyle {
    /**
     * [descr:dxChartOptions.valueAxis.strips.color]
     */
    color?: string;
    /**
     * [descr:dxChartOptions.valueAxis.strips.endValue]
     */
    endValue?: number | Date | string;
    /**
     * [descr:dxChartOptions.valueAxis.strips.label]
     */
    label?: dxChartValueAxisStripsLabel;
    /**
     * [descr:dxChartOptions.valueAxis.strips.startValue]
     */
    startValue?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisStripsLabel
    extends dxChartCommonAxisSettingsStripStyleLabel {
    /**
     * [descr:dxChartOptions.valueAxis.strips.label.text]
     */
    text?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxChartValueAxisTitle
    extends dxChartCommonAxisSettingsTitle {
    /**
     * [descr:dxChartOptions.valueAxis.title.text]
     */
    text?: string;
  }
  /**
   * [descr:dxCircularGauge]
   */
  export class dxCircularGauge extends BaseGauge<dxCircularGaugeOptions> {}
  module dxCircularGauge {
    export type DisposingEvent = DevExpress.events.EventInfo<dxCircularGauge>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxCircularGauge>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxCircularGauge>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxCircularGauge> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxCircularGauge>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxCircularGauge> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxCircularGauge>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxCircularGauge> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxCircularGaugeOptions;
    export type TooltipHiddenEvent =
      DevExpress.events.EventInfo<dxCircularGauge> &
        DevExpress.viz.BaseGauge.TooltipInfo;
    export type TooltipShownEvent =
      DevExpress.events.EventInfo<dxCircularGauge> &
        DevExpress.viz.BaseGauge.TooltipInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxCircularGaugeOptions
    extends BaseGaugeOptions<dxCircularGauge> {
    /**
     * [descr:dxCircularGaugeOptions.geometry]
     */
    geometry?: {
      /**
       * [descr:dxCircularGaugeOptions.geometry.endAngle]
       */
      endAngle?: number;
      /**
       * [descr:dxCircularGaugeOptions.geometry.startAngle]
       */
      startAngle?: number;
    };
    /**
     * [descr:dxCircularGaugeOptions.rangeContainer]
     */
    rangeContainer?: dxCircularGaugeRangeContainer;
    /**
     * [descr:dxCircularGaugeOptions.scale]
     */
    scale?: dxCircularGaugeScale;
    /**
     * [descr:dxCircularGaugeOptions.subvalueIndicator]
     */
    subvalueIndicator?: GaugeIndicator;
    /**
     * [descr:dxCircularGaugeOptions.valueIndicator]
     */
    valueIndicator?: GaugeIndicator;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxCircularGaugeRangeContainer
    extends BaseGaugeRangeContainer {
    /**
     * [descr:dxCircularGaugeOptions.rangeContainer.orientation]
     */
    orientation?: 'center' | 'inside' | 'outside';
    /**
     * [descr:dxCircularGaugeOptions.rangeContainer.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxCircularGaugeScale extends BaseGaugeScale {
    /**
     * [descr:dxCircularGaugeOptions.scale.label]
     */
    label?: dxCircularGaugeScaleLabel;
    /**
     * [descr:dxCircularGaugeOptions.scale.orientation]
     */
    orientation?: 'center' | 'inside' | 'outside';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxCircularGaugeScaleLabel extends BaseGaugeScaleLabel {
    /**
     * [descr:dxCircularGaugeOptions.scale.label.hideFirstOrLast]
     */
    hideFirstOrLast?: 'first' | 'last';
    /**
     * [descr:dxCircularGaugeOptions.scale.label.indentFromTick]
     */
    indentFromTick?: number;
  }
  /**
   * [descr:dxFunnel]
   */
  export class dxFunnel extends BaseWidget<dxFunnelOptions> {
    /**
     * [descr:dxFunnel.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:dxFunnel.getAllItems()]
     */
    getAllItems(): Array<DevExpress.viz.dxFunnel.Item>;
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxFunnel.hideTooltip()]
     */
    hideTooltip(): void;
  }
  module dxFunnel {
    export type DisposingEvent = DevExpress.events.EventInfo<dxFunnel>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxFunnel>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxFunnel>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxFunnel> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxFunnel>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface FunnelItemInfo {
      readonly item: Item;
    }
    export type HoverChangedEvent = DevExpress.events.EventInfo<dxFunnel> &
      FunnelItemInfo;
    export type IncidentOccurredEvent = DevExpress.events.EventInfo<dxFunnel> &
      DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxFunnel>;
    export type ItemClickEvent = DevExpress.events.NativeEventInfo<
      dxFunnel,
      MouseEvent | PointerEvent
    > &
      FunnelItemInfo;
    export type LegendClickEvent = DevExpress.events.NativeEventInfo<
      dxFunnel,
      MouseEvent | PointerEvent
    > &
      FunnelItemInfo;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxFunnel> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxFunnelOptions;
    export type SelectionChangedEvent = DevExpress.events.EventInfo<dxFunnel> &
      FunnelItemInfo;
  }
  /**
   * @deprecated Use Item instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFunnelItem {
    /**
     * [descr:dxFunnelItem.argument]
     */
    argument?: string | Date | number;
    /**
     * [descr:dxFunnelItem.data]
     */
    data?: any;
    /**
     * [descr:dxFunnelItem.getColor()]
     */
    getColor(): string;
    /**
     * [descr:dxFunnelItem.hover(state)]
     */
    hover(state: boolean): void;
    /**
     * [descr:dxFunnelItem.isHovered()]
     */
    isHovered(): boolean;
    /**
     * [descr:dxFunnelItem.isSelected()]
     */
    isSelected(): boolean;
    /**
     * [descr:dxFunnelItem.percent]
     */
    percent?: number;
    /**
     * [descr:dxFunnelItem.select(state)]
     */
    select(state: boolean): void;
    /**
     * [descr:dxFunnelItem.showTooltip()]
     */
    showTooltip(): void;
    /**
     * [descr:dxFunnelItem.value]
     */
    value?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFunnelLegend extends BaseLegend {
    /**
     * [descr:dxFunnelOptions.legend.customizeHint]
     */
    customizeHint?: (itemInfo: {
      item?: DevExpress.viz.dxFunnel.Item;
      text?: string;
    }) => string;
    /**
     * [descr:dxFunnelOptions.legend.customizeItems]
     */
    customizeItems?: (
      items: Array<DevExpress.viz.dxFunnel.LegendItem>
    ) => Array<DevExpress.viz.dxFunnel.LegendItem>;
    /**
     * [descr:dxFunnelOptions.legend.customizeText]
     */
    customizeText?: (itemInfo: {
      item?: DevExpress.viz.dxFunnel.Item;
      text?: string;
    }) => string;
    /**
     * [descr:dxFunnelOptions.legend.markerTemplate]
     */
    markerTemplate?:
      | DevExpress.core.template
      | ((
          legendItem: DevExpress.viz.dxFunnel.LegendItem,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxFunnelOptions.legend.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFunnelOptions extends BaseWidgetOptions<dxFunnel> {
    /**
     * [descr:dxFunnelOptions.adaptiveLayout]
     */
    adaptiveLayout?: {
      /**
       * [descr:dxFunnelOptions.adaptiveLayout.height]
       */
      height?: number;
      /**
       * [descr:dxFunnelOptions.adaptiveLayout.keepLabels]
       */
      keepLabels?: boolean;
      /**
       * [descr:dxFunnelOptions.adaptiveLayout.width]
       */
      width?: number;
    };
    /**
     * [descr:dxFunnelOptions.algorithm]
     */
    algorithm?: 'dynamicHeight' | 'dynamicSlope';
    /**
     * [descr:dxFunnelOptions.argumentField]
     */
    argumentField?: string;
    /**
     * [descr:dxFunnelOptions.colorField]
     */
    colorField?: string;
    /**
     * [descr:dxFunnelOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
    /**
     * [descr:dxFunnelOptions.hoverEnabled]
     */
    hoverEnabled?: boolean;
    /**
     * [descr:dxFunnelOptions.inverted]
     */
    inverted?: boolean;
    /**
     * [descr:dxFunnelOptions.item]
     */
    item?: {
      /**
       * [descr:dxFunnelOptions.item.border]
       */
      border?: {
        /**
         * [descr:dxFunnelOptions.item.border.color]
         */
        color?: string;
        /**
         * [descr:dxFunnelOptions.item.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxFunnelOptions.item.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxFunnelOptions.item.hoverStyle]
       */
      hoverStyle?: {
        /**
         * [descr:dxFunnelOptions.item.hoverStyle.border]
         */
        border?: {
          /**
           * [descr:dxFunnelOptions.item.hoverStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxFunnelOptions.item.hoverStyle.border.visible]
           */
          visible?: boolean;
          /**
           * [descr:dxFunnelOptions.item.hoverStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxFunnelOptions.item.hoverStyle.hatching]
         */
        hatching?: {
          /**
           * [descr:dxFunnelOptions.item.hoverStyle.hatching.direction]
           */
          direction?: HatchingDirectionType;
          /**
           * [descr:dxFunnelOptions.item.hoverStyle.hatching.opacity]
           */
          opacity?: number;
          /**
           * [descr:dxFunnelOptions.item.hoverStyle.hatching.step]
           */
          step?: number;
          /**
           * [descr:dxFunnelOptions.item.hoverStyle.hatching.width]
           */
          width?: number;
        };
      };
      /**
       * [descr:dxFunnelOptions.item.selectionStyle]
       */
      selectionStyle?: {
        /**
         * [descr:dxFunnelOptions.item.selectionStyle.border]
         */
        border?: {
          /**
           * [descr:dxFunnelOptions.item.selectionStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxFunnelOptions.item.selectionStyle.border.visible]
           */
          visible?: boolean;
          /**
           * [descr:dxFunnelOptions.item.selectionStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxFunnelOptions.item.selectionStyle.hatching]
         */
        hatching?: {
          /**
           * [descr:dxFunnelOptions.item.selectionStyle.hatching.direction]
           */
          direction?: HatchingDirectionType;
          /**
           * [descr:dxFunnelOptions.item.selectionStyle.hatching.opacity]
           */
          opacity?: number;
          /**
           * [descr:dxFunnelOptions.item.selectionStyle.hatching.step]
           */
          step?: number;
          /**
           * [descr:dxFunnelOptions.item.selectionStyle.hatching.width]
           */
          width?: number;
        };
      };
    };
    /**
     * [descr:dxFunnelOptions.label]
     */
    label?: {
      /**
       * [descr:dxFunnelOptions.label.backgroundColor]
       */
      backgroundColor?: string;
      /**
       * [descr:dxFunnelOptions.label.border]
       */
      border?: {
        /**
         * [descr:dxFunnelOptions.label.border.color]
         */
        color?: string;
        /**
         * [descr:dxFunnelOptions.label.border.dashStyle]
         */
        dashStyle?: DashStyleType;
        /**
         * [descr:dxFunnelOptions.label.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxFunnelOptions.label.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxFunnelOptions.label.connector]
       */
      connector?: {
        /**
         * [descr:dxFunnelOptions.label.connector.color]
         */
        color?: string;
        /**
         * [descr:dxFunnelOptions.label.connector.opacity]
         */
        opacity?: number;
        /**
         * [descr:dxFunnelOptions.label.connector.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxFunnelOptions.label.connector.width]
         */
        width?: number;
      };
      /**
       * [descr:dxFunnelOptions.label.customizeText]
       */
      customizeText?: (itemInfo: {
        item?: DevExpress.viz.dxFunnel.Item;
        value?: number;
        valueText?: string;
        percent?: number;
        percentText?: string;
      }) => string;
      /**
       * [descr:dxFunnelOptions.label.font]
       */
      font?: Font;
      /**
       * [descr:dxFunnelOptions.label.format]
       */
      format?: DevExpress.ui.Format;
      /**
       * [descr:dxFunnelOptions.label.horizontalAlignment]
       */
      horizontalAlignment?: 'left' | 'right';
      /**
       * [descr:dxFunnelOptions.label.horizontalOffset]
       */
      horizontalOffset?: number;
      /**
       * [descr:dxFunnelOptions.label.position]
       */
      position?: 'columns' | 'inside' | 'outside';
      /**
       * [descr:dxFunnelOptions.label.showForZeroValues]
       */
      showForZeroValues?: boolean;
      /**
       * [descr:dxFunnelOptions.label.textOverflow]
       */
      textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
      /**
       * [descr:dxFunnelOptions.label.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxFunnelOptions.label.wordWrap]
       */
      wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
    };
    /**
     * [descr:dxFunnelOptions.legend]
     */
    legend?: dxFunnelLegend;
    /**
     * [descr:dxFunnelOptions.neckHeight]
     */
    neckHeight?: number;
    /**
     * [descr:dxFunnelOptions.neckWidth]
     */
    neckWidth?: number;
    /**
     * [descr:dxFunnelOptions.onHoverChanged]
     */
    onHoverChanged?: (e: DevExpress.viz.dxFunnel.HoverChangedEvent) => void;
    /**
     * [descr:dxFunnelOptions.onItemClick]
     */
    onItemClick?:
      | ((e: DevExpress.viz.dxFunnel.ItemClickEvent) => void)
      | string;
    /**
     * [descr:dxFunnelOptions.onLegendClick]
     */
    onLegendClick?:
      | ((e: DevExpress.viz.dxFunnel.LegendClickEvent) => void)
      | string;
    /**
     * [descr:dxFunnelOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.viz.dxFunnel.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxFunnelOptions.palette]
     */
    palette?: Array<string> | PaletteType;
    /**
     * [descr:dxFunnelOptions.paletteExtensionMode]
     */
    paletteExtensionMode?: PaletteExtensionModeType;
    /**
     * [descr:dxFunnelOptions.resolveLabelOverlapping]
     */
    resolveLabelOverlapping?: 'hide' | 'none' | 'shift';
    /**
     * [descr:dxFunnelOptions.selectionMode]
     */
    selectionMode?: 'multiple' | 'none' | 'single';
    /**
     * [descr:dxFunnelOptions.sortData]
     */
    sortData?: boolean;
    /**
     * [descr:dxFunnelOptions.tooltip]
     */
    tooltip?: dxFunnelTooltip;
    /**
     * [descr:dxFunnelOptions.valueField]
     */
    valueField?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxFunnelTooltip extends BaseWidgetTooltip {
    /**
     * [descr:dxFunnelOptions.tooltip.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          info: {
            item?: DevExpress.viz.dxFunnel.Item;
            value?: number;
            valueText?: string;
            percent?: number;
            percentText?: string;
          },
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxFunnelOptions.tooltip.customizeTooltip]
     */
    customizeTooltip?: (info: {
      item?: DevExpress.viz.dxFunnel.Item;
      value?: number;
      valueText?: string;
      percent?: number;
      percentText?: string;
    }) => any;
  }
  /**
   * [descr:dxLinearGauge]
   */
  export class dxLinearGauge extends BaseGauge<dxLinearGaugeOptions> {}
  module dxLinearGauge {
    export type DisposingEvent = DevExpress.events.EventInfo<dxLinearGauge>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxLinearGauge>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxLinearGauge>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxLinearGauge> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxLinearGauge>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxLinearGauge> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxLinearGauge>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxLinearGauge> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxLinearGaugeOptions;
    export type TooltipHiddenEvent =
      DevExpress.events.EventInfo<dxLinearGauge> &
        DevExpress.viz.BaseGauge.TooltipInfo;
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxLinearGauge> &
      DevExpress.viz.BaseGauge.TooltipInfo;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLinearGaugeOptions
    extends BaseGaugeOptions<dxLinearGauge> {
    /**
     * [descr:dxLinearGaugeOptions.geometry]
     */
    geometry?: {
      /**
       * [descr:dxLinearGaugeOptions.geometry.orientation]
       */
      orientation?: 'horizontal' | 'vertical';
    };
    /**
     * [descr:dxLinearGaugeOptions.rangeContainer]
     */
    rangeContainer?: dxLinearGaugeRangeContainer;
    /**
     * [descr:dxLinearGaugeOptions.scale]
     */
    scale?: dxLinearGaugeScale;
    /**
     * [descr:dxLinearGaugeOptions.subvalueIndicator]
     */
    subvalueIndicator?: GaugeIndicator;
    /**
     * [descr:dxLinearGaugeOptions.valueIndicator]
     */
    valueIndicator?: GaugeIndicator;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLinearGaugeRangeContainer extends BaseGaugeRangeContainer {
    /**
     * [descr:dxLinearGaugeOptions.rangeContainer.horizontalOrientation]
     */
    horizontalOrientation?: 'center' | 'left' | 'right';
    /**
     * [descr:dxLinearGaugeOptions.rangeContainer.verticalOrientation]
     */
    verticalOrientation?: 'bottom' | 'center' | 'top';
    /**
     * [descr:dxLinearGaugeOptions.rangeContainer.width]
     */
    width?:
      | {
          /**
           * [descr:dxLinearGaugeOptions.rangeContainer.width.start]
           */
          start?: number;
          /**
           * [descr:dxLinearGaugeOptions.rangeContainer.width.end]
           */
          end?: number;
        }
      | number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLinearGaugeScale extends BaseGaugeScale {
    /**
     * [descr:dxLinearGaugeOptions.scale.horizontalOrientation]
     */
    horizontalOrientation?: 'center' | 'left' | 'right';
    /**
     * [descr:dxLinearGaugeOptions.scale.label]
     */
    label?: dxLinearGaugeScaleLabel;
    /**
     * [descr:dxLinearGaugeOptions.scale.scaleDivisionFactor]
     */
    scaleDivisionFactor?: number;
    /**
     * [descr:dxLinearGaugeOptions.scale.verticalOrientation]
     */
    verticalOrientation?: 'bottom' | 'center' | 'top';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxLinearGaugeScaleLabel extends BaseGaugeScaleLabel {
    /**
     * [descr:dxLinearGaugeOptions.scale.label.indentFromTick]
     */
    indentFromTick?: number;
  }
  /**
   * [descr:dxPieChart]
   */
  export class dxPieChart extends BaseChart<dxPieChartOptions> {
    /**
     * [descr:dxPieChart.getInnerRadius()]
     */
    getInnerRadius(): number;
  }
  module dxPieChart {
    export type DisposingEvent = DevExpress.events.EventInfo<dxPieChart>;
    export type DoneEvent = DevExpress.events.EventInfo<dxPieChart>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxPieChart>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxPieChart>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxPieChart> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxPieChart>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxPieChart> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxPieChart>;
    export type LegendClickEvent = DevExpress.events.NativeEventInfo<
      dxPieChart,
      MouseEvent | PointerEvent
    > & {
      readonly target: string | number;
      readonly points: Array<piePointObject>;
    };
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxPieChart> &
      DevExpress.events.ChangedOptionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type PieSeriesType = 'donut' | 'doughnut' | 'pie';
    export type PointClickEvent = DevExpress.events.NativeEventInfo<
      dxPieChart,
      MouseEvent | PointerEvent
    > &
      DevExpress.viz.BaseChart.PointInteractionInfo;
    export type PointHoverChangedEvent =
      DevExpress.events.EventInfo<dxPieChart> &
        DevExpress.viz.BaseChart.PointInteractionInfo;
    export type PointSelectionChangedEvent =
      DevExpress.events.EventInfo<dxPieChart> &
        DevExpress.viz.BaseChart.PointInteractionInfo;
    export type Properties = dxPieChartOptions;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type SegmentsDirectionType = 'anticlockwise' | 'clockwise';
    export type TooltipHiddenEvent = DevExpress.events.EventInfo<dxPieChart> &
      DevExpress.viz.BaseChart.TooltipInfo;
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxPieChart> &
      DevExpress.viz.BaseChart.TooltipInfo;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPieChartAdaptiveLayout extends BaseChartAdaptiveLayout {
    /**
     * [descr:dxPieChartOptions.adaptiveLayout.keepLabels]
     */
    keepLabels?: boolean;
  }
  /**
   * [descr:dxPieChartAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPieChartAnnotationConfig
    extends dxPieChartCommonAnnotationConfig {
    /**
     * [descr:dxPieChartAnnotationConfig.name]
     */
    name?: string;
  }
  /**
   * [descr:dxPieChartCommonAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPieChartCommonAnnotationConfig
    extends BaseWidgetAnnotationConfig {
    /**
     * [descr:dxPieChartCommonAnnotationConfig.location]
     */
    location?: 'center' | 'edge';
    /**
     * [descr:dxPieChartCommonAnnotationConfig.argument]
     */
    argument?: number | Date | string;
    /**
     * [descr:dxPieChartCommonAnnotationConfig.series]
     */
    series?: string;
    /**
     * [descr:dxPieChartCommonAnnotationConfig.customizeTooltip]
     */
    customizeTooltip?: (annotation: dxPieChartAnnotationConfig | any) => any;
    /**
     * [descr:dxPieChartCommonAnnotationConfig.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          annotation: dxPieChartAnnotationConfig | any,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxPieChartCommonAnnotationConfig.tooltipTemplate]
     */
    tooltipTemplate?:
      | DevExpress.core.template
      | ((
          annotation: dxPieChartAnnotationConfig | any,
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPieChartLegend extends BaseChartLegend {
    /**
     * [descr:dxPieChartOptions.legend.customizeHint]
     */
    customizeHint?: (pointInfo: {
      pointName?: any;
      pointIndex?: number;
      pointColor?: string;
    }) => string;
    /**
     * [descr:dxPieChartOptions.legend.customizeItems]
     */
    customizeItems?: (
      items: Array<DevExpress.viz.dxPieChart.LegendItem>
    ) => Array<DevExpress.viz.dxPieChart.LegendItem>;
    /**
     * [descr:dxPieChartOptions.legend.customizeText]
     */
    customizeText?: (pointInfo: {
      pointName?: any;
      pointIndex?: number;
      pointColor?: string;
    }) => string;
    /**
     * [descr:dxPieChartOptions.legend.hoverMode]
     */
    hoverMode?: 'none' | 'allArgumentPoints';
    /**
     * [descr:dxPieChartOptions.legend.markerTemplate]
     */
    markerTemplate?:
      | DevExpress.core.template
      | ((
          legendItem: DevExpress.viz.dxPieChart.LegendItem,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPieChartOptions extends BaseChartOptions<dxPieChart> {
    /**
     * [descr:dxPieChartOptions.adaptiveLayout]
     */
    adaptiveLayout?: dxPieChartAdaptiveLayout;
    /**
     * [descr:dxPieChartOptions.centerTemplate]
     */
    centerTemplate?:
      | DevExpress.core.template
      | ((
          component: dxPieChart,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxPieChartOptions.commonSeriesSettings]
     */
    commonSeriesSettings?: any;
    /**
     * [descr:dxPieChartOptions.diameter]
     */
    diameter?: number;
    /**
     * [descr:dxPieChartOptions.innerRadius]
     */
    innerRadius?: number;
    /**
     * [descr:dxPieChartOptions.legend]
     */
    legend?: dxPieChartLegend;
    /**
     * [descr:dxPieChartOptions.minDiameter]
     */
    minDiameter?: number;
    /**
     * [descr:dxPieChartOptions.onLegendClick]
     */
    onLegendClick?:
      | ((e: DevExpress.viz.dxPieChart.LegendClickEvent) => void)
      | string;
    /**
     * [descr:dxPieChartOptions.palette]
     */
    palette?: Array<string> | PaletteType;
    /**
     * [descr:dxPieChartOptions.resolveLabelOverlapping]
     */
    resolveLabelOverlapping?: 'hide' | 'none' | 'shift';
    /**
     * [descr:dxPieChartOptions.segmentsDirection]
     */
    segmentsDirection?: DevExpress.viz.dxPieChart.SegmentsDirectionType;
    /**
     * [descr:dxPieChartOptions.series]
     */
    series?: PieChartSeries | Array<PieChartSeries>;
    /**
     * [descr:dxPieChartOptions.seriesTemplate]
     */
    seriesTemplate?: {
      /**
       * [descr:dxPieChartOptions.seriesTemplate.customizeSeries]
       */
      customizeSeries?: (seriesName: any) => PieChartSeries;
      /**
       * [descr:dxPieChartOptions.seriesTemplate.nameField]
       */
      nameField?: string;
    };
    /**
     * [descr:dxPieChartOptions.sizeGroup]
     */
    sizeGroup?: string;
    /**
     * [descr:dxPieChartOptions.startAngle]
     */
    startAngle?: number;
    /**
     * [descr:dxPieChartOptions.type]
     */
    type?: DevExpress.viz.dxPieChart.PieSeriesType;
    /**
     * [descr:dxPieChartOptions.annotations]
     */
    annotations?: Array<dxPieChartAnnotationConfig | any>;
    /**
     * [descr:dxPieChartOptions.commonAnnotationSettings]
     */
    commonAnnotationSettings?: dxPieChartCommonAnnotationConfig;
    /**
     * [descr:dxPieChartOptions.customizeAnnotation]
     */
    customizeAnnotation?: (
      annotation: dxPieChartAnnotationConfig | any
    ) => dxPieChartAnnotationConfig;
  }
  /**
   * [descr:dxPieChartSeriesTypes]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPieChartSeriesTypes {
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries]
     */
    CommonPieChartSeries?: dxPieChartSeriesTypesCommonPieChartSeries;
    /**
     * [descr:dxPieChartSeriesTypes.DoughnutSeries]
     */
    DoughnutSeries?: any;
    /**
     * [descr:dxPieChartSeriesTypes.PieSeries]
     */
    PieSeries?: any;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPieChartSeriesTypesCommonPieChartSeries {
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.argumentField]
     */
    argumentField?: string;
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.argumentType]
     */
    argumentType?: 'datetime' | 'numeric' | 'string';
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.border]
     */
    border?: {
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.border.color]
       */
      color?: string;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.border.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.color]
     */
    color?: string;
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverMode]
     */
    hoverMode?: 'none' | 'onlyPoint';
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle]
     */
    hoverStyle?: {
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.border]
       */
      border?: {
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.border.dashStyle]
         */
        dashStyle?: DashStyleType;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.color]
       */
      color?: string;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.hatching]
       */
      hatching?: {
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.hatching.direction]
         */
        direction?: HatchingDirectionType;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.hatching.opacity]
         */
        opacity?: number;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.hatching.step]
         */
        step?: number;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.hoverStyle.hatching.width]
         */
        width?: number;
      };
    };
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label]
     */
    label?: {
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.argumentFormat]
       */
      argumentFormat?: DevExpress.ui.Format;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.backgroundColor]
       */
      backgroundColor?: string;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.border]
       */
      border?: {
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.border.color]
         */
        color?: string;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.border.dashStyle]
         */
        dashStyle?: DashStyleType;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.connector]
       */
      connector?: {
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.connector.color]
         */
        color?: string;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.connector.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.connector.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.customizeText]
       */
      customizeText?: (pointInfo: any) => string;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.font]
       */
      font?: Font;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.format]
       */
      format?: DevExpress.ui.Format;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.position]
       */
      position?: 'columns' | 'inside' | 'outside';
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.radialOffset]
       */
      radialOffset?: number;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.rotationAngle]
       */
      rotationAngle?: number;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.textOverflow]
       */
      textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.label.wordWrap]
       */
      wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
    };
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.maxLabelCount]
     */
    maxLabelCount?: number;
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.minSegmentSize]
     */
    minSegmentSize?: number;
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionMode]
     */
    selectionMode?: 'none' | 'onlyPoint';
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle]
     */
    selectionStyle?: {
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.border]
       */
      border?: {
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.border.dashStyle]
         */
        dashStyle?: DashStyleType;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.color]
       */
      color?: string;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.hatching]
       */
      hatching?: {
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.hatching.direction]
         */
        direction?: HatchingDirectionType;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.hatching.opacity]
         */
        opacity?: number;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.hatching.step]
         */
        step?: number;
        /**
         * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.selectionStyle.hatching.width]
         */
        width?: number;
      };
    };
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.smallValuesGrouping]
     */
    smallValuesGrouping?: {
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.smallValuesGrouping.groupName]
       */
      groupName?: string;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.smallValuesGrouping.mode]
       */
      mode?: 'none' | 'smallValueThreshold' | 'topN';
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.smallValuesGrouping.threshold]
       */
      threshold?: number;
      /**
       * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.smallValuesGrouping.topCount]
       */
      topCount?: number;
    };
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.tagField]
     */
    tagField?: string;
    /**
     * [descr:dxPieChartSeriesTypes.CommonPieChartSeries.valueField]
     */
    valueField?: string;
  }
  /**
   * [descr:dxPolarChart]
   */
  export class dxPolarChart extends BaseChart<dxPolarChartOptions> {
    /**
     * [descr:dxPolarChart.getValueAxis()]
     */
    getValueAxis(): chartAxisObject;
    /**
     * [descr:dxPolarChart.resetVisualRange()]
     */
    resetVisualRange(): void;
  }
  module dxPolarChart {
    export type ArgumentAxisClickEvent = DevExpress.events.NativeEventInfo<
      dxPolarChart,
      MouseEvent | PointerEvent
    > & {
      readonly argument: Date | number | string;
    };
    export type DisposingEvent = DevExpress.events.EventInfo<dxPolarChart>;
    export type DoneEvent = DevExpress.events.EventInfo<dxPolarChart>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxPolarChart>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxPolarChart>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxPolarChart> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxPolarChart>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxPolarChart> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxPolarChart>;
    export type LegendClickEvent = DevExpress.events.NativeEventInfo<
      dxPolarChart,
      MouseEvent | PointerEvent
    > & {
      readonly target: polarChartSeriesObject;
    };
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxPolarChart> &
      DevExpress.events.ChangedOptionInfo;
    export type PointClickEvent = DevExpress.events.NativeEventInfo<
      dxPolarChart,
      MouseEvent | PointerEvent
    > &
      DevExpress.viz.BaseChart.PointInteractionInfo;
    export type PointHoverChangedEvent =
      DevExpress.events.EventInfo<dxPolarChart> &
        DevExpress.viz.BaseChart.PointInteractionInfo;
    export type PointSelectionChangedEvent =
      DevExpress.events.EventInfo<dxPolarChart> &
        DevExpress.viz.BaseChart.PointInteractionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export type PolarChartSeriesType =
      | 'area'
      | 'bar'
      | 'line'
      | 'scatter'
      | 'stackedbar';
    export type Properties = dxPolarChartOptions;
    export type SeriesClickEvent = DevExpress.events.NativeEventInfo<
      dxPolarChart,
      MouseEvent | PointerEvent
    > & {
      readonly target: polarChartSeriesObject;
    };
    export type SeriesHoverChangedEvent =
      DevExpress.events.EventInfo<dxPolarChart> & SeriesInteractionInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    interface SeriesInteractionInfo {
      target: polarChartSeriesObject;
    }
    export type SeriesSelectionChangedEvent =
      DevExpress.events.EventInfo<dxPolarChart> & SeriesInteractionInfo;
    export type TooltipHiddenEvent = DevExpress.events.EventInfo<dxPolarChart> &
      DevExpress.viz.BaseChart.TooltipInfo;
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxPolarChart> &
      DevExpress.viz.BaseChart.TooltipInfo;
    export type ZoomEndEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<
        dxPolarChart,
        MouseEvent | TouchEvent
      > & {
        readonly axis: chartAxisObject;
        readonly range: VizRange;
        readonly previousRange: VizRange;
        readonly actionType: 'zoom' | 'pan';
        readonly zoomFactor: number;
        readonly shift: number;
      };
    export type ZoomStartEvent = DevExpress.events.Cancelable &
      DevExpress.events.NativeEventInfo<
        dxPolarChart,
        MouseEvent | TouchEvent
      > & {
        readonly axis: chartAxisObject;
        readonly range: VizRange;
        readonly actionType: 'zoom' | 'pan';
      };
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartAdaptiveLayout extends BaseChartAdaptiveLayout {
    /**
     * [descr:dxPolarChartOptions.adaptiveLayout.height]
     */
    height?: number;
    /**
     * [descr:dxPolarChartOptions.adaptiveLayout.width]
     */
    width?: number;
  }
  /**
   * [descr:dxPolarChartAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartAnnotationConfig
    extends dxPolarChartCommonAnnotationConfig {
    /**
     * [descr:dxPolarChartAnnotationConfig.name]
     */
    name?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxis
    extends dxPolarChartCommonAxisSettings {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.argumentType]
     */
    argumentType?: 'datetime' | 'numeric' | 'string';
    /**
     * [descr:dxPolarChartOptions.argumentAxis.axisDivisionFactor]
     */
    axisDivisionFactor?: number;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.categories]
     */
    categories?: Array<number | string | Date>;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.constantLines]
     */
    constantLines?: Array<dxPolarChartArgumentAxisConstantLines>;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.firstPointOnStartAngle]
     */
    firstPointOnStartAngle?: boolean;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.hoverMode]
     */
    hoverMode?: 'allArgumentPoints' | 'none';
    /**
     * [descr:dxPolarChartOptions.argumentAxis.label]
     */
    label?: dxPolarChartArgumentAxisLabel;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.linearThreshold]
     */
    linearThreshold?: number;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.logarithmBase]
     */
    logarithmBase?: number;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.minorTick]
     */
    minorTick?: dxPolarChartArgumentAxisMinorTick;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.minorTickCount]
     */
    minorTickCount?: number;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.minorTickInterval]
     */
    minorTickInterval?: VizTimeInterval;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.originValue]
     */
    originValue?: number;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.period]
     */
    period?: number;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.startAngle]
     */
    startAngle?: number;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.strips]
     */
    strips?: Array<dxPolarChartArgumentAxisStrips>;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.tick]
     */
    tick?: dxPolarChartArgumentAxisTick;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.tickInterval]
     */
    tickInterval?: VizTimeInterval;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.type]
     */
    type?: 'continuous' | 'discrete' | 'logarithmic';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxisConstantLines
    extends dxPolarChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.constantLines.displayBehindSeries]
     */
    displayBehindSeries?: boolean;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.constantLines.extendAxis]
     */
    extendAxis?: boolean;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.constantLines.label]
     */
    label?: dxPolarChartArgumentAxisConstantLinesLabel;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.constantLines.value]
     */
    value?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxisConstantLinesLabel
    extends dxPolarChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.constantLines.label.text]
     */
    text?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxisLabel
    extends dxPolarChartCommonAxisSettingsLabel {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.label.customizeHint]
     */
    customizeHint?: (argument: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.label.customizeText]
     */
    customizeText?: (argument: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.label.format]
     */
    format?: DevExpress.ui.Format;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxisMinorTick
    extends dxPolarChartCommonAxisSettingsMinorTick {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.minorTick.shift]
     */
    shift?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxisStrips
    extends dxPolarChartCommonAxisSettingsStripStyle {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.strips.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.strips.endValue]
     */
    endValue?: number | Date | string;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.strips.label]
     */
    label?: dxPolarChartArgumentAxisStripsLabel;
    /**
     * [descr:dxPolarChartOptions.argumentAxis.strips.startValue]
     */
    startValue?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxisStripsLabel
    extends dxPolarChartCommonAxisSettingsStripStyleLabel {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.strips.label.text]
     */
    text?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartArgumentAxisTick
    extends dxPolarChartCommonAxisSettingsTick {
    /**
     * [descr:dxPolarChartOptions.argumentAxis.tick.shift]
     */
    shift?: number;
  }
  /**
   * [descr:dxPolarChartCommonAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAnnotationConfig
    extends BaseChartAnnotationConfig {
    /**
     * [descr:dxPolarChartCommonAnnotationConfig.angle]
     */
    angle?: number;
    /**
     * [descr:dxPolarChartCommonAnnotationConfig.radius]
     */
    radius?: number;
    /**
     * [descr:dxPolarChartCommonAnnotationConfig.customizeTooltip]
     */
    customizeTooltip?: (annotation: dxPolarChartAnnotationConfig | any) => any;
    /**
     * [descr:dxPolarChartCommonAnnotationConfig.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          annotation: dxPolarChartAnnotationConfig | any,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxPolarChartCommonAnnotationConfig.tooltipTemplate]
     */
    tooltipTemplate?:
      | DevExpress.core.template
      | ((
          annotation: dxPolarChartAnnotationConfig | any,
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettings {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.allowDecimals]
     */
    allowDecimals?: boolean;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.constantLineStyle]
     */
    constantLineStyle?: dxPolarChartCommonAxisSettingsConstantLineStyle;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.discreteAxisDivisionMode]
     */
    discreteAxisDivisionMode?: 'betweenLabels' | 'crossLabels';
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.endOnTick]
     */
    endOnTick?: boolean;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.grid]
     */
    grid?: {
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.grid.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.grid.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.grid.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.grid.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.inverted]
     */
    inverted?: boolean;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.label]
     */
    label?: dxPolarChartCommonAxisSettingsLabel;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.minorGrid]
     */
    minorGrid?: {
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.minorGrid.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.minorGrid.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.minorGrid.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPolarChartOptions.commonAxisSettings.minorGrid.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.minorTick]
     */
    minorTick?: dxPolarChartCommonAxisSettingsMinorTick;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.stripStyle]
     */
    stripStyle?: dxPolarChartCommonAxisSettingsStripStyle;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.tick]
     */
    tick?: dxPolarChartCommonAxisSettingsTick;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.constantLineStyle.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.constantLineStyle.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.constantLineStyle.label]
     */
    label?: dxPolarChartCommonAxisSettingsConstantLineStyleLabel;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.constantLineStyle.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.constantLineStyle.label.font]
     */
    font?: Font;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.constantLineStyle.label.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettingsLabel {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.label.font]
     */
    font?: Font;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.label.indentFromAxis]
     */
    indentFromAxis?: number;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.label.overlappingBehavior]
     */
    overlappingBehavior?: 'none' | 'hide';
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.label.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettingsMinorTick {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.minorTick.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.minorTick.length]
     */
    length?: number;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.minorTick.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.minorTick.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.minorTick.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettingsStripStyle {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.stripStyle.label]
     */
    label?: dxPolarChartCommonAxisSettingsStripStyleLabel;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettingsStripStyleLabel {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.stripStyle.label.font]
     */
    font?: Font;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonAxisSettingsTick {
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.tick.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.tick.length]
     */
    length?: number;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.tick.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.tick.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings.tick.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartCommonSeriesSettings
    extends dxPolarChartSeriesTypesCommonPolarChartSeries {
    /**
     * [descr:dxPolarChartOptions.commonSeriesSettings.area]
     */
    area?: any;
    /**
     * [descr:dxPolarChartOptions.commonSeriesSettings.bar]
     */
    bar?: any;
    /**
     * [descr:dxPolarChartOptions.commonSeriesSettings.line]
     */
    line?: any;
    /**
     * [descr:dxPolarChartOptions.commonSeriesSettings.scatter]
     */
    scatter?: any;
    /**
     * [descr:dxPolarChartOptions.commonSeriesSettings.stackedbar]
     */
    stackedbar?: any;
    /**
     * [descr:dxPolarChartOptions.commonSeriesSettings.type]
     */
    type?: DevExpress.viz.dxPolarChart.PolarChartSeriesType;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartLegend extends BaseChartLegend {
    /**
     * [descr:dxPolarChartOptions.legend.customizeHint]
     */
    customizeHint?: (seriesInfo: {
      seriesName?: any;
      seriesIndex?: number;
      seriesColor?: string;
    }) => string;
    /**
     * [descr:dxPolarChartOptions.legend.customizeText]
     */
    customizeText?: (seriesInfo: {
      seriesName?: any;
      seriesIndex?: number;
      seriesColor?: string;
    }) => string;
    /**
     * [descr:dxPolarChartOptions.legend.hoverMode]
     */
    hoverMode?: 'excludePoints' | 'includePoints' | 'none';
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartOptions extends BaseChartOptions<dxPolarChart> {
    /**
     * [descr:dxPolarChartOptions.adaptiveLayout]
     */
    adaptiveLayout?: dxPolarChartAdaptiveLayout;
    /**
     * [descr:dxPolarChartOptions.annotations]
     */
    annotations?: Array<dxPolarChartAnnotationConfig | any>;
    /**
     * [descr:dxPolarChartOptions.argumentAxis]
     */
    argumentAxis?: dxPolarChartArgumentAxis;
    /**
     * [descr:dxPolarChartOptions.barGroupPadding]
     */
    barGroupPadding?: number;
    /**
     * [descr:dxPolarChartOptions.barGroupWidth]
     */
    barGroupWidth?: number;
    /**
     * [descr:dxPolarChartOptions.commonAnnotationSettings]
     */
    commonAnnotationSettings?: dxPolarChartCommonAnnotationConfig;
    /**
     * [descr:dxPolarChartOptions.commonAxisSettings]
     */
    commonAxisSettings?: dxPolarChartCommonAxisSettings;
    /**
     * [descr:dxPolarChartOptions.commonSeriesSettings]
     */
    commonSeriesSettings?: dxPolarChartCommonSeriesSettings;
    /**
     * [descr:dxPolarChartOptions.containerBackgroundColor]
     */
    containerBackgroundColor?: string;
    /**
     * [descr:dxPolarChartOptions.customizeAnnotation]
     */
    customizeAnnotation?: (
      annotation: dxPolarChartAnnotationConfig | any
    ) => dxPolarChartAnnotationConfig;
    /**
     * [descr:dxPolarChartOptions.dataPrepareSettings]
     */
    dataPrepareSettings?: {
      /**
       * [descr:dxPolarChartOptions.dataPrepareSettings.checkTypeForAllData]
       */
      checkTypeForAllData?: boolean;
      /**
       * [descr:dxPolarChartOptions.dataPrepareSettings.convertToAxisDataType]
       */
      convertToAxisDataType?: boolean;
      /**
       * [descr:dxPolarChartOptions.dataPrepareSettings.sortingMethod]
       */
      sortingMethod?:
        | boolean
        | ((
            a: { arg?: Date | number | string; val?: Date | number | string },
            b: { arg?: Date | number | string; val?: Date | number | string }
          ) => number);
    };
    /**
     * [descr:dxPolarChartOptions.legend]
     */
    legend?: dxPolarChartLegend;
    /**
     * [descr:dxPolarChartOptions.negativesAsZeroes]
     */
    negativesAsZeroes?: boolean;
    /**
     * [descr:dxPolarChartOptions.onArgumentAxisClick]
     */
    onArgumentAxisClick?:
      | ((e: {
          component?: dxPolarChart;
          element?: DevExpress.core.DxElement;
          model?: any;
          event?: DevExpress.events.DxEvent;
          argument?: Date | number | string;
        }) => void)
      | string;
    /**
     * [descr:dxPolarChartOptions.onLegendClick]
     */
    onLegendClick?:
      | ((e: DevExpress.viz.dxPolarChart.LegendClickEvent) => void)
      | string;
    /**
     * [descr:dxPolarChartOptions.onSeriesClick]
     */
    onSeriesClick?:
      | ((e: DevExpress.viz.dxPolarChart.SeriesClickEvent) => void)
      | string;
    /**
     * [descr:dxPolarChartOptions.onSeriesHoverChanged]
     */
    onSeriesHoverChanged?: (
      e: DevExpress.viz.dxPolarChart.SeriesHoverChangedEvent
    ) => void;
    /**
     * [descr:dxPolarChartOptions.onSeriesSelectionChanged]
     */
    onSeriesSelectionChanged?: (
      e: DevExpress.viz.dxPolarChart.SeriesSelectionChangedEvent
    ) => void;
    /**
     * [descr:dxPolarChartOptions.onZoomEnd]
     */
    onZoomEnd?: (e: DevExpress.viz.dxPolarChart.ZoomEndEvent) => void;
    /**
     * [descr:dxPolarChartOptions.onZoomStart]
     */
    onZoomStart?: (e: DevExpress.viz.dxPolarChart.ZoomStartEvent) => void;
    /**
     * [descr:dxPolarChartOptions.resolveLabelOverlapping]
     */
    resolveLabelOverlapping?: 'hide' | 'none';
    /**
     * [descr:dxPolarChartOptions.series]
     */
    series?: PolarChartSeries | Array<PolarChartSeries>;
    /**
     * [descr:dxPolarChartOptions.seriesSelectionMode]
     */
    seriesSelectionMode?: 'multiple' | 'single';
    /**
     * [descr:dxPolarChartOptions.seriesTemplate]
     */
    seriesTemplate?: {
      /**
       * [descr:dxPolarChartOptions.seriesTemplate.customizeSeries]
       */
      customizeSeries?: (seriesName: any) => PolarChartSeries;
      /**
       * [descr:dxPolarChartOptions.seriesTemplate.nameField]
       */
      nameField?: string;
    };
    /**
     * [descr:dxPolarChartOptions.tooltip]
     */
    tooltip?: dxPolarChartTooltip;
    /**
     * [descr:dxPolarChartOptions.useSpiderWeb]
     */
    useSpiderWeb?: boolean;
    /**
     * [descr:dxPolarChartOptions.valueAxis]
     */
    valueAxis?: dxPolarChartValueAxis;
  }
  /**
   * [descr:dxPolarChartSeriesTypes]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypes {
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries]
     */
    CommonPolarChartSeries?: dxPolarChartSeriesTypesCommonPolarChartSeries;
    /**
     * [descr:dxPolarChartSeriesTypes.areapolarseries]
     */
    areapolarseries?: dxPolarChartSeriesTypesAreapolarseries;
    /**
     * [descr:dxPolarChartSeriesTypes.barpolarseries]
     */
    barpolarseries?: dxPolarChartSeriesTypesBarpolarseries;
    /**
     * [descr:dxPolarChartSeriesTypes.linepolarseries]
     */
    linepolarseries?: dxPolarChartSeriesTypesLinepolarseries;
    /**
     * [descr:dxPolarChartSeriesTypes.scatterpolarseries]
     */
    scatterpolarseries?: any;
    /**
     * [descr:dxPolarChartSeriesTypes.stackedbarpolarseries]
     */
    stackedbarpolarseries?: dxPolarChartSeriesTypesStackedbarpolarseries;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesAreapolarseries
    extends dxPolarChartSeriesTypesCommonPolarChartSeries {
    /**
     * [descr:dxPolarChartSeriesTypes.areapolarseries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxPolarChartSeriesTypes.areapolarseries.point]
     */
    point?: dxPolarChartSeriesTypesAreapolarseriesPoint;
    /**
     * [descr:dxPolarChartSeriesTypes.areapolarseries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesAreapolarseriesPoint
    extends dxPolarChartSeriesTypesCommonPolarChartSeriesPoint {
    /**
     * [descr:dxPolarChartSeriesTypes.areapolarseries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesBarpolarseries
    extends dxPolarChartSeriesTypesCommonPolarChartSeries {
    /**
     * [descr:dxPolarChartSeriesTypes.barpolarseries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxPolarChartSeriesTypes.barpolarseries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesCommonPolarChartSeries {
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.argumentField]
     */
    argumentField?: string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.barPadding]
     */
    barPadding?: number;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.barWidth]
     */
    barWidth?: number;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.border]
     */
    border?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.border.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.border.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.closed]
     */
    closed?: boolean;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.dashStyle]
     */
    dashStyle?: DashStyleType;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverMode]
     */
    hoverMode?:
      | 'allArgumentPoints'
      | 'allSeriesPoints'
      | 'excludePoints'
      | 'includePoints'
      | 'nearestPoint'
      | 'none'
      | 'onlyPoint';
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle]
     */
    hoverStyle?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.border]
       */
      border?: {
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.border.dashStyle]
         */
        dashStyle?: DashStyleType;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.hatching]
       */
      hatching?: {
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.hatching.direction]
         */
        direction?: HatchingDirectionType;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.hatching.opacity]
         */
        opacity?: number;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.hatching.step]
         */
        step?: number;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.hatching.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.hoverStyle.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.ignoreEmptyPoints]
     */
    ignoreEmptyPoints?: boolean;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label]
     */
    label?: dxPolarChartSeriesTypesCommonPolarChartSeriesLabel;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.maxLabelCount]
     */
    maxLabelCount?: number;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.minBarSize]
     */
    minBarSize?: number;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.opacity]
     */
    opacity?: number;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point]
     */
    point?: dxPolarChartSeriesTypesCommonPolarChartSeriesPoint;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionMode]
     */
    selectionMode?:
      | 'allArgumentPoints'
      | 'allSeriesPoints'
      | 'excludePoints'
      | 'includePoints'
      | 'none'
      | 'onlyPoint';
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle]
     */
    selectionStyle?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.border]
       */
      border?: {
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.border.dashStyle]
         */
        dashStyle?: DashStyleType;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.hatching]
       */
      hatching?: {
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.hatching.direction]
         */
        direction?: HatchingDirectionType;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.hatching.opacity]
         */
        opacity?: number;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.hatching.step]
         */
        step?: number;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.hatching.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.selectionStyle.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.showInLegend]
     */
    showInLegend?: boolean;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.stack]
     */
    stack?: string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.tagField]
     */
    tagField?: string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar]
     */
    valueErrorBar?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.displayMode]
       */
      displayMode?: 'auto' | 'high' | 'low' | 'none';
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.edgeLength]
       */
      edgeLength?: number;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.highValueField]
       */
      highValueField?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.lineWidth]
       */
      lineWidth?: number;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.lowValueField]
       */
      lowValueField?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.type]
       */
      type?: 'fixed' | 'percent' | 'stdDeviation' | 'stdError' | 'variance';
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueErrorBar.value]
       */
      value?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.valueField]
     */
    valueField?: string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.visible]
     */
    visible?: boolean;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.width]
     */
    width?: number;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesCommonPolarChartSeriesLabel {
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.argumentFormat]
     */
    argumentFormat?: DevExpress.ui.Format;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.backgroundColor]
     */
    backgroundColor?: string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.border]
     */
    border?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.border.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.border.dashStyle]
       */
      dashStyle?: DashStyleType;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.border.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.connector]
     */
    connector?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.connector.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.connector.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.connector.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.customizeText]
     */
    customizeText?: (pointInfo: any) => string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.font]
     */
    font?: Font;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.format]
     */
    format?: DevExpress.ui.Format;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.position]
     */
    position?: 'inside' | 'outside';
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.rotationAngle]
     */
    rotationAngle?: number;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.showForZeroValues]
     */
    showForZeroValues?: boolean;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.label.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesCommonPolarChartSeriesPoint {
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.border]
     */
    border?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.border.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.border.visible]
       */
      visible?: boolean;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.border.width]
       */
      width?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverMode]
     */
    hoverMode?: 'allArgumentPoints' | 'allSeriesPoints' | 'none' | 'onlyPoint';
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverStyle]
     */
    hoverStyle?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverStyle.border]
       */
      border?: {
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverStyle.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.hoverStyle.size]
       */
      size?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.image]
     */
    image?:
      | string
      | {
          /**
           * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.image.height]
           */
          height?: number;
          /**
           * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.image.url]
           */
          url?: string;
          /**
           * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.image.width]
           */
          width?: number;
        };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionMode]
     */
    selectionMode?:
      | 'allArgumentPoints'
      | 'allSeriesPoints'
      | 'none'
      | 'onlyPoint';
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionStyle]
     */
    selectionStyle?: {
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionStyle.border]
       */
      border?: {
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionStyle.border.color]
         */
        color?: string;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionStyle.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionStyle.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionStyle.color]
       */
      color?: string;
      /**
       * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.selectionStyle.size]
       */
      size?: number;
    };
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.size]
     */
    size?: number;
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.symbol]
     */
    symbol?: 'circle' | 'cross' | 'polygon' | 'square' | 'triangle';
    /**
     * [descr:dxPolarChartSeriesTypes.CommonPolarChartSeries.point.visible]
     */
    visible?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesLinepolarseries
    extends dxPolarChartSeriesTypesCommonPolarChartSeries {
    /**
     * [descr:dxPolarChartSeriesTypes.linepolarseries.hoverMode]
     */
    hoverMode?: 'nearestPoint' | 'includePoints' | 'excludePoints' | 'none';
    /**
     * [descr:dxPolarChartSeriesTypes.linepolarseries.selectionMode]
     */
    selectionMode?: 'includePoints' | 'excludePoints' | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesStackedbarpolarseries
    extends dxPolarChartSeriesTypesCommonPolarChartSeries {
    /**
     * [descr:dxPolarChartSeriesTypes.stackedbarpolarseries.hoverMode]
     */
    hoverMode?: 'onlyPoint' | 'allSeriesPoints' | 'allArgumentPoints' | 'none';
    /**
     * [descr:dxPolarChartSeriesTypes.stackedbarpolarseries.label]
     */
    label?: dxPolarChartSeriesTypesStackedbarpolarseriesLabel;
    /**
     * [descr:dxPolarChartSeriesTypes.stackedbarpolarseries.selectionMode]
     */
    selectionMode?:
      | 'onlyPoint'
      | 'allSeriesPoints'
      | 'allArgumentPoints'
      | 'none';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartSeriesTypesStackedbarpolarseriesLabel
    extends dxPolarChartSeriesTypesCommonPolarChartSeriesLabel {
    /**
     * [descr:dxPolarChartSeriesTypes.stackedbarpolarseries.label.position]
     */
    position?: 'inside' | 'outside';
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartTooltip extends BaseChartTooltip {
    /**
     * [descr:dxPolarChartOptions.tooltip.shared]
     */
    shared?: boolean;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartValueAxis
    extends dxPolarChartCommonAxisSettings {
    /**
     * [descr:dxPolarChartOptions.valueAxis.axisDivisionFactor]
     */
    axisDivisionFactor?: number;
    /**
     * [descr:dxPolarChartOptions.valueAxis.categories]
     */
    categories?: Array<number | string | Date>;
    /**
     * [descr:dxPolarChartOptions.valueAxis.constantLines]
     */
    constantLines?: Array<dxPolarChartValueAxisConstantLines>;
    /**
     * [descr:dxPolarChartOptions.valueAxis.endOnTick]
     */
    endOnTick?: boolean;
    /**
     * [descr:dxPolarChartOptions.valueAxis.label]
     */
    label?: dxPolarChartValueAxisLabel;
    /**
     * [descr:dxPolarChartOptions.valueAxis.linearThreshold]
     */
    linearThreshold?: number;
    /**
     * [descr:dxPolarChartOptions.valueAxis.logarithmBase]
     */
    logarithmBase?: number;
    /**
     * [descr:dxPolarChartOptions.valueAxis.maxValueMargin]
     */
    maxValueMargin?: number;
    /**
     * [descr:dxPolarChartOptions.valueAxis.minValueMargin]
     */
    minValueMargin?: number;
    /**
     * [descr:dxPolarChartOptions.valueAxis.minVisualRangeLength]
     */
    minVisualRangeLength?: VizTimeInterval;
    /**
     * [descr:dxPolarChartOptions.valueAxis.minorTickCount]
     */
    minorTickCount?: number;
    /**
     * [descr:dxPolarChartOptions.valueAxis.minorTickInterval]
     */
    minorTickInterval?: VizTimeInterval;
    /**
     * [descr:dxPolarChartOptions.valueAxis.showZero]
     */
    showZero?: boolean;
    /**
     * [descr:dxPolarChartOptions.valueAxis.strips]
     */
    strips?: Array<dxPolarChartValueAxisStrips>;
    /**
     * [descr:dxPolarChartOptions.valueAxis.tick]
     */
    tick?: dxPolarChartValueAxisTick;
    /**
     * [descr:dxPolarChartOptions.valueAxis.tickInterval]
     */
    tickInterval?: VizTimeInterval;
    /**
     * [descr:dxPolarChartOptions.valueAxis.type]
     */
    type?: 'continuous' | 'discrete' | 'logarithmic';
    /**
     * [descr:dxPolarChartOptions.valueAxis.valueMarginsEnabled]
     */
    valueMarginsEnabled?: boolean;
    /**
     * [descr:dxPolarChartOptions.valueAxis.valueType]
     */
    valueType?: 'datetime' | 'numeric' | 'string';
    /**
     * [descr:dxPolarChartOptions.valueAxis.visualRange]
     */
    visualRange?: VizRange | Array<number | string | Date>;
    /**
     * [descr:dxPolarChartOptions.valueAxis.visualRangeUpdateMode]
     */
    visualRangeUpdateMode?: 'auto' | 'keep' | 'reset';
    /**
     * [descr:dxPolarChartOptions.valueAxis.wholeRange]
     */
    wholeRange?: VizRange | Array<number | string | Date>;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartValueAxisConstantLines
    extends dxPolarChartCommonAxisSettingsConstantLineStyle {
    /**
     * [descr:dxPolarChartOptions.valueAxis.constantLines.displayBehindSeries]
     */
    displayBehindSeries?: boolean;
    /**
     * [descr:dxPolarChartOptions.valueAxis.constantLines.extendAxis]
     */
    extendAxis?: boolean;
    /**
     * [descr:dxPolarChartOptions.valueAxis.constantLines.label]
     */
    label?: dxPolarChartValueAxisConstantLinesLabel;
    /**
     * [descr:dxPolarChartOptions.valueAxis.constantLines.value]
     */
    value?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartValueAxisConstantLinesLabel
    extends dxPolarChartCommonAxisSettingsConstantLineStyleLabel {
    /**
     * [descr:dxPolarChartOptions.valueAxis.constantLines.label.text]
     */
    text?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartValueAxisLabel
    extends dxPolarChartCommonAxisSettingsLabel {
    /**
     * [descr:dxPolarChartOptions.valueAxis.label.customizeHint]
     */
    customizeHint?: (axisValue: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxPolarChartOptions.valueAxis.label.customizeText]
     */
    customizeText?: (axisValue: {
      value?: Date | number | string;
      valueText?: string;
    }) => string;
    /**
     * [descr:dxPolarChartOptions.valueAxis.label.format]
     */
    format?: DevExpress.ui.Format;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartValueAxisStrips
    extends dxPolarChartCommonAxisSettingsStripStyle {
    /**
     * [descr:dxPolarChartOptions.valueAxis.strips.color]
     */
    color?: string;
    /**
     * [descr:dxPolarChartOptions.valueAxis.strips.endValue]
     */
    endValue?: number | Date | string;
    /**
     * [descr:dxPolarChartOptions.valueAxis.strips.label]
     */
    label?: dxPolarChartValueAxisStripsLabel;
    /**
     * [descr:dxPolarChartOptions.valueAxis.strips.startValue]
     */
    startValue?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartValueAxisStripsLabel
    extends dxPolarChartCommonAxisSettingsStripStyleLabel {
    /**
     * [descr:dxPolarChartOptions.valueAxis.strips.label.text]
     */
    text?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxPolarChartValueAxisTick
    extends dxPolarChartCommonAxisSettingsTick {
    /**
     * [descr:dxPolarChartOptions.valueAxis.tick.visible]
     */
    visible?: boolean;
  }
  /**
   * [descr:dxRangeSelector]
   */
  export class dxRangeSelector extends BaseWidget<dxRangeSelectorOptions> {
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxRangeSelector.getValue()]
     */
    getValue(): Array<number | string | Date>;
    render(): void;
    /**
     * [descr:dxRangeSelector.render(skipChartAnimation)]
     */
    render(skipChartAnimation: boolean): void;
    /**
     * [descr:dxRangeSelector.setValue(value)]
     */
    setValue(value: Array<number | string | Date> | VizRange): void;
  }
  module dxRangeSelector {
    export type DisposingEvent = DevExpress.events.EventInfo<dxRangeSelector>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxRangeSelector>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxRangeSelector>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxRangeSelector> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxRangeSelector>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxRangeSelector> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxRangeSelector>;
    export type OptionChangedEvent =
      DevExpress.events.EventInfo<dxRangeSelector> &
        DevExpress.events.ChangedOptionInfo;
    export type Properties = dxRangeSelectorOptions;
    export type ValueChangedEvent = DevExpress.events.NativeEventInfo<
      dxRangeSelector,
      MouseEvent | TouchEvent
    > & {
      readonly value: Array<number | string | Date>;
      readonly previousValue: Array<number | string | Date>;
    };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxRangeSelectorOptions
    extends BaseWidgetOptions<dxRangeSelector> {
    /**
     * [descr:dxRangeSelectorOptions.background]
     */
    background?: {
      /**
       * [descr:dxRangeSelectorOptions.background.color]
       */
      color?: string;
      /**
       * [descr:dxRangeSelectorOptions.background.image]
       */
      image?: {
        /**
         * [descr:dxRangeSelectorOptions.background.image.location]
         */
        location?:
          | 'center'
          | 'centerBottom'
          | 'centerTop'
          | 'full'
          | 'leftBottom'
          | 'leftCenter'
          | 'leftTop'
          | 'rightBottom'
          | 'rightCenter'
          | 'rightTop';
        /**
         * [descr:dxRangeSelectorOptions.background.image.url]
         */
        url?: string;
      };
      /**
       * [descr:dxRangeSelectorOptions.background.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxRangeSelectorOptions.behavior]
     */
    behavior?: {
      /**
       * [descr:dxRangeSelectorOptions.behavior.allowSlidersSwap]
       */
      allowSlidersSwap?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.behavior.animationEnabled]
       */
      animationEnabled?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.behavior.callValueChanged]
       */
      callValueChanged?: 'onMoving' | 'onMovingComplete';
      /**
       * [descr:dxRangeSelectorOptions.behavior.manualRangeSelectionEnabled]
       */
      manualRangeSelectionEnabled?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.behavior.moveSelectedRangeByClick]
       */
      moveSelectedRangeByClick?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.behavior.snapToTicks]
       */
      snapToTicks?: boolean;
    };
    /**
     * [descr:dxRangeSelectorOptions.chart]
     */
    chart?: {
      /**
       * [descr:dxRangeSelectorOptions.chart.barGroupPadding]
       */
      barGroupPadding?: number;
      /**
       * [descr:dxRangeSelectorOptions.chart.barGroupWidth]
       */
      barGroupWidth?: number;
      /**
       * [descr:dxRangeSelectorOptions.chart.bottomIndent]
       */
      bottomIndent?: number;
      /**
       * [descr:dxRangeSelectorOptions.chart.commonSeriesSettings]
       */
      commonSeriesSettings?: dxChartCommonSeriesSettings;
      /**
       * [descr:dxRangeSelectorOptions.chart.dataPrepareSettings]
       */
      dataPrepareSettings?: {
        /**
         * [descr:dxRangeSelectorOptions.chart.dataPrepareSettings.checkTypeForAllData]
         */
        checkTypeForAllData?: boolean;
        /**
         * [descr:dxRangeSelectorOptions.chart.dataPrepareSettings.convertToAxisDataType]
         */
        convertToAxisDataType?: boolean;
        /**
         * [descr:dxRangeSelectorOptions.chart.dataPrepareSettings.sortingMethod]
         */
        sortingMethod?:
          | boolean
          | ((
              a: { arg?: Date | number | string; val?: Date | number | string },
              b: { arg?: Date | number | string; val?: Date | number | string }
            ) => number);
      };
      /**
       * [descr:dxRangeSelectorOptions.chart.maxBubbleSize]
       */
      maxBubbleSize?: number;
      /**
       * [descr:dxRangeSelectorOptions.chart.minBubbleSize]
       */
      minBubbleSize?: number;
      /**
       * [descr:dxRangeSelectorOptions.chart.negativesAsZeroes]
       */
      negativesAsZeroes?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.chart.palette]
       */
      palette?: Array<string> | PaletteType;
      /**
       * [descr:dxRangeSelectorOptions.chart.paletteExtensionMode]
       */
      paletteExtensionMode?: PaletteExtensionModeType;
      /**
       * [descr:dxRangeSelectorOptions.chart.series]
       */
      series?: ChartSeries | Array<ChartSeries>;
      /**
       * [descr:dxRangeSelectorOptions.chart.seriesTemplate]
       */
      seriesTemplate?: {
        /**
         * [descr:dxRangeSelectorOptions.chart.seriesTemplate.customizeSeries]
         */
        customizeSeries?: (seriesName: any) => ChartSeries;
        /**
         * [descr:dxRangeSelectorOptions.chart.seriesTemplate.nameField]
         */
        nameField?: string;
      };
      /**
       * [descr:dxRangeSelectorOptions.chart.topIndent]
       */
      topIndent?: number;
      /**
       * [descr:dxRangeSelectorOptions.chart.valueAxis]
       */
      valueAxis?: {
        /**
         * [descr:dxRangeSelectorOptions.chart.valueAxis.inverted]
         */
        inverted?: boolean;
        /**
         * [descr:dxRangeSelectorOptions.chart.valueAxis.logarithmBase]
         */
        logarithmBase?: number;
        /**
         * [descr:dxRangeSelectorOptions.chart.valueAxis.max]
         */
        max?: number;
        /**
         * [descr:dxRangeSelectorOptions.chart.valueAxis.min]
         */
        min?: number;
        /**
         * [descr:dxRangeSelectorOptions.chart.valueAxis.type]
         */
        type?: 'continuous' | 'logarithmic';
        /**
         * [descr:dxRangeSelectorOptions.chart.valueAxis.valueType]
         */
        valueType?: 'datetime' | 'numeric' | 'string';
      };
    };
    /**
     * [descr:dxRangeSelectorOptions.containerBackgroundColor]
     */
    containerBackgroundColor?: string;
    /**
     * [descr:dxRangeSelectorOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
    /**
     * [descr:dxRangeSelectorOptions.dataSourceField]
     */
    dataSourceField?: string;
    /**
     * [descr:dxRangeSelectorOptions.indent]
     */
    indent?: {
      /**
       * [descr:dxRangeSelectorOptions.indent.left]
       */
      left?: number;
      /**
       * [descr:dxRangeSelectorOptions.indent.right]
       */
      right?: number;
    };
    /**
     * [descr:dxRangeSelectorOptions.onValueChanged]
     */
    onValueChanged?: (
      e: DevExpress.viz.dxRangeSelector.ValueChangedEvent
    ) => void;
    /**
     * [descr:dxRangeSelectorOptions.scale]
     */
    scale?: {
      /**
       * [descr:dxRangeSelectorOptions.scale.aggregateByCategory]
       */
      aggregateByCategory?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.scale.aggregationGroupWidth]
       */
      aggregationGroupWidth?: number;
      /**
       * [descr:dxRangeSelectorOptions.scale.aggregationInterval]
       */
      aggregationInterval?: VizTimeInterval;
      /**
       * [descr:dxRangeSelectorOptions.scale.allowDecimals]
       */
      allowDecimals?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.scale.breakStyle]
       */
      breakStyle?: {
        /**
         * [descr:dxRangeSelectorOptions.scale.breakStyle.color]
         */
        color?: string;
        /**
         * [descr:dxRangeSelectorOptions.scale.breakStyle.line]
         */
        line?: 'straight' | 'waved';
        /**
         * [descr:dxRangeSelectorOptions.scale.breakStyle.width]
         */
        width?: number;
      };
      /**
       * [descr:dxRangeSelectorOptions.scale.breaks]
       */
      breaks?: Array<ScaleBreak>;
      /**
       * [descr:dxRangeSelectorOptions.scale.categories]
       */
      categories?: Array<number | string | Date>;
      /**
       * [descr:dxRangeSelectorOptions.scale.endOnTick]
       */
      endOnTick?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.scale.endValue]
       */
      endValue?: number | Date | string;
      /**
       * [descr:dxRangeSelectorOptions.scale.holidays]
       */
      holidays?: Array<Date | string> | Array<number>;
      /**
       * [descr:dxRangeSelectorOptions.scale.label]
       */
      label?: {
        /**
         * [descr:dxRangeSelectorOptions.scale.label.customizeText]
         */
        customizeText?: (scaleValue: {
          value?: Date | number | string;
          valueText?: string;
        }) => string;
        /**
         * [descr:dxRangeSelectorOptions.scale.label.font]
         */
        font?: Font;
        /**
         * [descr:dxRangeSelectorOptions.scale.label.format]
         */
        format?: DevExpress.ui.Format;
        /**
         * [descr:dxRangeSelectorOptions.scale.label.overlappingBehavior]
         */
        overlappingBehavior?: 'hide' | 'none';
        /**
         * [descr:dxRangeSelectorOptions.scale.label.topIndent]
         */
        topIndent?: number;
        /**
         * [descr:dxRangeSelectorOptions.scale.label.visible]
         */
        visible?: boolean;
      };
      /**
       * [descr:dxRangeSelectorOptions.scale.linearThreshold]
       */
      linearThreshold?: number;
      /**
       * [descr:dxRangeSelectorOptions.scale.logarithmBase]
       */
      logarithmBase?: number;
      /**
       * [descr:dxRangeSelectorOptions.scale.marker]
       */
      marker?: {
        /**
         * [descr:dxRangeSelectorOptions.scale.marker.label]
         */
        label?: {
          /**
           * [descr:dxRangeSelectorOptions.scale.marker.label.customizeText]
           */
          customizeText?: (markerValue: {
            value?: Date | number;
            valueText?: string;
          }) => string;
          /**
           * [descr:dxRangeSelectorOptions.scale.marker.label.format]
           */
          format?: DevExpress.ui.Format;
        };
        /**
         * [descr:dxRangeSelectorOptions.scale.marker.separatorHeight]
         */
        separatorHeight?: number;
        /**
         * [descr:dxRangeSelectorOptions.scale.marker.textLeftIndent]
         */
        textLeftIndent?: number;
        /**
         * [descr:dxRangeSelectorOptions.scale.marker.textTopIndent]
         */
        textTopIndent?: number;
        /**
         * [descr:dxRangeSelectorOptions.scale.marker.topIndent]
         */
        topIndent?: number;
        /**
         * [descr:dxRangeSelectorOptions.scale.marker.visible]
         */
        visible?: boolean;
      };
      /**
       * [descr:dxRangeSelectorOptions.scale.maxRange]
       */
      maxRange?: VizTimeInterval;
      /**
       * [descr:dxRangeSelectorOptions.scale.minRange]
       */
      minRange?: VizTimeInterval;
      /**
       * [descr:dxRangeSelectorOptions.scale.minorTick]
       */
      minorTick?: {
        /**
         * [descr:dxRangeSelectorOptions.scale.minorTick.color]
         */
        color?: string;
        /**
         * [descr:dxRangeSelectorOptions.scale.minorTick.opacity]
         */
        opacity?: number;
        /**
         * [descr:dxRangeSelectorOptions.scale.minorTick.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxRangeSelectorOptions.scale.minorTick.width]
         */
        width?: number;
      };
      /**
       * [descr:dxRangeSelectorOptions.scale.minorTickCount]
       */
      minorTickCount?: number;
      /**
       * [descr:dxRangeSelectorOptions.scale.minorTickInterval]
       */
      minorTickInterval?: VizTimeInterval;
      /**
       * [descr:dxRangeSelectorOptions.scale.placeholderHeight]
       */
      placeholderHeight?: number;
      /**
       * [descr:dxRangeSelectorOptions.scale.showCustomBoundaryTicks]
       */
      showCustomBoundaryTicks?: boolean;
      /**
       * [descr:dxRangeSelectorOptions.scale.singleWorkdays]
       */
      singleWorkdays?: Array<Date | string> | Array<number>;
      /**
       * [descr:dxRangeSelectorOptions.scale.startValue]
       */
      startValue?: number | Date | string;
      /**
       * [descr:dxRangeSelectorOptions.scale.tick]
       */
      tick?: {
        /**
         * [descr:dxRangeSelectorOptions.scale.tick.color]
         */
        color?: string;
        /**
         * [descr:dxRangeSelectorOptions.scale.tick.opacity]
         */
        opacity?: number;
        /**
         * [descr:dxRangeSelectorOptions.scale.tick.width]
         */
        width?: number;
      };
      /**
       * [descr:dxRangeSelectorOptions.scale.tickInterval]
       */
      tickInterval?: VizTimeInterval;
      /**
       * [descr:dxRangeSelectorOptions.scale.type]
       */
      type?: 'continuous' | 'discrete' | 'logarithmic' | 'semidiscrete';
      /**
       * [descr:dxRangeSelectorOptions.scale.valueType]
       */
      valueType?: 'datetime' | 'numeric' | 'string';
      /**
       * [descr:dxRangeSelectorOptions.scale.workWeek]
       */
      workWeek?: Array<number>;
      /**
       * [descr:dxRangeSelectorOptions.scale.workdaysOnly]
       */
      workdaysOnly?: boolean;
    };
    /**
     * [descr:dxRangeSelectorOptions.selectedRangeColor]
     */
    selectedRangeColor?: string;
    /**
     * [descr:dxRangeSelectorOptions.selectedRangeUpdateMode]
     */
    selectedRangeUpdateMode?: 'auto' | 'keep' | 'reset' | 'shift';
    /**
     * [descr:dxRangeSelectorOptions.shutter]
     */
    shutter?: {
      /**
       * [descr:dxRangeSelectorOptions.shutter.color]
       */
      color?: string;
      /**
       * [descr:dxRangeSelectorOptions.shutter.opacity]
       */
      opacity?: number;
    };
    /**
     * [descr:dxRangeSelectorOptions.sliderHandle]
     */
    sliderHandle?: {
      /**
       * [descr:dxRangeSelectorOptions.sliderHandle.color]
       */
      color?: string;
      /**
       * [descr:dxRangeSelectorOptions.sliderHandle.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxRangeSelectorOptions.sliderHandle.width]
       */
      width?: number;
    };
    /**
     * [descr:dxRangeSelectorOptions.sliderMarker]
     */
    sliderMarker?: {
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.color]
       */
      color?: string;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.customizeText]
       */
      customizeText?: (scaleValue: {
        value?: Date | number | string;
        valueText?: string;
      }) => string;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.font]
       */
      font?: Font;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.format]
       */
      format?: DevExpress.ui.Format;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.invalidRangeColor]
       */
      invalidRangeColor?: string;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.paddingLeftRight]
       */
      paddingLeftRight?: number;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.paddingTopBottom]
       */
      paddingTopBottom?: number;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.placeholderHeight]
       */
      placeholderHeight?: number;
      /**
       * [descr:dxRangeSelectorOptions.sliderMarker.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxRangeSelectorOptions.tooltip]
     */
    tooltip?: BaseWidgetTooltip;
    /**
     * [descr:dxRangeSelectorOptions.value]
     */
    value?: Array<number | string | Date> | VizRange;
  }
  /**
   * [descr:dxSankey]
   */
  export class dxSankey extends BaseWidget<dxSankeyOptions> {
    /**
     * [descr:dxSankey.getAllLinks()]
     */
    getAllLinks(): Array<dxSankeyLink>;
    /**
     * [descr:dxSankey.getAllNodes()]
     */
    getAllNodes(): Array<dxSankeyNode>;
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxSankey.hideTooltip()]
     */
    hideTooltip(): void;
  }
  module dxSankey {
    export type DisposingEvent = DevExpress.events.EventInfo<dxSankey>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxSankey>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxSankey>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxSankey> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxSankey>;
    export type IncidentOccurredEvent = DevExpress.events.EventInfo<dxSankey> &
      DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSankey>;
    export type LinkClickEvent = DevExpress.events.NativeEventInfo<
      dxSankey,
      MouseEvent | PointerEvent
    > & {
      readonly target: dxSankeyLink;
    };
    export type LinkHoverEvent = DevExpress.events.EventInfo<dxSankey> & {
      readonly target: dxSankeyLink;
    };
    export type NodeClickEvent = DevExpress.events.NativeEventInfo<
      dxSankey,
      MouseEvent | PointerEvent
    > & {
      readonly target: dxSankeyNode;
    };
    export type NodeHoverEvent = DevExpress.events.EventInfo<dxSankey> & {
      readonly target: dxSankeyNode;
    };
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxSankey> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSankeyOptions;
  }
  /**
   * [descr:dxSankeyConnectionInfoObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSankeyConnectionInfoObject {
    /**
     * [descr:dxSankeyConnectionInfoObject.source]
     */
    source?: string;
    /**
     * [descr:dxSankeyConnectionInfoObject.target]
     */
    target?: string;
    /**
     * [descr:dxSankeyConnectionInfoObject.weight]
     */
    weight?: number;
  }
  /**
   * [descr:dxSankeyLink]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSankeyLink {
    /**
     * [descr:dxSankeyLink.connection]
     */
    connection?: dxSankeyConnectionInfoObject;
    /**
     * [descr:dxSankeyLink.hideTooltip()]
     */
    hideTooltip(): void;
    /**
     * [descr:dxSankeyLink.hover(state)]
     */
    hover(state: boolean): void;
    /**
     * [descr:dxSankeyLink.isHovered()]
     */
    isHovered(): boolean;
    /**
     * [descr:dxSankeyLink.showTooltip()]
     */
    showTooltip(): void;
  }
  /**
   * [descr:dxSankeyNode]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSankeyNode {
    /**
     * [descr:dxSankeyNode.hideTooltip()]
     */
    hideTooltip(): void;
    /**
     * [descr:dxSankeyNode.hover(state)]
     */
    hover(state: boolean): void;
    /**
     * [descr:dxSankeyNode.isHovered()]
     */
    isHovered(): boolean;
    /**
     * [descr:dxSankeyNode.label]
     */
    label?: string;
    /**
     * [descr:dxSankeyNode.linksIn]
     */
    linksIn?: Array<any>;
    /**
     * [descr:dxSankeyNode.linksOut]
     */
    linksOut?: Array<any>;
    /**
     * [descr:dxSankeyNode.showTooltip()]
     */
    showTooltip(): void;
    /**
     * [descr:dxSankeyNode.title]
     * @deprecated [depNote:dxSankeyNode.title]
     */
    title?: string;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSankeyOptions extends BaseWidgetOptions<dxSankey> {
    /**
     * [descr:dxSankeyOptions.adaptiveLayout]
     */
    adaptiveLayout?: {
      /**
       * [descr:dxSankeyOptions.adaptiveLayout.height]
       */
      height?: number;
      /**
       * [descr:dxSankeyOptions.adaptiveLayout.keepLabels]
       */
      keepLabels?: boolean;
      /**
       * [descr:dxSankeyOptions.adaptiveLayout.width]
       */
      width?: number;
    };
    /**
     * [descr:dxSankeyOptions.alignment]
     */
    alignment?:
      | 'bottom'
      | 'center'
      | 'top'
      | Array<'bottom' | 'center' | 'top'>;
    /**
     * [descr:dxSankeyOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
    /**
     * [descr:dxSankeyOptions.hoverEnabled]
     */
    hoverEnabled?: boolean;
    /**
     * [descr:dxSankeyOptions.label]
     */
    label?: {
      /**
       * [descr:dxSankeyOptions.label.border]
       */
      border?: {
        /**
         * [descr:dxSankeyOptions.label.border.color]
         */
        color?: string;
        /**
         * [descr:dxSankeyOptions.label.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxSankeyOptions.label.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxSankeyOptions.label.customizeText]
       */
      customizeText?: (itemInfo: dxSankeyNode) => string;
      /**
       * [descr:dxSankeyOptions.label.font]
       */
      font?: Font;
      /**
       * [descr:dxSankeyOptions.label.horizontalOffset]
       */
      horizontalOffset?: number;
      /**
       * [descr:dxSankeyOptions.label.overlappingBehavior]
       */
      overlappingBehavior?: 'ellipsis' | 'hide' | 'none';
      /**
       * [descr:dxSankeyOptions.label.shadow]
       */
      shadow?: {
        /**
         * [descr:dxSankeyOptions.label.shadow.blur]
         */
        blur?: number;
        /**
         * [descr:dxSankeyOptions.label.shadow.color]
         */
        color?: string;
        /**
         * [descr:dxSankeyOptions.label.shadow.offsetX]
         */
        offsetX?: number;
        /**
         * [descr:dxSankeyOptions.label.shadow.offsetY]
         */
        offsetY?: number;
        /**
         * [descr:dxSankeyOptions.label.shadow.opacity]
         */
        opacity?: number;
      };
      /**
       * [descr:dxSankeyOptions.label.useNodeColors]
       */
      useNodeColors?: boolean;
      /**
       * [descr:dxSankeyOptions.label.verticalOffset]
       */
      verticalOffset?: number;
      /**
       * [descr:dxSankeyOptions.label.visible]
       */
      visible?: boolean;
    };
    /**
     * [descr:dxSankeyOptions.link]
     */
    link?: {
      /**
       * [descr:dxSankeyOptions.link.border]
       */
      border?: {
        /**
         * [descr:dxSankeyOptions.link.border.color]
         */
        color?: string;
        /**
         * [descr:dxSankeyOptions.link.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxSankeyOptions.link.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxSankeyOptions.link.color]
       */
      color?: string;
      /**
       * [descr:dxSankeyOptions.link.colorMode]
       */
      colorMode?: 'none' | 'source' | 'target' | 'gradient';
      /**
       * [descr:dxSankeyOptions.link.hoverStyle]
       */
      hoverStyle?: {
        /**
         * [descr:dxSankeyOptions.link.hoverStyle.border]
         */
        border?: {
          /**
           * [descr:dxSankeyOptions.link.hoverStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxSankeyOptions.link.hoverStyle.border.visible]
           */
          visible?: boolean;
          /**
           * [descr:dxSankeyOptions.link.hoverStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxSankeyOptions.link.hoverStyle.color]
         */
        color?: string;
        /**
         * [descr:dxSankeyOptions.link.hoverStyle.hatching]
         */
        hatching?: {
          /**
           * [descr:dxSankeyOptions.link.hoverStyle.hatching.direction]
           */
          direction?: HatchingDirectionType;
          /**
           * [descr:dxSankeyOptions.link.hoverStyle.hatching.opacity]
           */
          opacity?: number;
          /**
           * [descr:dxSankeyOptions.link.hoverStyle.hatching.step]
           */
          step?: number;
          /**
           * [descr:dxSankeyOptions.link.hoverStyle.hatching.width]
           */
          width?: number;
        };
        /**
         * [descr:dxSankeyOptions.link.hoverStyle.opacity]
         */
        opacity?: number;
      };
      /**
       * [descr:dxSankeyOptions.link.opacity]
       */
      opacity?: number;
    };
    /**
     * [descr:dxSankeyOptions.node]
     */
    node?: {
      /**
       * [descr:dxSankeyOptions.node.border]
       */
      border?: {
        /**
         * [descr:dxSankeyOptions.node.border.color]
         */
        color?: string;
        /**
         * [descr:dxSankeyOptions.node.border.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxSankeyOptions.node.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxSankeyOptions.node.color]
       */
      color?: string;
      /**
       * [descr:dxSankeyOptions.node.hoverStyle]
       */
      hoverStyle?: {
        /**
         * [descr:dxSankeyOptions.node.hoverStyle.border]
         */
        border?: {
          /**
           * [descr:dxSankeyOptions.node.hoverStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxSankeyOptions.node.hoverStyle.border.visible]
           */
          visible?: boolean;
          /**
           * [descr:dxSankeyOptions.node.hoverStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxSankeyOptions.node.hoverStyle.color]
         */
        color?: string;
        /**
         * [descr:dxSankeyOptions.node.hoverStyle.hatching]
         */
        hatching?: {
          /**
           * [descr:dxSankeyOptions.node.hoverStyle.hatching.direction]
           */
          direction?: HatchingDirectionType;
          /**
           * [descr:dxSankeyOptions.node.hoverStyle.hatching.opacity]
           */
          opacity?: number;
          /**
           * [descr:dxSankeyOptions.node.hoverStyle.hatching.step]
           */
          step?: number;
          /**
           * [descr:dxSankeyOptions.node.hoverStyle.hatching.width]
           */
          width?: number;
        };
        /**
         * [descr:dxSankeyOptions.node.hoverStyle.opacity]
         */
        opacity?: number;
      };
      /**
       * [descr:dxSankeyOptions.node.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxSankeyOptions.node.padding]
       */
      padding?: number;
      /**
       * [descr:dxSankeyOptions.node.width]
       */
      width?: number;
    };
    /**
     * [descr:dxSankeyOptions.onLinkClick]
     */
    onLinkClick?:
      | ((e: DevExpress.viz.dxSankey.LinkClickEvent) => void)
      | string;
    /**
     * [descr:dxSankeyOptions.onLinkHoverChanged]
     */
    onLinkHoverChanged?: (e: DevExpress.viz.dxSankey.LinkHoverEvent) => void;
    /**
     * [descr:dxSankeyOptions.onNodeClick]
     */
    onNodeClick?:
      | ((e: DevExpress.viz.dxSankey.NodeClickEvent) => void)
      | string;
    /**
     * [descr:dxSankeyOptions.onNodeHoverChanged]
     */
    onNodeHoverChanged?: (e: DevExpress.viz.dxSankey.NodeHoverEvent) => void;
    /**
     * [descr:dxSankeyOptions.palette]
     */
    palette?: Array<string> | PaletteType;
    /**
     * [descr:dxSankeyOptions.paletteExtensionMode]
     */
    paletteExtensionMode?: PaletteExtensionModeType;
    /**
     * [descr:dxSankeyOptions.sortData]
     */
    sortData?: any;
    /**
     * [descr:dxSankeyOptions.sourceField]
     */
    sourceField?: string;
    /**
     * [descr:dxSankeyOptions.targetField]
     */
    targetField?: string;
    /**
     * [descr:dxSankeyOptions.tooltip]
     */
    tooltip?: dxSankeyTooltip;
    /**
     * [descr:dxSankeyOptions.weightField]
     */
    weightField?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSankeyTooltip extends BaseWidgetTooltip {
    /**
     * [descr:dxSankeyOptions.tooltip.customizeLinkTooltip]
     */
    customizeLinkTooltip?: (info: {
      source?: string;
      target?: string;
      weight?: number;
    }) => any;
    /**
     * [descr:dxSankeyOptions.tooltip.customizeNodeTooltip]
     */
    customizeNodeTooltip?: (info: {
      title?: string;
      label?: string;
      weightIn?: number;
      weightOut?: number;
    }) => any;
    /**
     * [descr:dxSankeyOptions.tooltip.enabled]
     */
    enabled?: boolean;
    /**
     * [descr:dxSankeyOptions.tooltip.linkTooltipTemplate]
     */
    linkTooltipTemplate?:
      | DevExpress.core.template
      | ((
          info: { source?: string; target?: string; weight?: number },
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxSankeyOptions.tooltip.nodeTooltipTemplate]
     */
    nodeTooltipTemplate?:
      | DevExpress.core.template
      | ((
          info: { label?: string; weightIn?: number; weightOut?: number },
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * [descr:dxSparkline]
   */
  export class dxSparkline extends BaseSparkline<dxSparklineOptions> {
    getDataSource(): DevExpress.data.DataSource;
  }
  module dxSparkline {
    export type DisposingEvent = DevExpress.events.EventInfo<dxSparkline>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxSparkline>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxSparkline>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxSparkline> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxSparkline>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxSparkline> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxSparkline>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxSparkline> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxSparklineOptions;
    export type TooltipHiddenEvent = DevExpress.events.EventInfo<dxSparkline>;
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxSparkline>;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxSparklineOptions
    extends BaseSparklineOptions<dxSparkline> {
    /**
     * [descr:dxSparklineOptions.argumentField]
     */
    argumentField?: string;
    /**
     * [descr:dxSparklineOptions.barNegativeColor]
     */
    barNegativeColor?: string;
    /**
     * [descr:dxSparklineOptions.barPositiveColor]
     */
    barPositiveColor?: string;
    /**
     * [descr:dxSparklineOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
    /**
     * [descr:dxSparklineOptions.firstLastColor]
     */
    firstLastColor?: string;
    /**
     * [descr:dxSparklineOptions.ignoreEmptyPoints]
     */
    ignoreEmptyPoints?: boolean;
    /**
     * [descr:dxSparklineOptions.lineColor]
     */
    lineColor?: string;
    /**
     * [descr:dxSparklineOptions.lineWidth]
     */
    lineWidth?: number;
    /**
     * [descr:dxSparklineOptions.lossColor]
     */
    lossColor?: string;
    /**
     * [descr:dxSparklineOptions.maxColor]
     */
    maxColor?: string;
    /**
     * [descr:dxSparklineOptions.maxValue]
     */
    maxValue?: number;
    /**
     * [descr:dxSparklineOptions.minColor]
     */
    minColor?: string;
    /**
     * [descr:dxSparklineOptions.minValue]
     */
    minValue?: number;
    /**
     * [descr:dxSparklineOptions.pointColor]
     */
    pointColor?: string;
    /**
     * [descr:dxSparklineOptions.pointSize]
     */
    pointSize?: number;
    /**
     * [descr:dxSparklineOptions.pointSymbol]
     */
    pointSymbol?: 'circle' | 'cross' | 'polygon' | 'square' | 'triangle';
    /**
     * [descr:dxSparklineOptions.showFirstLast]
     */
    showFirstLast?: boolean;
    /**
     * [descr:dxSparklineOptions.showMinMax]
     */
    showMinMax?: boolean;
    /**
     * [descr:dxSparklineOptions.type]
     */
    type?:
      | 'area'
      | 'bar'
      | 'line'
      | 'spline'
      | 'splinearea'
      | 'steparea'
      | 'stepline'
      | 'winloss';
    /**
     * [descr:dxSparklineOptions.valueField]
     */
    valueField?: string;
    /**
     * [descr:dxSparklineOptions.winColor]
     */
    winColor?: string;
    /**
     * [descr:dxSparklineOptions.winlossThreshold]
     */
    winlossThreshold?: number;
  }
  /**
   * [descr:dxTreeMap]
   */
  export class dxTreeMap extends BaseWidget<dxTreeMapOptions> {
    /**
     * [descr:dxTreeMap.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:dxTreeMap.drillUp()]
     */
    drillUp(): void;
    /**
     * [descr:dxTreeMap.getCurrentNode()]
     */
    getCurrentNode(): dxTreeMapNode;
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:dxTreeMap.getRootNode()]
     */
    getRootNode(): dxTreeMapNode;
    /**
     * [descr:dxTreeMap.hideTooltip()]
     */
    hideTooltip(): void;
    /**
     * [descr:dxTreeMap.resetDrillDown()]
     */
    resetDrillDown(): void;
  }
  module dxTreeMap {
    export type ClickEvent = DevExpress.events.NativeEventInfo<
      dxTreeMap,
      MouseEvent | PointerEvent
    > & {
      readonly node: dxTreeMapNode;
    };
    export type DisposingEvent = DevExpress.events.EventInfo<dxTreeMap>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxTreeMap>;
    export type DrillEvent = DevExpress.events.EventInfo<dxTreeMap> & {
      readonly node: dxTreeMapNode;
    };
    export type ExportedEvent = DevExpress.events.EventInfo<dxTreeMap>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxTreeMap> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxTreeMap>;
    export type HoverChangedEvent = DevExpress.events.EventInfo<dxTreeMap> &
      InteractionInfo;
    export type IncidentOccurredEvent = DevExpress.events.EventInfo<dxTreeMap> &
      DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxTreeMap>;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface InteractionInfo {
      readonly node: dxTreeMapNode;
    }
    export type NodesInitializedEvent =
      DevExpress.events.EventInfo<dxTreeMap> & {
        readonly root: dxTreeMapNode;
      };
    export type NodesRenderingEvent = DevExpress.events.EventInfo<dxTreeMap> & {
      readonly node: dxTreeMapNode;
    };
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxTreeMap> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxTreeMapOptions;
    export type SelectionChangedEvent = DevExpress.events.EventInfo<dxTreeMap> &
      InteractionInfo;
  }
  /**
   * [descr:dxTreeMapNode]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTreeMapNode {
    /**
     * [descr:dxTreeMapNode.customize(options)]
     */
    customize(options: any): void;
    /**
     * [descr:dxTreeMapNode.data]
     */
    data?: any;
    /**
     * [descr:dxTreeMapNode.drillDown()]
     */
    drillDown(): void;
    /**
     * [descr:dxTreeMapNode.getAllChildren()]
     */
    getAllChildren(): Array<dxTreeMapNode>;
    /**
     * [descr:dxTreeMapNode.getAllNodes()]
     */
    getAllNodes(): Array<dxTreeMapNode>;
    /**
     * [descr:dxTreeMapNode.getChild(index)]
     */
    getChild(index: number): dxTreeMapNode;
    /**
     * [descr:dxTreeMapNode.getChildrenCount()]
     */
    getChildrenCount(): number;
    /**
     * [descr:dxTreeMapNode.getParent()]
     */
    getParent(): dxTreeMapNode;
    /**
     * [descr:dxTreeMapNode.index]
     */
    index?: number;
    /**
     * [descr:dxTreeMapNode.isActive()]
     */
    isActive(): boolean;
    /**
     * [descr:dxTreeMapNode.isHovered()]
     */
    isHovered(): boolean;
    /**
     * [descr:dxTreeMapNode.isLeaf()]
     */
    isLeaf(): boolean;
    /**
     * [descr:dxTreeMapNode.isSelected()]
     */
    isSelected(): boolean;
    /**
     * [descr:dxTreeMapNode.label()]
     */
    label(): string;
    /**
     * [descr:dxTreeMapNode.label(label)]
     */
    label(label: string): void;
    /**
     * [descr:dxTreeMapNode.level]
     */
    level?: number;
    /**
     * [descr:dxTreeMapNode.resetCustomization()]
     */
    resetCustomization(): void;
    /**
     * [descr:dxTreeMapNode.select(state)]
     */
    select(state: boolean): void;
    /**
     * [descr:dxTreeMapNode.showTooltip()]
     */
    showTooltip(): void;
    /**
     * [descr:dxTreeMapNode.value()]
     */
    value(): number;
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTreeMapOptions extends BaseWidgetOptions<dxTreeMap> {
    /**
     * [descr:dxTreeMapOptions.childrenField]
     */
    childrenField?: string;
    /**
     * [descr:dxTreeMapOptions.colorField]
     */
    colorField?: string;
    /**
     * [descr:dxTreeMapOptions.colorizer]
     */
    colorizer?: {
      /**
       * [descr:dxTreeMapOptions.colorizer.colorCodeField]
       */
      colorCodeField?: string;
      /**
       * [descr:dxTreeMapOptions.colorizer.colorizeGroups]
       */
      colorizeGroups?: boolean;
      /**
       * [descr:dxTreeMapOptions.colorizer.palette]
       */
      palette?: Array<string> | PaletteType;
      /**
       * [descr:dxTreeMapOptions.colorizer.paletteExtensionMode]
       */
      paletteExtensionMode?: PaletteExtensionModeType;
      /**
       * [descr:dxTreeMapOptions.colorizer.range]
       */
      range?: Array<number>;
      /**
       * [descr:dxTreeMapOptions.colorizer.type]
       */
      type?: 'discrete' | 'gradient' | 'none' | 'range';
    };
    /**
     * [descr:dxTreeMapOptions.dataSource]
     */
    dataSource?: DevExpress.data.DataSource.DataSourceLike<any>;
    /**
     * [descr:dxTreeMapOptions.group]
     */
    group?: {
      /**
       * [descr:dxTreeMapOptions.group.border]
       */
      border?: {
        /**
         * [descr:dxTreeMapOptions.group.border.color]
         */
        color?: string;
        /**
         * [descr:dxTreeMapOptions.group.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxTreeMapOptions.group.color]
       */
      color?: string;
      /**
       * [descr:dxTreeMapOptions.group.padding]
       */
      padding?: number;
      /**
       * [descr:dxTreeMapOptions.group.headerHeight]
       */
      headerHeight?: number;
      /**
       * [descr:dxTreeMapOptions.group.hoverEnabled]
       */
      hoverEnabled?: boolean;
      /**
       * [descr:dxTreeMapOptions.group.hoverStyle]
       */
      hoverStyle?: {
        /**
         * [descr:dxTreeMapOptions.group.hoverStyle.border]
         */
        border?: {
          /**
           * [descr:dxTreeMapOptions.group.hoverStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxTreeMapOptions.group.hoverStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxTreeMapOptions.group.hoverStyle.color]
         */
        color?: string;
      };
      /**
       * [descr:dxTreeMapOptions.group.label]
       */
      label?: {
        /**
         * [descr:dxTreeMapOptions.group.label.font]
         */
        font?: Font;
        /**
         * [descr:dxTreeMapOptions.group.label.textOverflow]
         */
        textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
        /**
         * [descr:dxTreeMapOptions.group.label.visible]
         */
        visible?: boolean;
      };
      /**
       * [descr:dxTreeMapOptions.group.selectionStyle]
       */
      selectionStyle?: {
        /**
         * [descr:dxTreeMapOptions.group.selectionStyle.border]
         */
        border?: {
          /**
           * [descr:dxTreeMapOptions.group.selectionStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxTreeMapOptions.group.selectionStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxTreeMapOptions.group.selectionStyle.color]
         */
        color?: string;
      };
    };
    /**
     * [descr:dxTreeMapOptions.hoverEnabled]
     */
    hoverEnabled?: boolean;
    /**
     * [descr:dxTreeMapOptions.idField]
     */
    idField?: string;
    /**
     * [descr:dxTreeMapOptions.interactWithGroup]
     */
    interactWithGroup?: boolean;
    /**
     * [descr:dxTreeMapOptions.labelField]
     */
    labelField?: string;
    /**
     * [descr:dxTreeMapOptions.layoutAlgorithm]
     */
    layoutAlgorithm?:
      | 'sliceanddice'
      | 'squarified'
      | 'strip'
      | ((e: {
          rect?: Array<number>;
          sum?: number;
          items?: Array<any>;
        }) => any);
    /**
     * [descr:dxTreeMapOptions.layoutDirection]
     */
    layoutDirection?:
      | 'leftBottomRightTop'
      | 'leftTopRightBottom'
      | 'rightBottomLeftTop'
      | 'rightTopLeftBottom';
    /**
     * [descr:dxTreeMapOptions.margin]
     */
    margin?: BaseWidgetMargin;
    /**
     * [descr:dxTreeMapOptions.maxDepth]
     */
    maxDepth?: number;
    /**
     * [descr:dxTreeMapOptions.onClick]
     */
    onClick?: ((e: DevExpress.viz.dxTreeMap.ClickEvent) => void) | string;
    /**
     * [descr:dxTreeMapOptions.onDrill]
     */
    onDrill?: (e: DevExpress.viz.dxTreeMap.DrillEvent) => void;
    /**
     * [descr:dxTreeMapOptions.onHoverChanged]
     */
    onHoverChanged?: (e: DevExpress.viz.dxTreeMap.HoverChangedEvent) => void;
    /**
     * [descr:dxTreeMapOptions.onNodesInitialized]
     */
    onNodesInitialized?: (
      e: DevExpress.viz.dxTreeMap.NodesInitializedEvent
    ) => void;
    /**
     * [descr:dxTreeMapOptions.onNodesRendering]
     */
    onNodesRendering?: (
      e: DevExpress.viz.dxTreeMap.NodesRenderingEvent
    ) => void;
    /**
     * [descr:dxTreeMapOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.viz.dxTreeMap.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxTreeMapOptions.parentField]
     */
    parentField?: string;
    /**
     * [descr:dxTreeMapOptions.selectionMode]
     */
    selectionMode?: 'multiple' | 'none' | 'single';
    /**
     * [descr:dxTreeMapOptions.tile]
     */
    tile?: {
      /**
       * [descr:dxTreeMapOptions.tile.border]
       */
      border?: {
        /**
         * [descr:dxTreeMapOptions.tile.border.color]
         */
        color?: string;
        /**
         * [descr:dxTreeMapOptions.tile.border.width]
         */
        width?: number;
      };
      /**
       * [descr:dxTreeMapOptions.tile.color]
       */
      color?: string;
      /**
       * [descr:dxTreeMapOptions.tile.hoverStyle]
       */
      hoverStyle?: {
        /**
         * [descr:dxTreeMapOptions.tile.hoverStyle.border]
         */
        border?: {
          /**
           * [descr:dxTreeMapOptions.tile.hoverStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxTreeMapOptions.tile.hoverStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxTreeMapOptions.tile.hoverStyle.color]
         */
        color?: string;
      };
      /**
       * [descr:dxTreeMapOptions.tile.label]
       */
      label?: {
        /**
         * [descr:dxTreeMapOptions.tile.label.font]
         */
        font?: Font;
        /**
         * [descr:dxTreeMapOptions.tile.label.textOverflow]
         */
        textOverflow?: DevExpress.viz.BaseWidget.VizTextOverflowType;
        /**
         * [descr:dxTreeMapOptions.tile.label.visible]
         */
        visible?: boolean;
        /**
         * [descr:dxTreeMapOptions.tile.label.wordWrap]
         */
        wordWrap?: DevExpress.viz.BaseWidget.WordWrapType;
      };
      /**
       * [descr:dxTreeMapOptions.tile.selectionStyle]
       */
      selectionStyle?: {
        /**
         * [descr:dxTreeMapOptions.tile.selectionStyle.border]
         */
        border?: {
          /**
           * [descr:dxTreeMapOptions.tile.selectionStyle.border.color]
           */
          color?: string;
          /**
           * [descr:dxTreeMapOptions.tile.selectionStyle.border.width]
           */
          width?: number;
        };
        /**
         * [descr:dxTreeMapOptions.tile.selectionStyle.color]
         */
        color?: string;
      };
    };
    /**
     * [descr:dxTreeMapOptions.tooltip]
     */
    tooltip?: dxTreeMapTooltip;
    /**
     * [descr:dxTreeMapOptions.valueField]
     */
    valueField?: string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxTreeMapTooltip extends BaseWidgetTooltip {
    /**
     * [descr:dxTreeMapOptions.tooltip.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          info: { value?: number; valueText?: string; node?: dxTreeMapNode },
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxTreeMapOptions.tooltip.customizeTooltip]
     */
    customizeTooltip?: (info: {
      value?: number;
      valueText?: string;
      node?: dxTreeMapNode;
    }) => any;
  }
  /**
   * [descr:dxVectorMap]
   */
  export class dxVectorMap extends BaseWidget<dxVectorMapOptions> {
    /**
     * [descr:dxVectorMap.center()]
     */
    center(): Array<number>;
    /**
     * [descr:dxVectorMap.center(centerCoordinates)]
     */
    center(centerCoordinates: Array<number>): void;
    /**
     * [descr:dxVectorMap.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:dxVectorMap.convertCoordinates(x, y)]
     * @deprecated [depNote:dxVectorMap.convertCoordinates(x, y)]
     */
    convertCoordinates(x: number, y: number): Array<number>;
    /**
     * [descr:dxVectorMap.convertToGeo(x, y)]
     */
    convertToGeo(x: number, y: number): Array<number>;
    /**
     * [descr:dxVectorMap.convertToXY(longitude, latitude)]
     */
    convertToXY(longitude: number, latitude: number): Array<number>;
    /**
     * [descr:dxVectorMap.getLayerByIndex(index)]
     */
    getLayerByIndex(index: number): MapLayer;
    /**
     * [descr:dxVectorMap.getLayerByName(name)]
     */
    getLayerByName(name: string): MapLayer;
    /**
     * [descr:dxVectorMap.getLayers()]
     */
    getLayers(): Array<MapLayer>;
    /**
     * [descr:dxVectorMap.viewport()]
     */
    viewport(): Array<number>;
    /**
     * [descr:dxVectorMap.viewport(viewportCoordinates)]
     */
    viewport(viewportCoordinates: Array<number>): void;
    /**
     * [descr:dxVectorMap.zoomFactor()]
     */
    zoomFactor(): number;
    /**
     * [descr:dxVectorMap.zoomFactor(zoomFactor)]
     */
    zoomFactor(zoomFactor: number): void;
  }
  module dxVectorMap {
    export type CenterChangedEvent =
      DevExpress.events.EventInfo<dxVectorMap> & {
        readonly center: Array<number>;
      };
    export type ClickEvent = DevExpress.events.NativeEventInfo<
      dxVectorMap,
      MouseEvent | PointerEvent
    > & {
      readonly target: MapLayerElement;
    };
    export type DisposingEvent = DevExpress.events.EventInfo<dxVectorMap>;
    export type DrawnEvent = DevExpress.events.EventInfo<dxVectorMap>;
    export type ExportedEvent = DevExpress.events.EventInfo<dxVectorMap>;
    export type ExportingEvent = DevExpress.events.EventInfo<dxVectorMap> &
      DevExpress.viz.BaseWidget.ExportInfo;
    export type FileSavingEvent =
      DevExpress.viz.BaseWidget.FileSavingEventInfo<dxVectorMap>;
    export type IncidentOccurredEvent =
      DevExpress.events.EventInfo<dxVectorMap> &
        DevExpress.viz.BaseWidget.IncidentInfo;
    export type InitializedEvent =
      DevExpress.events.InitializedEventInfo<dxVectorMap>;
    export type OptionChangedEvent = DevExpress.events.EventInfo<dxVectorMap> &
      DevExpress.events.ChangedOptionInfo;
    export type Properties = dxVectorMapOptions;
    export type SelectionChangedEvent =
      DevExpress.events.EventInfo<dxVectorMap> & {
        readonly target: MapLayerElement;
      };
    export type TooltipHiddenEvent = DevExpress.events.EventInfo<dxVectorMap> &
      TooltipInfo;
    /**
     * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
     */
    export interface TooltipInfo {
      target?: MapLayerElement | dxVectorMapAnnotationConfig;
    }
    export type TooltipShownEvent = DevExpress.events.EventInfo<dxVectorMap> &
      TooltipInfo;
    export type ZoomFactorChangedEvent =
      DevExpress.events.EventInfo<dxVectorMap> & {
        readonly zoomFactor: number;
      };
  }
  /**
   * [descr:dxVectorMapAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxVectorMapAnnotationConfig
    extends dxVectorMapCommonAnnotationConfig {
    /**
     * [descr:dxVectorMapAnnotationConfig.name]
     */
    name?: string;
  }
  /**
   * [descr:dxVectorMapCommonAnnotationConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxVectorMapCommonAnnotationConfig
    extends BaseWidgetAnnotationConfig {
    /**
     * [descr:dxVectorMapCommonAnnotationConfig.coordinates]
     */
    coordinates?: Array<number>;
    /**
     * [descr:dxVectorMapCommonAnnotationConfig.customizeTooltip]
     */
    customizeTooltip?: (annotation: dxVectorMapAnnotationConfig | any) => any;
    /**
     * [descr:dxVectorMapCommonAnnotationConfig.template]
     */
    template?:
      | DevExpress.core.template
      | ((
          annotation: dxVectorMapAnnotationConfig | any,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxVectorMapCommonAnnotationConfig.tooltipTemplate]
     */
    tooltipTemplate?:
      | DevExpress.core.template
      | ((
          annotation: dxVectorMapAnnotationConfig | any,
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxVectorMapLegends extends BaseLegend {
    /**
     * [descr:dxVectorMapOptions.legends.customizeHint]
     */
    customizeHint?: (itemInfo: {
      start?: number;
      end?: number;
      index?: number;
      color?: string;
      size?: number;
    }) => string;
    /**
     * [descr:dxVectorMapOptions.legends.customizeItems]
     */
    customizeItems?: (
      items: Array<DevExpress.viz.dxVectorMap.LegendItem>
    ) => Array<DevExpress.viz.dxVectorMap.LegendItem>;
    /**
     * [descr:dxVectorMapOptions.legends.customizeText]
     */
    customizeText?: (itemInfo: {
      start?: number;
      end?: number;
      index?: number;
      color?: string;
      size?: number;
    }) => string;
    /**
     * [descr:dxVectorMapOptions.legends.font]
     */
    font?: Font;
    /**
     * [descr:dxVectorMapOptions.legends.markerColor]
     */
    markerColor?: string;
    /**
     * [descr:dxVectorMapOptions.legends.markerShape]
     */
    markerShape?: 'circle' | 'square';
    /**
     * [descr:dxVectorMapOptions.legends.markerSize]
     */
    markerSize?: number;
    /**
     * [descr:dxVectorMapOptions.legends.markerTemplate]
     */
    markerTemplate?:
      | DevExpress.core.template
      | ((
          legendItem: DevExpress.viz.dxVectorMap.LegendItem,
          element: SVGGElement
        ) => string | DevExpress.core.UserDefinedElement<SVGElement>);
    /**
     * [descr:dxVectorMapOptions.legends.source]
     */
    source?: {
      /**
       * [descr:dxVectorMapOptions.legends.source.grouping]
       */
      grouping?: string;
      /**
       * [descr:dxVectorMapOptions.legends.source.layer]
       */
      layer?: string;
    };
  }
  /**
   * @deprecated use Properties instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxVectorMapOptions extends BaseWidgetOptions<dxVectorMap> {
    /**
     * [descr:dxVectorMapOptions.background]
     */
    background?: {
      /**
       * [descr:dxVectorMapOptions.background.borderColor]
       */
      borderColor?: string;
      /**
       * [descr:dxVectorMapOptions.background.color]
       */
      color?: string;
    };
    /**
     * [descr:dxVectorMapOptions.bounds]
     */
    bounds?: Array<number>;
    /**
     * [descr:dxVectorMapOptions.center]
     */
    center?: Array<number>;
    /**
     * [descr:dxVectorMapOptions.controlBar]
     */
    controlBar?: {
      /**
       * [descr:dxVectorMapOptions.controlBar.borderColor]
       */
      borderColor?: string;
      /**
       * [descr:dxVectorMapOptions.controlBar.color]
       */
      color?: string;
      /**
       * [descr:dxVectorMapOptions.controlBar.enabled]
       */
      enabled?: boolean;
      /**
       * [descr:dxVectorMapOptions.controlBar.horizontalAlignment]
       */
      horizontalAlignment?: 'center' | 'left' | 'right';
      /**
       * [descr:dxVectorMapOptions.controlBar.margin]
       */
      margin?: number;
      /**
       * [descr:dxVectorMapOptions.controlBar.opacity]
       */
      opacity?: number;
      /**
       * [descr:dxVectorMapOptions.controlBar.verticalAlignment]
       */
      verticalAlignment?: 'bottom' | 'top';
    };
    /**
     * [descr:dxVectorMapOptions.layers]
     */
    layers?:
      | Array<{
          /**
           * [descr:dxVectorMapOptions.layers.borderColor]
           */
          borderColor?: string;
          /**
           * [descr:dxVectorMapOptions.layers.borderWidth]
           */
          borderWidth?: number;
          /**
           * [descr:dxVectorMapOptions.layers.color]
           */
          color?: string;
          /**
           * [descr:dxVectorMapOptions.layers.colorGroupingField]
           */
          colorGroupingField?: string;
          /**
           * [descr:dxVectorMapOptions.layers.colorGroups]
           */
          colorGroups?: Array<number>;
          /**
           * [descr:dxVectorMapOptions.layers.customize]
           */
          customize?: (elements: Array<MapLayerElement>) => void;
          /**
           * [descr:dxVectorMapOptions.layers.dataField]
           */
          dataField?: string;
          /**
           * [descr:dxVectorMapOptions.layers.dataSource]
           */
          dataSource?: object | DevExpress.data.DataSource.DataSourceLike<any>;
          /**
           * [descr:dxVectorMapOptions.layers.elementType]
           */
          elementType?: 'bubble' | 'dot' | 'image' | 'pie';
          /**
           * [descr:dxVectorMapOptions.layers.hoverEnabled]
           */
          hoverEnabled?: boolean;
          /**
           * [descr:dxVectorMapOptions.layers.hoveredBorderColor]
           */
          hoveredBorderColor?: string;
          /**
           * [descr:dxVectorMapOptions.layers.hoveredBorderWidth]
           */
          hoveredBorderWidth?: number;
          /**
           * [descr:dxVectorMapOptions.layers.hoveredColor]
           */
          hoveredColor?: string;
          /**
           * [descr:dxVectorMapOptions.layers.label]
           */
          label?: {
            /**
             * [descr:dxVectorMapOptions.layers.label.dataField]
             */
            dataField?: string;
            /**
             * [descr:dxVectorMapOptions.layers.label.enabled]
             */
            enabled?: boolean;
            /**
             * [descr:dxVectorMapOptions.layers.label.font]
             */
            font?: Font;
          };
          /**
           * [descr:dxVectorMapOptions.layers.maxSize]
           */
          maxSize?: number;
          /**
           * [descr:dxVectorMapOptions.layers.minSize]
           */
          minSize?: number;
          /**
           * [descr:dxVectorMapOptions.layers.name]
           */
          name?: string;
          /**
           * [descr:dxVectorMapOptions.layers.opacity]
           */
          opacity?: number;
          /**
           * [descr:dxVectorMapOptions.layers.palette]
           */
          palette?: Array<string> | PaletteType;
          /**
           * [descr:dxVectorMapOptions.layers.paletteSize]
           */
          paletteSize?: number;
          /**
           * [descr:dxVectorMapOptions.layers.paletteIndex]
           */
          paletteIndex?: number;
          /**
           * [descr:dxVectorMapOptions.layers.selectedBorderColor]
           */
          selectedBorderColor?: string;
          /**
           * [descr:dxVectorMapOptions.layers.selectedBorderWidth]
           */
          selectedBorderWidth?: number;
          /**
           * [descr:dxVectorMapOptions.layers.selectedColor]
           */
          selectedColor?: string;
          /**
           * [descr:dxVectorMapOptions.layers.selectionMode]
           */
          selectionMode?: 'multiple' | 'none' | 'single';
          /**
           * [descr:dxVectorMapOptions.layers.size]
           */
          size?: number;
          /**
           * [descr:dxVectorMapOptions.layers.sizeGroupingField]
           */
          sizeGroupingField?: string;
          /**
           * [descr:dxVectorMapOptions.layers.sizeGroups]
           */
          sizeGroups?: Array<number>;
          /**
           * [descr:dxVectorMapOptions.layers.type]
           */
          type?: 'area' | 'line' | 'marker';
        }>
      | {
          borderColor?: string;
          borderWidth?: number;
          color?: string;
          colorGroupingField?: string;
          colorGroups?: Array<number>;
          customize?: (elements: Array<MapLayerElement>) => any;
          dataField?: string;
          dataSource?: object | DevExpress.data.DataSource.DataSourceLike<any>;
          elementType?: 'bubble' | 'dot' | 'image' | 'pie';
          hoverEnabled?: boolean;
          hoveredBorderColor?: string;
          hoveredBorderWidth?: number;
          hoveredColor?: string;
          label?: { dataField?: string; enabled?: boolean; font?: Font };
          maxSize?: number;
          minSize?: number;
          name?: string;
          opacity?: number;
          palette?: Array<string> | PaletteType;
          paletteSize?: number;
          selectedBorderColor?: string;
          selectedBorderWidth?: number;
          selectedColor?: string;
          selectionMode?: 'multiple' | 'none' | 'single';
          size?: number;
          sizeGroupingField?: string;
          sizeGroups?: Array<number>;
          type?: 'area' | 'line' | 'marker';
        };
    /**
     * [descr:dxVectorMapOptions.legends]
     */
    legends?: Array<dxVectorMapLegends>;
    /**
     * [descr:dxVectorMapOptions.margin]
     */
    margin?: BaseWidgetMargin;
    /**
     * [descr:dxVectorMapOptions.maxZoomFactor]
     */
    maxZoomFactor?: number;
    /**
     * [descr:dxVectorMapOptions.onCenterChanged]
     */
    onCenterChanged?: (
      e: DevExpress.viz.dxVectorMap.CenterChangedEvent
    ) => void;
    /**
     * [descr:dxVectorMapOptions.onClick]
     */
    onClick?: ((e: DevExpress.viz.dxVectorMap.ClickEvent) => void) | string;
    /**
     * [descr:dxVectorMapOptions.onSelectionChanged]
     */
    onSelectionChanged?: (
      e: DevExpress.viz.dxVectorMap.SelectionChangedEvent
    ) => void;
    /**
     * [descr:dxVectorMapOptions.onTooltipHidden]
     */
    onTooltipHidden?: (
      e: DevExpress.viz.dxVectorMap.TooltipHiddenEvent
    ) => void;
    /**
     * [descr:dxVectorMapOptions.onTooltipShown]
     */
    onTooltipShown?: (e: DevExpress.viz.dxVectorMap.TooltipShownEvent) => void;
    /**
     * [descr:dxVectorMapOptions.onZoomFactorChanged]
     */
    onZoomFactorChanged?: (
      e: DevExpress.viz.dxVectorMap.ZoomFactorChangedEvent
    ) => void;
    /**
     * [descr:dxVectorMapOptions.panningEnabled]
     */
    panningEnabled?: boolean;
    /**
     * [descr:dxVectorMapOptions.projection]
     */
    projection?:
      | 'equirectangular'
      | 'lambert'
      | 'mercator'
      | 'miller'
      | VectorMapProjectionConfig
      | string
      | any;
    /**
     * [descr:dxVectorMapOptions.tooltip]
     */
    tooltip?: dxVectorMapTooltip;
    /**
     * [descr:dxVectorMapOptions.touchEnabled]
     */
    touchEnabled?: boolean;
    /**
     * [descr:dxVectorMapOptions.wheelEnabled]
     */
    wheelEnabled?: boolean;
    /**
     * [descr:dxVectorMapOptions.zoomFactor]
     */
    zoomFactor?: number;
    /**
     * [descr:dxVectorMapOptions.zoomingEnabled]
     */
    zoomingEnabled?: boolean;
    /**
     * [descr:dxVectorMapOptions.commonAnnotationSettings]
     */
    commonAnnotationSettings?: dxVectorMapCommonAnnotationConfig;
    /**
     * [descr:dxVectorMapOptions.annotations]
     */
    annotations?: Array<dxVectorMapAnnotationConfig | any>;
    /**
     * [descr:dxVectorMapOptions.customizeAnnotation]
     */
    customizeAnnotation?: (
      annotation: dxVectorMapAnnotationConfig | any
    ) => dxVectorMapAnnotationConfig;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface dxVectorMapTooltip extends BaseWidgetTooltip {
    /**
     * [descr:dxVectorMapOptions.tooltip.contentTemplate]
     */
    contentTemplate?:
      | DevExpress.core.template
      | ((
          info: MapLayerElement,
          element: DevExpress.core.DxElement
        ) => string | DevExpress.core.UserDefinedElement);
    /**
     * [descr:dxVectorMapOptions.tooltip.customizeTooltip]
     */
    customizeTooltip?: (info: MapLayerElement) => any;
    /**
     * [descr:dxVectorMapOptions.tooltip.format]
     */
    format?: DevExpress.ui.Format;
  }
  /**
   * [descr:viz.exportFromMarkup(markup, options)]
   */
  export function exportFromMarkup(
    markup: string,
    options: {
      fileName?: string;
      format?: string;
      backgroundColor?: string;
      proxyUrl?: string;
      width?: number;
      height?: number;
      onExporting?: Function;
      onExported?: Function;
      onFileSaving?: Function;
      margin?: number;
      svgToCanvas?: Function;
    }
  ): void;
  /**
   * [descr:viz.exportWidgets(widgetInstances)]
   */
  export function exportWidgets(
    widgetInstances: Array<Array<DOMComponent>>
  ): void;
  /**
   * [descr:viz.exportWidgets(widgetInstances, options)]
   */
  export function exportWidgets(
    widgetInstances: Array<Array<DOMComponent>>,
    options: {
      fileName?: string;
      format?: 'GIF' | 'JPEG' | 'PDF' | 'PNG' | 'SVG';
      backgroundColor?: string;
      margin?: number;
      gridLayout?: boolean;
      verticalAlignment?: 'bottom' | 'center' | 'top';
      horizontalAlignment?: 'center' | 'left' | 'right';
      proxyUrl?: string;
      onExporting?: Function;
      onExported?: Function;
      onFileSaving?: Function;
      svgToCanvas?: Function;
    }
  ): void;
  /**
   * [descr:Font]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface Font {
    /**
     * [descr:Font.color]
     */
    color?: string;
    /**
     * [descr:Font.family]
     */
    family?: string;
    /**
     * [descr:Font.opacity]
     */
    opacity?: number;
    /**
     * [descr:Font.size]
     */
    size?: string | number;
    /**
     * [descr:Font.weight]
     */
    weight?: number;
  }
  /**
   * @deprecated Use LegendItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface FunnelLegendItem extends BaseLegendItem {
    /**
     * [descr:FunnelLegendItem.item]
     */
    item?: DevExpress.viz.dxFunnel.Item;
  }
  /**
   * [descr:GaugeIndicator]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface GaugeIndicator extends CommonIndicator {
    /**
     * [descr:GaugeIndicator.type]
     */
    type?: DevExpress.viz.BaseGauge.GaugeIndicatorType;
  }
  /**
   * [descr:viz.generateColors(palette, count, options)]
   */
  export function generateColors(
    palette: PaletteType | Array<string>,
    count: number,
    options: {
      paletteExtensionMode?: PaletteExtensionModeType;
      baseColorSet?: 'simpleSet' | 'indicatingSet' | 'gradientSet';
    }
  ): Array<string>;
  /**
   * [descr:viz.getMarkup(widgetInstances)]
   */
  export function getMarkup(widgetInstances: Array<DOMComponent>): string;
  /**
   * [descr:viz.getPalette(paletteName)]
   */
  export function getPalette(paletteName: string): any;
  /**
   * [descr:viz.getTheme(theme)]
   */
  export function getTheme(theme: string): any;
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type HatchingDirectionType = 'left' | 'none' | 'right';
  /**
   * [descr:MapLayer]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface MapLayer {
    /**
     * [descr:MapLayer.clearSelection()]
     */
    clearSelection(): void;
    /**
     * [descr:MapLayer.elementType]
     */
    elementType?: string;
    /**
     * [descr:MapLayer.getDataSource()]
     */
    getDataSource(): DevExpress.data.DataSource;
    /**
     * [descr:MapLayer.getElements()]
     */
    getElements(): Array<MapLayerElement>;
    /**
     * [descr:MapLayer.index]
     */
    index?: number;
    /**
     * [descr:MapLayer.name]
     */
    name?: string;
    /**
     * [descr:MapLayer.type]
     */
    type?: string;
  }
  /**
   * [descr:MapLayerElement]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface MapLayerElement {
    /**
     * [descr:MapLayerElement.applySettings(settings)]
     */
    applySettings(settings: any): void;
    /**
     * [descr:MapLayerElement.attribute(name)]
     */
    attribute(name: string): any;
    /**
     * [descr:MapLayerElement.attribute(name, value)]
     */
    attribute(name: string, value: any): void;
    /**
     * [descr:MapLayerElement.coordinates()]
     */
    coordinates(): any;
    /**
     * [descr:MapLayerElement.layer]
     */
    layer?: any;
    /**
     * [descr:MapLayerElement.selected()]
     */
    selected(): boolean;
    /**
     * [descr:MapLayerElement.selected(state)]
     */
    selected(state: boolean): void;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type PaletteExtensionModeType = 'alternate' | 'blend' | 'extrapolate';
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type PaletteType =
    | 'Bright'
    | 'Harmony Light'
    | 'Ocean'
    | 'Pastel'
    | 'Soft'
    | 'Soft Pastel'
    | 'Vintage'
    | 'Violet'
    | 'Carmine'
    | 'Dark Moon'
    | 'Dark Violet'
    | 'Green Mist'
    | 'Soft Blue'
    | 'Material'
    | 'Office';
  /**
   * @deprecated Use LegendItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PieChartLegendItem extends BaseLegendItem {
    /**
     * [descr:PieChartLegendItem.argument]
     */
    argument?: string | Date | number;
    /**
     * [descr:PieChartLegendItem.argumentIndex]
     */
    argumentIndex?: number;
    /**
     * [descr:PieChartLegendItem.points]
     */
    points?: Array<piePointObject>;
    /**
     * [descr:PieChartLegendItem.text]
     */
    text?: any;
  }
  /**
   * [descr:PieChartSeries]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PieChartSeries
    extends dxPieChartSeriesTypesCommonPieChartSeries {
    /**
     * [descr:PieChartSeries.name]
     */
    name?: string;
    /**
     * [descr:PieChartSeries.tag]
     */
    tag?: any;
  }
  /**
   * [descr:pieChartSeriesObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface pieChartSeriesObject extends baseSeriesObject {
    /**
     * [descr:pieChartSeriesObject.hover()]
     */
    hover(): void;
    /**
     * [descr:pieChartSeriesObject.clearHover()]
     */
    clearHover(): void;
    /**
     * [descr:pieChartSeriesObject.isHovered()]
     */
    isHovered(): boolean;
  }
  /**
   * [descr:piePointObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface piePointObject extends basePointObject {
    /**
     * [descr:piePointObject.hide()]
     */
    hide(): void;
    /**
     * [descr:piePointObject.isVisible()]
     */
    isVisible(): boolean;
    /**
     * [descr:piePointObject.percent]
     */
    percent?: string | number | Date;
    /**
     * [descr:piePointObject.show()]
     */
    show(): void;
  }
  /**
   * [descr:PolarChartSeries]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface PolarChartSeries
    extends dxPolarChartSeriesTypesCommonPolarChartSeries {
    /**
     * [descr:PolarChartSeries.name]
     */
    name?: string;
    /**
     * [descr:PolarChartSeries.tag]
     */
    tag?: any;
    /**
     * [descr:PolarChartSeries.type]
     */
    type?: DevExpress.viz.dxPolarChart.PolarChartSeriesType;
  }
  /**
   * [descr:polarChartSeriesObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface polarChartSeriesObject extends baseSeriesObject {}
  /**
   * [descr:polarPointObject]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface polarPointObject extends basePointObject {}
  /**
   * [descr:viz.refreshPaths()]
   */
  export function refreshPaths(): void;
  /**
   * [descr:viz.refreshTheme()]
   */
  export function refreshTheme(): void;
  /**
   * [descr:viz.registerPalette(paletteName, palette)]
   */
  export function registerPalette(paletteName: string, palette: any): void;
  /**
   * [descr:viz.registerTheme(customTheme, baseTheme)]
   */
  export function registerTheme(customTheme: any, baseTheme: string): void;
  /**
   * [descr:ScaleBreak]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface ScaleBreak {
    /**
     * [descr:ScaleBreak.endValue]
     */
    endValue?: number | Date | string;
    /**
     * [descr:ScaleBreak.startValue]
     */
    startValue?: number | Date | string;
  }
  /**
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type TimeIntervalType =
    | 'day'
    | 'hour'
    | 'millisecond'
    | 'minute'
    | 'month'
    | 'quarter'
    | 'second'
    | 'week'
    | 'year';
  /**
   * @deprecated Use LegendItem instead
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface VectorMapLegendItem extends BaseLegendItem {
    /**
     * [descr:VectorMapLegendItem.color]
     */
    color?: string;
    /**
     * [descr:VectorMapLegendItem.end]
     */
    end?: number;
    /**
     * [descr:VectorMapLegendItem.size]
     */
    size?: number;
    /**
     * [descr:VectorMapLegendItem.start]
     */
    start?: number;
  }
  /**
   * [descr:VectorMapProjectionConfig]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface VectorMapProjectionConfig {
    /**
     * [descr:VectorMapProjectionConfig.aspectRatio]
     */
    aspectRatio?: number;
    /**
     * [descr:VectorMapProjectionConfig.from]
     */
    from?: (coordinates: Array<number>) => Array<number>;
    /**
     * [descr:VectorMapProjectionConfig.to]
     */
    to?: (coordinates: Array<number>) => Array<number>;
  }
  /**
   * [descr:VizRange]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export interface VizRange {
    /**
     * [descr:VizRange.endValue]
     */
    endValue?: number | Date | string;
    /**
     * [descr:VizRange.length]
     */
    length?: VizTimeInterval;
    /**
     * [descr:VizRange.startValue]
     */
    startValue?: number | Date | string;
  }
  /**
   * [descr:VizTimeInterval]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export type VizTimeInterval =
    | number
    | {
        /**
         * [descr:VizTimeInterval.days]
         */
        days?: number;
        /**
         * [descr:VizTimeInterval.hours]
         */
        hours?: number;
        /**
         * [descr:VizTimeInterval.milliseconds]
         */
        milliseconds?: number;
        /**
         * [descr:VizTimeInterval.minutes]
         */
        minutes?: number;
        /**
         * [descr:VizTimeInterval.months]
         */
        months?: number;
        /**
         * [descr:VizTimeInterval.quarters]
         */
        quarters?: number;
        /**
         * [descr:VizTimeInterval.seconds]
         */
        seconds?: number;
        /**
         * [descr:VizTimeInterval.weeks]
         */
        weeks?: number;
        /**
         * [descr:VizTimeInterval.years]
         */
        years?: number;
      }
    | TimeIntervalType;
}
declare module DevExpress.viz.dxBarGauge {
  export type LegendItem = BarGaugeLegendItem;
}
declare module DevExpress.viz.dxFunnel {
  export type Item = dxFunnelItem;
  export type LegendItem = FunnelLegendItem;
}
declare module DevExpress.viz.dxPieChart {
  export type LegendItem = PieChartLegendItem;
}
declare module DevExpress.viz.dxVectorMap {
  export type LegendItem = VectorMapLegendItem;
}
declare module DevExpress.viz.map {
  /**
   * [descr:viz.map.projection(data)]
   * @deprecated Attention! This type is for internal purposes only. If you used it previously, please describe your scenario in the following GitHub Issue, and we will suggest a public alternative: {@link https://github.com/DevExpress/DevExtreme/issues/17885|Internal Types}.
   */
  export const projection: {
    /**
     * [descr:viz.map.projection.add(name, projection)]
     */
    add(name: string, projection: VectorMapProjectionConfig | any): void;

    /**
     * [descr:viz.map.projection.get(name)]
     */
    get(
      name: 'equirectangular' | 'lambert' | 'mercator' | 'miller' | string
    ): any;

    (data: VectorMapProjectionConfig): any;
  };
}

declare module DevExpress {
    /** @deprecated Use DevExpress.events.EventObject instead */
    export type dxEvent = DevExpress.events.EventObject
    /** @deprecated Use DevExpress.events.event instead */
    export type event = DevExpress.events.event
}

declare module DevExpress.viz {
    /** @deprecated Use DevExpress.viz.ChartSeries instead */
    export type dxChartSeries =  DevExpress.viz.ChartSeries;
    /** @deprecated Use DevExpress.viz.PieChartSeries instead */
    export type dxPieChartSeries =  DevExpress.viz.PieChartSeries;
    /** @deprecated Use DevExpress.viz.PolarChartSeries instead */
    export type dxPolarChartSeries =  DevExpress.viz.PolarChartSeries;

    /** @deprecated Use DevExpress.viz instead */
    export module charts {
        export type dxChartOptions = DevExpress.viz.dxChartOptions;
        export type dxChartArgumentAxis = DevExpress.viz.dxChartArgumentAxis;
        export type dxChartArgumentAxisConstantLines  = DevExpress.viz.dxChartArgumentAxisConstantLines;
        export type dxChartArgumentAxisConstantLinesLabel = DevExpress.viz.dxChartArgumentAxisConstantLinesLabel;
        export type dxChartArgumentAxisConstantLineStyle = DevExpress.viz.dxChartArgumentAxisConstantLineStyle;
        export type dxChartArgumentAxisConstantLineStyleLabel = DevExpress.viz.dxChartArgumentAxisConstantLineStyleLabel;
        export type dxChartArgumentAxisLabel = DevExpress.viz.dxChartArgumentAxisLabel;
        export type dxChartArgumentAxisStrips = DevExpress.viz.dxChartArgumentAxisStrips;
        export type dxChartArgumentAxisStripsLabel = DevExpress.viz.dxChartArgumentAxisStripsLabel;
        export type dxChartArgumentAxisTitle = DevExpress.viz.dxChartArgumentAxisTitle;
        export type dxChartCommonAxisSettings = DevExpress.viz.dxChartCommonAxisSettings;
        export type dxChartCommonAxisSettingsConstantLineStyle = DevExpress.viz.dxChartCommonAxisSettingsConstantLineStyle;
        export type dxChartCommonAxisSettingsConstantLineStyleLabel = DevExpress.viz.dxChartCommonAxisSettingsConstantLineStyleLabel;
        export type dxChartCommonAxisSettingsLabel = DevExpress.viz.dxChartCommonAxisSettingsLabel;
        export type dxChartCommonAxisSettingsStripStyle = DevExpress.viz.dxChartCommonAxisSettingsStripStyle;
        export type dxChartCommonAxisSettingsStripStyleLabel = DevExpress.viz.dxChartCommonAxisSettingsStripStyleLabel;
        export type dxChartCommonAxisSettingsTitle = DevExpress.viz.dxChartCommonAxisSettingsTitle;
        export type dxChartCommonPaneSettings = DevExpress.viz.dxChartCommonPaneSettings;
        export type dxChartCommonSeriesSettings = DevExpress.viz.dxChartCommonSeriesSettings;
        export type dxChartLegend = DevExpress.viz.dxChartLegend;
        export type dxChartPanes = DevExpress.viz.dxChartPanes;
        export type dxChartSeries = DevExpress.viz.dxChartSeries;
        export type dxChartTooltip = DevExpress.viz.dxChartTooltip;
        export type dxChartValueAxis = DevExpress.viz.dxChartValueAxis;
        export type dxChartValueAxisConstantLines = DevExpress.viz.dxChartValueAxisConstantLines;
        export type dxChartValueAxisConstantLinesLabel = DevExpress.viz.dxChartValueAxisConstantLinesLabel;
        export type dxChartValueAxisConstantLineStyle = DevExpress.viz.dxChartValueAxisConstantLineStyle;
        export type dxChartValueAxisConstantLineStyleLabel = DevExpress.viz.dxChartValueAxisConstantLineStyleLabel;
        export type dxChartValueAxisLabel = DevExpress.viz.dxChartValueAxisLabel;
        export type dxChartValueAxisStrips = DevExpress.viz.dxChartValueAxisStrips;
        export type dxChartValueAxisStripsLabel = DevExpress.viz.dxChartValueAxisStripsLabel;
        export type dxChartValueAxisTitle = DevExpress.viz.dxChartValueAxisTitle;

        export type dxPieChartOptions = DevExpress.viz.dxPieChartOptions;
        export type dxPieChartAdaptiveLayout = DevExpress.viz.dxPieChartAdaptiveLayout;
        export type dxPieChartLegend = DevExpress.viz.dxPieChartLegend;
        export type dxPieChartSeries = DevExpress.viz.dxPieChartSeries;

        export type dxPolarChartOptions = DevExpress.viz.dxPolarChartOptions;
        export type dxPolarChartAdaptiveLayout = DevExpress.viz.dxPolarChartAdaptiveLayout;
        export type dxPolarChartArgumentAxis = DevExpress.viz.dxPolarChartArgumentAxis;
        export type dxPolarChartArgumentAxisConstantLines = DevExpress.viz.dxPolarChartArgumentAxisConstantLines;
        export type dxPolarChartArgumentAxisConstantLinesLabel = DevExpress.viz.dxPolarChartArgumentAxisConstantLinesLabel;
        export type dxPolarChartArgumentAxisLabel = DevExpress.viz.dxPolarChartArgumentAxisLabel;
        export type dxPolarChartArgumentAxisStrips = DevExpress.viz.dxPolarChartArgumentAxisStrips;
        export type dxPolarChartArgumentAxisStripsLabel = DevExpress.viz.dxPolarChartArgumentAxisStripsLabel;
        export type dxPolarChartCommonAxisSettings = DevExpress.viz.dxPolarChartCommonAxisSettings;
        export type dxPolarChartCommonAxisSettingsConstantLineStyle = DevExpress.viz.dxPolarChartCommonAxisSettingsConstantLineStyle;
        export type dxPolarChartCommonAxisSettingsConstantLineStyleLabel = DevExpress.viz.dxPolarChartCommonAxisSettingsConstantLineStyleLabel;
        export type dxPolarChartCommonAxisSettingsLabel = DevExpress.viz.dxPolarChartCommonAxisSettingsLabel;
        export type dxPolarChartCommonAxisSettingsStripStyle = DevExpress.viz.dxPolarChartCommonAxisSettingsStripStyle;
        export type dxPolarChartCommonAxisSettingsStripStyleLabel = DevExpress.viz.dxPolarChartCommonAxisSettingsStripStyleLabel;
        export type dxPolarChartCommonAxisSettingsTick = DevExpress.viz.dxPolarChartCommonAxisSettingsTick;
        export type dxPolarChartCommonSeriesSettings = DevExpress.viz.dxPolarChartCommonSeriesSettings;
        export type dxPolarChartLegend = DevExpress.viz.dxPolarChartLegend;
        export type dxPolarChartSeries = DevExpress.viz.dxPolarChartSeries;
        export type dxPolarChartTooltip = DevExpress.viz.dxPolarChartTooltip;
        export type dxPolarChartValueAxis = DevExpress.viz.dxPolarChartValueAxis;
        export type dxPolarChartValueAxisConstantLines = DevExpress.viz.dxPolarChartValueAxisConstantLines;
        export type dxPolarChartValueAxisConstantLinesLabel = DevExpress.viz.dxPolarChartValueAxisConstantLinesLabel;
        export type dxPolarChartValueAxisLabel = DevExpress.viz.dxPolarChartValueAxisLabel;
        export type dxPolarChartValueAxisStrips = DevExpress.viz.dxPolarChartValueAxisStrips;
        export type dxPolarChartValueAxisStripsLabel = DevExpress.viz.dxPolarChartValueAxisStripsLabel;
        export type dxPolarChartValueAxisTick = DevExpress.viz.dxPolarChartValueAxisTick;
    }

    /** @deprecated Use DevExpress.viz instead */
    export module funnel {
        export type dxFunnelOptions = DevExpress.viz.dxFunnelOptions;
        export type dxFunnelTooltip = DevExpress.viz.dxFunnelTooltip;
    }

    /** @deprecated Use DevExpress.viz instead */
    export module gauges {
        export type dxCircularGaugeOptions = DevExpress.viz.dxCircularGaugeOptions;
        export type dxCircularGaugeRangeContainer = DevExpress.viz.dxCircularGaugeRangeContainer;
        export type dxCircularGaugeScale = DevExpress.viz.dxCircularGaugeScale;
        export type dxCircularGaugeScaleLabel = DevExpress.viz.dxCircularGaugeScaleLabel;

        export type dxLinearGaugeOptions = DevExpress.viz.dxLinearGaugeOptions;
        export type dxLinearGaugeRangeContainer = DevExpress.viz.dxLinearGaugeRangeContainer;
        export type dxLinearGaugeScale = DevExpress.viz.dxLinearGaugeScale;
        export type dxLinearGaugeScaleLabel = DevExpress.viz.dxLinearGaugeScaleLabel;

        export type dxBarGaugeOptions = DevExpress.viz.dxBarGaugeOptions;
        export type dxBarGaugeTooltip = DevExpress.viz.dxBarGaugeTooltip;
    }

    /** @deprecated Use DevExpress.viz instead */
    export module rangeSelector {
        export type dxRangeSelectorOptions = DevExpress.viz.dxRangeSelectorOptions;
    }

    /** @deprecated Use DevExpress.viz instead */
    export module sparklines {
        export type dxSparklineOptions = DevExpress.viz.dxSparklineOptions;
        export type dxBulletOptions = DevExpress.viz.dxBulletOptions;
    }

    /** @deprecated Use DevExpress.viz instead */
    export module map {
        export type dxVectorMapOptions = DevExpress.viz.dxVectorMapOptions;
        export type dxVectorMapTooltip = DevExpress.viz.dxVectorMapTooltip;
    }

    /** @deprecated Use DevExpress.viz instead */
    export module treeMap {
        export type dxTreeMapOptions = DevExpress.viz.dxTreeMapOptions;
        export type dxTreeMapTooltip = DevExpress.viz.dxTreeMapTooltip;
    }
}

declare module DevExpress.ui {
    /** @deprecated Use DevExpress.ui.dxAccordionItem */
    export type dxAccordionItemTemplate = DevExpress.ui.dxAccordionItem;

    /** @deprecated Use DevExpress.ui.dxActionSheetItem */
    export type dxActionSheetItemTemplate = DevExpress.ui.dxActionSheetItem;

    /** @deprecated Use DevExpress.ui.dxBoxItem */
    export type dxBoxItemTemplate = DevExpress.ui.dxBoxItem;

    /** @deprecated Use DevExpress.ui.dxGalleryItem */
    export type dxGalleryItemTemplate = DevExpress.ui.dxGalleryItem;

    /** @deprecated Use DevExpress.ui.dxMultiViewItem */
    export type dxMultiViewItemTemplate = DevExpress.ui.dxMultiViewItem;

    /** @deprecated Use DevExpress.ui.dxNavBarItem */
    export type dxNavBarItemTemplate = DevExpress.ui.dxNavBarItem;

    /** @deprecated Use DevExpress.ui.dxResponsiveBoxItem */
    export type dxResponsiveBoxItemTemplate = DevExpress.ui.dxResponsiveBoxItem;

    /** @deprecated Use DevExpress.ui.dxSchedulerAppointment */
    export type dxSchedulerAppointmentTemplate = DevExpress.ui.dxSchedulerAppointment;

    /** @deprecated Use DevExpress.ui.dxSlideOutItem */
    export type dxSlideOutItemTemplate = DevExpress.ui.dxSlideOutItem;

    /** @deprecated Use DevExpress.ui.dxTabsItem */
    export type dxTabsItemTemplate = DevExpress.ui.dxTabsItem;

    /** @deprecated Use DevExpress.ui.dxTabPanelItem */
    export type dxTabPanelItemTemplate = DevExpress.ui.dxTabPanelItem;

    /** @deprecated Use DevExpress.ui.dxTileViewItem */
    export type dxTileViewItemTemplate = DevExpress.ui.dxTileViewItem;

    /** @deprecated Use DevExpress.ui.dxToolbarItem */
    export type dxToolbarItemTemplate = DevExpress.ui.dxToolbarItem;

    /** @deprecated Use DevExpress.ui.CollectionWidgetItem */
    export type CollectionWidgetItemTemplate = DevExpress.ui.CollectionWidgetItem;

    /** @deprecated Use DevExpress.ui.dxContextMenuItem */
    export type dxContextMenuItemTemplate = DevExpress.ui.dxContextMenuItem;

    /** @deprecated Use DevExpress.ui.dxMenuBaseItem */
    export type dxMenuBaseItemTemplate = DevExpress.ui.dxMenuBaseItem;

    /** @deprecated Use DevExpress.ui.CollectionWidgetItem */
    export type DataExpressionMixinItemTemplate = DevExpress.ui.CollectionWidgetItem;

    /** @deprecated Use DevExpress.ui.dxListItem */
    export type dxListItemTemplate = DevExpress.ui.dxListItem;

    /** @deprecated Use DevExpress.ui.dxMenuItem */
    export type dxMenuItemTemplate = DevExpress.ui.dxMenuItem;

    /** @deprecated Use DevExpress.ui.dxTreeViewItem */
    export type dxTreeViewItemTemplate = DevExpress.ui.dxTreeViewItem;
}

export default DevExpress;